<?php
/**
 * stanislav.elbl, 03.08.2015
 */

namespace RonPortal\Common;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\ControllerResult;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\OutputObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxesRozdeleni;
use RonPortal\Controls\ComboBoxZamestnanec;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;


class ReportsController extends JsonController
{

    /** @var Database */
    var $database;


    /**
     * ReportsController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * Zobrazi formular pro zadani filtru reportu
     * @param $reportName
     * @return BaseViewObject
     * @throws Exception
     */
    public function reportForm($reportName)
    {
        $filterForm = new ReportsFilterForm($this->database);
        $filterForm ->load($reportName);

        //Model::addData('reportsFilterForm', $this);
        return $filterForm;
    }


    /**
     * Zpracuje odeslana data formulare pro tisk
     * @throws Exception
     */
    public function handleReportDialogPost()
    {
        $reportName = $_POST['reportName'];
        $report = ReportsFilterForm::loadReport($this ->database, $reportName);

        $format = 'pdf';
        foreach (Report::FORMATY as $f) {
            if (isset($_POST['FORMAT_' . $f])) {
                $format = $f;
            }
        }
        //throw new \Exception($format);

//        if ($report ->appReportName) {
            // report z manazera sestav
            //$result = new XmlReportOutputObject($reportName, $format, ReportsFilterForm::getFiltersXml($this->database, $report)); // prozatim pdf
            //$result = new XmlReportOutputObject($report, $format);
            $result = ReportsFilterForm::getXmlReportOutputObject($this ->database, $report, $format);
//        } else {
//            // puvodni report ze sestav pro intraweb - (zatim) nelze pres XML
//            $result = ReportsFilterForm::getReportOutputObject($this ->database, $report, $format);
//        }

        session_write_close(); // zamezi blokovani dalsich requestu

        return $result;
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function nactiOsoby($output)
    {
        $report = Report::loadReport($this ->database, StringUtils::fromUTF8($_GET['report']));

        $datumOd = time();
        $datumDo = null;

        if ($_GET['obdobi']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['obdobi']);
            $datumDo = DateUtils::lastDayOfMonth($datumOd);
        } else if ($_GET['ctvrtleti']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['ctvrtleti']);
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 3), -1);
            //throw new \Exception("$datumOd - $datumDo");
        } else if ($_GET['rok']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['rok']);
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        } else {
            if ($_GET['datumOd']) {
                $datumOd = DateUtils::dateStringToTimestamp($_GET['datumOd']);
            }
            if ($_GET['datumDo']) {
                $datumDo = DateUtils::dateStringToTimestamp($_GET['datumDo']);
            }
        }

        $datumDo = $datumDo ?: $datumOd;

        Prava::nastavObdobiProPrava($datumOd, $datumDo);
        $prava = Prava::vratPrava($report ->prava ?: 'prohlizeni');

        $comboOsoba = new ComboBoxZamestnanec('osoba');

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)  &&  (int)$_GET['jenRozdeleni']) { // $report->zobrazitPanelRozdeleni  && - POZOR - panely nejsou nactene
            $comboOsoba ->nastavFiltrRozdeleni(
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP),
                //StringUtils::fromUTF8($_GET['rozdeleni']),
                $_GET['rozdeleni'],
                (int)$_GET['vcetnePodrizenych']);
        }

        $comboOsoba ->loadDataPP($this ->database, $datumOd, $datumDo, $prava, false, null);
        $output ->addData('options', $comboOsoba ->getJsonData());
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function nactiRozdeleni($output)
    {
        $report = Report::loadReport($this ->database, StringUtils::fromUTF8($_GET['report']));

        $datumOd = time();
        $datumDo = null;

        if ($_GET['obdobi']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['obdobi']);
            $datumDo = DateUtils::lastDayOfMonth($datumOd);
        } else if ($_GET['ctvrtleti']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['ctvrtleti']);
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 3), -1);
            //throw new \Exception("$datumOd - $datumDo");
        } else if ($_GET['rok']) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['rok']);
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        } else {
            if ($_GET['datumOd']) {
                $datumOd = DateUtils::dateStringToTimestamp($_GET['datumOd']);
            }
            if ($_GET['datumDo']) {
                $datumDo = DateUtils::dateStringToTimestamp($_GET['datumDo']);
            }
        }

        $datumDo = $datumDo ?: $datumOd;

        Prava::nastavObdobiProPrava($datumOd, $datumDo);
        $prava = Prava::vratPrava($report ->prava ?: 'prohlizeni');

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {  // prozatim jen typ, ktery je nastaveny
            $comboRozdeleni = new ComboBoxesRozdeleni($this->database, 'rozdeleni', 'rozdeleni');
            $comboRozdeleni->loadRozdeleni(false, null,
                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), null,
                true, $datumOd, $datumDo);

            $output ->addData('optionsPopis', $comboRozdeleni ->combo1 ->getJsonData());
            $output ->addData('optionsKod', $comboRozdeleni ->combo2 ->getJsonData());
        }
    }



    /**
     * @throws Exception
     */
    public function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'nactiosoby' :
                $this ->nactiOsoby($output);
                break;

            case 'nactirozdeleni' :
                $this ->nactiRozdeleni($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return string|ControllerResult
     * @throws Exception
     */
    public function handleAction($action)
    {
        HtmlHeaders::getInstance() ->addScript('script/reports.js');
        TextManager::getInstance() ->loadStrings('reports.ini');

        if (isset($_GET['action'])) {
            return parent::handleAction($action);
        }

        if (isset($_POST['reportDialog']) && $_POST['reportDialog']) {
            return $this ->handleReportDialogPost();
        }

//        if (isset($_POST['reportname']) && $_POST['reportname']) {
//            return $this ->renderReport();
//        };

        return $this ->reportForm($_GET['reportname'] ?? '');

        //return $this ->reportsList();
    }

}


return new ReportsController($this ->database);
