<?php
/**
 * UTF8
 * stanislav.elbl, 01.08.2016
 */

namespace RonPortal\XML;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\TextViewObject;
use RonPortal\User;
use Ron\StringUtils;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\RonPortalSettings;
use function Ron\ronText;


class SeznamXMLController extends JsonController
{
    /** @var  Database */
    var $database;

    /**
     * SeznamXMLController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }

    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function mojeFormulare($output)
    {
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=seznamxmlformularu">' .
            '<tr><th>' . ronText('Typ') . '</th><th>' . ronText('Zamestnanec') . '</th><th>' . ronText('Rozdeleni') . '</th><th>' . ronText('Obdobi') . '</th><th>' . ronText('Posledni_zmena') . '</th><th></th></tr>';

        $params = $this->database->createParameters();
        $sql =
            'SELECT VyplnenyXMLFormular.Id, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.Stav, ' . // 0..2
            'Rozdeleni.Popis, PoradiPredavani.KodUrovne, VyplnenyXMLFormular.DodatecnyPopis, ' . // 3..5
            'VyplnenyXMLFormular.Popis, Osoba.Prijmeni, Osoba.Jmeno, Rozdeleni.Kod, ' . // 6..9
            'VyplnenyXMLFormular.DatumVytvoreni, VyplnenyXMLFormular.PosledniZmena, S_StavXMLFormulare.Popis, ' . // 10..12
            'VyplnenyXMLFormular.KodProcesu, VyplnenyXMLFormular.PocatecniUroven, VyplnenyXMLFormular.PoradiPredavani, ' . // 13..15
            'PoradiPredavani.Vlastni, PoradiPredavani.NekontrolovatHierarchii, VyplnenyXMLFormular.Obdobi, {fn_CASTWIDEMEMO(VyplnenyXMLFormular.Poznamka)} ' . // 16..19
            'FROM ( ( ( ( VyplnenyXMLFormular LEFT OUTER JOIN Osoba ' .
            '          ON (VyplnenyXMLFormular.RC = Osoba.RC  OR   VyplnenyXMLFormular.PP_RC = Osoba.RC)' .
            '      ) LEFT OUTER JOIN PredavaciProces ' .
            '        ON PredavaciProces.Kod = VyplnenyXMLFormular.KodProcesu ' .
            '    ) LEFT OUTER JOIN PoradiPredavani ' .
            '      ON PoradiPredavani.KodProcesu = VyplnenyXMLFormular.KodProcesu ' .
            '         AND PoradiPredavani.PocatecniUroven = VyplnenyXMLFormular.PocatecniUroven ' .
            '         AND PoradiPredavani.Poradi = VyplnenyXMLFormular.PoradiPredavani ' .
            '  ) LEFT OUTER JOIN Rozdeleni ' .
            '    ON Rozdeleni.KodTypuRozdeleni = VyplnenyXMLFormular.KodTypuRozdeleni ' .
            '       AND Rozdeleni.Kod = VyplnenyXMLFormular.Rozdeleni ' .
            ') INNER JOIN S_StavXMLFormulare ' .
            '  ON S_StavXMLFormulare.Kod = VyplnenyXMLFormular.Stav ' .
            'WHERE (VyplnenyXMLFormular.Stav < 128) ' .
            '  AND ';


        // formulare ktere vlozil aktualni uzivatel - v puvodnich verzich se nepouzivala tabulka XMLFormularVyplnil
        $sql .=
            "( (VyplnenyXMLFormular.KodProcesu IS NULL AND ((RC_Vlozil = " . $params->addParameter(User::getRC()) . ") "; // prava zavorka bude dale !

        // formulare ktere vyplnil aktualni uzivatel (i kdyz jsou zrovna na jine urovni predavani)
        $sql .=
            " OR (EXISTS( " .
            " SELECT * FROM XMLFormularVyplnil " .
            " WHERE XMLFormularVyplnil.RC = " . $params->addParameter(User::getRC()) .
            "   AND XMLFormularVyplnil.IdFormulare = VyplnenyXMLFormular.Id" .
            ") ) )) ";


        // formulare na urovni oznacene "vlastni"
        $sql .=
            'OR (PoradiPredavani.Vlastni = 1 ' .
            " AND (VyplnenyXMLFormular.RC = " . $params->addParameter(User::getRC()) . "  OR  VyplnenyXMLFormular.PP_RC = " . $params->addParameter(User::getRC()) . ") " .
            ')';

        // zjisti do kterych urovni hodnoceni jsem zarazeny (podle mych pracovnich mist)
        $filtrurovne = PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne');
        $kodtypurozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI); // pouzije se, pokud neni typ rozdeleni u procesu zadan

        $XML_jenPodrizene = RonPortalSettings::getIntValue('jenvlastnirozdelenihodnoceni') == 2;

        if ($filtrurovne) // mam pravo na nejake urovne hodnoceni
        {
            // vybere XML zamestnancu, na ktere mam pravo (podle pracovniho mista a hierarchie)
            $sql .=
                'OR ( PoradiPredavani.Vlastni = 0 ' . // urovne s volbou "vlastni" jsou resene samostatne
                " AND ( " . PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne', $params) . " ) " .  // formular na "moji" urovni predavani
                ' AND  (' .
                // na aktualni urovni formulare se nekontroluje hierarchie
                'PoradiPredavani.NekontrolovatHierarchii = 1 '.
                '  OR ' .
                // nebo jsem ve stejnem nebo nadrazenem rozdeleni
                'EXISTS  ( SELECT * ' .
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
                "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = " . $params->addParameter($kodtypurozdeleni) . ") " . // typ rozdeleni u procesu neni zadan
                ' )' .
                '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) . '  OR  R_Ja.PlatnostDo IS NULL)' .

                '    AND R_Hodnoceny.KodTypuRozdeleni = R_JA.KodTypuRozdeleni ' .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                '    AND R_Hodnoceny.PlatnostOd <= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                '    AND (R_Hodnoceny.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .

                '    AND ( ' .
                '      R_Hodnoceny.RC = VyplnenyXMLFormular.RC ' .
                '      OR ( ' .
                '        R_Hodnoceny.RC = VyplnenyXMLFormular.PP_RC ' .
                '        AND R_Hodnoceny.OsobniCislo = VyplnenyXMLFormular.PP_OsobniCislo ' .
                '        AND R_Hodnoceny.DatumNastupu = VyplnenyXMLFormular.PP_DatumNastupu ' .
                '      ) ' .
                '    ) ' .
                ')' .
                ')' .
                ')';

            // Vybere XML, ktere nejsou pro konkretni osobu, ale jsou pro moje a podrizene rozdeleni
            $sql .=
                ' OR (' .
                "( " . PredavaciProcesyUtils::filtrUrovnePredavani($this ->database, 'PoradiPredavani.KodUrovne', $params) . " ) " .  // dokument na "moji" urovni predavani
                ' AND VyplnenyXMLFormular.RC IS NULL ' .    // neni pro konkretni osobu
                ' AND VyplnenyXMLFormular.PP_RC IS NULL ' .
                ' AND ('.
                '    PoradiPredavani.NekontrolovatHierarchii = 1 '.
                '   OR ' .
                '    EXISTS ( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                //"    AND R_JA.KodTypuRozdeleni = $kodtypurozdeleni". - puvodni verze

                '    AND ( R_JA.KodTypuRozdeleni = PredavaciProces.KodTypuRozdeleni ' . // proces ma vlastni typ rozdeleni
                "          OR  (PredavaciProces.KodTypuRozdeleni IS NULL  AND  R_JA.KodTypuRozdeleni = " . $params->addParameter($kodtypurozdeleni) . ") " . // typ rozdeleni u procesu neni zadan
                '    )' .

                '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($this->database->dateToParam(DateUtils::today())) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = VyplnenyXMLFormular.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = VyplnenyXMLFormular.Rozdeleni ' .
                ($XML_jenPodrizene ? ' AND H.KodNadrazene <> H.KodPodrizene ' : '') .
                ')' .
                ')' .
                ')';
        }

        $sql .= ')' .
            ' ORDER BY VyplnenyXMLFormular.PosledniZmena';


        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                    '<td>' . StringUtils::htmlTextConvert($row[6]) . '</td>' .
                    '<td>' . StringUtils::htmlTextConvert($row[7] . ' ' . $row[8]) . '</td>' .
                    '<td>' . StringUtils::htmlTextConvert($row[9]) . '</td>' .
                    '<td>' . StringUtils::format('date', $this->database->stringToTimestamp($row[18])) . '</td>' .
                    '<td>' . StringUtils::format('date', $this->database->stringToTimestamp($row[11])) . '</td>' .
                    "<td><a href='?menuid=editovatdotaznik&record=$row[0]'>" . ronText('Editovat_xml') . '</a></td>' .
                '</tr>';
            if ($row[5] || $row[19]) {
                $html .=
                '<tr>' .
                    '<td class="padding-left-2">' . ($row[5] ? StringUtils::htmlTextConvert($row[5]) . ' - ' : '') . StringUtils::htmlTextConvert($row[19]) . '</td>' .
                '</tr>';
            }
        }

        $html .= '</table>';

        $output ->addText('html', $html);
        $output->addDebugMessage($sql);
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    public function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'mojeformulare' :
                $this ->mojeFormulare($output);
                break;
        }
    }

    /**
     * @param $action
     * @return \Ron\MVC\InternalRedirect|JsonOutputObject|\Ron\MVC\OutputObject|\Ron\MVC\Redirect|\Ron\MVC\ViewObject|VyplnenyXMLFormularBrowser|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (isset($_GET['action']) && $_GET['action'] == 'mojeformulare') {
            return parent::handleAction($_GET['action']);
        }

        $xmlmenuid = isset($_GET['xmlmenuid']) ? $_GET['xmlmenuid'] : '';

        if ($xmlmenuid  &&  !RonPortalSettings::getInstance() ->getIntValue(RonPortalSettings::NACIST_XML_DO_MENU)) {
            throw new Exception('Tato funkce neni povolena!'); // sablony v menu nejsou zapnute
        }

        $_SESSION['last_xmlmenuid'] = $xmlmenuid; // posledni si zapamatuje, aby pri uzavreni formulare presel do stejneho seznamu

        $b = new SablonaXMLFormulareBrowser($this ->database, $xmlmenuid);
        $b ->load();
        Model::addViewObject($b);

        //echo "\n<BR>\n";
        Model::addViewObject(new TextViewObject('<br/><br/>'));

        $b2 = new VyplnenyXMLFormularBrowser($this ->database, $xmlmenuid);
        $b2 ->load();

        return $b2;
    }

}

return new SeznamXMLController($this ->database);
