<?php
/**
 * UTF8
 * stanislav.elbl, 01.11.2015 * roman.szablatura, 19.10.2016
 */

namespace RonPortal\Common;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;


/**
 * Class UkolyController
 * @package Ron\Personalistika
 */
class UkolyController extends JsonController
{
    /** @var  Database */
    var $database;

    /**
     * UkolyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @return UkolyBrowser
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function seznamUkolu()
    {
        Prava::nastavObdobiProPrava(time(), time());

        $b = new UkolyBrowser($this ->database);
        $b ->load();
        return $b;
    }


    /**
     * @param $kodSerial
     * @return Redirect|UkolFormular
     * @throws NeniPravoException
     * @throws DBException
     */
    public function ukolFormular($kodSerial = null)
    {
        $ukol = null;

        //ukol jiz existuje v DB
        if (!empty($kodSerial)) {
            $ukol = new Ukol();
            $ukol->load($this->database, $kodSerial);

            $pravaProhl = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"
            $pravaVedouci = Prava::vratPrava('ukoly_nast_resitele'); // prava na prirazeni ukolu osobe - nastaveni RC_Osoby - muze jen pokud je prirazen u kategorie
            if (!($pravaProhl->lzeZobrazitRC($ukol->RC_Osoby))) {
                throw new NeniPravoException();
            }
            if (!(($pravaVedouci->jsouNejakePrava() && $ukol->uzivatelMaPovolenuKategoriiUkolu($this->database)) ||
                 ((User::hasRC($ukol->RC_Osoby) || User::hasRC($ukol->RC_KdoZadal) || User::hasRC($ukol->RC_Vlozil))) //uzivatel muze byt nastaven jako resitel, zadavatel (ten co ukol priradi zamestnanci), nebo ukol vlozil do DB, pak ma pravo
                )) {
                throw new NeniPravoException();
            }
        } else {
            //pro vlozeni nesmi mit uzivatel pravo nastaveno na zakazat
            $prava = Prava::vratPrava('ukoly_vlozeni');
            if (!$prava->jsouNejakePrava()) {
                throw new NeniPravoException();
            }
        }

        $f = new UkolFormular($this ->database, $ukol);
        if (!$f ->handleAction()) {
            return $f;
        }

        return new Redirect(null,
            'menuid=ukoly' .
            (RonPortalSettings::getIntValue('typ_tlacitka') ? '&record=' . $ukol ->kodSerial : ''));
    }


    /**
     * Nacte obsah pro dashboard "Moje ukoly"
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    public function mojeUkoly($output)
    {
        $html = '<table class="dashboardTable" data-ron-href="?menuid=ukoly">';
        $html .='<tr>';
        $html .= "<td>" . TextManager::_('') . '</td>';
        $html .= "<td>" . TextManager::_('Typ_ukolu') . '</td>';
        $html .= "<td>" . TextManager::_('Popis') . '</td>';
        $html .= "<td>" . TextManager::_('Termin_ukolu') . '</td>';
        $html .= '</tr>';


        $params = $this ->database ->createParameters();
        $sql =
            'SELECT UkolOsoby.KodSerial, UkolOsoby.RC_Osoby, UkolOsoby.StavUkolu, UkolOsoby.RC_Hodnotitele_Def, UkolOsoby.Hodnoceno,' .             //0..4
            ' UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.Schvaleno1, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.Schvaleno2, KategorieUkolu.Popis,' .//5..9
            ' UkolOsoby.ZkracenyPopis, datumSplneni_Narizene' .                                                                                     //10..11

            ' FROM UkolOsoby LEFT JOIN KategorieUkolu ON (KategorieUkolu.kod = UkolOsoby.kodKategorieUkolu) ' .

            'WHERE StavUkolu <> -1 AND (RC_Osoby = ' . $params->addParameter(User::getRC()) . '
                OR RC_Hodnotitele_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele1_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele2_Def = ' . $params->addParameter(User::getRC()) . ')';

        $cursor = $this->database->getSqlResult($sql,$params);

        $prihlasenaOsoba = User::getRC();

        while ($row = $cursor->fetch()) {
            $text = '';

            if ($row[1] == $prihlasenaOsoba && $row[2] == 0 ){
                //Ukol ke splneni
                $text = TextManager::_('Mam_splnit');
            } else if ($row[3] == $prihlasenaOsoba && $row[4] == 0 && $row[2] == 1){
                //Ukol pripraven k hodnoceni
                $text = TextManager::_('Mam_hodnotit');
            } else if (($row[5] == $prihlasenaOsoba && $row[2] == 1 && $row[4] == 1 && $row[6] == 0) || ($row[7] == $prihlasenaOsoba && $row[2] == 1 && $row[6] == 1 && $row[8] == 0)){
                //Ukol pripraven ke schvaleni 1
                //Ukol pripraven ke schvaleni 2
                $text = TextManager::_('Mam_schvalit');
            }

            if ($text) {
                $html .=
                    "<tr>" .
                    "<td>" .
                    "<a href='?menuid=ukoly&record={$row[0]}'>" . $text . "</a>" .
                    "</td>" .
                    '<td>' . StringUtils::htmlTextConvert($row[9]) . '</td>' .
                    '<td>' . StringUtils::htmlTextConvert($row[10]) . '</td>' .
                    '<td>' . StringUtils::format('finedate', $cursor->stringToTimestamp($row[11])) . '</td>' .
                    "</tr>";
            }
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'mojeukoly':
                $this->mojeUkoly($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|string
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()
            ->loadStrings('ukoly.ini');

        HtmlHeaders::getInstance()
            ->setAppTitle('Ukoly');

        Prava::nastavObdobiProPrava(time(), time());

        if (isset($_POST['formaction'])  &&  $_POST['formaction']) {
            return $this ->ukolFormular((int)$_POST['KodSerial']);
        }

        if (isset($_GET['action'])) {
            //json action
            return parent::handleAction($action);
        }

        if (isset($_GET['record'])) {
            return $this->ukolFormular((int)$_GET['record']);
        }

        if (isset($_GET['formaction']) && $_GET['formaction'] == 'new') {
            return  $this->ukolFormular();
        }

        return $this->seznamUkolu();
    }
}

return new UkolyController($this ->database);
