<?php
/**
 * UTF8
 * stanislav.elbl, 25.11.2015
 */

namespace RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use RonPortal\Controller\LoginController;


class UtilsController extends JsonController
{
    /** @var  Database */
    var $database;


    /**
     * UtilsController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this ->enableAccessWithoutLogin = true;
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function zkontrolujPrava($output)
    {
        if (!User::getRC()) {
            throw new \Exception('Chyba autentizace pri kontrole prav!');
        }

        $idPP = $_GET['idpp'];
        if (!$idPP) {
            throw new \Exception('Chyba pri kontrole prav! Neni zadan pracovni pomer!');
        }

        Prava::nactiPravaUzivatele($this ->database, User::getRC());

        $datumDo = null;
        $datumOd = null;

        if (isset($_GET['datumod'])) {
            $datumOd = DateUtils::dateStringToTimestamp($_GET['datumod']);

            if (isset($_GET['datumdo'])) {
                $datumDo = DateUtils::dateStringToTimestamp($_GET['datumdo']);
            }
        }

        if ( !$datumOd ) $datumOd = time();
        if ( !$datumDo ) $datumDo = $datumOd;

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $prava = null;

        if ( $_GET['typprava']  &&  Prava::existujePravo($_GET['typprava']) ) {
            $prava = Prava::vratPrava($_GET['typprava']);
        } else {
            // typprava2 jen v pripade, ze vubec neexistuje typprava
            if ( $_GET['typprava2']  &&  Prava::existujePravo($_GET['typprava2']) ) {
                $prava = Prava::vratPrava($_GET['typprava2']);
            }
        }

        $lzezobrazit = $prava
            ? $prava ->lzeZobrazitPP($idPP, $datumOd, $datumDo)
            : false;

        $output ->setResult( $lzezobrazit ? 1 : 0 );
    }


    /**
     * Zjisteni zbyvajiciho casu pro automaticke odhlaseni.
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function lastAccess($output, $resetTimer)
    {
        $this ->enlargeSessionValidity = $resetTimer; // aby neprodluzoval platnost session

        $logoutTimer = RonPortalSettings::getLogoutTimer();
        $output ->addData('logoutTimer', $logoutTimer);

        if ($logoutTimer > 0) {
            if (User::isAuthenticated()) {
                if ($resetTimer) {
                    RonPortalSession::lastSessionAccess(true);
                }

                $remaining = RonPortalSession::lastSessionAccess() + RonPortalSettings::getLogoutTimer() - time();
                $output->addData('timestamp', RonPortalSession::lastSessionAccess() + RonPortalSettings::getLogoutTimer());
                $output->addData('remaining', $remaining);
                $output->addData('valid', $remaining > 0);

                if ($remaining <= 0) {
                    LoginController::logoutUser();
                }
            } else {
                $output->addData('valid', User::isAuthenticated());
            }
        }
    }




    function handleJsonAction($action, $output)
    {
        switch($_GET['action']) {
            case 'prava' :
                $this ->zkontrolujPrava($output);
                break;

            case 'lastAccess' :
                $this ->lastAccess($output, false);
                break;

            case 'resetTimer' :
                $this ->lastAccess($output, true);
                break;


            default :
                throw new \Exception("Neznama akce!");
        }
    }


}

return new UtilsController($this ->database);
