<?php
/**
 * UTF8
 * roman.szablatura 14.2.2017
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\StringUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MVC\InternalRedirect;
use Ron\MVC\OutputObject;
use Ron\MVC\ViewObject;
use Ron\MVC\Redirect;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\JsonOutputObject;
use Ron\Http\HtmlHeaders;
use RonPortal\RonPortalSettings;
use RonPortal\User;


class VitrinaController extends HttpController
{
    /** @var  Database $database */
    public $database;

    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->enableAccessWithoutLogin = true;
    }

    /**
     * @param $datum
     * @param $pocetSloupcu
     * @param $cena
     * @param null $kodjidelny
     * @param null $kodterminalu
     * @return JsonOutputObject|string
     * @throws DBException
     */
    public function vitrina($datum, $pocetSloupcu, $cena, $kodjidelny = null, $kodterminalu = null)
    {
        $vitrinaAction = isset($_GET['action']) ? $_GET['action'] : null;

        if ($vitrinaAction == 'kontrolaPolozek') {
            return $this->kontrolaPolozek($_GET['kodjidelny'], $_GET['kodterminalu'], $_GET['dnes'], $_GET['zitra']);
        }

        if ($vitrinaAction == 'nactipolozky') {
            return $this->nacteniPolozek($datum, $pocetSloupcu, $cena, $kodjidelny, $kodterminalu);
        }

        Model::addData('pocetSloupcu', $pocetSloupcu == 0 ? 2 : $pocetSloupcu );
        Model::addData('kodterminalu', $kodterminalu);
        Model::addData('kodjidelny', $kodjidelny);
        Model::addData('cena', $cena);
        Model::addData('interval', RonPortalSettings::getIntValue('vitrina_refresh_interval', 120));
        Model::addData('jedinyVyskytTypuPolozkyNaRadku', RonPortalSettings::getIntValue('typ_polozky_na_radku', true));

        return 'ronportal/jidelna/view_vitrina.php';
    }

    /**
     * @param $datum
     * @param $pocetSloupcu
     * @param null $cena
     * @param null $kodjidelny
     * @param null $kodterminalu
     * @return JsonOutputObject
     * @throws DBException
     * @throws Exception
     */
    private function nacteniPolozek($datum, $pocetSloupcu, $cena = null, $kodjidelny = null, $kodterminalu = null)
    {
        //Aktualni cas s datem z URL (mozno nastavit)- pro pripad, ze zakaznik bude chtit zobrazit nabidku jineho dne
        // nastaveni data
        if ($datum) {
            try {
                $aktualniCas = (time() - DateUtils::startOfTheDay(time())) + DateUtils::dateStringToTimestamp($datum);
            } catch (Exception $e) {
                // pri chybne zadanem datumu se nastavi aktualni datum
                $aktualniCas = time();
            }
        } else {
            $aktualniCas = time();
        }

        $dnes = DateUtils::startOfTheDay($aktualniCas);
        $zitra = DateUtils::incDayStart($dnes);

        $params = $this->database->createParameters();

        $sql =
            "SELECT DISTINCT PolozkaJidelnicku.den, CasovaPredvolbaVydeje.CasOd, CasovaPredvolbaVydeje.CasDo, PolozkaJidelnicku.ID, PolozkaJidelnicku.kodSerialFoto, " .    //0 - 4
            "TypDennihoJidla.Poradi, PolozkaJidelnicku.typDennihoJidla " .      //5 - 6
            "FROM PolozkaJidelnicku, CasovaPredvolbaVydeje, Terminal, MapovaniJidlaVTerminalu, TypDennihoJidla ".
            "WHERE (PolozkaJidelnicku.den = " . $params->addParameter($this->database->dateToParam($dnes)) . " OR PolozkaJidelnicku.den = " . $params->addParameter($this->database->dateToParam($zitra)) . ")".
            " AND TypDennihoJidla.Kod = PolozkaJidelnicku.typDennihoJidla ".
            " AND Terminal.kodTypuRozdeleni = PolozkaJidelnicku.kodTypuRozdeleni ";

        if ($kodjidelny != null){
            $sql .= " AND Terminal.KodRozdeleni = " . $kodjidelny;
        }


        if ($kodterminalu != null){
            $sql .= " AND Terminal.kod = " . $kodterminalu;
        }

        $sql .=
            " AND Terminal.KodRozdeleni = PolozkaJidelnicku.jidelna".
            " AND MapovaniJidlaVTerminalu.Terminal = Terminal.kod".
            " AND CasovaPredvolbaVydeje.Terminal = Terminal.kod".
            " AND MapovaniJidlaVTerminalu.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla".
            " AND ( CasovaPredvolbaVydeje.KodTypuVTerminalu =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha1 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha2 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu )".
            " ORDER BY PolozkaJidelnicku.Den, CasovaPredvolbaVydeje.CasOd, TypDennihoJidla.Poradi";

        $pocetPolozek = 0;
        $zbyvaNacistFotografii = 0;
        $nactenyCasOd = 0;
        $nactenyCasDo = 0;
        $polozkaJson = [];
        $polozkyJson = [];

        $cursor = $this ->database->getSqlResult($sql, $params);
        while ($row = $cursor->fetch()) {
            $casPolozkyOd = (int)($this->database->stringToTimestamp($row[0])+ 3600 * $this->database->stringToTimeOfDay($row[1]));
            $casPolozkyDo = (int)($this->database->stringToTimestamp($row[0])+ 3600 * $this->database->stringToTimeOfDay($row[2]));

            //jeste se polozky nevydavaji, nacte se seznam polozek, ktere se budou vydavat
            if ((($aktualniCas <= $casPolozkyOd)) or (($casPolozkyOd <= $aktualniCas) and ($aktualniCas <= $casPolozkyDo))) {
                if (($nactenyCasOd > 0) and ($casPolozkyOd >= $nactenyCasDo)) {
                    break;
                }
                $polozka = new PolozkaJidelnicku();
                $polozka->nactiPolozku($this->database, $row[3]);

                $polozkaJson['id'] = $polozka->f_id;
                $polozkaJson['cisloNabidky'] = $polozka->f_cislonabidky;
                $polozkaJson['popis'] = StringUtils::jsonText($polozka->f_popisnabidky);
                $polozkaJson['popis2'] = StringUtils::jsonText($polozka->f_popisNabidky_Jazyk2);
                $polozkaJson['fotografie'] = $polozka->kodFotografie;
                $polozkaJson['cena'] = $polozka->f_cenajidlaA;
                $polozkaJson['typ'] = $polozka->f_typdennihojidla;

                $polozkyJson[] = $polozkaJson;

                //Pokud chybi jedina fotografie, pak budu z view pomoci ajax dotazovat, zdali je jiz ulozena
                $zbyvaNacistFotografii = ($polozka->kodFotografie == 0) ? ++$zbyvaNacistFotografii : $zbyvaNacistFotografii;

                $nactenyCasOd = $casPolozkyOd;
                $nactenyCasDo = $casPolozkyDo;

                $pocetPolozek++;
            }
        }
        $cursor->close();

        $pCasPolozekProVitrinu['od'] = $nactenyCasOd;
        $pCasPolozekProVitrinu['do'] = $nactenyCasDo;
        $datumProNacteniSQL = DateUtils::startOfTheDay($nactenyCasOd);

        //JSON
        $vysledek = [];
        $vysledek['pocetSloupcu'] = $pocetSloupcu == 0 ? 2 : $pocetSloupcu;
        $vysledek['pocetPolozek'] = $pocetPolozek;
        $vysledek['zbyvaNacistFotografii'] = $zbyvaNacistFotografii;
        $vysledek['pole__casPolozekProVitrinu'] = $pCasPolozekProVitrinu;
        $vysledek['datum'] = $datumProNacteniSQL;
        $vysledek['dnes'] = $dnes;
        $vysledek['zitra'] = $zitra;
        $vysledek['kodjidelny'] = $kodjidelny;
        $vysledek['cena'] = $cena;

        $vysledek['polozkyJson'] = $polozkyJson;

        $json = new JsonOutputObject($vysledek);
        $json->setResult('OK');

        return $json;
    }

    /**
     * Metoda slouzici fc Ajax pro kontrolni dotaz zdali doslo ke zmene poctu polozek nebo pridani obrazku
     * @param $kodjidelny
     * @param $kodterminalu
     * @param $dnes
     * @param $zitra
     * @return JsonOutputObject
     * @throws DBException
     */
    private function kontrolaPolozek($kodjidelny, $kodterminalu, $dnes, $zitra)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT DISTINCT PolozkaJidelnicku.den, CasovaPredvolbaVydeje.CasOd, CasovaPredvolbaVydeje.CasDo, PolozkaJidelnicku.ID, PolozkaJidelnicku.kodSerialFoto, " .    //0 - 4
            "TypDennihoJidla.Poradi, PolozkaJidelnicku.typDennihoJidla " .      //5 - 6
            "FROM PolozkaJidelnicku, CasovaPredvolbaVydeje, Terminal, MapovaniJidlaVTerminalu, TypDennihoJidla ".
            "WHERE (PolozkaJidelnicku.den = " . $params->addParameter($this->database->dateToParam($dnes)) . " OR PolozkaJidelnicku.den = " . $params->addParameter($this->database->dateToParam($zitra)) . ")".
            " AND TypDennihoJidla.Kod = PolozkaJidelnicku.typDennihoJidla ".
            " AND Terminal.kodTypuRozdeleni = PolozkaJidelnicku.kodTypuRozdeleni ";

        if ($kodjidelny != null){
            $sql .= " AND (Terminal.KodRozdeleni = " . $kodjidelny . ")";
        }

        if ($kodterminalu != null){
            $sql .= " AND Terminal.kod = " . $kodterminalu;
        }

        $sql .=
            " AND Terminal.KodRozdeleni = PolozkaJidelnicku.jidelna".
            " AND MapovaniJidlaVTerminalu.Terminal = Terminal.kod".
            " AND CasovaPredvolbaVydeje.Terminal = Terminal.kod".
            " AND MapovaniJidlaVTerminalu.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla".
            " AND ( CasovaPredvolbaVydeje.KodTypuVTerminalu =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha1 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha2 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu )".
            " ORDER BY PolozkaJidelnicku.Den, CasovaPredvolbaVydeje.CasOd, TypDennihoJidla.Poradi";

        $dotaz = $sql;

        $nactenyCasOd = 0;
        $nactenyCasDo = 0;

        $zbyvaNacistFotografii = 0;
        $pocetNactenychPolozek = 0;

        $aktualniCas = time();

        $vypisId = [];
        $vypisFotografii = [];
        $cursor = $this ->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            $casPolozkyOd = (int)($this->database->stringToTimestamp($row[0])+ 3600 * $this->database->stringToTimeOfDay($row[1]));
            $casPolozkyDo = (int)($this->database->stringToTimestamp($row[0])+ 3600 * $this->database->stringToTimeOfDay($row[2]));

            //jeste se polozky nevydavaji, nacte se seznam polozek, ktere se budou vydavat
            if ((($aktualniCas <= $casPolozkyOd)) or (($casPolozkyOd <= $aktualniCas) and ($aktualniCas <= $casPolozkyDo))) {
                if (($nactenyCasOd > 0) and ($casPolozkyOd >= $nactenyCasDo)) {
                    break;
                }

                if ($row[4] == '' or $row[4] == null){
                    $zbyvaNacistFotografii++;
                }

                $nactenyCasOd = $casPolozkyOd;
                $nactenyCasDo = $casPolozkyDo;

                $vypisId[] = $row[3];
                $vypisFotografii[] = (($row[4] != null) ? ($row[4]) : ("0")) ;
                $pocetNactenychPolozek++ ;
            }
        }

        $cursor->close();

        $vysledek = [];
        $vysledek['dnes'] = $dnes;
        $vysledek['zbyvaNacistFotografii'] = $zbyvaNacistFotografii;
        $vysledek['pocetNactenychPolozek'] = $pocetNactenychPolozek;
        $vysledek['dotaz'] = $dotaz;
        $vysledek['vypisId'] = $vypisId;
        $vysledek['vypisFotografii'] = $vypisFotografii;
        $json = new JsonOutputObject($vysledek);
        $json->setResult('OK');

        return $json;
    }

    /**
     * @param string $action
     * @return InternalRedirect|JsonOutputObject|OutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        HtmlHeaders::getInstance()->addCss('css/vitrina.css');
        HtmlHeaders::getInstance()->addScript('script/vitrinascript.js');
        HtmlHeaders::getInstance()->setIcon('images/icon_jidelna.png');

        if (isset($_POST['kodjidelny'])) $kodjidelny = $_POST['kodjidelny'];
        else if (isset($_GET['kodjidelny'])) $kodjidelny = $_GET['kodjidelny'];
        else $kodjidelny = '';

        if (isset($_POST['kodterminalu'])) $kodterminalu = $_POST['kodterminalu'];
        else if (isset($_GET['kodterminalu'])) $kodterminalu = $_GET['kodterminalu'];
        else $kodterminalu = '';

        if (isset($_POST['cena'])) $cena = $_POST['cena'];
        else if (isset($_GET['cena'])) $cena = $_GET['cena'];
        else $cena = '';

        // akce povolene bez prihlaseni
        switch ($action) {
            case 'vitrina' :
                return $this->vitrina(
                    isset($_GET['datum']) ? $_GET['datum'] : null,
                    isset($_GET['pocetsloupcu']) ? $_GET['pocetsloupcu'] : null,
                    $cena, $kodjidelny, $kodterminalu);
        }

        if (!User::userAuthenticated()) {
            return new Redirect(null, '?menuid=logout');
        }
        throw new Exception('Chyba v metode VitrinaController::handleAction - neznama akce!');
    }
}

return new VitrinaController($this ->database);
