<?php
/**
 * UTF8
 * roman.Szablatura 17.02.2017
 */

namespace RonPortal\Terminaly;

use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MVC\HttpController;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\System\FotoOutputObject;


/**
 * Class ZaznamySnimacuController
 */
class ZaznamySnimacuController extends HttpController
{
    /** @var  Database */
    private $database;

    /** @var  PravaNaOsoby $prava */
    public $prava;


    /**
     * ZaznamySnimacuController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->prava = Prava::vratPrava('prohl_terminaly');
        Prava::nastavObdobiProPrava(time(), time());
    }

    /**
     * @param $idFotoZKarty
     * @return FotoOutputObject
     */
    public function downloadFoto($idFotoZKarty)
    {
        $this->jeMoznoZobrazitFotoZKarty($idFotoZKarty);

        $result = new FotoOutputObject($this->database, $idFotoZKarty);
        return $result;
    }

    /**
     * @param $idFotoZKarty
     * @throws \Exception
     */
    private function jeMoznoZobrazitFotoZKarty($idFotoZKarty)
    {
        //Kontrola, zdali je pravo na rc vlastnika fotografie
        $params = $this->database->createParameters();

        $rcOsoby = DBUtils::getSqlValue($this->database,
            'SELECT Osoba.RC FROM Osoba' .
            ' WHERE Osoba.IdFotografie = ' . $params->addParameter($idFotoZKarty),
            $params);

        if (!($this->prava->lzeZobrazitRC($rcOsoby, time(),time()))) {
            throw new NeniPravoException();
        }
    }

    /**
     * @param $idFotoZTerminalu
     * @return FotoOutputObject
     */
    public function downloadFotoZeSnimace($idFotoZTerminalu)
    {
        //Dle Pavla neni potreba tato kontrola, jelikoz jiz jednou kontrolou uzivatel musi projit v handleAction
        //$this->jeMoznoZobrazitFotoZTerminalu($idFotoZTerminalu);

        $result = new FotoOutputObject($this->database, $idFotoZTerminalu, 'FotoZTerminalu');
        return $result;
    }


//    private function jeMoznoZobrazitFotoZTerminalu($idFotoZTerminalu)
//    {
//        //Kontrola, zdali je pravo na rc vlastnika fotografie z terminalu
//        $params = $this->database->createParameters();
//
//        $sqlTabulkaFotoZTerminalu =
//            'SELECT FotoZTerminalu.RC ' .
//            'FROM FotoZTerminalu ' .
//            "WHERE FotoZTerminalu.id = " . $params->addParameter($idFotoZTerminalu);
//
//        $rcFotoZTerminalu = DBUtils::getSqlValue($this->database, $sqlTabulkaFotoZTerminalu, $params);
//
//        if (!$this->prava->lzeZobrazitRC($rcFotoZTerminalu,time(),time())) {
//            throw new \Exception(\Ron\ronText('Nemate_pravo'));
//        }
//    }



    public function handleAction($action)
    {
        if (!$this->prava->jsouNejakePrava()){
            throw new NeniPravoException();
        }

        TextManager::getInstance()->loadStrings('terminaly.ini');

        if (isset($_GET['fotozterminalu'])) {
            return $this ->downloadFotoZeSnimace( (int)$_GET['fotozterminalu'] );
        }

        if (isset($_GET['foto'])) {
            return $this ->downloadFoto( (int)$_GET['foto'] );
        }

        $b = new ZaznamySnimacuBrowser($this->database, $action);
        $b->load();
        return $b;
    }
}

return new ZaznamySnimacuController($this->database);
