<?php

// Funkce pro praci s MSSQL pomoci PDO

namespace Ron\DB;


class PDOMSSQLDatabase extends Database
{

    /**
     * @var \PDO
     */
    private $pdo;

    private $connectionString;
    private $attributes;

    public function __construct($connectionString)
    {
        $this->connectionString = $connectionString;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @param mixed $attributes
     */
    public function setAttributes($attributes)
    {
        $this->attributes = $attributes;
    }



    /**
     * @return string
     */
    public function getDbType()
    {
        return 'pdo_mssql';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        try {
            $this->pdo = new \PDO($this->connectionString, $user, $password);
        } catch (\PDOException $e) {
            throw new DBException($this, '', $e ->getMessage(), $e, $e ->errorInfo);
        }

        // $pdo -> setAttribute(PDO::SQLSRV_ATTR_ENCODING, PDO::SQLSRV_ENCODING_SYSTEM); - nepouzivat, protoze zalezi na driveru!
        if ($this ->attributes) {
            foreach ($this ->attributes as $attribute => $value)
                $this ->pdo->setAttribute($attribute, $value);
        }
    }

    /**
     * @return bool
     */
    public function close()
    {
        return true;
    }


    public function createParameters()
    {
        return new PlaceholderParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            };
//$paraData = $params ->getSqlParameters();
//foreach($paraData as $i => $param) {
//    $data ->bindValue($i + 1, $param);
//}
//if (!$data ->execute()) return false;
            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $data = $this->pdo->query($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }

        return new PdoMSSQLCursor($this, $data);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);

            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $count = $this->pdo->query($sqlText);

            // count obsahuje pocet zmenenych zaznamu nebo NULL pri chybe
            if ($count == NULL) {
                throw new DBException($this, $sqlText);
            }
        }
    }


    /**
     * @return string
     */
    public function lastErrorText()
    {
        $errorinfo = $this ->pdo ->errorInfo();

        if ($errorinfo) {
            $result = 'SQLSTATE: ' . $errorinfo[0] . '<br/>';
            $result .= 'Code: ' . $errorinfo[1] . '<br/>';
            $result .= 'Message: ' . $errorinfo[2] . '<br/>';
        } else $result = '';

        return $result;
    }


    public function beginTransaction()
    {
        $this ->sql('BEGIN TRAN');
    }

    public function commitTransaction()
    {
        $this ->sql('COMMIT TRAN');
    }

    public function rollbackTransaction()
    {
        $this ->sql('ROLLBACK TRAN');
    }


    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return "CAST($text AS TEXT)";
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return "YEAR($text)";
    }


    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        // datum z databaze muze mit ruzny format podle nastaveni v php.ini
        // Bud je v php.ini mssql.datetimeconvert = On (defaultni hodnota)
        //    a pak vraci datum ve formatu   5 V 2005 0:00
        // nebo je tam mssql.datetimeconvert = Off
        //    a pak vraci datum ve formatu   2005-05-01 00:00:00
        // a nebo jeste muze vracet
        //    may 5 2005 0:00 nebo tak nejak
        // V prvnim pripade lze primo puzit strtotime, v druhem pripade nee, ve tretim jo
        // Ve vsech pripadech ale windowsy nezvladaji prevod data mensiho nez 1.1.1970
        if (!isset($str)) return NULL;

        $str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval

        // Pokusi se prevest datum primo. Pokud se to nepovede tak funkce
        // vraci -1 a musi se to prechrchlat
        $result = strtotime($str);
        if ($result && $result != -1) {
            return $result;
        }

        // Nepovedlo se prevest datum, protoze se vraci ve formatu s rimskym cislem mesice
        $poma = explode(' ', $str);
        switch ($poma[1]) {
            case 'I' :
                $poma[1] = 'Jan';
                break;
            case 'II' :
                $poma[1] = 'Feb';
                break;
            case 'III' :
                $poma[1] = 'Mar';
                break;
            case 'IV' :
                $poma[1] = 'Apr';
                break;
            case 'V' :
                $poma[1] = 'May';
                break;
            case 'VI' :
                $poma[1] = 'Jun';
                break;
            case 'VII' :
                $poma[1] = 'Jul';
                break;
            case 'VIII' :
                $poma[1] = 'Aug';
                break;
            case 'IX' :
                $poma[1] = 'Sep';
                break;
            case 'X' :
                $poma[1] = 'Oct';
                break;
            case 'XI' :
                $poma[1] = 'Nov';
                break;
            case 'XII' :
                $poma[1] = 'Dec';
                break;
        }

        $newstr = implode(' ', $poma);
        if (!$newstr) return NULL;
        $result = strtotime($newstr);

        return $result;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "{ ts '" . date('Y-m-d H:i:s', $timestamp) . "'}";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "{ d '" . date("Y-m-d", $timestamp) . "'}";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);

            return "{ ts '1899-12-30 $h:$m:$s'}";
        }

        return "{ ts '1899-12-30 " . date('H:i:s', $timestamp) . "'}";
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) return null;
        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('Y-m-d', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);

            return "1899-12-30 $h:$m:$s";
        }

        return '1899-12-30 ' . date('H:i:s', $timestamp);
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if (!$str) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou
        $poma = explode(':', $timestr);

        $add = stristr($timestr, 'pm') === false ? 0 : 12;

        return $poma[0] + $add + $poma[1] / 60.0 + $poma[2] / 3600.0;
    }


    function stringToBinary($value)
    {
        return pack('H*', $value);
    }

}



class PdoMSSQLCursor extends Cursor
{
    /**
     * @var \PDOStatement
     */
    protected $data;


    /**
     * @param PDOMSSQLDatabase $database
     * @param \PDOStatement $data
     */
    function __construct($database, $data)
    {
        parent::__construct($database);
        $this->data = $data;
    }

    public function close()
    {
        $this ->data ->closeCursor();
    }


    /**
     * @return \PDO
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        return $this ->data ->fetch(\PDO::FETCH_NUM);
    }

    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows()
    {
        return $this ->data ->rowCount(); // pozor! u nekterych databazi nemusi fungovat!
    }

}



require('_appdbset.php');

function createDefaultDatabase() {
    global $db_connectionstring;

    $database = new PDOMSSQLDatabase($db_connectionstring);

    global $pdo_attributes;
    $database ->setAttributes($pdo_attributes);

    return $database;
}

