<?php
/**
 * UTF8
 * stanislav.elbl, 08.02.2017
 */

namespace Ron;

class ArrayUtils
{
    const FIND_FIRST = 0;
    const FIND_ANY = 1;
    const FIND_LAST = 2;
    const FIND_ALL = 3;


    public static function compare($needle, $key, $value)
    {
        if (is_callable($needle)) return $needle($key, $value);

        if ($value > $needle) return -1;
        if ($value < $needle) return 1;

        return 0;
    }

    /**
     * Vyhleda v serazenem poli zadanou hodnotu. Pouziva metodu puleni intervalu (nelze pouzit pro asociativni pole!).
     * @param callable|mixed $needle Vyhledavana hodnota. Pokud je zadana funkce, tak musi vracet vysledek porovnani (-/0/+)
     * @param array $array   Serazene pole, ve kterem se vyhledava
     * @param int $options
     * @return int|int[] Vraci index nalezeneho prvku nebo pole indexu pokud je $options = FIND_ALL
     */
    public static function findInSortedArray($needle, $array, $options = self::FIND_ANY)
    {
        $min = 0;
        $max = count($array) - 1;

        while ($min <= $max) {
            $i = (int)(($min + $max) / 2);

            $compare = static::compare($needle, $i, $array[$i]);

            if ($compare < 0) {
                $max = $i - 1;
            } else if ($compare > 0) {
                $min = $i + 1;
            } else {
                // nasel hodnotu
                if ($options == self::FIND_ANY) return $i;

                // najde prvni hodnotu
                $first = $i;
                if ($options == self::FIND_FIRST || $options == self::FIND_ALL) {
                    while ($first > 0  &&  !static::compare($needle, $first - 1, $array[$first - 1]) ) {
                        $first--;
                    }

                    if ($options == self::FIND_FIRST) return $first;
                }

                // najde posledni hodnotu
                $last = $i;
                if ($options == self::FIND_LAST || $options == self::FIND_ALL) {
                    while ($last < count($array) - 1  &&  !static::compare($needle, $last + 1, $array[$last + 1]) ) {
                        $last++;
                    }

                    if ($options == self::FIND_LAST) return $last;
                }

                // pokud dosel az sem, tak je to FIND_ALL
                $result = [];
                for ($i = $first;  $i <= $last;  $i++) $result[] = $i;
                return $result;
            }
        }

        return null;
    }


    /**
     * @param callable|mixed $needle Vyhledavana hodnota. Pokud je zadana funkce (s parametry key, value), tak musi vracet true, pokud je zadany prvek shodny
     * @param array $array   Serazene pole, ve kterem se vyhledava
     * @param int   $options
     * @return mixed|mixed[] Vraci index nalezeneho prvku nebo pole indexu
     */
    public static function findInArray($needle, $array, $options = self::FIND_FIRST)
    {
        if (!is_callable($needle)) {
            switch ($options) {
                case self::FIND_ANY :
                case self::FIND_FIRST :
                    $result = array_search($needle, $array, true); // strict = true
                    if ($result === false) return null;
                    return $result;

                case self::FIND_ALL :
                    return array_keys($array, $needle, true);
            }
        }

        $result = ($options == self::FIND_ALL) ? [] : null;
        foreach ($array as $key => $value) {
            $compare = is_callable($needle) ? $needle($key, $value) : $value === $needle;
            if ($compare) {
                // nasel hodnotu
                switch ($options) {
                    case self::FIND_FIRST :
                    case self::FIND_ANY :
                        return $key;

                    case self::FIND_LAST :
                        $result = $key;
                        break;

                    case self::FIND_ALL :
                        $result[] = $key;
                        break;
                }
            }
        }

        return $result;
    }

    public static function firstKey($array)
    {
        if (function_exists('array_key_first')) {
            return array_key_first($array);
        }

        foreach ($array as $key => $item) {
            return $key;
        }

        return null;
    }
}
