<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

/**
 * ArrayCursor je kurzor, ktery jako zdroj dat pouziva pole.
 * Pole muze byt zadano explicitne (v konstruktoru) nebo jej muze nacist z jineho Cursor
 */
class ArrayCursor extends Cursor
{
    /** @var array      Data. */
    protected $data;
    /** @var int        Aktualni pozice v kurzoru. Meni se pri prochazeni (fetch) nebo pomoci seek. */
    protected $position = 0;

    /**
     * ArrayCursor constructor.
     * @param Cursor|array $data    Data pro kurzor. Pokud je zadan jiny Cursor, tak z nej data nacte do pole.
     */
    public function __construct($data)
    {
        parent::__construct(null);

        if ($data instanceof Cursor) {
            $this->loadData($data);
        } else {
            $this ->data = $data;
        }
    }

    /**
     * Nacte data ze zadaneho kurzoru do interniho pole.
     * @param Cursor $cursor
     * @return void
     */
    public function loadData(Cursor $cursor)
    {
        $this ->position = 0;
        $this ->data = [];

        if (!$cursor) {
            $this ->data = null;
            return;
        }

        $this ->database = $cursor ->database;

        while ($row = $cursor ->fetch()) {
            $this ->data[] = $row;
        }
    }


    public function fetch()
    {
        if ($this ->data  &&  $this ->position < count($this ->data)) {
            $result = $this ->data[$this ->position];
            $this ->position ++;

            return $result;
        }

        return null;
    }


    public function close()
    {
        $this ->data = null;
        $this ->position = 0;
    }


    /**
     * Vraci pocet radku.
     * @return int
     */
    public function numRows()
    {
        return $this ->data ? count($this ->data) : 0;
    }


    public function seek($rowNumber)
    {
        $this ->position = $rowNumber;
    }


    public function last()
    {
        $this ->position = $this ->data ? count($this ->data) - 1 : 0;
    }
}
