<?php

namespace Ron\DB;

use Ron\StringUtils;

class LogWriter
{
    /** @var Database  */
    protected $database;

    /** @var string     Jmeno aplikace pro zapis do logu */
    protected $appName;

    /** @var string     Verze aplikace */
    protected $appVersion;

    /** @var string     Jmeno uzivatele */
    protected $userName;



    public function __construct(Database $database, string $appName, string $appVersion, string $userName)
    {
        $this ->database = $database;
        $this ->appName = $appName;
        $this ->appVersion = $appVersion;
        $this ->userName = $userName;
    }


    protected static function getLogRecordTimeStampAddValue()
    {
        /*
           // puvodni
           global $logrecordtimestampaddvalue;
           $logrecordtimestampaddvalue = (int)$logrecordtimestampaddvalue + 1;
           return $logrecordtimestampaddvalue;
        */
        $timestampAddValue = isset($_SESSION['g_timestampaddvalue'])
            ? $_SESSION['g_timestampaddvalue'] + 1
            : rand(1, 100);

        $_SESSION['g_timestampaddvalue'] = $timestampAddValue;

        return $timestampAddValue;
    }


    /**
     * @param string $description
     * @param string $category
     * @param string $comment
     * @param string $userName
     * @throws \Exception
     */
    public function addRecordToLog($description, $category, $comment = '', $userName = null)
    {
        if (!isset($userName)) {
            $userName = $this ->userName;
//            $userName = User::isLoggedAs()
//                ? StringUtils::substr(User::getRealLoginName(), 0, 23) . '/z'
//                : StringUtils::substr(User::getLoginName() ?? '', 0, 25);
        } else {
            $userName = StringUtils::substr($userName, 0, 25);
        }

        $category = $category
            ? StringUtils::substr($category, 0, 25)
            : null;

        $description = StringUtils::substr($description, 0, 120);

        $comment = $comment ?: null;

        $netUser = isset($_SERVER['REMOTE_USER'])
            ? StringUtils::substr(strrchr($_SERVER['REMOTE_USER'], '\\'), 1)
            : null;

        $netUser = $netUser
            ? StringUtils::substr($netUser, 0, 25)
            : null;

        $computerName = isset($_SERVER['REMOTE_HOST'])
            ? StringUtils::substr($_SERVER['REMOTE_HOST'], 0, 25)
            : null;

        $params = $this ->database ->createParameters();
        $sql =
            'INSERT INTO SYS_LOG (' .
            'applicationID, GTimeStamp, GTimeStampAddValue,' .
            ' UserName, GComputerName, GNetUser,' .
            ' GCategory, GDecription, GComment, ApplicationVersion) ' .
            ' VALUES ( ' .
            $params ->addParameter($this ->appName) . ', ' .
            $params ->addTimestampParameter(time()) . ', ' . $params ->addParameter(self::getLogRecordTimeStampAddValue()) . ', ' .
            $params ->addParameter($userName) . ', ' . $params ->addParameter($computerName) . ', ' . $params ->addParameter($netUser) . ', ' .
            $params ->addParameter($category) . ', ' . $params ->addParameter($description) . ', ' . $params ->addParameter($comment) . ', ' .
            $params ->addParameter(StringUtils::substr($this ->appVersion, 0, 25)) .
            ')';

        try {
            $this ->database->sql($sql, $params);
        } catch (\Exception $e) {
            // pripadnou chybu ignoruje a zaloguje
            error_log("RONPortal: Chyba pri zapisu do LOGu: " . $e ->getMessage() . "\n" . StringUtils::substr($sql, 0, 1024));
        }
    }


    /**
     * Zaloguje zapis/editaci($eventtype=UPDATE/INSERT/SELECT) do tabulky ($tablename)
     * @param string $tableName
     * @param string $eventType
     * @param string|object $srcDescription     zdroj udalosti - muze to byt primo text nebo object a pak se pouzije nazev jeho tridy
     * @param string $comment
     */
    public function addRecordToDBLog($tableName, $eventType, $srcDescription = '', $comment ='')
    {
//        $username = User::isLoggedAs()
//            ? StringUtils::substr(User::getRealLoginName(), 0, 23) . '/z'
//            : StringUtils::substr(User::getLoginName(), 0, 25);

        $tableName = StringUtils::substr($tableName, 0, 25);

        $netUser = isset($_SERVER['REMOTE_USER'])
            ? StringUtils::substr(strrchr($_SERVER['REMOTE_USER'], '\\'), 1)
            : null;

        $netUser = $netUser
            ? StringUtils::substr($netUser, 0, 25)
            : null;

        $computerName = isset($_SERVER['REMOTE_HOST'])
            ? StringUtils::substr($_SERVER['REMOTE_HOST'], 0, 25)
            : null;

        if (is_object($srcDescription)) {
            $srcDescription = StringUtils::substr(strrchr(get_class($srcDescription), "\\"), 1);
        }

        $srcDescription = $srcDescription
            ? StringUtils::substr($srcDescription, 0, 120)
            : null;

        $comment = $comment ?: null;

        $params = $this ->database ->createParameters();
        $sql =
            'INSERT INTO SYS_DB_LOG ' .
            ' ( applicationID, GTimeStamp, GTimeStampAddValue,' .
            '  UserName, GComputerName, GNetUser,' .
            '  GTableName, GEventType, GSrcDescription, GComment )' .
            ' VALUES (' .
            $params ->addParameter($this ->appName) . ', ' .
            $params ->addTimestampParameter(time()) . ', ' . $params ->addParameter(self::getLogRecordTimeStampAddValue()) . ', ' .
            $params ->addParameter($this ->userName) . ', ' . $params ->addParameter($computerName) . ', ' . $params ->addParameter($netUser) . ', ' .
            $params ->addParameter($tableName) . ', ' . $params ->addParameter($eventType) . ', ' .
            $params ->addParameter($srcDescription) . ', ' . $params ->addParameter($comment) .
            ')';
        try {
            $this ->database ->sql($sql, $params);
        } catch (\Exception $e) {
            // pripadnou chybu ignoruje a zaloguje
            error_log("RONPortal: Chyba pri zapisu do LOGu: " . $e ->getMessage() . "\n" . StringUtils::substr($sql, 0, 1024));
        }
    }


//    static function logSQL($database, $sql, $srcDescription = '')
//    {
//        self::addRecordToDBLog($database, SQLUtils::getTableNameFromSQL($sql), SQLUtils::getEventTypeFromSQL($sql), $srcDescription, $sql);
//    }


}
