<?php
/**
 * stanislav.elbl, 23.09.2021 13:44
 */

namespace Ron\DB;

class SqlDialectFirebird implements SqlDialect
{
    public function getDbType()
    {
        return 'interbase';
    }


    public function getConcatenateOperator()
    {
        return '||';
    }


    public function getReplaceFunctions()
    {
        return [
            'GETDATE' => '#current_timestamp',
            'LEN' => 'CHAR_LENGTH',
            'EXTRACTDATE' => 'CAST(? AS DATE)',
            'EXTRACTTIME' => 'CAST(? AS TIME)'
        ];
    }


    public function getPdoParameterLobOptions()
    {
        //return \PDO::SQLSRV_ENCODING_BINARY;
    }


    public function extractYearSQL($text)
    {
        return "EXTRACT(YEAR FROM $text)";
    }


    /**
     * Prevede hodnotu datoveho pole nacteneho z databaze na timestamp
     * @param mixed $data
     * @return int|null
     */
    public function dataToTimestamp($data)
    {
        if (!isset($data)) {
            return NULL;
        }

        // format: 2012-04-02 08:00:00
        //         1899-12-30 09:00:00
        $data = str_replace('1899', '1970', $data); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval
        //echo ' ('.$str.' -> '.strtotime($str).' -> '.date('d.m.Y H:i:s', strtotime($str)).') ';
        return strtotime($data);
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp)
     * @param int $timestamp
     * @return string
     */
    public function timestampToSQL($timestamp)
    {
        if (!isset($timestamp)) {
            return 'null';
        }

        return "'" . date('m/d/Y H:i:s', $timestamp) . "'";
    }


    /**
     * Vraci SQL konstantu pro zadany datum (unix timestamp) - cas ignoruje
     * @param int $timestamp
     * @return string
     */
    public function dateToSQL($timestamp)
    {
        if (!isset($timestamp)) {
            return 'null';
        }

        return "'" . date('m/d/Y', $timestamp) . "'";
    }


    /**
     * Vraci SQL konstantu pro zadany cas (unix timestamp) bez datumu
     * @param int $timestamp
     * @return string
     */
    public function timeToSQL($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "'12/30/1899 $h:$m:$s'";
        }

        return "'" . date('12/30/1899 H:i:s', $timestamp) . "'";
    }


    /**
     * Vraci SQL vyraz pro zjisteni nove hodnoty pro zadanou sekvenci
     * @param string $sequenceName
     * @param bool $standalone  Urcuje, jestli se jedna o samostatny vyraz pro zjisteni nove hodnoty, nebo je to soucast jineho vyrazu (napr. INSERT...)
     * @return string
     */
    public function nextSequenceValueSQL(string $sequenceName, $standalone = true): string
    {
        return "NEXT VALUE FOR $sequenceName" . ($standalone ? ' FROM RDB$DATABASE' : '');
    }


    public function sequenceExistsSQL(string $sequenceName): string
    {
        return
            'SELECT RDB$GENERATOR_NAME AS name FROM RDB$GENERATORS' .
            ' WHERE UPPER(RDB$GENERATOR_NAME) = ' . "'" . strtoupper($sequenceName) . "'";
    }




//    /**
//     * Vraci SQL konstantu pro vlozeni binarnich dat
//     * @param $text
//     * @return mixed
//     */
//    public function stringToBinarySQL($text)
//    {
//        throw new \Exception("SqlDialectFirebird::stringToBinarySQL neni implementovano!");
//        return $text;
//    }

}
