<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

/**
 * SqlStatement reprezentuje SQL vyraz (nebo jeho cast) vcetne parametru.
 */
class SqlStatement
{
    /** @var string     Text SQL dotazu */
    public $sqlText;
    /** @var Parameters Parametry SQL dotazu */
    public $parameters;

    /**
     * SqlStatement constructor.
     * @param string $sqlText
     * @param Parameters $parameters
     */
    public function __construct($sqlText, Parameters $parameters)
    {
        $this->sqlText = $sqlText;
        $this->parameters = $parameters;
    }

    function __toString()
    {
        return $this ->sqlText;
    }


    /**
     * Pripoji k tomuto SQL vyrazu dalsi vyraz - pripoji text pripojovaneho vyrazu a prida jeho parametry ke svym parametrum
     * @param string|SqlStatement $sql Pripojovany SqlStatement
     * @param Parameters|null $params
     */
    public function append($sql, $params = null)
    {
        if (!$sql) {
            return;
        }

        if ($sql instanceof SqlStatement) {
            $this ->sqlText .= ' ' . $sql ->sqlText;
            $this ->parameters ->appendParameters($sql ->parameters);
        } else {
            $this ->sqlText .= ' ' . $sql;
            if ($params) {
                $this ->parameters ->appendParameters($params);
            }
        }
    }


    const SEPARATOR_AND = 'AND';
    const SEPARATOR_OR = 'OR';

    /**
     * Spoji zadane casti SQL dotazu
     * @param array $sqlParts       Pole casti SQL dotazu - mohou to byt retezce a/nebo SqlStatement
     * @param string $separator     Oddelovac, ktery vlozi mezi jednotlive casti
     * @param ?bool $useBrackets    Urcuje, jestli se maji jednotlive casti vkladat do zavorek. Pokud je NULL, tak se zavorky pouziji v pripade, ze $separator je AND nebo OR
     * @return string|SqlStatement
     */
    public static function concatenate($sqlParts, $separator = "\n", $useBrackets = null)
    {
        $parameters = null;
        $sqlText = '';

        if (($useBrackets === null)  &&  ($separator === self::SEPARATOR_AND  ||  $separator === self::SEPARATOR_OR)) {
            $useBrackets = true;
        }

        foreach ($sqlParts as $sqlPart) {
            if ($sqlPart instanceof SqlStatement) {
                if (!$parameters) {
                    $parameters = $sqlPart ->parameters ->newInstance();
                }

                $parameters ->appendParameters($sqlPart ->parameters);

                $sqlPart = $sqlPart ->sqlText;
            }

            if ($sqlPart)
                $sqlText .=
                    ($sqlText ? $separator : '') .
                    (($sqlText  &&  $useBrackets) ? ' (' : '') .
                    $sqlPart .
                    (($sqlText  &&  $useBrackets) ? ')' : '');
        }

        return $parameters
            ? new SqlStatement($sqlText, $parameters)
            : $sqlText;
    }
}
