<?php
/**
 * stanislav.elbl, 20.07.2018
 */

namespace Ron;

class GlobalsVariable
{
    /** @var mixed */
    public $value;
    /** @var string */
    public $scope;

    /**
     * GlobalsVariable constructor.
     * @param $value
     * @param string $scope
     */
    public function __construct($value, $scope = 'request')
    {
        $this->value = $value;
        $this->scope = $scope;
    }
}


class Globals implements \ArrayAccess
{
    const SCOPE_REQUEST = 'request';
    //const SCOPE_SESSION = 'session';
    // TODO: Udelat SCOPE_NEXT_REQUEST?  prenese se do dalsiho requestu, ve kterem se nacte se SCOPE_REQUEST?

    /** @var GlobalsVariable[]  Asociativni pole promennych */
    private $data;

    private static $instance = null;


    private function __construct()
    {
        $this ->data = [];
    }

    public static function getInstance() {
        if (!self::$instance) {
            self::$instance = new static();
        }

        return self::$instance;
    }


    /**
     * @return GlobalsVariable[]
     */
    public function getData()
    {
        return $this->data;
    }


    /**
     * Nastavi prvek zadaneho jmena na zadanou hodnotu
     * @param string $name
     * @param string $value Pokud je zadano null, tak prislusnou promennou odstrani, jinak ji nastavi na tuto hodnotu
     * @param string $scope
     */
    public function setValue($name, $value, $scope = 'request') {
        if ( isset($value) ) {
            if (isset($this ->data[$name] ->value)) {
                $this->data[$name]->value = $value;
                $this->data[$name]->scope = $scope;
            } else {
                $this->data[$name] = new GlobalsVariable($value, $scope);
            }
        } else {
            unset($this ->data[$name]);
        }

//        if ($scope == self::SCOPE_SESSION) {
//            if (!isset($_SESSION['GLOBALS'])) $_SESSION['GLOBALS'] = [];
//            $_SESSION['GLOBALS'][$name] = $value;
//        } else {
//            // jine scope nez session - pokud byla drive v session, tak ji zrusim
//            if (isset($_SESSION['GLOBALS'][$name])) {
//                unset($_SESSION['GLOBALS'][$name]);
//            }
//        }
    }


    /**
     * Odstrani hodnotu zadaneho jmena
     * @param string $name
     */
    public function clearValue($name): void
    {
        $this ->setValue($name, null);
    }


    /**
     * Whether a offset exists
     * @return null
     */
    public function offsetExists($offset): bool
    {
        return isset($this ->data[$offset]);
    }

    /**
     * @param mixed $offset
     * @return mixed Can return all value types.
     */
    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        return $this ->offsetExists($offset)
            ? $this ->data[$offset] ->value
            : null;
    }

    /**
     * @param mixed $offset The offset to assign the value to
     * @param mixed $value
     * @return void
     */
    public function offsetSet($offset, $value): void
    {
        $this ->setValue($offset, $value);
    }

    /**
     * @param mixed $offset
     * @return void
     */
    public function offsetUnset($offset): void
    {
        $this ->setValue($offset, null);
    }

}
