<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron;

class KeyValuesArray implements \ArrayAccess
{
    /** @var (string|string[])[]  Asociativni pole */
    private $data;


    public function __construct()
    {
        $this ->data = [];
    }


    /**
     * @return mixed
     */
    public function getData()
    {
        return $this->data;
    }


    /**
     * Nastavi prvek zadaneho jmena na zadanou hodnotu
     * @param string $name
     * @param string $value   Pokud je zadano null, tak prislusny prvek odstrani, jinak jej nastavi na tuto hodnotu
     */
    public function setValue($name, $value) {
        if ( isset($value) ) {
            $this ->data[$name] = $value;
        } else {
            unset($this ->data[$name]);
        }
    }


    /**
     * Odstrani hodnotu zadaneho jmena
     * @param string $name
     */
    public function clearValue($name) {
        $this ->setValue($name, null);
    }


    /**
     * Prida hodnotu zadaneho jmena.
     * @param $name
     * @param $value
     */
    public function addValue($name, $value) {
        if (!isset($value)) return;

        if (isset($this ->data[$name])) {
            if (is_array($this ->data[$name])) {
                // zadany klic uz existuje a je to pole - prida novou hodnotu na konec seznamu
                $this ->data[$name][] = $value;
            } else {
                // zadany header uz existuje, ale neni to pole - vytvori z nej pole
                $this ->data[$name] = [$this ->data[$name], $value];
            }
        } else {
            $this ->data[$name] = $value;
        }
    }

    /**
     * Whether a offset exists
     */
    public function offsetExists($offset): bool
    {
        return isset($this ->data[$offset]);
    }

    /**
     * @param mixed $offset
     * @return mixed Can return all value types.
     */
    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        return $this ->data[$offset];
    }

    /**
     * (PHP 5 &gt;= 5.0.0)<br/>
     * Offset to set
     * @link http://php.net/manual/en/arrayaccess.offsetset.php
     * @param mixed $offset <p>
     * The offset to assign the value to.
     * </p>
     * @param mixed $value <p>
     * The value to set.
     * </p>
     * @return void
     */
    public function offsetSet($offset, $value): void
    {
        $this ->setValue($offset, $value);
    }

    /**
     * @param mixed $offset
     * @return void
     */
    public function offsetUnset($offset): void
    {
        $this ->clearValue($offset);
    }

}
