<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\MVC;

use Ron\Controls\Browser;
use Ron\Controls\InputForm;
use Ron\DB\Database;


class BrowserController extends HttpController
{
    /** @var Database */
    var $database;

    /** @var string  menuId*/
    var $action;

    /** @var callable */
    var $createBrowserFunction;
    /** @var callable */
    var $createFormFunction;


    /**
     * @param Database $database
     * @param string   $action
     * @param callable $createBrowserFunction
     * @param callable $createFormFunction
     */
    function __construct($database, $action = null, $createBrowserFunction = null, $createFormFunction = null)
    {
        $this ->database = $database;
        $this ->action = $action;

        $this ->createBrowserFunction = $createBrowserFunction;
        $this ->createFormFunction = $createFormFunction;
    }


    function createBrowser()
    {
        if ($this ->createBrowserFunction) {
            return ($this->createBrowserFunction)();
//            $createBrowserFunction = $this->createBrowserFunction;
//            $browser = $createBrowserFunction();
        }
        return null;
    }


    function createForm($formAction)
    {
        if ($this ->createFormFunction) {
            return ($this ->createFormFunction)($formAction);
        }

        return null;
    }

    /**
     * @return ViewObject|string
     */
    function getBrowser()
    {
        $browser = $this ->createBrowser();

        if ($browser) {
            $browser ->load();
            return $browser;
        }

        return null;
    }


    /**
     * @param int $recordId
     * @return ViewObject|string
     */
    function getRecordViewObject($recordId)
    {
        return null;
    }


    /**
     * @param string $formAction
     * @return InputForm
     */
    function getForm($formAction = 'new')
    {
        return $this ->createForm($formAction);
    }


    /**
     * Vraci redirect response po provedeni POST (tj. po vlozeni nebo ulozni zaznamu z formulare)
     * @return Redirect
     */
    function redirectAfterPost()
    {
        return new Redirect(null, '?menuid=' . $this ->action);
    }


    /**
     * @return ViewObject|Redirect|null
     */
    function formAction()
    {
        /** @var \Ron\Controls\InputForm $f */
        $f = $this ->getForm();

        if (!$f) {
            return null;
        }

        if (!$f ->handleAction()) {
            return $f;
        }

        if ($this ->action) {
            return $this ->redirectAfterPost();
        }

        return null;
    }


    function detailAction($recordKey)
    {
        $view = $this ->getRecordViewObject($recordKey);
        if ($view) return $view;

        // no detail view => create form
        $f = $this ->getForm('edit');
        if (!$f) return null;

        return $f;
    }


    function recordAction()
    {
        try {
            $post = $this->getRequest()->isPost();
            $insert = ($_GET['record'] ?? '') === '';

            $f = $this->getForm($insert ? ($post ? 'insert' : 'new') : ($post ? 'save' : 'edit'));

            if ($f->handleAction()) {
                return new JsonOutputObject([
                    'action' => 'reload'
                ]);
            }
        } catch (\Exception $e) {
            return new PartialView(new ExceptionViewObject($e));
        }

        return new DialogPartialView($f);
    }


    /**
     * @param string $action
     * @return ViewObject|OutputObject
     */
    public function handleAction($action)
    {
        if (isset($_GET['formaction'])) {       //   ||  isset ($_POST['formaction'])) {
            // formular pro editaci
            $form = $this ->formAction();
            if ($form) {{
                return $form;
            }}
        }

        if (isset($_GET['record'])) {
            // dialog
            $view = $this ->recordAction();
            if ($view) {
                return $view;
            }
        }

        // browser
        return $this ->getBrowser();
    }

}
