<?php

namespace Ron\MVC;

use Ron\DB\DBException;
use Ron\Http\App;
use Ron\RonException;


/**
 * View object zobrazujici vyjimku
 */
class ExceptionViewObject implements ViewObject
{
    /**
     * @var \Exception
     */
    protected $exception;


    /**
     * @param \Exception $exception
     */
    public function __construct(\Exception $exception)
    {
        $this->exception = $exception;

        if ($exception instanceof RonException  &&  $exception ->getResponseCode()) {
            http_response_code($exception ->responseCode);
        }
    }


    public static function RenderErrorText($text)
    {
        echo
            '<div class="errortext">' .
            $text .
            '</div>';
    }


    public function render()
    {
        self::RenderErrorText($this ->exception ->getMessage());

        $development = $GLOBALS['development'] ?? false;

        if ($this ->exception instanceof DBException  &&  ($development || App::getDebugMode())) {
            // chybovou hlasku zobrazi jen v debug rezimu - aby uzivateli nezobrazoval hodnoty z SQL dotazu, napr. RC
            self::RenderErrorText($this ->exception ->dbMessage);
            echo '<pre>' . $this ->exception ->sql . '</pre>';
        } else if ($this ->exception instanceof RonException) {
            echo '<div>' . $this ->exception ->getErrorDescription() . '</div>';
        }

        if ($development || App::getDebugMode()) {
            echo
                '<div>' .
                str_replace("\n", '<br/>', $this ->exception->getTraceAsString()) .
                '</div>';
        }
    }
}
