<?php
/**
 * stanislav.elbl, 17.02.2021
 */

namespace Ron;


/**
 * Implementuje pole, ktere pri pristupu k prvkum testuje, jestli prvek existuje. Pokud neexistuje, vraci pro dany index null.
 * Class ProxyArray
 */
class ProxyArray implements \ArrayAccess, \IteratorAggregate, \Countable
{
    /** @var mixed[] */
    private $data;

    /**
     * ProxyArray constructor.
     * @param \mixed[] $data
     */
    public function __construct(&$data = null)
    {
        $this ->data = isset($data) ? $data : [];
    }


    public function offsetExists($offset): bool
    {
        return isset($this ->data[$offset]);
    }

    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        return $this->data[$offset] ?? null;
    }

    public function offsetSet($offset, $value): void
    {
        if (!isset($offset)) {
            $this->data[] = $value;
        } else {
            $this->data[$offset] = $value;
        }
    }

    public function offsetUnset($offset): void
    {
        unset($this ->data[$offset]);
    }


    public function getIterator(): \Traversable
    {
        return new ProxyArrayIterator($this ->data);
    }

    public function count(): int
    {
        return count($this ->data);
    }
}


/**
 * Pomocna trida pro prochazeni ProxyArray pomoci foreach
 * Class ProxyArrayIterator
 */
class ProxyArrayIterator implements \Iterator
{
    /** @var mixed[] */
    private $data;

    /** @var int */
    private $currentIndex = 0;

    /** @var mixed[] */
    private $keys;

    /**
     * ProxyArrayIterator constructor.
     * @param $data
     */
    public function __construct(&$data)
    {
        $this->data = $data;

        $this ->keys = isset($data) ? array_keys($data) : [];
    }


    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function current()
    {
        return $this ->data[$this ->keys[$this ->currentIndex]];
    }

    public function next(): void
    {
        $this ->currentIndex++;
    }

    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function key()
    {
        return $this ->keys[$this ->currentIndex];
    }

    public function valid(): bool
    {
        return $this ->currentIndex < count($this ->keys);
    }

    public function rewind(): void
    {
        $this ->currentIndex = 0;
    }

}
