<?php
/**
 * UTF8
 * stanislav.elbl, 27.02.2017
 */

namespace RonPortal\Admin;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\NastaveniUzivatele;

class NovinkaPortalu
{
    public $datum = null;
    public $nadpis = null;
    public $text = '';

    /**
     * @return NovinkaPortalu[]
     * @throws \Exception
     */
    public static function loadNovinky()
    {
        $rows = explode("\n", StringUtils::fromUTF8(file_get_contents('doc/novinky.dat')));
        $result = [];

        /** @var NovinkaPortalu $novinka */
        $novinka = null;
        foreach ($rows as $row) {
            $row = trim($row);

            if ($row === '') {
                // prazdny radek - pokud uz nacetl novinku, tak ji prida do seznamu
                if ($novinka) {
                    $result[] = $novinka;
                    $novinka = null;
                }
            } else if ($row[0] == ';') {
                // poznamka - ignoruje
            } else {
                if (!$novinka) {
                    // prvni radek musi byt datum
                    $novinka = new NovinkaPortalu();
                    $novinka ->datum = DateUtils::dateStringToTimestamp($row);
                } else if (!$novinka ->nadpis) {
                    // druhy je nadpis
                    $novinka ->nadpis = $row;
                } else {
                    // dale nasleduje text
                    $novinka ->text .= $row . "\n";
                }
            }
        }

        if ($novinka) $result[] = $novinka;

        return $result;
    }


    /**
     * @param Database $database
     * @return bool
     */
    public static function jsouNovinky($database)
    {
        $data = self::loadNovinky();
        if (!$data) return false;

        $posledniDatumStr = NastaveniUzivatele::getUserSettingsItem($database, 'novinky_zobrazeny_do');

        if ($posledniDatumStr) {
            // zkontroluje, jestli jsou nove zaznamy
            try {
                $posledniDatum = DateUtils::dateStringToTimestamp($posledniDatumStr);
                return $posledniDatum < $data[0] ->datum;
            } catch (\Exception $e) {
            }
        }

        return true;
    }
}


