<?php
/**
 * pavel.rudol, 25.5.2022 15:30
 */

namespace RonPortal\Benefity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\System\AgentManager;
use RonPortal\User;

class PohybBenefituDBRecord extends RonPortalDBRecord
{
    public function __construct($database, $logDescription = 'Intraweb')
    {
        parent::__construct($database, 'PohybBenefitu', $logDescription);

        $this->addDateField('DatumVlozeni', true, DateUtils::today());
        $this->addStringField('RC', true, 15);

        $this->addDateField('DatumSchvaleni');
        $this->addStringField('Schvalil', true, 25, User::getLoginName());

        $this->addStringField('Cerpani_Username', true, 25, User::getLoginName());
        $this->addDateField('DatumCerpani');
    }


    public function addPohybBenefituFields()
    {
        $this->addIntField('DruhPohybu', true);
        $this->addIntField('IDBenefitu', true);
        $this->addIntField('ID', true);
        $this->addDateField('DatumRezervaceDo', true);
        $this->addDateField('DatumOmezeniPlatnosti', true);
        $this->addIntField('PocetBodu', true);
        $this->addIntField('PocetBenefitu', true);
        $this->addIntField('TypDoplatku', true);
        $this->addIntField('DoplatekCastka', true);
        $this->addMemoField('Poznamka', true);
    }
    
//    /**
//     * @param PohybBenefitu $pohyb
//     * @return static
//     */
//    public static function loadFromObject($database, $pohyb)
//    {
//        $rec = new static($database);
//
//        $rec->findFieldByName('RC')->setNewValue($pohyb->rc);
//        $rec->findFieldByName('DatumVlozeni')->setNewValue($pohyb->datumVlozeni);
//        $rec->findFieldByName('DatumSchvaleni')->setNewValue($pohyb->datumSchvaleni);
//        $rec->findFieldByName('Schvalil')->setNewValue($pohyb->schvalil);
//        $rec->findFieldByName('Cerpani_Username')->setNewValue($pohyb->cerpaniUserName);
//        $rec->findFieldByName('DatumCerpani')->setNewValue($pohyb->datumCerpani);
//
//        $rec->addIntField('DruhPohybu')->setNewValue($pohyb->druhPohybu);
//        $rec->addIntField('IDBenefitu')->setNewValue($pohyb->rc);
//        $rec->addIntField('ID')->setNewValue($pohyb->id);
//        $rec->addDateField('DatumRezervaceDo')->setNewValue($pohyb->datumRezervaceDo);
//        $rec->addDateField('DatumOmezeniPlatnosti')->setNewValue($pohyb->datumOmezeniPlatnosti);
//        $rec->addIntField('PocetBodu')->setNewValue($pohyb->pocetBodu);
//        $rec->addIntField('PocetBenefitu')->setNewValue($pohyb->pocetBenefitu);
//        $rec->addIntField('TypDoplatku')->setNewValue($pohyb->typDoplatku);
//        $rec->addIntField('DoplatekCastka')->setNewValue($pohyb->doplatekCastka);
//        $rec->addMemoField('Poznamka')->setNewValue($pohyb->poznamka);
//
//        return $rec;
//    }
    
    public function setSchvaleni()
    {
        $fSchvaleni = $this->findFieldByName('DatumSchvaleni');
        if (!$fSchvaleni->value) {
            $fSchvaleni->setNewValue(DateUtils::today());
            $this->findFieldByName('Schvalil')->setNewValue(User::getLoginName());
        }
    }
    
    public function setVydani()
    {
        $fCerpani = $this->findFieldByName('DatumCerpani');
        if (!$fCerpani->value) {
            $fCerpani->setNewValue(DateUtils::today());
            $this->findFieldByName('Cerpani_Username')->setNewValue(User::getLoginName());
        }
    }

    public function checkConditions()
    {
        if ($this['IDBenefitu'] !== null) { // nekontroluje se, pokud benefit neni zadan - napriklad pohyb typu "Vznik bodu"
            $benefit = Benefit::nactiBenefit($this->database, $this['IDBenefitu']);

            if (!$benefit->neomezenyPocetJedenZam && $benefit->maxPocetJedenZam) {
                if ($this['PocetBenefitu'] > $benefit->maxPocetJedenZam) {
                    throw new \Exception(TextManager::_('prekrocen_pocet_na_osobu'));
                }
            }
        }
    }


    /**
     * Overi, jestli ma uzivatel pravo na vydani benefitu zadane osobe. Pokud ma uzivatel omezene typy benefitu,
     * tak overi, jestli muze vydat zadany benefit. V tom pripade musi byt zadan typ nebo id Benefitu
     * (pokud neni zadan typ tak jej nacte z tabulky Benefit)
     * @param Database $database
     * @param string $rc
     * @param string|null $typBenefitu
     * @param int|null $idBenefitu
     * @return bool
     * @throws \Exception
     */
    public static function jePravoNaVydaniBenefitu($database, $rc, $typBenefitu, $idBenefitu = null)
    {
        $pravoVydani = Prava::vratPrava('benefit_vydani');

        if (!$pravoVydani ->lzeZobrazitRC($rc, time(), time())) {
            return false;
        }
echo $rc;
        if (trim($pravoVydani->priznak3 ?? '') !== '') {
            // uzivatel ma omezene typy benefitu, ktere muze vydavat
            if (!isset($typBenefitu)) {
                // neni zadan typ benefitu - musi ho nacist ze zaznamu benefitu

                if (!isset($idBenefitu)) {
                    return true; // nekontroluje - napriklad pri vkladani noveho, kdy jeste neni vybran benefit
                }

                $params = $database ->createParameters();
                $typBenefitu = DBUtils::getSqlValue($database,
                    'SELECT TypBenefitu FROM Benefit WHERE Id = ' . $params ->addParameter($idBenefitu),
                    $params
                );
            }

            if (!isset($typBenefitu)) {
                throw new \Exception('Chyba v metode PohybBenefituDBRecord::jePravoNaVydaniBenefitu - nelze nacist typ benefitu!');
            }

            if (StringUtils::strpos($pravoVydani->priznak3, $typBenefitu) === false) {
                return false;
            }
        }

        return true;
    }


    public function runAgent()
    {
        $this->checkConditions();

        $pravoSchvaleni = Prava::vratPrava('benefit_schvaleni');
        $pravoVydani = Prava::vratPrava('benefit_vydani');

        //50 - Kontrola benefitu
        $params =
            '50' .
            ' -BenefitRC:' . $this['RC'] .
            ' -BenefitDruhPohybu:' . $this['DruhPohybu'] .
            ' -BenefitID:' . $this['IDBenefitu'] . 
            ' -BenefitIDPohybu:' . ($this['ID']) .
            ($pravoSchvaleni ->lzeZobrazitRC($this['RC'], time(), time())
                ? ' -BenefitDatumSchvaleni:' . ($this['DatumSchvaleni'] ? date('Y-m-d', $this['DatumSchvaleni']) : '')
                : ''
            ) .
            ' -BenefitDatumRezervaceDo:' . ($this['DatumRezervaceDo'] ? date('Y-m-d', $this['DatumRezervaceDo']) : '') .
            (self::jePravoNaVydaniBenefitu($this ->database, $this['RC'], null, $this['IDBenefitu'])
                ? ' -BenefitDatumCerpani:' . ($this['DatumCerpani'] ? date('Y-m-d', $this['DatumCerpani']) : '')
                : ''
            ) .
            ' -BenefitCerpaniUsername:' . $this['Cerpani_Username'] .
            ' -BenefitDatumVlozeni:' . ($this['DatumVlozeni'] ? date('Y-m-d', $this['DatumVlozeni']) : '') .
            ' -BenefitDatumOmezeniPlatnosti:' . ($this['DatumOmezeniPlatnosti'] ? date('Y-m-d', $this['DatumOmezeniPlatnosti']) : "") .
            ' -BenefitPocetBodu:' . $this['PocetBodu'] .
            ' -BenefitPocet:' . $this['PocetBenefitu'] .
            ' -BenefitDruhZdroje:' . PohybBenefitu::DRUH_ZDROJE_PORTAL . //RON portal
            ' -BenefitTypDoplatku:' . $this['TypDoplatku'] .
            ' -BenefitDoplatekCastka:' . $this['DoplatekCastka'] .
            ' -BenefitPoznamka:"' . $this['Poznamka'] .
            '" ';

        AgentManager::executePersonAgent($params);
    }

}