<?php
/**
 * stanislav.elbl, 18.11.2019
 */

namespace RonPortal\Common;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxesRozdeleni;
use RonPortal\Controls\ComboBoxObdobi;
use RonPortal\Controls\ComboBoxOsoba;
use RonPortal\Controls\ComboBoxZamestnanec;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\System\ReportServer;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use Tracy\Debugger;

class ReportsFilterForm extends BaseViewObject
{
    /** @var Database */
    var $database;

    /** @var string */
    var $reportName;

    /** @var Report */
    var $report;

    /** @var ComboBoxObdobi */
    var $comboObdobi;
    /** @var ComboBox */
    var $comboCtvrtleti;
    /** @var ComboBox */
    var $comboRok;

    /** @var  Report[] */
    var $reportList;

    /** @var ComboBox */
    var $comboTypRozdeleni;
    /** @var ComboBoxesRozdeleni */
    var $comboRozdeleni;

    /** @var ComboBox */
    var $comboTypRozdeleni2;
    /** @var ComboBoxesRozdeleni */
    var $comboRozdeleni2;

    /** @var ComboBoxOsoba */
    var $comboOsoba;
    /** @var ComboBox */
    var $comboTypOsoby;
    /** @var ComboBox */
    var $comboTypPP;
    /** @var bool   Urcuje, jestli ma uzivatel pravo sam na sebe */
    var $jePravoNaVlastni;

    /** @var ComboBox */
    var $comboKalendar;

    /**
     * ReportsFilterForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('view/reportfilters.php');
        $this->database = $database;
    }


    /**
     * Nacte zadany report a doplni k nemu informace z ccf
     * @param Database  $database
     * @param string    $reportName
     * @return Report
     * @throws Exception
     */
    public static function loadReport($database, $reportName)
    {
        $report = Report::loadReport($database, $reportName);

        if ($report ->appReportName) {
            ReportServer::loadReportInfo($report);
        } else {
            // vychozi nastaveni pro sestavy, ktere nejsou z manazera
            $report ->zobrazitPanelObdobi = true;
            $report ->zobrazitPanelRozdeleni = true;
            $report ->zobrazitPanelRozdeleni2 = (bool)RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP);
            $report ->zobrazitPanelOsoba = true;
            $report ->zobrazitPanelKalendar = false;
            $report ->zobrazitPanelRazeni = false;
        }

        if (!$report ->prava) {
            $report ->prava = 'prohlizeni';
        }
        return $report;
    }


    // panel obdobi
    public static function zvolenyTypObdobi()
    {
        return RonPortalSession::getValue('report_obdobi');
    }

    public static function setZvolenyTypObdobi($value)
    {
        RonPortalSession::setValue('report_obdobi', $value);
    }

    public static function zvoleneObdobiOd()
    {
        $value = RonPortalSession::getValue('report_obdobi_od');
        return $value ?: StringUtils::format('date', DateUtils::firstDayOfMonth(time()));
    }

    /**
     * @param string $value
     */
    public static function setZvoleneObdobiOd($value)
    {
        RonPortalSession::setValue('report_obdobi_od', $value);
    }

    public static function zvoleneObdobiDo()
    {
        $value = RonPortalSession::getValue('report_obdobi_do');
        return $value ?: StringUtils::format('date', DateUtils::lastDayOfMonth(time()));
    }

    public static function setZvoleneObdobiDo($value)
    {
        RonPortalSession::setValue('report_obdobi_do', $value);
    }


    // panel rozdeleni
    public static function getJenRozdeleni()
    {
        return (int)RonPortalSession::getValue('report_jen_rozdeleni');
    }

    public static function getTypRozdeleni()
    {
        return (int)RonPortalSession::getValue('report_typ_rozdeleni');
    }

    public static function getRozdeleni($database)
    {
        $result = RonPortalSession::getValue('report_rozdeleni');
        if (!isset($result)) {
            $result = RonPortalSession::rozdeleni();

            if (!isset($result))  {
                $result = NastaveniUzivatele::getUserSettingsItem($database, 'ads_rozdeleni');
            }
        }
        return $result;
    }

    public static function getRozdeleniVcetnePodrizenych()
    {
        return (int)RonPortalSession::getValue('report_rozdeleni_vcetne_podrizenych');
    }

    public static function setRozdeleni($jenZvoleneRozdeleni, $typRozdeleni, $kodRozdeleni, $vcetnePodrizenych)
    {
        RonPortalSession::setValue('report_jen_rozdeleni', $jenZvoleneRozdeleni ? 1 : 0);
        if ($jenZvoleneRozdeleni) {
            RonPortalSession::setValue('report_typ_rozdeleni', $typRozdeleni);
            RonPortalSession::setValue('report_rozdeleni', $kodRozdeleni);
            RonPortalSession::setValue('report_rozdeleni_vcetne_podrizenych', $vcetnePodrizenych ? 1 : 0);
        }
    }


    // panel osoba
    public static function getJenZvolenaOsoba()
    {
        return (int)RonPortalSession::getValue('report_jen_osoba');
    }

    public static function setJenZvolenaOsoba($value)
    {
        RonPortalSession::setValue('report_jen_osoba', $value ? 1 : 0);
    }


    public static function getJenTypOsoby()
    {
        return (int)RonPortalSession::getValue('report_jen_typ_osoby');
    }

    public static function setJenTypOsoby($value)
    {
        RonPortalSession::setValue('report_jen_typ_osoby', $value ? 1 : 0);
    }

    public static function getTypOsoby()
    {
        return (int)RonPortalSession::getValue('report_typ_osoby');
    }

    public static function setTypOsoby($value)
    {
        RonPortalSession::setValue('report_typ_osoby', (int)$value);
    }


    public static function getJenTypPP()
    {
        return (int)RonPortalSession::getValue('report_jen_typ_pp');
    }

    public static function setJenTypPP($value)
    {
        RonPortalSession::setValue('report_jen_typ_pp', $value ? 1 : 0);
    }

    public static function getTypPP()
    {
        return RonPortalSession::getValue('report_typ_pp');
    }

    public static function setTypPP($value)
    {
        RonPortalSession::setValue('report_typ_pp', $value);
    }


    // panel kalendar
    public static function getJenKalendar()
    {
        return (int)RonPortalSession::getValue('report_jen_kalendar');
    }

    public static function setJenKalendar($value)
    {
        RonPortalSession::setValue('report_jen_kalendar', $value ? 1 : 0);
    }

    public static function getKalendar()
    {
        return RonPortalSession::getValue('report_kalendar');
    }

    public static function setKalendar($value)
    {
        RonPortalSession::setValue('report_kalendar', $value);
    }



    private function createComboCtvrtletiRok()
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT MIN (PocatekObdobi), MAX(PocatekObdobi) FROM Obdobi' .
            //' WHERE Aplikace = ' . $params ->addParameter($this->report->applicationId ?: 'ADS'),
            ' WHERE Aplikace = ' . $params ->addParameter('ADS'),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        $minObdobi = DateUtils::firstDayOfQuarter($row ? $this ->database ->stringToTimestamp($row[0]) : time());
        $maxObdobi = DateUtils::firstDayOfQuarter($row ? $this ->database ->stringToTimestamp($row[1]) : time());

        $options = [];
        for ($obdobi = $maxObdobi;  $obdobi >= $minObdobi;  $obdobi = DateUtils::incMonth($obdobi, -3)) {
            $options[] = new Option(date('j.n.Y', $obdobi), DateUtils::quarterDescription($obdobi));
        }

        $this ->comboCtvrtleti = new ComboBox('obdobiCtvrtleti', $options);
        $this ->comboCtvrtleti ->setSelectedOption(self::zvoleneObdobiOd());
        $this ->comboCtvrtleti ->htmlParams['id'] = 'obdobiCtvrtleti';
        $this ->comboCtvrtleti ->className = 'form-control obdobiInput';

        $minRok = DateUtils::firstDayOfYear($minObdobi);
        $maxRok = DateUtils::firstDayOfYear($maxObdobi);

        $optionsRok = [];
        for ($rok = $maxRok;  $rok >= $minRok;  $rok = DateUtils::incMonth($rok, -12)) {
            $optionsRok[] = new Option(date('j.n.Y', $rok), date('Y', $rok));
        }

        $this ->comboRok = new ComboBox('obdobiRok', $optionsRok);
        $this ->comboRok ->setSelectedOption(self::zvoleneObdobiOd());
        $this ->comboRok ->htmlParams['id'] = 'obdobiRok';
        $this ->comboRok ->className = 'form-control obdobiInput';
    }


    /**
     * @param $reportName
     * @throws Exception
     */
    public function load($reportName)
    {
        $this ->reportList = Report::loadSQL($this ->database, " Povolit = 1  AND  (Formular IS NULL  OR  Formular = '')", null);

        if (count($this ->reportList) === 1) {
            $reportName = $this ->reportList[0] ->kod;
        }

        $this ->reportName = $reportName;

        if ($reportName) {
            try {
                $report = self::loadReport($this->database, $reportName);

                if ($report->prava && Prava::jsouNejakePrava($report->prava)) {

                    $this->report = $report;

                    $obdobiOd = DateUtils::startOfTheDay(time());
                    $obdobiDo = DateUtils::endOfTheDay($obdobiOd);

                    if ($this->report->zobrazitPanelObdobi) {
                        $this->comboObdobi = new ComboBoxObdobi('obdobiMesic');
                        $this->comboObdobi->htmlParams = ['id' => 'obdobiMesic'];
                        $this->comboObdobi->className = 'form-control obdobiInput';
                        $this->comboObdobi->loadObdobi($this->database, $this->report->applicationId ?: 'ADS', false, false, false, RonPortalSession::obdobi());

                        $this->createComboCtvrtletiRok();

                        switch (self::zvolenyTypObdobi()) {
                            case 1 :    // ctvrtleti
                            case 2 :    // rok
                            case 3 :    // datum od-do
                                try {
                                    $obdobiOd = self::zvoleneObdobiOd() ? DateUtils::dateStringToTimestamp(self::zvoleneObdobiOd()) : $obdobiOd;
                                    $obdobiDo = self::zvoleneObdobiDo() ? DateUtils::dateStringToTimestamp(self::zvoleneObdobiDo()) : $obdobiDo;
                                } catch (Exception $e) {
                                    MessageManager::addErrorMessage($e);
                                }
                                break;

//                    case 1 :
//                        $obdobiOd = DateUtils::firstDayOfQuarter(self::zvoleneObdobiOd());
//                        $obdobiDo = DateUtils::incDay(DateUtils::incMonth(DateUtils::firstDayOfQuarter(self::zvoleneObdobiOd()), 3), -1);
//                        break;

                            case 0 :    // obdobi aplikace
                            default :
                                try {
                                    $obdobi = Obdobi::vratObdobi($this->database, $this->report->applicationId ?: 'ADS',
                                        $this->comboObdobi->pocatekObdobi ?: RonPortalSession::obdobi()
                                    );
                                    $obdobiOd = $obdobi->zacatek;
                                    $obdobiDo = $obdobi->konec;
                                } catch (Exception $e) {
                                    // ignoruje
                                }
                        }
                    }

                    Prava::nastavObdobiProPrava($obdobiOd, $obdobiDo);

                    if ($this->report->zobrazitPanelRozdeleni && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {  // prozatim jen typ, ktery je nastaveny
                        //            $this ->comboTypRozdeleni = new ComboBox('typRozdeleni');
                        //            $this ->comboTypRozdeleni ->htmlParams['id'] = 'typRozdeleni';
                        //            $this ->comboTypRozdeleni ->dataTypeComparison = false;
                        //            $this ->comboTypRozdeleni ->loadSQL($this ->database, 'SELECT Kod, Popis FROM TypRozdeleni ORDER BY Kod', null);
                        //            $this ->comboTypRozdeleni ->setSelectedOption(self::getTypRozdeleni());

                        $this->comboRozdeleni = new ComboBoxesRozdeleni($this->database, 'rozdeleni', 'rozdeleni');
                        $this->comboRozdeleni->loadRozdeleni(false, null,
                            RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), self::getRozdeleni($this->database),
                            true, $obdobiOd, $obdobiDo);
                    }

                    $prava = Prava::vratPrava($this->report->prava);
                    if ($this->report->zobrazitPanelOsoba) {
                        $this->comboOsoba = new ComboBoxZamestnanec('osoba');
                        $this->comboOsoba->htmlParams = [
                            'id' => 'osoba',
                            'data-columns' => 'jmeno;osobni-cislo;datum-nastupu',
                            'data-captions' => \Ron\ronText('Jmeno') . ';' . \Ron\ronText('Osobni_cislo') . ';' . \Ron\ronText('Datum_nastupu')];
                        $this->comboOsoba->className = 'form-control ronselect';
                        if ($this->report->zobrazitPanelRozdeleni && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP) && self::getJenRozdeleni()) {
                            $this->comboOsoba->nastavFiltrRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP),
                                self::getRozdeleni($this->database), self::getRozdeleniVcetnePodrizenych());
                        }
                        $this->comboOsoba->loadDataPP($this->database, $obdobiOd, $obdobiDo, $prava, false, RonPortalSession::vybranaOsoba());

                        $this->comboTypOsoby = new ComboBox('typOsoby');
                        $this->comboTypOsoby->htmlParams['id'] = 'typOsoby';
                        $this->comboTypOsoby->dataTypeComparison = false;
                        $this->comboTypOsoby->loadSQL($this->database, 'SELECT Kod, Popis FROM S_TypOsoby ORDER BY Kod', null);
                        $this->comboTypOsoby->setSelectedOption(self::getTypOsoby());

                        $this->comboTypPP = new ComboBox('typPP');
                        $this->comboTypPP->htmlParams['id'] = 'typPP';
                        $this->comboTypPP->loadSQL($this->database, 'SELECT Kod, Popis FROM TypPracovnihoPomeru ORDER BY Kod', null);
                        $this->comboTypPP->setSelectedOption(self::getTypPP());
                    }

                    $this->jePravoNaVlastni = $prava->lzeZobrazitRC(User::getRC());

                    if ($this->report->zobrazitPanelKalendar) {
                        $this->comboKalendar = new ComboBox('kalendar');
                        $params = $this->database->createParameters();
                        $this->comboKalendar->loadSQL($this->database, 'SELECT Kod, Popis FROM Kalendar WHERE Aktivni = 1 ORDER BY Popis', $params);
                        $this->comboKalendar->showCode = true;
                        $this->comboKalendar->setSelectedOption(self::getKalendar());
                    }
                } else {
                    MessageManager::addErrorMessage(\Ron\ronText('Nemate_pravo_pro_zobrazeni_zvolene_sestavy'));
                }
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }
    }


    public function renderReportList()
    {
        if (!$this ->reportList) {
            return;
        }

        $kategorie = null;

        echo "<ul class='list-unstyled list-padding reportlist'>\n";
        foreach ($this ->reportList as $report) {
            if (!$report ->prava  ||  Prava::jsouNejakePrava($report ->prava)) {
                if ($report->kategorie && $report->kategorie != $kategorie) {
                    $kategorie = $report->kategorie;
                    echo '<li class="form-section margin-top padding-left">' . StringUtils::htmlTextConvert($kategorie) . '</li>';
                }

                $classStr = $report->appReportName ? 'managerReport' : 'IWReport';

                if ($this->report && $this->report->kod == $report->kod) {
                    $classStr = ' active font-weight-bold';
                }

                echo
                    "<li class='$classStr'>" .
                    '<a href="?menuid=reports&reportname=' . StringUtils::htmlTextConvert($report->kod) . '">' .
                    StringUtils::htmlTextConvert($report->nazev) .
                    '</a>' .
                    '</li>';
            }
        }
        echo "</ul>\n";

    }


    public static function getPostPP($database, $report)
    {
        $pp = null;
        if ($report ->zobrazitPanelOsoba) {
            self::setJenZvolenaOsoba(isset($_POST['jenOsoba'])  &&  $_POST['jenOsoba']);

            if (self::getJenZvolenaOsoba()) {
                $pp = PracovniPomer::nactiPP($database, $_POST['osoba']);
                $pp ->vyberNactenyPP();
            }

            self::setJenTypOsoby(isset($_POST['jenTypOsoby'])  &&  $_POST['jenTypOsoby']);
            self::setTypOsoby(isset($_POST['typOsoby']) ? (int)$_POST['typOsoby'] : 0);

            self::setJenTypPP(isset($_POST['jenTypPP'])  &&  $_POST['jenTypPP']);
            self::setTypPP(isset($_POST['typPP']) ? $_POST['typPP'] : '');
        }

        return $pp;
    }


    public static function getPostRozdeleni($database, $report)
    {
        if (!$report ->zobrazitPanelRozdeleni) {
            return null;
        }

        $jenRozdeleni = isset($_POST['jenRozdeleni'])  &&  $_POST['jenRozdeleni'];
        $rozdeleni = null;

        if ($jenRozdeleni  &&  RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)  &&  $_POST['rozdeleni'] != '') {
            $rozdeleni = Rozdeleni::loadRozdeleni($database, RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $_POST['rozdeleni']);
        }

        self::setRozdeleni($jenRozdeleni,
            $rozdeleni ? $rozdeleni ->typ : null,
            $rozdeleni ? $rozdeleni ->kod : null,
            isset($_POST['vcetnePodrizenych']) &&  $_POST['vcetnePodrizenych']);

        return $rozdeleni;
    }


    /**
     * @param Database $database
     * @param Report $report
     * @param string $format
     * @return ReportOutputObject
     * @throws Exception
     */
    public static function getXmlReportOutputObject($database, $report, $format)
    {
        $result = new ReportOutputObject($report, $format);

        if ($report ->zobrazitPanelObdobi) {
            $typObdobi = (int)$_POST['typObdobi'];
            self::setZvolenyTypObdobi($typObdobi);

            switch ($typObdobi) {
                case 1 :    // ctvrtleti
                    $obdobiOd = DateUtils::firstDayOfQuarter(DateUtils::dateStringToTimestamp($_POST['obdobiCtvrtleti']));
                    $obdobiDo = DateUtils::incDay(DateUtils::incMonth($obdobiOd, 3), -1);
                    self::setZvoleneObdobiOd(date('j.n.Y', $obdobiOd));
                    self::setZvoleneObdobiDo(date('j.n.Y', $obdobiDo));

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 2 :    // rok
                    $obdobiOd = DateUtils::firstDayOfYear(DateUtils::dateStringToTimestamp($_POST['obdobiRok']));
                    $obdobiDo = DateUtils::incDay(DateUtils::incMonth($obdobiOd, 12), -1);
                    self::setZvoleneObdobiOd(date('j.n.Y', $obdobiOd));
                    self::setZvoleneObdobiDo(date('j.n.Y', $obdobiDo));

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 3 :    // datum od-do
                    $obdobiOd = DateUtils::dateStringToTimestamp($_POST['obdobiOd']);
                    self::setZvoleneObdobiOd($_POST['obdobiOd']);

                    $obdobiDo = DateUtils::dateStringToTimestamp($_POST['obdobiDo']);
                    self::setZvoleneObdobiDo($_POST['obdobiDo']);

                    $result ->nastavObdobi($typObdobi, $obdobiOd, $obdobiDo);
                    break;

                case 0 :
                default :
                    $pomObdobi = DateUtils::dateStringToTimestamp($_POST['obdobiMesic']);
                    try {
                        $obdobi = Obdobi::vratObdobi($database, $report->applicationId ?: 'ADS', $pomObdobi);
                        RonPortalSession::setObdobi($obdobi->zacatek, $obdobi->konec);
                        $result->nastavObdobi($typObdobi, $obdobi->zacatek, $obdobi->konec, $obdobi->popis);
                    } catch (Exception $e) {
                        // pro Personalistiku/APS urci mesic jen podle zadaneho datum
                        $result->nastavObdobi($typObdobi, DateUtils::firstDayOfMonth($pomObdobi), DateUtils::lastDayOfMonth($pomObdobi), date('n/Y', $pomObdobi));
                    }
                    break;
            }
        }

        if ($report ->zobrazitPanelOsoba) {
            $pp = self::getPostPP($database, $report);
            $result ->nastavPP($pp, false); // prava se kontroluji az v sestave

            $result ->nastavTypOsoby(self::getJenTypOsoby(), self::getTypOsoby());
            $result ->nastavTypPP(self::getJenTypPP(), self::getTypPP());
        }

        if ($report ->zobrazitPanelKalendar) {
            $jenKalendar = isset($_POST['jenKalendar']) && $_POST['jenKalendar'];
            self::setJenKalendar($jenKalendar);

            if ($jenKalendar) {
                $kalendar = Kalendar::nactiKalendar($database, $_POST['kalendar']);
                self::setKalendar($kalendar->kod);
            } else {
                $kalendar = null;
            }

            $result ->nastavKalendar($jenKalendar, $kalendar);
        }

        if ($report ->zobrazitPanelRozdeleni) {
            $rozdeleni = self::getPostRozdeleni($database, $report);

            $typRozdeleni = $rozdeleni ? TypRozdeleni::getTypRozdeleni($database, $rozdeleni->typ) : null;

            $result ->nastavRozdeleni(self::getJenRozdeleni(), $rozdeleni, self::getRozdeleniVcetnePodrizenych(), $typRozdeleni);
        }

        $order = (isset($_POST['order'])  &&  $_POST['order'])
            ? $_POST['order']
            : null;

        if ($order  &&  $text = TextManager::getInstance() ->getTranslatedText('report_sort_' . strtolower($order), '')) {
            NastaveniUzivatele::setUserSettingsItem($database, 'reports_sort', $order);
            // ordering a ordering_name
            $result ->nastavRazeni($order, $text);
        }

        return $result;
    }
}
