<?php

namespace RonPortal\Common;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class RozdeleniOsobRecord extends RonPortalDBRecord
{
    /** @var DBRecordField */
    public $fRC;
    /** @var DBRecordField */
    public $fOsobniCislo;
    /** @var DBRecordField */
    public $fDatumNastupu;

    /** @var DBRecordField */
    public $fKodTypuRozdeleni;
    /** @var DBRecordField */
    public $fRozdeleni;

    /** @var DBRecordField */
    public $fPlatnostOd;
    /** @var DBRecordField */
    public $fPlatnostDo;


    /**
     * RozdeleniOsobRecord constructor.
     * @param Database $database
     * @param object|string $logDescription
     * @param PracovniPomer $pp
     * @param int $kodTypuRozdeleni
     * @param string $rozdeleni
     */
    public function __construct($database, $logDescription, $pp = null, $kodTypuRozdeleni = null, $rozdeleni = null)
    {
        parent::__construct($database, 'RozdeleniOsob', $logDescription);

        $this ->fRC = $this->addStringField('RC', true, 15, $pp ? $pp ->rc : null);
        $this ->fOsobniCislo = $this->addStringField('OsobniCislo', true, 15, $pp ? $pp ->osobniCislo : null);
        $this ->fDatumNastupu = $this->addDateField('DatumNastupu', true, $pp ? $pp ->datumNastupu : null);

        $this ->fKodTypuRozdeleni = $this->addIntField('KodTypuRozdeleni', true, $kodTypuRozdeleni);
        $this ->fRozdeleni = $this->addStringField('Rozdeleni', true, 30, $rozdeleni);

        $this ->fPlatnostOd = $this->addDateField('PlatnostOd');
        $this ->fPlatnostDo = $this->addDateField('PlatnostDo');

        $where = '';
        $params = $database ->createParameters();

        if ($pp) {
            $where .=
                ($where ? ' AND ' : '') .
                'RC = ' . $params->addParameter($pp->rc) .
                ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp->datumNastupu));
        }

        if ($kodTypuRozdeleni) {
            $where .=
                ($where ? ' AND ' : '') .
                'KodTypuRozdeleni = ' . $params ->addParameter($kodTypuRozdeleni);
        }
        if (isset($rozdeleni)) {
            $where .=
                ($where ? ' AND ' : '') .
                'Rozdeleni = ' . $params ->addParameter($rozdeleni);
        }

        if ($where) {
            $this ->setWhere($where, $params);
        }
    }

}
