<?php
/** UTF-8
 * roman.szablatura, 10.10.2016
 */

namespace RonPortal\Common;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterSwitchLinks;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RozdeleniUtils;
use RonPortal\User;

/**
 * Class UkolFilterForm
 */
class UkolFilterForm extends RonPortalFilterForm
{

    /**
     * UkolFilterForm constructor.
     * @param Database $database
     */
    function __construct(Database $database)
    {
        parent::__construct($database, 'ukoly', true);

        $prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"

        $filtrResitele = $prava->pouzeSebe()
            ? 'mojeukoly'
            : ($_GET['filtrresiteleukolu'] ?? $_SESSION['filterform_filtrresiteleukolu'] ?? 'mojeukoly');


        if (!$prava->pouzeSebe()) {
            // prepinani moje / vsechny
            $this ->addFilter(
                (new FilterFormFilter('filtrresiteleukolu'))
                    ->addItem(
                        (new FilterFormFilterItem('mojeukoly', TextManager::_('Moje_ukoly')))
                            ->setUrl('?menuid=ukoly&filtrresiteleukolu=mojeukoly')
                    )
                    ->addItem(
                        (new FilterFormFilterItem('ukolyvsech', TextManager::_('Vse')))
                            ->setUrl('?menuid=ukoly&filtrresiteleukolu=ukolyvsech')
                    )
                    ->setDefaultValue('mojeukoly')
                    ,
                true
            );
        }

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setCaption(TextManager::_('Termin_ukolu') . ' ' . TextManager::_('Od')),
            true
        );

        if ($filtrResitele === 'ukolyvsech') {
            $this ->addFilterRozdeleni1();
            $this ->addFilterRozdeleni2();

            $this ->addFilterPP(true, $prava)
                ->setDatumOd(time())
                ->setDatumDo(time())
                ->setDefaultValue('');
        }

        $this ->addFilter(
            (new FilterFormFilter('typukolu', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('Typ_ukolu'))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM KategorieUkolu WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1', 'Popis'
                )
                ->setSqlFilter('KategorieUkolu.kod = ')
        );

        $this ->addFilter(
            (new FilterFormFilter('prubehukolu', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('prubeh_ukolu'))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM PrubehUkoluOsoby WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1', 'Popis'
                )
                ->setSqlFilter('UkolOsoby.PrubehUkoluOsoby = ')
        );

        $this ->addFilter(
            (new FilterFormFilter('prioritaukolu', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('priorita_ukolu'))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql(
                    'SELECT Kod, Popis FROM PrioritaUkoluOsoby WHERE %FILTER% ORDER BY %ORDER%',
                    '1 = 1', 'Popis'
                )
                ->setSqlFilter('UkolOsoby.PrioritaUkoluOsoby = ')
        );

        $this ->addFilter(
            (new FilterFormFilterSwitchLinks('stavukolu'))
                ->addItem('', TextManager::_('Vse'))
                ->addItem('nesplnene', TextManager::_('Nesplnene'))
                ->addItem('poterminu', TextManager::_('Po_terminu_nesplnene'))
                ->addItem('splnene', TextManager::_('Splnene'))
                ->addItem('mamhodnotit', TextManager::_('Mam_hodnotit'))
                ->addItem('mamschvalit', TextManager::_('Mam_schvalit'))
                ->addItem('bezresitele', TextManager::_('Bez_resitele'))
        );

        $this ->addFilter(
            (new FilterFormFilter('cekajici', FilterFormFilter::TYPE_TEXT))
                ->setDefaultValue(TextManager::_('ukoly_cekajici'))
                ->setUrl('?menuid=ukoly&stavukolu_nesplnene=1&stavukolu_mamhodnotit=1&stavukolu_mamschvalit=1')
        );

        if (Prava::vratPrava('ukoly_vlozeni') ->jsouNejakePrava()) {
            $this ->addFilter(
                (new FilterFormFilter('insert_link', FilterFormFilter::TYPE_TEXT))
                    ->setDefaultValue("<a href='?menuid=ukoly&formaction=new'>" . TextManager::_('novy_ukol') . '</a>')
                    ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                    ->setRowClass('margin-top-2')
                    ->setCaption(false)
            );
        }
    }
}


class PrekazkaBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // pokud neni v ukolu nastaveno "Prekazka", tak nezobrazuje nic (bud prekazka neni zadana, nebo jiz byla odstranena)
        if (!$rowData['col-prekazka']) {
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class StavUkoluBrowserField
 */
class StavUkoluBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value == 1) {  // UkolOsoby.StavUkolu = splnen
            return TextManager::_('Str_Yes');
        }

        if (!$value) {  // probiha
            return ($rowData['col-prekazka'] == 1)  // UkolOsoby.Prekazka
                ? TextManager::_('Prekazka_ukolu')
                : TextManager::_('Str_No');
        }

        // -1 = zruseno - nezobrazuji se
        return '';
    }
}



class OsobaUkoluField extends JmenoPrijmeniBrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        if ($value  &&  User::hasRC($value)) {
            return '<strong>' . $result . '</strong>';
        }

        return $result;
    }
}



/**
 * Class Ukol_TerminField
 */
class Ukol_TerminField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        $class = '';
        //Pokud existuje zaznam DatumSplneni_Skutecne, pak se promenna $class naplni hodnotou 'splnene'
        if ($rowData['col-datumsplneni']) {
            $class = 'splnene';
        } else {
            //ulozeni hodnoty datumu - DatumSplneni_Narizene
            $datum = $this ->database ->stringToTimestamp($value);
            //Pokud je aktualni datum vyssi nez DatumSplneni_Narizene je pouzito k obarveni data tridy po_terminu
            if (DateUtils::endOfTheDay($datum) < time()) {
                $class = 'po_terminu';
            }
        }

        //Vysledne obarveni datumu na zaklade splneni nebo po_terminu
        if ($class) {
            return
                '<div class="' . $class . '">' . $result . '</div>';
        }

        return $result;
    }
}

/**
 * Class UkolyBrowser
 */
class UkolyBrowser extends RonPortalBrowser
{
    public $prava;    //Prava na prohlizeni a "splneni"

    /**
     * SeznamUkoluBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'ukoly');

        $this->prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"

        $this->title = TextManager::_('Seznam_ukolu');
        $this->code = 'browserukoly';

        $this->sqlSelectPart =
            ' UkolOsoby.KodSerial'; // 0

        $this->sqlFromPart =
            '((((((((UkolOsoby LEFT OUTER JOIN Osoba Resitel' .
            '           ON UkolOsoby.RC_Osoby = Resitel.RC ' .
            '             )' .
            '          LEFT OUTER JOIN Osoba Vlozil ' .
            '          ON UkolOsoby.RC_Vlozil = Vlozil.RC ' .
            '          )' .
            '        LEFT OUTER JOIN Osoba Zadavatel ' .
            '        ON UkolOsoby.RC_KdoZadal = Zadavatel.RC ' .
            '        )' .
            '       LEFT OUTER JOIN Osoba Hodnotitel ' .
            '       ON UkolOsoby.RC_Hodnotitele_Def = Hodnotitel.RC' .
            '       )'.
            '     LEFT OUTER JOIN Osoba VazanaOsoba ' .
            '     ON UkolOsoby.RC_OsobaUkolu = VazanaOsoba.RC' .
            '     )'.
            '    LEFT OUTER JOIN KategorieUkolu ' .
            '    ON KategorieUkolu.Kod = UkolOsoby.KodKategorieUkolu ' .
            '    )' .
            '   LEFT OUTER JOIN PrubehUkoluOsoby ' .
            '   ON PrubehUkoluOsoby.Kod = UkolOsoby.PrubehUkoluOsoby ' .
            '   )' .
            '  LEFT OUTER JOIN PrioritaUkoluOsoby ' .
            '  ON PrioritaUkoluOsoby.Kod = UkolOsoby.PrioritaUkoluOsoby ' .
            ')';

        $this->primaryField = 0;

//        $this->najdiUkolyVyzadujiciPozornost($database);

        // termin
        $this->addField(new Ukol_TerminField($database))
            ->setSqlFieldName('UkolOsoby.DatumSplneni_Narizene') ->setHeaderText('Termin_ukolu')
            ->setCode('col-datumsplneni_narizene')
            ->setFormat('finedate') ->setLink('?menuid=ukoly')
            ->setOrdering(true);

        $this ->defaultSort = 'col-datumsplneni_narizene';

        //Splneno - Ano/Ne/Prekazka
        $this->addField(new StavUkoluBrowserField($this->database))
            ->setSqlFieldName('UkolOsoby.StavUkolu') ->setHeaderText('Splneno')
            ->setCode('col-stavukolu') ->setOrdering(true)
            ->setLink('?menuid=ukoly');

        //Datum splneni
        $this->addField()
            ->setSqlFieldName('UkolOsoby.DatumSplneni_Skutecne') ->setHeaderText('Datum_splneni')
            ->setCode('col-datumsplneni')  // ->setVisible($this ->filterForm ->stavSplnene) - zruseno - filtr se vytvari az dodatecne
            ->setFormat('finedate') ->setOrdering(true);


        // Zadano osobe - pokud jsou vybrany vsechny osoby pak sloupec skryt
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Resitel.Jmeno')
            ->setCode('col-resitel-jmeno');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Resitel.Prijmeni')
            ->setCode('col-resitel-prijmeni');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Resitel.Titul')
            ->setCode('col-resitel-titul');

        $this->addField(new OsobaUkoluField($this ->database, 'col-resitel-jmeno', 'col-resitel-prijmeni', null, 'col-resitel-titul'))
            ->setSqlFieldName('UkolOsoby.RC_Osoby') ->setHeaderText('Resitel')
            ->setCode('col-resitel') ->setOrdering(true)
            ->setSortSQL('Resitel.Prijmeni, Resitel.Jmeno')
            ->setSortSQLDesc('Resitel.Prijmeni DESC, Resitel.Jmeno DESC');

        // typ ukolu - v KODAMU(kategorie)
        $this->addField()
            ->setSqlFieldName('KategorieUkolu.Popis') ->setHeaderText('Typ_ukolu') ->setOrdering(true)
            ->setCode('col-kategorie');

        // popis
        $this->addField()
            ->setSqlFieldName('UkolOsoby.ZkracenyPopis') ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setLink('?menuid=ukoly') ->setOrdering(true) ->setWrap(true);

        // Datum zadani
        $this->addField()
            ->setSqlFieldName('UkolOsoby.DatumZadani') ->setHeaderText('Datum_zadani')
            ->setCode('col-datumzadani')
            ->setFormat('finedate') ->setOrdering(true);

        // Zadal
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zadavatel.Jmeno')
            ->setCode('col-zadavatel-jmeno');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zadavatel.Prijmeni')
            ->setCode('col-zadavatel-prijmeni');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zadavatel.Titul')
            ->setCode('col-zadavatel-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-zadavatel-jmeno', 'col-zadavatel-prijmeni', null, 'col-zadavatel-titul'))
            ->setSqlFieldName('UkolOsoby.RC_KdoZadal')->setHeaderText('Zadavatel') ->setOrdering(true)
            ->setCode('col-zadavatel')
            ->setSortSQL('Zadavatel.Prijmeni, Zadavatel.Jmeno')
            ->setSortSQLDesc('Zadavatel.Prijmeni DESC, Zadavatel.Jmeno DESC');

        // Vlozil
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Vlozil.Jmeno')
            ->setCode('col-vlozil-jmeno');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Vlozil.Prijmeni')
            ->setCode('col-vlozil-prijmeni');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Vlozil.Titul')
            ->setCode('col-vlozil-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-vlozil-jmeno', 'col-vlozil-prijmeni', null, 'col-vlozil-titul'))
            ->setSqlFieldName('UkolOsoby.RC_Vlozil')->setHeaderText('Vlozil') ->setOrdering(true)
            ->setCode('col-vlozil')
            ->setSortSQL('Vlozil.Prijmeni, Vlozil.Jmeno')
            ->setSortSQLDesc('Vlozil.Prijmeni DESC, Vlozil.Jmeno DESC');

        // Hodnotitel
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Hodnotitel.Jmeno')
            ->setCode('col-hodnotitel-jmeno');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Hodnotitel.Prijmeni')
            ->setCode('col-hodnotitel-prijmeni');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Hodnotitel.Titul')
            ->setCode('col-hodnotitel-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-hodnotitel-jmeno', 'col-hodnotitel-prijmeni', null, 'col-hodnotitel-titul'))
            ->setHeaderText('Hodnotitel') ->setOrdering(true)
            ->setCode('col-hodnotitel')
            ->setSortSQL('Hodnotitel.Prijmeni, Hodnotitel.Jmeno')
            ->setSortSQLDesc('Hodnotitel.Prijmeni DESC, Hodnotitel.Jmeno DESC');

        // Hodnoceno
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Hodnoceno') ->setHeaderText('Hodnoceno') ->setOrdering(true) ->setAlign('center')
            ->setCode('col-hodnoceno')
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);

        // zjisti, jestli existuje typ ukolu s nastavenou vazbou
        $cursor = $database ->getSqlResult('SELECT MAX(PopisVazbyNaOsobu) FROM KategorieUkolu');
        $rowVazba = $cursor ->fetch();
        $existujeVazba = $rowVazba  &&  $rowVazba[0];
        $cursor ->close();

        // Vazana osoba - popis
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN) // skryto - je v titulku sloupce, lze zobrazit pomoci xml nastaveni
            ->setSqlFieldName('KategorieUkolu.PopisVazbyNaOsobu') ->setHeaderText('Vazana_osoba_popis') ->setOrdering(true)
            ->setCode('col-popisvazby');

        // Vazana osoba - nacteni jmena atd.
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VazanaOsoba.Jmeno') ->setVisible($existujeVazba)
            ->setCode('col-vazanaosoba-jmeno');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VazanaOsoba.Prijmeni') ->setVisible($existujeVazba)
            ->setCode('col-vazanaosoba-prijmeni');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VazanaOsoba.Titul') ->setVisible($existujeVazba)
            ->setCode('col-vazanaosoba-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-vazanaosoba-jmeno', 'col-vazanaosoba-prijmeni', null, 'col-vazanaosoba-titul'))
            ->setHeaderText('Vazana_osoba_prijmeni_jmeno') ->setOrdering(true)
            ->setCode('col-vazanaosoba') ->setVisible($existujeVazba) // jen pokud existuje nejaky typ ukolu s nastavenou vazbou
            ->setSortSQL('VazanaOsoba.Prijmeni, VazanaOsoba.Jmeno')
            ->setSortSQLDesc('VazanaOsoba.Prijmeni DESC, VazanaOsoba.Jmeno DESC')
            ->setTitleFieldIndex('col-popisvazby');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('UkolOsoby.PodrobnyPopis') ->setCastWideMemo(true)
            ->setCode('col-podrobnypopis')
            ->setClassName('browserpoznamka')
            ->setHeaderColSpan(-1)
            ->setHeaderText(TextManager::_('Detailni_popis') . ': ');


        // prekazka
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('UkolOsoby.Prekazka')
            ->setCode('col-prekazka');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('UkolOsoby.PoznamkaKPrekazce') ->setCastWideMemo(true)
            ->setCode('col-poznamkakprekazce');

        $this ->addField((new PrekazkaBrowserField($database))
            ->setType(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('UkolOsoby.DatumZapisuPrekazky')
            ->setFormat('finedatetime')
            ->setCode('col-prekazka-info')
            ->setClassName('browserpoznamka prekazkaukolu color-red')
            ->setHeaderColSpan(-1)
            ->setHeaderText(TextManager::_('Prekazka_ukolu') . ': ')
            ->setSqlFieldIndex2('col-poznamkakprekazce')
            ->setSqlFieldIndex2Separator(' - ')
            ->setWrap(true)
        );

        // Prubeh ukolu
        $this->addField()
            ->setSqlFieldName('PrubehUkoluOsoby.Popis') ->setHeaderText('prubeh_ukolu') ->setOrdering(true)
            ->setCode('col-prubeh');
        // Priorita ukolu
        $this->addField()
            ->setSqlFieldName('PrioritaUkoluOsoby.Popis') ->setHeaderText('priorita_ukolu') ->setOrdering(true)
            ->setCode('col-priorita');

        //kod kategrorie (typu ukolu)
        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('KategorieUkolu.Kod')
            ->setCode('col-kategorie-kod');

        //pocet osob prirazenych ke kategorii, pokud jsou prirazene osoby - mohou ji videt jen osoby zde prirazene, resitel, zadavatel nebo osoba, ktera ukol vlozila
        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('(SELECT COUNT(OsobaKategorieUkolu.RC) FROM OsobaKategorieUkolu WHERE OsobaKategorieUkolu.kodKategorieUkolu = KategorieUkolu.Kod)')
            ->setCode('col-kategorie-pocetosob');
    }


    function setFilterForm()
    {
        $this->filterForm = new UkolFilterForm($this->database);
    }



    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $render = true;

        if (!empty($values['col-kategorie-kod']) && (int)$values['col-kategorie-pocetosob'] > 0) {
            $params = $this->database->createParameters();
            $prirazenUzivatel = DBUtils::getSqlValue($this->database, '(SELECT COUNT(OsobaKategorieUkolu.RC) FROM OsobaKategorieUkolu WHERE OsobaKategorieUkolu.kodKategorieUkolu = ' . $params->addParameter($values['col-kategorie-kod']) .
                ' AND OsobaKategorieUkolu.RC = ' . $params->addParameter(User::getRC()) . ')', $params);

            //pokud jsou prirazene osoby - mohou ji videt jen osoby zde prirazene
            if ($prirazenUzivatel == 0) {
                $render = false;
            }
            //nebo uzivatel, ktery je resitel, zadavatel nebo osoba, ktera ukol vlozila
            if (User::hasRC($values['col-resitel']) || User::hasRC($values['col-zadavatel']) || User::hasRC($values['col-vlozil'])) {
                $render = true;
            }
        }

        if ($render) {
            parent::renderLine($values, $className, $rowNumber);

            $this->renderSeparator();
        }
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'StavUkolu <> -1'; // nezobrazovat zrusene

        $filtrPrava = $this ->prava ->sqlFiltr('Resitel', time(), time(), $params, false, true);
        if ($filtrPrava) {
            $result .=
                ($result ? ' AND ' : '') .
                $filtrPrava;
        }

        // Filtr na osobu, pripadne na rozdeleni
        if ($this ->filterForm ->getFilterValue('filtrresiteleukolu') !== 'ukolyvsech' || $this->prava->pouzeSebe()) {
            // jen vlastni
            $result .=
                ($result ? ' AND ' : '') .
                'UkolOsoby.RC_Osoby = ' . $params ->addParameter(User::getRC()) .
                "\n";
        } else if ($this ->filterForm ->getFilterValue('osoba')) {
            // vybrana osoba
            $result .=
                ($result ? ' AND ' : '') .
                'UkolOsoby.RC_Osoby = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) . "\n";
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                if ($rozdeleniFilter ->getValue() !== null) {
                    $filtrRozdeleni = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'Resitel',
                        $rozdeleniFilter->getValue('typRozdeleni'),
                        $rozdeleniFilter->getValue(),
                        time(), time(),
                        $rozdeleniFilter->getValue('vcetnePodrizenych'),
                        $params,
                        true); // $jenRC = true

                    if ($filtrRozdeleni) {
                        $result .=
                            ($result ? ' AND ' : '') .
                            '(' . $filtrRozdeleni . ')' .
                            "\n";
                    }
                }
            }
        }

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' DatumSplneni_Narizene >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                ' AND DatumSplneni_Narizene <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        //Filtr stav ukolu
        $filtrStavUkolu = '';
        //if ($this ->filterForm ->stavNesplnene) {
        if ($this ->filterForm ->getFilterValue('stavukolu', 'nesplnene')) {
            $filtrStavUkolu .=
                'UkolOsoby.StavUkolu = 0';
        }

//        if ($this ->filterForm ->stavPoTerminu) {
        if ($this ->filterForm ->getFilterValue('stavukolu', 'poterminu')) {
            $filtrStavUkolu .=
                ($filtrStavUkolu ? ' OR ' : '') .
                '(DatumSplneni_Skutecne IS NULL AND DatumSplneni_Narizene < ' . $params ->addDateParameter(time()) . ')';
        }

//        if ($this ->filterForm ->stavSplnene) {
        if ($this ->filterForm ->getFilterValue('stavukolu', 'splnene')) {
            $filtrStavUkolu .=
                ($filtrStavUkolu ? ' OR ' : '') .
                'DatumSplneni_Skutecne IS NOT NULL';
        }

        //Aby se vyfiltroval musi byt ukol splnen, musi byt jeste neohodnocen a musi RC_Hodnotitele_Def souhlasit s prihlasenym uzivatelem
//        if ($this ->filterForm ->stavMamHodnotit) {
        if ($this ->filterForm ->getFilterValue('stavukolu', 'mamhodnotit')) {
            $filtrStavUkolu .=
                ($filtrStavUkolu ? ' OR ' : '') .
                '(UkolOsoby.StavUkolu = 1 AND UkolOsoby.Hodnoceno = 0 AND UkolOsoby.RC_Hodnotitele_Def = ' . $params ->addParameter(User::getRC()) . ')';
        }

        //Aby se vyfiltroval musi byt ukol splnen, musi byt neschvaleny a musi RC_Schvalovatele1_Def nebo RC_Schvalovatele2_Def souhlasit s prihlasenym uzivatelem
//        if ($this ->filterForm ->stavMamSchvalit) {
        if ($this ->filterForm ->getFilterValue('stavukolu', 'mamschvalit')) {
            $filtrStavUkolu .=
                ($filtrStavUkolu ? ' OR ' : '') .
                ' ((UkolOsoby.StavUkolu = 1 AND UkolOsoby.Hodnoceno = 1) ' .
                ' AND ((UkolOsoby.Schvaleno1 = 0 AND UkolOsoby.RC_Schvalovatele1_Def = ' . $params ->addParameter(User::getRC()) . ')' .
                ' OR (UkolOsoby.Schvaleno1 = 1 AND UkolOsoby.Schvaleno2 = 0 AND UkolOsoby.RC_Schvalovatele2_Def = ' . $params ->addParameter(User::getRC()) . ')' .
                "))\n";
        }

        if ($this ->filterForm ->getFilterValue('stavukolu', 'bezresitele')) {
            $filtrStavUkolu .=
                ($filtrStavUkolu ? ' OR ' : '') .
                ' (UkolOsoby.RC_Osoby IS NULL)' .
                "\n";
        }



        if ($filtrStavUkolu) {
            $result .=
                ($result ? ' AND ' : '') .
                '(' . $filtrStavUkolu . ')';
        }

        return new SqlStatement($result, $params);
    }
}
