<?php
/**
 * pavel.rudol, 05.06.2024 9:59
 */

namespace RonPortal\Controller;

use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\TextManager;
use RonPortal\Dochazka\ENeschopenkaBrowser;
use RonPortal\Dochazka\ENeschopenkaDialog;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

class EneschopenkaController extends JsonController
{
    /** @var Database */
    private $database;

    /** @var PravaNaOsoby */
    private $prava;

    public function __construct($database)
    {
        $this->database = $database;
    }

    private function jePravoNaZaznam($cisloDokladu)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT RC, DatumPocatkuDavky, DatumUkonceniDavky FROM ENeschopenka WHERE CisloDokladu = " . $params->addParameter($cisloDokladu);

        $cursor = $this->database->select($sql, $params);

        $row = $cursor->fetch();

        if (!($this->prava->lzeZobrazitRC($row[0]) && $this->prava->priznak)) {
            throw new NeniPravoException();
        }
    }

    /**
     * Nacte data dialogu pro editaci radku
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function nactiZaznam($output)
    {
        $cisloDokladu = $_GET['record'];

        $this->jePravoNaZaznam($cisloDokladu);

        $f = new ENeschopenkaDialog($this->database, $cisloDokladu);

        $output->addData('data', $f->getJsonData());
    }

    /**
     * Nacte data dialogu pro editaci radku
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function editujZaznam($output)
    {
        $cisloDokladu = $_POST['CisloDokladu'];

        $this->jePravoNaZaznam($cisloDokladu);

        $f = new ENeschopenkaDialog($this->database, $cisloDokladu);
        $f->utf8 = true;

        if (!$f->handleAction()) {
            $output->addMessages(MessageManager::getInstance());
        }
    }

    function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'dialog' :
                $this->nactiZaznam($output);
                break;

            case 'edit' :
                $this->editujZaznam($output);
                break;
        }
    }

    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('dochazka.ini');

        $this->prava = Prava::vratPrava('eneschopenka_prohl');

        if (!$this->prava->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $neschAction = isset($_GET['action']) ? $_GET['action'] : '';

        //js udalosti, musi byt pred form action, protoze dialog se odesila take s form action a nezpracovalo by se spravne ulozeni z dialogu
        if (($neschAction == 'dialog') || ($neschAction == 'edit')) {
            return parent::handleAction($neschAction);
        }

        $b = new ENeschopenkaBrowser($this->database, $this->prava);
        $b->load();

        return $b;
    }
}