<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use RonPortal\Kalendar;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSession;
use Tracy\Debugger;

/**
 * Obecny browser, ktery zobrazi tabulku, ve ktere bude pro kazdy den radek
 * a na nem zobrazeny prislusny zaznam (napr. planovane smeny)
 * SQL dotaz musi vracet na prvni pozici datum a musi byt podle tohoto data serazeny.
 * Format prvniho pole by mel byt timestamp. Pokud bude pro jeden den vice zaznamu, tak bude ten den na vice radcich.
 *
 * Class DenniZaznamyBrowser
 */
class DenniZaznamyBrowser extends RonPortalBrowser
{
    var $dateFrom; // nastavi se v load
    var $dateTo;


    /**
     * @param Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->cssClass = 'browser browserdenni grid';
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(
            'ADS', 'prohlizeni',
            false, RonPortalBrowser::OSOBY_JEN_S_DOCHAZKOU);
    }


    function nactiSvatky()
    {
        if ($this ->filterForm ->getFilterValue('osoba')) {
            $pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->getFilterValue('osoba'));
            Kalendar::nactiSvatkyOsoby($this ->database, $pp, $this ->dateFrom, $this ->dateTo);
        }
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        $this ->dateFrom = $this ->filterForm
            ? $this ->filterForm ->getOmezitObdobiOd()
            : DateUtils::firstDayOfMonth(time());

        $this ->dateTo = $this ->filterForm
            ? $this ->filterForm ->getOmezitObdobiDo()
            : DateUtils::lastDayOfMonth(time());

        $this ->nactiSvatky();
    }


    /**
     * Zobrazi pro kazdy den v obdobi minimalne jeden radek.
     * @param Cursor $cursor
     */
    function renderCursor($cursor)
    {
        $datum = $this->dateFrom;
        $i = 1;

        $zaznam = $cursor ->fetch();
        if ($zaznam) {
            $zaznam[0] = $this ->database ->stringToTimestamp($zaznam[0]);
        }

        $browserRowArray = new BrowserRowArray($this ->fields);

        // Pro kazdy den vytvori radek
        while ($datum <= $this->dateTo) {
            $dalsiDatum = DateUtils::incDayStart($datum, 1);

            $zobrazeno = false;

            // Zobrazi vsechny zaznamy pro aktualni den
            while ($zaznam && $zaznam[0] < $dalsiDatum) {
                $browserRowArray ->setData($zaznam);

                $this->renderLine($browserRowArray, Kalendar::getStyleForDate($this ->database, $zaznam[0]), $i);

                $zobrazeno = true;
                $i++;
                $zaznam = $cursor ->fetch();

                if ($zaznam) {
                    $zaznam[0] = $this ->database ->stringToTimestamp($zaznam[0]);
                }
            }

            // Pokud nebyl pro aktualni datum zobrazen zadny zaznam, tak zobrazi prazdny radek
            if (!$zobrazeno) {
                $emptyRow = [$datum];
                // doplni hodnoty null pro ostatni pole (jinak zobrazi nesmysly ve sloupcich, ktere maji zaporny index - napr. v uzivatelsky pridanych)
                foreach ($this ->fields as $index => $field) {
                    if ($index  &&  $field ->sqlFieldName  &&  $field ->visible) {
                        $emptyRow[] = null;
                    }
                }
                $browserRowArray ->setData($emptyRow);

                $this->renderLine($browserRowArray, Kalendar::getStyleForDate($this ->database, $datum), $i);

                $i++;
            }

            $datum = $dalsiDatum;
        }
    }

}
