<?php

namespace RonPortal\Controls;

use Ron\Controls\DateInput;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Filtr pro zadani datumu (a pripadne casu) od - do
 */
class FilterFormFilterDatumOdDo extends FilterFormFilter
{
    const SPEC_DATUM_DO = 'datumDo';
    const SPEC_FILTROVAT_DATUM = 'filtrovatDatum';

    /** @var bool   Urcuje, jestli se ma zobrazit i cas */
    protected $showTime = false;

    /** @var bool   Urcuje, jestli uzivatel zakliknul volbu "filtrovatDatum" */
    protected $filtrovatDatum = false;

    /** @var bool   Urcuje, jestli uzivatel muze zadat prazdnou hodnotu pro datum do */
    protected $povolitPrazdneDatumDo = false;

    /** @var int    Hodnota zadana uzivatelem v poli pro datum do */
    protected $datumDo;

    /** @var string|string[]|array[]    Umoznuje nastavit text, ktery se zobrazi za filtrem pro datum od-do. Pokud se zada pole, lze pouzit "zkratky", napr. ['today', 'currentMonth'] */
    protected $datumOdDoLinks = null;


    public function __construct(RonPortalFilterForm $filterForm)
    {
        parent::__construct('datumod');

        $this ->definesPeriod = true; // filtr omezuje obdobi pro prava a pro dalsi filtry (rozdeleni atd.)

        $this ->filterForm = $filterForm;
        $this->rowId = 'filterrow_datum_od_do'; // pro kompatibilitu starsich uzivatelskych css

        $this ->caption = TextManager::_('Datum_od');
        //$this ->sessionName = 'filterform_datumod'; // pridava se datumod a datumdo a filtrjendatumoddo

//        $this ->htmlAttributes = [
//            'data-ron-autosubmit' => $this ->filterForm ->autoSubmit ? '1' : '0'
//        ];
    }

    /**
     * @return bool
     */
    public function isShowTime(): bool
    {
        return $this->showTime;
    }



    /**
     * @param bool $showTime
     * @return FilterFormFilterDatumOdDo
     */
    public function setShowTime(bool $showTime): FilterFormFilterDatumOdDo
    {
        $this->showTime = $showTime;
        return $this;
    }


    /**
     * @return bool
     */
    public function filtrovatDatum(): bool
    {
        return $this->filtrovatDatum;
    }

    /**
     * @param bool $povolitPrazdneDatumDo
     * @return FilterFormFilterDatumOdDo
     */
    public function setPovolitPrazdneDatumDo(bool $povolitPrazdneDatumDo): FilterFormFilterDatumOdDo
    {
        $this->povolitPrazdneDatumDo = $povolitPrazdneDatumDo;
        return $this;
    }


    /**
     * @return array[]|string|string[]|null
     */
    public function getDatumOdDoLinks()
    {
        return $this->datumOdDoLinks;
    }

    /**
     * @param array[]|string|string[]|null $datumOdDoLinks
     * @return FilterFormFilterDatumOdDo
     */
    public function setDatumOdDoLinks($datumOdDoLinks)
    {
        $this->datumOdDoLinks = $datumOdDoLinks;
        return $this;
    }

    /**
     * @param string|array $link
     * @return $this
     */
    public function addLink($link)
    {
        if (!is_array($this ->datumOdDoLinks)) {
            $this ->datumOdDoLinks = [];
        }

        $this ->datumOdDoLinks[] = $link;

        return $this;
    }


    public function getValue(string $specification = null)
    {
        if (!$this ->required  &&  !$this ->filtrovatDatum) {
            return null;
        }

        switch ($specification) {
            case 'datumOd' :
                return $this ->value;

            case 'datumDo' :
                return $this ->datumDo;

            case 'filtrovatDatum' :
                return $this ->filtrovatDatum;
        }

        return parent::getValue($specification);
    }


    public function storeValue(Database $database)
    {
        parent::storeValue($database);

        $this ->storeValueByName($database, $this ->datumDo, null, '_datumdo');
        if (!$this ->required) {
            $this->storeValueByName($database, $this->filtrovatDatum ? 1 : 0, 0, '_filtrovat');
        }
    }

    public function loadDefaultValue(Database $database)
    {
        parent::loadDefaultValue($database);

        $this ->datumDo = $this ->getDefaultValueByName($database, $this ->defaultValue, '_datumdo');
        if (!$this ->required) {
            $this ->filtrovatDatum = $this ->getDefaultValueByName($database, 0, '_filtrovat');
        }
    }


    /**
     * Nastavi filtrovani datumu podle zadanych hodnot (z POST nebo GET - viz loadDatumOdDo)
     */
    protected function nastavHodnotyDatumOdDo($filtrovat, $datumOdStr, $datumDoStr, $casOdStr, $casDoStr)
    {
        $this ->filtrovatDatum = $this ->required ||
            ($this ->filterForm ->obdobiNeboDatumPovinne ?
                !$this ->filterForm ->getFilterValue('obdobi', 'jenObdobi')
                : $filtrovat);
        try {
            $this->value = $datumOdStr
                ? DateUtils::dateStringToTimestamp($datumOdStr)
                : null;

            if ($this ->showTime  &&  isset($casOdStr)  &&  $casOdStr !== '') {
                $this ->value = DateUtils::timeStringToTimestamp($casOdStr, $this ->value);
            }

            $this->datumDo = ($datumDoStr)
                ? DateUtils::dateStringToTimestamp($datumDoStr)
                : null;

            if (!$this->datumDo && !$this->povolitPrazdneDatumDo) {
                $this->datumDo = $this->value;
            }

            if ($this ->showTime  &&  ($casDoStr ?? '') !== '') {
                $this ->datumDo = DateUtils::timeStringToTimestamp($casDoStr, $this ->datumDo);
            }

            return true;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            return false;
        }
    }

    protected function loadSubmittedValue()
    {
        if (isset($_GET[$this ->name])) {
            return $this ->nastavHodnotyDatumOdDo(
                $this ->required ||
                ($this ->filterForm ->obdobiNeboDatumPovinne
                    ? (int)($_GET['jenobdobi'] ?? 1) == 2
                    : $_GET['jendatumoddo'] ?? null),
                $_GET[$this ->name] ?? null, $_GET['datumdo'] ?? null,
                $_GET['casod'] ?? null, $_GET['casdo'] ?? null);
        }

        return false;
    }


    public function load(Database $database)
    {
        if (!$this->enabled) {
            $this ->filtrovatDatum = false;
            return;
        }

        parent::load($database);

        if ($this ->required) {
            if (!$this ->value) {
                $this ->value = (is_array($this ->defaultValue) ? $this ->defaultValue[''] : $this ->defaultValue) ?? DateUtils::firstDayOfMonth(time());
                $this ->datumDo = (is_array($this ->defaultValue) ? $this ->defaultValue['_datumdo'] : $this ->defaultValue) ?? DateUtils::lastDayOfMonth(time());

                if ($this ->showTime) {
                    $this ->datumDo = DateUtils::incDayStart($this ->datumDo) - 1; // konec dne
                }
            }
        }
    }

    public function getUrlParameters()
    {
        $result = parent::getUrlParameters();
        if (!$this ->required  &&  $this ->filtrovatDatum) {
            if ($this ->filterForm ->obdobiNeboDatumPovinne) {
                $result['jenobdobi'] = 2;
            } else {
                $result['jendatumoddo'] = 1;
            }
        }

        return $result;
    }


    public function renderCaption()
    {
        if ($this->required) {
            echo $this ->caption;
        } else {
            GuiUtils::renderCheckbox(
                $this ->filterForm ->obdobiNeboDatumPovinne ? 'jenobdobi' : 'jendatumoddo',
                $this ->caption, $this ->filtrovatDatum,
                [
                    'data-excluded-checkboxes' => 'input[name=jenobdobi]',
                    'type' => $this ->filterForm ->obdobiNeboDatumPovinne ? 'radio' : 'checkbox'
                ],
                $this ->filterForm ->autoSubmit ? 'clickAutoSubmit' : '',
                $this ->filterForm ->obdobiNeboDatumPovinne ? 2 : 1,
                true // !$this ->datumOdDo_CheckboxFirst
            );
        }
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param string $text
     * @param string[]|null $otherHtmlParams
     * @return void
     */
    public function renderDateLink($datumOd, $datumDo, $text, $otherHtmlParams = null)
    {
        $selected = $this ->value === $datumOd  &&  $this ->datumDo === $datumDo;

        $params = [
            'class' => 'js setValue' . ($selected ? ' selectedlink' : ''),
            'data-trigger-change' => '1',
            'data-submit' => '1',
            'data-selectors-count' => '4',
            // datum od
            'data-selector1' => '#filterrow_datum_od_do input[name=datumod]',
            'data-value1' => $datumOd ? StringUtils::format('date', $datumOd) : '',
            // datum do
            'data-selector2' => '#filterrow_datum_od_do input[name=datumdo]',
            'data-value2' => $datumDo ? StringUtils::format('date', $datumDo) : '',
            // checkbox jen datum od
            'data-selector3' => '#filterrow_datum_od_do input[name=' . ($this ->filterForm ->obdobiNeboDatumPovinne ? 'jenobdobi' : 'jendatumoddo') . ']',
            'data-value3' => '1',
            // checkbox jen obdobi
            'data-selector4' => '#filterrow_obdobi input[name=jenobdobi]',
            'data-value4' => '0'
        ];

        if ($this ->showTime) {
            $params['data-selectors-count'] = '6';
            $params['data-selector5'] = '#filterrow_datum_od_do input[name=casod]';
            $params['data-value5'] = $datumOd ? StringUtils::format('time', $datumOd) : '';
            $params['data-selector6'] = '#filterrow_datum_od_do input[name=casdo]';
            $params['data-value6'] = $datumDo ? StringUtils::format('time', $datumDo) : '';
        }

        if ($otherHtmlParams) {
            $params = array_merge($params, $otherHtmlParams);
        }

        echo
            '<li ' . GuiUtils::getAttributesHtml($params) . '>' .
            $text .
            '</li>';
    }


    /**
     * Umoznuje zobrazit dalsi obsah za vyber datumu od-do
     */
    public function renderDatumOdDoLinks()
    {
        if (is_string($this ->datumOdDoLinks)) {
            echo $this ->datumOdDoLinks;
        } else if (is_array($this ->datumOdDoLinks)) {
            echo "\n<ul class='linkslist margin-left-2'>";

            foreach ($this ->datumOdDoLinks as $item) {
                if (is_string($item)) {
                    switch ($item) {
                        case 'today' :
                        case 'currentDay' :
                            $this ->renderDateLink(DateUtils::today(), DateUtils::today(), TextManager::_('Today'), ['id' => 'datum_od_do_today']);
                            break;

                        case 'currentMonth' :
                            $this ->renderDateLink(DateUtils::firstDayOfMonth(time()), DateUtils::lastDayOfMonth(time()), TextManager::_('Aktualni_mesic'));
                            break;

                        case 'currentYear' :
                            $this ->renderDateLink(DateUtils::firstDayOfYear(time()), DateUtils::lastDayOfYear(time()), TextManager::_('Aktualni_rok'));
                            break;

                        case 'prevMonth' :
                            $this ->renderDateLink(DateUtils::incMonth(DateUtils::firstDayOfMonth(time()), -1), DateUtils::incDay(DateUtils::firstDayOfMonth(time()), -1), TextManager::_('Predchozi_mesic'));
                            break;

                        case 'currentQuarter' :
                            $this ->renderDateLink(
                                DateUtils::firstDayOfQuarter(time()),
                                DateUtils::incDay(DateUtils::incMonth(DateUtils::firstDayOfQuarter(time()), 3), -1),
                                TextManager::_('Aktualni_ctvrtleti'));
                            break;

                        default :
                            echo  $item;
                    }
                } else if (is_array($item)) {
                    // polozka je pole ve tvaru "datumOd" => ..., "datumDo" => ..., "text" => ..., "htmlParams" => []
                    $this ->renderDateLink($item['datumOd'], $item['datumDo'], $item['text'], $item['htmlParams'] ?? null);
                }
            }

            echo "</ul>\n";
        }
    }


    public function render()
    {
        DateInput::renderDateSelect('datumod', date('j.n.Y', $this ->value ?? time()), $this ->filterForm ->autoSubmit); // submitonenter

        if ($this ->showTime) {
            $casOd = date('G:i', $this ->value);
            echo "<input type='text' title='hh:mm' name='casod' value='$casOd' maxlength='5' size='5'>";
        }

        echo '<span class="margin-left-3 margin-right-2"> ' . TextManager::_('Do') . '</span>';
        DateInput::renderDateSelect('datumdo', $this ->datumDo
            ? date('j.n.Y', $this ->datumDo)
            : null,
            $this ->filterForm ->autoSubmit);

        if ($this ->showTime) {
            $casDo = date('G:i', $this ->datumDo);
            echo "<input type='text' title='hh:mm' name='casdo' value='$casDo' maxlength='5' size='5'>";
        }

        $this ->renderDatumOdDoLinks();
    }


}
