<?php

namespace RonPortal\Controls;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Kalendar;


/**
 * Filtr, ktery zobrazi seznam dnu v obdobi jako linky
 */
class FilterFormFilterDny extends FilterFormFilter
{
    /** @var  string|SqlStatement|PodrobnostiDnuSqlBuilder|null   SQL vyraz pro nacteni podrobnosti k jednotlivym dnu - pokud je zadan, musi v prvnim sloupci vracet datum */
    protected $sqlPodrobnostiDnu = null;

    /** @var string|null    Nadpis, ktery se zobrazi na zacatku radku s podrobnostmi (druhy radek pod seznamem dnu) */
    protected $nadpisPodrobnostiDnu = null;

    /** @var array          Seznam podrobnosti k jednotlivym dnum - standardne se nacita SQL dotazem sqlPodrobnostiDnu a kazda polozka pole obsahuje radek nacteny z databaze */
    protected $podrobnosti;

    /** @var array          Seznam CSS class pro jednotlive dny - standardne se nastavi podle typu dne (pracovni/vikend) */
    protected $dayCssClasses;

    /** @var array          Seznam titulku (atribut title) pro jednotlive dny - nastavi se v pripade, ze SQL dotaz pro podrobnosti vraci vice nez 2 sloupce */
    protected $titles;


    public function __construct(RonPortalFilterForm $filterForm, string $name)
    {
        parent::__construct($name);

        $this ->filterForm = $filterForm;

        $this
            ->setRowId('filter_dnyobdobi')
            ->setRowClass('margin-top-2 margin-bottom')
            ->setPosition(self::POSITION_BELOW_FORM);
    }

    /**
     * @return string|SqlStatement|PodrobnostiDnuSqlBuilder|null
     */
    public function getSqlPodrobnostiDnu()
    {
        return $this->sqlPodrobnostiDnu;
    }

    /**
     * @param string|SqlStatement|PodrobnostiDnuSqlBuilder|null $sqlPodrobnostiDnu
     * @return FilterFormFilterDny
     */
    public function setSqlPodrobnostiDnu($sqlPodrobnostiDnu): FilterFormFilterDny
    {
        $this->sqlPodrobnostiDnu = $sqlPodrobnostiDnu;
        return $this;
    }


    /**
     * @return string|null
     */
    public function getNadpisPodrobnostiDnu(): ?string
    {
        return $this->nadpisPodrobnostiDnu;
    }

    /**
     * @param string|null $nadpisPodrobnostiDnu
     * @return FilterFormFilterDny
     */
    public function setNadpisPodrobnostiDnu(?string $nadpisPodrobnostiDnu): FilterFormFilterDny
    {
        $this->nadpisPodrobnostiDnu = $nadpisPodrobnostiDnu;
        return $this;
    }


    /**
     * @param Database $database
     * @return SqlStatement|string|null
     */
    protected function buildSqlPodrobnostiDnu(Database $database)
    {
        return ($this ->sqlPodrobnostiDnu instanceof PodrobnostiDnuSqlBuilder)
            ? $this ->sqlPodrobnostiDnu ->getSqlPodrobnostiDnu($database)
            : $this ->sqlPodrobnostiDnu;
    }


    /**
     * @param array $row    Radek s podrobnostmi nacteny z databaze - prvni prvek je datum, dalsi je text (a dale muze nasledovat title)
     * @return string
     */
    public function getHtmlPodrobnostiKeDni($row)
    {
        return $row[1] ?? '';
    }


    /**
     * @param array $row    Radek s podrobnostmi nacteny z databaze - prvni prvek je datum, dalsi je text (a dale muze nasledovat title)
     * @return string
     */
    public function getHtmlTitlePodrobnostiKeDni($row)
    {
        return $row  &&  count($row) > 2 ? $row[2] : null;
    }


    public function loadCssClasses(Database $database)
    {
        $datum = $this ->filterForm ->getOmezitObdobiOd();

        // nacte kalendar prirazeny vybrane osobe
        $kodKalendare = null;
        $idPP = $this ->filterForm ->getFilterValue('osoba');
        if ($idPP) {
            $kalendare = PrirazeniKalendare::nactiKalendarProObdobi($database, $idPP, $datum, $datum);
            if ($kalendare) {
                $kodKalendare = $kalendare[0] ->kod;
            }
        }

        if ($kodKalendare === null  &&  $this ->filterForm ->getFilterValue('rozdeleni') !== null) {
            $kodKalendare = Kalendar::vratKalendarRozdeleni($database,
                $this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
                $this ->filterForm ->getFilterValue('rozdeleni')
            );
        }

        if ($kodKalendare !== null) {
            Kalendar::nactiSvatkyVObdobi($database,
                $this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(),
                $kodKalendare);
        }

        // nacte css tridu pro jednotlive dny v obdobi
        while ($datum <= $this ->filterForm ->getOmezitObdobiDo()) {
            $className = Kalendar::getStyleForDate($database, $datum, $kodKalendare !== null, $kodKalendare);
            $this ->dayCssClasses[(int)date('j', $datum)] = $className;

            $datum = DateUtils::incDay($datum);
        }
    }


    public function loadPodrobnostiDnu(Database $database)
    {
        $this ->podrobnosti = [];

        $sqlPodrobnosti = $this ->buildSqlPodrobnostiDnu($database);
        if ($sqlPodrobnosti) {
            $cursor = $database ->getSqlResult($sqlPodrobnosti);
            while ($row = $cursor ->fetch()) {
                $date = $database ->stringToTimestamp($row[0]);
                $den = (int)date('j', $date);
                $this ->podrobnosti[$den] = $row;
            }
        }
    }


    public function load(Database $database)
    {
        if (!$this->enabled) {
            return;
        }

        $valueStr = trim($_GET[$this->name] ?? '');

        if ($valueStr !== '') {
            try {
                $datum = DateUtils::dateStringToTimestamp($valueStr);
                $this ->value = $datum;
            } catch (\Exception $e) {
                $this->loadDefaultValue($database);
            }

            $this->storeValue($database);
        } else {
            $this->loadDefaultValue($database);
        }

        if ($this ->value < $this ->filterForm ->getOmezitObdobiOd()  ||  $this ->value > $this ->filterForm ->getOmezitObdobiDo()) {
            $this ->value = ($this ->filterForm ->getOmezitObdobiOd() < time()  &&  $this ->filterForm ->getOmezitObdobiDo() >= DateUtils::today())
                ? DateUtils::today()
                : $this ->filterForm ->getOmezitObdobiOd();
        }

        $this ->loadPodrobnostiDnu($database);
        $this ->loadCssClasses($database);
    }


    public function render()
    {
        echo
            "<div id='{$this ->rowId}' class='{$this ->rowClass}'>" .
            '<input ' . GuiUtils::getAttributesHtml(
                array_merge(
                    [
                        'type' => 'hidden',
                        'name' => $this ->name,
                        'id' => 'filter_dny_obdobi_' . $this ->name,
                        'value' => DateUtils::dateToUrlString($this ->value)
                    ],
                    $this ->htmlAttributes ?: []
                )
            ) .
            '/>';

        $htmlElement = 'span';
        if ($this ->podrobnosti) {
            echo
                '<table><tr>' . ($this ->nadpisPodrobnostiDnu ? '<td></td>' : '');
            $htmlElement = 'td';
        }

        $rowPodrobnosti = $this ->nadpisPodrobnostiDnu
            ? '<td>' . $this ->nadpisPodrobnostiDnu . '</td>'
            : '';

        $datum = $this ->filterForm ->getOmezitObdobiOd();
        while ($datum <= $this ->filterForm ->getOmezitObdobiDo()) {
            $index = (int)date('j', $datum);

            $className =
                'js margin-right padding-all setValue ' . ($this ->dayCssClasses[$index] ?? '') .
                ($datum === $this ->value ? ' font-weight-bold' : '');

            $valueStr = DateUtils::dateToUrlString($datum);

            echo
                "<$htmlElement class='$className' data-selector='#filter_dny_obdobi_{$this ->name}' data-value='$valueStr' data-submit='1'>" .
                date('j.n.', $datum) .
                "</$htmlElement>";

            $titlePodrobnosti = $this ->getHtmlTitlePodrobnostiKeDni($this ->podrobnosti[$index] ?? null);
            $titlePodrobnosti = $titlePodrobnosti != null
                ? ' title="' . StringUtils::htmlTextConvert($titlePodrobnosti) . '"'
                : '';

            $rowPodrobnosti .= "<td class='align-center' $titlePodrobnosti>" . $this ->getHtmlPodrobnostiKeDni($this ->podrobnosti[$index] ?? null) . '</td>';

            $datum = DateUtils::incDay($datum);
        }

        if ($this ->podrobnosti) {
            echo
                '</tr>' .
                "<tr>$rowPodrobnosti</tr>" .
                '</table>';
        }

        echo '</div>';
    }
}

