<?php

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Filtr pro zobrazeni seznamu osob (jen podle RC)
 */
class FilterFormFilterOsoba extends FilterFormFilter
{
    /** @var OsobaFilterComboBox */
    protected $comboOsoby;

    protected $vcetneExternich = false;

    /** @var PravaNaOsoby|PravaNaOsoby[] */
    protected $prava;

    public function __construct(RonPortalFilterForm $filterForm, string $name)
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);

        $this ->filterForm = $filterForm;
        $this ->prava = Prava::vratPrava('prohlizeni');
    }


    /**
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return FilterFormFilterOsoba
     */
    public function setPrava($prava): FilterFormFilterOsoba
    {
        $this->prava = $prava;
        return $this;
    }

    /**
     * @param bool $vcetneExternich
     * @return FilterFormFilterOsoba
     */
    public function setVcetneExternich(bool $vcetneExternich): FilterFormFilterOsoba
    {
        $this->vcetneExternich = $vcetneExternich;
        return $this;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'prava' :
                $this ->setPrava(Prava::vratPrava($value));
                break;

            case 'vcetneExternich' :
                $this ->setVcetneExternich((int)$value);
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    public function getValue(?string $specification = null)
    {
        if ($this ->value  &&  $specification) {
            switch ($specification) {
                case 'rc' :
                    return $this ->comboOsoby ->rc;

                case 'osobniCislo' :
                    return $this ->comboOsoby ->osobniCislo;

                case 'datumNastupu' :
                    return $this ->comboOsoby ->datumNastupu;
            }
        }

        return parent::getValue($specification);
    }


    public function load(Database $database)
    {
        $defaultRows = [];
        if ($this ->items) {
            foreach ($this ->items as $item) {
                $defaultRows[$item ->key] = $item ->caption;
            }
        }

        $this ->comboOsoby = new OsobaFilterComboBox($this ->name);
        $this ->comboOsoby
            ->setClassName($this ->getCssClass())
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setDefaultRows($defaultRows)
            ->setDataTypeComparison(false);

        $this ->loadDefaultValue($database);
        $value = $this ->comboOsoby ->getPostValue($database, $this ->value);

        $this ->comboOsoby ->filtr = $this->optionsSqlFilter;
        $this ->comboOsoby ->defaultRows = $defaultRows;

        $this ->comboOsoby->loadDataPP($database,
            $this ->filterForm ->osobyDatumOd, $this ->filterForm ->osobyDatumDo, $this->prava,
            null,
            $value,
            $this ->vcetneExternich);

        $this ->value = $this ->comboOsoby ->selectedOption ->key ?? null;
        if ($this ->value === '') {
            $this ->value = null;
        }

        $this ->storeValue($database);
    }


    public function render()
    {
        $this ->comboOsoby ->render();
    }
}



class OsobaFilterComboBox extends ComboBoxZamestnanec
{
    /** @var string[]   Seznam jiz nactenych osob, aby se v comboboxu neopakovaly. */
    private $seznamOsob = [];

    function __construct($varName = 'osoba')
    {
        parent::__construct($varName);
        $this ->ronSelectColumns = 'jmeno;osobni-cislo';
        $this ->ronSelectCaptions = TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo');
    }


    /**
     * Prida zaznam pro osobu ze zadaneho DB radku
     * @param Database $database
     * @param array $row
     * @param bool  $rozlisitPP     Urcuje, jestli se maji rozlisit jednotlive PP
     * @param bool  $nastavit       Pokud je true, tak se udaje osoby nastavi do vlastnosti f_...
     */
    protected function pridejOsobu($database, $row, $rozlisitPP, $nastavit)
    {
        if (array_key_exists($row[0], $this ->seznamOsob)) {
            // osoba s RC=$row[0] uz byla pridana (jiny PP) => nedela nic
            return;
        }

        $this ->seznamOsob[$row[0]] = 1;

        parent::pridejOsobu($database, $row, false, $nastavit); // nerozlisovat PP
    }


    /**
     * Nacte seznam osob ze zadaneho dotazu
     * dale nacte promenne pro vybrany zaznam - tj. zaznam z $_POST nebo $selectedvalue (prednost ma hodnota v $_POST)
     * @param Database $database
     * @param SqlStatement|string   $sqlText
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param bool     $emptyRow
     * @param int      $selectedValue
     * @param int      $datumOd
     * @param int      $datumDo
     * @return bool
     * @throws DBException
     */
    function loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo)
    {
        $this ->seznamOsob = [];
        return parent::loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo);
    }
}
