<?php

namespace RonPortal\Controls;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\HttpUtils;
use Ron\TextManager;


/**
 * Filtr pro vyber tydne s proklikem na predchozi a nasledujici
 */
class FilterFormFilterTyden extends FilterFormFilter
{
    public function __construct()
    {
        parent::__construct('datum');
        $this ->defaultValue = DateUtils::firstDayOfWeek(time());
        $this ->definesPeriod = true;
    }

    public function render()
    {
        $prevDateStr = DateUtils::dateToUrlString(DateUtils::firstDayOfWeek(DateUtils::incDay($this->value, -1)));
        $nextDateStr = DateUtils::dateToUrlString(DateUtils::firstDayOfWeek(DateUtils::incDay($this->value, 8)));

        $urlParams =
            (isset($_GET['frame']) ? '&frame=1' : '') .
            (isset($_GET['vyberjidelny']) ? '&vyberjidelny=' . $_GET['vyberjidelny'] : '');

        $aktualniTyden = DateUtils::firstDayOfWeek(time());

        echo "<div class='padding-all-2'>";

        $linkParams = $this ->filterForm ->getUrlParameters(true);

        if (isset($_GET['frame'])) {
            $linkParams['frame'] = '1';
        }


        // o tyden zpet
        echo
            "<span class='margin-right-2'>".
            "<a href='" . HttpUtils::getQueryStringFromParams(array_merge($linkParams, ['datum' => $prevDateStr]))  . "'>" .
            "<i class='fa glyph-left inline-glyph glyph-button'></i>" .
            '</a></span>';

        // aktualne zvolene datum
        echo ($this->value === $aktualniTyden)
            ? TextManager::_('Aktualni_tyden')
            : date('j.n.Y', $this->value);

        // o tyden dal
        echo
            "<span class='margin-left-2'>" .
            "<a href='" . HttpUtils::getQueryStringFromParams(array_merge($linkParams, ['datum' => $nextDateStr]))  . "'>" .
            "<i class='fa glyph-right inline-glyph glyph-button'></i>" .
            '</a></span>';

        if ($this->value !== $aktualniTyden) {
            $aktualniTydenStr = DateUtils::dateToUrlString($aktualniTyden);

            echo
                '<span class="margin-left-2">' .
                "<a href='" . HttpUtils::getQueryStringFromParams(array_merge($linkParams, ['datum' => $aktualniTydenStr]))  . "'>" .
                TextManager::_('Aktualni_tyden') .
                '</a></span>';
        }

        echo "</div>\n";
    }


    public function load(Database $database)
    {
        if (isset($_GET['datum'])) {
            try {
                $this ->value = DateUtils::firstDayOfWeek(DateUtils::dateStringToTimestamp($_GET['datum']));
                $this ->storeValue($database);
            } catch (\Exception $e) {
                $this ->loadDefaultValue($database);
            }
        } else {
            $this ->loadDefaultValue($database);
        }
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case 'datumOd' :
                return $this ->value;

            case 'datumDo' :
                return DateUtils::incDay($this ->value, 6);
        }

        return parent::getValue($specification);
    }
}

