<?php
/**
 * stanislav.elbl, 16.11.2021 21:19
 */

namespace RonPortal\Controls;

use Ron\Controls\FormField;
use Ron\DB\DBRecordField;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

/**
 * Class RCOsobyFormField
 * Pole pro vyber osoby (jen RC, ne pracovni pomer)
 */
class RCOsobyFormField extends FormField
{
    /** @var string|PravaNaOsoby */
    public $prava;
    public $datumPlatnostiPP;
    public $vcetneExternich;

    public $filtr;

    /**
     * OsobaField constructor.
     * @param string|PravaNaOsoby $prava kod prava
     * @param int $datumPlatnostiPP datum ke kteremu se bere platnost prac. pomeru
     */
    function __construct($prava, $datumPlatnostiPP)
    {
        parent::__construct();

        $this->fieldType = 'osoba';
        $this->fieldName = 'osoba';
        $this->prava = $prava;
        $this->required = true;
        $this->fieldCssClass = 'form-control ronselect';
        $this->maxLength = 15;

        $this->datumPlatnostiPP = $datumPlatnostiPP;
        if (!$this->datumPlatnostiPP) {
            $this->datumPlatnostiPP = time();
        }

        $this ->vcetneExternich = false;
    }


    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);

        if (!isset($result['data-max-visible-rows'])) {
            $result['data-max-visible-rows'] = 200;
        }

        return $result;
    }


    /**
     * Nacteni konfigurace
     * @param string $key
     * @param string $value
     */
    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'prava' :
                $this ->prava = $value;
                break;

            case 'vcetneExternich' :
                $this ->vcetneExternich = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * @param \Ron\DB\Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            $prava = ($this ->prava instanceof PravaNaOsoby)
                ? $this ->prava
                : Prava::vratPrava($this ->prava);

            $combo = new ComboBoxOsoba($this->fieldName); // jmeno promenne
            $combo->filtr = $this->filtr;
            $combo->className = $this ->fieldCssClass . ' ' . MessageManager::getClass($this ->fieldName);
            $combo->htmlParams = $this ->getFieldAttributes();
            //$combo->htmlParams = array_merge($combo ->htmlParams, $this->otherHtmlParams ?: []);
            $combo->loadDataPP($database,
                $this->datumPlatnostiPP, $this->datumPlatnostiPP,
                $prava, ' ', $value, $this ->vcetneExternich); // $emptyRow = ' ', $selectedValue, $vcetneExternich
            $combo->render();
        } else {
            if ($value) {
                $pp = PracovniPomer::najdiPPOsoby($database, $value, false);
                if ($pp) {
                    echo $pp ->jmenoPrijmeni;
                }
            }
        }
    }


    public function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        if (!$this->sqlFieldName) {
            return NULL;
        }

        $pp = null;

        if (!$this ->readOnly || $insert) {
            $idPP = (isset($_POST[$this->fieldName]) && trim($_POST[$this->fieldName]) !== '')
                ? (int)$_POST[$this->fieldName]
                : null;

            if ($idPP) {
                $pp = PracovniPomer::nactiPP($database, $idPP);

                if ($this ->prava) {
                    $prava = ($this->prava instanceof PravaNaOsoby)
                        ? $this->prava
                        : Prava::vratPrava($this->prava);

                    if (!$prava->lzeZobrazitRC($pp->rc)) {
                        throw new NeniPravoException();
                    }
                }
            }
        }

        $field = new DBRecordField($this->sqlFieldName, '', $this->readOnly, $this->maxLength); // format = ''
        $field ->setValue($pp ? $pp ->rc : null);

        $field->setUpdateLog($this->updateLog);
        $field->setChangeable($this->changeable);

        return $field;
    }
}
