<?php
/**
 * stanislav.elbl, 28.10.2021 17:52
 */

namespace RonPortal\Controls;

use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBRecordField;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;
use RonPortal\User;

/**
 * Formularove pole pro vyber rozdeleni.
 * Class RozdeleniFormField
 * @package RonPortal\Controls
 */
class RozdeleniFormField extends FormField
{
    /** @var int */
    protected $kodTypuRozdeleni;

    /** @var string     Jmeno databazoveho pole, do ktereho se ma ulozit kod typu rozdeleni */
    public $typRozdeleniFieldName;

    /** @var PravaNaOsoby   Pokud jsou zadana, tak se pouziji pro filtrovani seznamu rozdeleni */
    public $prava;

    /** @var int            Urcuje, jestli se maji nacist pouze vlastni rozdeleni - viz RozdeleniUtils::sqlSeznamRozdeleniOsoby */
    public $pouzeVlastniRozdeleni = 0;
    /** @var PracovniPomer  Pracovni pomer, pro ktery se nactou vlastni rozdeleni - pokud neni zadan, pouzije se PP uzivatele */
    public $pracovniPomer;

    /** @var string         Omezuje vyber rozdeleni jen na zadane kody (oddelene strednikem) */
    private $vycetRozdeleni;


    /**
     * RozdeleniFormField constructor.
     * @param string $kodTypuRozdeleni
     * @param int $pouzeVlastni
     * @param PracovniPomer $pp
     */
    public function __construct($kodTypuRozdeleni, $pouzeVlastni = 0, $pp = null)
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);

        $this->kodTypuRozdeleni = $kodTypuRozdeleni;

        $this->pouzeVlastniRozdeleni = $pouzeVlastni;
        $this->pracovniPomer = $pp;

        $this ->fieldCssClass .= ($this ->fieldCssClass ? ' ' : '') . 'ronselect';

        $this ->selectSqlFilter = "cssClass IS NULL  OR  cssClass <> 'hidden'";
    }


    /**
     * Nastavi kod typu rozdeleni, ktere se ma zobrazit
     * @param int $kodTypuRozdeleni
     * @return RozdeleniFormField
     */
    public function setKodTypuRozdeleni($kodTypuRozdeleni)
    {
        $this->kodTypuRozdeleni = $kodTypuRozdeleni;
        return $this;
    }


    /**
     * Nastavi jmeno databazoveho pole, do ktereho se uklada kod typu rozdeleni
     * @param string $typRozdeleniFieldName
     * @return $this
     */
    public function setTypRozdeleniFieldName($typRozdeleniFieldName)
    {
        $this->typRozdeleniFieldName = $typRozdeleniFieldName;
        return $this;
    }


    /**
     * @param PravaNaOsoby $prava
     * @return $this
     */
    public function setPrava($prava)
    {
        $this->prava = $prava;
        return $this;
    }

    /**
     * @param int $pouzeVlastniRozdeleni
     * @return $this
     */
    public function setPouzeVlastniRozdeleni($pouzeVlastniRozdeleni)
    {
        $this->pouzeVlastniRozdeleni = $pouzeVlastniRozdeleni;
        return $this;
    }

    /**
     * @return string
     */
    public function getVycetRozdeleni()
    {
        return $this->vycetRozdeleni;
    }


    /**
     * @param string $vycetRozdeleni
     * @return $this
     */
    public function setVycetRozdeleni($vycetRozdeleni)
    {
        $this->vycetRozdeleni = $vycetRozdeleni;
        return $this;
    }



    /**
     * @param Database $database
     * @return $this
     */
    public function loadCaption($database)
    {
        $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, $this ->kodTypuRozdeleni);
        $this ->setCaption($typRozdeleni ->popis);

        return $this;
    }


    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'pouzeVlastniRozdeleni' :
                $this ->pouzeVlastniRozdeleni = (int)$value;
                break;

            case 'prava' :
                $this ->prava = (isset($value)  &&  $value != '')
                    ? Prava::vratPrava($value)
                    : null;
                break;

            case 'kodTypuRozdeleni' :
                $this ->kodTypuRozdeleni = (int)$value;
                break;

            case 'vycetRozdeleni' :
                $this ->vycetRozdeleni = $value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


    protected function renderComboBoxes($comboBoxes)
    {
        echo '<div class="row">';
            echo '<div class="col-12 col-sm-8">';
            $comboBoxes ->combo1 ->render();
            echo '</div>';

            echo '<div class="d-xs-none col-4">';
            $comboBoxes ->combo2 ->render();
            echo '</div>';
        echo '</div>';
    }



    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {
            $comboBoxes = new ComboBoxesRozdeleni($database, $this ->fieldName, $this ->fieldId);
            $comboBoxes ->prava = $this ->prava;
            $comboBoxes ->vycetRozdeleni = $this ->vycetRozdeleni;

            if ($this ->otherHtmlParams) {
                foreach ($this ->otherHtmlParams as $paramName => $paramValue) {
                    $comboBoxes ->combo1 ->htmlParams[$paramName] = $paramValue;
                    $comboBoxes ->combo2 ->htmlParams[$paramName] = $paramValue;
                }
                //var_dump($comboBoxes ->combo1 ->htmlParams);
            }

            $class = $this ->fieldCssClass . ' ' . MessageManager::getClass($this ->fieldName);
            $comboBoxes ->combo1 ->className = $class;
            $comboBoxes ->combo2 ->className = $class;

            $comboBoxes ->loadRozdeleni($this ->pouzeVlastniRozdeleni, $this->pracovniPomer ?: User::getRC(), // $this ->pouzeVlastniRozdeleni,  $PP,    // pouze vlastni - podle jakeho PP? Prvni uzivatel, nebo aktualni?
                $this ->kodTypuRozdeleni, $value,
                $this ->required);

            $this ->renderComboBoxes($comboBoxes);
        } else if ($value) {
            $rozdeleni =  new Rozdeleni();
            $rozdeleni->load($database, $this->kodTypuRozdeleni, $value);
            echo
                StringUtils::htmlTextConvert($rozdeleni->popis) .
                ($this ->selectShowCode ? ' (' . $rozdeleni ->kod . ')' : '');
        }
    }

    public function createDBRecordField($database, $tableName, $insert)
    {
        $field = parent::createDBRecordField($database, $tableName, $insert);

        if (!$this ->typRozdeleniFieldName) {
            return $field;
        }

        $fieldTyp = new DBRecordField($this->typRozdeleniFieldName, 'int', $this->readOnly);
        if (isset($field ->value)) {
            $fieldTyp ->value = $this ->kodTypuRozdeleni;
        } else {
            $fieldTyp ->value = null;
        }

        return [$field, $fieldTyp];
    }


}

