<?php

namespace RonPortal\Controls;

use Ron\Controls\Option;
use Ron\Controls\OptionLoader;
use Ron\Controls\Options;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\MVC\ViewObject;
use Ron\StringUtils;

/**
 * Zobrazi seznam rozdeleni jako "pills"
 */
class RozdeleniList implements ViewObject
{
    /**
     * @var Option[]
     */
    protected $options;

    public $cssClass = 'pills';

    /**
     * @param $options
     */
    public function __construct($options = null)
    {
        $this->options = $options;
    }


    /**
     * @param Database $database
     * @param SqlStatement|string $sql  Sql dotaz pro nacteni seznamu rozdeleni - musi obsahovat pole kod, popis, popis typu a musi byt serazen podle popisu typu
     * @param Options|null $params
     * @return RozdeleniList
     */
    public function loadOptions($database, $sql, $params = null)
    {
        $this->options = (new OptionLoader($database))
            ->setSql($sql, $params)
            ->setAttrIndexes(['typ' => 2])
            ->loadData();

        return $this;
    }


    public function render()
    {
        if (!$this ->options) {
            return;
        }

        $lastTyp = null;
        $first = true;

        foreach ($this ->options as $option) {
            if ($first  ||  $lastTyp !== $option ->params['typ']) {
                if (!$first) {
                    echo "</ul>\n";
                }

                $first = false;

                echo
                    ($option ->params['typ']    // nadpis s popisem typu rozdeleni
                        ? "<div>" . StringUtils::htmlTextConvert($option ->params['typ']) . "</div>"
                        : ''
                    ) .
                    "<ul class='{$this ->cssClass}'>";

                $lastTyp = $option ->params['typ'];
            }

            // key, caption
            echo "<li>" . StringUtils::htmlTextConvert($option ->caption) . "</li>";
        }

        if (!$first) {
            echo "</ul>\n";
        }
    }
}
