<?php

namespace RonPortal\Controls;

use Ron\Controls\Dialog;
use Ron\Controls\Option;
use Ron\Controls\OptionLoader;
use Ron\Controls\Options;
use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\Http\Csrf;
use Ron\MVC\Model;
use Ron\TextManager;
use RonPortal\TypRozdeleni;


/**
 * ViewObject pro vyrenderovani dialogu pro vyber rozdeleni (shift form)
 */
class RozdeleniShiftFormView extends Dialog
{
    /** @var TypRozdeleni */
    protected $typRozdeleni;

    /** @var string */
    protected $url;

    /** @var Option[] */
    protected $typyRozdeleni;


    /**
     * @param ?TypRozdeleni $typRozdeleni
     * @param string $url
     */
    public function __construct($typRozdeleni, $url, $csrfRequestId = "rozdeleniosoby")
    {
        parent::__construct();
        $this ->typRozdeleni = $typRozdeleni;
        $this ->url = $url;

        $this
            ->setId('rozdeleniosobydialog')
            ->setTitle($typRozdeleni->popis ?? TextManager::_('Rozdeleni'))
            ->setView('ronportal/controls/view_rozdeleni_osoby.php')
            ->setHtmlParam('data-url', $this ->url)
            ->setHtmlParam('data-csrf-key', Csrf::getCsrfKey($csrfRequestId))
            ->setFormTag(false);

        Model::getInstance() ->addScript('script/rozdeleniosoby.js', 'rozdeleniosoby');
    }


    /**
     * @param Database $database
     * @param string|int[]|null $kody
     * @return void
     */
    public function loadTypyRozdeleni(Database $database, $kody = null)
    {
        if (is_string($kody)) {
            $kody = explode(';', $kody);
        }

        $params = $database ->createParameters();
        $typyRozdeleni = (new OptionLoader($database))
            ->setSql('SELECT Kod, Popis FROM TypRozdeleni' .
                ($kody ? ' WHERE ' . SQLUtils::getINCondition($database, 'Kod', $kody, $params) : '') .
                ' ORDER BY Popis', $params)
            ->loadData();

        if (count($typyRozdeleni) === 1) {
            // jen jeden typ rozdeleni
            $this ->typRozdeleni = $typyRozdeleni[0];
        } else {
            $this ->typyRozdeleni = $typyRozdeleni;
        }
    }

    // <script type="text/javascript" src="script/rozdeleniosoby.js?ver=..."></script>
}
