<?php
/**
 * stanislav.elbl, 04.11.2015
 */

namespace RonPortal\Dochazka;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;


class Absence
{
    /** @var  $f_PP PracovniPomer */
    public $f_PP;

    /** @var int  Pouzije se pri editaci, protoze je to soucast prim. klice */
    public $datumOdPuvodni;
    public $poradi; // soucast primarniho klice

    public $datumOd;
    public $delka;
    public $casOd;

    public $datumDo;
    public $casDo;

    /** @var string     Kod operace */
    public $operaceDochazky;

    /** @var int    0=bez rozhodnuti, 1=schvaleno, 2=zamitnuto, 3=zadost o zruseni, 4=zruseno, 5=zruseni zamitnuto */
    public $schvaleni;
    /** @var int    Priznak jestli se ma automaticky schvalovat - nacteny z OperaceDochazky */
    public $absence_AutoSchvalovat;
    /** @var int    Priznak jestli lze zrusit i kdyz je schvalena - nacteny z OperaceDochazky */
    public $absence_LzeZrusit;
    public $poznamka;
    public $schvalil;
    /** @var int */
    public $datumSchvaleni;

    /** @var int */
    public $doporuceni;
    /** @var int */
    public $datumDoporuceni;
    /** @var string */
    public $doporucil;

    /** @var string     Log schvalovani */
    public $historieSchvaleni;
    /** @var string     Nastavi se pri nacteni zaznamu z databaze podle OperaceDochazky */
    public $popisAbsence;

    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;
    /** @var string     Rozdeleni.Popis */
    public $popisRozdeleni;
    /** @var string     TypRozdeleni.Popis */
    public $popisTypuRozdeleni;

    /** @var string */
    public $uzivatelska1;
    /** @var string */
    public $uzivatelska2;
    /** @var string */
    public $uzivatelska3;
    /** @var string */
    public $uzivatelska4;
    /** @var string */
    public $uzivatelska5;


    /**
     * @param Database $database
     * @param array $row
     * @throws Exception
     */
    private function loadRow($database, $row)
    {
        $this ->f_PP = new PracovniPomer();
        if (!$this ->f_PP ->nactiPPPodleId($database, $row[0])) {
            throw new Exception('Chyba! Nebyl nalezen pracovni pomer k absenci!');
        }

        $this ->poradi = (int)$row[18]; // soucast primarniho klice

        $this ->datumOd = $database ->stringToTimestamp($row[4]);
        $this ->delka = (int)$row[5];
        $this ->casOd = $database ->stringToSecondsOfDay($row[6]); // stringToTimestamp($row[6]);

        $this ->datumDo = $database ->stringToTimestamp($row[7]);
        $this ->casDo = $database ->stringToSecondsOfDay($row[8]); // stringToTimestamp($row[8]);

        $this ->operaceDochazky = $row[9];

        $this ->schvaleni = (int)$row[10];
        $this ->schvalil = $row[11];
        $this ->datumSchvaleni = $database ->stringToTimestamp($row[12]);

        $this ->doporuceni = (int)$row[14];
        $this ->doporucil = $row[15];
        $this ->datumDoporuceni = $database ->stringToTimestamp($row[16]);

        $this ->poznamka = $row[17];

        $this ->historieSchvaleni = $row[13];
        $this ->popisAbsence = $row[19];
        $this ->absence_AutoSchvalovat = $row[20];
        $this ->absence_LzeZrusit = $row[21];

        $this ->kodTypuRozdeleni = isset($row[25]) ? (int)$row[25] : null;
        $this ->rozdeleni = $row[26];
        $this ->popisRozdeleni = $row[27];
        $this ->popisTypuRozdeleni = $row[28];

        $this ->uzivatelska1 = $row[29];
        $this ->uzivatelska2 = $row[30];
        $this ->uzivatelska3 = $row[31];
        $this ->uzivatelska4 = $row[32];
        $this ->uzivatelska5 = $row[33];
    }

    /**
     * Nacte zadanou absenci
     * @param Database $database
     * @param int $idPP
     * @param int $datumOd
     * @param int $poradi
     * @return Absence
     * @throws DBException
     * @throws Exception
     */
    public static function nactiAbsenci($database, $idPP, $datumOd, $poradi)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, Absence.RC, Absence.OsobniCislo, Absence.DatumNastupu, ' . // 0 - 3
            ' Absence.DatumOd, Absence.Delka, Absence.CasOd, Absence.DatumDo, Absence.CasDo, Absence.OperaceDochazky, ' . // 4 - 9
            ' Absence.Schvaleni, Absence.Schvalil, Absence.DatumSchvaleni, ' . $database ->castWideMemo('Absence.HistorieSchvalovani') . ', ' . // 10 - 13
            ' Absence.Doporuceni, Absence.Doporucil, Absence.DatumDoporuceni, ' . $database ->castWideMemo('Absence.Poznamka') . ', ' . // 14 - 17
            ' Absence.Poradi, OperaceDochazky.Popis, OperaceDochazky.Absence_AutoSchvalovat, Absence_LzeZrusit, ' . // 18 - 21
            ' Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . // 22 - 24
            ' Absence.KodTypuRozdeleni, Absence.Rozdeleni, Rozdeleni.Popis, TypRozdeleni.Popis,' . // 25..28
            ' Absence.Uzivatelska1, Absence.Uzivatelska2, Absence.Uzivatelska3, Absence.Uzivatelska4, Absence.Uzivatelska5' . // 29..33

            ' FROM Absence INNER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = Absence.RC ' .
            '  AND PracovniPomer.OsobniCislo = Absence.OsobniCislo ' .
            '  AND PracovniPomer.DatumNastupu = Absence.DatumNastupu ' .
            ' INNER JOIN Osoba ' .
            '  ON Osoba.RC = Absence.RC ' .
            ' LEFT OUTER JOIN OperaceDochazky ' .
            '  ON OperaceDochazky.Kod = Absence.OperaceDochazky ' .
            ' LEFT OUTER JOIN Rozdeleni ' .
            '  ON Rozdeleni.Kod = Absence.Rozdeleni AND Rozdeleni.KodTypuRozdeleni = Absence.KodTypuRozdeleni' .
            ' LEFT OUTER JOIN TypRozdeleni ' .
            ' ON TypRozdeleni.Kod = Absence.KodTypuRozdeleni ' .

            ' WHERE PracovniPomer.Id = ' . $params ->addParameter((int)$idPP) .
            '   AND Absence.DatumOd = ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND Absence.Poradi = ' . $params ->addParameter((int)$poradi);

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception('Chyba - zadana absence nebyla nalezena!');
        }

        $absence = new self();
        $absence ->loadRow($database, $row);

        return $absence;
    }


//    /**
//     * @param Database $database
//     * @param int $hodnota
//     * TODO: prehodit do AbsenceDBRecord
//     */
//    function nastavDoporuceni($database, $hodnota)
//    {
//        //$rec = new AbsenceDBRecord($database, $this ->f_PP)
//
//        // nastaveni popisu pro zapis do logu a do HistorieSchvalovani
//        $desc = TextManager::_('Log_nastaveni_doporuceni'). ' ' . $hodnota;
//
//        $historie = $this ->historieSchvaleni;
//
//        $historie .=
//            date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' . $desc . "\n";
//
//        $cas = time();
//
//        $rec = new RonPortalDBRecord($database, 'Absence');
//        $rec ->addField('RC', '', true);
//        $rec ->addField('OsobniCislo', '', true);
//        $rec ->addField('DatumNastupu', 'timestamp', true);
//        $rec ->addField('', '', true, 0, '   ' .  $this ->f_PP ->jmenoPrijmeni);
//        $rec ->addField('Poradi', 'int', true);
//        $rec ->addField('OperaceDochazky', '', true);
//        $rec ->addField('DatumOd', 'timestamp', true);
//        $rec ->addField('Datumdo', 'timestamp', true);
//        $rec ->addField('Delka', 'int', true);
//        $rec ->addField('Doporuceni', 'int', false, 0, $hodnota);
//        $rec ->addField('Doporucil', '', false, 25, RonPortalUtils::getLogUserName());
//        $rec ->addField('DatumDoporuceni', 'timestamp', false, 0, $cas);
//        $rec ->addField('HistorieSchvalovani', 'memo', false, 0, $historie);
//
//        $rec ->logDescription = $desc;
//
//        $params = $database ->createParameters();
//        $filtr =
//            'RC = ' . $params ->addParameter($this ->f_PP ->rc) .
//            ' AND OsobniCislo = ' . $params ->addParameter($this ->f_PP ->osobniCislo) .
//            ' AND Datumnastupu = ' . $params ->addParameter($database ->dateToParam($this ->f_PP ->datumNastupu)) .
//            ' AND DatumOd = ' . $params ->addParameter($database ->timestampToParam($this ->datumOd)) .
//            ' AND Poradi = ' . $params ->addParameter((int)$this ->poradi);
//
//        if ($rec ->loadRecord($filtr, $params)) {
//            $rec ->updateRecord($filtr, $params);
//        }
//    }


    public function getPopisObdobiAbsence()
    {
        $result = TextManager::_('Day_' . date('w', $this->datumOd)) . ' ' . date('j.n.Y', $this->datumOd);

        if ($this->delka == 4) {
            // kontinualni - prida cas primo k datumu
            $result .= ' ' . DateUtils::minutesToString($this->casOd / 60);
        }

        if ($this->datumDo != $this->datumOd) {
            // vicedenni - prida datum do
            $result .= ' - ' . TextManager::_('Day_' . date('w', $this->datumDo)) . ' ' . date('j.n.Y', $this->datumDo);
        }

        if ($this->delka == 4) {
            // kontinualni - prida cas k datumu
            $result .= ($this->datumDo == $this->datumOd)
                ? ' - '
                : ' ';

            $result .= DateUtils::minutesToString($this->casDo / 60);
        }

        if ($this->delka == 3) {
            // cas od-do
            //$result .= ' (' . date('G:i', $this->casOd) . ' - ' . date('G:i', $this->casDo) . ')';
            $result .= ' (' . DateUtils::minutesToString($this->casOd / 60) . ' - ' . DateUtils::minutesToString($this->casDo / 60) . ')';
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param Absence|PracovniPomer $pp_absence
     * @param string $kodOperace
     * @param int $datumOd
     * @param int $datumDo
     * @param int $delka
     * @param int $casOd
     * @param int $casDo
     * @param int $datumOdPuvodni
     * @param int $poradi
     * @return Absence[]
     * @throws Exception
     */
    public static function nactiPrekryvaneAbsence($database, $pp_absence, $kodOperace = null, $datumOd = null, $datumDo = null, $delka = null, $casOd = null, $casDo = null, $datumOdPuvodni = null, $poradi = null)
    {
        if ($pp_absence instanceof Absence) {
            $pp = $pp_absence ->f_PP;

            $kodOperace = $pp_absence ->operaceDochazky;
            $datumOd = $pp_absence ->datumOd;
            $datumDo = $pp_absence ->datumDo;
            $delka = $pp_absence ->delka;
            $casOd = $pp_absence ->casOd;
            $casDo = $pp_absence ->casDo;
            $datumOdPuvodni = $pp_absence ->datumOd;
            $poradi = $pp_absence ->poradi;
        } else {
            $pp = $pp_absence;
        }

        $operace = isset($kodOperace)
            ? OperaceDochazky::nactiOperaci($database, $kodOperace)
            : null;

        $params = $database->createParameters();
        $sql =
            'SELECT Absence.Delka, Absence.CasOd, Absence.CasDo, Absence.DatumOd, Absence.DatumDo, Absence.Poradi, Absence.Schvaleni,' .
            '  Absence.OperaceDochazky, OperaceDochazky.Popis' .
            ' FROM Absence, OperaceDochazky ' .
            ' WHERE OperaceDochazky.Kod = Absence.OperaceDochazky ' .
            ' AND Absence.DatumOd <= ' . $params->addParameter($database->dateToParam($datumDo)) .
            ' AND Absence.DatumDo >= ' . $params->addParameter($database->dateToParam($datumOd)) .
            ' AND Absence.RC = ' . $params->addParameter($pp->rc) .
            ' AND Absence.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND Absence.DatumNastupu = ' . $params->addParameter($database->dateToParam($pp->datumNastupu)) .
            ' AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4'; // zamitnute a zrusene ignoruje

//        $sql .= RonPortalSettings::getIntValue('prekryvaniabsenci')
//            ? ' AND Absence.Schvaleni = 1'  // kontroluje jen schvalene
//            : ' AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4'; // zamitnute a zrusene ignoruje

        if (!$operace  ||  $operace->pocatekIntervalu) {
            // operace generuje interval - nesmi se prekryvat s jinou absenci, ktera taky generuje interval
            $sql .= ' AND OperaceDochazky.PocatekIntervalu = 1';
        } else {
            // operace negeneruje interval - kontroluje se jen prekryvani se stejnou operaci
            $sql .= ' AND Absence.OperaceDochazky = ' . $params->addParameter($kodOperace);
        }

        if ($datumOdPuvodni) {
            // pri editaci vynecha z kontroly puvodni zaznam
            $sql .=
                ' AND ( Absence.DatumOd <> ' . $params->addParameter($database->dateToParam($datumOdPuvodni)) .
                ' OR Absence.Poradi <> ' . $params->addParameter((int)$poradi) .
                ')';
        }


        switch ($delka) {
            case 1 : // prvni pulka
            case 2 : // druha pulka
                // nesmi se prekryvat s celodenni a se stejnou pulkou smeny
                // kontrola s absencemi cas od-do se neprovadi
                $sql .=
                    ' AND (Absence.Delka = 0  OR  Absence.Delka =' . $params->addParameter((int)$delka) . ')';
                break;


            case 3 : // cas od-do
            case 4 : // kontinualni
                // nesmi se prekryvat s celodenni nebo s jinou "cas od-do" nebo s kontinualni
                // kontrola pulek smen je problematicka, proto se vubec neprovadi
                // dale jeste probiha kontrola jednotlivych zaznamu, aby se navzajem porovnaly i kontinualni absence. Navic na sqlsrv ten dotaz nefunguje spravne
                $sql .=
                    ' AND (Absence.Delka = 0  OR  Absence.Delka = 4  OR ' .
                    '  (Absence.Delka = 3   AND  Absence.CasOd < ' . $params->addParameter($database->timeToParam($casDo)) .
                    '   AND  Absence.CasDo > ' . $params->addParameter($database->timeToParam($casOd)) . ')' .
//                    '  (Absence.Delka = 3   AND  Absence.CasOd < ' . $params->addParameter($database->timeToParam(DateUtils::timeOf($casDo))) .
//                    '   AND  Absence.CasDo > ' . $params->addParameter($database->timeToParam(DateUtils::timeOf($casOd))) . ')' .
                    ')';
                break;

            default : // celodenni - nesmi se prekryvat s zadnou jinou
                break;
        }

        $result = [];

        $cursor = $database->getSqlResult($sql, $params);

        // Na sqlsrv ten sql dotaz blbne - najde zaznam i kdyz se neprekryvaji az o hodinu.
        // Je to asi neco s posunem casu, protoze kdyz je rozdil mezi zaznamy nejmene hodina, tak je to OK
        while ($row = $cursor ->fetch()) {
            $prekryvani = false;

            $casPrekryvaneOd = $database ->stringToSecondsOfDay($row[1]);
            $casPrekryvaneDo = $database ->stringToSecondsOfDay($row[2]);

            if ($delka == 3) {
                if ($row[0] == 3) {
                    // jina absence se zadanym casem od do - zkontroluje prekryvani casu
                    //if (DateUtils::timeOf($casOd) < $database->stringToSecondsOfDay($row[2]) && DateUtils::timeOf($casDo) > $database->stringToSecondsOfDay($row[1])) {
                    if (DateUtils::timeOf($casOd) < DateUtils::timeOf($casPrekryvaneDo)
                        && DateUtils::timeOf($casDo) > DateUtils::timeOf($casPrekryvaneOd))
                    {
                        $prekryvani = true;
                    }
                } else if ($row[0] == 4) {
                    // nasel kontinualni absenci
                    $kontinualniDatumOd = $database->stringToTimestamp($row[3]);
                    $kontinualniDatumDo = $database->stringToTimestamp($row[4]);
                    if ($kontinualniDatumDo > $datumOd && $kontinualniDatumOd < $datumDo) {
                        $prekryvani = true;
                    } else if ($kontinualniDatumDo == $datumOd) {
                        if (DateUtils::timeOf($casOd) < DateUtils::timeOf($casPrekryvaneDo)) {   // $database->stringToSecondsOfDay($row[2])
                            $prekryvani = true;
                        }
                    } else if ($kontinualniDatumOd == $datumDo) {
                        if (DateUtils::timeOf($casDo) > DateUtils::timeOf($casPrekryvaneOd)) {   // $database->stringToSecondsOfDay($row[1])
                            $prekryvani = true;
                        }
                    }
                } else {
                    $prekryvani = true; // nasel celodenni absenci
                }
            } else if ($delka == 4) {
                // kontinualni
                $prekryvani = false;
                while (!$prekryvani && $row = $cursor->fetch()) {
                    if ($row[0] == 4 || $row[0] == 3) {
                        // nasel absenci s casem od-do nebo kontinualni
                        $existujiciDatumOd = $database->stringToTimestamp($row[3]);
                        $existujiciDatumDo = $database->stringToTimestamp($row[4]);
                        if ($datumDo > $existujiciDatumOd && $datumOd < $existujiciDatumDo) {
                            $prekryvani = true;
                        } else if ($datumOd == $existujiciDatumDo) {
                            if (DateUtils::timeOf($casOd) < DateUtils::timeOf($casPrekryvaneDo)) {  // $database->stringToSecondsOfDay($row[2])
                                $prekryvani = true;
                            }
                        } else if ($datumDo == $existujiciDatumOd) {
                            if (DateUtils::timeOf($casDo) > DateUtils::timeOf($casPrekryvaneOd)) {  // $database->stringToSecondsOfDay($row[1])
                                $prekryvani = true;
                            }
                        }
                    } else {
                        $prekryvani = true; // nasel celodenni absenci
                    }
                }
            } else {
                $prekryvani = true;
            }

            if ($prekryvani) {
                // absence se prekryva - prida ji do seznamu
                $absence = new Absence();
                //'SELECT Delka, CasOd, CasDo, DatumOd, DatumDo, Poradi, Schvaleni' .
                $absence ->delka = (int)$row[0];

                $absence ->casOd = $casPrekryvaneOd;
                $absence ->casDo = $casPrekryvaneDo;
                $absence ->datumOd = $database->stringToTimestamp($row[3]);
                $absence ->datumDo = $database->stringToTimestamp($row[4]);

                $absence ->poradi = $row[5];
                $absence ->schvaleni = $row[6];

                $absence ->operaceDochazky = $row[7];
                $absence ->popisAbsence = $row[8];

                $result[] = $absence;
            }
        }

        $cursor->close();

        return $result;
    }
}
