<?php
/**
 * seznam planovanych absenci
 */

namespace RonPortal\Dochazka;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\ProgressDialog;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Obdobi;
use RonPortal\PoznamkaKeDni;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class AbsenceFilterForm extends RonPortalFilterForm
{
    function __construct($database, $action)
    {
        parent::__construct($database, $action, true);
    }


    protected function afterLoadFilter($filter)
    {
        if ($filter ->name === 'planovani_filtr') {
            $this ->getFilter('obdobi') ->setRequired(
                $this ->getFilterValue('planovani_filtr') == 1  ||  $this ->getFilterValue('planovani_filtr') == 2
            );
        }
        parent::afterLoadFilter($filter);
    }
}


/**
 * Zobrazeni delky absence - bud popis delky nebo cas od - do
 * Class DelkaAbsenceField
 */
class DelkaAbsenceField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData['col-delka']) {
            case 0 :
                return TextManager::_('Absence_cela_smena');

            case 1 :
                return TextManager::_('Absence_pulka_smeny1');

            case 2 :
                return TextManager::_('Absence_pulka_smeny2');

            case 3 : // cas od-do
                return
                    $rowData['@col-casod'] .
                    ' - ' .
                    $rowData['@col-casdo'];

            case 4 : // kontinualne
                return
                    date('j.n.', $this ->database ->stringToTimestamp($rowData['col-datumod']) ) .
                    ' ' . $rowData['@col-casod'] . // date('G:i', $this ->database ->stringToTimestamp($rowData[14])) .
                    ' - ' .
                    date('j.n.', $this ->database ->stringToTimestamp($rowData['col-datumdo']) ) .
                    ' ' . $rowData['@col-casdo']; // date('G:i', $this ->database ->stringToTimestamp($rowData[15]));
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData); // popis delky absence primo z databaze
    }
}



/**
 * Odkaz na zruseni absence
 */
class ZruseniAbsenceField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datumOd = $this ->database ->stringToTimestamp($rowData['col-datumod']);
        $datumDo = $this ->database ->stringToTimestamp($rowData['col-datumdo']);

        if (!Prava::vratPrava('absence_delete', 'absence_delete_' . $rowData['col-operacedochazky'])
            ->lzeZobrazitPP($rowData['col-idpp'], $datumOd, $datumDo))
        {
            // nemuze smazat schvalene absence (pokud muze, tak neni potreba kontrolovat dalsi prava)
            $prava = Prava::vratPrava('absence', 'absence_' . $rowData['col-operacedochazky']); // prava podle Absence.OperaceDochazky

            if (!$prava->lzeZobrazitPP($rowData['col-idpp'], $datumOd, $datumDo)) {
                // nemuze editovat => nemuze zrusit
                return '';
            }

            if ($rowData['col-kodschvaleni'] == 4) { // jiz je zruseno
                return '';
            }

            if ($rowData['col-kodschvaleni'] == 5) { // // zruseni zamitnuto -> zrusit jde jedine ve formulari
                return '';
            }

            // Pokud nemuze uzivatel schvalovat, tak muze zrusit jedine absenci, ktera jeste nebyla schvalena
            if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI))
                if ($rowData['col-kodschvaleni']) {
                    $pravaSchvalovani = Prava::vratPrava('absence_schval', 'absence_schval_' . $rowData['col-operacedochazky']);

                    if (!$pravaSchvalovani->lzeZobrazitPP($rowData['col-idpp'], $datumOd, $datumDo)) {
                        return '';
                    }
                }
        }

        $text = $this ->text ?? ($rowData['col-kodschvaleni'] == 3 ? 'Potvrdit_zadost_o_zruseni' : null);

        return
            AbsenceController::getPostLinkZrusit($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], true, true, $text);
    }
}


/**
 * Odkaz na zobrazeni absence: menuid=absence&action=zobrazabsenci&idpp=...&datum=...&poradi=...
 * Class EditaceAbsenceField
 */
class EditaceAbsenceField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this ->setType(BrowserField::FIELD_TYPE_TEXT);
    }


    function getLinkUrl($recordId, $rowData)
    {
        return
            '?menuid=absence&action=zobrazabsenci&idpp=' . $rowData['col-idpp'] .
            '&datum=' . DateUtils::dateToUrlString($this ->database ->stringToTimestamp($rowData['col-datumod'])) .
            '&poradi=' . $rowData['col-poradi'];
    }
}


/**
 * Odkaz na schvaleni absence
 * Class SchvaleniAbsenceField
 */
class SchvaleniAbsenceField extends BrowserField
{
    private $pravaDoporuceni;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this ->pravaDoporuceni = Prava::vratPrava('absence_doporuc');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch($rowData['col-kodschvaleni']) {
            case 1 :
                return TextManager::_('Str_Yes') . ($rowData['col-schvalil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-schvalil']) . ')' : ''); // uz schvaleno

            case 2 :
                return TextManager::_('Str_No') . ($rowData['col-schvalil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-schvalil']) . ')' : ''); // zamitnuto

            case 3 :
                return TextManager::_('Zadost_o_zruseni');

            case 4 :
                return TextManager::_('Zruseno') . ($rowData['col-schvalil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-schvalil']) . ')' : ''); // zruseno

            case 5 :
                return TextManager::_('Zruseni_zamitnuto') . ($rowData['col-schvalil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-schvalil']) . ')' : '');
        }

        $datumOd = $this ->database ->stringToTimestamp($rowData['col-datumod']);
        $datumDo = $this ->database ->stringToTimestamp($rowData['col-datumdo']);

        $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $rowData['col-operacedochazky']);
        $lzeSchvalit = $prava ->lzeZobrazitPP($rowData['col-idpp'], $datumOd, $datumDo);

        $result = '';

        // schvaleni jeste neni urceno

        if (RonPortalSettings::getIntValue(AbsenceController::SETTINGS_DOPORUCOVANI)) {
            if ($rowData['col-doporuceni'] == 2) {
                // nedoporuceno - zamitnout
                $result = TextManager::_('Nedoporuceno') . ($rowData['col-doporucil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-doporucil']) . ')' : '') . '&nbsp;';

                if ($lzeSchvalit) {
                    $result .= AbsenceController::getPostLinkZamitnout($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], true);
                }

                return $result;

            } else if ($rowData['col-doporuceni'] == 1) {
                // doporuceno
                $result = TextManager::_('Doporuceno') . ($rowData['col-doporucil'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-doporucil']) . ')' : '') . '&nbsp;';

                if ($lzeSchvalit) {
                    // muze schvalovat - prida odkaz na schvaleni
                    $result .= AbsenceController::getPostLinkSchvalit($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], true);
                }

                return $result;

            } else if (!$rowData['col-doporuceni']) {
                // doporuceni neni jeste nastaveno - pokud nemuze schvalovat, tak zobrazi "doporucit
                if (!$lzeSchvalit && $this ->pravaDoporuceni ->lzeZobrazitPP($rowData['col-idpp'], $datumOd, $datumDo)) {
                    if ($this ->pravaDoporuceni ->priznak3ObsahujeKod($rowData['col-operacedochazky'])) {
                        return AbsenceController::getPostLinkDoporucit($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], 1);
                    }
                }
            }
        }

        if ($lzeSchvalit) {
            // muze schvalovat
            $result .=
                AbsenceController::getPostLinkSchvalit($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], true) .
                ' / ' .
                AbsenceController::getPostLinkZamitnout($rowData['col-idpp'], $datumOd, $rowData['col-poradi'], true);
        }

        return $result;
    }
}


/**
 * Poznamky ke dni v rozsahu zadane absence
 * Class Absence_PoznamkyKeDniField
 */
class Absence_PoznamkyKeDniField extends BrowserField
{
    private $typyPoznamek;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $typPoznamky = RonPortalSettings::getValue('absence_typpoznamky');
        $this ->typyPoznamek = ($typPoznamky == '*') ? null : explode(';', $typPoznamky);

        $this ->code = 'col-poznamkykedni';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $poznamky = PoznamkaKeDni::nactiPoznamkyProOsobu($this ->database, (int)$rowData['col-idpp'],
            $this ->database ->stringToTimestamp($rowData['col-datumod']), $this ->database ->stringToTimestamp($rowData['col-datumdo']),
            $this ->typyPoznamek);

        $result = '';
        if ($poznamky) {
            $title = '';
            foreach ($poznamky as $poznamka) {
                $title .= StringUtils::htmlTextConvert($poznamka ->text) . "\n";
            }

            $result = '<i class="inline-glyph fa glyph-info" title="' . $title . '"></i>';
        }

        return $result;
    }
}

class AbsenceBrowser extends RonPortalBrowser
{
    const FILTR_AKTUALNI = 0;
    const FILTR_VSE = 1;
    const FILTR_ZRUSENE = 2;
    const FILTR_BEZ_SCHVALENI = 3;
    const FILTR_KE_SCHVALENI = 4;

    private $pravaProhlizeni;
    private $pravaSchvalovani;
    private $pravaPlanovani;

    /** @var string     Kod prava, ktere se pouzije pro filtrovani seznamu.  */
    private $pravoProFiltrovani = null;

    private $RC; // pokud je zadano, zobrazi se data jen pro jednu osobu
    public $menuId; // cilova stranka pro odkazy

    private $radekLzeSchvalit = false;    
    //private $jsonRowIndex;

    function __construct($database, $action) // $RC = null
    {
        parent::__construct($database, $action);

        $this ->title = TextManager::_('Prehled_planovanych_absenci');

        $this ->code = 'browserplanovaneabsence';
        $this ->tableId = 'schvalovaniAbsenciTable';

//        $this ->RC = $RC;
//        $this ->menuId = $RC ? 'planovani' : 'schvalovaniabsence';
//        $this ->browserUrl = '?menuid=' . $this ->menuId . '&';

        $this ->sqlFromPart =
            '( ( ( ( Absence INNER JOIN OperaceDochazky ' .
            '        ON Absence.OperaceDochazky = OperaceDochazky.Kod ' .
            '      ) ' .
            '      INNER JOIN PracovniPomer ' .
            '       ON PracovniPomer.RC = Absence.RC' .
            '      AND PracovniPomer.OsobniCislo = Absence.OsobniCislo ' .
            '      AND PracovniPomer.DatumNastupu = Absence.DatumNastupu ' .
            '    ) ' .
            '    INNER JOIN Osoba ' .
            '     ON Osoba.RC = Absence.RC ' .
            '  ) ' .
            '  INNER JOIN S_DelkaAbsence ' .
            '   ON S_DelkaAbsence.Kod = Absence.Delka' .
            ') ' .
            'LEFT OUTER JOIN Rozdeleni ' .
            '  ON Rozdeleni.KodTypuRozdeleni = Absence.KodTypuRozdeleni ' .
            ' AND Rozdeleni.Kod = Absence.Rozdeleni ';


//        if ($this ->RC) {
//            // jen data jedne osoby (pri planovani)
//            $params = $database ->createParameters();
//            $this ->sqlWherePart = new SqlStatement('Osoba.RC = ' . $params ->addParameter($this ->RC), $params);
//        }

        if (Prava::jsouNejakePrava('absence_prehled')) {
            $this ->pravoProFiltrovani = 'absence_prehled';
        }

        $this ->pravaProhlizeni = Prava::vratPrava('prohlizeni');
        $this ->pravaSchvalovani = Prava::vratPrava('absence_schval');
        $this ->pravaPlanovani = Prava::vratPrava('absence');

        $this ->primaryField = 0;

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Poradi')
            ->setCode('col-poradi');

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno') ->setHeaderText('Jmeno') ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo')
            ->setCode('col-osobnicislo') ->setHeaderText('Osobni_cislo') ->setOrdering(true);

        // datum
        $this ->addField()
            ->setSqlFieldName('Absence.DatumOd') ->setFormat('date')
            ->setCode('col-datumod') ->setHeaderText('Datum_od') ->setOrdering(true);
        $this ->defaultSort = 'col-datumod';

        $this ->addField()
            ->setSqlFieldName('Absence.DatumDo') ->setFormat('date')
            ->setCode('col-datumdo') ->setHeaderText('Datum_do') ->setOrdering(true);

        // operace
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.OperaceDochazky')
            ->setCode('col-operacedochazky');

        $this ->addField()
            ->setSqlFieldName('OperaceDochazky.Popis')
            ->setCode('col-operace') ->setHeaderText('Popis_absence') ->setOrdering(true)
            ->setTranslationPrefix('db_operacedochazky_popis_', 'col-operacedochazky');

        // delka
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Delka')
            ->setCode('col-delka');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.CasOd') ->setFormat('timeofday')
            ->setCode('col-casod');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.CasDo') ->setFormat('timeofday')
            ->setCode('col-casdo');

        $this ->addField(new DelkaAbsenceField($database))
            ->setSqlFieldName('S_DelkaAbsence.Popis') ->setHeaderText('Delka_absence')
            ->setCode('col-delka-popis');




        $this ->addField(new EditaceAbsenceField($database))
            ->setCode('col-zobrazit') ->setText(TextManager::_('Zobrazit'));

        $this ->addField(new ZruseniAbsenceField($database));

        // pole pro schvaleni absence vcetne odkazu a osoby, ktera to schvalila
        $this ->addField(new SchvaleniAbsenceField($database))
            ->setCode('col-schvaleni') ->setHeaderText('Schvaleno')
            ->setVisible(RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI))
            ->setAlign('center');

        // zjednodusene zobrazeni schvaleni - pouze textovy stav - implicitne se nezobrazuje
        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Absence.Schvaleni', 'col-kodschvaleni') ->setHeaderText('Stav_absence') ->setOrdering(true)
            ->setCode('col-stav') ->setTitleFieldIndex('col-schvalil') ->setVisible(false)
            ->setOptions([
                0 => TextManager::_('Stav_Absence_0'),
                1 => TextManager::_('Stav_Absence_1'),
                2 => TextManager::_('Stav_Absence_2'),
                3 => TextManager::_('Stav_Absence_3'),
                4 => TextManager::_('Stav_Absence_4'),
                5 => TextManager::_('Stav_Absence_5')
            ]);

        if (RonPortalSettings::getValue('absence_typpoznamky')) {
            $this ->addField(new Absence_PoznamkyKeDniField($database));
        }

        // poznamka
//        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
//            ->setSqlFieldName('{fn_castwidememo(Absence.Poznamka)}')
//            ->setCode('col-poznamka');

        // doporuceni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Doporuceni')
            ->setCode('col-doporuceni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Doporucil')
            ->setCode('col-doporucil');

        // schvaleni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Schvaleni')
            ->setCode('col-kodschvaleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Absence.Schvalil')
            ->setCode('col-schvalil');


        // rozdeleni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Rozdeleni.Popis')
            ->setCode('col-rozdeleni-popis');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('Absence.Rozdeleni') ->setClassName('poznamka browserpoznamka')
            ->setCode('col-rozdeleni')
            ->setHeaderColSpan(1)
            ->setSqlFieldIndex2('col-rozdeleni-popis') ->setSqlFieldIndex2Separator(' - ');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(Absence.Poznamka)}') ->setClassName('poznamka browserpoznamka')
            ->setCode('col-poznamka')
            ->setHeaderColSpan(1);
    }


    function setFilterForm()
    {
        $this ->filterForm = new AbsenceFilterForm($this ->database, $this ->action);

        $stavFilter = new FilterFormFilter('planovani_filtr');
        $stavFilter
            ->setSessionName('planovani_filtr')
            ->setUserSettingsName('browser_absence_filtr')
            ->setRowId('filtr_urovenschvaleni')
            // doplni jednotlive volby
            ->addItem('0', TextManager::_('Absence_aktualni'))
            ->addItem('1', TextManager::_('Absence_vse'))
            ->addItem('2', TextManager::_('Absence_zrusene'));

        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            $stavFilter ->addItem('3', TextManager::_('Absence_bez_schvaleni'));

            if (Prava::vratHodnotuPrava('prohlizeni') != Prava::vratHodnotuPrava('absence_schval')) {
                // POZOR! Neresi situaci, kdy je uzivatelsky vytvorene pravo pro konkretni absenci
                $stavFilter ->addItem('4', TextManager::_('Absence_ke_schvaleni'));
            }
        }
        $this ->filterForm ->addFilter($stavFilter, true);

        // standardni filtry (obdobi, rozdeleni, osoby)
        //$this ->filterForm ->nastavObdobi(true, 'ADS', false, true); // jenotevrene = false, povinne - nastavi se v load
        $this ->filterForm ->addFilterObdobi();


//        if (!$this ->RC) {
            $prava = null;

            $pravaPrehled = Prava::vratPrava('absence_prehled');

            //Nastavi pravo na rozdeleni, kde muze nekoho videt. POZOR: nejde nastavit soucet prav, pokud tedy u schvalovani ma pouze jednu osobu, uvidi pouze jeji rozdeleni.
            if ($pravaPrehled->jsouNejakePrava()) {
                $prava = $pravaPrehled;
            }
            else if ($this->pravaSchvalovani->jsouNejakePrava()) {
                $prava = $this->pravaSchvalovani;
            }
            else if ($this->pravaProhlizeni->jsouNejakePrava()) {
                $prava = $this->pravaProhlizeni;
            }
            else if ($this->pravaPlanovani->jsouNejakePrava()) {
                $prava = $this->pravaPlanovani;
            }

            // filtr rozdeleni - v planovani se nezobrazuje
            $this ->filterForm ->addFilterRozdeleni1(null, $prava);
            $this ->filterForm ->addFilterRozdeleni2(null, $prava);

            $this ->filterForm ->addFilterPP(true, $prava);

//        }

        $filtrOperace = new FilterFormFilter('operace', FilterFormFilter::TYPE_COMBOBOXES);
        $filtrOperace
            ->setSessionName('absence_operace')
            ->setCaption(TextManager::_('Jen_typ_absence'))
            ->setOptionsTranslationPrefix('db_operacedochazky_popis_')
            ->setOptionsSql('SELECT Kod, Popis FROM OperaceDochazky WHERE %FILTER% ORDER BY %ORDER%', 'Absence_Intraweb = 1', 'Popis')
            ->setOptionsSqlOrder2('Kod')
            ->addItem('', TextManager::_('_vse_'))
            ->addItem2('', TextManager::_('_vse_'))
            ->setHtmlAttributes(['id' => 'filtr_operace', 'data-ron-select2' => '#filtr_operace_kod', 'data-ron-autosubmit' => '1'])
            ->setHtmlAttributes2(['id' => 'filtr_operace_kod', 'data-ron-select2' => '#filtr_operace', 'data-ron-autosubmit' => '1'])
            ;
        $this ->filterForm ->addFilter($filtrOperace);
//        $combo = new ComboBox('operace');
//        $combo->showCode = true;
//        $combo->className = 'adsfilterforminput_popis';
//        $combo->emptyRow = TextManager::_('_vse_');
//        $combo->translationPrefix = 'db_operacedochazky_popis_';
//        $combo->translationValueIndex = 0;
//        $combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Popis, Kod', RonPortalSession::getValue('absence_operace'));
//        $combo->htmlParams = [];

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('skrytauto'))
                ->setSessionName('planovani_skrytauto')
                ->setUserSettingsName('browser_absence_skrytauto')
                ->addItem(0, TextManager::_('Zobrazit_automaticky_schvalovane'))
                ->addItem(1, TextManager::_('Skryt_automaticky_schvalovane'))
                ->setDefaultValue(0)
                ->setShowSelectedItem(false)
                ->setRowId('filtr_automatickyschvalovane')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('poznamkyabsence'))
                ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                ->setCookieName('absence_zobrazitpoznamky')
                ->addItem(0, TextManager::_('Skryt_poznamky'))
                ->addItem(1, TextManager::_('Zobrazit_poznamky'))
                ->setDefaultValue(0)
                ->setShowSelectedItem(false)
                ->setRowId('absence_poznamky')
                ->setRowClass('margin-top-2')
        );
    }



    public function getTableAttributes()
    {
        $result = parent::getTableAttributes();
        $result['data-pravoProFiltrovani'] = $this ->pravoProFiltrovani ?? '';
        return $result;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'pravoProFiltrovani') {
            $this ->pravoProFiltrovani = $value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function loadFilterData()
    {
        parent::loadFilterData();

        if (!$this ->filterForm ->getFilterValue('poznamkyabsence')) {
            $f = $this ->findFieldByCode('col-rozdeleni');
            if ($f) {
                $f ->setVisible(false);
            };

            $f = $this ->findFieldByCode('col-poznamka');
            if ($f) {
                $f->setVisible(false);
            }
        }
    }


    /**
     * Zobrazi formular pro hromadne schvalovani
     */
    function hromadneSchvalovaniForm()
    {
        if ( !RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) ) {
            return;
        }
        if ( !$this ->pravaSchvalovani ->jsouNejakePrava() ) {
            return;
        }

        echo
            '<div id="absence_hromadneschvalovani">' .
            '<form method="POST" name="hromadneSchvalovaniForm" action="?menuid=schvalovaniabsence">' .
            '<input type="hidden" name="hromadneSchvaleni" value="1" />' .
            '<input type="hidden" name="akceHromadneSchvaleni" value="0" />' . // nastavi se tlacitkem 1=schvalit, 2=zamitnout
            '<input type="hidden" name="vybraneZaznamy" value="" />' .
            Csrf::csrfHiddenInput('hromadneschvaleniabsenci') .

            '<span class="buttons">' .
            '<input type="button" class="button midbutton" name="schvaleniMenuButton" id="schvaleniMenuButton" value="..." />' .

            '<input id="schvalitVybraneAbsence" type="button" class="button" name="schvalitVybrane" disabled="disabled" value="' . TextManager::_('Schvalit_vybrane') . '" />' .
            '<input id="zamitnoutVybraneAbsence" type="button" class="button" name="zamitnoutVybrane" disabled="disabled" value="' . TextManager::_('Zamitnout_vybrane') . '" />' .
            '<input id="schvalitVsechnyAbsence" type="button" class="button" name="schvalitVse" value="' . TextManager::_('Schvalit_vse') . '" />' .

            '</span>' .
            '</form>'.
            "</div>\n";

        echo
            '<ul class="popupmenu"  id="schvaleniMenu">' .
            "<li id='schvaleniMenuVse'>" . TextManager::_('Oznacit_vse') . "</li>" .
            "<li id='schvaleniMenuNic'>" . TextManager::_('Zrusit_oznaceni') . "</li>" .
            '</ul>';
    }


    function renderFilter()
    {
        parent::renderFilter();

        $this ->hromadneSchvalovaniForm();

        ProgressDialog::renderProgressDialog(TextManager::_('Absence'), [
            //'caption' => TextManager::_('Probiha_schvalovani_sluzebnich_cest')
        ]);

    }


    /**
     * @return string
     */
    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = '';

        switch ($this ->filterForm ->getFilterValue('planovani_filtr')) {
            case 1 : // vse
                break;

            case 2 : // zrusene
                $result =
                    'Absence.Schvaleni = 4';
                break;

            case 3 : // bez schvaleni
                $obdobiOd = Obdobi::prvniNeschvaleneObdobi($this ->database, 'ADS');
                $result =
                    'Absence.Schvaleni = 0' .
                    ($obdobiOd
                        ? ' AND Absence.DatumDo >= ' . $params ->addDateParameter($obdobiOd)
                        : '');
                break;

            case 4 : // ke schvaleni - jako bez schvaleni, ale jen ty, ktere muzu schvalovat
                $obdobiOd = Obdobi::prvniNeschvaleneObdobi($this ->database, 'ADS');
                $result =
                    '(Absence.Schvaleni = 0  OR  Absence.Schvaleni = 3)' .
                    ($obdobiOd
                        ? ' AND Absence.DatumDo >= ' . $params ->addDateParameter($obdobiOd)
                        : '');
                break;

            default : // aktualni a budouci
                $result =
                    'Absence.Schvaleni <> 4 ' . // vynecha zrusene
                    ' AND Absence.DatumDo >= ' . $params ->addDateParameter(time());
                break;
        }

        if ($this ->pravoProFiltrovani) {
            $pravo = Prava::vratPrava($this ->pravoProFiltrovani);
            $filtrPrava = $pravo ->sqlFiltr('Absence', 'Absence.DatumOd', 'Absence.DatumDo', $params);

            if ($filtrPrava) {
                $result .=
                    ($result ? "\n AND " : '') .
                    $filtrPrava;
            }
        }

        $filtr = $this ->filterForm ->buildSqlFilter($params);
        if ($filtr) {
            $result .=
                ($result ? "\n AND " : '') .
                $filtr;
        }

        if ($this ->filterForm ->getFilterValue('skrytauto')) {
            $result .=
                ($result ? "\n AND " : '') .
                'OperaceDochazky.Absence_AutoSchvalovat = 0';
        }

        if ($this ->filterForm ->getFilterValue('operace') !== null) {
            $result .=
                ($result ? "\n AND " : '') .
                'Absence.OperaceDochazky = ' . $params ->addParameter($this ->filterForm ->getFilterValue('operace'));
        }


        if ($this ->filterForm ->getFilterValue('osoba') !== null) {
            // filtr na osobu
            $result .=
                ($result ? "\n AND " : '') .
                'PracovniPomer.Id = ' . $params ->addParameter((int)$this ->filterForm ->getFilterValue('osoba'));
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            // rozdeleni
            $filtr = $rozdeleniFilter ->sestavSQLFiltr($this ->database,'PracovniPomer', $params);

            if ($filtr) {
                $result .=
                    ($result ? "\n AND " : '') .
                    '(' . $filtr . ') ';
            }
        }

        // obdobi (pro "vse" a "zrusene" je povinne)
        if ($this ->filterForm ->getFilterValue('planovani_filtr') == 1 || $this ->filterForm ->getFilterValue('planovani_filtr')  == 2 || $this ->filterForm ->getFilterValue('obdobi')) {
            $result .=
                ($result ? "\n AND " : '') .
                ' Absence.DatumDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
                ' AND Absence.DatumOd <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi', 'datumDo'));
        }

        return $result ? new SqlStatement($result, $params) : null;
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @return string
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .=
            ($className ? ' ' : '') .
            'absence-row' .
            ' absence-typ-' . str_replace(' ', '_', $values['col-operacedochazky']);

        switch ($values['col-kodschvaleni']) {
            case 1 :
            case 5 : // zadost o zruseni zamitnuta = schvaleno
                $className .= ' absence_schvalena';
                break;

            case 2 :
                $className .= ' absence_zamitnuta';
                break;

            case 3 :
                $className .= ' absence_zadostozruseni';
                break;

            case 4 :
                $className .= ' absence_zrusena';
                break;

            default :
                $className .= ' absence_neschvalena';

                if (RonPortalSettings::getIntValue(AbsenceController::SETTINGS_DOPORUCOVANI)) {
                    if ($values['col-doporuceni'] == 1) {
                        $className .= ' absence_doporucena';
                    } else if ($values['col-doporuceni'] == 2) {
                        $className .= ' absence_nedoporucena';
                    }
                }
                break;
        }


        $result = parent::getRowAttributes($values, $className, $rowNumber);

        if ($this ->radekLzeSchvalit) {
            $datumOd = $this ->database ->stringToTimestamp($values['col-datumod']);
            $datumDo = $this ->database ->stringToTimestamp($values['col-datumdo']);

//            $result .=
//                //" data-ron-indexabsence='{$this ->jsonRowIndex}' id='absencerow_{$this ->jsonRowIndex}'" .
//                " data-lze-schvalit='1'" .
//                " data-idpp='" . (int)$values['col-idpp'] . "' data-datum='" . date('j.n.Y', $datumOd) . "' data-poradi='" . (int)$values['col-poradi'] . "'";
            $result['data-lze-schvalit'] = 1;
            $result['data-idpp'] = (int)$values['col-idpp'];
            $result['data-datum'] = date('j.n.Y', $datumOd);
            $result['data-poradi'] = (int)$values['col-poradi'];
            $result['data-csrfkey'] = Csrf::getCsrfKey('schvaleniabsence_' . (int)$values['col-idpp']);
            $result['data-progress-info'] =
                $values['col-jmeno'] . ' ' . $values['col-prijmeni'] .
                ' - ' .
                $values['col-operace'] . ' ' .
                $values['@col-datumod'] .
                ($datumDo != $datumOd ? " - {$values['@col-datumdo']}" : '');
        }

        return $result;
    }


    function enableLine($values)
    {
        $datumOd = $this ->database ->stringToTimestamp($values['col-datumod']);
        $datumDo = $this ->database ->stringToTimestamp($values['col-datumdo']);

        $this ->radekLzeSchvalit = false;
        $planovaniFiltr = $this ->filterForm ->getFilterValue('planovani_filtr');

        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            $pravo = Prava::vratPrava('absence_schval', 'absence_schval_' . $values['col-operacedochazky']);

            if($pravo ->lzeZobrazitPP($values['col-idpp'], $datumOd, $datumDo)) {
                $this ->radekLzeSchvalit = true; 
                return true;
            }

            if ($values['col-doporuceni']  &&  $planovaniFiltr == self::FILTR_KE_SCHVALENI) {
                // zapnut filtr ke schvaleni  a  absence uz je doporucena nebo nedoporucena
                return false;
            }

            $pravo = Prava::vratPrava('absence_doporuc');
            if ($pravo ->lzeZobrazitPP($values['col-idpp'], $datumOd, $datumDo)) {
                // muze doporucit - jeste zkontroluje, jestli nema omezene typy absenci, ktere muze doporucit
                if ($pravo ->priznak3ObsahujeKod($values['col-operacedochazky'])) {
                    return true;
                }
            }

            if ($planovaniFiltr == self::FILTR_KE_SCHVALENI) {
                // filtr "ke schvaleni" - pokud nemuzu schvalovat, tak ho ani nezobrazuju
                return false;
            }
        }

        if ($this ->pravaProhlizeni ->lzeZobrazitPP($values['col-idpp'], $datumOd, $datumDo)) {
            // ma pravo prohlizet
            return true;
        }

        $pravo = Prava::vratPrava('absence_prehled', 'absence_' .$values['col-operacedochazky']);
        if ($pravo ->lzeZobrazitPP($values['col-idpp'], $datumOd, $datumDo)) {
            return true;
        }

        $pravo = Prava::vratPrava('absence', 'absence_' . $values['col-operacedochazky']);
        return $pravo ->lzeZobrazitPP($values['col-idpp'], $datumOd, $datumDo);
    }


    function render()
    {
        echo '<div id="browserabsence">';

        parent::render();

        echo '</div>';
    }
}

