<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

class DenniPoznamkaDochazky
{
    public $idPP;
    public $den;
    public $poznamka;


    function getJson()
    {
        return [
            'idPP' => $this->idPP,
            'datumStr' => StringUtils::jsonDate($this->den),
            'text' => StringUtils::jsonText($this->poznamka)
        ];
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] [$idPP]     Muze byt null, jeden id pracovniho pomeru, nebo pole id pracovnich pomeru
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return DenniPoznamkaDochazky[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPoznamkyDochazky($database, $datumOd, $datumDo, $idPP = null, $prava = null)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, DenniPoznamkaDochazky.Den, ' . $database->castWideMemo('DenniPoznamkaDochazky.Poznamka') .
            ' FROM DenniPoznamkaDochazky, PracovniPomer ' .
            ' WHERE DenniPoznamkaDochazky.RC = PracovniPomer.RC ' .
            '   AND DenniPoznamkaDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND DenniPoznamkaDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND DenniPoznamkaDochazky.Den >= ' . $params->addParameter($database->dateToParam($datumOd)) .
            '   AND DenniPoznamkaDochazky.Den <= ' . $params->addParameter($database->dateToParam($datumDo));

        if ($idPP) {
            if (is_array($idPP)) {
                $sql .= ' AND (';
                $first = true;
                foreach ($idPP as $id) {
                    if (!$first) $sql .= ' OR ';
                    $first = false;

                    $sql .= 'PracovniPomer.Id = ' . $params->addParameter((int)$id);
                }
                $sql .= ')';
            } else {
                $sql .= ' AND PracovniPomer.Id = ' . $params->addParameter($idPP);
            }
        }

//        if ($filtr) {
//            $sql .= ' AND (' . $filtr . ')';
//        }

        $result = [];
        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if (!$prava || Prava::lzeZobrazitPP($prava, $row[0])) {
                $poznamka = new DenniPoznamkaDochazky();
                $poznamka->idPP = $row[0];
                $poznamka->den = $cursor->stringToTimestamp($row[1]);
                $poznamka->poznamka = $row[2];

                $result[] = $poznamka;
            }
        }

        $cursor->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $datum
     * @param int $idPP
     * @return string
     */
    public static function vratPoznamkuDochazky($database, $datum, $idPP)
    {
        $poznamky = self::nactiPoznamkyDochazky($database, $datum, $datum, $idPP);
        if (!count($poznamky)) return null;

        return $poznamky[0] ->poznamka;
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int|int[] [$idPP]     Muze byt null, jeden id pracovniho pomeru, nebo pole id pracovnich pomeru
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return DenniPoznamkaDochazky[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPoznamkyDochazkyJSON($database, $datumOd, $datumDo, $idPP = null, $prava = null)
    {
        $poznamky = self::nactiPoznamkyDochazky($database, $datumOd, $datumDo, $idPP, $prava);
        $jsonPoznamky = [];
        foreach ($poznamky as $poznamka) {
            $jsonPoznamky[] = $poznamka->getJson();
        }

        return $jsonPoznamky;
    }
}
