<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\PracovniPomer;

class DenniVysledek
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    /** @var int */
    public $den;
    /** @var string */
    public $typCasoveSlozkyMzdy;

    /** @var float */
    public $hodiny;
    /** @var float */
    public $dny;
    /** @var float */
    public $kalendarniDny;

    public $poznamka;


    /**
     * @param Database $database
     * @param array $row
     */
    protected function loadBasicData($database, $row)
    {
        $this ->rc = $row[0];
        $this ->osobniCislo = $row[1];
        $this ->datumNastupu = $database ->stringToTimestamp($row[2]);

        $this ->den = $database ->stringToTimestamp($row[3]);
        $this ->typCasoveSlozkyMzdy = $row[4];

        $this ->hodiny = (float)$row[5];
        $this ->dny = (float)$row[6];
        $this ->kalendarniDny = (float)$row[7];
    }


    /**
     * @param Database $database
     * @param array $row
     */
    private function load($database, $row)
    {
        $this ->loadBasicData($database, $row);
        $this ->poznamka = $row[8];
    }


    /**
     * @param Database $database
     * @param PracovniPomer $osoba
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string|string[] $typCasoveSlozkyMzdy
     * @return DenniVysledek[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiDenniVysledky($database, $osoba, $obdobiOd, $obdobiDo, $typCasoveSlozkyMzdy = null)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT RC, OsobniCislo, DatumNastupu, Den, TypCasoveSlozkyMzdy, ' .
            ' Hodiny, Dny, KalendarniDny, Poznamka' .
            ' FROM DenniVysledekVypoctu';

        $filtr = '';
        if ($osoba) {
            $filtr =
                ' RC = ' . $params ->addParameter($osoba ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($osoba ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($osoba ->datumNastupu));
        }

        if ($obdobiOd) {
            if ($filtr) $filtr .= ' AND';
            $filtr .=
                ' Den >= ' . $params ->addParameter($database ->dateToParam($obdobiOd));
        }

        if ($obdobiDo) {
            if ($filtr) $filtr .= ' AND';
            $filtr .=
                ' Den <= ' . $params ->addParameter($database ->dateToParam($obdobiDo));
        }

        if ($typCasoveSlozkyMzdy) {
            if ($filtr) $filtr .= ' AND';

            if (is_array($typCasoveSlozkyMzdy)) {
                $filtr .= ' (';

                $first = true;
                foreach ($typCasoveSlozkyMzdy as $slozka) {
                    if (!$first) {
                        $filtr .= ' OR';
                    }
                    $first = false;

                    $filtr .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($slozka);
                }

                $filtr .= ')';

            } else {
                $filtr .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy);
            }
        }

        if ($filtr) $sql .= ' WHERE ' . $filtr;

        $sql .= ' ORDER BY RC, OsobniCislo, DatumNastupu, Den, TypCasoveSlozkyMzdy';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani dennich vysledku doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            $result[] = $vysledek;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param PracovniPomer $osoba
     * @param int    $den
     * @param string $casovaSlozka
     * @param string $jednotka
     * @return float|null
     */
    public static function nactiHodnotu($database, $osoba, $den, $casovaSlozka, $jednotka = 'H')
    {
        $data = self::nactiDenniVysledky($database, $osoba, $den, $den, $casovaSlozka);
        if (!count($data)) return null;

        switch ($jednotka) {
            case 'D' :
            case 'd' :
                return $data[0] ->dny;
            case 'K' :
            case 'k' :
                return $data[0] ->kalendarniDny;

            default :
                return $data[0] ->hodiny;
        }
    }


    function vratHodnotu($jednotka)
    {
        switch (strtoupper($jednotka)) {
            case 'H' :
                return (float)$this ->hodiny;

            case 'D' :
                return (float)$this ->dny;

            case 'K' :
                return (float)$this ->kalendarniDny;
        }

        return null;
    }


    function vratText($jednotka, $format = null, $zobrazitNuloveHodnoty = false)
    {
        switch (strtoupper($jednotka)) {
            case 'H' :
                return (isset($this ->hodiny)  &&  ($zobrazitNuloveHodnoty || $this ->hodiny))
                    ? StringUtils::format($format ?: 'minuty', round($this ->hodiny * 60))
                    : '';

            case 'D' :
                return (isset($this ->dny)  &&  ($zobrazitNuloveHodnoty || $this ->dny))
                    ? StringUtils::format($format ?: 'number_2', $this ->dny)
                    : '';

            case 'K' :
                return (isset($this ->kalendarniDny)  &&  ($zobrazitNuloveHodnoty || $this ->kalendarniDny))
                    ? StringUtils::format($format ?: 'number_2', $this ->kalendarniDny)
                    : '';

            case 'P' :
                return $this ->poznamka;
        }

        return '';
    }
}
