<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

class DokumentDochazky
{
    public $idPP;
    public $datum;
    public $id;
    public $jmenoSouboru;
    public $popis;

    /**
     * Vraci pole pro generovani json
     * @return array
     */
    public function getJson()
    {
        return [
            'idPP' => $this ->idPP,
            'datumStr' => StringUtils::jsonDate($this ->datum),
            'id' => $this ->id,
            'jmenoSouboru' => StringUtils::jsonText($this ->jmenoSouboru),
            'popis' => StringUtils::jsonText($this ->popis)
        ];
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param string [$filter]
     * @param \Ron\DB\Parameters [$filterParams]
     * @return DokumentDochazky[]
     * @throws DBException
     */
    public static function nactiDokumentyDochazky($database, $datumOd, $datumDo, $prava, $filter = null, $filterParams = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PracovniPomer.Id, DokumentDochazky.Den, DokumentDochazky.Id, ' .
            ' Dokument.JmenoSouboru, ' . $database ->castWideMemo('Dokument.Popis') .
            ' FROM DokumentDochazky, PracovniPomer, Dokument ' .
            ' WHERE DokumentDochazky.RC = PracovniPomer.RC ' .
            '   AND DokumentDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND DokumentDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND DokumentDochazky.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND DokumentDochazky.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND Dokument.Id = DokumentDochazky.Id';

        if ($filter) {
            $sql .= ' AND (' . $filter . ')';
            if ($filterParams) {
                $params ->appendParameters($filterParams);
            }
        }

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            if (!$prava || Prava::lzeZobrazitPP($prava, $row[0])) {
                $dokument = new self();
                $dokument ->idPP = (int)$row[0];
                $dokument ->datum = $database ->stringToTimestamp($row[1]);
                $dokument ->id = (int)$row[2];
                $dokument ->jmenoSouboru = $row[3];
                $dokument ->popis = $row[4];

                $result[] = $dokument;
            }
        }

        $cursor ->close();

        return $result;
    }

    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int $idPP
     * @return DokumentDochazky[]
     * @throws DBException
     */
    public static function nactiDokumentyOsoby($database, $datumOd, $datumDo, $idPP)
    {
        $params = $database ->createParameters();
        $filter = 'PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        return self::nactiDokumentyDochazky($database, $datumOd, $datumDo, null, $filter, $params);
    }


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int[] $seznamPP
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @return DokumentDochazky[]
     * @throws DBException
     */
    public static function nactiDokumentyOsob($database, $datumOd, $datumDo, $seznamPP, $prava)
    {
        $params = $database ->createParameters();
        $filter = '(0 = 1';
        foreach ($seznamPP as $idPP) {
            if (!$prava || Prava::lzeZobrazitPP($prava, (int)$idPP)) {
                $filter .= ' OR PracovniPomer.Id = ' . $params->addParameter((int)$idPP);
            }
        }
        $filter .= ')';
        return self::nactiDokumentyDochazky($database, $datumOd, $datumDo, null, $filter, $params);
    }


    /**
     * Vraci html pro zobrazeni odkazu na stahnuti. Pokud neni $obdobi zadano, urci se jako pocatek mesice
     * @param null $obdobi
     * @return string
     */
    public function getDownloadHtml($obdobi = null)
    {
        if (!$obdobi) {
            $obdobi = DateUtils::firstDayOfMonth($this ->datum);
        }

        return
            ' <a href="?menuid=editace&action=download&idpp=' . $this->idPP . '&obdobi=' . DateUtils::dateToUrlString($obdobi) . '&record=' . $this ->id . '" target="_blank">' .
            '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . StringUtils::htmlTextConvert($this ->popis . ' (' . $this ->jmenoSouboru . ')') . '"></i>' .
            '</a>';
    }
}
