<?php
/**
 * stanislav.elbl, 16.03.2021
 * Formular pro hromadne schvaleni dochazky
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;

class HromadneSchvaleniFormular implements ViewObject
{
    /** @var Database */
    private $database;

    /** @var Obdobi */
    private $obdobi;
    private $datumZpracovaniDochazky;

    /** @var int        Kod typu rozdeleni*/
    private $typRozdeleni;
    /** @var string     Kod rozdeleni */
    private $kodRozdeleni;
    /** @var  Rozdeleni */
    private $rozdeleni;
    /** @var int */
    private $rozdeleniVcetnePodrizenych;

    /** @var int        Kod typu rozdeleni 2 */
    private $typRozdeleni2;
    /** @var string     Kod rozdeleni 2 */
    private $kodRozdeleni2;
    /** @var  Rozdeleni */
    private $rozdeleni2;
    /** @var int */
    private $rozdeleni2VcetnePodrizenych;

    /** @var bool       pokud uzivatel muze editovat alespon jednu osobu, nastavi se na true */
    private $editovatZpracovano;
    /** @var int */
    private $zpracovanoDo;
    /** @var string */
    private $zpracoval;

    /** @var bool */
    private $editovatZkontrolovano;
    /** @var int */
    private $zkontrolovanoDo;
    /** @var string */
    private $zkontroloval;

    /** @var bool */
    private $editovatSchvaleno;
    /** @var int */
    private $schvalenoDo;
    /** @var string */
    private $schvalil;

    /** @var string */
    private $poznamka;


    function __construct($database)
    {
        $this ->database = $database;

        // rozdeleni 1
        $this->typRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);

        if (isset($_POST['rozdeleni'])) {
            $this->kodRozdeleni = $_POST['rozdeleni'];
        } else if (isset($_GET['rozdeleni'])) {
            $this->kodRozdeleni = $_GET['rozdeleni'];
        } else {
            $this->kodRozdeleni = '';
        }

        if (isset($_POST['rozdeleni_vcetne_podrizenych'])) {
            $this ->rozdeleniVcetnePodrizenych = (int)$_POST['rozdeleni_vcetne_podrizenych'];
        } else if (isset($_GET['rozdeleni_vcetne_podrizenych'])) {
            $this ->rozdeleniVcetnePodrizenych = (int)$_GET['rozdeleni_vcetne_podrizenych'];
        } else {
            $this ->rozdeleniVcetnePodrizenych = 1;
        }

        // rozdeleni 2
        $this->typRozdeleni2 = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP);

        if (isset($_POST['rozdeleni2'])) {
            $this->kodRozdeleni2 = $_POST['rozdeleni2'];
        } else if (isset($_GET['rozdeleni2'])) {
            $this->kodRozdeleni2 = $_GET['rozdeleni2'];
        } else {
            $this->kodRozdeleni2 = '';
        }

        if (isset($_POST['rozdeleni2_vcetne_podrizenych'])) {
            $this ->rozdeleni2VcetnePodrizenych = (int)$_POST['rozdeleni2_vcetne_podrizenych'];
        } else if (isset($_GET['rozdeleni2_vcetne_podrizenych'])) {
            $this ->rozdeleni2VcetnePodrizenych = (int)$_GET['rozdeleni2_vcetne_podrizenych'];
        } else {
            $this ->rozdeleni2VcetnePodrizenych = 1;
        }
    }


    /**
     * @param int $obdobi
     * @throws \Exception
     */
    function nastavObdobi($obdobi)
    {
        $this->obdobi = new Obdobi();

        if (!$this->obdobi->nactiObdobi($this ->database, 'ADS', $obdobi)) {
            // nemelo by nastat
            throw new \Exception('Chyba! Nelze nacist obdobi!');
        }

        if ($this->obdobi->uzavreno) {
            throw new \Exception(\Ron\ronText('Obdobi_uzavreno'));
        }
        $this->obdobi->nastavSession();

        Prava::nastavObdobiProPrava($this->obdobi->zacatek, $this->obdobi->konec);
    }


    /**
     * @param Cursor $cursor
     * @return bool
     */
    function nactiZaznamySchvaleni($cursor)
    {
        $this->editovatZpracovano = false;
        $this->editovatZkontrolovano = false;
        $this->editovatSchvaleno = false;

        $prava1 = Prava::vratPrava('zpracovani');
        $prava2 = Prava::vratPrava('zkontrolovani');
        $prava3 = Prava::vratPrava('schvaleni');

        $first = true;
        while ($row = $cursor ->fetch()) {
            if ($first) {
                // prvni nacteny zaznam - inicializuje hodnoty
                if ($row[0]) $this->zpracovanoDo = $this->database ->stringToTimestamp($row[0]);
                $this->zpracoval = $row[1];

                if ($row[2]) $this->zkontrolovanoDo = $this->database ->stringToTimestamp($row[2]);
                $this->zkontroloval = $row[3];

                if ($row[4]) $this->schvalenoDo = $this->database ->stringToTimestamp($row[4]);
                $this->schvalil = $row[5];

                $this->poznamka = $row[6];

                $first = false;
            } else {
                // dalsi nacteny zaznam - porovna hodnoty a pokud nejsou stejne tak je "vypne"
                if (!$row[0] || $this->database ->stringToTimestamp($row[0]) != $this->zpracovanoDo) {
                    $this->zpracovanoDo = null;
                }
                if ($row[1] != $this->zpracoval) {
                    $this->zpracoval = '';
                }


                if (!$row[2] || $this->database ->stringToTimestamp($row[2]) != $this->zkontrolovanoDo) {
                    $this->zkontrolovanoDo = null;
                }
                if ($row[3] != $this->zkontroloval) {
                    $this->zkontroloval = '';
                }


                if (!$row[4] || $this->database ->stringToTimestamp($row[4]) != $this->schvalenoDo) {
                    $this->schvalenoDo = null;
                }
                if ($row[5] != $this->schvalil) {
                    $this->schvalil = '';
                }

                if ($row[6] != $this->poznamka) {
                    $this->poznamka = '';
                }
            }

            // kontrola prav
            if (!$this->editovatZpracovano  &&  $prava1->lzeZobrazitPP($row[7])) {
                $this->editovatZpracovano = true;
            }
            if (!$this->editovatZkontrolovano  &&  $prava2->lzeZobrazitPP($row[7])) {
                $this->editovatZkontrolovano = true;
            }
            if (!$this->editovatSchvaleno  &&  $prava3->lzeZobrazitPP($row[7])) {
                $this->editovatSchvaleno = true;
            }
        }

        return true;
    }



    /**
     * Nacte seznam osob a prislusnych zaznamu schvaleni
     * @return Cursor
     * @throws \Ron\DB\DBException
     */
    function vratZaznamy()
    {
        $sql =
            'SELECT SchvaleniDochazky.ZpracovanoDo, SchvaleniDochazky.Zpracoval, ' . // 0,1
            '   SchvaleniDochazky.ZkontrolovanoDo, SchvaleniDochazky.Zkontroloval, ' . // 2,3
            '   SchvaleniDochazky.SchvalenoDo, SchvaleniDochazky.Schvalil, ' . // 4,5
            '   {fn_CASTWIDEMEMO(SchvaleniDochazky.Poznamka)}, PracovniPomer.Id, ' . // 6,7
            '   PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu ' . //8..10
            ' FROM PracovniPomer LEFT OUTER JOIN SchvaleniDochazky ' .
            '   ON SchvaleniDochazky.RC = PracovniPomer.RC ' .
            '  AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '  AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu ' .

            " AND SchvaleniDochazky.Aplikace = 'ADS' " .
            ' AND SchvaleniDochazky.PocatekObdobi = ' . $this->database ->timestampToString($this->obdobi->zacatek) .

            ' WHERE PracovniPomer.Zamestnanec > 0 ' .
            '   AND PracovniPomer.ZpracovatDochazku = 1 ' .
            '   AND PracovniPomer.DatumNastupu < ' . $this->database ->timestampToString($this->obdobi->konec) . // jen platne PP
            '   AND ( PracovniPomer.DatumUkonceni >= ' . $this->database ->timestampToString($this->obdobi->zacatek) .
            ' OR PracovniPomer.DatumUkonceni IS NULL )';

        if ($this->kodRozdeleni) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                $this->typRozdeleni, $this->kodRozdeleni,
                $this->obdobi->zacatek, $this->obdobi->konec,
                $this ->rozdeleniVcetnePodrizenych);

            if ($filtr) {
                $sql .= ' AND (' . $filtr . ')';
            }
        }

        if ($this->kodRozdeleni2) {
            $filtr = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                $this->typRozdeleni2, $this->kodRozdeleni2,
                $this->obdobi->zacatek, $this->obdobi->konec,
                $this ->rozdeleni2VcetnePodrizenych);

            if ($filtr) {
                $sql .= ' AND (' . $filtr . ')';
            }
        }

        return $this ->database ->getSqlResult($sql, null, 'Chyba pri nacitani schvaleni!');
    }


    /**
     * Nacte hodnoty pro osoby z rozdeleni a vybrane obdobi. Pokud jsou hodnoty ruzne, tak je nezobrazuje
     * @return bool
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function nactiData()
    {
        $this->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        $data = $this->vratZaznamy();

        $this ->rozdeleni = ($this ->typRozdeleni  &&  $this ->kodRozdeleni)
            ? Rozdeleni::loadRozdeleni($this ->database, $this ->typRozdeleni, $this ->kodRozdeleni)
            : null;

        $this ->rozdeleni2 = ($this ->typRozdeleni2  &&  $this ->kodRozdeleni2)
            ? Rozdeleni::loadRozdeleni($this ->database, $this ->typRozdeleni2, $this ->kodRozdeleni2)
            : null;

        return $this->nactiZaznamySchvaleni($data);
    }



    function vlozPolickoSDatem($caption, $captionclass, $name, $value, $username, $editace, $datumPredplneni)
    {
        $checked = isset($_POST[$name . '_check']) ? $_POST[$name . '_check'] : false;
        $error = MessageManager::getInstance() ->isError($name);

        echo "<div class='form-row' id='hromadneschvaleni_$name'><div class='form-col1 caption $captionclass'><label>$caption";
        if ($editace) {
            $checkedstr = $checked ? "checked='checked'" : '';
            echo "<INPUT type='checkbox' name='{$name}_check' $checkedstr>";
        }
        echo "</label></div>";

        echo '<div class="form-col2">';
        $valueStr = $value
            ? date('j.n.Y', $value)
            : '';

        if ($editace) {
            $classname = 'form-control date-input' . ($error ? ' is-invalid' : '');  // TODO: form-control !

            if (isset($_POST[$name])) {
                $valueStr = StringUtils::htmlTextConvert($_POST[$name]);
            }

            echo
                '<input type="text" name="' . $name . '" value="' . $valueStr . '" class= "' . $classname . '" maxlength="10" size="10" '.
                //' inputmode="numeric"' .
                ' data-roncalendar-mindate="' . date('j.n.Y', $this ->obdobi ->zacatek) . '" ' .
                ' data-roncalendar-maxdate="' . date('j.n.Y', $datumPredplneni) . '" ' .
                '>';
//            echo
//                '<i class="showcalendar inline-glyph fa glyph-calendar glyph-button margin-all-2"  data-roncalendar-inputname="' . $name . '"></i>';

            // odkaz pro predplneni do konce obdobi/posledniho zpracovani
            if ($datumPredplneni > 0) {
                $datumPredplneniStr = date('j.n.Y', $datumPredplneni);
                echo
                    "<span class='margin-left-2 ml-2'><a href='#' class='setValue' data-selectors-count='2'" .
                    " data-selector1='input[name=$name]' data-value1='$datumPredplneniStr'" .
                    " data-selector2='input[name={$name}_check]' data-value2='1'" .
                    ">" . $datumPredplneniStr . '</a></span>';
            }
        } else {
            echo $valueStr;
        }

        echo ' ' . StringUtils::htmlTextConvert($username) . '</div>';

        echo "</div>\n";
    }



    function render()
    {
        require('ronportal/dochazka/view_hromadne_schvaleni.php');
    }


    /**
     * Kontrola datumu zadaneho uzivatelem (format, rozsah) - vraci zadane datum. V pripade chyby vyhodi vyjimku
     * @param string $postName
     * @param string $errorStr
     * @param int $maximalnidatum
     * @return int|null
     * @throws RonException
     */
    function zkontrolujDatum($postName, $errorStr, $maximalnidatum)
    {
        if ($_POST[$postName]) {
            try {
                $datum = DateUtils::dateStringToTimestamp($_POST[$postName]);
            } catch (\Exception $e) {
                throw new RonException($errorStr, $postName, null, 0, $e);
            }

            // kontrola rozsahu
            if ($datum < $this->obdobi->zacatek || $this->zpracovanoDo > $maximalnidatum) {
                throw new RonException(
                    $errorStr . ' ' .
                    sprintf(\Ron\ronText('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $this->obdobi->zacatek),
                        date('j.n.Y', $maximalnidatum)),
                    $postName
                );
            }
        } else {
            $datum = null;
        }

        return $datum;
    }


    /**
     * Prevede zadane retezcove udaje ve formulari na timestampy. Pokud je neco zadano spatne tak vyhodi vyjimku.
     * @throws \Exception
     */
    function zkontrolujZadaneUdaje()
    {
        Csrf::validateCSRFPostParam('schvaleni_doch_' . $this ->obdobi ->zacatek);

        $maximalniDatum = $this->datumZpracovaniDochazky;
        if ($maximalniDatum > $this->obdobi->konec) {
            $maximalniDatum = $this->obdobi->konec;
        }
        $maximalniDatum = DateUtils::startOfTheDay($maximalniDatum);

        // nacte zadane hodnoty a zkontroluje je
        $this->poznamka = $_POST['poznamka'];

        $this->zpracovanoDo = $this ->zkontrolujDatum('zpracovanodo', \Ron\ronText('Nespravne_datum_schvaleni1'), $maximalniDatum);

        // kontrola "odschvaleni" az u konkretni osoby!

        $this->zkontrolovanoDo = $this ->zkontrolujDatum('zkontrolovanodo', \Ron\ronText('Nespravne_datum_schvaleni2'), $maximalniDatum);

        $this->schvalenoDo = $this ->zkontrolujDatum('schvalenodo', \Ron\ronText('Nespravne_datum_schvaleni1'), $maximalniDatum);
    }


    /**
     * @param array $row
     * @return bool
     */
    function zapisSchvaleniOsoby($row)
    {
        $rec = new SchvaleniDochazkyRecord($this ->database, $this);

        $pp = PracovniPomer::nactiPP($this ->database, $row[7]);

        $exists = $rec ->loadDataProOsobu($pp, $this->obdobi);

        $rec ->addField('', '', true, 0, '   Hromadne schvaleni pro rozdeleni: ' . $this->kodRozdeleni);
        if ($this ->typRozdeleni2) {
            $rec ->addField('', '', true, 0, '   rozdeleni 2: ' . $this->kodRozdeleni2);
        }

        $prava1 = Prava::vratPrava('zpracovani');
        $editovatZpracovano = isset($_POST['zpracovanodo_check'])  &&  $_POST['zpracovanodo_check']  &&  $prava1 ->lzeZobrazitPP($row[7]);

        $prava2 = Prava::vratPrava('zkontrolovani');
        $editovatZkontrolovano = isset($_POST['zkontrolovanodo_check'])  &&  $_POST['zkontrolovanodo_check']  &&  $prava2 ->lzeZobrazitPP($row[7]);

        $prava3 = Prava::vratPrava('schvaleni');
        $editovatSchvaleno = isset($_POST['schvalenodo_check'])  &&  $_POST['schvalenodo_check']  &&  $prava3 ->lzeZobrazitPP($row[7]);

        $zapsat = false;

        $msgPrefix = $pp->jmenoPrijmeni . ' (' . $pp->osobniCislo . '): ';

        if ($editovatZpracovano  &&  $this ->zpracovanoDo != $rec ->fZpracovano ->originalValue) {
            // doslo ke zmene schvaleni 1
            if (!$rec->nastavPoleSchvaleni(1, $this->zpracovanoDo, $msgPrefix)) {
                return false;
            }
            $zapsat = true;
        }


        if ($editovatZkontrolovano  &&  $this ->zkontrolovanoDo != $rec ->fZkontrolovano ->originalValue) {
            // doslo ke zmene schvaleni 2
            if (!$rec->nastavPoleSchvaleni(2, $this->zkontrolovanoDo, $msgPrefix)) {
                return false;
            }
            $zapsat = true;
        }

        if ($editovatSchvaleno  &&  $this ->schvalenoDo != $rec ->fSchvaleno ->originalValue) {
            // doslo ke zmene schvaleni 3
            if (!$rec->nastavPoleSchvaleni(3, $this->schvalenoDo, $msgPrefix)) {
                return false;
            }
            $zapsat = true;
        }


        if (!$zapsat) {
            // neni co menit
            return true;
        }

        if (isset($_POST['poznamka_check'])) if ($_POST['poznamka_check']) {
            // poznamku meni jen v pripade, ze ma pravo alespon na jedno pole
            $rec ->fPoznamka ->setNewValue($_POST['poznamka']);
        }

        return $rec ->zkontrolujAZapisSchvaleni(!$exists, $msgPrefix);
    }



    /**
     * Zapise zadane hodnoty. V pripade uspechu vraci true.
     * @return bool
     * @throws \Exception
     */
    function zapisData()
    {
        $this->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        $this->zkontrolujZadaneUdaje();

        // nacte seznam osob (idpp)
        $cursor = $this->vratZaznamy();

        $result = true;

        $rows = [];
        while ($row = $cursor ->fetch()) {
            $rows[] = $row;
        }

        foreach ($rows as $row) {
            //$editovat = true;
            try {
                $this->zapisSchvaleniOsoby($row);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $result = false;
            }
        }

        return $result;
    }
}
