<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\StringUtils;
use RonPortal\PracovniPomer;

class MzdovaPolozka
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    /** @var int */
    public $obdobi;

    /** @var string */
    public $mzdovaPolozka;

    /** @var int */
    public $druhZdrojeMzdovePolozky;
    /** @var int */
    public $serioveCislo;

    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;

    /** @var float */
    public $pocetJednotek;
    /** @var float */
    public $cenaZaJednotku;
    /** @var float */
    public $castka;

    /** @var string     Naplni se podle Update_UserName nebo Insert_UserName */
    public $lastUpdateUser;
    /** @var string     Naplni se podle Update_Timestamp nebo Insert_Timestamp */
    public $lastUpdateTimestamp;


    /**
     * @param Database $database
     * @param array $row
     */
    private function load(Database $database, $row)
    {
        $this ->rc = $row[0];
        $this ->osobniCislo = $row[1];
        $this ->datumNastupu = $database ->stringToTimestamp($row[2]);

        $this ->obdobi = $database ->stringToTimestamp($row[3]);
        $this ->mzdovaPolozka = $row[4];

        $this ->druhZdrojeMzdovePolozky = (int)$row[5];
        $this ->serioveCislo = $row[6];
        $this ->kodTypuRozdeleni = isset($row[7]) ? (int)$row[7] : null;
        $this ->rozdeleni = $row[8];

        $this ->pocetJednotek = (float)$row[9];
        $this ->cenaZaJednotku = (float)$row[10];
        $this ->castka = (float)$row[11];

        if ($row[14]) {
            $this ->lastUpdateUser = $row[14];
            $this ->lastUpdateTimestamp = $database ->stringToTimestamp($row[15]);
        } else {
            $this ->lastUpdateUser = $row[12];
            $this ->lastUpdateTimestamp = $database ->stringToTimestamp($row[13]);
        }
    }


    /**
     * @param Database $database
     * @param PracovniPomer|null $osoba
     * @param int $pocatekObdobi
     * @param string|string[] $typMzdovePolozky
     * @return MzdovaPolozka[]
     * @throws DBException
     */
    public static function nactiMzdovePolozky(Database $database, $osoba, $pocatekObdobi, $typMzdovePolozky = null, $druhZdrojeMzdovePolozky = 4)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT RC, OsobniCislo, DatumNastupu, Obdobi, MzdovaPolozka, ' . // 0..4
            ' DruhZdrojeMzdovePolozky, SerioveCislo, KodTypuRozdeleni, Rozdeleni, ' . // 5..8
            ' PocetJednotek, CenaZaJednotku, Castka,' . // 9..11
            ' Insert_UserName, Insert_Timestamp,' . // 12..
            ' Update_UserName, Update_Timestamp' .  // ..15
            "\nFROM MzdovaPolozkaVeMzde";

        $filtr = '';
        if ($osoba) {
            $filtr =
                ' RC = ' . $params ->addParameter($osoba ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($osoba ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addDateParameter($osoba ->datumNastupu);
        }

        if ($druhZdrojeMzdovePolozky) {
            $filtr .=
                ($filtr ? ' AND ' : '') .
                ' DruhZdrojeMzdovePolozky = ' . $params ->addParameter((int)$druhZdrojeMzdovePolozky);
        }

        if ($pocatekObdobi) {
            $filtr .=
                ($filtr ? ' AND ' : '') .
                ' Obdobi = ' . $params ->addDateParameter($pocatekObdobi);
        }

        if ($typMzdovePolozky) {
            if ($filtr) $filtr .= ' AND ';

            if (is_array($typMzdovePolozky)) {

                $filtr .= SQLUtils::getINCondition($database, 'MzdovaPolozka', $typMzdovePolozky, $params);
//                $filtr .= ' (0 = 1';
//
//                foreach ($typMzdovePolozky as $polozka) {
//                    if (isset($polozka)) {
//                        $filtr .= ' OR MzdovaPolozka = ' . $params ->addParameter($polozka);
//                    }
//                }
//
//                $filtr .= ')';

            } else {
                $filtr .= ' MzdovaPolozka = ' . $params ->addParameter($typMzdovePolozky);
            }
        }

        if ($filtr) {
            $sql .= "\nWHERE " . $filtr;
        }

        $sql .= "\nORDER BY RC, OsobniCislo, DatumNastupu, Obdobi, MzdovaPolozka";

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani mesicnich vysledku doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            $result[] = $vysledek;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param string $pole      Jmeno datoveho pole, jehoz hodnota se ma vratit (Castka/PocetJednotek/CenaZaJednotku)
     * @param string $format    Format zobrazene hodnoty
     * @param bool $skrytNuly   Urcuje, jestli se ma nulova hodnota skryt (nevraci 0, ale prazdny retezec)
     * @return string
     */
    public function vratText($pole = 'castka', $format = null, $skrytNuly = false)
    {
        switch (strtolower($pole)) {
            case 'pocetjednotek' :
                $hodnota = $this ->pocetJednotek;
                break;

            case 'cenazajednotku' :
                $hodnota = $this ->cenaZaJednotku;
                break;

            default :
                $hodnota = $this ->castka;
        }

        if (!isset($hodnota)) {
            return '';
        }

        if (!$hodnota && $skrytNuly) {
            return '';
        }

        return StringUtils::format($format ?: 'number2', $hodnota);
    }


    public function vratHint($format = null) {
        $userHint = $this ->lastUpdateUser
            ? $this ->lastUpdateUser . ', ' . StringUtils::format('timestamp', $this ->lastUpdateTimestamp)
            : '';

        if ($this ->pocetJednotek != 1) {
            return
                $this ->pocetJednotek . ' x ' . StringUtils::format($format ?: 'number2', $this ->cenaZaJednotku) .
                ($userHint ? ' (' . $userHint . ')' : '');
        }

        return $userHint;
    }
}
