<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\RozdeleniOsob;

class OsobaDochazky extends PracovniPomer
{
    /** @var  string[] pole uzivatelskych polozek */
    public $uzivatelska;
    /** @var  string[] */
    public $uzivatelskaPP;

    /** @var string */
    public $kodRozdeleni;
    public $popisRozdeleni;

    /** @var SchvaleniDochazky */
    public $schvaleni;

    /** @var RozdeleniOsob[]    Seznam prirazenych rozdeleni (agenda nacita ty, ktere potrebuje) */
    public $rozdeleni = [];

    /**
     * Vraci zakladni cast SQL dotazu pro vyber osob
     * @param Database $database
     * @param Obdobi $obdobi
     * @param int $vyberPPVObdobi Urcuje, ktere PP v obdobi vybere - 1=platne na zacatku obdobi, 2=na konci, 3=kdykoliv v obdobi
     * @param ?int $typRozdeleni
     * @param string $kodRozdeleni Nemusi byt zadane - v tom pripade rozdeleni nefiltruje, ale jen nacte aktualne prirazene
     * @param int $vcetnePodrizenych
     * @param bool $nefiltrovatZamestnance Pokud je true, tak se do dotazu nevklada podminka "zamestnanec>0 AND ZpracovatDochazku=1"
     * @param Parameters $params
     * @return string
     */
    public static function vratZakladniSQL($database, $obdobi, $vyberPPVObdobi, $typRozdeleni, $kodRozdeleni, $vcetnePodrizenych, $nefiltrovatZamestnance = false, $params = null)  // , $filtr
    {
        $typRozdeleni = (int)($typRozdeleni ?? 0);
        $vcetnePodrizenych = (int)$vcetnePodrizenych;

        switch ($vyberPPVObdobi) {
            case 1 : // zacatek obdobi
                $datumOd = $obdobi ->zacatek;
                $datumDo = $obdobi ->zacatek;
                break;

            case 2 : // konec obdobi
                $datumOd = $obdobi ->konec;
                $datumDo = $obdobi ->konec;
                break;

            default :
                $datumOd = ($vyberPPVObdobi < -1)
                    // kdykoliv behem obdobi a zadaneho poctu predchozich mesicu (napr. -3 je ctvrtleti koncici zadanym obdobim)
                    ? DateUtils::incMonth($obdobi->zacatek, $vyberPPVObdobi + 1)
                    // kdykoliv behem obdobi
                    : $obdobi->zacatek;
                $datumDo = $obdobi->konec;
                break;
        }

        $sql =
            'SELECT PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.Id, ' . // 0 - 3
            '  Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . // 4 - 6
            ($typRozdeleni ? '  RozdeleniOsob.Rozdeleni, Rozdeleni.Popis, ' : 'NULL AS Rozdeleni, NULL AS PopisRozdeleni, ') . // 7, 8
            '  PracovniPomer.DatumUkonceni, PracovniPomer.TypPracovnihoPomeru, ' . // 9, 10
            '  Osoba.Uzivatelska1, Osoba.Uzivatelska2, Osoba.Uzivatelska3, Osoba.Uzivatelska4, Osoba.Uzivatelska5, ' . // 11..15
            '  Osoba.Uzivatelska6, Osoba.Uzivatelska7, Osoba.Uzivatelska8, Osoba.Uzivatelska9, Osoba.Uzivatelska10, ' . // 16..20
            '  Osoba.Uzivatelska11, Osoba.Uzivatelska12, Osoba.Uzivatelska13, Osoba.Uzivatelska14, Osoba.Uzivatelska15, ' . // 21..25
            '  Osoba.Uzivatelska16, Osoba.Uzivatelska17, Osoba.Uzivatelska18, Osoba.Uzivatelska19, Osoba.Uzivatelska20, ' . // 26..30
            '  PracovniPomer.Uzivatelska1, PracovniPomer.Uzivatelska2, PracovniPomer.Uzivatelska3, PracovniPomer.Uzivatelska4, PracovniPomer.Uzivatelska5, ' . // 31..35
            '  PracovniPomer.Uzivatelska6, PracovniPomer.Uzivatelska7, PracovniPomer.Uzivatelska8, PracovniPomer.Uzivatelska9, PracovniPomer.Uzivatelska10, ' . // 36..40
            '  PracovniPomer.Uzivatelska11, PracovniPomer.Uzivatelska12, PracovniPomer.Uzivatelska13, PracovniPomer.Uzivatelska14, PracovniPomer.Uzivatelska15, ' . // 41..45
            '  PracovniPomer.Uzivatelska16, PracovniPomer.Uzivatelska17, PracovniPomer.Uzivatelska18, PracovniPomer.Uzivatelska19, PracovniPomer.Uzivatelska20, ' . // 46..50
            '  SchvaleniDochazky.ZpracovanoDo, SchvaleniDochazky.ZkontrolovanoDo, SchvaleniDochazky.SchvalenoDo, SchvaleniDochazky.ZpracovanoVeMzdach, ' . // 51..54
            '  Osoba.IdFotografie '; // 55

        if ($typRozdeleni) {
            ///$this ->f_hlavnityprozdeleni = $typrozdeleni;
            // Bude nacteno i rozdeleni dane osoby

            $fromStr =
                '( ( ( Osoba INNER JOIN PracovniPomer' .
                '    ON Osoba.RC = PracovniPomer.RC' .
                '    )'.
                '       LEFT OUTER JOIN SchvaleniDochazky' .
                '        ON SchvaleniDochazky.RC = PracovniPomer.RC' .
                '       AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo' .
                '       AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu' .
                "       AND SchvaleniDochazky.Aplikace = 'ADS'".
                '       AND SchvaleniDochazky.PocatekObdobi = ' . ($params ? $params ->addDateParameter($obdobi ->zacatek) : $database ->dateToString($obdobi ->zacatek)) .
                '  )' .
                '  INNER JOIN RozdeleniOsob' .
                '   ON RozdeleniOsob.RC = PracovniPomer.RC' .
                '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND RozdeleniOsob.PlatnostOd <= ' . ($params ? $params ->addDateParameter($datumDo) : $database ->timestampToString($datumDo)) .
                '  AND (RozdeleniOsob.PlatnostDo >= ' . ($params ? $params ->addDateParameter($datumOd) : $database ->timestampToString($datumOd)) . ' OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')' .
                ' INNER JOIN Rozdeleni' .
                '  ON Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                ' AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni';

            if ($vcetnePodrizenych  &&  isset($kodRozdeleni)) {
                if ($vcetnePodrizenych == 1  ||  $vcetnePodrizenych == 2) {
                    // A) Jen vybrane rozdeleni vcetne podrizenych
                    $sql .=
                        ' FROM ( ' . $fromStr . ') ' .
                        ' INNER JOIN HierarchieRozdeleni ' .
                        '  ON HierarchieRozdeleni.TypPodrizene = RozdeleniOsob.KodTypuRozdeleni ' .
                        ' AND HierarchieRozdeleni.KodPodrizene = RozdeleniOsob.Rozdeleni ' .
                        "\nWHERE HierarchieRozdeleni.TypNadrazene = $typRozdeleni " .
                        '   AND HierarchieRozdeleni.KodNadrazene = ' .  ($params ? $params ->addParameter($kodRozdeleni) : "'" . $database->addSlashes($kodRozdeleni) . "' ") .
                        (($vcetnePodrizenych == 2)
                            ? ' AND HierarchieRozdeleni.KodPodrizene <> HierarchieRozdeleni.KodNadrazene' // jen podrizene
                            : ''
                        );
                } else if ($vcetnePodrizenych == 3) {
                    $sql .=
                        ' FROM ( ' . $fromStr . ') ' .
                        "\nWHERE Rozdeleni.RozdeleniRodice = $typRozdeleni" .
                        ' AND Rozdeleni.Rodic = ' . ($params ? $params ->addParameter($kodRozdeleni) : "'" . $database->addSlashes($kodRozdeleni) . "'");
                }
            } else {
                // B) Jen vybrane rozdeleni bez podrizenych, nebo konkretni PP
                $sql .=
                    ' FROM ' . $fromStr .
                    "\nWHERE RozdeleniOsob.KodTypuRozdeleni = $typRozdeleni";

                if (isset($kodRozdeleni)) {
                    $sql .= ' AND RozdeleniOsob.Rozdeleni = ' . ($params ? $params ->addParameter($kodRozdeleni) : "'" . $database->addSlashes($kodRozdeleni) . "' ");
                }
            }
        } else {
            // Vsechny osoby bez rozdeleni, rozdeleni se ani nezobrazuje
            $sql .=
                '  FROM ( Osoba INNER JOIN PracovniPomer' .
                '         ON Osoba.RC = PracovniPomer.RC' .
                '       )'.
                '       LEFT OUTER JOIN SchvaleniDochazky' .
                '       ON SchvaleniDochazky.RC = PracovniPomer.RC' .
                '       AND SchvaleniDochazky.OsobniCislo = PracovniPomer.OsobniCislo' .
                '       AND SchvaleniDochazky.DatumNastupu = PracovniPomer.DatumNastupu' .
                "       AND SchvaleniDochazky.Aplikace = 'ADS'".
                '       AND SchvaleniDochazky.PocatekObdobi = ' . ($params ? $params ->addDateParameter($obdobi ->zacatek) : $database ->dateToString($obdobi ->zacatek)) .
                "\nWHERE (1 = 1)\n";
        }

        $sql .=
            ' AND PracovniPomer.DatumNastupu <= ' . ($params ? $params ->addDateParameter($datumDo) : $database ->dateToString($datumDo)) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . ($params ? $params ->addDateParameter($datumOd) : $database ->dateToString($datumOd)) . '  OR  PracovniPomer.DatumUkonceni IS NULL)';

        if (!$nefiltrovatZamestnance) {
            $sql .=
                ' AND PracovniPomer.Zamestnanec > 0 ' .
                ' AND PracovniPomer.ZpracovatDochazku = 1 ';
        }

//        if ($filtr) {
//            $sql .= ' AND (' . $filtr . ')';
//        }

        return $sql;
    }


    public function loadRow($database, $row)
    {
        // POZOR! pro jistotu nastavuji i stara pole!
        $this->rc = $row[0];
        $this->osobniCislo = $row[1];
        $this->datumNastupu = $database ->stringToTimestamp($row[2]);
        $this->idPP = (int)$row[3];

        $this->jmeno = $row[4];
        $this->prijmeni = $row[5];
        $this->jmenoPrijmeni = $this->prijmeni . ' ' . $this->jmeno;
        if ($row[6]) {
            $this->jmenoPrijmeni = $row[6] . ' ' . $this->jmenoPrijmeni;
        }

        $this->kodRozdeleni = $row[7];
        $this->popisRozdeleni = $row[8];

        $this->datumUkonceni = $database ->stringToTimestamp($row[9]);
        $this->typPP = $row[10];

        $this->uzivatelska = [];
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelska[$i] = $row[10 + $i]; // 11 .. 30
        }

        $this->uzivatelskaPP = [];
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPP[$i] = $row[30 + $i]; // 31 .. 50
        }

        $this->schvaleni = new SchvaleniDochazky();
        $this->schvaleni ->schvaleni1 = $database ->stringToTimestamp($row[51]);
        $this->schvaleni ->schvaleni2 = $database ->stringToTimestamp($row[52]);
        $this->schvaleni ->schvaleni3 = $database ->stringToTimestamp($row[53]);
        $this->schvaleni ->zpracovanoVeMzdach = (int)$row[54];

        $this ->idFotografie = $row[55];
    }

    /**
     * Nacte seznam osob ze zadaneho sql dotazu
     * @param Database $database
     * @param string|SqlStatement $sql
    //     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     * @param ?callback $filterFunc   Funkce pro kontrolu prav - pokud je zadana, musi vracet bool.
     * @return OsobaDochazky[] pri volani metody z potomka vraci tridu potomka!
     * @throws \Ron\DB\DBException
     */
    public static function nactiSeznamOsobSQL($database, $sql, $filterFunc)
    {
        $result = [];
        $cursor = $database ->getSqlResult($sql);

        while ($row = $cursor ->fetch()) {
            $osoba = new static();
            $osoba ->loadRow($database, $row);

            if (!$filterFunc || $filterFunc($osoba)) {
                $result[] = $osoba;
            }
        }

        $cursor ->close();

        return $result;
    }

}
