<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\ComboBox;
use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;use Ron\DateUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Kalendar;
use RonPortal\RonPortalSettings;

class PuvodniDochazkaBrowser extends DochazkaBrowser
{
    const SETTINGS_ZOBRAZIT_TERMINAL = 'terminalvpuvodni';

    function __construct($database)
    {
        parent::__construct($database, 'dochazkazamestnance');

        $this->title = TextManager::_('Dochazka_zamestnance');

        $this->cssClass = 'browser grid browserdenni puvodnidochazka mobile-table-rows';
        $this->code = 'browserpuvodni';

        $this ->zobrazitFiltrPoznamky = true;
    }



    function inicializujData($datumOd, $datumDo)
    {
        if ($this ->filterForm ->getFilterValue('osoba')) {

            $this->kalendar = Kalendar::vratKalendarOsoby($this->database,
                $this->filterForm->getFilterValue('osoba', 'rc'),
                $this->filterForm->getFilterValue('osoba', 'osobniCislo'),
                $this->filterForm->getFilterValue('osoba', 'datumNastupu'),
                $this->filterForm->getFilterValue('obdobi'),
                $this->filterForm->getFilterValue('obdobi', 'datumDo')
            );
        }

        parent::inicializujData($datumOd, $datumDo);
    }


    function nactiData()
    {
        if ($this ->filterForm ->getFilterValue('osoba') === null) { // kvuli firebirdu
            return null;
        }

        $params = $this ->database ->createParameters();
        $filter = $this ->filterForm ->buildSqlFilter($params);

        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace, OperaceDochazky.PopisProDochazku,' .   // 0,1
            ' PuvodniDochazka.Poznamka, PuvodniDochazka.Terminal, PuvodniDochazka.KodOperace,' .    // 2..4
            ' PuvodniDochazka.PopisPolohy, PuvodniDochazka.ZemSirka, PuvodniDochazka.ZemDelka,' .   // 5..7
            ' Terminal.Popis, Rozdeleni.Kod, Rozdeleni.Popis' . // 8..10
            "\nFROM PuvodniDochazka INNER JOIN OperaceDochazky" .
            '  ON PuvodniDochazka.KodOperace = OperaceDochazky.Kod' .
            "\n LEFT OUTER JOIN Terminal" .
            '  ON Terminal.Kod = PuvodniDochazka.Terminal' .
            "\n LEFT OUTER JOIN Rozdeleni" .
            '  ON Rozdeleni.Kod = PuvodniDochazka.Rozdeleni  AND  Rozdeleni.KodTypuRozdeleni = PuvodniDochazka.KodTypuRozdeleni' .
            "\nWHERE " . ($filter ? $filter . ' AND ' : '') .
            ' PuvodniDochazka.DatumCasOperace >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND PuvodniDochazka.DatumCasOperace <= ' . $params ->addTimestampParameter(DateUtils::endOfTheDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'))).
            ' AND PuvodniDochazka.RC = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND PuvodniDochazka.OsobniCislo = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND PuvodniDochazka.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' ORDER BY PuvodniDochazka.DatumCasOperace';

        return $this ->database ->getSqlResult($sql, $params);
    }


    function vratTextKZaznamu($zaznam, $posledni)
    {
        $time = $this ->database ->stringToTimestamp($zaznam[0]);

        $title = '';
        $poznamka = '';
        $data =
            "data-timestamp='$time' data-operace='" . StringUtils::htmlTextConvert($zaznam[4]) . "' " .
            "data-infostr='" . StringUtils::format('datetime', $time) . ", " . StringUtils::htmlTextConvert($zaznam[1]) . "' " .
            "data-terminal='" . StringUtils::htmlTextConvert($zaznam[3]) . "'";

        $result =
            "<div $data class='zaznamPuvodni'>" .
            date('H:i', $time) . ' &nbsp; ' . StringUtils::htmlTextConvert($zaznam[1]);

        if (RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_TERMINAL) && $zaznam[3]) {
            $className = (RonPortalSettings::getIntValue('vkladani_zpusob_terminal') == 4) ? ' js' : '';
            $result .= " <div class='terminaldochazky$className' title='" . TextManager::_('Terminal') . ' ' . StringUtils::htmlTextConvert($zaznam[8]) . "'>(" . StringUtils::htmlTextConvert($zaznam[3]) . ')</div>';
        }

        $result .= "</div>";

        $zobrazitPoznamky = $this ->filterForm ->getFilterValue('zobrazitpoznamky');

        if ($zaznam[9]) {
            if ($zobrazitPoznamky) {
                $poznamka .=
                    '<div class="rozdelenidochazky">' .
                    StringUtils::htmlTextConvert($zaznam[10] . ' (' . $zaznam[9] . ')') .
                    '</div>';
            } else {
                $title .= $zaznam[10] . ' (' . $zaznam[9] . ")\n";
            }
        }

        if ($zaznam[5]  ||  ($zaznam[6]  &&  $zaznam[7])) {
            $dataPoloha = ($zaznam[6]  &&  $zaznam[7]) ? " data-latitude='{$zaznam[6]}' data-longitude='{$zaznam[7]}'" : '';

            if ($zobrazitPoznamky) {
                $poznamka .=
                    "<div class='polohadochazky mapurl' $dataPoloha>" .
                    StringUtils::htmlTextConvert($zaznam[5]) .
                    '<i class="inline-glyph fa glyph-gps"></i>' .
                    '</div>';
            } else {
                $title .= $zaznam[5] . "\n";
            }
        }

        if ($zaznam[2]) {
            if ($zobrazitPoznamky) {
                $poznamka .= '<div class="poznamka">' . StringUtils::htmlTextConvert($zaznam[2]) . '</div>';
            } else {
                $title .= $zaznam[2];
            }
        }

        if ($title) {
            return '<span title="' . StringUtils::htmlTextConvert($title) . '">'. $result . '</span>' . $poznamka;
        }

        return $result . $poznamka;
    }


    function render()
    {
        parent::render();

        if (RonPortalSettings::getIntValue('vkladani_zpusob_terminal') == 4) {  // povolen rucni vyber terminalu
            $comboTerminal = new ComboBox('terminal');
            $comboTerminal ->htmlParams = 'id="vyberTerminaluProVkladani"';
            $comboTerminal ->className .= ' ronform-field form-select';
            $comboTerminal ->loadData($this ->database, "SELECT Kod, Popis FROM Terminal WHERE Aktivni = 1 AND KodUrceni IN (1, 6, 7)", null);

            $dialog = new Dialog();

            $dialog
                ->setView('ronportal/dochazka/view_puvodni_terminal.php')
                ->setTitle('Vyber_terminalu')
                ->setId('vyberTerminaluForm')
                ->setHtmlParam('data-ronform-submit-url', '?menuid=dochazkazamestnance&action=nastavterminal')
                ->setHtmlParam('data-ronform-after-submit', 'reload')
                ->addClass('ronform')
                ->addButton(
                    (new DialogButton())
                        ->setValue('OK')
                        ->setId('vyberTerminaluOK')
                        ->addClass('submitbutton ronform-submit')
                )
                ->addButton(DialogButton::cancelButton('vyberTerminaluStorno'))
                ->addParameter('comboTerminal', $comboTerminal);

            $dialog ->render();
        }
    }


}
