<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\DBRecordField;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\System\AgentManager;

class SchvaleniDochazkyRecord extends RonPortalDBRecord
{
    /** @var PracovniPomer */
    var $pp;
    /** @var Obdobi */
    var $obdobi;

    /** @var DBRecordField */
    var $fZpracovano;
    /** @var DBRecordField */
    var $fZkontrolovano;
    /** @var DBRecordField */
    var $fSchvaleno;

    /** @var DBRecordField */
    var $fZpracovanoVeMzdach;

    /** @var DBRecordField */
    var $fPoznamka;

    var $datumZpracovaniDochazky;


    /**
     * @param \Ron\DB\Database $database
     * @param object|string $logDescription
     */
    function __construct($database, $logDescription = 'Schvalovani dochazky')
    {
        parent::__construct($database, 'SchvaleniDochazky', $logDescription);
    }


    /**
     * Nacte data pro zadanou osobu a obdobi
     * @param PracovniPomer $pp
     * @param Obdobi $obdobi
     * @return bool
     */
    function loadDataProOsobu($pp, $obdobi)
    {
        $this ->pp = $pp;
        $this ->obdobi = $obdobi;

        $this ->addField('RC', '', true, 15, $this ->pp ->rc);
        $this ->addField('OsobniCislo', '', true, 15, $this ->pp ->osobniCislo);
        $this ->addField('DatumNastupu', 'date', true, 0, $this ->pp ->datumNastupu);

        $this ->addField('', '', true, 0, '   ' . $this ->pp ->jmenoPrijmeni); // zalogovani

        $this ->addField('PocatekObdobi', 'date', true, 0, $this ->obdobi ->zacatek);
        $this ->addField('Aplikace', '', true, 15, 'ADS');

        // prozatim vse readonly
        $this ->fZpracovano = $this ->addField('ZpracovanoDo', 'date', true);
        $this ->fZkontrolovano = $this ->addField('ZkontrolovanoDo', 'date', true);
        $this ->fSchvaleno = $this ->addField('SchvalenoDo', 'date', true);

        $this ->fZpracovanoVeMzdach = $this ->addField('ZpracovanoVeMzdach', 'int', true);

        $this ->fPoznamka = $this ->addField('Poznamka', 'memo', true);

        $params = $this ->database ->createParameters();
        $this ->setWhere(
            " Aplikace = 'ADS' " .
            ' AND RC = ' . $params->addParameter($this ->pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->pp ->datumNastupu)) .
            ' AND PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi ->zacatek)),
            $params);

        return $this ->loadRecord();
    }


    function loadDataJS()
    {
        $pp = new PracovniPomer();
        if (!$pp ->nactiPPPodleId($this ->database, (int)$_POST['idpp'])) {
            throw new \Exception('Nelze nacist pracovni pomer!');
        }

        $pocatekobdobi = DateUtils::urlStringToDate($_POST['obdobi'], true);  //(int)$_POST['obdobi']; // datestringtotimestamp($_POST['obdobi']);
        if (!$pocatekobdobi) {
            throw new \Exception('Chyba - spatne zadane obdobi!');
        }

        $obdobi = new Obdobi();
        if ( !$obdobi ->nactiObdobi($this ->database, 'ADS', $pocatekobdobi) ) {
            throw new \Exception("Chyba - nelze nacist obdobi! ({$_POST['obdobi']})");
        }

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        return $this ->loadDataProOsobu($pp, $obdobi);
    }


    /**
     * Kontrola datumu zadaneho uzivatelem (format, rozsah) - vraci zadane datum nebo false v pripade chyby
     * @param string $postName
     * @param string $errorStr
     * @return int|null
     * @throws \Exception
     */
    function zkontrolujDatum($postName, $errorStr)
    {
        if ($_POST[$postName]) {
            $datum = DateUtils::dateStringToTimestamp($_POST[$postName]);
            if (!$datum) throw new \Exception($errorStr, null, 'schvaleni1');
        } else {
            $datum = null;
        }

        $maximalnidatum = $this ->obdobi ->konec; // jeste zpracovani!

        // kontrola, jestli je v obdobi
        if ($datum  &&  $datum < $this ->obdobi ->zacatek  ||  $datum > $maximalnidatum) {
            throw new RonException(
                $errorStr . ' ' .
                sprintf(TextManager::_('Datum_musi_byt_v_rozsahu'), date('j.n.Y', $this->obdobi->zacatek), date('j.n.Y', $maximalnidatum), $postName),
                $postName );
        }

        return $datum;
    }


    /**
     * @param string $errorMsgPrefix
     * @throws \Exception
     */
    function zkontrolujCasHierarchii($errorMsgPrefix = '')
    {
        if ( !RonPortalSettings::getIntValue('dodrzovatcashierarchiischvalovani') ) {
            return;
        }

        global $STR_ZpracovanoDo;   // kompat
        global $STR_ZkontrolovanoDo;// kompat
        global $STR_SchvalenoDo;    // kompat

        if ($this ->fZkontrolovano ->value > $this ->fZpracovano ->value) {
            throw new \Exception(sprintf($errorMsgPrefix . TextManager::_('Chyba_cashierarchie_schvalovani'),
                $STR_ZkontrolovanoDo ?: TextManager::_('Dochazka_schvaleni_2_do') ,
                $STR_ZpracovanoDo ?: TextManager::_('Dochazka_schvaleni_1_do')));
        }

        if ($this ->fSchvaleno ->value > $this ->fZkontrolovano ->value) {
            throw new \Exception(sprintf($errorMsgPrefix . TextManager::_('Chyba_cashierarchie_schvalovani'),
                $STR_SchvalenoDo ?: TextManager::_('Dochazka_schvaleni_3_do'),
                $STR_ZkontrolovanoDo ?: TextManager::_('Dochazka_schvaleni_2_do')));
        }

        return;
    }


    function loadPost()
    {
        $jepravo = false;

        $jezmena = false;

        $prava = Prava::vratPrava('zpracovani');
        if (isset($_POST['schvaleni1'])  &&  $prava ->lzeZobrazitPP($this ->pp ->idPP)) {  // pokud neni sloupec editovatelny, tak se hodnota vubec neodesila (proto isset)
            $jepravo = true;

            $datum = $this->zkontrolujDatum('schvaleni1', TextManager::_('Nespravne_datum_schvaleni1'));

            // kontrola, jestli protipravne nesnizil datum
            if ($prava->priznak && $datum < $this->fZpracovano->originalValue) {
                throw new RonException(TextManager::_('Nelze_snizit_schvaleni1'), 'schvaleni1');
            }

            if ($this->fZpracovano->value != $datum) {
                $this->fZpracovano->setNewValue($datum);
                $this->addField('Zpracoval', '', false, 25, RonPortalUtils::getLogUserName());
                $this->addField('DT_Zpracovani', 'datetime', false, 0, time());
                $jezmena = true;
            }
        }

        $prava = Prava::vratPrava('zkontrolovani');
        if (isset($_POST['schvaleni2'])  &&  $prava ->lzeZobrazitPP($this ->pp ->idPP)) {
            $jepravo = true;

            $datum = $this ->zkontrolujDatum('schvaleni2', TextManager::_('Nespravne_datum_schvaleni2'));
            if ($datum === false) return false;

            // kontrola, jestli protipravne nesnizil datum
            if ($prava ->priznak  &&  $datum < $this ->fZkontrolovano ->originalValue) {
                throw new RonException(TextManager::_('Nelze_snizit_schvaleni2'), 'schvaleni2');
            }

            if ($this ->fZkontrolovano ->value != $datum) {
                $this ->fZkontrolovano ->setNewValue($datum);
                $this ->addField('Zkontroloval', '', false, 25, RonPortalUtils::getLogUserName() );
                $this ->addField('DT_Zkontrolovani', 'datetime', false, 0, time());
                $jezmena = true;
            }
        }

        $prava = Prava::vratPrava('schvaleni');
        if (isset($_POST['schvaleni3'])  &&  $prava ->lzeZobrazitPP($this ->pp ->idPP)) {
            $jepravo = true;

            $datum = $this ->zkontrolujDatum('schvaleni3', TextManager::_('Nespravne_datum_schvaleni3'));
            if ($datum === false) return false;

            // kontrola, jestli protipravne nesnizil datum
            if ($prava ->priznak  &&  $datum < $this ->fSchvaleno ->originalValue) {
                throw new RonException(TextManager::_('Nelze_snizit_schvaleni3'), 'schvaleni3');
            }

            if ($this ->fSchvaleno ->value != $datum) {
                $this ->fSchvaleno ->setNewValue($datum);
                $this ->addField('Schvalil', '', false, 25, RonPortalUtils::getLogUserName() );
                $this ->addField('DT_Schvaleni', 'datetime', false, 0, time());
                $jezmena = true;
            }
        }

        if ( $jezmena ) {
            if ($this ->fZpracovanoVeMzdach ->value) throw new \Exception( TextManager::_('Nelze_zmenit_schvaleni_kvuli_mzdy') );

            $this -> zkontrolujCasHierarchii();
        }

        if ( $jepravo  &&  isset($_POST['poznamka']) ) $this ->fPoznamka ->setNewValue( StringUtils::fromUTF8($_POST['poznamka']) );

        return true;
    }


    /**
     * @param string|string[] $scriptName
     */
    function executeScripts($scriptName)
    {
        $parametrskriptu = "Aplikace=ADS;IdPP={$this->pp->idPP};obdobi=" . date('j.n.Y', $this->obdobi->zacatek);

        //if ($newparams) {
        // prida parametry s nove zadanymi hodnotami
        $parametrskriptu .=
            ';ZpracovanoDo=' . ($this ->fZpracovano ->value ? date('j.n.Y', $this ->fZpracovano ->value) : '') .
            ';ZkontrolovanoDo=' . ($this ->fZkontrolovano ->value ? date('j.n.Y', $this ->fZkontrolovano ->value) : '') .
            ';SchvalenoDo=' . ($this ->fSchvaleno ->value ? date('j.n.Y', $this ->fSchvaleno ->value) : '');
        //}
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_SCHVALOVANI, $scriptName, $parametrskriptu);
    }


    function zkontrolujZpracovaniDochazky()
    {
        if (!$this ->datumZpracovaniDochazky) {
            // nacte a zapamatuje si, aby to nemusel nacitat u kazde osoby
            $this ->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this->database);
        }

        if ($this ->fZpracovano ->value > $this ->datumZpracovaniDochazky  ||  $this ->fZkontrolovano ->value > $this ->datumZpracovaniDochazky  ||  $this ->fSchvaleno ->value > $this ->datumZpracovaniDochazky) {
            throw new \Exception( TextManager::_('Nelze_schvalit_nezpracovanou') );
        }
    }


    /**
     * Nacte vysledky kontrol - pokud nejsou v richtigu, tak vraci false
     * @param string $errorMsgPrefix
     * @throws \Exception
     */
    function zkontrolujChybyDochazky($errorMsgPrefix = '')
    {
        $filter = '';

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT VysledekKontrolyDochazky.Datum, TypKontrolyDochazky.Kod, TypKontrolyDochazky.Popis, TypKontrolyDochazky.Mesicni ' . // 0..3
            ' FROM VysledekKontrolyDochazky, TypKontrolyDochazky ' .
            ' WHERE VysledekKontrolyDochazky.TypKontroly = TypKontrolyDochazky.Kod ' .
            ' AND RC = ' . $params->addParameter($this ->pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($this ->pp->datumNastupu) .
            ' AND Datum >= ' . $params ->addDateParameter($this ->obdobi ->zacatek) .
            //' AND Datum <= ' . db_datetostring($hodnota) .
            ' AND Ignorovat = 0';

        if ($this ->fZpracovano ->value > $this ->fZpracovano ->originalValue) {
            // zvysuje se "zpracovanoDo"
            $filter .=
                '(TypKontrolyDochazky.BlokovatSchvaleni1 = 1 '.
                ' AND Datum <= ' . $params ->addDateParameter($this ->fZpracovano ->value) .
                ($this ->fZpracovano ->value < $this ->obdobi ->konec
                    ? ' AND TypKontrolyDochazky.Mesicni = 0 '
                    : '') .
                ')';
        }

        if ($this ->fZkontrolovano ->value > $this ->fZkontrolovano ->originalValue) {
            // zvysuje se "zkontrolovanoDo"
            $filter .=
                ($filter ? ' OR ' : '') .
                '(TypKontrolyDochazky.BlokovatSchvaleni2 = 1 '.
                ' AND Datum <= ' . $params ->addDateParameter($this ->fZkontrolovano ->value) .
                ($this ->fZkontrolovano ->value < $this ->obdobi ->konec
                    ? ' AND TypKontrolyDochazky.Mesicni = 0 '
                    : '') .
                ')';
        }

        if ($this ->fSchvaleno ->value > $this ->fSchvaleno ->originalValue) {
            // zvysuje se "schvalenoDo"
            $filter .=
                ($filter ? ' OR ' : '') .
                '(TypKontrolyDochazky.BlokovatSchvaleni3 = 1 '.
                ' AND Datum <= ' . $params ->addDateParameter($this ->fSchvaleno ->value) .
                ($this ->fSchvaleno ->value < $this ->obdobi ->konec
                    ? ' AND TypKontrolyDochazky.Mesicni = 0 '
                    : '') .
                ')';
        }

        if (!$filter) {
            // nezvysuje se zadna uroven - netreba kontrolovat
            return;
        }

        $sql .=
            ' AND (' . $filter . ')' .
            ' ORDER BY Datum';


        $cursor = $this ->database ->getSqlResult($sql, $params, $errorMsgPrefix . 'Chyba pri nacitani vysledku kontrol!');

        $result = true;
        $errorText = '';

        while ($row = $cursor ->fetch()) {
            // TODO: OPRAVIT (errormsg v javascriptech)

            $errorText .=
                ($row[3] ? '' : date('j.n.Y', $this ->database ->stringToTimestamp($row[0])) . ': ') .
                TextManager::_('db_typkontrolydochazky_popis_' . $row[1] ,$row[2]) .
                "\n";

            $result = false;
        }

        $cursor ->close();

        if (!$result) {
            throw new \Exception($errorMsgPrefix . TextManager::_('Nelze_schvalit_kontroly') . "\n" . $errorText); // prvni chyba - zobrazi hlasku "nelze schvalit
        }
    }


    /**
     * Pokud se ma pred schvalenim provest prepocet, tak jej spusti
     * @throws \Exception
     */
    function prepocetPredSchvalenim()
    {
        $parametry = RonPortalSettings::getValue('parametryagentapredschvalenim', '');
        if ($parametry == '') return;

        if ($this ->fZpracovanoVeMzdach ->value) {
            $datumDo = $this ->obdobi ->konec;
        } else {
            // nejvyssi datum schvaleni
            $datumDo = $this->fZpracovano->value;
            if (!$datumDo || $this->fZkontrolovano->value > $datumDo) $datumDo = $this->fZkontrolovano->value;
            if (!$datumDo || $this->fSchvaleno->value > $datumDo) $datumDo = $this->fSchvaleno->value;
        }

        $params = $parametry . // uloha musi byt zadana v parametrech
            ' -interval:' . date('Y-n-j', $this ->obdobi ->zacatek) . ';' . date('Y-n-j', $datumDo) .
            ' -IDPP:' . $this ->pp ->idPP;

        AgentManager::executeADS('ADS_Agent_Processing.exe', $params );
    }


    function zkontrolujAZapisSchvaleni($insert, $errorMsgPrefix = '')
    {
        $this ->zkontrolujZpracovaniDochazky();
        $this ->zkontrolujCasHierarchii($errorMsgPrefix);

        $this ->prepocetPredSchvalenim();

        $this ->zkontrolujChybyDochazky($errorMsgPrefix);

        if ( !$this ->executeXmlEvents(RonPortalXMLEvents::SCHVALENI_DOCHAZKY_BEFORE, '') ) {
            return false;
        }

        $scriptNames = [RonPortalScriptEvents::SCHVALENI_DOCHAZKY_BEFORE];
        if ($this ->fZpracovano ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_1_BEFORE;
        }
        if ($this ->fZkontrolovano ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_2_BEFORE;
        }
        if ($this ->fSchvaleno ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_3_BEFORE;
        }

        $this ->executeScripts($scriptNames);

        if ($insert) {
            $this ->insertRecord();
        } else {
            $this ->updateOrInsert();
        }

        $result = $this ->executeXmlEvents(RonPortalXMLEvents::SCHVALENI_DOCHAZKY_AFTER, '');

        $scriptNames = [RonPortalScriptEvents::SCHVALENI_DOCHAZKY_AFTER];
        if ($this ->fZpracovano ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_1_AFTER;
        }
        if ($this ->fZkontrolovano ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_2_AFTER;
        }
        if ($this ->fSchvaleno ->isChanged()) {
            $scriptNames[] = RonPortalScriptEvents::SCHVALENI_DOCHAZKY_3_AFTER;
        }

        $this ->executeScripts($scriptNames); // volat vzdy, i kdyz Xml vratilo false!

        return $result;
    }


    /**
     * Nastavi pole pro zadanou uroven schvalovani. V pripade chyby vraci false
     *
     * @param int $uroven
     * @param int $hodnota
     * @param string $errorMsgPrefix
     * @return bool
     * @throws \Exception
     */
    function nastavPoleSchvaleni($uroven, $hodnota, $errorMsgPrefix = '') {
        switch ($uroven) {
            case 1 :
                $prava = Prava::vratPrava('zpracovani');

                if (!$prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
                    throw new NeniPravoException();
                }

                if ($prava ->priznak  &&  $hodnota < $this ->fZpracovano ->originalValue) {
                    throw new \Exception( $errorMsgPrefix . (TextManager::_('Nelze_snizit_schvaleni1')) );
                }

                $this ->fZpracovano ->setNewValue($hodnota);

                $f = $this ->addField('Zpracoval', '', false, 25, RonPortalUtils::getLogUserName());
                $f ->updateLog = true;
                $f = $this ->addField('DT_Zpracovani', 'datetime', false, 0, time());
                $f ->updateLog = true;

                break;


            case 2 :
                $prava = Prava::vratPrava('zkontrolovani');

                if (!$prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
                    throw new NeniPravoException();
                }

                if ($prava ->priznak  &&  $hodnota < $this ->fZkontrolovano ->originalValue) {
                    throw new \Exception( $errorMsgPrefix . (TextManager::_('Nelze_snizit_schvaleni2')) );
                }

                $this ->fZkontrolovano ->setNewValue($hodnota);

                $f = $this ->addField('Zkontroloval', '', false, 25, RonPortalUtils::getLogUserName());
                $f ->updateLog = true;
                $f = $this ->addField('DT_Zkontrolovani', 'datetime', false, 0, time());
                $f ->updateLog = true;

                break;

            case 3 :
                $prava = Prava::vratPrava('schvaleni');

                if (!$prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
                    RonPortalUtils::neniPravoException();
                }

                if ($prava ->priznak  &&  $hodnota < $this ->fSchvaleno ->originalValue) {
                    throw new \Exception($errorMsgPrefix . (TextManager::_('Nelze_snizit_schvaleni3')) );
                }

                $this ->fSchvaleno ->setNewValue($hodnota);

                $f = $this ->addField('Schvalil', '', false, 25, RonPortalUtils::getLogUserName());
                $f ->updateLog = true;
                $f = $this ->addField('DT_Schvaleni', 'datetime', false, 0, time());
                $f ->updateLog = true;

                break;
        }

        return true;
    }


    /**
     * Nastavi zadanou uroven schvaleni na zadanou hodnotu
     *
     * @param PracovniPomer $pp
     * @param Obdobi $obdobi
     * @param int    $uroven
     * @param int    $hodnota
     * @return bool
     * @throws \Exception
     */
    function nastavSchvaleni($pp, $obdobi, $uroven, $hodnota)
    {
        $exists = $this ->loadDataProOsobu($pp, $obdobi);

        if ($this ->fZpracovanoVeMzdach ->value) {
            throw new \Exception( TextManager::_('Nelze_zmenit_schvaleni_kvuli_mzdy') );
        }

        if ( !$uroven ) {
            // nejvyssi povolena uroven uzivatele
            $prava = Prava::vratPrava('schvaleni');
            if ($prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) $uroven = 3;
            else {
                $prava = Prava::vratPrava('zkontrolovani');
                if ($prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) $uroven = 2;
                else {
                    $prava = Prava::vratPrava('zpracovani');
                    if ($prava ->lzeZobrazitPP($this ->pp ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
                        $uroven = 1;
                    } else {
                        throw new NeniPravoException();
                    }
                }
            }
        }


        if ( !$this ->nastavPoleSchvaleni($uroven, $hodnota) ) {
            return false;
        }

        return $this ->zkontrolujAZapisSchvaleni(!$exists);
    }

}
