<?php
/**
 * stanislav.elbl, 16.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use RonPortal\Editace\MesicniData;
use RonPortal\Editace\OsobaProEditaci;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Seznam osob pro schvalovani dochazky - konfigurace seznamu (zobrazene sloupce) se nacita z XML formulare
 */
class SchvalovaniXML extends MesicniData implements ViewObject
{
    /** @var OsobaProEditaci[] */
    protected $osoby; // seznam osob

    /** @var  SchvalovaniFilterForm */
    protected $filterForm;


    protected $dochazkaZpracovanaDo;

    public function __construct($database)
    {
        parent::__construct($database);

        $this ->vyberPPVObdobi = 0; // defaultne kdykoliv v prubehu obdobi
        $this -> filterForm = new SchvalovaniFilterForm($this ->database);
    }


    /**
     * Nacte obsah zadaneho souboru (z pole ObsahXML nebo souboru)
     * @param integer $xmlId
     * @throws \Exception
     */
    function loadXMLForm($xmlId)
    {
        $sql =
            'SELECT ' . $this ->database ->castWideMemo('SablonaXMLFormulare.ObsahXML') . ', SablonaXMLFormulare.SouborXML ' . // 0,1
            ' FROM SablonaXMLFormulare ' .
            ' WHERE Id = ' . (int)$xmlId;

        $cursor = $this ->database ->getSqlResult($sql);

        $row = $cursor ->fetch();
        if (!$row) {
            throw new \Exception("Chyba - formular s ID = $xmlId nenalezen!");
        }

        if ($row[0]) {
            $this ->loadXML($row[0]);
        } else if ($row[1]) {
            $this ->loadXML(file_get_contents($row[1]));
        } else {
            throw new \Exception('Chyba! XML formular s nastavenim je prazdny!');
        }
    }


    public function getPocatekObdobi()
    {
        return $this ->filterForm ->getFilterValue('obdobi');
    }



    function vratSQL()
    {
        $params = $this ->database ->createParameters();

//        if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)  &&  $this ->filterForm ->kodRozdeleni !== null ) {
//            //$sql = $this ->vratZakladniSQL(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), RonPortalSession::rozdeleni(), $filtr);
//            $sql = OsobaProEditaci::vratZakladniSQL(
//                $this ->database, $this ->obdobi, $this ->vyberPPVObdobi,
//                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this ->filterForm ->kodRozdeleni,
//                isset($this ->vcetnePodrizenych) ? $this ->vcetnePodrizenych : $this ->filterForm ->vcetnePodrizenychRozdeleni, false, $params);
//        } else {
//            $sql = OsobaProEditaci::vratZakladniSQL($this ->database, $this ->obdobi, $this ->vyberPPVObdobi, null, null, false, false, $params);
//        }

        $sql = OsobaProEditaci::vratZakladniSQL($this ->database, $this ->obdobi, $this ->vyberPPVObdobi, null, null, false, false, $params);

        $filtr = $this ->filterForm ->buildSqlFilter($params);

        // prida filtry podle rozdeleni
        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $rozdeleniSQL = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PracovniPomer', $params);
                if ($rozdeleniSQL) {
                    $filtr .=
                        ($filtr ? ' AND ' : '') .
                        '(' . $rozdeleniSQL . ')';
                }
            }
        }

        $filtrPrava = Prava::sqlFiltrPodlePrav(
            [Prava::vratPrava('zpracovani'), Prava::vratPrava('zkontrolovani'), Prava::vratPrava('schvaleni')],
            'PracovniPomer',
            $this ->filterForm ->getFilterValue('obdobi'), $this ->filterForm ->getFilterValue('obdobi', 'datumDo'),
            false,
            $params);

        if ($filtrPrava) {
            $filtr .=
                ($filtr ? ' AND ' : '') .
                '(' . $filtrPrava . ')';
        }

        if ($filtr) {
            $sql .= ' AND (' . $filtr . ')';
        }

        // order by
        $sql .=
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno';

        return new SqlStatement($sql, $params);
    }


    function nactiSeznamOsobSQL($sql) // , $kontrolovatprava)
    {
        $this ->osoby = OsobaProEditaci::nactiSeznamOsobSQL($this ->database, $sql,
            function($osoba) {
                return SchvaleniDochazky::lzeSchvalovatOsobu($osoba ->idPP,
                    $this ->filterForm ->getFilterValue('obdobi'),
                    $this ->filterForm ->getFilterValue('obdobi', 'datumDo'));
            }
        );

        foreach ($this ->osoby as $osoba) {
            $this ->nastavPravaNaOsobu($osoba);
            $this ->nactiDataOsoby($osoba);
        }

    }


    function nactiSeznamOsob($obdobi)
    {
        $this ->nastavObdobi($obdobi);
        $this ->nastavPrava(false); // povolitvse = false

        $sql = $this ->vratSQL();

        $this ->nactiSeznamOsobSQL($sql); // , false      // kontrolovatprava = false - zobrazi vsechny nactene osoby
    }


    /**
     * @param int $indexSloupce
     * @param OsobaProEditaci $osoba
     * @param int|null $hodnota
     * @param int $uroven
     * @param PravaNaOsoby $prava
     * @return string
     */
    function vratHTMLSchvaleni($osoba, $indexSloupce, $hodnota, $uroven, $prava)
    {
        $hodnotaStr = $hodnota ? date('j.n.Y', $hodnota) : '';
        $maxDate = $this ->obdobi ->konec < $this ->dochazkaZpracovanaDo
            ? $this ->obdobi ->konec
            : $this ->dochazkaZpracovanaDo;

        switch ( $this ->polozky[$indexSloupce] ->lzeEditovat ) {
            case 1 :
                // edit
                if ($prava ->lzeZobrazitPP($osoba ->idPP)) {
                    return
                        "<input type='hidden' name='originalschvaleni_{$osoba->idPP}_$uroven' value='$hodnotaStr'/>" .
                        // readonly verze
                        "<span class='hromadne_readonly textschvaleni{$uroven}'>" .
                        $hodnotaStr .
                        "</span>" .
                        // editovatelna verze
                        '<span class="no-wrap">' .
                        "<input class='hromadne_edit inputDate' maxlength='10' size='10'" .
                            " name='schvaleni_{$osoba->idPP}_$uroven' id='schvaleni_{$osoba->idPP}_$uroven' value='$hodnotaStr'" .
                            ' data-roncalendar-mindate="' . date('j.n.Y', $this ->obdobi ->zacatek) . '" ' .
                            ' data-roncalendar-maxdate="' . date('j.n.Y', $maxDate) . '" ' .
                            "/>" .
                        '<i class="hromadne_edit showcalendar inline-glyph fa glyph-calendar glyph-button padding-all"  data-roncalendar-inputname="schvaleni_' . $osoba->idPP . '_' . $uroven . '"></i>' .
                        '</span>';
                } else {
                    return
                        "<span class='textschvaleni{$uroven}'>" .
                        $hodnotaStr .
                        "</span>";
                }


            case 2 :
                // checkbox
                $checked = $hodnota == $this ->obdobi ->konec;
                $checkedstr = $checked ? 'checked="checked"' : '';
                $hintstr = ($hodnota  &&  $hodnota < $this ->obdobi ->konec) ? "title='" . \Ron\ronText('Do') . ' ' . date('j.n.Y', $hodnota) . "'" : '';

                if ( !$prava ->lzeZobrazitPP($osoba ->idPP)) {
                    return
                        "<div $hintstr>" .
                        "<span class='hromadne_norights'>" . (\Ron\ronText($checked ? 'Str_Yes' : 'Str_No')) . "</span>" .
                        "</div>";
                } else {
                    return
                        "<div $hintstr>" .
                        "<input type='hidden' name='originalschvaleni_{$osoba->idPP}_$uroven' value='$hodnotaStr'/>" .
                        "<span class='hromadne_readonly" . ($checked ? ' textschvaleni' . $uroven : '') . "'>" . (\Ron\ronText($checked ? 'Str_Yes' : 'Str_No')) . "</span>" .
                        "<input class='hromadne_edit' type='checkbox' name='schvaleni_{$osoba->idPP}_$uroven' id='schvaleni_{$osoba->idPP}_$uroven'  $checkedstr/>" .
                        "</div>";
                }
        }

        return $this->vratHTMLText($osoba, $indexSloupce);
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param int $index Index radku
     * @param bool $zobrazitOsobniUdaje
     * @return string
     */
    function vratRadekOsoby($osoba, $index, $zobrazitOsobniUdaje)
    {
        $href = '?menuid=schvalovani&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi')) . "&record={$osoba ->idPP}";

        $result =
            '<tr class="schvalovani_row"' .
            ' data-idpp="' . (int)$osoba ->idPP . '"' .
            ' data-osobni-cislo="' . StringUtils::htmlTextConvert($osoba ->osobniCislo) . '"' .
            ' data-jmeno="' . StringUtils::htmlTextConvert($osoba ->jmenoPrijmeni) . '"' .
            ' data-schvaleni1="' . ($osoba ->schvaleni -> schvaleni1 ? StringUtils::format('date', $osoba ->schvaleni -> schvaleni1) : '') . '"' .
            ' data-schvaleni2="' . ($osoba ->schvaleni -> schvaleni2 ? StringUtils::format('date', $osoba ->schvaleni -> schvaleni2) : '') . '"' .
            ' data-schvaleni3="' . ($osoba ->schvaleni -> schvaleni3 ? StringUtils::format('date', $osoba ->schvaleni -> schvaleni3) : '') . '"' .
            ' data-prava1="' . (Prava::lzeZobrazitPP('zpracovani', $osoba ->idPP) ? '1' : '0') . '"' .
            ' data-prava2="' . (Prava::lzeZobrazitPP('zkontrolovani', $osoba ->idPP) ? '1' : '0') . '"' .
            ' data-prava3="' . (Prava::lzeZobrazitPP('schvaleni', $osoba ->idPP) ? '1' : '0') . '"' .
            '>';

        if ($zobrazitOsobniUdaje) {
            $result .=
                '<td class="align-left no-wrap">' .
                "<a href='$href'>" .
                StringUtils::xmlTextConvert($osoba->osobniCislo) .
                "</a>" .
                "<input type='hidden' name='IdPP_$index' value='{$osoba ->idPP}' />" .
                "</td>" .
                '<td class="align-left no-wrap">' .
                "<a href='$href'>" . StringUtils::xmlTextConvert($osoba->jmenoPrijmeni) . '</a>' .
                '</td>';
        }

        // hodnoty sloupcu
        foreach ($this ->polozky as $i => $sloupec) {
            $td_styl = $sloupec ->HTMLStyl();
            if ($td_styl) {
                $td_styl = "style='$td_styl'";
            }

            $hintStr = ($osoba ->f_data[$i] ->f_hint)
                ? "title='" . StringUtils::htmlTextConvert($osoba ->f_data[$i] ->f_hint) . "'"
                : '';

            $classStr = $sloupec ->cssClass;
            if ($sloupec ->isTimeFormat() || $sloupec ->isNumberFormat()) {
                if ($osoba ->f_data[$i] ->f_hodnota < 0) {
                    $classStr .= ' minus';
                }
                $sloupec ->soucet += $osoba ->f_data[$i] ->f_hodnota;
            }
            if ($classStr) {
                $classStr = "class='$classStr'";
            }


            $result .= "<td $classStr $td_styl $hintStr>";
            switch ($sloupec ->typ) {
                case 'SCHVAL1' :
                    $result .= $this->vratHTMLSchvaleni($osoba, $i, $osoba ->schvaleni ->schvaleni1, 1, Prava::vratPrava('zpracovani'));
                    break;

                case 'SCHVAL2' :
                    $result .= $this->vratHTMLSchvaleni($osoba, $i, $osoba ->schvaleni ->schvaleni2, 2, Prava::vratPrava('zkontrolovani'));
                    break;

                case 'SCHVAL3' :
                    $result .= $this->vratHTMLSchvaleni($osoba, $i, $osoba ->schvaleni ->schvaleni3, 3, Prava::vratPrava('schvaleni'));
                    break;

                default :
                    $result .= $this->vratHTMLText($osoba, $i);
            }

            $result .= '</td>';
        }

        $result .= "</tr>\n";

        return $result;
    }


    function vratRadekSoucet()
    {
        // hromadny radek - jen v editacnim rezimu
        $result =
            '<tr class="browserfooter hromadne_edit">' .
            '<td></td><td>' . \Ron\ronText('Schvalit_hromadne') . '</td>';

        $vynechatSloupce = 0;
        //pokud je jmeno a osobni cislo zobrazeno jako sloupec, je nutne skryt (nezobrazit] 2 (viz napis schvalit hromadne) sloupce, aby sedela zatrzitka
        if ($this ->jeSloupecOsoba()) {
            $vynechatSloupce = 2;
        }

        foreach ($this ->polozky as $i => $sloupec) {
            if ($vynechatSloupce > 0 && !($sloupec->typ == 'SCHVAL1' || $sloupec->typ == 'SCHVAL2' ||$sloupec->typ == 'SCHVAL3')) {
                $vynechatSloupce--;
            } else {
                $td_styl = $sloupec->HTMLStyl();
                if ($td_styl) $td_styl = "style='$td_styl'";

                $classstr = $sloupec->cssClass;
                if ($classstr) $classstr = "class='$classstr'";

                $result .= "<td $td_styl $classstr>";

                $uroven = 0;
                $prava = null;

                switch ($sloupec->typ) {
                    case 'SCHVAL1' :
                        $uroven = 1;
                        $prava = Prava::vratPrava('zpracovani');
                        break;

                    case 'SCHVAL2' :
                        $uroven = 2;
                        $prava = Prava::vratPrava('zkontrolovani');
                        break;

                    case 'SCHVAL3' :
                        $uroven = 3;
                        $prava = Prava::vratPrava('schvaleni');
                        break;
                }

                if ($sloupec->lzeEditovat) {
                    switch ($sloupec->lzeEditovat) {
                        case 0 :
                            break;

                        case 1 :
                            break;

                        case 2 :
                            // checkbox
                            if ($prava &&  $prava->jsouNejakePrava()) {
                                $result .= "<input type='checkbox' data-hromadne-schvalit-uroven='$uroven' />";
                            }
                            break;
                    }
                }

                $result .= '</td>';
            }
        }

        $result .=
            "</tr>\n";

        return $result;
    }


    function jeSloupecOsoba()
    {
        foreach ($this ->polozky as $sloupec) {
            $typ = strtoupper($sloupec ->typ);
            if ($typ == 'JMENO'  ||  $typ == 'PRIJMENI'  ||  $typ == 'JMENOPRIJMENI') {
                return true;
            }
        }

        return false;
    }

    /**
     * @return string  Vraci HTML text pro zobrazeni formulare
     */
    function vratObsahFormulareH()
    {
        $jeSloupecOsoba = $this ->jeSloupecOsoba();

        $result = '<div id="schvalovanidochazky">';

        //$hlp_cnt = count($this ->f_osoby);
        //$result .= "<INPUT type='hidden' name='{$this ->f_prefixHTML}POCET_RADKU' value='$hlp_cnt' />\n";

        //$hlp_cnt = count($this ->f_polozky);
        //$result .= "<INPUT type='hidden' name='{$this ->f_prefixHTML}POCET_SLOUPCU' value='$hlp_cnt' />\n";

        // tlacitko, ktere zajisti prepnuti seznamu do editovatelneho rezimu
        $result .=
            "<div >" .
            "<input type='button' id='schvalitDochazkuHromadne' class='button hromadne_readonly toggleClass' data-selector='#schvalovanidochazky' data-add-class='hromadne' value='" . \Ron\ronText('Schvalit_hromadne') . "' />" .
            "</div>\n";

        $obdobiStr = DateUtils::dateToUrlString($this ->obdobi ->zacatek);
        $konecObdobiStr = DateUtils::dateToUrlString($this ->obdobi ->konec);
        $editace1 = $editace2 = $editace3 = 0;


        foreach ($this ->polozky as $polozka) {
            if ( $polozka ->typ == 'SCHVAL1'  &&  $polozka ->lzeEditovat) {
                $editace1 = $polozka ->lzeEditovat;
            } else if ( $polozka ->typ == 'SCHVAL2' &&  $polozka ->lzeEditovat) {
                $editace2 = $polozka ->lzeEditovat;
            } else if ( $polozka ->typ == 'SCHVAL3' &&  $polozka ->lzeEditovat) {
                $editace3 = $polozka ->lzeEditovat;
            }
        }

        $result .=
            "<table class='browser middle rowOver oddEvenRows' style='{$this ->cssStyl}'" .
            " id='seznam_schvalovani_dochazky'" .
            " data-obdobi='$obdobiStr'" .
            " data-konec-obdobi='$konecObdobiStr'" .
            " data-editace1='$editace1'" .
            " data-editace2='$editace2'" .
            " data-editace3='$editace3'" .
            ">\n";

        // prvni 2 sloupce pro jmeno a osobni cislo
        $result .=
            '<thead class="floating-table-header"><tr>' .
            ($jeSloupecOsoba
                ? ''
                : "<th class='align-left'><strong>" . \Ron\ronText('Osobni_cislo') . "</strong></th><th class='align-left'><strong>" . \Ron\ronText('Jmeno') . "</strong></th>");

        // hlavicka - nadpisy sloupcu
        foreach ($this ->polozky as $i => $sloupec) {
            $attrs = '';
            if ($sloupec ->title) {
                $attrs .= ' title="' . StringUtils::htmlTextConvert($sloupec ->title) . '"';
            }
            $result .= "<th$attrs>" . $sloupec ->nadpis . "</th>\n";
        }

        // konec hlavicky
        $result .=
            "</tr></thead>\n";

        // Data osob
        if ($this ->osoby) {
            foreach ($this->osoby as $i => $osoba) {
                $result .= $this->vratRadekOsoby($osoba, $i + 1, !$jeSloupecOsoba); // v HTML indexovano od 1
            }
        }

        $result .= $this ->vratRadekSoucet();

        $result .= "</table>\n";

        $result .=
            "<div class='buttons'>\n" .
            "<input type='button' id='provestHromadneSchvaleniDochazky' class='button hromadne_edit' value='" . \Ron\ronText('Provest') . "' />\n" .
            "<input type='button' id='stornoHromadneSchvaleniDochazky' class='button hromadne_edit clickHref' data-ron-href='?menuid=schvalovani' value='" . \Ron\ronText('Cancel') . "' />\n" .
            "</div>\n";

        $result .= "</div>\n";

        return $result;
    }


//    /*
//     * Vypise javascriptovy objekt s daty
//     */
//    function schvalovaniScript()
//    {
//        $data = [
//            'pocetOsob' => count($this ->osoby),
//            'obdobi' => date('j.n.Y', $this ->obdobi ->zacatek),
//            'konecObdobi' => date('j.n.Y', $this ->obdobi ->konec),
//        ];
//
//        foreach ($this ->polozky as $polozka) {
//            if ( $polozka ->typ == 'SCHVAL1'  &&  $polozka ->lzeEditovat) {
//                $data['editace1'] = $polozka ->lzeEditovat;
//            } else if ( $polozka ->typ == 'SCHVAL2' &&  $polozka ->lzeEditovat) {
//                $data['editace2'] = $polozka ->lzeEditovat;
//            } else if ( $polozka ->typ == 'SCHVAL3' &&  $polozka ->lzeEditovat) {
//                $data['editace3'] = $polozka ->lzeEditovat;
//            }
//        }
//
//        $osoby = [];
//
//        $prava1 = Prava::vratPrava('zpracovani');
//        $prava2 = Prava::vratPrava('zkontrolovani');
//        $prava3 = Prava::vratPrava('schvaleni');
//
//        foreach ($this ->osoby as $index => $osoba) {
//            // objekt pro osobu
//            $osoba_info = [
//                'idPP' => (int)$osoba ->idPP,
//                'jmeno' => StringUtils::jsonText($osoba ->jmenoPrijmeni, false),
//                'osobniCislo' => StringUtils::jsonText($osoba ->osobniCislo),
//                'schvaleni1' => $osoba ->schvaleni -> schvaleni1 ? date('j.n.Y', $osoba ->schvaleni -> schvaleni1) : null,
//                'prava1' => $prava1 ->lzeZobrazitPP($osoba ->idPP) ? 1 : 0,
//                'schvaleni2' => $osoba ->schvaleni -> schvaleni2 ? date('j.n.Y', $osoba ->schvaleni -> schvaleni2) : null,
//                'prava2' => $prava2 ->lzeZobrazitPP($osoba ->idPP) ? 1 : 0,
//                'schvaleni3' => $osoba ->schvaleni -> schvaleni3 ? date('j.n.Y', $osoba ->schvaleni -> schvaleni3) : null,
//                'prava3' => $prava3 ->lzeZobrazitPP($osoba ->idPP) ? 1 : 0
//            ];
//            $osoby[] = $osoba_info;
//        }
//
//        $data['osoby'] = $osoby;
//
//        echo '<script' . ' type="text/javascript">ronPortal.dochazka.hromadneSchvalovani(' . json_encode($data) . ')</script>';
//    }



    function load($xmlId)
    {
        $this ->loadXMLForm($xmlId);
        if (isset($this ->vcetnePodrizenych)) {
            // V konfiguraci je natvrdo nastaveno, jestli se maji zobrazovat i podrizene => nebude zobrazovat vyber u rozdeleni1
            //$this ->filterForm ->showVcetnePodrizenychRozdeleni = false;
            $filter = $this ->filterForm ->getRozdeleniFilters()['rozdeleni'] ?? null;
            if ($filter) {
                $filter ->setShowVcetnePodrizenych(false);
            }
        }

        $this->filterForm ->loadData();

        $this ->nactiSeznamOsob($this ->filterForm ->getFilterValue('obdobi'));

        $this ->dochazkaZpracovanaDo = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);
    }


    function render()
    {
        echo '<h1>' . \Ron\ronText('Schvalovani') . "</h1>\n";
        $this ->filterForm ->render();

        MessageManager::getInstance() ->showMessages();

        if (!count($this ->osoby)) {
            echo "<br/>";

            echo "<div class='errortext'>" . ( StringUtils::htmlTextConvert(\Ron\ronText('Zadna_data')) ) . "</div>\n";
        } else {
            echo $this->vratObsahFormulareH();

//            $this->schvalovaniScript(); // data pro javascript
        }
    }

}
