<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ronportal\Controls\FilterFormFilterVariabilni;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;


class VariantniVysledkyBrowser1 extends VysledkyBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'variantnivysledky');

        $this ->code = 'browservariabilni';
        $this->title = \Ron\ronText('Prehled_variantnich_vysledku');

        $this->sqlFromPart = 'VariantniVysledek, TypCasoveSlozkyMzdy';
        $this->sqlWherePart = 'TypCasoveSlozkyMzdy.Kod = VariantniVysledek.TypCasoveSlozkyMzdy';

        $this->addField()
            -> setSqlFieldName('VariantniVysledek.TypCasoveSlozkyMzdy')
            -> setCode('col-kod')
            -> setHeaderText('Kod_casove_slozky') -> setOrdering(true);

        $this->addField()
            -> setSqlFieldName('TypCasoveSlozkyMzdy.Popis')
            -> setCode('col-popis')
            -> setHeaderText('Casova_slozka') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.Hodiny')
            ->setCode('col-hodiny')
            ->setHeaderText('Vysledek_Hodiny') ->setFormat('hodiny') ->setAlign('right') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.Dny')
            ->setCode('col-dny')
            ->setHeaderText('Vysledek_Pracovni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('VariantniVysledek.KalendarniDny')
            ->setCode('col-kalendarni-dny')
            ->setHeaderText('Vysledek_Kalendarni_dny') ->setFormat('number_2') ->setAlign('right') ->setOrdering(true);

        $this->defaultSort = 'col-kod';
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);
        $this ->filterForm ->addFilter(
            new FilterFormFilterVariabilni($this ->filterForm),
            true
        );
        //$this->filterForm->nastavVariantni(true);

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava->zobrazitVyberRozdeleni()) {
            $this->filterForm->addFilterRozdeleni1(null, $prava);
            $this->filterForm->addFilterRozdeleni2(null, $prava);
        }

//        $this->filterForm->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
//        $this->filterForm->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';

        ($this ->filterForm ->addFilterPP(false, $prava))
            ->setOptionsSqlFilter('PracovniPomer.ZpracovatDochazku = 1');


        $this->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->browserUrl =
            '?menuid=variantnivysledky&idpp=' . $this ->filterForm ->getFilterValue('osoba') .
            '&variantni=' . DateUtils::dateToUrlString($this ->filterForm ->getOmezitObdobiOd());
    }


    function renderFilter()
    {
        $this->odkazyVariabilniVysledky('variantnivysledky');
        parent::renderFilter();
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        return new SqlStatement(
            $result .
            ($result ? ' AND ' : '') .
            'VariantniVysledek.VariantniObdobi = ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) .
            ' AND VariantniVysledek.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND VariantniVysledek.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND VariantniVysledek.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }
}
