<?php
/**
 * stanislav.elbl, 23.10.2014
 * Formular pro hromadne schvaleni dochazky - zobrazuje:
 *   vybrane rozdeleni
 *   pro kazdou uroven schvaleni (na kterou ma uzivatel nejake pravo) zobrazuje checkbox a hodnotu
 *   + totez pro poznamku
 * Formular se odesle a schvalovani se provadi hromadne v jednom requestu
 * Pouziva se v pripade, ze neni nastaveno zobrazeni seznamu schvalovani pomoci XML
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\RonPortalSettings;

/** @var $connection resource */
/** @var $this HromadneSchvaleniFormular */

$datumPredplneni = $this -> datumZpracovaniDochazky;

if ( $datumPredplneni < $this -> obdobi -> zacatek ) {
    $datumPredplneni = 0;
} else if ( $datumPredplneni > $this -> obdobi -> konec ) {
    $datumPredplneni = $this -> obdobi -> konec;
}

?>
<h1><?= TextManager::_('Schvalovani') ?></h1>

<?php MessageManager::getInstance() ->showMessages(); ?>

<form name="schvalovaniform" id="hromadneschvalenidochazky" method="post" action="?menuid=schvalovani">
    <input type="hidden" name="schvaleniaction" value="hromadneschvaleni" />
    <input type="hidden" name="obdobi" value="<?= DateUtils::dateToUrlString($this -> obdobi -> zacatek) ?>" />
    <input type="hidden" name="rozdeleni" value="<?= StringUtils::htmlTextConvert($this -> kodRozdeleni) ?>" />
    <input type="hidden" name="rozdeleni2" value="<?= StringUtils::htmlTextConvert($this -> kodRozdeleni2) ?>" />

    <?= Csrf::csrfHiddenInput('schvaleni_doch_' . $this ->obdobi ->zacatek) ?>

    <div class='form form-30-70' id="hromadneschvalenitable">
        <div class="form-content">
            <div class="form-row" id="hromadneschvaleni_obdobi">
                <div class="form-col form-col1 caption"><?= TextManager::_('Obdobi') ?></div>
                <div class="form-col form-col2 font-weight-bold"><?= StringUtils::htmlTextConvert($this -> obdobi -> popis) ?></div>
            </div>

            <?php if ($this -> kodRozdeleni) { ?>
                <div class="form-row" id="hromadneschvaleni_rozdeleni">
                    <div class="form-col form-col1 caption"><?= RonPortalSettings::getValue(RonPortalSettings::ROZDELENI_NADPIS) ?></div>
                    <div class="form-col form-col2">
                        <div class="row">
                            <div class="col-8 font-weight-bold">
                                <?= $this ->rozdeleni ? StringUtils::htmlTextConvert($this ->rozdeleni ->popis . ' (' . $this ->rozdeleni ->kod . ')') : '' ?>
                            </div>
                            <div class="col-4">
                                <select name="rozdeleni_vcetne_podrizenych" class="form-control">
                                    <option value="0" <?= !$this ->rozdeleniVcetnePodrizenych ? " selected='selected'" : '' ?>><?= TextManager::_('Rozdeleni_jen_vybrane') ?></option>
                                    <option value="1" <?= $this ->rozdeleniVcetnePodrizenych == 1 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_vcetne_podrizenych') ?></option>
                                    <option value="2" <?= $this ->rozdeleniVcetnePodrizenych == 2 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_jen_podrizena') ?></option>
                                    <option value="3" <?= $this ->rozdeleniVcetnePodrizenych == 3 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_jen_primo_podrizena') ?></option>
                                </select>
                            </div>
                        </div>
                    </div>
                </div>
            <?php } ?>

            <?php if ($this -> kodRozdeleni2) { ?>
                <div class="form-row" id="hromadneschvaleni_rozdeleni2">
                    <div class="form-col form-col1 caption"><?= RonPortalSettings::getValue(RonPortalSettings::ROZDELENI2_NADPIS) ?></div>
                    <div class="form-col form-col2">
                        <div class="row">
                            <div class="col-8 font-weight-bold">
                                <?= $this ->rozdeleni2 ? StringUtils::htmlTextConvert($this ->rozdeleni2 ->popis . ' (' . $this ->rozdeleni2 ->kod . ')') : '' ?>
                            </div>
                            <div class="col-4">
                                <select name="rozdeleni2_vcetne_podrizenych" class="form-control">
                                    <option value="0" <?= !$this ->rozdeleni2VcetnePodrizenych ? " selected='selected'" : '' ?>><?= TextManager::_('Rozdeleni_jen_vybrane') ?></option>
                                    <option value="1" <?= $this ->rozdeleni2VcetnePodrizenych == 1 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_vcetne_podrizenych') ?></option>
                                    <option value="2" <?= $this ->rozdeleni2VcetnePodrizenych == 2 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_jen_podrizena') ?></option>
                                    <option value="3" <?= $this ->rozdeleni2VcetnePodrizenych == 3 ? " selected='selected'" : ''?>><?= TextManager::_('Rozdeleni_jen_primo_podrizena') ?></option>
                                </select>
                            </div>
                        </div>
                    </div>
                </div>
            <?php } ?>

            <div class="form-row" id="hromadneschvaleni_zpracovani">
                <div class="form-col form-col1"></div>
                <div class="form-col form-col2"><?= TextManager::_('Datum_posledniho_zpracovani') ?> <?= date('j.n.Y', $this -> datumZpracovaniDochazky) ?></div>
            </div>

            <div class="form-gap"></div>

        <?php

            // Zpracovano do
            $this -> vlozPolickoSDatem(TextManager::_('Dochazka_schvaleni_1_do'), 'formtablecaption textschvaleni1', 'zpracovanodo',
                $this -> zpracovanoDo, $this -> zpracoval,
                $this -> editovatZpracovano, $datumPredplneni);

            // Zkontrolovano do
            $this -> vlozPolickoSDatem(TextManager::_('Dochazka_schvaleni_2_do'), 'formtablecaption textschvaleni2', 'zkontrolovanodo',
                $this -> zkontrolovanoDo, $this -> zkontroloval,
                $this -> editovatZkontrolovano, $datumPredplneni);

            // Schvaleno do
            $this -> vlozPolickoSDatem(TextManager::_('Dochazka_schvaleni_3_do'), 'formtablecaption textschvaleni3', 'schvalenodo',
                $this -> schvalenoDo, $this -> schvalil,
                $this -> editovatSchvaleno, $datumPredplneni);

            // Poznamka
            $checkedstr = ( isset($_POST['poznamka_check'])  &&  $_POST['poznamka_check'] ) ? "checked='checked'" : '';
        ?>

            <div class="form-row" id="hromadneschvaleni_poznamka">
                <div class="form-col form-col1 caption">
                    <label>
                        <?= TextManager::_('Poznamka') ?>
                        <input type="checkbox" id="hromadneschvaleni_poznamkacheck" name="poznamka_check" <?= $checkedstr ?> />
                    </label>
                </div>
                <div class="form-col form-col2">
                    <textarea name="poznamka" class="form-input" cols="80" rows="5" id="hromadneschvaleni_poznamkaedit"><?= StringUtils::htmlTextConvert(isset($_POST['poznamka']) ? $_POST['poznamka'] : $this -> poznamka) ?></textarea>
                </div>
            </div>
        </div>

        <div class="buttons">
            <div class="form-row" id="hromadneschvaleni_buttons">
                <div class="form-col form-col1"></div>
                <div class="form-col form-col2">
                    <input type="submit" class="submitbutton" value="<?= TextManager::_('Provest') ?>"/>
                </div>
            </div>
            <div class="form-row">
                <div class="form-col form-col1-2">
                    <a href="?menuid=schvalovani"><?= TextManager::_('Zpet') ?></a>
                </div>
            </div>
        </div>
    </div>
</form>
