<?php
/**
 * stanislav.elbl, 22.02.2021
 */

namespace RonPortal\Dokumenty;

use Ron\DB\Database;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;

/**
 * OutputObject pro stahnuti dokumentu z databaze
 * Class DokumentOutputObject
 */
class DokumentOutputObject implements OutputObject
{
    /** @var  Database */
    protected $database;
    /** @var  int */
    protected $idDokumentu;
    /** @var  bool */
    protected $verze;
    /** @var  int */
    protected  $platnostOd;


    /**
     * @param Database $database
     * @param int  $idDokumentu
     * @param bool $verze
     * @param int  $platnostOd
     */
    function __construct($database, $idDokumentu, $verze = false, $platnostOd = null)
    {
        $this ->database = $database;
        $this ->idDokumentu = $idDokumentu;
        $this ->verze = $verze;
        $this ->platnostOd = $platnostOd;
    }

    /**
     * @return void
     * @throws \Exception
     */
    public function output()
    {
        if (($GLOBALS['kiosek_mode'] ?? false)  &&  !($_GET['kiosekdownload'] ?? false)) {
            // zobrazeni dokumentu na kiosku => zobrazi appview_dokument
            HttpResponse::renderHeaders();
            require 'view/appview_dokument.php';
            return;
        }

        if ( ini_get('mssql.textlimit') < 8388608 ) ini_set('mssql.textlimit', '8388608');
        if ( ini_get('mssql.textsize') < 8388608 ) ini_set('mssql.textsize', '8388608');

        if ( ini_get('odbc.defaultlrl') < 8388608 ) ini_set('odbc.defaultlrl', '8388608');

        $params = $this ->database ->createParameters();
        $sql = $this ->verze
            ?
            'SELECT JmenoSouboru, Data, TypKomprimace, {fn_CASTWIDEMEMO(Link)}, Odkaz FROM VerzeDokumentu ' .
            ' WHERE Id = ' . $params ->addParameter((int)$this ->idDokumentu) .
            '   AND PlatnostOd = ' . $params ->addParameter($this ->database ->timestampToParam($this ->platnostOd))
            :
            'SELECT JmenoSouboru, Data, TypKomprimace, {fn_CASTWIDEMEMO(Link)}, Odkaz FROM Dokument WHERE Id = ' . $params ->addParameter((int)$this ->idDokumentu);

        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Chyba! Pozadovany dokument neexistuje!');
        }


        // headers

        if ($row[1]) {
            // dokument s daty
            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($row[0]);
            HttpResponse::renderHeaders();

            switch ($row[2]) {
                case 0 :
                    print $cursor ->stringToBinary($row[1]);
                    exit;

                case 1 :
                    print gzuncompress($cursor ->stringToBinary($row[1]));
                    exit;
            }
        } else if ($row[3]) {
            // dokument ma definovany link - presmeruje se na zadany link
            $r = new Redirect($row[3]);
            $r ->render(); // nastavi headers

            HttpResponse::renderHeaders();

            exit;
        } else if ($row[4]) {
            if (!file_exists($row[4])) {
                throw new \Exception('Pozadovany soubor nebyl nalezen!');
            }
            // dokument obsahuje jmeno souboru na disku
            HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent($row[4]);
            HttpResponse::renderHeaders();

            readfile($row[4]);
        }

        // nic - tu by nemel dojit
    }
}
