<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Dokumenty;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;

/**
 * Class ZruseniDokumentuField
 * Mazani dokumentu pomoci linku (GET)
 */
class ZruseniDokumentuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete"></i>',
            TextManager::_('Opravdu_odstranit_dokument'),
            $this->link . '&record=' . $rowData[0]);
    }
}


/**
 * Class ZruseniDokumentuPostField
 * Mazani dokumentu pomoci POST nadrazeneho formulare. Formular musi obsahovat input s id="deleteDocumentId"
 */
class ZruseniDokumentuPostField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $text = TextManager::_('Opravdu_odstranit_dokument');
        return "<i class='inline-glyph glyph-button fa glyph-delete delete-document-post' data-ron-confirm-text='$text' data-document-id='{$rowData[0]}'></i>";
    }
}



class DokumentyBrowser extends Browser
{
    var $downloadLink = '';

    var $deleteType = 0;
    var $deleteLink = null;

    var $datumField = null;

//    var $prvniDokument;  // pomocne promenne pro zobrazeni sekci
//    var $nadpisSekce;

    var $downloadField;

    /** @var string seznam nazvu poli oddeleny strednikem, ktere budou poslany v JSON k aktualizaci sloupcu v browseru<br>
     * Sloupce je nutne definovat v tom poradi, jak jsou zobrazeny v browseru!<br>
     * Urceno pro browsery, ktere maji doplnene sloupce. Sloupce musi nasledovat za odkazem ke stazeni.
     */
    public $updateFields;

    // nactena data - jednotlive sekce
    //var $cursors = [];
    //var $sectionCaptions = [];


    /**
     * @param Database $database
     * @param int    $deleteType    Zpusob smazani (0=nic, 1=pres odkaz, 2=pres POST - nadrazeny formular musi obsahovat hidden pole deleteDocumentId)
     * @param string $downloadLink  Pouzije se pro stahnuti dokumentu
     * @param string $deleteLink    Pouzije se pro smazani, pokud je $deleteType=1
     */
    function __construct($database, $downloadLink = null, $deleteType = 0, $deleteLink = null)
    {
        parent::__construct($database);

        $this->downloadLink = $downloadLink;
        $this ->deleteType = $deleteType;
        $this ->deleteLink = $deleteLink;

        $this->title = TextManager::_('Dokumenty');
        $this->tableId = 'Dokumenty';
        $this->sqlFromPart = 'Dokument, TypDokumentu';
        $this->sqlWherePart = 'Dokument.Typ = TypDokumentu.Kod';
        $this->sqlSelectPart = 'Dokument.Id';

        $this ->primaryField = 0;
        $this->distinct = true;

        $this->cssClass = 'browser minwidth';

        $this ->tableHtmlParams = [
            'data-download-link' => $this ->downloadLink .(substr($this ->downloadLink, -1) == '=' ? '' : '&record=')
        ];

        //$this->prvniDokument = true;

        $this ->datumField = $this ->addField()
            ->setSqlFieldName('Dokument.Datum') ->setHeaderText('Date') ->setFormat('date')
            ->setCode('col-datum')
            ->setHeaderAttributes(['data-name' => 'datum']);

        $this->defaultSort = 'col-datum';

        $this ->addField()
            ->setSqlFieldName('TypDokumentu.Popis') ->setHeaderText('Typ_dokumentu')
            ->setCode('col-typ')
            ->setHeaderAttributes(['data-name' => 'typDokumentu']);

        $this ->addField()
            ->setSqlFieldName('Dokument.Popis') ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setHeaderAttributes(['data-name' => 'popis']);

        // odkaz na stahnuti souboru
        $this ->addField()
            ->setSqlFieldName('Dokument.JmenoSouboru')
            ->setCode('col-jmenosouboru')
            ->setLink($this ->downloadLink) ->setLinkAttributes(['target' => '_blank'])
            ->setHeaderAttributes(['data-name' => 'jmenoSouboru']);

        switch ($this ->deleteType) {
            case 0 :
                break;

            case 1 :
                // odkaz na smazani souboru
                if ($this->deleteLink) {
                    $deleteField = new ZruseniDokumentuField();
                    $deleteField ->link = $this->deleteLink; // '?menuid=zrusitdokument&' . $this->downloadLink;
                    $this->addField($deleteField);
                }
                break;

            case 2 :
                $deleteField = new ZruseniDokumentuPostField();
                $this->addField($deleteField);
                break;
        }
    }

    public function getTableAttributes()
    {
        return parent::getTableAttributes()
            ->append(['data-update-columns' => $this->updateFields]);
    }

}
