<?php
/**
 * pavel.rudol, 01.11.2022 18:10
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\TextManager;
use RonPortal\RonPortalSession;

class VyhledaniDovednostiFilterForm extends DovednostiFilterForm
{
    /** @var int pocet nastavenych filtru na dovednost */
    private $pocetDovednosti;
    /** @var Option[] typy dovednosti do comba */
    private $typyDovednostiOptions;

    public $seznamTypuDovednosti;
    public $seznamDovednosti;
    public $seznamUrovniDovednosti;

    /**
     * @param Database $database
     * @param $typRozdeleniFiltr1
     * @param $typRozdeleniPracMisto
     * @throws DBException
     */
    public function __construct(Database $database, $typRozdeleniFiltr1, $typRozdeleniPracMisto)
    {
        parent::__construct($database, 'ziskane', $typRozdeleniFiltr1, $typRozdeleniPracMisto, false);

        $this->setUseGet(true, ['menuid' => 'maticedovednosti', 'action' => 'vyhledani']);

        $this->skrytTlacitkoNastavit = true;
        $this->pocetDovednosti = 1;

        $params = $this->database->createParameters();
        $sql = 'SELECT Kod, Popis FROM TypDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.TypDovednosti = TypDovednosti.Kod AND Aktivni = ' . $params->addParameter(1) . ') ORDER BY Popis';
        $this->typyDovednostiOptions = Option::loadSQL($this->database, $sql, $params);
    }

    /** Filtry na dovednosti z predka se nepouziji.
     * @return void
     */
    protected function addFilters()
    {
        //v DovednostiFilterForm se zde nastavují filtry na dovednosti, ktere se zde nepouziji
    }

    /** Prida do stranky filtr na typ dovednosti a zatrzitko pro moznost odebrat filtr na dovednost.
     * @param int $index index prvku ve strance
     */
    protected function renderTypDovednostiOsoby($index)
    {
        echo '<DIV class="filterrow_dovednost">';
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo TextManager::_('Dovednosti_typ_dovednosti');
        echo '</SPAN>';

        $htmlKod = 'typ_dovednosti'.$index;

        // vyber typu dovednosti
        $combo = new ComboBox($htmlKod, $this->typyDovednostiOptions);
        $combo
            ->setClassName('form-input adsfilterforminput_full ronselect')
            ->setShowCode(true)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setTranslationPrefix("db_typdovednosti_popis_")
            ->setHtmlParams([
                'data-ron-select2' => "#ronportalfilter_{$htmlKod}_kod",
                'data-ronselect-class' => "ronselect adsfilterforminput_full",
                'data-ron-autosubmit' => ($this ->autoSubmit ? '1' : '0')
            ]);

        if (isset($this->seznamTypuDovednosti[$index-1])) {
            $combo->setSelectedOption($this->seznamTypuDovednosti[$index-1]);
        }

        $combo->render();

        echo '<span class="dovednosti_caption"></span>';
        GuiUtils::renderCheckbox('deletefilter' . $index, TextManager::_('Odebrat'), false);

        echo '</DIV>';
    }

    /** Prida do stranky filtr na dovednost, pokud byl zvoleny typ, pak seznam odfiltruje na zvoleny.
     * @param int $index index prvku ve strance
     * @throws DBException
     */
    protected function renderDovednostOsoby($index)
    {
        $indexDovednosti = $index-1;
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        echo TextManager::_('Dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =  "SELECT Kod, Popis FROM Dovednost WHERE Aktivni = 1 ";

        if (isset($this->seznamTypuDovednosti[$indexDovednosti]) && ($this->seznamTypuDovednosti[$indexDovednosti] != '')) {
            $sql .= "AND TypDovednosti = " . $params->addParameter($this->seznamTypuDovednosti[$indexDovednosti]);
        }

        $sql .= ' ORDER BY Popis';

        $options = Option::loadSQL($this->database, $sql, $params);

        $htmlKod = 'dovednost'.$index;

        // vyber typu dovednosti
        $combo = new ComboBox($htmlKod, $options);
        $combo
            ->setClassName('form-input adsfilterforminput_full ronselect')
            ->setShowCode(true)
            ->setEmptyRow("&nbsp;"/*TextManager::_('_vse_')*/)
            ->setTranslationPrefix("db_dovednost_popis_")
            ->setHtmlParams([
                'data-ronselect-class' => "ronselect adsfilterforminput_full",
                'data-ron-autosubmit' => ($this ->autoSubmit ? '1' : '0')
            ]);

        if (isset($this->seznamDovednosti[$indexDovednosti])) {
            $combo->setSelectedOption($this->seznamDovednosti[$indexDovednosti]);
        }

        $combo->render();
    }

    /** Prida do stranky filtr na uroven dovednosti, pokud byla zvolena konkretni dovednost, pak zobrazi jen urovne pro tuto dovednost.
     * @param int $index index prvku ve strance
     * @throws DBException
     */
    protected function renderUrovenDovednostiOsoby($index)
    {
        $indexDovednosti = $index-1;

        echo '<span class="dovednost_filter_uroven">';

        echo '<SPAN class="dovednosti_caption">';
        echo TextManager::_('Dovednosti_uroven_dovednosti');
        echo '</SPAN>';

        $params = $this->database->createParameters();

        // combobox pro vyber
        $sql =
            "SELECT DISTINCT KodUrovne FROM UrovenDovednosti WHERE EXISTS(SELECT Kod FROM Dovednost WHERE Dovednost.Kod = UrovenDovednosti.KodDovednosti AND Aktivni = 1)";

        if (isset($this->seznamDovednosti[$indexDovednosti]) && ($this->seznamDovednosti[$indexDovednosti] != '')) {
            $sql .= " AND KodDovednosti = " . $params->addParameter($this->seznamDovednosti[$indexDovednosti]);
        }

        // vyber urovne dovednosti
        $combo = new ComboBox('uroven_dovednosti'.$index);
        $combo->className = 'form-input adsfilterforminput_kod';
        $combo->showCode = true;
        $combo->captionIndex = 0;
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->loadData($this->database, $sql . ' ORDER BY KodUrovne', $this->seznamUrovniDovednosti[$indexDovednosti] ?? null, $params);
        $combo->render();

        echo "</span></DIV></DIV>\n";
    }

    /** Zobrazi filtry na typ dovednosti, dovednost a jeji uroven + tlacitka pro pridani noveho filtru, nebo odebrani jiz nastavenych filtru.
     * @throws DBException
     */
    private function renderSeznamDovednosti()
    {
        for ($i = 1; $i <= $this->pocetDovednosti; $i++) {
            $this->renderTypDovednostiOsoby($i);
            $this->renderDovednostOsoby($i);
            $this->renderUrovenDovednostiOsoby($i);
        }

        echo '<INPUT type="submit" class="submitbutton" name="addfilter" value="' . TextManager::_('Pridat') . '">';
        echo '<INPUT type="submit" class="submitbutton" name="deletefilter" value="' . TextManager::_('Odebrat') . '">';
    }

    /** Zobrazi tlacitko pro vyhledani osob dle zvolenych dovednosti a urovni.
     * @return void
     */
    function renderFoto()
    {
        echo "<td><div><INPUT type='submit' class='submitbutton' name='find' value='" . TextManager::_('Vyhledat') . "'></div></td>";
    }

    /** Vypise filtry do stranky
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters(); // rozdeleni

        $this->renderSeznamDovednosti();
    }

    /** Nacte nastavene filtry z GETu. Prida/odstrani filtr na dovednost.
     * @return void
     */
    protected function loadSeznamDovednosti()
    {
        $pridejDovednost = isset($_GET['addfilter']);

        if (isset($_GET['pocet_dovednosti'])) {
            $this->pocetDovednosti = $_GET['pocet_dovednosti'];
        } elseif (isset($_SESSION['dovednost_pocet_dovednosti'])) {
            $this->pocetDovednosti = RonPortalSession::getValue('dovednost_pocet_dovednosti');
        }

        if ($pridejDovednost) {
            $this->pocetDovednosti++;
        }

        $puvodniPocetDovednosti = (int)$this->pocetDovednosti;

        if (isset($_GET['pocet_dovednosti'])) {
            for ($index = 1; $index <= $puvodniPocetDovednosti; $index++) {
                $nacistDovednost = true;
                if (isset($_GET['deletefilter' . $index])) {
                    $nacistDovednost = false;
                    $this->pocetDovednosti--;
                }
                if ($nacistDovednost) {
                    if (isset($_GET['typ_dovednosti' . ($index)]) && $_GET['typ_dovednosti' . ($index)] !== '') {
                        $this->seznamTypuDovednosti[] = $_GET['typ_dovednosti' . ($index)];
                    }
                    if (isset($_GET['dovednost' . ($index)]) && $_GET['dovednost' . ($index)] !== '') {
                        $this->seznamDovednosti[] = $_GET['dovednost' . ($index)];
                    }
                    if (isset($_GET['uroven_dovednosti' . ($index)]) && $_GET['uroven_dovednosti' . ($index)] !== '') {
                        $this->seznamUrovniDovednosti[] =  $_GET['uroven_dovednosti' . ($index)];
                    }
                }
            }
        } else {
            $this->seznamTypuDovednosti = RonPortalSession::getValue('typydovednosti');
            $this->seznamDovednosti = RonPortalSession::getValue('seznamdovednosti');
            $this->seznamUrovniDovednosti = RonPortalSession::getValue('seznam_urovni_dovednosti');
        }

        RonPortalSession::setValue('typydovednosti', $this->seznamTypuDovednosti);
        RonPortalSession::setValue('seznamdovednosti', $this->seznamDovednosti);
        RonPortalSession::setValue('seznam_urovni_dovednosti', $this->seznamUrovniDovednosti);

        $this->hiddenFields['pocet_dovednosti'] = $this->pocetDovednosti;
        RonPortalSession::setValue('dovednost_pocet_dovednosti', $this->pocetDovednosti);
    }

    /** Nacte nastavene filtry z GETu, pokud jiz byly osoby jednou vyhledani prida do filtru hidden pole "showdata", aby se pri dalsim doplneni filtru jiz zobrazoval seznam osob.
     * @throws Exception
     */
    function loadData()
    {
        parent::loadData();

        $this->loadSeznamDovednosti();

        if (isset($_GET['find']) || isset($_GET['showdata']))
            $this->hiddenFields['showdata'] = 1;
    }
}