<?php
/**
 * formular pro editaci vsech zaznamu zpracovane dochazky v jednom dni
 *
 * TODO: DOPLNIT:
 * - vlozeni noveho zaznamu s moznosti zaskrtnout "automaticky vlozit prestavky" (tj. vzdy bude zobrazen jeden zaznam navic?)
 * - smeny pres pulnoc editovat v obou dnech?
 * - javascripty
*/


namespace RonPortal\Editace;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\Redirect;
use RonPortal\Controls\RozdeleniShiftFormView;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use RonPortal\TypRozdeleni;
use RonPortal\Vyroba\DenniVyrobaOsobyForm;
use RonPortal\Vyroba\VyrobaUtils;
use RonPortal\XML\RonPortalXMLForm;


class EditaceDneFormular extends EditaceDochazkyFormular
{
    /** @var ZaznamDochazky[] */
    var $zaznamy;
    /** @var int */
    var $den;

    /** @var $pp    PracovniPomer */
    var $pp;

    var $zobrazitNovy;
    /** @var  ZaznamDochazky   pridani noveho zaznamu */
    var $novyZaznam;

//    var $f_errors; // kody poli, ve kterych je chyba - pouziva se pro rozdeleni vysledku

    /** @var int    Datum, od ktereho je potreba provest prepocet */
    var $prepocetOd;

    /** @var RozdeleniVysledkuFormular */
    var $rozdeleniForm;

    /** @var DenniVyrobaOsobyForm */
    var $vyrobaForm;

    var $povolitEditaciRozdeleni;
    var $povolitEditaciVyroby;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

//        $this ->f_errors = [];
    }


    function nastavReadOnly($idpp, $datumOd, $datumDo, $operace = null)
    {
        parent::nastavReadOnly($idpp, $datumOd, $datumDo, null);

        if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $this ->den)) {
            // dochazka uz je schvalena
            $this->povolitEditaciRozdeleni = false;
        } else if ($this ->editacePovolena) {
            if ($this ->pravaEditace) {
                $this->povolitEditaciRozdeleni = true;
            } else {
                $prava = Prava::vratPrava('editace_r');
                $this->povolitEditaciRozdeleni = $prava ->lzeZobrazitPP($idpp, $datumOd, $datumDo);
            }
        } else {
            $this->povolitEditaciRozdeleni = false;
        }

        $this ->povolitEditaciVyroby = Prava::vratPrava('editacevyroby') ->lzeZobrazitPP($idpp, $datumOd, $datumDo);
    }


    /**
     * @param int $idpp
     * @param int $den
     * @throws \Exception
     */
    function nactiZakladniUdaje($idpp, $den)
    {
        $this ->pp = PracovniPomer::nactiPP($this ->database, $idpp);
        $this ->den = $den;

        $this ->schvaleni = SchvaleniDochazky::nactiSchvaleni($this ->database, $this ->pp, $den);

        $this ->nastavReadOnly($this ->pp ->idPP, $den, $den);

        if ($this ->readOnly) {
            // pokud nemuze editovat, tak jeste zkontroluje, jestli ma vubec pravo prohlizet
            $prava = Prava::vratPrava('prohlizeni');
            if (!$prava ->lzeZobrazitPP($idpp, $den, $den)) {
                throw new NeniPravoException();
            }
        }

        if (!$this ->schvaleni ->lzeEditovatPP($idpp, $den)) {
            $this ->readOnly = true;
        }
    }


    /**
     * @param int $den
     * @throws \Exception
     * @throws DBException
     */
    function nactiDochazku($den)
    {
        // zaznamy zpracovane dochazky
        $this ->zaznamy = [];

        $rec = new ZaznamDochazky($this ->database, $this ->pp, $this);
        $params = $this ->database ->createParameters();
        $rec ->loadRecords(
                    'Dochazka.DatumCasPocatku >= ' . $params ->addParameter($this ->database ->dateToParam($den)) .
                    ' AND Dochazka.DatumCasPocatku < ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay($den, 1))),
                    $params);

        $newRec = new ZaznamDochazky($this ->database, $this ->pp, $this);

        while ($rec ->next($newRec)) {
            $newRec ->nactiRozdeleni();
            $newRec ->nactiPoznamku();

            $this ->zaznamy[] = $newRec;

            $newRec = new ZaznamDochazky($this ->database, $this ->pp, $this);
        }

        $this ->novyZaznam = $newRec;
    }


    /**
     * @param int $idpp
     * @param int $den
     * @throws \Exception
     */
    function nactiDataProDen($idpp, $den)
    {
        Prava::nastavObdobiProPrava($den, $den);

        $this ->nactiZakladniUdaje($idpp, $den);

        $this ->rozdeleniForm = new RozdeleniVysledkuFormular($this ->database, $this ->pp, $this ->den);
        $this ->rozdeleniForm ->nactiData_DB();

        if (RonPortalSettings::getIntValue('vyroba')  &&   RonPortalSettings::getIntValue('vyrobaveditaci')) {
            $this ->vyrobaForm = new DenniVyrobaOsobyForm($this ->database, $this ->pp, $this ->den);
            $this ->vyrobaForm ->load();
        }

        $this ->nactiDochazku($den);
    }


    /**
     * @return void
     */
    function zobrazRozdeleniVysledku()
    {
        if (RonPortalSettings::getPouzivatRozdeleniDV()) {
            $this->rozdeleniForm->show(!$this->povolitEditaciRozdeleni);
        }
    }


    /**
     */
    function render()
    {
        $this ->zobrazitNovy = $this ->zobrazitNovy ? 1 : 0;
        if ($this ->zobrazitNovy) {
            $this ->zaznamy [] = $this ->novyZaznam;
        }

        require('ronportal/editace/view_editace_dne.php');

        if (RonPortalSettings::getPouzivatRozdeleniDV() && RonPortalSettings::getIntValue(RonPortalSettings::DV_POVOLIT_PRIRAZENI_ROZDELENI)) {
            // Dialog pro vyber rozdeleni osoby
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI));
            $url = '?menuid=editace&action=rozdeleniosoby&idpp=' . $this ->pp ->idPP;

            $shiftForm = new RozdeleniShiftFormView($typRozdeleni, $url);
            $shiftForm ->render();
        }

        $this ->showJsonData();
    }


    /**
     * @param int      $idpp
     * @param int      $den
     */
    function show($idpp, $den)
    {
        $this ->nactiDataProDen($idpp, $den);
        $this ->render();
    }


    /**
     * @param  int $datumCasPocatku
     * @returns ZaznamDochazky
     */
    function najdiZaznamPodlePocatku($datumCasPocatku)
    {
        // vraci zaznam, ktery ma f_Puvodni_DatumCasPocatku = $datumCasPocatku
        foreach ($this ->zaznamy as $zaznam) {
            if ($zaznam->fDatumCasPocatku->originalValue == $datumCasPocatku) {
                return $zaznam;
            }
        }

        return null; // nenasel
    }


    function nactiDenPodlePost()
    {
        $result = true;

        for ($i = 0; $i < $_POST['pocet']; $i++) {
            $zaznam = $this ->najdiZaznamPodlePocatku($_POST['datumcaspocatku' . $i]);

            if ($zaznam) {
                $result &= $zaznam ->nactiDataPodlePost(false, $i, !RonPortalSettings::getIntValue(ZaznamDochazky::SETTINGS_SKRYT_ZAPOCTENY)); // $insert = false, $nacistZapoctenyCas = dle nastaveni
                $zaznam ->smazatZaznam = ($_POST['deleterow'] != '')  &&  ((int)$_POST['deleterow'] == $i);
            } else {
                // nenalezen puvodni zaznam
                // zobrazit chybu?
            }
        }

        return $result;
    }


    /**
     * @param int $datumOd
     */
    function prepocetDne($datumOd)
    {
        // provede prepocet dochazky
        AgentManager::prepocetDochazky($this ->pp ->idPP, $datumOd, DateUtils::lastDayOfMonth($datumOd));
    }


    /**
     * Kontrola odeslanych dat
     */
    function zkontrolujData()
    {
        // Zkontroluje, jestli se zaznamy neprekryvaji
        foreach ($this ->zaznamy as $i => $zaznam) {
            if (!$zaznam ->smazatZaznam) {
                for ($j = $i + 1; $j < count($this->zaznamy); $j++) {
                    $zaznam2 = $this->zaznamy[$j];

                    if (!$zaznam2 ->smazatZaznam) {
                        if ($zaznam->fDatumCasPocatku->value < $zaznam2->fDatumCasUkonceni->value && $zaznam->fDatumCasUkonceni->value > $zaznam2->fDatumCasPocatku->value) {
                            throw new \Exception(\Ron\ronText('Chyba_prekryvani_zaznamu'));
                        }

                        if ($zaznam->fDatumCasUkonceni->value > $zaznam2->fDatumCasPocatku->value && $zaznam->fDatumCasPocatku->value < $zaznam2->fDatumCasUkonceni->value) {
                            throw new \Exception(\Ron\ronText('Chyba_prekryvani_zaznamu'));
                        }
                    }
                }
            }
        }
    }


    /**
     * @return bool
     */
    function ulozZaznamy()
    {
        $this ->prepocetOd = 0;

        $result = true;

        foreach ($this ->zaznamy as $zaznam) {
            //$zaznam ->urciDenPrirazeni($connection); - od 05-2015 zadava den prirazeni primo uzivatel

            $provestPrepocet = $zaznam ->smazatZaznam  ||  $zaznam ->isChanged();

            if ($zaznam ->smazatZaznam) {
                $zaznam ->smazZaznam();
            } else {
                $jeZmenaPoznamky = $zaznam->ulozPoznamku(false, true); // $insert = false, pouzije puvodni datumCasPocatku ($zaznam jeste neni ulozeny)

                if ($zaznam->ulozRozdeleni(false, true)) { // $insert = false, pouzije puvodni datumCasPocatku
                    $jeZmenaPoznamky = true;
                }

                // pokud doslo k editaci poznamky nebo rozdeleni, nastavi vzdy Zpracoval (jinak jen v pripade, ze doslo ke zmene samotneho zaznamu
                $zaznam ->fZpracoval ->setUpdateLog(!$jeZmenaPoznamky);

                if ($zaznam->ulozZmenenyZaznam()) {

                    if ($provestPrepocet) {
                        if ($this->prepocetOd == 0 || $zaznam->fDatumCasPocatku->value < $this->prepocetOd) {
                            $this->prepocetOd = $zaznam->fDatumCasPocatku->value;
                        }

                        if (!$zaznam->executeScripts(RonPortalScriptEvents::EDITACE_DOCHAZKY_IW_AFTER, false, false)) $result = false;
                    }
                } else {
                    $result = false;
                }
            }
        }

        // odstrani smazany zaznam (aby se pri pripadne chybe uz nezobrazil)
        $noveZaznamy = [];
        foreach ($this ->zaznamy as $zaznam) {
            if (!$zaznam ->smazatZaznam) {
                $noveZaznamy[] = $zaznam;
            }
        }
        $this ->zaznamy = $noveZaznamy;

        return $result;
    }


    /**
     * @return bool
     */
    function ulozRozdeleniVysledku()
    {
        if (!RonPortalSettings::getPouzivatRozdeleniDV()) return true; // dochazka se nerozdeluje

        if (!$this ->povolitEditaciRozdeleni) return true;

        if (!$this ->schvaleni ->lzeEditovatPP($this ->pp ->idPP, $this ->den)) {
            // dochazka uz je schvalena
            return false;
        }

        $this ->rozdeleniForm = new RozdeleniVysledkuFormular($this ->database, $this ->pp, $this ->den);

        $result = $this ->rozdeleniForm ->save();

        if ($result && $this ->rozdeleniForm ->jeZmena) {
            if ($this ->prepocetOd == 0 || $this ->den < $this ->prepocetOd) $this ->prepocetOd = $this ->den;
        }

        return $result;
    }


    function ulozRozdeleniVyroby()
    {
        // zkontroluje nastaveni
        if (!RonPortalSettings::getIntValue('vyroba')  ||   !RonPortalSettings::getIntValue('vyrobaveditaci')) {
            return;
        }

        // zkontroluje prava
        if (!$this ->povolitEditaciVyroby) return;

        $this ->vyrobaForm ->process();
    }


    /**
     * Pokud doslo ke zmene, tak ulozi zaznamy. V pripade chyby vraci false.
     * @return bool
     * @throws \Exception
     */
    function save()
    {
        $result = false;
        $idPP = (int)$_POST['idpp'];
        $den = (int)$_POST['datum'];
        $callAfterXml = false;

        // nejdrive nacte puvodni data
        $this ->nactiDataProDen($idPP, $den);

        if ( (isset($_POST['ulozzaznamyproden'])  &&  $_POST['ulozzaznamyproden']) ) {
            if ($this->readOnly) {
                // prakticky by nemelo nastat
                throw new \Exception('Chyba! Data nelze editovat (obdobi je uzavreno, nebo nemate pravo pro editaci).');
            }

            if ($this->nactiDenPodlePost()) {
                Csrf::validateCSRFPostParam("editacedne_{$idPP}_{$den}");

                $this ->zkontrolujData();

                EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $den, 'before');

                $result = $this->ulozZaznamy();

                if ($result && $this->prepocetOd && RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET)) {
                    // je vybrana casova slozka pro porovnani v rozdeleni dennich vysledku - provede prepocet, aby se spravne napocitala jeste pred ulozenim rozdeleni
                    $this->prepocetDne($this->prepocetOd);
                    $this->prepocetOd = 0;
                }

                $callAfterXml = true;
            }
        }

        if (isset($_POST['ulozrozdelenivysledku'])  &&  $_POST['ulozrozdelenivysledku']) {
            if (!$this->ulozRozdeleniVysledku()) $result = false;
        }


        if (isset($_POST['ulozrozdelenivyroby'])  &&  $_POST['ulozrozdelenivyroby']) {
            //if (!$this ->ulozRozdeleniVyroby()) $result = false; // zajisti, ze se formular zobrazi znovu napr. po kliknuti na "novy radek"
            $this ->ulozRozdeleniVyroby();

            if (MessageManager::getInstance() ->isError()) {
                $result = false;
            }
        }

        if ($result && $this->prepocetOd) $this->prepocetDne($this->prepocetOd);

        if ($callAfterXml) {
            // POZOR: tady se zdvoji chybova hlaska (napr. spatny format casu v rozdeleni vyroby) - je potreba s tim neco udelat!
            EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $den, 'after', false);
        }

        if (MessageManager::getInstance() ->isError()) {
            $result = false;
        }
        if ($this ->rozdeleniForm && $this ->rozdeleniForm ->pokracovat) {
            // znovu se zobrazi stejny formular
            $result = false;
        }
        if ($this ->vyrobaForm && $this ->vyrobaForm ->pokracovat) {
            // znovu se zobrazi stejny formular
            $result = false;
        }

        if ($_POST['deleterow'] != '') {
            // po smazani nejakeho zaznamu zobrazi stejny formular
            $result = false;
        }

        return $result;
    }


    /**
     * @param int $idpp
     * @param int $den
     * @param int $uloha
     * @throws NeniPravoException
     * @throws \Exception
     */
    function spustAgentaProDen($idpp, $den, $uloha)
    {
        if (!RonPortalSettings::getSpousteniADS()) {
            throw new \Exception('Neni nastaveno spousteni agentu. Operaci nelze provest!');
        }

        // nacte pracovni pomer,
        $this ->nactiZakladniUdaje($idpp, $den);

        if ($this ->readOnly) {
            throw new NeniPravoException();
        }

        $params = $uloha .
            ' -interval:' . date('Y-n-j', $den) . ';' . date('Y-n-j', $den) .
            ' -IDPP:' . $idpp .
            ' -alwaysLog ';

        AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
    }



    private function podlePuvodni($idpp, $den)
    {
        Prava::nastavObdobiProPrava($den, $den);

        try {
            $this ->spustAgentaProDen($idpp, $den, '8');
            MessageManager::addInfoMessage( \Ron\ronText('Pregenerovani_podle_puvodni_OK') );
        } catch (\Exception $e) {
            MessageManager::addErrorMessage( \Ron\ronText('Pregenerovani_podle_puvodni_err') );
            MessageManager::addErrorMessage($e);
        }

        return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $idpp . '&obdobi=' . DateUtils::firstDayOfMonth($den) . '&datum=' . $den);
    }



    private function vlozitPrestavky($idpp, $den)
    {
        Prava::nastavObdobiProPrava($den, $den);

        try {
            $this ->spustAgentaProDen($idpp, $den, '7');
            MessageManager::addInfoMessage(\Ron\ronText('Vlozeni_prestavek_OK'));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage( \Ron\ronText('Vlozeni_prestavek_err') );
            MessageManager::addErrorMessage($e);
        }

        return new Redirect(null, '?menuid=editacedne&idpp=' . $idpp . '&datum=' . $den);
    }


    private function zrusitZaznamy($idpp, $den)
    {
        $den = DateUtils::startOfTheDay($den);
        Prava::nastavObdobiProPrava($den, $den);

        // kontrola prav, uzavreni obdobi atd.
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            // nedela nic, editace dochazky neni povolena
            return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $idpp);
        }

        // nacte pracovni pomer,
        $this ->nactiZakladniUdaje($idpp, $den);

        if ($this ->readOnly) {
            throw new \Exception('Chyba! Data nelze editovat (obdobi je uzavreno, nebo nemate pravo pro editaci).');
        }


        $schvaleni = new SchvaleniDochazky();
        $schvaleni ->load($this ->database, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu, $den, $den);
        if (!$schvaleni ->lzeEditovatPP($idpp, $den)) {
            throw new \Exception('Nelze smazat zaznamy! Dochazka je jiz schvalena.');
        }

        try {
            $rec = new DochazkaRecord($this->database, $this->pp, $this);
            $rec->logInfoMessage = 'Smazani dochazky pro den ' . date('j.n.Y', $den);
            $params = $this ->database ->createParameters();
            $rec->smazZaznamyDochazky(
                ' DatumCasPocatku >= ' . $params ->addParameter($this->database->dateToParam($den)) .
                ' AND DatumCasPocatku < ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay($den, 1))),
                $params
            );

            AgentManager::prepocetDochazky($idpp, $den, DateUtils::lastDayOfMonth($den));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }

        return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $idpp . '&datum=' . $den);
    }


    private function podlePuvodniVyroby($idpp, $den)
    {
        Prava::nastavObdobiProPrava($den, $den);

        try {
            if (!RonPortalSettings::getValue('vyroba_agenthost')) {
                throw new \Exception('Neni nastaveno spousteni agentu vyroby. Operaci nelze provest!');
            }

            // nacte pracovni pomer,
            $this ->nactiZakladniUdaje($idpp, $den);

            if ($this ->readOnly) {
                throw new NeniPravoException();
            }

            VyrobaUtils::znovuZpracovaniHromadne($idpp, $den);
            MessageManager::addInfoMessage( \Ron\ronText('Pregenerovani_podle_puvodni_vyroby_OK') );
        } catch (\Exception $e) {
            MessageManager::addErrorMessage( \Ron\ronText('Pregenerovani_podle_puvodni_vyroby_err') );
            MessageManager::addErrorMessage($e);
        }

        return new Redirect(null, '?menuid=editacedne&idpp=' . $idpp . '&obdobi=' . DateUtils::firstDayOfMonth($den) . '&datum=' . $den);
    }



    function provedAkci($akce)
    {
        $idPP = (int)$_GET['idpp'];
        $den = (int)$_GET['datum'];

        $this ->pp = PracovniPomer::nactiPP($this ->database, $idPP);

        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $den, 'before');

        $result = null;

        switch ($akce) {
            case 'prestavkyproden' :
                $result = $this ->vlozitPrestavky($idPP, $den);
                break;

            case 'podlepuvodni' :
                $result = $this ->podlePuvodni($idPP, $den);
                break;

            case 'zrusitzaznamyproden' :
                $result = $this ->zrusitZaznamy($idPP, $den);
                break;

            case 'podlepuvodnivyroby' :
                $result = $this ->podlePuvodniVyroby($idPP, $den);
                break;
        }

        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $den, 'after');

        return RonPortalXMLForm::$eventRedirect ?: $result;
    }
}

