<?php
/**
 * stanislav.elbl, 23.06.2022 10:23
 * Formular pro zmenu prirazeneho kalendare zamestnance
 */

namespace RonPortal\Editace;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBRecord;
use Ron\GuiUtils;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;


class HistorieKalendaruField extends FormField
{
    /** @var PrirazeniKalendare[] */
    private $kalendare;

    private $maxRows = 10;

    /**
     * HistorieKalendaruField constructor.
     * @param \RonPortal\Dochazka\PrirazeniKalendare[] $kalendare
     */
    public function __construct(array $kalendare)
    {
        parent::__construct(self::FIELD_TYPE_TEXT);
        $this->kalendare = $kalendare;
        $this->dataColumnClass = 'col-12 padding-all-2';
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!$this ->showColumns) {
            echo '<div ' . GuiUtils::getAttributesHtml($this ->getDataColAttributes('')) . '>';
        }


        // nadpis
        echo
            '<div class="row border-bottom">' .
            '<div class="col-6 col-sm-2">' . TextManager::_('Platnost_od') . '</div>' .
            '<div class="col-6 col-sm-2">' . TextManager::_('Platnost_do') . '</div>' .
            '<div class="col-4 col-sm-2">' . TextManager::_('Kod') . '</div>' .
            '<div class="col-8 col-sm-6">' . TextManager::_('Popis') . '</div>' .
            "</div>\n";

        echo
            "<div class='oddEvenChild'>\n";

        for ($i = count($this ->kalendare) - 1;  $i >= 0  &&  (!$this ->maxRows  ||  $i >= count($this ->kalendare) - $this ->maxRows);  $i--) {
            echo
                '<div class="row padding-top padding-bottom">' .
                '<div class="col-6 col-sm-2">' . StringUtils::format('date', $this ->kalendare[$i] ->platnostOd) . '</div>' .
                '<div class="col-6 col-sm-2">' . StringUtils::format('date', $this ->kalendare[$i] ->platnostDo) . '</div>' .
                '<div class="col-4 col-sm-2">' . StringUtils::htmlTextConvert($this ->kalendare[$i] ->kod) . '</div>' .
                '<div class="col-8 col-sm-6">' . StringUtils::htmlTextConvert($this ->kalendare[$i] ->popis) . '</div>' .
                "</div>\n";
        }

        echo
            "</div>\n";

        if (!$this ->showColumns) {
            echo '</div>';
        }
    }
}


class KalendarOsobyForm extends RonPortalForm
{
    /** @var PracovniPomer */
    private $pp;

    /**
     * KalendarOsobyForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     */
    public function __construct($database, $pp)
    {
        parent::__construct($database);
        $this ->pp = $pp;

        $this ->readOnly = !RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA);
        $this ->title = TextManager::_($this ->readOnly ? 'Detail_kalendare_osoby' : 'Zmena_kalendare_osoby');
        $this ->code = 'formprirazenikalendare';

        $this ->sqlTableName = 'PrirazeniKalendare';

        $this ->stornoAction = '?menuid=zpracovanadochazka&osoba=' . $pp ->idPP;

        $this ->setXmlEventsName('PRIRAZENI_KALENDARE');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('*osoba') ->setDefaultValue($pp ->idPP);

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText($pp ->jmenoPrijmeni) ->setCaption('Jmeno')
            ->setCode('field-jmeno');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText($pp ->osobniCislo) ->setCaption('Osobni_cislo')
            ->setCode('field-osobnicislo');

        $kalendar = PrirazeniKalendare::nactiKalendarProDen($this ->database, $pp->idPP, time());

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCaption('Prirazeny_kalendar')
            ->setCode('header-prirazenykalendar');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText($kalendar
                ? $kalendar ->popis . ' (' . $kalendar ->kod . ')'
                : 'Zamestnanec_nema_kalendar')
            ->setCode('field-prirazenykalendar');

        if ($kalendar) {
            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setText(
                    StringUtils::format('date', $kalendar ->platnostOd) .
                    ($kalendar ->platnostDo
                        ? ' ' . TextManager::_('do') . ' ' . StringUtils::format('date', $kalendar ->platnostDo)
                        : ''))
                ->setCaption('Platnost_od')
                ->setCode('field-platnostprirazenehokalendare');
        }

        if (!$this ->readOnly) {
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCaption('Prirazeni_noveho_kalendare')
                ->setCode('header-prirazenikalendare');

            $this ->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('PlatnostOd') ->setCaption('Platnost_od') ->setRequired(true)
                ->setCode('field-platnostod');

            $this ->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('PlatnostDo') ->setCaption('Platnost_do')
                ->setCode('field-platnostdo');

//            $osoba = [
//                'IDPP' => $pp ->idPP,
//                'ObdobiOd' => $database->timestampToString(RonPortalSession::obdobi()),
//                'ObdobiDo' => $database->timestampToString(RonPortalSession::konecObdobi())
//            ];
//
//            $select = trim(RonPortalXMLForm::xmlEvents($database, RonPortalXMLEvents::PRIRAZENI_KALENDARE_SELECT, $osoba, '', true));

            $this ->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('Kalendar') ->setCaption('Kalendar') ->setRequired(true)
                ->setSelectSql('SELECT Kod, Popis FROM Kalendar WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'Popis')
                ->setCode('field-kalendar');

            $this ->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('*zpracovat') ->setCaption('Zpracovat_podle_puvodni')
                ->setCode('field-zpracovat');
        }


        $kalendare = PrirazeniKalendare::nactiKalendarProObdobi($this ->database, $pp->idPP, 1, time());
        if ($kalendare) {
            // zamestnanec ma prirazene nejake kalandare => zobrazit historii
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCaption('Historie_prirazeni_kalendaru')
                ->setCode('header-historie');

            $this->addField(new HistorieKalendaruField($kalendare))
                ->setCode('field-historie')
                ->setShowColumns(false);
        }
    }


    private function zkontrolujObdobi($rec)
    {
        if (SchvaleniDochazky::jeSchvalenoObdobi($this ->database, $this ->pp, $rec['PlatnostOd'], $rec['PlatnostDo'] ?: time())) {
            throw new \Exception(TextManager::_('Dochazka_je_jiz_schvalena'));
        }

        if (Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $rec['PlatnostOd'], $rec['PlatnostDo'] ?: time())) {
            throw new \Exception(TextManager::_('Obdobi_uzavreno'));
        }
    }

    /**
     * Smaze nebo zkrati zaznamy, ktere se prekryvaji s novym zaznamem
     * @param $rec
     * @throws \Exception
     */
    private function prerusPrekryvaneZaznamy($rec)
    {
        $prekryvane = new RonPortalDBRecord($this ->database, 'PrirazeniKalendare', $this);
        $prekryvane ->addStringField('RC', true);
        $prekryvane ->addStringField('OsobniCislo', true);
        $prekryvane ->addDateField('DatumNastupu', true);
        $prekryvane ->addDateField('PlatnostOd', true);
        $prekryvane ->addDateField('PlatnostDo', true);
        $prekryvane ->addStringField('Kalendar', true);

        $prekryvane ->checkMultiple = false;

        $params = $this ->database ->createParameters();
        $prekryvane ->setWhere(
            // pro stejnou osobu
            'RC = ' . $params ->addParameter($rec['RC']) .
            ' AND OsobniCislo = ' . $params ->addParameter($rec['OsobniCislo']) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($rec['DatumNastupu']),
            $params
        );

        $filtrParams = $this ->database ->createParameters();
        $prekryvane ->loadRecords(
            // ve stejnem obdobi
            '(PlatnostDo IS NULL OR PlatnostDo >= ' . $filtrParams ->addDateParameter($rec['PlatnostOd']) . ')' .
            ($rec['PlatnostDo']
                ? ' AND PlatnostOd <= ' . $filtrParams ->addDateParameter($rec['PlatnostDo'])
                : ''
            ),
            $filtrParams
        );

        while ($prekryvane ->next()) {
            $fPlatnostOd = $prekryvane ->findFieldByName('PlatnostOd');
            $fPlatnostDo = $prekryvane ->findFieldByName('PlatnostDo');

            if ($fPlatnostOd ->originalValue >= $rec['PlatnostOd']) {
                // puvodni kalendar zacina stejne nebo pozdeji nez novy
                if (!$rec['PlatnostDo'] || ($fPlatnostDo ->originalValue  &&  $rec['PlatnostDo'] >= $fPlatnostDo ->originalValue)) {
                    // puvodni kalendar konci stejne nebo drive nez novy => stary je potreba smazat
                    $deleteParams = $this ->database ->createParameters();
                    $prekryvane ->doDelete(
                        'PlatnostOd = ' . $deleteParams ->addDateParameter($fPlatnostOd ->originalValue),
                        $deleteParams
                    );
                } else if (!$fPlatnostDo ->originalValue  ||  $fPlatnostDo ->originalValue > $rec['PlatnostDo']) {
                    // puvodni kalendar konci pozdeji => je potreba ho zkratit
                    $fPlatnostDo ->readOnly = true;
                    $fPlatnostOd ->setNewValue(DateUtils::incDayStart($rec['PlatnostDo'], 1));

                    $updateParams = $this ->database ->createParameters();
                    $prekryvane ->updateRecord(
                        'PlatnostOd = ' . $updateParams ->addDateParameter($fPlatnostOd ->originalValue),
                        $updateParams);
                }
            } else {
                // stary kalendar zacina drive => zacatek se musi ponechat
                if ($rec['PlatnostDo']  &&  (!$fPlatnostDo ->originalValue  ||  $fPlatnostDo ->originalValue > $rec['PlatnostDo'])) {
                    // ... a konci pozdeji - stary zaznam se musi rozdelit na 2 zaznamy
                    // nejdrive usekne konec
                    $originalPlatnostDo = $fPlatnostDo ->originalValue;
                    $fPlatnostOd ->readOnly = true;
                    $fPlatnostDo ->setNewValue(DateUtils::incDayStart($rec['PlatnostOd'], -1));
                    $updateParams = $this ->database ->createParameters();
                    $prekryvane ->updateRecord(
                        'PlatnostOd = ' . $updateParams ->addDateParameter($fPlatnostOd ->originalValue),
                        $updateParams);

                    // a pak vlozi novy zaznam za konec noveho zaznamu
                    $fPlatnostOd ->setNewValue(DateUtils::incDayStart($rec['PlatnostDo'], 1));
                    $fPlatnostDo ->setNewValue($originalPlatnostDo);
                    $prekryvane ->insertRecord();

                } else if (!$rec['PlatnostDo'] || $rec['PlatnostDo'] >= $fPlatnostDo ->originalValue) {
                    // ... a konci drive nez stary  - konec se musi useknout
                    $fPlatnostDo ->setNewValue(DateUtils::incDayStart($rec['PlatnostOd'], -1));
                    $fPlatnostOd ->readOnly = true; // v predchozim kroku se mohl readonly priznak zmenit
                    $updateParams = $this ->database ->createParameters();
                    $prekryvane ->updateRecord(
                        'PlatnostOd = ' . $updateParams ->addDateParameter($fPlatnostOd ->originalValue),
                        $updateParams);
                }
            }
        }
    }


    function beforeInsertRecord($rec)
    {
        $this ->zkontrolujObdobi($rec);

        if (!parent::beforeInsertRecord($rec)) {
            return false;
        };

        $this ->prerusPrekryvaneZaznamy($rec);

        return true;
    }


    function beforeUpdateRecord($rec)
    {
        // nelze delat update - musela by se upravit kontrola prekryvani zaznamu (a pripadne mezer)
        throw new \Exception('UPDATE zaznamu neni implementovan!');
    }


    public function createDBRecordFields($rec, $insert)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            throw new \Exception('Editace_neni_povolena');
        }

        // prida pole pro pracovni pomer
        $rec ->addStringField('RC', true, 15, $this ->pp ->rc);
        $rec ->addStringField('OsobniCislo', true, 15, $this ->pp ->osobniCislo);
        $rec ->addDateField('DatumNastupu', true, $this ->pp ->datumNastupu);

        parent::createDBRecordFields($rec, $insert);

        // Kontrola platnost do - musi byt null nebo >= platnost od
        if ($rec['PlatnostDo']  &&  $rec['PlatnostDo'] < $rec['PlatnostOd']) {
            throw new RonException(TextManager::_('Platnost_do_je_mensi_nez_platnost_od'), 'PlatnostDo');
        }
    }



    function afterInsertRecord($rec)
    {
        //Provest prepocet - zpracovat zadane obdobi podle puvodni dochazky ($uloha=8),
        //ale jen maximalne do data do kdy je dochazka zpracovana.
        if (isset($_POST['zpracovat'])  &&  $_POST['zpracovat']) {
            //zpracovat zadane obdobi podle puvodni dochazky - uloha 8
            $zpracovaniDochazky =  DochazkaUtils::datumCasZpracovaniDochazky($this->database);

            $params =
                '8 -interval:' . date('Y-n-j', $rec['PlatnostOd']) . ';' .
                date('Y-n-j', (!$rec['PlatnostDo']  ||  $zpracovaniDochazky < $rec['PlatnostDo']) ? $zpracovaniDochazky : $rec['PlatnostDo']) .
                ' -IdPP:' . $this ->pp ->idPP;

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        return parent::afterInsertRecord($rec);
    }


}
