<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;

class MzdovaPolozkaVeMzdeRecord extends RonPortalDBRecord
{
    /** @var PracovniPomer */
    public $pp;
    /** @var string */
    public $kodMzdovePolozky;
    /** @var int */
    public $mesic;

    public $fCastka;
    public $fPocetJednotek;
    public $fCenaZaJednotku;

    public $fSerial;

    public $fTypRozdeleni;
    public $fRozdeleni;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $mesic
     * @param string $kodMzdovePolozky
     * @param object|string $logDescription
     * @param int $kodTypuRozdeleni
     * @param string $rozdeleni
     */
    function __construct($database, $pp, $mesic, $kodMzdovePolozky, $logDescription, $kodTypuRozdeleni = null, $rozdeleni = null)
    {
        parent::__construct($database, 'MzdovaPolozkaVeMzde', $logDescription);

        $this ->pp = $pp;
        $this ->mesic = $mesic;
        $this ->kodMzdovePolozky = $kodMzdovePolozky;

        $this->addField('RC', '', true, 0, $pp->rc); // readonly = true
        $this->addField('OsobniCislo', '', true, 0, $pp->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addField('Obdobi', 'date', true, 0, $mesic); // readonly = true

        $this ->addField('DruhZdrojeMzdovePolozky', 'int', true, 0, 4); // MP z dochazky
        $this ->addField('MzdovaPolozka', '', true, 0, $kodMzdovePolozky);
        $this ->fSerial = $this ->addField('SerioveCislo', 'int', true); // hodnota pro insert se nastavi dodatecne

        $this ->fCastka = $this ->addField('Castka', 'float', true);
        $this ->fPocetJednotek = $this ->addField('PocetJednotek', 'float', true);
        $this ->fCenaZaJednotku = $this ->addField('CenaZaJednotku', 'float', true);

        $this ->fTypRozdeleni = $this ->addField('KodTypuRozdeleni', 'int', true);
        $this ->fRozdeleni = $this ->addField('Rozdeleni', '', true);

        $this ->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this ->addField('Insert_Timestamp', 'timestamp', true, 0, time());

        $f = $this ->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $f ->setUpdateLog(true);
        $f ->noInsert = true;
        $f = $this ->addField('Update_Timestamp', 'timestamp', false, 0, time());
        $f ->setUpdateLog(true);
        $f ->noInsert = true;

        $params = $this ->database ->createParameters();
        $where =
            ' RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Obdobi = ' . $params ->addParameter($database ->dateToParam($mesic)) .
            ' AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            ' AND MzdovaPolozka = ' . $params ->addParameter($kodMzdovePolozky);

        if ( isset($this ->$kodTypuRozdeleni) && isset($rozdeleni) ) {
            // jen mzdove polozky daneho rozdeleni
            $where .=
                ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
                ' AND Rozdeleni = ' . $params->addParameter($rozdeleni);
        }

        $this ->setWhere($where, $params);
    }


    /**
     * Vraci nove seriove cislo pro mzdovou polozku dane osoby
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int    $obdobi
     * @param string $kodMP
     * @return bool|int
     */
    public static function noveSerioveCisloMP($database, $pp, $obdobi, $kodMP)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT MAX(SerioveCislo) FROM MzdovaPolozkaVeMzde ' .
            ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            '   AND Obdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)) .
            '   AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            '   AND MzdovaPolozka = ' . $params ->addParameter($kodMP);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0] + 1;
        }

        return 1; // mzdova polozka jeste neexistuje
    }


    function zapisHodnotu($castka, $pocetJednotek, $cenaZaJednotku, $typRozdeleni = null, $rozdeleni = null)
    {
        $this ->checkMultiple = false; // vypne kontrolu "vice zaznamu"

        $this ->fCastka ->setNewValue($castka);
        $this ->fPocetJednotek ->setNewValue($pocetJednotek);
        $this ->fCenaZaJednotku ->setNewValue($cenaZaJednotku);

        if ($typRozdeleni  &&  $rozdeleni) {
            $this->fTypRozdeleni->setNewValue($typRozdeleni);
            $this->fRozdeleni->setNewValue($rozdeleni);
        }

        $params = $this ->database ->createParameters();
        $filter = (isset($typRozdeleni)  &&  isset($rozdeleni))
            ? 'KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            ' AND Rozdeleni = ' . $params->addParameter($rozdeleni)
            : '';

        if ($this ->loadRecord($filter, $params)) {
            // zaznam uz existuje
            // doplni do filtru seriove cislo nactene MP, aby editoval prave ten jeden zaznam
            $updateParams = $this ->database ->createParameters();
            return $this ->updateRecord('SerioveCislo = ' . $updateParams ->addParameter((int)$this ->fSerial ->originalValue), $updateParams);
        } else {
            // zaznam jeste neexistuje
            if (!isset($castka)) {
                // neni zadana castka, nic neni potreba delat
                return true;
            }

            if ($typRozdeleni && $rozdeleni) {
                $this ->fSerial ->setNewValue( self::noveSerioveCisloMP($this ->database, $this ->pp, $this ->mesic, $this ->kodMzdovePolozky) );
            } else {
                $this ->fSerial ->setNewValue(1);
            }

            $this->insertRecord();
            return 1;
        }

    }

}
