<?php
/**
 * stanislav.elbl, 15.10.2021 14:14
 */

namespace RonPortal\Editace;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DebugException;
use Ron\MVC\ViewObject;
use RonPortal\Obdobi;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;

class DenPrehleduRozdeleniDV
{
    /* @var int */
    public $datum;

    /* @var ZaznamRozdeleniVysledku[]   Nactene zaznamy */
    public $zaznamy;

    public $schvaleno;

    /* @var float */
    public $soucet;


    /* @param int $datum */
    function __construct($datum)
    {
        $this->datum = $datum;
        $this->zaznamy = [];
    }
}


class PrehledRozdeleniDVFormular implements ViewObject
{

    /** @var Database */
    private $database;

    /** @var Rozdeleni */
    public $rozdeleni;
    /* @var Obdobi */
    public $obdobi;

    /* @var DenPrehleduRozdeleniDV[] */
    var $dataList;
    /** @var bool   Urcuje, jestli se zobrazuje i kod casove slozky - nezobrazuje se, pokud je jen jedna a zaroven neni u zaznamu nejaka jina */
    var $zobrazitCS = true;
    /** @var string     Kod vychozi casove slozky - tato slozka se nezobrazuje, pokud je na vyber jen jedna */
    var $vychoziCS;

    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param Obdobi $obdobi
     */
    function __construct($database, $rozdeleni, $obdobi)
    {
        $this ->database = $database;
        $this ->obdobi = $obdobi;

        $this ->kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI);
        $this ->rozdeleni = $rozdeleni;
    }


    private function inicializujData()
    {
        $this ->dataList = [];

        for ($datum = $this ->obdobi ->zacatek; $datum <= $this ->obdobi ->konec; $datum = DateUtils::incDayStart($datum, 1)) {
            $denrozdeleni = new DenPrehleduRozdeleniDV($datum);
            $this ->dataList[] = $denrozdeleni;
        }
    }

    /**
     * @param int $datum
     * @return DenRozdeleniDV
     */
    private function vratZaznamProDen($datum)
    {
        foreach ($this ->dataList as $den)
            if ($den ->datum == $datum) return $den;

        return null;
    }


    private function nactiSchvaleni()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, Schvaleno ' .
            ' FROM SchvaleniRozdeleniDoch ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($this ->rozdeleni ->typ) .
            ' AND Rozdeleni = ' . $params ->addParameter($this ->rozdeleni ->kod) .
            ' AND Den >= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi ->zacatek)) .
            ' AND Den <= ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi ->konec));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $den = $this ->database ->stringToTimestamp($row[0]);
            $this ->dataList[(int)date('j', $den) - 1] ->schvaleno = (int)$row[1];
        }
        $cursor ->close();
    }

    private function nactiDataDB()
    {
        $this->inicializujData();

//        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
//            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
//            $this ->obdobi ->zacatek, $this ->obdobi ->konec);

        $items = ZaznamRozdeleniVysledku::nactiZaznamyRozdeleni($this->database, $this->rozdeleni ->typ, $this->rozdeleni ->kod, $this->obdobi->zacatek, $this->obdobi->konec);
        foreach ($items as $item) {
            $denRozdeleni = $this->vratZaznamProDen($item->den);
            if ($denRozdeleni) {
                //$item ->js_id = 'zaznamrozdeleniDV_' . $itemid;
                //$itemid++;

                $denRozdeleni->zaznamy[] = $item;
                $denRozdeleni->soucet += $item->hodiny;
            }
        }

        $this ->nactiSchvaleni();
    }



    public function nactiData()
    {
        $cs = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
        $this ->zobrazitCS = count($cs) > 1;
        $this ->vychoziCS = (count($cs) == 1)  ?  $cs[0] : '';
        $this ->nactiDataDB();
    }


    public function zpracujPost()
    {
        $this ->nactiData();

        // nacte POST do dat
        foreach ($this ->dataList as $den) {
            $name = 'schvaleno' . date('j', $den ->datum);
            $den ->schvaleno = (isset($_POST[$name])  &&  $_POST[$name]) ? 1 : 0;
        }

        foreach ($this ->dataList as $den) {
            $rec = new RonPortalDBRecord($this ->database, 'SchvaleniRozdeleniDoch');
            $rec ->addField('Den', 'timestamp', true, 0, $den ->datum);
            $rec ->addField('KodTypuRozdeleni', 'int', true, 0, $this ->rozdeleni ->typ);
            $rec ->addField('Rozdeleni', '', true, 30, $this ->rozdeleni ->kod);
            $rec ->addField('Schvaleno', 'int', false, 0, $den ->schvaleno ? 1 : 0 );

            $params = $this ->database ->createParameters();
            $rec ->setWhere(
                'Den = ' . $params ->addParameter($this ->database ->dateToParam($den ->datum)) .
                ' AND KodTypuRozdeleni = ' . $params ->addParameter($this ->rozdeleni ->typ) .
                ' AND Rozdeleni = ' . $params ->addParameter($this ->rozdeleni ->kod),
                $params
            );

            if ($rec ->loadRecord()) {
                $rec ->updateRecord();
            } else if ($den ->schvaleno) {
                // novy zaznam vklada jen kdyz je schvaleno
                $rec->insertRecord();
            }
        }
    }


    public function render()
    {
        if (!isset($this ->dataList)) {
            throw new DebugException('Chyba ve funkci PrehledRozdeleniDVFormular::Show - nejsou nactena data!');
        }

        require 'ronportal/editace/view_rozdeleni_dv_prehled.php';
    }
}
