<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;
use Ron\ArrayUtils;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Dochazka\DenDochazky;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\DenniVysledekR;
use RonPortal\Dochazka\DokumentDochazky;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\MzdovaPolozka;
use RonPortal\Dochazka\OsobaDochazky;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\Dochazka\VysledekKontroly;
use RonPortal\Dochazka\ZaznamAbsence;
use RonPortal\Dochazka\ZaznamZpracovaneDochazky;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;
use RonPortal\RozdeleniUtils;
use RonPortal\User;

/**
 * Polozka denniho prehledu zpracovane - napr. denni vysledek, planovana absence, graficka dochazka, ...
 * Class PolozkaPrehleduZpracovane
 * @package Ron\RonPortal
 */
class PolozkaPrehleduZpracovane
{
    /** @var string Co ma zobrazit */
    public $typ;

    // obecne
    public $width;
    public $height;
    public $cssClass;
    public $style;
//    public $minHeight;

    // denni vysledky
    public $kod;
    public $jednotka;

    public $format;
    public $titleFormat;

    public $zobrazitSoucet = true;

    /**
     * Nacte nastaveni z pole nacteneho z XML
     * @param array $settings
     */
    public function loadSettings($settings)
    {
        foreach ($settings['#attr'] as $name => $value) {
            switch ($name) {
                case 'width' :
                case 'height' :
                    $this ->{$name} = (int)$value;
                    break;

                case 'class' :
                    $this ->cssClass = $value;
                    break;

                case 'styl' :
                    $this ->style = $value;
                    break;

                case 'soucet' :
                    $this ->zobrazitSoucet = (int)$value;

                default :
                    $this ->{$name} = $value;
            }
        }

        // doplni defaultni hodnoty
        switch ($this ->typ) {
            case 'DOCH_G' :
                if (!isset($this ->width)) $this ->width = 30;
                break;
        }

    }
}


/**
 * Nastaveni jedne bunky dne dochazky - muze zobrazovat vice polozek
 * Class NastaveniDneDochazky
 * @package Ron\RonPortal
 */
class NastaveniDneDochazky
{
    /** @var PolozkaPrehleduZpracovane[] */
    public $data = [];

    public $cssClass;
    public $style;
    /** @var string     podle ceho se ma bunka obarvit - absence/dochazka/schvaleni/kontrola */
    public $obarvit;

    /** @var string     nadpis pred sloupcem */
    public $nadpis;
    /** @var string     html titulek */
    public $title;

    /** @var string     nadpis ve filtru - uzivatel si muze tuto polozku vypnout */
    public $filtr;
    /** @var string     jedinecny kod pro zapinani a zapamatovani filtru */
    public $kodFiltru;
    /** @var bool       nastavi se podle filtru, prav, ale muze slouzit i pro vypnuti */
    public $zobrazit = true;
    /** @var string     Kod profil uzivatelu, kterym se sloupec zobrazi (pokud neni nastaven, zobrazi se vsem) */
    public $profil;


    /**
     * Nacte nastaveni z pole nacteneho z XML
     * @param array $settings
     */
    public function loadSettings($settings)
    {
        foreach ($settings['#attr'] as $name => $value) {
            switch ($name) {
                case 'data' :
                    break;

                case 'class' :
                    $this ->cssClass = $value;
                    break;

                case 'kodfiltru' :
                    $this ->kodFiltru = $value;
                    break;

                case 'filtr' :
                    $this ->filtr = TextManager::_($value);
                    break;

                case 'zobrazit' :
                    $this ->{$name} = (int)$value;
                    break;

                case 'nadpis' :
                case 'title' :
                    $this ->{$name} = TextManager::_($value);
                    break;

                default : // style, kodFiltru, profil
                    $this ->{$name} = $value;
                    break;
            }
        }

        if (isset($settings['data'])) {
            foreach ($settings['data'] as $nastaveniPolozky) {
                $polozka = new PolozkaPrehleduZpracovane();
                $polozka ->loadSettings($nastaveniPolozky);
                $this ->data[] = $polozka;
            }
        }

        foreach ($this ->data as $polozka) {
            if ($polozka ->typ == 'DOCH_G'  &&  !$this ->cssClass) $this ->cssClass = 'graficka';
        }
    }
}



class SloupecPrehleduZpracovane extends SloupecMesicniDochazky
{
    public $kodProRazeni;

    public function nastavProPrehled()
    {
        switch ($this ->typ) {
            case 'UZIVATELSKA' :
            case 'UZIVATELSKAPP' :
                $this ->kodProRazeni = $this ->typ . $this ->index;
                break;

            case 'SCHVALENI' :
                $this ->kodProRazeni = $this ->typ . ($this ->uroven ?: '');
                break;

            case 'MP' :
            case 'MV' :
                $this ->kodProRazeni = $this ->typ . $this ->kod;
                break;

            case 'ROZDIMG' :
            case 'FOTO' :
            case 'SQL' :
                $this ->kodProRazeni = null;
                break;

            default :   // JMENOPRIJMENI, JMENO, PRIJMENI, ...
                $this ->kodProRazeni = $this ->typ;
        }
    }
}



class ZpracovavanaOsoba extends OsobaDochazky
{
    /** @var DenDochazky[] */
    public $data = null;

    /** @var ZaznamZpracovaneDochazky[] */
    public $zaznamyDochazky = null;
    /** @var DenniVysledek[] */
    public $denniVysledky = null;
    /** @var DenniVysledekR[] */
    public $denniVysledkyR = null;
    /** @var MesicniVysledek[] */
    public $mesicniVysledky = null;
    /** @var MzdovaPolozka[] */
    public $mzdovePolozky = null;

    /** @var PrirazeniKalendare[] */
    public $kalendare = null;
    /** @var SmenaProDen[] */
    public $smeny = null;

    /** @var ZaznamAbsence[] */
    public $absence = null;

    /** @var VysledekKontroly[] */
    public $kontroly = null;
    /** @var DokumentDochazky[] */
    public $dokumenty = null;

    /** @var mixed  Hodnota, podle ktere se osoby seradi (napr hodnota mesicniho vysledku) */
    public $hodnotaProRazeni = null;

    /** @var mixed  Hodnoty pro SQL sloupce. Pole je indexovano podle sloupcu a kazda hodnota je pole nactene pomoci SloupecDochazky::loadSQL  */
    public $sloupceSQL = [];


    public function najdiDenniVysledek($datum, $typCasoveSlozky)
    {
        if (!$this ->denniVysledky) return null;

        /**
         * @param int $index
         * @param DenniVysledek $vysledek
         * @return int
         */
        $compare = function($index, $vysledek) use ($datum, $typCasoveSlozky) {
            if ($vysledek ->den > $datum) return -1;
            if ($vysledek ->den < $datum) return 1;

            return strcasecmp($typCasoveSlozky, $vysledek ->typCasoveSlozkyMzdy);
        };

        $index = ArrayUtils::findInSortedArray($compare, $this ->denniVysledky);
        return isset($index) ? $this ->denniVysledky[$index] : null;
    }
}



class PrehledZpracovaneDochazky implements ViewObject
{
    /** @var Database */
    public $database;

    /** @var  SloupecPrehleduZpracovane[] */
    public $sloupce = [];

    /** @var NastaveniDneDochazky[] */
    public $nastaveniDne = [];

    /** @var int    Priznak, jestli se ma zobrazit soucet dennich vysledku 0=ne, 1=ano, 2=ano na zacatku */
    public $zobrazitSoucet = 1;

    public $maximalniPocetOsob = 100;
    public $seznamOsobOmezen = false;

    /** @var int    Jestli se ma zobrazit odkaz na zpracovanou dochazku (0=ne, 1=ve stejnem okne (default), 2=v novem pojmenovanem okne, 3=v novem okne) */
    var $odkazNaZpracovanou = 1;

    /** @var LinksList */
    var $filterLinks;

    /** @var Obdobi */
    var $obdobi;
    /** @var Rozdeleni */
    var $rozdeleni;
    /** @var int    Urcuje, jestli se nactou osoby vcetne podrizenych */
    protected $vcetnePodrizenychRozdeleni = 1;

    /** @var  ZpracovavanaOsoba[] */
    var $osoby;

    /** @var string     Kod sloupce, podle ktereho se radi */
    var $order;
    /** @var bool       Priznak, ze se ma radit sestupne */
    var $orderDesc;
    /** @var int        Index sloupce, podle ktereho se radi */
    var $orderCol;

    /** @var string  CSS zadany primo v XML pomoci tagu <style> */
    var $style;
    /** @var string  Javascript zadany primo v XML pomoci tagu <script> */
    var $script;

    /**
     * Priznaky, jestli se maji nacitat konkretni data - podle sloupcu
     * @var bool
     */
    private $nacitatMV = false;
    private $nacitatMP = [];  // seznam kodu MP, ktere se maji nacitat
    private $nacitatDVR = []; // seznam kodu casovych slozek, pro ktere se maji nacitat denni vysledky R
    private $nacitatRozdeleni = []; // seznam typu rozdeleni, ktere se maji nacitat

    /** @var bool   Priznak, jesstli se maji nacitat kalendare */
    private $nacitatKalendare = false;
    private $nacitatSmenyKalendare = [];

    /** @var int    Urcuje, ke kteremu datu se bude prirazovat rozdeleni - viz OsobaDochazky::vratZakladniSQL */
    private $vyberPPVObdobi = 0;

    /** @var SmenaProDen[][]    Asociativni pole seznamu smen pro jednotlive kalendare (cache) */
    private $smenyProKalendare = [];

    /**
     * PrehledZpracovaneDochazky constructor.
     * @param $database
     */
    public function __construct($database)
    {
        $this->database = $database;
        $this ->filterLinks = new LinksList();
    }


    public function nactiNastaveni($settingsStr)
    {
        $loader = new XmlLoader();
        $loader->convertFromUtf8 = true;
        $nastaveni = $loader->readXml($settingsStr);

        $sloupecDenni = null;
        $jeSloupecOsoba = false;

        // Nacte nastaveni sloupcu. Pro prvni polozku "dendochazky" vytvori sloupec s typem 'DEN'
        foreach ($nastaveni['#children'] as $child) {
            $sloupec = null;

            if ($child['name'] == 'dendochazky') {
                if (!$sloupecDenni) {
                    // vytvori jen jeden sloupec s dennimi daty
                    $sloupec = new SloupecPrehleduZpracovane();
                    $sloupec->typ = 'DEN';
                    $sloupecDenni = $sloupec;
                }

                $den = new NastaveniDneDochazky();
                $den->loadSettings($child['element']);
                $this->nastaveniDne[] = $den;

                if ($den ->kodFiltru) {
                    $this ->filterLinks ->addOrUpdateItem($den ->kodFiltru, $den ->filtr); // , $den ->filtrGroup);
                }
            } else if ($child['name'] == 'sloupec') {
                $sloupec = new SloupecPrehleduZpracovane();
                $sloupec->loadSettings($child['element']['#attr']);
                $sloupec->nastavProPrehled();
                if ($sloupec ->typ == 'JMENO' || $sloupec ->typ == 'PRIJMENI'  ||  $sloupec ->typ == 'JMENOPRIJMENI') {
                    $jeSloupecOsoba = true;
                }
            } else if ($child['name'] == 'style') {
                $this ->style = $child['element']['#text'];
            } else if ($child['name'] == 'script') {
                $this ->script = $child['element']['#text'];
            }

            if ($sloupec) {
                if ($sloupec ->filtrKey) {
                    $this ->filterLinks ->addOrUpdateItem($sloupec ->filtrKey, $sloupec ->filtrCaption, $sloupec ->filtrGroup);
                }

                $this->sloupce[] = $sloupec;
            }
        }

        foreach ($nastaveni['#attr'] as $attrName => $value) {
            switch ($attrName) {
                case 'soucet' :
                    $this ->zobrazitSoucet = (int)$value;
                    break;

                case 'proklikzpracovana' :
                    $this ->odkazNaZpracovanou = (int)$value;
                    break;

                case 'datumpp' :
                    $this ->vyberPPVObdobi = (int)$value;
                    break;

                case 'maximalnipocetosob' :
                    $this ->maximalniPocetOsob = (int)$value;
                    break;
            }
        }

        if (!$jeSloupecOsoba) {
            $sloupec = new SloupecPrehleduZpracovane();
            $sloupec ->loadSettings(['typ' => 'JMENOPRIJMENI', 'nadpis' => TextManager::_('Jmeno'), 'class' => 'jmeno', 'proklikzpracovana' => $this ->odkazNaZpracovanou]);
            $sloupec ->nastavProPrehled();
            array_unshift($this ->sloupce, $sloupec);

            $sloupec = new SloupecPrehleduZpracovane();
            $sloupec ->loadSettings(['typ' => 'OSOBNICISLO', 'nadpis' => TextManager::_('Osobni_cislo'), 'class' => 'osobnicislo']);
            $sloupec ->nastavProPrehled();
            array_unshift($this ->sloupce, $sloupec);
        }
    }


    public function skryjSloupcePodlePrav()
    {
        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->zobrazit && $sloupec->profil) {
                // vypne sloupec, pokud je pro jiny profil uzivatele
                $sloupec->zobrazit = User::jeVProfilu($sloupec ->profil);
            }

            if ($sloupec ->zobrazit) {
                // vypne sloupec, pokud uzivatel nema prislusna prava
                if ($sloupec ->prava  &&  !$sloupec ->prava ->jsouNejakePrava()) {
                    // uzivatel nema zadna prava na tento sloupec => vypne ho
                    $sloupec ->zobrazit = false;
                }
            }

            if ($sloupec ->zobrazit  &&  $sloupec ->condition) {
                $sloupec ->zobrazit = $sloupec ->evaluateCondition($this ->obdobi ->zacatek);
            }
        }

        foreach ($this->nastaveniDne as $nastaveniDne) {
            if ($nastaveniDne->profil  &&  $nastaveniDne ->zobrazit) {
                $nastaveniDne ->zobrazit = User::jeVProfilu($nastaveniDne->profil);
            }
        }
    }


    /**
     * Skryje sloupce, ktere si uzivatel vypnul pomoci rychlych filtru
     */
    public function skryjVypnute()
    {
        if ($this ->filterLinks ->isEmpty()) {
            return;
        }

        $vybraneSloupce = NastaveniUzivatele::getUserSettingsItem($this->database, 'prehledzpracovane_sloupce');
        $this ->filterLinks ->loadSettingString($vybraneSloupce);
        $this ->filterLinks ->loadGet();

        foreach ($this->nastaveniDne as $nastaveniDne) {
            if ($nastaveniDne->kodFiltru) {
                if ($nastaveniDne ->zobrazit) {
                    $nastaveniDne ->zobrazit = $this ->filterLinks ->itemIsActive($nastaveniDne ->kodFiltru);
                }
            }
        }

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->filtrKey && $sloupec ->zobrazit) {
                $sloupec ->zobrazit = $this ->filterLinks ->itemIsActive($sloupec ->filtrKey);
            }
        }

        $vybraneNew = $this ->filterLinks ->getSettingsString();
        if ($vybraneNew != $vybraneSloupce) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'prehledzpracovane_sloupce', $vybraneNew, '');
        }
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @throws \Exception
     */
    public function nactiDataOsoby($osoba)
    {
        $osoba->data = DenDochazky::seznamDnu($this->obdobi->zacatek, $this->obdobi->konec);
        $osoba->zaznamyDochazky = DenDochazky::nactiZpracovanouDochazku($this->database, $osoba->data, $osoba, false); // nacistPoznamky = false

        $osoba->denniVysledky = DenniVysledek::nactiDenniVysledky($this->database, $osoba, $this->obdobi->zacatek, $this->obdobi->konec);

        $osoba->kontroly = VysledekKontroly::nactiVysledkyKontrol($this->database, $osoba, $this->obdobi->zacatek, $this->obdobi->konec);

        $osoba->dokumenty = DokumentDochazky::nactiDokumentyOsoby($this->database, $this->obdobi->zacatek, $this->obdobi->konec, $osoba->idPP);

        $osoba->absence = ZaznamAbsence::nactiAbsenceOsoby($this->database, $osoba, $this->obdobi->zacatek, $this->obdobi->konec);

        if ($this ->nacitatMV) {
            $osoba->mesicniVysledky = MesicniVysledek::nactiMesicniVysledky($this->database, $osoba ->idPP, $this->obdobi->zacatek);
        }

        if ($this ->nacitatMP) {  // nacitatMP obsahuje seznam kodu mzdovych polozek
            $osoba ->mzdovePolozky = MzdovaPolozka::nactiMzdovePolozky($this ->database, $osoba, $this ->obdobi ->zacatek, $this ->nacitatMP);
        }

        if ($this ->nacitatDVR) {
            $osoba ->denniVysledkyR = DenniVysledekR::nactiDenniVysledkyR($this ->database, $osoba, $this ->obdobi ->zacatek, $this ->obdobi ->konec, $this ->nacitatDVR);
        }

        if ($this ->nacitatRozdeleni) {
            $osoba ->rozdeleni = RozdeleniOsob::nactiRozdeleniOsoby($this ->database, $osoba, $this ->nacitatRozdeleni, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        }

        if ($this ->nacitatKalendare  ||  $this ->nacitatSmenyKalendare) {
            $osoba ->kalendare = PrirazeniKalendare::nactiKalendarProObdobi($this ->database, $osoba ->idPP, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
        }

        if ($this ->nacitatSmenyKalendare) {
            // prozatim podle prvniho prirazeneho kalendare
            if ($osoba ->kalendare) {
                if ($osoba ->kalendare[0] ->dochazkaPodlePlanKapacit) {
                    $osoba ->smeny = SmenaProDen::nactiSmenyPodleKapacit($this ->database, $osoba, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
                } else {
                    $osoba ->smeny = SmenaProDen::nactiSmenyPodleKalendare($this ->database, $osoba, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
                }
            }
        }

        // TODO: smeny podle kalendare, podle kapacit, sql

        if (isset($this ->orderCol)) {
            $osoba ->hodnotaProRazeni = $this ->sloupce[$this ->orderCol] ->vratHodnotu($osoba, $this ->obdobi, $this ->rozdeleni);
        }

        foreach ($this ->sloupce as $index => $sloupec) {
            if ($sloupec ->typ == 'SQL') {
                $hodnota = $sloupec ->loadSQLProOsobu($this ->database,
                    $osoba, $this ->obdobi,
                    $this ->rozdeleni ? $this ->rozdeleni ->typ : null,
                    $this ->rozdeleni ? $this ->rozdeleni ->kod : null);

                $osoba ->sloupceSQL[$index] = $hodnota;
                // TODO: nastavit $osoba ->hodnotaProRazeni a umoznit razeni?
            }
        }

    }


    public function nastavNacitaniDat()
    {
        $this ->nacitatMP = [];
        $this ->nacitatKalendare = false;

        foreach ($this ->sloupce as $sloupec) if ($sloupec ->zobrazit) {
            switch ($sloupec ->typ) {
                case 'MV' :
                case 'PMV' :
                    $this ->nacitatMV = true;
                    break;

                case 'MP' :
                    $this ->nacitatMP[] = $sloupec ->kod;
                    break;

                case 'ROZD' :
                case 'ROZDP' :
                case 'ROZDIMG' :
                    $this ->nacitatRozdeleni[$sloupec ->kodTypuRozdeleni] = $sloupec ->kodTypuRozdeleni;
                    break;

                case 'KAL' :
                    $this ->nacitatKalendare = true;
                    break;
            }
        }

        $this ->nacitatDVR = [];
        foreach ($this ->nastaveniDne as $nastaveniDne) {
            foreach ($nastaveniDne ->data as $polozka) {
                if ($polozka ->typ == 'DVR') {
                    $this ->nacitatDVR[] = $polozka ->kod;
                }
                if ($polozka ->typ == 'DV') {
                    $this ->nacitatMV = true;   // pro soucty
                }

                if ($polozka ->typ == 'SMENA') {
                    $this ->nacitatSmenyKalendare = true;
                }
            }
        }
    }


    private function getOrderBy()
    {
        // zjisti kod sloupce pro razeni - podle GET nebo podle posledni hodnoty ulozene v session
        $orderCodeOriginal = RonPortalSession::getValue('prehledZpracovaneOrder');
        $desc = $descOriginal = $orderCodeOriginal && $orderCodeOriginal[0] == '-';
        if ($descOriginal) {
            $orderCodeOriginal = substr($orderCodeOriginal, 1);
        }
        $orderCode = $orderCodeOriginal;

        if (isset($_GET['order'])  && $_GET['order']) {
            $orderCode = $_GET['order'];
            $desc = $orderCode  &&  $orderCode[0] == '-';
            if ($desc) {
                $orderCode = substr($orderCode, 1);
            }
        }

        $orderCodeOriginal = strtoupper($orderCodeOriginal ?? '');
        $orderCode = strtoupper($orderCode ?? '');

        // zkontroluje, jestli takovy sloupec existuje a je zobrazeny
        $indexSloupce = null;
        $indexSloupcePuvodni = null;

        for ($i = 0;  !isset($indexSloupce)  &&  $i < count($this ->sloupce);  $i++) {
            if ($this ->sloupce[$i] ->zobrazit) {
                if ($orderCode  &&  $this->sloupce[$i]->kodProRazeni == $orderCode) {
                    $indexSloupce = $i;
                } else if ($orderCodeOriginal  &&  $this->sloupce[$i]->kodProRazeni == $orderCodeOriginal) {
                    $indexSloupcePuvodni = $i;
                }
            }
        }

        $this ->order = 'JMENOPRIJMENI';

        if ($orderCode == 'JMENOPRIJMENI'  ||  $orderCode == 'PRIJMENI'  ||  $orderCode == 'OSOBNICISLO') {
            $this->order = $orderCode;
            $this->orderDesc = $desc;
            $_SESSION['prehledZpracovaneOrder'] = ($desc ? '-' : '') . $this->order;
        } else if (isset($indexSloupce)) {
            $this ->order = $this ->sloupce[$indexSloupce] ->kodProRazeni;
            $this ->orderDesc = $desc;
            $this ->orderCol = $indexSloupce;
            $_SESSION['prehledZpracovaneOrder'] = ($desc ? '-' : '') . $this ->order;
        } else if (isset($indexSloupcePuvodni)) {
            $this ->order = $this ->sloupce[$indexSloupcePuvodni] ->kodProRazeni;
            $this ->orderDesc = $descOriginal;
            $this ->orderCol = $indexSloupcePuvodni;
            $indexSloupce = $indexSloupcePuvodni;
        }

        $this ->order = strtoupper($this ->order);
        $orderBy = '';

        $descSQL = $this ->orderDesc ? ' DESC' : '';

        if (isset($indexSloupce)) {
            if ($this ->sloupce[$indexSloupce] ->typ == 'UZIVATELSKA') {
                $orderBy = 'Osoba.Uzivatelska' . $this ->sloupce[$indexSloupce] ->index . ' ' . $descSQL . ', '; // predradi pred prijmeni
            } else if ($this ->sloupce[$indexSloupce] ->typ == 'UZIVATELSKAPP') {
                $orderBy = 'PracovniPomer.Uzivatelska' . $this ->sloupce[$indexSloupce] ->index . ' ' . $descSQL . ', '; // predradi pred prijmeni
            } else if ($this ->order == 'NASTUP') {
                $orderBy = 'PracovniPomer.DatumNastupu ' . $descSQL . ', ';
            }
        }


        switch ($this ->order) {
            case 'OSOBNICISLO' :
                return "PracovniPomer.OsobniCislo $descSQL, Osoba.Prijmeni $descSQL, Osoba.Jmeno $descSQL";

            case 'JMENO' :
                return "Osoba.Jmeno $descSQL, Osoba.Prijmeni $descSQL, PracovniPomer.OsobniCislo $descSQL";

            case 'JMENOPRIJMENI' :
            case 'PRIJMENI' :
            default :
                return "$orderBy Osoba.Prijmeni $descSQL, Osoba.Jmeno $descSQL, PracovniPomer.OsobniCislo $descSQL";
        }
    }


    private function seradSeznam()
    {
        if (!isset($this ->orderCol)) {
            return;
        }

        switch ($this ->sloupce[$this ->orderCol] ->typ) {
            case 'MV' :
            case 'MP' :
            case 'SCHVALENI' :
                usort($this ->osoby, function($osoba1, $osoba2) {
                    /** @var ZpracovavanaOsoba $osoba1 */
                    /** @var ZpracovavanaOsoba $osoba2 */
                    if ($osoba1 ->hodnotaProRazeni < $osoba2 ->hodnotaProRazeni) {
                        return $this ->orderDesc ? 1 : -1;
                    }

                    if ($osoba1 ->hodnotaProRazeni > $osoba2 ->hodnotaProRazeni) {
                        return $this ->orderDesc ? -1 : 1;
                    }

                    return ($this ->orderDesc ? -1 : 1) * ZpracovavanaOsoba::compare($osoba1, $osoba2);
                });
                break;

            case 'ROZD' :
            case 'ROZDP' :
                usort($this ->osoby, function($osoba1, $osoba2) {
                    $result = StringUtils::compareStrings($osoba1->hodnotaProRazeni, $osoba2->hodnotaProRazeni);
                    if ($result) {
                        return ($this ->orderDesc ? -1 : 1) * $result;
                    }

                    return ($this ->orderDesc ? -1 : 1) * ZpracovavanaOsoba::compare($osoba1, $osoba2);
                });
                break;

        }
    }

    /**
     * @throws \Exception
     */
    public function load(RonPortalFilterForm $filterForm)
    {
        $this->obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $filterForm ->getFilterValue('obdobi'));

        $this ->skryjSloupcePodlePrav();
        $this ->skryjVypnute();

        // filtr rozdeleni
        $this ->nastavNacitaniDat();
        $kodRozdeleni = $filterForm ->getFilterValue('rozdeleni');
        if ($kodRozdeleni !== null) {
            $this->rozdeleni = Rozdeleni::loadRozdeleni($this->database,
                $filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
                $kodRozdeleni);
        }
        $this ->vcetnePodrizenychRozdeleni = $filterForm ->getFilterValue('rozdeleni', 'vcetnePodrizenych');

        $params = $this ->database ->createParameters();

        // Zakladni SQL dotaz
        // TODO: Pokud ma pravo na vsechny a nema zapnuty filtr na rozdeleni, tak se nezobrazuji lidi, kteri nemaji rozdeleni prirazene!
        $sql = ZpracovavanaOsoba::vratZakladniSQL($this->database,
            $this->obdobi, $this ->vyberPPVObdobi, // kdykoliv v obdobi
            $this ->rozdeleni ->typ ?? null, $this ->rozdeleni ->kod ?? null, $this ->vcetnePodrizenychRozdeleni,
            false,
            $params);

        // filtr podle prav uzivatele
        $filtr = Prava::sqlFiltrPodlePrav(
            Prava::vratPrava('prohlizeni'),
            'PracovniPomer',
            $this->obdobi->zacatek, $this->obdobi->konec,
            false,
            $params);

        foreach ($filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->name !== 'rozdeleni'  &&  $rozdeleniFilter ->getValue() !== null) { // filtr na rozdeleni uz je pouzit v zakladnim SQL dotazu
                $sqlFilter = RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
                    'PracovniPomer',
                    $rozdeleniFilter ->getValue('typRozdeleni'), $rozdeleniFilter ->getValue(),
                    $this->obdobi->zacatek, $this->obdobi->konec,
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params
                );

                if ($sqlFilter) {
                    $filtr .=
                        ($filtr ? ' AND ' : '') .
                        '(' . $sqlFilter . ')';
                }
            }
        }

//        // nacte seznam osob
//        if ($filterForm ->kodTypuRozdeleni2  &&  $filterForm ->kodRozdeleni2) {
//            // filtr podle rozdeleni 2
//            $filtr2 = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PracovniPomer',
//                $filterForm ->kodTypuRozdeleni2, $filterForm ->kodRozdeleni2,
//                $this->obdobi->zacatek, $this->obdobi->konec, $filterForm ->vcetnePodrizenychRozdeleni2);
//        } else {
//            $filtr2 = '';
//        }


//        $filtr = SQLUtils::concatenateWherePartsOfSql([$filtrPrava, $filtr2]);


        if ($filtr) {
            $sql .= ' AND (' . $filtr . ')';
        }

        $sql .=
            ' ORDER BY ' . $this ->getOrderBy();

        $this->osoby = ZpracovavanaOsoba::nactiSeznamOsobSQL($this->database, new SqlStatement($sql, $params), null); // , $filtr

        if ($this ->maximalniPocetOsob  &&  count($this ->osoby) > $this ->maximalniPocetOsob) {
            array_splice($this ->osoby, $this ->maximalniPocetOsob);
            $this ->seznamOsobOmezen = true;
        }

        foreach ($this->osoby as $osoba) {
            $this->nactiDataOsoby($osoba);
        }

        $this ->seradSeznam();
    }


    protected function filtr()
    {
        $this ->filterLinks ->url = '?menuid=prehledzpracovane&obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek) . '&';
        return $this ->filterLinks ->getHtml();
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $indexDne
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlGraficka($osoba, $indexDne, $polozka)
    {
        $result = '<div class="graficka_timediv" style="width:' . $polozka->width . 'px">';

        foreach ($osoba->data[$indexDne]->zaznamy as $zaznamDochazky) {
            $result .= $zaznamDochazky->htmlGraficka($polozka->width);
        }

        $result .= '</div>';

        return $result;
    }


    /**
     * @param OsobaDochazky $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlSchvaleni($osoba, $datum, $polozka)
    {
        $cssClass = $polozka->cssClass ?: 'schvaleni';
        switch ($polozka->typ) {
            case 'SCHVAL1' :
                if ($osoba->schvaleni->jeSchvaleno($datum, 1)) {
                    $cssClass .= ' schvaleni1';
                }
                break;

            case 'SCHVAL2' :
                if ($osoba->schvaleni->jeSchvaleno($datum, 2)) {
                    $cssClass .= ' schvaleni2';
                }
                break;

            case 'SCHVAL3' :
                if ($osoba->schvaleni->jeSchvaleno($datum, 3)) {
                    $cssClass .= ' schvaleni3';
                }
                break;

            case 'SCHVAL' :
                $uroven = $osoba->schvaleni->nejvyssiUrovenSchvaleni($datum);
                if ($uroven) {
                    $cssClass .= ' schvaleni' . $uroven;
                }
                break;
        }

        $styleStr = $polozka->style ? " style='{$polozka ->style}'" : '';

        return '<div class="' . $cssClass . '" ' . $styleStr . '></div>';
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlDenniVysledek($osoba, $datum, $polozka)
    {
        // najde denni vysledek pro zadanou casovou slozku a datum
        $vysledek = $osoba->najdiDenniVysledek($datum, $polozka->kod);
        $text = $vysledek ? $vysledek->vratText($polozka->jednotka, $polozka ->format) : '';
        if ($text === '') {
            return '';
        }

        $class = $polozka->cssClass ?: 'dennivysledek';
        if ($vysledek->hodiny < 0) {
            $class .= ' minus';
        }

        $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

        return
            "<div class='$class' $styleStr>$text</div>";
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlAbsence($osoba, $datum, $polozka)
    {
        $text = '';

        for ($i = 0; $i < count($osoba->absence) && $osoba->absence[$i]->datumOd <= $datum; $i++) {
            if ($osoba->absence[$i]->datumDo >= $datum) {
                $text .=
                    ($text ? ', ' : '') . $osoba->absence[$i]->html($polozka->format, $polozka->titleFormat, 'span', 'prehledzpracovane');
            }
        }

        $class = $polozka->cssClass ?: 'absence';
        $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

        return $text
            ? "<div class='$class' $styleStr>" . $text . '</div>'
            : '';
    }



    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlSmena($osoba, $datum, $polozka)
    {
        $text = '';

        if ($osoba ->smeny) {
            // findInSortedArray($needle, $array, $options = self::FIND_ANY)
            for ($i = 0; $i < count($osoba->smeny) && $osoba->smeny[$i]->datum <= $datum; $i++) {
                if ($osoba->smeny[$i]->datum == $datum) {
                    $text = $osoba->smeny[$i]->format($polozka->format);
                }
            }
        }

        $class = $polozka->cssClass ?: 'absence';
        $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

        return $text
            ? "<div class='$class' $styleStr>" . $text . '</div>'
            : '';
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlKontroly($osoba, $datum, $polozka)
    {
        $result = '';
        for ($i = 0; $i < count($osoba->kontroly) && $osoba->kontroly[$i]->datum <= $datum; $i++) {
            if (!$osoba->kontroly[$i]->mesicni && $osoba->kontroly[$i]->datum == $datum) {
                $result .= $osoba->kontroly[$i]->htmlPriznak($i);
            }
        }

        $class = $polozka->cssClass ?: 'kontroly';
        $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

        return $result
            ? "<div class='$class' $styleStr>" . $result . '</div>'
            : '';
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param int $datum
     * @param PolozkaPrehleduZpracovane $polozka
     * @return string
     */
    protected function htmlDokumenty($osoba, $datum, $polozka)
    {
        $result = '';
        for ($i = 0; $i < count($osoba->dokumenty) && $osoba->dokumenty[$i]->datum <= $datum; $i++) {
            if ($osoba->dokumenty[$i]->datum == $datum) {
                $result .= $osoba->dokumenty[$i]->getDownloadHtml($this->obdobi->zacatek);
            }
        }

        $class = $polozka->cssClass ?: 'dokumenty';
        $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

        return $result
            ? "<div class='$class' $styleStr>" . $result . '</div>'
            : '';
    }


    /**
     * @param NastaveniDneDochazky $nastaveniDne
     * @param ZpracovavanaOsoba $osoba
     * @param int $indexDne
     * @return string
     */
    protected function getStyle($nastaveniDne, $osoba, $indexDne)
    {
        $datum = $osoba ->data[$indexDne] ->datum;

        if ($nastaveniDne->obarvit == 'kontrola') {
            for ($i = 0; $i < count($osoba->kontroly) && $osoba->kontroly[$i]->datum <= $datum; $i++) {
                if (!$osoba->kontroly[$i]->mesicni && $osoba->kontroly[$i]->datum == $datum) {
                    if ($osoba->kontroly[$i]->barva) {
                        return 'background: ' . GuiUtils::getHtmlColor($osoba->kontroly[$i]->barva) . ' important;';
                    }
                }
            }

            return '';
        }

        if ($nastaveniDne->obarvit == 'absence') {
            for ($i = 0; $i < count($osoba->absence) && $osoba->absence[$i]->datumOd <= $datum; $i++) {
                if ($osoba->absence[$i]->datumDo >= $datum) {
                    if ($osoba->absence[$i]->barvaProVykresleni) {
                        return 'background: ' . GuiUtils::getHtmlColor($osoba->absence[$i]->barvaProVykresleni) . ' !important;';
                    }
                }
            }

            return '';
        }

        if ($nastaveniDne ->obarvit == 'dochazka') {
            $zaznam = ZaznamZpracovaneDochazky::vratNejdelsiOperaci($osoba ->data[$indexDne] ->zaznamy);
            return ($zaznam && $zaznam ->barva)
                ? 'background: ' . $zaznam ->barva . ' !important;'
                : '';
        }

        return '';
    }


    /**
     * Vraci radky dennich vysledku podle rozdeleni
     * @param NastaveniDneDochazky $nastaveniDne
     * @param ZpracovavanaOsoba $osoba
     * @return array
     */
    function vratRadkyDVR($nastaveniDne, $osoba)
    {
        if (!$this ->nacitatDVR) {
            return [];
        }

        // zjisti casove slozky, pro ktere se maji zobrazit data
        $casoveSlozky = [];
        $styly = [];

        foreach ($nastaveniDne ->data as $polozka) {
            if ($polozka ->typ == 'DVR') {
                $casoveSlozky[$polozka ->kod] = $polozka ->kod;
                $styly[$polozka ->kod] = $polozka ->style;
            }
        }

        // nacte data osoby - pro kazde rozdeleni vytvari pole ["popis" => ..., 0 => ..., ...., 31 => ..., "sum" => ..., "style" => ...]
        $result = [];

        foreach ($osoba ->denniVysledkyR as $denniVysledekR) {
            if (in_array($denniVysledekR ->typCasoveSlozkyMzdy, $casoveSlozky)) {
                if (!isset($result[$denniVysledekR ->rozdeleni])) {
                    $result[$denniVysledekR->rozdeleni] = [
                        'popis' => $denniVysledekR->popisRozdeleni,
                        'sum' => 0,
                        'style' => $styly[$denniVysledekR ->typCasoveSlozkyMzdy]
                    ];
                }

                $indexDne = date('j', $denniVysledekR ->den) - date('j', $this ->obdobi ->zacatek);
                $result[$denniVysledekR->rozdeleni][$indexDne] = StringUtils::format('minuty', round($denniVysledekR ->hodiny * 60));
                $result[$denniVysledekR->rozdeleni]['sum'] += $denniVysledekR ->hodiny;
            }
        }

        return $result;
    }


    function cssClassProDen($datum)
    {
        $typDne = Kalendar::vratTypDne($this->database, $datum);
        $className = '';
        if ($typDne & Kalendar::VIKEND) {
            $className = 'vikend';
        }
        if ($typDne & Kalendar::SVATEK) {
            $className .= ($className ? ' ' : '') . 'svatek';
        }

        return $className;
    }


    /**
     * @param NastaveniDneDochazky $nastaveniDne
     * @param ZpracovavanaOsoba $osoba
     * @param array $dataDVR
     * @return string
     */
    protected function htmlSoucet($nastaveniDne, $osoba, $dataDVR)
    {
        $result = '<td class="soucetdv">';
        foreach ($nastaveniDne->data as $polozka) {
            if ($polozka->zobrazitSoucet) {
                $class = $polozka->cssClass ?: 'dennivysledek';

                $text = '';

                switch ($polozka->typ) {
                    case 'DV' :
                        if (isset($osoba->mesicniVysledky[$polozka->kod])) {
                            $text = $osoba->mesicniVysledky[$polozka->kod]->vratText($polozka->jednotka, $polozka->format);
                            if ($osoba->mesicniVysledky[$polozka->kod]->hodiny < 0) {
                                $class .= ' minus';
                            }
                        }
                        break;

                    case 'DVR' :
                        foreach ($dataDVR as $kod => $dataRozdeleni) {
                            $styleStr = $dataRozdeleni['style'] ? ' style="' . $dataRozdeleni['style'] . '"' : '';
                            $result .=
                                "<div $styleStr class='dennivysledekr'>" .
                                (isset($dataRozdeleni['sum']) ? StringUtils::format('minuty', round($dataRozdeleni['sum'] * 60)) : '') .
                                '<br/>';
                        }
                        break;
                }

                $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

                $result .=
                    "<div class='$class' $styleStr>$text</div>";
            }
        }
        $result .= '</td>';

        return $result;
    }


    /**
     * @param NastaveniDneDochazky $nastaveniDne
     * @param ZpracovavanaOsoba $osoba
     * @return string
     */
    protected function denniZaznamyOsoby($nastaveniDne, $osoba)
    {
        if (!$this->nastaveniDne) {
            return '';
        }

        $dataDVR = $this ->vratRadkyDVR($nastaveniDne, $osoba);
        $nadpisDVR = '';
        foreach ($dataDVR as $kod => $dataRozdeleni) {
            $styleStr = $dataRozdeleni['style'] ? ' style="' . $dataRozdeleni['style'] . '"' : '';
            $nadpisDVR .=
                "<div class='dennivysledekr' $styleStr title='" .
                StringUtils::htmlTextConvert($dataRozdeleni['popis']) .
                "'>" . StringUtils::htmlTextConvert($kod) . '</div>';
        }


        $result = '';
        $result .= '<td class="nadpis">' . $nastaveniDne->nadpis . $nadpisDVR . '</td>';

        if ($this ->zobrazitSoucet == 2) {
            $result .= $this ->htmlSoucet($nastaveniDne, $osoba, $dataDVR);
        }

        for ($indexDne = 0; $indexDne < count($osoba->data); $indexDne++) {
            $datum = $osoba->data[$indexDne]->datum;
            $className = $this ->cssClassProDen($datum);
            if ($nastaveniDne->cssClass) {
                $className .= ($className ? ' ' : '') . $nastaveniDne->cssClass;
            }


            $result .= '<td';
            if ($className) {
                $result .= " class='$className'";
            }
            $style = $this->getStyle($nastaveniDne, $osoba, $indexDne) . $nastaveniDne->style;
            if ($style) {
                $result .= " style='{$style}'";
            }
            if ($nastaveniDne ->title) {
                $result .= ' title="' . StringUtils::htmlTextConvert($nastaveniDne ->title) . '"';
            }
            $result .= '>';

            foreach ($nastaveniDne->data as $polozka) {

                switch ($polozka->typ) {
                    case 'DOCH_G' :
                        $result .= $this->htmlGraficka($osoba, $indexDne, $polozka);
                        break;

                    case 'SCHVAL' :
                    case 'SCHVAL1' :
                    case 'SCHVAL2' :
                    case 'SCHVAL3' :
                        $result .= $this->htmlSchvaleni($osoba, $datum, $polozka);
                        break;

                    case 'DV' :
                        $result .= $this->htmlDenniVysledek($osoba, $datum, $polozka);
                        break;

                    case 'DVR' :
                        foreach ($dataDVR as $kod => $dataRozdeleni) {
                            $styleStr = $polozka->style ? "style='{$polozka ->style}'" : '';

                            $result .=
                                "<div class='dennivysledekr' $styleStr>" .
                                (isset($dataRozdeleni[$indexDne]) ? $dataRozdeleni[$indexDne] : '') .
                                '<br/>';
                        }

                        //$result .= $this->htmlRozdeleniDV($osoba, $datum, $polozka);
                        break;

                    case 'ABSENCE' :
                        $result .= $this->htmlAbsence($osoba, $datum, $polozka);
                        break;

                    case 'SMENA' :
                        $result .= $this ->htmlSmena($osoba, $datum, $polozka);
                        break;

                    case 'KONTROLY' :
                        $result .= $this->htmlKontroly($osoba, $datum, $polozka);
                        break;

                    case 'DOC' :
                        $result .= $this->htmlDokumenty($osoba, $datum, $polozka);
                        break;
                }
            }

            $result .= '</td>';
        }

        // soucet
        if ($this ->zobrazitSoucet == 1) {
            $result .= $this ->htmlSoucet($nastaveniDne, $osoba, $dataDVR);
        }

        return $result;
    }


    function htmlHeader()
    {
        $orderUrl = '?menuid=prehledzpracovane&obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek);
        if ($this ->rozdeleni) {
            $orderUrl .= '&rozdeleni=' . StringUtils::htmlTextConvert($this ->rozdeleni ->kod) . '&rozdeleni_vcetne_podrizenych=' . $this ->vcetnePodrizenychRozdeleni;
        }

        $result =
            '<thead class="floating-table-header"><tr>';
//            "<th class='osobnicislo'><a href='{$orderUrl}&order=" . (!$this ->orderDesc  &&  $this ->order == 'OSOBNICISLO' ? '-' : '') . "OSOBNICISLO'>" . TextManager::_('Osobni_cislo') . '</a></th>' .
//            "<th class='jmeno'><a href='{$orderUrl}&order=" . (!$this ->orderDesc  &&  $this ->order == 'JMENOPRIJMENI' ? '-' : '') . "JMENOPRIJMENI'>" . TextManager::_('Jmeno') . '</th>';


        $dnyZobrazeny = false;

        foreach ($this ->sloupce as $sloupec) if ($sloupec ->typ  && $sloupec ->zobrazit) {
            if ($sloupec ->typ == 'DEN'  &&  !$dnyZobrazeny) {
                $result .= '<th></th>';  // nadpis radku

                if ($this ->zobrazitSoucet == 2) {
                    $result .= '<th></th>';  // soucet na zacatku
                }

                $datum = $this->obdobi->zacatek;
                while ($datum <= $this->obdobi->konec) {
                    $className = $this ->cssClassProDen($datum);
                    $classStr = $className ? "class='$className'" : '';

                    $result .=
                        "<th $classStr>" .
                        '<div class="dayname">' . TextManager::_('Day_short_' . date('w', $datum)) . '</div>' .
                        '<div class="daynumber">' . date('j', $datum) . '</div>' .
                        '</th>';

                    $datum = DateUtils::incDay($datum);
                }

                if ($this ->zobrazitSoucet == 1) {
                    $result .= '<th></th>';  // soucet
                }
            } else {
                $cssClass = $sloupec ->getCssClass(null, null);
                $classStr = $cssClass ? "class='$cssClass'" : '';
                $titleStr = $sloupec ->title ? 'title="' . StringUtils::htmlTextConvert($sloupec ->title) . '"' : '';

                $href = $sloupec ->kodProRazeni
                    ? $orderUrl . '&order=' . (!$this ->orderDesc  &&  $this ->order == $sloupec ->kodProRazeni ? '-' : '') . $sloupec ->kodProRazeni
                    : '';

                $orderIcon = '';
                if ($sloupec ->kodProRazeni  &&  $sloupec ->kodProRazeni == $this ->order) {
                    $orderIcon = '<i class="inline-glyph fa glyph-sort-' . ($this ->orderDesc ? 'down' : 'up') . '"></i>';
                }

                $result .=
                    "<th $classStr $titleStr>" .
                    ($href ? "<a href='$href'>" : '') .
                    $sloupec ->nadpis .
                    ($href ? '</a>' : '') .
                    $orderIcon .
                    '</th>';
            }
        }



        $result .= "</tr></thead>\n";

        return $result;
    }


    /**
     * @param int $indexSloupce
     * @param ZpracovavanaOsoba $osoba
     * @param int $rowspan
     * @return mixed
     */
    public function htmlSloupec($indexSloupce, $osoba, $rowspan)
    {
        $sloupec = $this ->sloupce[$indexSloupce];

        if ($sloupec ->typ == 'SQL') {
            $text = '';
            if ($osoba ->sloupceSQL[$indexSloupce]) {
                $text = $osoba ->sloupceSQL[$indexSloupce]['text'];
            }

            return $sloupec ->htmlForText($text, $osoba, $this ->obdobi, $rowspan);
        }

        return $sloupec->html($osoba, $this->obdobi, $this->rozdeleni, $rowspan);
    }

    /**
     * @return string
     * @throws \Exception
     */
    public function vratObsahHtml()
    {
        $result = '';

        if ($this ->style) {
            $result .= '<style>' . $this ->style . '</style>';
        }

        if ($this ->script) {
            $result .= '<script>' . $this ->script . '</script>';
        }

        if ($this ->seznamOsobOmezen) {
            $result .= '<div class="warningmsg">' . TextManager::_('Seznam_prehledu_zpracovane_omezen') . '</div>';
        }

        $result .=
            $this ->filtr() .
            "<table class='prehledzpracovane browser middle keyInputMove fixedCols oddEvenBody bodyOver bodySelect' data-ron-fixed-cols='2'>" .
            $this ->htmlHeader();

        $rowspan = 0;
        $zobrazeneNastaveniDne = [];
        foreach ($this ->nastaveniDne as $nastaveniDne) if ($nastaveniDne ->zobrazit) {
            $zobrazeneNastaveniDne[] = $nastaveniDne;
            $rowspan++;
        }
        $rowspanStr = ($rowspan > 1) ? 'rowspan="' . $rowspan . '"' : '';

        foreach ($this ->osoby as $osoba) {
            // 0 = ne, 1 = ve stejnem okne, 2=v novem pojmenovanem okne, 3=v novem okne
            $zobrazitOdkaz = true;
            switch ($this->odkazNaZpracovanou) {
                case 3 :
                    $targetStr = 'target="_blank"';
                    break;

                case 2 :
                    $targetStr = 'target="zpracovanadochazkawin"';
                    break;

                case 0 :
                    $zobrazitOdkaz = false;
                    break;

                case 1 :
                default :
                    $targetStr = '';
                    break;
            }

            $result .=
                "<tbody>" .
                "<tr data-ron-idpp='{$osoba ->idPP}' data-rowspan-index='0' data-rowspan-count='$rowspan'>";
//                "<td class='osobnicislo' {$rowspanStr}>" . StringUtils::xmlTextConvert($osoba->osobniCislo) . '</td>' .
//                "<td class='jmeno' {$rowspanStr}>" .
//                ($zobrazitOdkaz ? "<a $targetStr href='?menuid=zpracovanadochazka&amp;osoba={$osoba->idPP}&amp;obdobi=" . DateUtils::dateToUrlString($this->obdobi->zacatek) . "'>" : '') .
//                StringUtils::xmlTextConvert($osoba->jmenoPrijmeni) .
//                ($zobrazitOdkaz ? '</a>' : '').
//                '</td>';

            foreach ($this ->sloupce as $index => $sloupec) {
                if ($sloupec->typ && $sloupec->zobrazit) {
                    if ($sloupec->typ == 'DEN') {
                        if (isset($zobrazeneNastaveniDne[0])) {
                            $result .= $this->denniZaznamyOsoby($zobrazeneNastaveniDne[0], $osoba);
                        }
                    } else {
                        $result .= $this->htmlSloupec($index, $osoba, $rowspan);
                    }
                }
            }

            $result .= "</tr>\n\n";


            // zobrazi dalsi radky pro dny dochazky
            for ($rowspanIndex = 1;  $rowspanIndex < count($zobrazeneNastaveniDne);  $rowspanIndex++) { // prvni radek pro danou osobu uz je zobrazen!
                $result .=
                    "<tr data-ron-idpp='{$osoba ->idPP}' data-rowspan-index='$rowspanIndex'>" .
                    $this->denniZaznamyOsoby($zobrazeneNastaveniDne[$rowspanIndex], $osoba) .
                    '</tr>';
            }

            $result .=
                "</tbody>";
        }

        $result .= '</table>';

        if ($this ->seznamOsobOmezen) {
            $result .= '<div class="warningmsg">' . TextManager::_('Seznam_prehledu_zpracovane_omezen') . '</div>';
        }

        return $result;
    }


    /**
     * @throws \Exception
     */
    public function render()
    {
        echo $this ->vratObsahHtml();
    }

}
