<?php
/**
 * UTF8
 * User: pavel.rudol
 * Date: 28.12.2017
 * Time: 20:22
 */

namespace RonPortal\Editace;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\PracovniPomer;
use RonPortal\Obdobi;
use RonPortal\RonPortalDBRecord;
use function Ron\ronText;

class MesicniVysledekRozdeleni
{
    public $kodCS;
    public $popisCS;
    public $typRozdeleni;
    public $rozdeleni;
    public $rozdeleniPopis;
    public $hodiny;
    public $dny;
    public $kalendarniDny;
    public $poznamka;
    public $cssClass;
    public $cssStyle;
}

class MesicniVysledekRozdeleniSoucet
{
    public $kodCS;
    public $popisCS;
    public $hodiny;
    public $dny;
    public $kalendarniDny;
    public $soucetHodin;
    public $soucetDnu;
    public $soucetKalDnu;
    /** @var array */
    public $kodyCS;
    /** @var MesicniVysledekRozdeleni[] */
    public $mesicniVysledky;

    public function __construct()
    {
        $this->hodiny = 0;
        $this->dny = 0;
        $this->kalendarniDny = 0;
        $this->soucetHodin = 0;
        $this->soucetDnu = 0;
        $this->soucetKalDnu = 0;
    }

}

class RozdeleniMV
{

    /** @var PracovniPomer */
    public $pracovniPomer;
    /** @var Obdobi */
    public $obdobi;
    /** @var MesicniVysledekRozdeleniSoucet[] */
    public $mesicniDataRozdeleni;
    /** @var MesicniVysledekRozdeleni[] */
    public $mesicniData;

    /** @var bool pri nepovolene editaci zobrazi needitovatelny formular */
    public $povolitEditaci = true;

    /** @var array */
    public $kodyCS_R;
    /** @var array */
    private $kodyCS_M;
    private $nastaveniEditace;


    public function __construct($pracovniPomer, $obdobi, $nastaveniEditace)
    {
        $this->pracovniPomer = $pracovniPomer;
        $this->obdobi = $obdobi;

        $this->nastaveniEditace = $nastaveniEditace;

        $this->mesicniDataRozdeleni = [];
        $this->mesicniData = [];

        $this->kodyCS_R = [];
        $this->kodyCS_M = [];

        $this->zpracujNastaveni();
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param Obdobi $obdobi
     * @param array $nastaveniEditace
     * @return null|RozdeleniMV
     * @throws Exception
     */
    public static function nactiData($database, $pracovniPomer, $obdobi, $nastaveniEditace)
    {
        $rozdeleniMV = new RozdeleniMV($pracovniPomer, $obdobi, $nastaveniEditace);

        $rozdeleniMV->nactiMesicniVysledky($database);

        $params = $database->createParameters();

        $sql =
            "SELECT MesicniVysledek_R_E.TypCasoveSlozkyMzdy, MesicniVysledek_R_E.KodTypuRozdeleni, MesicniVysledek_R_E.Rozdeleni, ". // 0 - 2
            " MesicniVysledek_R_E.Hodiny, MesicniVysledek_R_E.Dny, MesicniVysledek_R_E.KalendarniDny, TypCasoveSlozkyMzdy.Popis, Rozdeleni.Popis, MesicniVysledek_R_E.Poznamka " . // 3 - 8
            "FROM MesicniVysledek_R_E " .
            " JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = MesicniVysledek_R_E.KodTypuRozdeleni AND Rozdeleni.Kod = MesicniVysledek_R_E.Rozdeleni) " .
            " JOIN TypCasoveSlozkyMzdy ON (TypCasoveSlozkyMzdy.Kod = MesicniVysledek_R_E.TypCasoveSlozkyMzdy) " .
            "WHERE RC = " . $params->addParameter($pracovniPomer->rc) .
            "  AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
            "  AND DatumNastupu = " . $params->addParameter($database->dateToParam($pracovniPomer->datumNastupu)) .
            "  AND Mesic = " . $params->addParameter($database->dateToParam($obdobi->zacatek));

        MessageManager::addDebugLog('Rozdeleni mesicnich vysledku: ' . $sql);

        try {
            $cursor = $database->getSqlResult($sql, $params);
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri nacitani mesicnich vysledku rozdeleni. ' . $exception->getMessage());
        }

        while ($row = $cursor->fetch()) {
            $radekNastaveniCS = $rozdeleniMV->vratRadekNastaveniCS($row[0]);

            $hodinyFloat = is_string($row[3]) ? StringUtils::stringToFloat($row[3]) : $row[3];
            $dnyFloat = is_string($row[4]) ? StringUtils::stringToFloat($row[4]) : $row[4];
            $kalendarniDnyFloat = is_string($row[5]) ? StringUtils::stringToFloat($row[5]) : $row[5];

            $mesicniVysledekRozdeleni = new MesicniVysledekRozdeleni();
            $mesicniVysledekRozdeleni->kodCS = $row[0];
            $mesicniVysledekRozdeleni->typRozdeleni = $row[1];
            $mesicniVysledekRozdeleni->rozdeleni = $row[2];
            $mesicniVysledekRozdeleni->hodiny = $hodinyFloat;
            $mesicniVysledekRozdeleni->dny = $dnyFloat;
            $mesicniVysledekRozdeleni->kalendarniDny = $kalendarniDnyFloat;
            $mesicniVysledekRozdeleni->rozdeleniPopis = $row[7];
            $mesicniVysledekRozdeleni->poznamka = $row[8];

            //pro casove slozky, ktere se maji scitat je definovano nastaveni
            if (!is_null($radekNastaveniCS)) {
                $mesicniVysledekRozdeleni->popisCS = $radekNastaveniCS['nadpis'] ?: $row[6];
                $mesicniVysledekRozdeleni->cssClass = $radekNastaveniCS['class'];
                $mesicniVysledekRozdeleni->cssStyle = $radekNastaveniCS['style'];

                $mesicniVysledekRozdeleniSoucet = $rozdeleniMV->mesicniDataRozdeleni[$radekNastaveniCS['souctovacs']];

                $mesicniVysledekRozdeleniSoucet->mesicniVysledky[] = $mesicniVysledekRozdeleni;
                $mesicniVysledekRozdeleniSoucet->soucetHodin += $hodinyFloat;
                $mesicniVysledekRozdeleniSoucet->soucetDnu += $dnyFloat;
                $mesicniVysledekRozdeleniSoucet->soucetKalDnu += $kalendarniDnyFloat;

                $rozdeleniMV->mesicniDataRozdeleni[$radekNastaveniCS['souctovacs']] = $mesicniVysledekRozdeleniSoucet;
            }
            //ostatni vysledky se nekontroluji na soucet
            else {
                $mesicniVysledekRozdeleni->popisCS = $row[6];

                $rozdeleniMV->mesicniData[] = $mesicniVysledekRozdeleni;
            }
        }

        $cursor->close();

        $rozdeleniMV->nactiCasoveSlozkyProSouctoveCS();

        //kontrola na editaci vyhazuje vyjimku, pokud k ni dojde, neumozni formular editovat
        try {
            RozdeleniMVController::lzeEditovat($database, $obdobi, $pracovniPomer);
        } catch (Exception $exception) {
            $rozdeleniMV->povolitEditaci = false;
        }

        return $rozdeleniMV;
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param Obdobi $obdobi
     * @param array $nastaveniEditace
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public static function zpracujPost($database, $pracovniPomer, $obdobi, $nastaveniEditace)
    {
        $rozdeleniMV = new RozdeleniMV($pracovniPomer, $obdobi, $nastaveniEditace);

        $rozdeleniMV->nactiZkontrolujSouctyPost($database);

        for ($index = 1; $index <= $_POST['pocet_radku']; $index++) {
            $record = new RonPortalDBRecord($database, 'MesicniVysledek_R_E');
            $record->logDescription = StringUtils::fromUTF8('Editace rozdělení měsíčních výsledků');

            $record->addField('RC', '', true);
            $record->addField('OsobniCislo', '', true);
            $record->addField('DatumNastupu', 'timestamp', true);
            $record->addField('Mesic', 'timestamp', true);
            $record->addField('TypCasoveSlozkyMzdy', '', true);
            $record->addField('KodTypuRozdeleni', 'int', true);
            $record->addField('Rozdeleni', '', true);
            $record->addField('Hodiny', 'float', false, 0, DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny_' . $index]), true));
            $record->addField('Dny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_' . $index])));
            $record->addField('KalendarniDny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_' . $index])));
            $record->addField('Poznamka', '', false, 250, $_POST['rozdelenimv_poznamka_' . $index]);

            $params = $database->createParameters();
            $where =
                "     RC = " . $params->addParameter($pracovniPomer->rc) .
                " AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($database->dateToParam($pracovniPomer->datumNastupu)) .
                " AND Mesic = " . $params->addParameter($database->dateToParam($obdobi->zacatek)) .
                " AND TypCasoveSlozkyMzdy = " . $params->addParameter($_POST['rozdelenimv_casova_slozka_' . $index]) .
                " AND KodTypuRozdeleni = " . $params->addParameter($_POST['rozdelenimv_typrozdeleni_' . $index]) .
                " AND Rozdeleni = " . $params->addParameter($_POST['rozdelenimv_rozdeleni_' . $index]);

            $record->setWhere($where, $params);

            $record->loadAndUpdateRecord();
        }

        foreach ($rozdeleniMV->kodyCS_M as $kodCS) {
            if ((DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny_' . $kodCS]), true) != 0) ||
                (StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_' . $kodCS])) != 0) ||
                (StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_' . $kodCS])) != 0))
            {
                $recordSouctoveCS = new RonPortalDBRecord($database, 'MesicniVysledek_R_E');
                $recordSouctoveCS->logDescription = StringUtils::fromUTF8('Editace rozdělení měsíčních výsledků');

                $recordSouctoveCS->addField('RC', '', true, 15, $pracovniPomer->rc);
                $recordSouctoveCS->addField('OsobniCislo', '', true, 15, $pracovniPomer->osobniCislo);
                $recordSouctoveCS->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer->datumNastupu);
                $recordSouctoveCS->addField('Mesic', 'timestamp', true, 0, $obdobi->zacatek);
                $recordSouctoveCS->addField('TypCasoveSlozkyMzdy', '', true, 10, $_POST['rozdelenimv_casova_slozka_' . $kodCS]);
                $recordSouctoveCS->addField('KodTypuRozdeleni', 'int', true, 0, $_POST['rozdelenimv_typ_rozdeleni']);
                $recordSouctoveCS->addField('Rozdeleni', '', true, 30, $_POST['rozdelenimv_rozdeleni_' . $kodCS]);
                $recordSouctoveCS->addField('Hodiny', 'float', false, 0, DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny_' . $kodCS]), true));
                $recordSouctoveCS->addField('Dny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_' . $kodCS])));
                $recordSouctoveCS->addField('KalendarniDny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_' . $kodCS])));
                $recordSouctoveCS->addField('Poznamka', '', false, 250, $_POST['rozdelenimv_poznamka_' . $kodCS]);

                $params = $database->createParameters();
                $where =
                    "     RC = " . $params->addParameter($pracovniPomer->rc) .
                    " AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
                    " AND DatumNastupu = " . $params->addParameter($database->dateToParam($pracovniPomer->datumNastupu)) .
                    " AND Mesic = " . $params->addParameter($database->dateToParam($obdobi->zacatek)) .
                    " AND TypCasoveSlozkyMzdy = " . $params->addParameter($_POST['rozdelenimv_casova_slozka_' . $kodCS]) .
                    " AND KodTypuRozdeleni = " . $params->addParameter($_POST['rozdelenimv_typ_rozdeleni']) .
                    " AND Rozdeleni = " . $params->addParameter($_POST['rozdelenimv_rozdeleni_' . $kodCS]);

                $recordSouctoveCS->setWhere($where, $params);

                if ($recordSouctoveCS->loadRecord() == 0) {
                    $recordSouctoveCS->insertRecord();
                } else {
                    throw new Exception( StringUtils::fromUTF8('Záznam rozdělení měsíčního výsledku již existuje.') );
                }
            }
        }

        if ((DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny']), true) != 0) ||
            (StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny'])) != 0) ||
            (StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny'])) != 0))
        {
            $recordInsert = new RonPortalDBRecord($database, 'MesicniVysledek_R_E');
            $recordInsert->logDescription = StringUtils::fromUTF8('Editace rozdělení měsíčních výsledků');

            $recordInsert->addField('RC', '', true, 15, $pracovniPomer->rc);
            $recordInsert->addField('OsobniCislo', '', true, 15, $pracovniPomer->osobniCislo);
            $recordInsert->addField('DatumNastupu', 'timestamp', true, 0, $pracovniPomer->datumNastupu);
            $recordInsert->addField('Mesic', 'timestamp', true, 0, $obdobi->zacatek);
            $recordInsert->addField('TypCasoveSlozkyMzdy', '', true, 10, $_POST['rozdelenimv_casova_slozka']);
            $recordInsert->addField('KodTypuRozdeleni', 'int', true, 0, $_POST['rozdelenimv_typ_rozdeleni']);
            $recordInsert->addField('Rozdeleni', '', true, 30, $_POST['rozdelenimv_rozdeleni']);
            $recordInsert->addField('Hodiny', 'float', false, 0, DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny']), true));
            $recordInsert->addField('Dny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny'])));
            $recordInsert->addField('KalendarniDny', 'float', false, 0, StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny'])));
            $recordInsert->addField('Poznamka', '', false, 250, $_POST['rozdelenimv_poznamka']);

            $params = $database->createParameters();
            $where =
                "     RC = " . $params->addParameter($pracovniPomer->rc) .
                " AND OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
                " AND DatumNastupu = " . $params->addParameter($database->dateToParam($pracovniPomer->datumNastupu)) .
                " AND Mesic = " . $params->addParameter($database->dateToParam($obdobi->zacatek)) .
                " AND TypCasoveSlozkyMzdy = " . $params->addParameter($_POST['rozdelenimv_casova_slozka']) .
                " AND KodTypuRozdeleni = " . $params->addParameter($_POST['rozdelenimv_typ_rozdeleni']) .
                " AND Rozdeleni = " . $params->addParameter($_POST['rozdelenimv_rozdeleni']);

            $recordInsert->setWhere($where, $params);

            if ($recordInsert->loadRecord() == 0) {
                $recordInsert->insertRecord();
            } else {
                throw new Exception( StringUtils::fromUTF8('Záznam rozdělení měsíčního výsledku již existuje.') );
            }
        }

        return true;
    }

    public function vratRadekNastaveniCS($kodCS)
    {
        foreach ($this->nastaveniEditace['radekeditace'] as $radekEditace) {
            if ($radekEditace['kod'] == $kodCS)
                return $radekEditace;
        }

        return null;
    }

    /**
     * @throws Exception
     */
    public function zkontrolujSoucty()
    {
        /** @var MesicniVysledekRozdeleniSoucet $mesicniVysledek */
        foreach ($this->mesicniDataRozdeleni as $mesicniVysledek) {

            $soucetHodin = round($mesicniVysledek->soucetHodin * 60);
            $hodiny = round($mesicniVysledek->hodiny * 60);

            if (($soucetHodin > $hodiny) || ($mesicniVysledek->soucetDnu > $mesicniVysledek->dny) || ($mesicniVysledek->soucetKalDnu > $mesicniVysledek->kalendarniDny)) {
                throw new Exception(sprintf(ronText('Mesicni_vysledky_rozdeleni_prekrocen_soucet'), $mesicniVysledek->popisCS));
            }
        }
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function nactiZkontrolujSouctyPost($database)
    {
        foreach ($this->kodyCS_M as $kodCS) {
            $mesicniVysledekRozdeleni = new MesicniVysledekRozdeleniSoucet();
            $mesicniVysledekRozdeleni->hodiny = StringUtils::stringToFloat(trim($_POST['rozdelenimv_hodiny_celkem_' . $kodCS]));
            $mesicniVysledekRozdeleni->dny = StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_celkem_' . $kodCS]));
            $mesicniVysledekRozdeleni->kalendarniDny = StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_celkem_' . $kodCS]));
            for ($index = $_POST['rozdelenimv_index_od_' . $kodCS]; $index <= $_POST['rozdelenimv_index_do_' . $kodCS]; $index++) {
                $mesicniVysledekRozdeleni->soucetHodin += DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny_' . $index]), true);
                $mesicniVysledekRozdeleni->soucetDnu += StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_' . $index]));
                $mesicniVysledekRozdeleni->soucetKalDnu += StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_' . $index]));
            }
            $mesicniVysledekRozdeleni->soucetHodin += DateUtils::timeStringToFloat(trim($_POST['rozdelenimv_hodiny_' . $kodCS]), true);
            $mesicniVysledekRozdeleni->soucetDnu += StringUtils::stringToFloat(trim($_POST['rozdelenimv_dny_' . $kodCS]));
            $mesicniVysledekRozdeleni->soucetKalDnu += StringUtils::stringToFloat(trim($_POST['rozdelenimv_kaldny_' . $kodCS]));

            $params = $database->createParameters();
            $mesicniVysledekRozdeleni->popisCS = DBUtils::getSqlValue($database, 'SELECT Popis FROM TypCasoveSlozkyMzdy WHERE Kod = ' . $params->addParameter($kodCS), $params);
            $this->mesicniDataRozdeleni[$kodCS] = $mesicniVysledekRozdeleni;
        }
        $this->zkontrolujSoucty();
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function nactiPopisProSouctoveCS($database)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT TypCasoveSlozkyMzdy.Kod, TypCasoveSlozkyMzdy.Popis ". // 0 - 1
            "FROM TypCasoveSlozkyMzdy " .
            "WHERE Kod IN ('" . implode("', '", $this->kodyCS_M) . "')";

        MessageManager::addDebugLog('Souctove casove slozky: ' . $sql);

        try {
            $cursor = $database->getSqlResult($sql, $params);
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri nacitani souctovych CS . ' . $exception->getMessage());
        }

        while ($row = $cursor->fetch()) {
            $mesicniVysledekRozdeleni = new MesicniVysledekRozdeleniSoucet();

            $mesicniVysledekRozdeleni->kodCS = $row[0];
            $mesicniVysledekRozdeleni->popisCS = $row[1];

            $this->mesicniDataRozdeleni[$row[0]] = $mesicniVysledekRozdeleni;
        }
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function nactiMesicniVysledky($database)
    {
        $this->nactiPopisProSouctoveCS($database);

        $params = $database->createParameters();

        $sql =
            "SELECT MesicniVysledekVypoctu.TypCasoveSlozkyMzdy, TypCasoveSlozkyMzdy.Popis, ". // 0 - 1
            " MesicniVysledekVypoctu.Hodiny, MesicniVysledekVypoctu.Dny, MesicniVysledekVypoctu.KalendarniDny " . // 2 - 4
            "FROM MesicniVysledekVypoctu " .
            " JOIN TypCasoveSlozkyMzdy ON (TypCasoveSlozkyMzdy.Kod = MesicniVysledekVypoctu.TypCasoveSlozkyMzdy) " .
            "WHERE RC = " . $params->addParameter($this->pracovniPomer->rc) .
            "  AND OsobniCislo = " . $params->addParameter($this->pracovniPomer->osobniCislo) .
            "  AND DatumNastupu = " . $params->addParameter($database->dateToParam($this->pracovniPomer->datumNastupu)) .
            "  AND Mesic = " . $params->addParameter($database->dateToParam($this->obdobi->zacatek)) .
            "  AND TypCasoveSlozkyMzdy IN ('" . implode("', '", $this->kodyCS_M) . "')";

        MessageManager::addDebugLog('Mesicni vysledky: ' . $sql);

        try {
            $cursor = $database->getSqlResult($sql, $params);
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri nacitani mesicnich vysledku . ' . $exception->getMessage());
        }

        while ($row = $cursor->fetch()) {
            $mesicniVysledekRozdeleni = $this->mesicniDataRozdeleni[$row[0]];

            $mesicniVysledekRozdeleni->kodCS = $row[0];
            $mesicniVysledekRozdeleni->popisCS = $row[1];
            $mesicniVysledekRozdeleni->hodiny = is_string($row[2]) ? StringUtils::stringToFloat($row[2]) : $row[2];
            $mesicniVysledekRozdeleni->dny = is_string($row[3]) ? StringUtils::stringToFloat($row[3]) : $row[3];
            $mesicniVysledekRozdeleni->kalendarniDny = is_string($row[4]) ? StringUtils::stringToFloat($row[4]) : $row[4];

            $this->mesicniDataRozdeleni[$row[0]] = $mesicniVysledekRozdeleni;
        }

        $cursor->close();
    }

    private function nactiCasoveSlozkyProSouctoveCS()
    {
        foreach ($this->nastaveniEditace['radekeditace'] as $radekEditace) {
            $this->mesicniDataRozdeleni[$radekEditace['souctovacs']]->kodyCS[] = $radekEditace['kod'];
        }
    }

    /**
     *  Nacte nastaveni souctovych CS
     */
    private function zpracujNastaveni()
    {
        foreach ($this->nastaveniEditace['radekeditace'] as $radekEditace) {
            $this->kodyCS_R[] = $radekEditace['kod'];
            if (!in_array($radekEditace['souctovacs'], $this->kodyCS_M))
                $this->kodyCS_M[] = $radekEditace['souctovacs'];
        }
    }
}