<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;

class RozdeleniVysledkuUtils
{
    const SETTINGS_POUZE_VLASTNI_ROZDELENI = 'dv_pouzevlastnir';

    /**
     * @param Database $database
     * @param string $rozdeleni
     * @param int $den
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function overSchvaleni($database, $rozdeleni, $den)
    {
        $params = $database ->createParameters();
        $sql =
            ' SELECT Schvaleno FROM SchvaleniRozdeleniDoch ' .
            ' WHERE SchvaleniRozdeleniDoch.Den = ' . $params ->addParameter($database ->dateToParam($den)) .
            ' AND SchvaleniRozdeleniDoch.KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI)) .
            ' AND SchvaleniRozdeleniDoch.Rozdeleni = ' . $params ->addParameter($rozdeleni);

        $cursor = $database ->getSqlResult($sql, $params);

        $result = ($row = $cursor ->fetch())
            ? (int)$row[0]
            : 0;

        $cursor ->close();

        if ($result) throw new \Exception( TextManager::_('Rozdeleni_DV_uz_je_schvaleno') );
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $typRozdeleni
     * @param string $hodnota
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $kod
     * @return SqlStatement
     */
    public static function getSQLVyberRozdeleniDV($database, $pp, $typRozdeleni, $hodnota, $datumOd, $datumDo, $kod = false)
    {
        $params = $database ->createParameters();

        if (RonPortalSettings::getIntValue(RonPortalSettings::DV_POVOLIT_PRIRAZENI_ROZDELENI)) {
            // uzivatel si muze sam vybirat rozdeleni - zobrazuji vzdy jen aktualne prirazene zaznamy
            $datumPrirazeniOd = time();
            $datumPrirazeniDo = time();
        } else {
            $datumPrirazeniOd = $datumOd;
            $datumPrirazeniDo = $datumDo;
        }

        if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI) == 3) { // podrizena
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni, HierarchieRozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addDateParameter($datumDo) .

                '   AND Rozdeleni.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
                '   AND Rozdeleni.Kod = HierarchieRozdeleni.KodPodrizene' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene ' .
                '   AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene ' .
                '   AND HierarchieRozdeleni.KodPodrizene <> HierarchieRozdeleni.KodNadrazene ' . // vynecha vlastni

                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumPrirazeniOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumPrirazeniDo) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu);
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI) == 2) { // vlastni a podrizena
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni, HierarchieRozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addDateParameter($datumDo) .

                '   AND Rozdeleni.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene' .
                '   AND Rozdeleni.Kod = HierarchieRozdeleni.KodPodrizene' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene ' .
                '   AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene ' .

                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumPrirazeniOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumPrirazeniDo) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu);
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else if (RonPortalSettings::getIntValue(self::SETTINGS_POUZE_VLASTNI_ROZDELENI)) { // pouze vlastni
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob, Rozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '   AND Rozdeleni.DatumVytvoreni <= ' . $params ->addDateParameter($datumDo) .
                '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumPrirazeniOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumPrirazeniDo) .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp->rc) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu);
            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
                ' AND (' .
                '( (DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR DatumZruseni IS NULL)' .
                '  AND Rozdeleni.DatumVytvoreni <= ' . $params ->addDateParameter($datumDo) .
                ')';

            if ($hodnota) {
                // zvolenou hodnotu zobrazi vzdy - pozor pri volbe "pouze vlastni ..." je to moc pomale
                $sql .= ' OR (Rozdeleni.Kod = ' . $params ->addParameter($hodnota) . ')';
            }

            $sql .=
                ')';
        }

        $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

        $sql .= $kod ? ' ORDER BY Kod ' : ' ORDER BY Popis ';

        return new SqlStatement($sql, $params);
    }


    /**
     * @param Database $database
     * @return string
     */
    public static function getSQLVyberCasSlozkyDV($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM TypCasoveSlozkyMzdy'.
            ' WHERE ';

        $found = false;

        $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
        foreach ( $slozky as $kodSlozky ) if ($kodSlozky) {
            $sql .= $found ? ', ' : ' Kod IN (';
            $found = true;

            $sql .= $params ->addParameter($kodSlozky);
        }

        $sql .=
            ($found ? ')' : '0 = 1') .
            ' ORDER BY Popis';

        return new SqlStatement($sql, $params);
    }

}
