<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;

class RozliseniDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fTypRozdeleni;
    /** @var DBRecordField  */
    var $fKodRozdeleni;

    /**
     * @param Database      $database
     * @param PracovniPomer $pp
     * @param int           $datumCasPocatku
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $datumCasPocatku, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'RozliseniDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this->addField('DatumCasPocatku', 'datetime', true, 0, $datumCasPocatku);

        $this->fTypRozdeleni = $this->addField('KodTypuRozdeleni', 'int', true, 0, null);
        $this->fKodRozdeleni = $this->addField('KodRozdeleni', '', true, 30, null);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND DatumCasPocatku = ' . $params ->addParameter($database ->timestampToParam($datumCasPocatku)) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)),
            $params);
    }


    /**
     * @param int $typ
     * @param string $kod
     * @param bool $onlyInsert  Pokud je true, tak nekontroluje, jestli zaznam existuje (pouzije se pri vlozeni zaznamu dochazky)
     * @return bool             Vraci true, pokud doslo ke zmene
     */
    function zapisRozdeleni($typ, $kod, $onlyInsert = false)
    {
        $this ->fTypRozdeleni ->setNewValue($typ);
        $this ->fKodRozdeleni ->setNewValue($kod);

        if ($onlyInsert) {
            $exists = false;
        } else {
            $exists = $this->loadRecord();
        }

        if ($exists) {
            if (trim($kod) == '') {
                $this->doDelete(null);
                return true;
            } else {
                $result = $this ->isChanged();
                $this->updateRecord();
                return $result;
            }
        } else {
            if (trim($kod) != '') {
                $this->insertRecord();
                return true;
            }
        }

        return false; // zadna zmena
    }
}
