<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\MzdovaPolozka;
use RonPortal\Dochazka\OsobaDochazky;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Rozdeleni;

class SloupecMesicniDochazky
{
    /** @var  string  DEN (dny dochazky), MV (mesicni vysledek), UZIVATELSKA, UZIVATELSKAPP, MP (mzdova polozka), R (rozdeleni), RP (popis rozdeleni) a dalsi */
    public $typ;
    public $nadpis;
    /** @var  string    atribut title pro hlavicku sloupce */
    public $title;

    /** @var string     Zarovnani left right nebo center.*/
    public $align;
    /** @var string     CSS styl */
    public $styl;
    public $cssClass;
    public $headerClass;
    public $footerClass;

    /** @var string     kod mzdove polozky, vysledku a podobne */
    public $kod;
    /** @var string     Jednotka pro mesicni vysledek H/D/K/P */
    public $jednotka;
    /** @var string     Jmeno zobrazeneho pole pro mzdovou polozku - Castka (default), CenaZaJednotku, PocetJednotek */
    public $jmenoPole;

    /** @var int        Priznak, jestli ma rozlisit rozdeleni mzdove polozky - 0=ne, 1=ano (podle formulare), 2=ano (podle osoby) */
    public $rozlisitRozdeleni;
    /** @var int        Pokud se ma rozlisovat rozdeleni, je mozne urcit jiny typ rozdeleni. Dale se pouziva pro sloupce ROZD a ROZDP */
    public $kodTypuRozdeleni;

    /** @var int        Index pro uzivatelske polozky */
    public $index;

    /** @var string     Uroven pro zobrazeni schvaleni, pripadne 'all' pro vsechny */
    public $uroven;

    /** @var string     Format zobrazeni (pro MP, pocet dnu, sql) */
    public $format;

    /** @var string     Umoznuje omezit, kdy se bude polozka zobrazovat - mozne hodnoty jsou export/html/<nic>(zobrazi se vzdy) */
    public $export;

    /** @var string     Format pro export */
    public $exportFormat;

    /** @var string     pro typ SQL - sql vyraz s promennymi {$RC}, {$OsobniCislo}, ... Prvni sloupec sql vysledku se zobrazi, pripadny druhy sloupec se nastavi do title */
    public $sql;

    /** @var int        Priznak, jestli se ma zobrazit proklik na zpracovanou (0=ne, 1=ano, 2=v pojmenovanem okne, 3=v novem okne). Pouzije se pro jmeno atd. */
    public $proklikZpracovana;

    /** @var bool       Priznak, ze se maji zobrazit i denni kontroly */
    public $zobrazitVse;

    /** @var PravaNaOsoby    prava na zobrazeni sloupce */
    public $prava = null;
    /** @var string     Kod profilu uzivatelu, kterym se sloupec zobrazi. Ostatni jej nevidi */
    public $profil = null;
    /** @var string     PHP vyraz s podminkou pro zobrazeni sloupce. Muze obsahovat promenne */
    public $condition = null;
    /** @var bool       Priznak, jestli se ma zobrazit - nastavuje se podle prav, pripadne profilu,
     *                  ale umoznuje i natvrdo skryt sloupec */
    public $zobrazit = true;

    /** @var string     Nastaveni filtru pro skryvani jednotlivych sloupcu*/
    public $filtrCaption;
    /** @var string */
    public $filtrKey;
    /** @var string */
    public $filtrGroup;

    /** @var int        Nastaveni poradi sloupce pro dodatecne razeni */
    public $razeni;


    /**
     * Zpracuje nastaveni z pole nacteneho z XML
     * @param array $settingsArray
     */
    public function loadSettings($settingsArray)
    {
        foreach ($settingsArray as $name => $value) {
            switch ($name) {
                case 'class' :
                    $this->cssClass = $value;
                    break;
                case 'headerclass' :
                    $this->headerClass = $value;
                    break;
                case 'footerclass' :
                    $this->footerClass = $value;
                    break;

                case 'style' :
                    $this->styl = $value;
                    break;

                case 'prava' :
                    $this ->prava = Prava::vratPrava($value);
                    break;

                case 'rozlisitrozdeleni' :
                    $this ->rozlisitRozdeleni = (int)$value;
                    break;

                case 'typrozdeleni' :
                case 'typRozdeleni' :
                case 'kodtypurozdeleni' :
                    $this ->kodTypuRozdeleni = (int)$value;
                    break;

                case 'kodFiltru' :
                    $this ->filtrKey = $value;
                    break;
                case 'filtr' :
                    $this ->filtrCaption = TextManager::_($value);
                    break;
                case 'filtrGroup' :
                    $this ->filtrGroup = $value;
                    break;

                case 'proklikzpracovana' :
                    $this ->proklikZpracovana = (int)$value;
                    break;

                case 'index' :
                case 'rozlisitRozdeleni' :
                case 'kodTypuRozdeleni' :
                case 'proklikZpracovana' :
                case 'zobrazitVse' :
                case 'zobrazit' :
                case 'razeni' :
                    $this ->{$name} = (int)$value;
                    break;

                case 'uroven' :
                    $this ->{$name} = strtolower($value);
                    break;

                case 'nadpis' :
                case 'title' :
                    $this ->{$name} = TextManager::_($value);
                    break;

                case 'pole' :
                    $this ->jmenoPole = $value;
                    break;

                case 'typ' :
                case 'styl' :
                case 'cssClass' :
                case 'align' :
                case 'kod' :
                case 'jednotka' :
                case 'format' :
                case 'export' :
                case 'exportFormat' :
                case 'profil' :
                case 'condition' :
                case 'sql' :
                    $this->{$name} = $value;
                    break;
            }
        }
        //var_dump($this);
    }


    /**
     * Pokud ma sloupec zadanou podminku, tak ji vyhodnoti
     * @param $zacatekObdobi
     * @return bool
     */
    public function evaluateCondition($zacatekObdobi)
    {
        if (!$this ->condition) {
            // podminka neni zadana
            return true;
        }

        $condition = str_replace('%OBDOBI%', $zacatekObdobi, $this ->condition);

        if ($condition) {
            return (bool)eval('return ' . $condition . ';');
        }

        return true;
    }


    /**
     * Najde mzdovou polozku pro tento sloupec
     * @param ZpracovavanaOsoba $osoba
     * @param Rozdeleni $rozdeleni
     * @return MzdovaPolozka
     */
    public function vratMzdovouPolozkuOsoby($osoba, $rozdeleni)
    {
        if (!$osoba ->mzdovePolozky) {
            return null;
        }

        foreach ($osoba ->mzdovePolozky as $mzdovaPolozka) {
            if ($mzdovaPolozka ->mzdovaPolozka === $this ->kod) {
                if (!$this ->rozlisitRozdeleni) {
                    return $mzdovaPolozka;
                }

                if ($rozdeleni  &&  $rozdeleni ->kod === $mzdovaPolozka ->rozdeleni  &&  $rozdeleni ->typ === $mzdovaPolozka ->kodTypuRozdeleni) {
                    return $mzdovaPolozka;
                }
            }
        }

        return null;
    }


    /**
     * @param MzdovaPolozka $mp
     * @return float|null
     */
    public function vratHodnotuMP($mp)
    {
        if ($mp) {
            switch (strtolower($this ->jmenoPole ?? '')) {
                case 'pocetjednotek' :
                    return $mp ->pocetJednotek;

                case 'cenazajednotku' :
                    return $mp ->cenaZaJednotku;

                default :
                    return $mp ->castka;
            }
        }

        return null;
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param bool $html
     * @return string
     */
    protected function vratMesicniKontroly($osoba, $html = true)
    {
        if (!$osoba ->kontroly) {
            return '';
        }

        $result = '';
        foreach ($osoba ->kontroly as $index => $kontrola) {
            if ($this ->zobrazitVse  || $kontrola ->mesicni) {
                if (!$html) {
                    return '!';
                }

                $result .= $kontrola->htmlPriznak($index, true);
            }
        }

        return $result;
    }


    /**
     * @param OsobaDochazky $osoba
     * @param Obdobi $obdobi
     * @param int $uroven
     * @return string
     */
    private function getHtmlProUrovenSchvaleni($osoba, $obdobi, $uroven)
    {
        switch ($uroven) {
            case '1' :
                $schvalenoDo = $osoba ->schvaleni ->schvaleni1;
                $cssClass = 'textschvaleni1';
                break;

            case '2' :
                $schvalenoDo = $osoba ->schvaleni ->schvaleni2;
                $cssClass = 'textschvaleni2';
                break;

            case '3' :
                $schvalenoDo = $osoba ->schvaleni ->schvaleni3;
                $cssClass = 'textschvaleni3';
                break;

            default :
                $schvalenoDo = null;
                $cssClass = '';
        }

        switch ($this ->format) {
            case 'table' :  // zadny text - jen nastylovana bunka tabulky
                return '';

            case 'check' :
                return ($schvalenoDo >= $obdobi ->konec)
                    ? "<div class='inline-glyph fa glyph-check $cssClass'></div>"
                    : '';

            case 'yes' :
                return ($schvalenoDo >= $obdobi ->konec)
                    ? "<span class='$cssClass'>" . TextManager::_('Str_Yes') . "</span>"
                    : '';

            case 'yesno' :
                return ($schvalenoDo >= $obdobi ->konec)
                    ? "<span class='$cssClass'>" . TextManager::_('Str_Yes') . "</span>"
                    : "<span class='$cssClass'>" . TextManager::_('Str_No') . "</span>";

            default :
                return $schvalenoDo
                    ? "<span class='$cssClass'>" . date('j.n.Y', $schvalenoDo) . '</span>'
                    : '';
        }
    }

    /**
     * @param OsobaDochazky $osoba
     * @param Obdobi    $obdobi
     * @return string
     */
    public function getHtmlSchvaleni($osoba, $obdobi)
    {
        if (!$osoba  ||  !$osoba ->schvaleni) {
            return '';
        }

        switch ($this ->uroven) {
            case '1' :
            case '2' :
            case '3' :
                return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, $this ->uroven);

            case 'all' :
                $result = $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 1);
                $pom = $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 2);
                $result .= ($result && $pom ? ', ' : '') . $pom;
                $pom = $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 3);
                $result .= ($result && $pom ? ', ' : '') . $pom;
                return $result;

            default :
                // pro formaty check a yesno vybere nejvyssi uroven schvalenou az do konce obdobi
                // jinak vybere nejvyssi uroven schvaleni
                if ($this ->format === 'check' || $this ->format === 'yes' || $this ->format === 'yesno') {
                    if ($osoba ->schvaleni ->schvaleni3 >= $obdobi ->konec) {
                        return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 3);
                    }

                    if ($osoba ->schvaleni ->schvaleni2 >= $obdobi ->konec) {
                        return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 2);
                    }

                    return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 1);
                }

                if ($osoba ->schvaleni ->schvaleni3) {
                    return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 3);
                }

                if ($osoba ->schvaleni ->schvaleni2) {
                    return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 2);
                }

                return $this ->getHtmlProUrovenSchvaleni($osoba, $obdobi, 1);
        }
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @return string
     * @throws Exception
     */
    protected function getTextOsoba($osoba)
    {
        switch ($this ->typ) {
            case 'JMENO' :
                return $osoba->jmeno;

            case 'PRIJMENI' :
                return $osoba->prijmeni;

            case 'JMENOPRIJMENI' :
                return $osoba->jmenoPrijmeni;

            case 'NASTUP' :
                return date('j.n.Y', $osoba->datumNastupu);

            case 'OSOBNICISLO' :
                return $osoba->osobniCislo;

            case 'TYPPP' :
                return $osoba->typPP;

            case 'UZIVATELSKA' :
                return $osoba->uzivatelska[$this->index]; // index uzivatelske polozky (1-20)

            case 'UZIVATELSKAPP' :
                return $osoba->uzivatelskaPP[$this->index]; // index uzivatelske polozky (1-20)

            case 'IDPP' :
                return $osoba->idPP;
        }

        return '';
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param Obdobi $obdobi
     * @param int    $odkazNaZpracovanou
     * @return string
     * @throws Exception
     */
    protected function getHtmlTextOsoba($osoba, $obdobi, $odkazNaZpracovanou = null)
    {
        $prava = Prava::vratPrava('prohlizeni'); // musim nacist znovu, protoze v XML se prava pro zobrazeni osoby nekontroluji

        if (!isset($odkazNaZpracovanou)) {
            $odkazNaZpracovanou = $this ->proklikZpracovana;
        }

        $zobrazitodkaz = $odkazNaZpracovanou && $prava->lzeZobrazitPP($osoba->idPP, $obdobi->zacatek, $obdobi->konec);
        $result = '';

        // jmeno osoby muze byt link na zpracovanou dochazku
        if ($zobrazitodkaz) { // 0 = ne, 1 = ve stejnem okne, 2=v novem pojmenovanem okne, 3=v novem okne
            switch ($odkazNaZpracovanou) {
                case 1 :
                    $targetstr = '';
                    break;

                case 3 :
                    $targetstr = 'target="_blank"';
                    break;

                case 2 :
                default :
                    $targetstr = 'target="zpracovanadochazkawin"';
                    break;
            }

            $result .=
                '<a ' . $targetstr .
                ' href="?menuid=zpracovanadochazka&amp;osoba=' . $osoba->idPP . '&amp;obdobi=' . DateUtils::dateToUrlString($obdobi->zacatek) . '">'; // V XML dela znak & problem!
        }

        $result .= StringUtils::xmlTextConvert($this ->getTextOsoba($osoba));

        if ($zobrazitodkaz) {
            $result .= '</a>';
        }

        return $result;

    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param bool $html    Urcuje, jestli ma vygenerovat HTML nebo pouze text
     * @return string
     */
    public function getTextRozdeleni($osoba, $html = true)
    {
        if (!$osoba ->rozdeleni) {
            return '';
        }

        $result = '';

        $format = $html
            ? $this ->format
            : $this ->exportFormat;

        foreach ($osoba ->rozdeleni as $rozdeleniOsoby) {
            if ($this ->kodTypuRozdeleni === $rozdeleniOsoby ->kodTypuRozdeleni) {
                if ($format) {
                    $result .=
                        ($result ? ', ' : '') .
                        str_replace(
                            ['%KOD%', '%POPIS%', '%IKONA%'],
                            [
                                StringUtils::htmlTextConvert($rozdeleniOsoby ->kod),
                                StringUtils::htmlTextConvert($rozdeleniOsoby ->popis),
                                $rozdeleniOsoby ->getIkonaHtml()
                            ],
                            $format);
                } else {
                    switch ($this->typ) {
                        case 'ROZD' :
                            $result .=
                                ($result ? ', ' : '') .
                                ($html
                                    ? '<span title="' . StringUtils::htmlTextConvert($rozdeleniOsoby ->popis) . '">' .
                                      StringUtils::htmlTextConvert($rozdeleniOsoby ->kod) .
                                      '</span>'
                                    : $rozdeleniOsoby ->kod);
                            break;

                        case 'ROZDP' :
                            $result .=
                                ($result ? ', ' : '') .
                                ($html
                                    ? StringUtils::htmlTextConvert($rozdeleniOsoby ->popis)
                                    : $rozdeleniOsoby ->popis);
                            break;

                        case 'ROZDIMG' :
                            if ($rozdeleniOsoby ->cssClassObrazku) {
                                $result .= $rozdeleniOsoby ->getIkonaHtml();
                            }
                            break;
                    }
                }
            }
        }

        return $result;
    }


    /**
     * Nacte hodnotu pro typ sloupce SQL. Vraci asociativni pole se zpracovanym vysledkem SQL dotazu
     * @param Database $database
     * @param array $paramValues Obsahuje hodnoty parametru. Pokud jsou zadany parametry SQL dotazu, tak se doplni do parametru, jinak primo do SQL dotazu.
     * @return array
     * @throws DBException
     */
    public function loadSQL($database, $paramValues)
    {
        $sql = $this ->sql;
        foreach ($paramValues as $name => $value) {
            $sql = str_replace($name, $value, $sql);
        }

        $text = '';
        $hint = '';
        $hodnota = null;

        if ($sql) {
            $cursor = $database->getSqlResult($sql);

            while ($row = $cursor->fetch()) {
                if ($row && isset($row[0])) {
                    $text .=
                        ($text ? ', ' : '') .
                        DBUtils::getFormatedValue($database, $this->format, $row[0]);

                    if (isset($row[1])) {
                        $hint .=
                            ($hint ? ', ' : '') .
                            $row[1];
                    }

                    $hodnota = $row[0];
                }
            }
        }

        return [
            'text' => $text,
            'title' => $hint,
            'hodnota' => $hodnota
        ];
    }


    /**
     * Nacte hodnotu pro typ sloupce SQL pro zadanou osobu. Vraci asociativni pole se zpracovanym vysledkem SQL dotazu
     * @param Database $database
     * @param PracovniPomer $osoba
     * @param Obdobi $obdobi
     * @param int $kodTypuRozdeleni
     * @param string $kodRozdeleni
     * @return array
     * @throws DBException
     */
    public function loadSQLProOsobu($database, $osoba, $obdobi, $kodTypuRozdeleni, $kodRozdeleni)
    {
        $predchoziObdobi = DateUtils::incMonth($obdobi ->zacatek, -1);
        $month = (int)date('n', $obdobi ->zacatek);
        $quarter = DateUtils::incMonth($obdobi ->zacatek,  $month -($month - 1) % 3 );

        $params = [
            // osoba
            '%RC%' => "'" . $database ->addSlashes($osoba ->rc) . "'",
            '%OsobniCislo%' => "'" . $database ->addSlashes($osoba ->osobniCislo) . "'",
            '%DatumNastupu%' => $database ->timestampToString($osoba ->datumNastupu),
            // obdobi
            '%PocatekObdobi%' => $database ->timestampToString($obdobi ->zacatek),
            '%KonecObdobi%' => $database ->timestampToString($obdobi ->konec),
            // nasledujici a predchozi obdobi
            '%PocatekNasledujicihoObdobi%' => $database ->timestampToString( DateUtils::incMonth($obdobi ->zacatek, 1) ),
            '%PocatekPredchozihoObdobi%' => $database ->timestampToString($predchoziObdobi),
            '%KonecPredchozihoObdobi%' => $database ->timestampToString(DateUtils::lastDayOfMonth($predchoziObdobi)),
            // ctvrtleti
            '%PocatekCtvrtleti%' => $database ->timestampToString($quarter),
            '%KonecCtvrtleti%' => $database ->timestampToString( DateUtils::lastDayOfMonth(DateUtils::incMonth($quarter, 2)) ),
            // predchozi ctvrtleti
            '%PocatekPredchozihoCtvrtleti%' => $database ->timestampToString(DateUtils::incMonth($quarter, -3)),
            '%KonecPredchozihoCtvrtleti%' => $database ->timestampToString( DateUtils::lastDayOfMonth(DateUtils::incMonth($quarter, -1)) ),
            '%TypRozdeleni%' => ($kodTypuRozdeleni  &&  $kodRozdeleni) ? (int)$kodTypuRozdeleni : 0,
            '%Rozdeleni%' => ($kodTypuRozdeleni  &&  $kodRozdeleni) ? "'" . $database ->addSlashes($kodRozdeleni) . "'" : "''"
        ];

        return $this ->loadSQL($database, $params);
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param Obdobi $obdobi
     * @param Rozdeleni $rozdeleni
     * @return string
     * @throws Exception
     */
    public function vratHodnotu($osoba, $obdobi, $rozdeleni)
    {
        if ($this ->prava  &&  !$this ->prava ->lzeZobrazitPP($osoba ->idPP)) {
            return null;
        }

        switch ($this ->typ) {
            case 'MV' :
                return isset($osoba ->mesicniVysledky[$this ->kod])
                    ? $osoba ->mesicniVysledky[$this ->kod] ->vratHodnotu($this ->jednotka)
                    : null;

            case 'MP' :
                return $this ->vratHodnotuMP($this ->vratMzdovouPolozkuOsoby($osoba, $rozdeleni));

//            case 'SCHVALENI' :
//                return $this ->getHtmlSchvaleni($osoba, $obdobi);

            case 'ROZD' :
                return ($osoba ->rozdeleni) ? $osoba ->rozdeleni[0] ->kod : '';
            case 'ROZDP' :
                return ($osoba ->rozdeleni) ? $osoba ->rozdeleni[0] ->popis : '';

            case 'SCHVALENI' :
                switch ($this ->uroven) {
                    case 1 :
                        return $osoba ->schvaleni ->schvaleni1;
                    case 2 :
                        return $osoba ->schvaleni ->schvaleni2;
                    case 3 :
                        return $osoba ->schvaleni ->schvaleni3;
                }
                // neni zadana uroven - vratim nejvyssi schvalenou - den + konstanta podle urovne schvaleni
                if ($osoba ->schvaleni ->schvaleni3) {
                    return 300 + date('j', $osoba ->schvaleni ->schvaleni3);
                }

                if ($osoba ->schvaleni ->schvaleni2) {
                    return 200 + date('j', $osoba ->schvaleni ->schvaleni2);
                }

                if ($osoba ->schvaleni ->schvaleni1) {
                    return 100 + date('j', $osoba ->schvaleni ->schvaleni1);
                }

                return null;

            default :
                return null;
        }
    }

    /**
     * @param ZpracovavanaOsoba $osoba
     * @param Obdobi $obdobi
     * @param Rozdeleni $rozdeleni
     * @param bool $html    Urcuje, jestli se ma generovat HTML, nebo jen cisty text pro export
     * @return string
     */
    public function getText($osoba, $obdobi, $rozdeleni, $html = true)
    {
        switch ($this ->typ) {
            case 'MV' :
                return isset($osoba ->mesicniVysledky[$this ->kod])
                    ? $osoba ->mesicniVysledky[$this ->kod] ->vratText($this ->jednotka, $this ->format)
                    : '';

            case 'MP' :
                $mp = $this ->vratMzdovouPolozkuOsoby($osoba, $rozdeleni);
                return $mp ? $mp ->vratText($this ->jmenoPole, $this ->format) : '';

            case 'JMENO' :
            case 'PRIJMENI' :
            case 'JMENOPRIJMENI' :
            case 'NASTUP' :
            case 'OSOBNICISLO' :
            case 'IDPP' :
                return $html
                    ? $this ->getHtmlTextOsoba($osoba, $obdobi)
                    : $this ->getTextOsoba($osoba);

            case 'UKONCENI' :
                return $osoba->datumUkonceni ? date('j.n.Y', $osoba->datumUkonceni) : '';

            case 'TYPPP' :
            case 'UZIVATELSKA' :
            case 'UZIVATELSKAPP' :
                return $html
                    ? StringUtils::htmlTextConvert($this ->getTextOsoba($osoba))
                    : $this ->getTextOsoba($osoba);

            case 'KONTROLY' :
                return $this ->vratMesicniKontroly($osoba, $html);

            case 'SCHVALENI' :
                return $this ->getHtmlSchvaleni($osoba, $obdobi);

            case 'SCHVALMZDY' :
                return \Ron\ronText($osoba->schvaleni->zpracovanoVeMzdach ? 'Str_Yes' : 'Str_No');

            case 'FOTO' :
                return $osoba ->idFotografie
                    ? "<div class='position-relative'><div class='browserfoto ronFoto'><img alt='foto' src='?menuid=editace&action=foto&idpp={$osoba ->idPP}'></div></div>"
                    : '';

            case 'ROZD' :
            case 'ROZDP' :
            case 'ROZDIMG' :
                return $this ->getTextRozdeleni($osoba, $html);

            case 'KAL' :
                return
                    $osoba ->kalendare
                        ? $osoba ->kalendare[0] ->format($this ->format ?: '<span title="%POPIS%">%KOD%</span>')
                        : '';

            default :
                return '';
        }
    }


    /**
     * @param ZpracovavanaOsoba $osoba
     * @param Obdobi    $obdobi
     * @return string
     */
    public function getCssClass($osoba, $obdobi)
    {
        if ($this ->typ === 'SCHVALENI' && $this ->format === 'table') {
            $result = 'schvaleni';

            if ($osoba) {
                switch ($this->uroven) {
                    case 1 :
                        if ($osoba->schvaleni->schvaleni1 >= $obdobi ->konec) {
                            $result .= ' schvaleni1';
                        }
                        break;
                    case 2 :
                        if ($osoba->schvaleni->schvaleni2 >= $obdobi ->konec) {
                            $result .= ' schvaleni2';
                        }
                        break;
                    case 3 :
                        if ($osoba->schvaleni->schvaleni3 >= $obdobi ->konec) {
                            $result .= ' schvaleni3';
                        }
                        break;
                    default :
                        if ($osoba->schvaleni->schvaleni3 >= $obdobi ->konec) {
                            $result .= ' schvaleni3';
                        }
                        else if ($osoba->schvaleni->schvaleni2 >= $obdobi ->konec) {
                            $result .= ' schvaleni2';
                        }
                        else if ($osoba->schvaleni->schvaleni1 >= $obdobi ->konec) {
                            $result .= ' schvaleni1';
                        }
                }

                if ($this->cssClass) {
                    $result .= ' ' . $this->cssClass;
                }
            }

            return $result;
        }

        if ($this ->cssClass) {
            return $this ->cssClass;
        }

        switch ($this ->typ) {
            case 'MV' :
                return 'mesicnivysledek';

            case 'MP' :
                return 'mzdovapolozka';

            case 'UZIVATELSKA' :
            case 'UZIVATELSKAPP' :
                return 'uzivatelska';

            default :
                return '';
        }
    }


    public function htmlForText($text, $osoba, $obdobi, $rowspan)
    {
        $cssClass = $this ->getCssClass($osoba, $obdobi);
        $classStr = $cssClass ? "class='{$cssClass}'" : '';

        $stylStr = $this ->styl ? "style='{$this ->styl}'" : '';
        $rowspanStr = $rowspan > 1 ? "rowspan='$rowspan'" : '';

        return "<td $classStr $stylStr $rowspanStr>" . $text . '</td>';
    }

    /**
     * @param ZpracovavanaOsoba $osoba
     * @param Obdobi    $obdobi
     * @param Rozdeleni $rozdeleni
     * @param int  $rowspan
     * @return string
     * @throws Exception
     */
    public function html($osoba, $obdobi, $rozdeleni, $rowspan)
    {
        // pokud nemam pravo, tak se text nezobrazi
        $text = ($this ->prava  &&  !$this ->prava ->lzeZobrazitPP($osoba ->idPP))
            ? ''
            : $this ->getText($osoba, $obdobi, $rozdeleni);

        return $this ->htmlForText($text, $osoba, $obdobi, $rowspan);
    }
}
