<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DateUtils;
use Ron\MessageManager;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;

class ZaznamDochazky extends DochazkaRecord
{
    const SETTINGS_SKRYT_ZAPOCTENY = 'skrytzapoctenycas';

    var $f_DatumDo; // pouze pri vkladani
    var $f_DatumDoStr;
    var $f_ViceDnu;

    /** @var string     podrizena tabulka RozliseniDochazky */
    var $rozdeleni;

    /** @var string     nacteno z podrizene tabulky PoznamkaDochazky */
    var $poznamka;

    /** @var int    Hodnota delta pred - prevedeno na sekundy */
    var $deltaPredSec;
    /** @var int    Hodnota delta za - prevedeno na sekundy */
    var $deltaZaSec;

    /** @var bool   Priznak, ze se ma tento zaznam smazat */
    var $smazatZaznam = false;

    // hodnoty prevedene na text (prevedene z databazovych hodnot nebo zadane uzivatelem)
    var $casPocatkuStr;
    var $casKonceStr;
    var $zapoctenyCasOdStr;
    var $zapoctenyCasDoStr;
    var $deltaPredStr;
    var $deltaZaStr;
    var $denStr;

    // priznaky chyb - pro zvyrazneni editu
    var $chybaCasPocatku;
    var $chybaCasKonce;
    var $chybaZapoctenyCasOd;
    var $chybaZapoctenyCasDo;
    var $chybaDen;

    /** @var bool       pouze pri vkladani zaznamu */
    var $chybaDatumDo;


    // funkce pro pristup k polozkam jako k  poli (pro XML)
    function offsetExists ( $offset ):bool
    {
        switch ( strtolower($offset) ) {
            case 'rozdeleni' :
            case 'poznamka' :
                return true;
        }

        return parent::offsetExists($offset);
    }


    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze prozatim nastavit typy
    function offsetGet ( $offset )
    {
        switch ( strtolower($offset) ) {
            case 'rozdeleni' :
                return $this ->rozdeleni;

            case 'poznamka' :
                return $this ->poznamka;
        }

        return parent::offsetGet($offset);
    }


    function offsetSet ( $offset , $value ):void
    {
        switch ( strtolower($offset) ) {
            case 'rozdeleni' :
                $this ->rozdeleni = $value;
                break;

            case 'poznamka' :
                $this ->poznamka = $value;
                break;

            default :
                parent::offsetSet($offset, $value);
        }
    }



    function loadRecordValues($row)
    {
        parent::loadRecordValues($row);

        $this ->casPocatkuStr = date('G:i', $this ->fDatumCasPocatku ->value);
        $this ->denStr = date('j.n.Y', $this ->fDen ->value);

        if ( $this ->fDatumCasUkonceni ->value == DateUtils::startOfTheDay($this ->fDatumCasUkonceni ->value)  &&  $this ->fDatumCasUkonceni ->value > $this ->fDatumCasPocatku ->value) {
            $this->casKonceStr = '24:00'; // 0:00 dalsiho dne
        } else {
            $this->casKonceStr = date('G:i', $this->fDatumCasUkonceni ->value);
        }

        $this ->chybaCasPocatku = false;
        $this ->chybaCasKonce = false;
        $this ->chybaZapoctenyCasOd = false;
        $this ->chybaZapoctenyCasDo = false;
        $this ->chybaDen = false;

        if ($this ->fDeltaPred ->value) {
            $this ->deltaPredStr = DateUtils::minutesToString($this ->fDeltaPred ->value / 60);
            $this ->deltaPredSec = round(24 * 60 * 60 * $this ->fDeltaPred ->value); // v sekundach
        } else {
            $this ->deltaPredStr = '';
            $this ->deltaPredSec = 0;
        }

        if ($this ->fDeltaZa ->value) {
            $this ->deltaZaStr = DateUtils::minutesToString($this ->fDeltaZa ->value / 60);
            $this ->deltaZaSec = round(24 * 60 * 60 * $this ->fDeltaZa ->value); // v sekundach;
        } else {
            $this ->deltaZaStr = '';
            $this ->deltaZaSec = 0;
        }

        $this ->zapoctenyCasOdStr = date('G:i', $this ->fDatumCasPocatku ->value  -  $this ->deltaPredSec);
        $zapoctenycasdo = $this ->fDatumCasUkonceni ->value  +  $this ->deltaZaSec;
        if ( $zapoctenycasdo == DateUtils::startOfTheDay($zapoctenycasdo)  &&  $zapoctenycasdo > $this ->fDatumCasPocatku ->value) {
            $this->zapoctenyCasDoStr = '24:00'; // 0:00 dalsiho dne
        } else {
            $this->zapoctenyCasDoStr = date('G:i', $zapoctenycasdo);
        }

        return true;
    }




    // -----     ZaznamDochazky     -------------------------------------------
    function nactiRozdeleni()
    {
        $this ->rozdeleni = '';

        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) return;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT KodRozdeleni FROM RozliseniDochazky' .
            ' WHERE RC = ' . $params->addParameter($this ->f_pp ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($this ->f_pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->f_pp ->datumNastupu)) .
            '   AND DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->value)) .
            '   AND KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)); // ostatni typy ignoruje

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) $this ->rozdeleni = $row[0];
        $cursor ->close();
    }


    // -----     ZaznamDochazky     -------------------------------------------
    function nactiPoznamku()
    {
        $this ->poznamka = '';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Poznamka FROM PoznamkaDochazky' .

            ' WHERE RC = ' . $params->addParameter($this ->f_pp ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($this ->f_pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($this ->f_pp ->datumNastupu)) .
            '   AND DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->value));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) $this ->poznamka = $row[0];
        $cursor ->close();
    }


    /**
     * @param bool   $insert    Urcuje, jestli se jedna o vlozeni noveho zaznamu
     * @param string $indexStr  Index v html (kvuli editaci celeho dne)
     * @param bool   $nacistZapoctenyCas
     * @return bool
     */
    function nactiDataPodlePost($insert, $indexStr = '', $nacistZapoctenyCas = true)
    {
        $result = true;

        $operace = null;

        if (!isset($_POST['operace' . $indexStr])  ||  ($_POST['operace' . $indexStr] == '')) {
            $result = false;
            MessageManager::addErrorMessage(\Ron\ronText('form.Missing_value') . \Ron\ronText('Operace'));
        } else {
            $operace = $_POST['operace' . $indexStr];
        }

        $datum = DateUtils::startOfTheDay($this ->fDatumCasPocatku ->value);

        // vice dnu, datum do (pouze pri vkladani)
        if (isset($_POST['provicednu'])) {
            $this ->f_ViceDnu = $_POST['provicednu'];
        }

        if ($this ->f_ViceDnu) {
            $this ->f_DatumDoStr = $_POST['datumdo'];
            $OK = true;
            try {
                $datumdo = DateUtils::dateStringToTimestamp($_POST['datumdo']);

                if ($datumdo < $datum) {
                    MessageManager::addErrorMessage( \Ron\ronText('Pocatecni_datum_vetsi_nez_koncove') );
                    $OK = false;
                }
                $this ->f_DatumDo = $datumdo;
            } catch (\Exception $e) {
                MessageManager::addErrorMessage( \Ron\ronText('Spatna_hodnota') . ' (' . $_POST['datumdo'] . ')' );
                $OK = false;
            }

            if (!$OK) {
                $this ->chybaDatumDo = true;
                $result = false;
            }
        }

        // zobrazeny cas od
        $caspocatku = 0;

        $this ->casPocatkuStr = $_POST['caspocatku' . $indexStr];
        try {
            $caspocatku = DateUtils::timeStringToTimestamp($this->casPocatkuStr, $datum);
            $this ->fDatumCasPocatku ->setNewValue($caspocatku);
        } catch (\Exception $e) {
            $result = false;
            $this ->chybaCasPocatku = true;
            MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota') . ' (' . $this ->casPocatkuStr . ')');
        }

        // zobrazeny cas do
        $this ->casKonceStr = $_POST['casukonceni' . $indexStr];

        try {
            $casukonceni = DateUtils::timeStringToTimestamp($this->casKonceStr, $datum, true); // enable_24_00 = true

            if ($casukonceni < $caspocatku) {
                // cas konce je mensi => posune ho na dalsi den
                //$this ->fDatumCasUkonceni ->setNewValue( DateUtils::incDay($casukonceni, 1) );
                // kvuli prechodu na letni cas je potreba pocitat s puvodni zadanou hodnotou a upravenym dnem
                $casukonceni = DateUtils::timeStringToTimestamp($this->casKonceStr, DateUtils::incDay($datum), true);
                $this ->fDatumCasUkonceni ->setNewValue( $casukonceni );
            } else {
                $this ->fDatumCasUkonceni ->setNewValue( $casukonceni );
            }
        } catch (\Exception $e) {
            $result = false;
            $this ->chybaCasKonce = true;
            MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota') . ' (' . $_POST['casukonceni' . $indexStr] . ')');
        }

        if ($nacistZapoctenyCas) {
            // zapocteny cas od
            $this ->zapoctenyCasOdStr = $_POST['zapoctenycasod' . $indexStr];
            if ($insert)
                if (!$this ->zapoctenyCasOdStr && !$this ->chybaCasPocatku) {
                    // pri insertu povoli prazdnou hodnotu v "zapocteny cas od", "... do"
                    // - v tom pripade pouzije stejny cas jako zobrazeny
                    $this ->zapoctenyCasOdStr = $this ->casPocatkuStr;
                }

            $zapoctenyCasOd = 0;
            try {
                $zapoctenyCasOd = DateUtils::timeStringToTimestamp($this->zapoctenyCasOdStr, $datum);

                $this ->deltaPredSec = $this ->fDatumCasPocatku ->value - $zapoctenyCasOd;
                $this ->fDeltaPred ->setNewValue( $this ->deltaPredSec ? $this ->deltaPredSec / 24 / 60 / 60 : null );
            } catch (\Exception $e) {
                $result = false;
                $this ->chybaZapoctenyCasOd = true;
                MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota') . ' (' . $_POST['zapoctenycasod' . $indexStr] . ')');
            }

            // zapocteny cas do
            $this ->zapoctenyCasDoStr = $_POST['zapoctenycasdo' . $indexStr];
            if ($insert)
                if (!$this ->zapoctenyCasDoStr && !$this ->chybaCasKonce) {
                    // pri insertu povoli prazdnou hodnotu v "zapocteny cas od", "... do"
                    // - v tom pripade pouzije stejny cas jako zobrazeny
                    $this ->zapoctenyCasDoStr = $this ->casKonceStr;
                }
            try {
                $zapoctenyCasDo = DateUtils::timeStringToTimestamp($this->zapoctenyCasDoStr, $datum, true); // enable_24_00 = true

                if ($zapoctenyCasDo < $zapoctenyCasOd) {
                    // cas konce je mensi => posune ho na dalsi den
                    //$zapoctenycasdo = DateUtils::incDay($zapoctenycasdo, 1);
                    // kvuli prechodu na letni cas je potreba znovu vypocitat cas podle zadane hodnoty
                    $zapoctenyCasDo = DateUtils::timeStringToTimestamp($this->zapoctenyCasDoStr, DateUtils::incDay($datum), true); // enable_24_00 = true
                }

                $this ->deltaZaSec = $zapoctenyCasDo - $this ->fDatumCasUkonceni ->value;
                $this ->fDeltaZa ->setNewValue( $this ->deltaZaSec ? $this ->deltaZaSec / 24 / 60 / 60 : null );
            } catch (\Exception $e) {
                $result = false;
                $this ->chybaZapoctenyCasDo = true;
                MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota') . ' (' . $_POST['zapoctenycasdo' . $indexStr] . ')');
            }

        } // zapocteny cas

        // den prirazeni
        $this ->denStr = $_POST['den' . $indexStr];
        try {
            $den = DateUtils::dateStringToTimestamp($this->denStr);

            $min = DateUtils::incDay($datum, -1);
            $max = DateUtils::incDayStart($datum, 1);

            // kontrola, jestli je den prirazeni +- 1 den od dne pocatku
            if ($den < $min  ||  $den > $max) {
                // den prirazeni je mimo povoleny rozsah
                $result = false;
                $this ->chybaDen = true;
                MessageManager::addErrorMessage( sprintf(\Ron\ronText('Den_prirazeni_v_rozsahu'), date('j.n.Y', $min), date('j.n.Y', $max)));
            } else {
                $this->fDen->setNewValue($den);
            }
        } catch (\Exception $e) {
            $result = false;
            $this ->chybaDen = true;
            MessageManager::addErrorMessage(\Ron\ronText('Spatna_hodnota') . ' (' . $this ->denStr . ')');
        }

        // rozdeleni
        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            $this ->rozdeleni = $_POST['rozdelenidochazky' . $indexStr];
            if (!$this ->rozdeleni)
                if ( isset($operace)  &&  EditaceUtils::jeRozdeleniDochazkyPovinne($this ->database, $operace) ) {
                    $result = false;
                    MessageManager::addErrorMessage(\Ron\ronText('Neni_zadano_rozdeleni_dochazky') . (RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) ? ' (' . RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) . ')' : ''));
                }
        }


        $this ->fOperace ->setNewValue($_POST['operace' . $indexStr]);
        $this ->fSmena ->setNewValue($_POST['smena' . $indexStr]);

        $changed = $this ->isChanged()  ||  ($this ->poznamka != $_POST['poznamka' . $indexStr]);

        $this ->poznamka = $_POST['poznamka' . $indexStr];

        if ($changed  &&  isset($operace)) {
            // zkontroluje pravo na editaci operace - nejdrive aktualne zadanou a v pripade potreby i tu puvodni
            if (!EditaceUtils::jePravoEditovatOperaci($operace)) {
                $result = false;
                MessageManager::addErrorMessage( \Ron\ronText('Neni_pravo_na_operaci'));
            } else if (isset($this ->fOperace ->originalValue)  &&  $this ->fOperace ->originalValue != $operace  &&  !EditaceUtils::jePravoEditovatOperaci($this ->fOperace ->originalValue)) {
                $result = false;
                MessageManager::addErrorMessage( \Ron\ronText('Neni_pravo_na_operaci'));
            }
        }

        return $result;
    }


    // -----     ZaznamDochazky     -------------------------------------------
    function ulozRozdeleni($insert, $pouzitOriginalniCasPocatku = true)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            // rozdeleni se nepouziva - zadna zmena
            return false;
        }

        $rec = new RozliseniDochazkyRecord($this ->database,
            $this ->f_pp,
            (!$insert && $pouzitOriginalniCasPocatku) ? $this ->fDatumCasPocatku ->originalValue : $this ->fDatumCasPocatku ->value,
            $this ->logDescription);
        return $rec ->zapisRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI), $this ->rozdeleni, $insert);
    }



    // -----     ZaznamDochazky     -------------------------------------------
    /**
     * @param string $name
     * @param bool   $puvodniCas
     * @param bool   $pridatHodnoty
     * @param string $parametry
     * @return bool
     */
    function executeScripts($name, $puvodniCas, $pridatHodnoty, $parametry = '')
    {
        if ($pridatHodnoty && $this ->rozdeleni) {
            if ($parametry) $parametry .= ';';
            $parametry .= 'Rozdeleni=' . $this ->rozdeleni;
        }

        return parent::executeScripts($name, $puvodniCas, $pridatHodnoty, $parametry);
    }


    function smazZaznam()
    {
        // volat udalosti ?
        $params = $this ->database ->createParameters();
        return $this ->deleteRecord(
            'DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue)),
            $params);
    }

    /**
     * Pokud doslo ke zmene, tak ulozi zmeneny zaznam a zaloguje. V pripade chyby (nemela by nastat) vraci false.
     * @return bool|int
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function ulozZmenenyZaznam()
    {
        if (!$this ->executeScripts(RonPortalScriptEvents::EDITACE_DOCHAZKY_IW_BEFORE, true, true)) {
            // vola se i kdyz nebylo zmeneno nic z hlavniho zaznamu (mohlo se zmenit napr. rozdeleni)
            return false;
        }

        if (!$this ->isChanged()) {
            return true;
        }

        if ( !$this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'update') ) {
            return false;
        }
        $params = $this ->database ->createParameters();
        return $this ->updateRecord(
            'DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($this ->fDatumCasPocatku ->originalValue)),
            $params);

        // skript ..._AFTER nevola - bude zavolan dodatecne - az po zapsani poznamky a rozdeleni
    }


    // -----     ZaznamDochazky     -------------------------------------------
    /**
     * Zapise poznamku k aktualnim zaznamu. Vraci true, pokud doslo ke zmene.
     * @param bool $insert
     * @param bool $pouzitOriginalniCasPocatku  Urcuje, jestli se ma pri editaci poznamky pouzit originalni cas pocatku
     * @return bool
     */
    function ulozPoznamku($insert, $pouzitOriginalniCasPocatku = true)
    {
        $rec = new PoznamkaDochazkyRecord($this ->database, $this ->f_pp,
            (!$insert && $pouzitOriginalniCasPocatku) ? $this ->fDatumCasPocatku ->originalValue : $this ->fDatumCasPocatku ->value,
            $this ->logDescription);
        return $rec ->zapisPoznamku($this ->poznamka, $insert);
    }


    /**
     * @return bool
     */
    function vlozNovyZaznam()
    {
        // $this ->urciDenPrirazeni($connection); - od 05-2015 zadava den prirazeni primo uzivatel

        if (!$this ->kontrolaPrekrytiZaznamuDochazky(true)) { // insert = true
            return false;
        }

        if ( !$this ->executeScripts(RonPortalScriptEvents::VLOZENI_DOCHAZKY_IW_BEFORE, false, true) ) return false;
        if ( !$this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_BEFORE, 'insert') ) return false;

        if ( RonPortalSettings::getIntValue(self::SETTINGS_PREPISOVAT) ) {
            // smaze nebo zkrati zaznamy, ktere se prekryvaji
            if ( !$this ->zkratPrekryvaneZaznamy(true) ) return false; // insert = true
        }

        if (!isset($this ->fDatumCasOperacePuvDoch ->value)) {
            $this->fDatumCasOperacePuvDoch->setNewValue($this->fDatumCasPocatku->value);
        }

        $this ->insertRecord();

        $this ->ulozPoznamku(true); // $insert = true
        $this ->ulozRozdeleni(true);

        if ( !$this ->executeScripts(RonPortalScriptEvents::VLOZENI_DOCHAZKY_IW_AFTER, false, false) ) ;
        if ( !$this ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'insert') ) ;
        // nevraci false - jinak by se znovu zobrazil formular pro vlozeni noveho zaznamu

        return true;
    }
}
