<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\PracovniPomer;

/**
 * Pomocny zaznam pro zpracovani zadaneho rozdeleni vysledku
 */
class ZaznamRozdeleniVysledku
{
    /** @var int */
    var $den;

    var $idPP;
    var $jmeno;
    var $prijmeni;
    var $osobniCislo;
    var $titul;

    /** @var string */
    var $rozdeleni;
    /** @var string */
    var $casovaSlozka;

    /** @var string     pouzije se v hromadnem zadavani */
    var $popisRozdeleni;
    var $popisCasoveSlozky;
    var $uzivatelska1;
    var $uzivatelska2;
    var $uzivatelska3;
    var $uzivatelska4;
    var $uzivatelska5;
    var $uzivatelska6;
    var $uzivatelska7;
    var $uzivatelska8;
    var $uzivatelska9;
    var $uzivatelska10;

    var $error_rozdeleni;
    var $error_casovaslozka;

    var $hodiny;
    /** @var string     zadany text */
    var $hodinyStr;
    var $error_hodiny;

    public $poznamka;

    /** @var bool   priznak, ze se ma radek smazat (uzivatel kliknul na ikonu x) */
    var $smazat;
    /** @var bool   priznak se pouzije pri zapisu */
    var $zapsano;
    /** @var bool   priznak, ze se jedna o novy radek */
    var $novy_radek;

    /** @var int    Priznak, ze uz je schvaleno rozdeleni vysledku pro tento den a kod rozdeleni (tabulka SchvaleniRozdeleniDoch) */
    var $rozdeleniSchvaleno;

    var $pravoSchvalovat;

    var $js_id; // pouzije se v javascriptu v hromadnem rozdeleni


    /**
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param PracovniPomer $pp
     * @param string[]|null $casoveSlozky   Seznam casovych slozek, ktere se maji nacitat. Zaznamy s jinou casovou slozkou ignoruje. Pokud je null, nacita vse. Kody musi byt malymi pismeny (strtolower)!
     * @param int $datumOd
     * @param int|null $datumDo
     * @return ZaznamRozdeleniVysledku[]
     * @throws DBException
     */
    public static function nactiZaznamyOsoby($database, $kodTypuRozdeleni, $pp, $casoveSlozky, $datumOd, $datumDo = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT DenniVysledek_R_E.Den, DenniVysledek_R_E.TypCasoveSlozkyMzdy, DenniVysledek_R_E.Rozdeleni, DenniVysledek_R_E.Hodiny, ' . // 0..3
            '   Rozdeleni.Popis, TypCasoveSlozkyMzdy.Popis, ' . // 4,5
            '   Rozdeleni.Uzivatelska1, Rozdeleni.Uzivatelska2, Rozdeleni.Uzivatelska3, Rozdeleni.Uzivatelska4, Rozdeleni.Uzivatelska5, ' . // 6..10
            '   Rozdeleni.Uzivatelska6, Rozdeleni.Uzivatelska7, Rozdeleni.Uzivatelska8, Rozdeleni.Uzivatelska9, Rozdeleni.Uzivatelska10, ' . // 11..15
            '   SchvaleniRozdeleniDoch.Schvaleno, ' . // 16
            '   DenniVysledek_R_E.Poznamka' . // 17
            ' FROM DenniVysledek_R_E INNER JOIN Rozdeleni' .
            '  ON Rozdeleni.KodTypuRozdeleni = DenniVysledek_R_E.KodTypuRozdeleni ' .
            '  AND Rozdeleni.Kod = DenniVysledek_R_E.Rozdeleni ' .
            ' INNER JOIN TypCasoveSlozkyMzdy ' .
            '  ON TypCasoveSlozkyMzdy.Kod = DenniVysledek_R_E.TypCasoveSlozkyMzdy ' .
            ' LEFT OUTER JOIN SchvaleniRozdeleniDoch ' .
            '  ON SchvaleniRozdeleniDoch.Den = DenniVysledek_R_E.Den ' .
            '  AND SchvaleniRozdeleniDoch.KodTypuRozdeleni = DenniVysledek_R_E.KodTypuRozdeleni ' .
            '  AND SchvaleniRozdeleniDoch.Rozdeleni = DenniVysledek_R_E.Rozdeleni ' .
            ' WHERE DenniVysledek_R_E.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            ' AND DenniVysledek_R_E.RC = ' . $params->addParameter($pp ->rc) .
            ' AND DenniVysledek_R_E.OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DenniVysledek_R_E.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
            ' AND DenniVysledek_R_E.Den >= ' . $params ->addDateParameter($datumOd) .
            ' AND DenniVysledek_R_E.Den <= ' . $params ->addDateParameter($datumDo ?? $datumOd) .
            ' ORDER BY DenniVysledek_R_E.Den, DenniVysledek_R_E.Rozdeleni';

        $cursor = $database ->getSqlResult($sql, $params);

        $itemid = 1;
        $result = [];

        while ($row = $cursor ->fetch()) {
            if (!$casoveSlozky  ||  in_array(strtolower($row[1]), $casoveSlozky, true)) {
                $item = new static();

                $item->den = $database->stringToTimestamp($row[0]);
                $item->casovaSlozka = $row[1];
                $item->rozdeleni = $row[2];
                $item->hodiny = $row[3];
                $item->hodinyStr = StringUtils::format('minuty', round($item->hodiny * 60));
                $item->popisRozdeleni = $row[4];
                $item->popisCasoveSlozky = $row[5];

                $item->uzivatelska1 = $row[6];
                $item->uzivatelska2 = $row[7];
                $item->uzivatelska3 = $row[8];
                $item->uzivatelska4 = $row[9];
                $item->uzivatelska5 = $row[10];
                $item->uzivatelska6 = $row[11];
                $item->uzivatelska7 = $row[12];
                $item->uzivatelska8 = $row[13];
                $item->uzivatelska9 = $row[14];
                $item->uzivatelska10 = $row[15];

                $item->rozdeleniSchvaleno = (int)$row[16];
                $item->poznamka = $row[17];

                $item->js_id = 'zaznamrozdeleniDV_' . $itemid;
                $itemid++;

                $result[] = $item;
            }
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param string $rozdeleni
     * @param int $datumOd
     * @param int $datumDo
     * @return ZaznamRozdeleniVysledku[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiZaznamyRozdeleni($database, $kodTypuRozdeleni, $rozdeleni, $datumOd, $datumDo = null)
    {
        if (!isset($datumDo)) $datumDo = $datumOd;

        $params = $database ->createParameters();

        $sql =
            'SELECT DenniVysledek_R_E.Den, DenniVysledek_R_E.TypCasoveSlozkyMzdy, DenniVysledek_R_E.Hodiny, ' . // 0..2
            '   DenniVysledek_R_E.OsobniCislo, PracovniPomer.Id, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . // 3..7
            '   TypCasoveSlozkyMzdy.Popis' . // 8
            ' FROM DenniVysledek_R_E INNER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = DenniVysledek_R_E.RC' .
            '  AND PracovniPomer.OsobniCislo = DenniVysledek_R_E.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = DenniVysledek_R_E.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '  ON Osoba.RC = DenniVysledek_R_E.RC' .
            ' INNER JOIN TypCasoveSlozkyMzdy ' .
            '  ON TypCasoveSlozkyMzdy.Kod = DenniVysledek_R_E.TypCasoveSlozkyMzdy ' .
            ' WHERE DenniVysledek_R_E.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            ' AND DenniVysledek_R_E.Rozdeleni = ' . $params ->addParameter($rozdeleni) .
            ' AND DenniVysledek_R_E.Den >= ' . $params ->addParameter($database ->timestampToParam($datumOd)) .
            ' AND DenniVysledek_R_E.Den <= ' . $params ->addParameter($database ->timestampToParam($datumDo)) .
            ' ORDER BY Den, Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.Id';

        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];

        while ($row = $cursor ->fetch()) {
            $item = new static();

            $item ->den = $database ->stringToTimestamp($row[0]);
            $item ->casovaSlozka = $row[1];
            $item ->hodiny = $row[2];
            $item ->hodinyStr = StringUtils::format('minuty', round($item ->hodiny * 60));
            $item ->osobniCislo = $row[3];
            $item ->idPP = (int)$row[4];
            $item ->jmeno = $row[5];
            $item ->prijmeni = $row[6];
            $item ->titul = $row[7];
            $item ->popisCasoveSlozky = $row[8];
            $item ->rozdeleni = $rozdeleni;

            $result[] = $item;
        }

        $cursor ->close();

        return $result;
    }
}
