<?php
/**
 * stanislav.elbl, 08.07.2015
 */

namespace RonPortal\ELearning;


use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\Log;



class KurzOsoby
{
    public $id;  // id zaznamu
    public $rc;  // osoba, ktere patri
    public $idRevize;
    public $idKurzu; // nacteno z revize
    public $stav;

    public $datumCasZahajeni;
    public $stranka;
    public $posledniPristup;

    // RC_Skoleni, DruhSkoleni, DatumVznikuPovinnosti
    public $rcSkoleni;
    public $druhSkoleni;
    public $datumVznikuPovinnosti;

    /**
     * @param Database $database
     * @param int $id
     * @return KurzOsoby
     * @throws \Exception
     */
    public static function load($database, $id)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT EKurzOsoby.Id, EKurzOsoby.RC, EKurzOsoby.IdRevize, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.Stranka, EKurzOsoby.PosledniPristup, ' . // 0..6
            '       RevizeEKurzu.IdKurzu, EKurzOsoby.RC_Skoleni, EKurzOsoby.DruhSkoleni, EKurzOsoby.DatumVznikuPovinnosti, ' . // 7..10
            '       RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.ulozitCertifikatDoDok' . // 11,12
            '  FROM EkurzOsoby, RevizeEKurzu ' .
            ' WHERE EKurzOsoby.Id = ' . $params->addParameter($id) .
            '   AND EKurzOsoby.IdRevize = RevizeEKurzu.Id';

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception("Zaznam EKurzOsoby nenalezen!");
        }

        $kurzOsoby = new KurzOsoby();
        $kurzOsoby->id = $row[0];
        $kurzOsoby->rc = $row[1];

        $kurzOsoby->idRevize = $row[2];
        $kurzOsoby->idKurzu = $row[7];
        $kurzOsoby->stav = $row[3];
        $kurzOsoby->datumCasZahajeni = $database->stringToTimestamp($row[4]);
        $kurzOsoby->stranka = $row[5];
        $kurzOsoby->posledniPristup = $database->stringToTimestamp($row[6]);

        $kurzOsoby ->rcSkoleni = $row[8];
        $kurzOsoby ->druhSkoleni = $row[9];
        $kurzOsoby ->datumVznikuPovinnosti = $database ->stringToTimestamp($row[10]);

        return $kurzOsoby;
    }


    /**
     * Ulozi posledni navstivenou stranku
     * @param Database $database
     * @param int $strana
     */
    function ulozStranku($database, $strana)
    {
        $params = $database ->createParameters();

        $sql =
            'UPDATE EKurzOsoby ' .
            ' SET Stranka = ' . $params ->addParameter($strana) . ', ' .
            ' PosledniPristup = ' . $params ->addTimestampParameter(time()) .
            ' WHERE Id = ' . $params ->addParameter($this ->id);

        try {
            $database->sql($sql, $params);
        } catch (DBException $e) {
            Log::addRecordToDBLog($database, 'EKurzOsoby', 'ERROR', 'ELearning', $e ->dbMessage . "\n\n" . $e ->getErrorDescription());
        }
    }


    /**
     * Nastavi stav v zaznamu EKurzOsoby
     * @param $database
     * @param $novyStav
     * @return void
     */
    function updateStav($database, $novyStav)
    {
        $params = $database ->createParameters();

        $sql =
            'UPDATE EKurzOsoby ' .
            ' SET Stav = ' . $params ->addParameter($novyStav) .
            ' WHERE Id = ' . $params ->addParameter($this ->id);

        $database->sql($sql, $params);
    }
}
