<?php
/**
 * stanislav.elbl, 25.11.2015
 */

namespace RonPortal\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Testy\Dotaznik;
use RonPortal\Testy\PripraveneTemaHodnoceniDBRecord;
use RonPortal\User;


/**
 * Vyplneny test k EKurzu - nacita se posledni vyplneny test
 */
class Test extends Dotaznik
{
    const TEST_STAV_NEABSOLVOVANO = 0;
    const TEST_STAV_PROBIHA = 1;
    const TEST_STAV_NEDOKONCENO_V_LIMITU = 2;
    const TEST_STAV_ABSOLVOVANO_USPESNE = 3;
    const TEST_STAV_ABSOLVOVANO_NEUSPESNE = 4;

    /** @var  int   Kurz Osoby (pri vyplnovani testu) */
    public $idKurzuOsoby;

    /** @var  int   Index stranky (pouziva se pri vyplnovani testu) */
    public $pageIndex;


    function getJson()
    {
        return json_encode($this ->getJsonData());
    }


    /**
     * @param Database $database
     * @param int $idPripravenehoTestu  ID zaznamu z tabulky PripraveneTemaHodnoceni
     * @return Test
     */
    private static function nactiTest(Database $database, $idPripravenehoTestu)
    {
        $test = new Test($database);
        $test ->nactiPripravenyDotaznik($idPripravenehoTestu);

        return $test;
    }


    /**
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param KurzOsoby $kurzOsoby
     * @param object|string $logDescription
     * @return Test
     * @throws DBException
     */
    public static function vygenerujTest(Database $database, $druhHodnoceni, $temaCislo, $kurzOsoby, $logDescription)
    {
        $rec = new PripraveneTemaHodnoceniDBRecord($database, $druhHodnoceni, $temaCislo, $logDescription);

        // pole pro elearning
        $rec ->addField('Test_Stav', 'int', false, 0, self::TEST_STAV_PROBIHA);
        $rec ->addField('Test_datumCasPocatku', 'timestamp', false, 0, time());
        $rec ->addField('EKurzOsoby', 'int', false, 0, $kurzOsoby ->id);

        $rec ->insertRecord();

        //return self::nactiTest($database, $newId);
        return self::nactiTest($database, $rec['ID']);
    }


    /**
     * Vraci instanci testu osoby k zadanemu kurzu osoby. Pokud neexistuje, vraci null
     * @param Database $database
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param KurzOsoby $kurzOsoby
     * @param string $filtr
     * @return Test
     */
    public static function nactiTestOsoby(Database $database, $druhHodnoceni, $temaCislo, $kurzOsoby, $filtr = null)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT ID FROM PripraveneTemaHodnoceni ' .
            ' WHERE EKurzOsoby = ' . $params ->addParameter($kurzOsoby ->id) .
            '   AND DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            '   AND TemaCislo = ' . $params ->addParameter($temaCislo);

        if ($filtr) {
            $sql .= ' AND ' . $filtr;
        }

        $sql .= ' ORDER BY Test_datumCasPocatku DESC'; // aby nacetl posledni pokus

        $cursor = $database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return null;
        }

        return self::nactiTest($database, $row[0]);
    }


    function zobrazHlavicku()
    {
        echo
            '<h2 class="nazevtestu">' . StringUtils::htmlTextConvert($this ->popisTematu) . '</h2>' .
            '<div class="poznamka popistestu">' . StringUtils::longTextToHtml($this ->poznamkaTematu, false) . '</div>';
    }



    /**
     * @param int $novyStav
     * @return bool|void
     * @throws DBException
     */
    function ulozVyplnenyTest($novyStav = 1) // 1=TEST_STAV_PROBIHA
    {
        if (!$this ->loadPost()) {
            return false;
        }

        $this ->ulozOdpovedi();

        // zapise jmeno hodnoticiho a cas
        $params = $this ->database ->createParameters();
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            '   SET HodnotilRC = ' . $params->addParameter(User::getRC()) . ', ' .
            '       Hodnotici = ' . $params->addParameter(User::getLoginName()) . ', ' .
            '       DatumHodnoceni = ' . $params ->addTimestampParameter(time()) . ', ' .
            '       Test_Stav = ' . $params ->addParameter($novyStav) . ', ' . // 1 = probiha
            '       Poznamka = ' . $params ->addParameter( $this ->poznamkaHodnoceni ) .
            ' WHERE Id = ' . $params ->addParameter((int)$this ->fId);

        $this ->database ->sql($sql, $params);

        $this ->test_stav = $novyStav;

        return true;
    }
}
