<?php
/**
 * stanislav.elbl, 25.11.2015
 */

namespace RonPortal\ELearning;

use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\Testy\DotaznikController;
use RonPortal\Testy\TestDBRecord;
use RonPortal\Testy\TypOdpovedi;


class TestyController extends DotaznikController
{
    public function overPravoNaDotaznik(int $druhHodnoceni, int $temaCislo)
    {
        // overi, ze se jedna o elearningovy test
        if ($druhHodnoceni !== 6) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @param int $idRevize Id revize e kurzu - pro nacteni informaci z TestEKurzu
     * @return string
     * @throws DBException
     */
    function zobrazTest($druhHodnoceni, $temaCislo, $idRevize)
    {
        $test = new Test($this ->database);
        $test ->nactiSablonu($druhHodnoceni, $temaCislo, true);
        $test ->nactiSpravneOdpovedi();

        Model::addData('test', $test);

        if ($idRevize) {
            $testEKurzu = TestEKurzu::nactiTestEKurzu($this ->database, $idRevize, $druhHodnoceni, $temaCislo);
            Model::addData('testEKurzu', $testEKurzu);

            $prilohy = PrilohaKurzu::loadPrilohy($this ->database, $idRevize);
            Model::addData('prilohy', $prilohy);
        }

//        $typyOdpovedi = TypOdpovedi::nactiTypyOdpovedi($this ->database, ($test ->druhHodnoceni == 6  ||  $test ->druhHodnoceni == 9  ||  $test ->druhHodnoceni == 10) ? 1 : 0); // nacte mozne testove odpovedi
//        Model::addData('typyOdpovedi', $typyOdpovedi);
        Model::addData('editace', true); // rezim editace - priznak pro layout

        Model::addViewObject( new BaseViewObject('ronportal/elearning/view_test.php') );

        HtmlHeaders::getInstance()
            ->addScript('tinymce/tinymce.min.js')
            ->addCss('css/elearning.css')
            ->addScript('script/elearning.js')
            ->addScript('script/testy.js');

        return 'ronportal/elearning/view_elearning.php';
        //return true;
    }


//    protected function loadData($output)
//    {
//        parent::loadData($output);
//
//        if (isset($_GET['revize'])) {
//            $prilohy = PrilohaKurzu::loadPrilohy($this ->database, (int)$_GET['revize']);
//            $output ->addData('prilohy', PrilohaKurzu::getJsonPrilohy($prilohy));
//        }
//    }


    function updateTest()
    {
        $druhHodnoceni = (int)$_POST['druhHodnoceni'];
        $temaCislo = (int)$_POST['temaCislo'];

        $this ->overPravoNaDotaznik($druhHodnoceni, $temaCislo);

        $rec = new TestDBRecord($this->database, $druhHodnoceni, $temaCislo, $this);
        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            ' AND TemaCislo = ' . $params ->addParameter($temaCislo),
            $params);

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba! Pozadovany test nebyl nalezen!');
        }

        $rec ->fKod ->setNewValue( StringUtils::fromUTF8($_POST['kod']) );
        $rec ->fPopisTematu ->setNewValue( StringUtils::fromUTF8($_POST['popis']) );
        $rec ->fPoznamka ->setNewValue( StringUtils::fromUTF8($_POST['poznamka']) );
        $rec ->fMinPocetBodu ->setNewValue( $_POST['minPocetBodu'] != '' ? (int)$_POST['minPocetBodu'] : null );
        $rec ->fMaxPocetBodu ->setNewValue( $_POST['maxPocetBodu'] != '' ? (int)$_POST['maxPocetBodu'] : null );
        $rec ->fSirkaSloupce ->setNewValue( $_POST['sirkaSloupce'] != '' ? (int)$_POST['sirkaSloupce'] : null );
        $rec ->fEditovatPoznamku ->setNewValue( (int)$_POST['editovatPoznamku'] );
        $rec ->fZobrazitOdpovediUspesne ->setNewValue( (int)$_POST['zobrazitOdpovediUspesne']);
        $rec ->fZobrazitOdpovediNeuspesne ->setNewValue( (int)$_POST['zobrazitOdpovediNeuspesne']);

        $rec ->updateRecord();

        if ((int)$_POST['idRevize']) {
            $rec2 = new RonPortalDBRecord($this->database, 'TestEKurzu', $this);
            $rec2->addField('IDRevize', 'int', true);
            $rec2->addField('DruhHodnoceni', 'int', true);
            $rec2->addField('TemaCislo', 'int', true);
            $rec2->addField('PocetPokusu', 'int', false, 0, $_POST['pocetPokusu'] == '' ? null : (int)$_POST['pocetPokusu']);
            $rec2->addField('CasovyLimit', 'int', false, 0, $_POST['casovyLimit'] == '' ? null : (int)$_POST['casovyLimit']);

            $updateParams = $this ->database ->createParameters();
            $rec2->setWhere(
                'DruhHodnoceni = ' . $updateParams ->addParameter($druhHodnoceni) .
                ' AND TemaCislo = ' . $updateParams ->addParameter($temaCislo) .
                ' AND IdRevize = ' . $updateParams ->addParameter((int)$_POST['idRevize']),
                $updateParams);

            $rec2->updateOrInsert();
        }
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'updatetest' :
                $this->updateTest();
                break;

            default :
                parent::handleJsonAction($action, $output);
                break;
        }
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('elearning.ini');

        HtmlHeaders::getInstance() ->setAppTitle(\Ron\ronText('E_Learning'));

        $this ->addTranslationsScript('kurzy');

        if (!Prava::vratHodnotuPrava('editacekurzu')) {
            throw new NeniPravoException();
        }

        switch ($_GET['action']) {
            case 'show' :
                return $this->zobrazTest( (int)$_GET['druhhodnoceni'], (int)$_GET['temacislo'], (int)$_GET['idrevize']);

            default :
                return parent::handleAction($action);
        }
    }
}
