<?php
/**
 * stanislav.elbl, 19.11.2015 (presunuto z person)
 * Rozsireni obecneho dotazniku o funkce specificke pro hodnoceni
 */
namespace RonPortal\Hodnoceni;

use Exception;
use Ron\DB\DBException;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\Testy\Dotaznik;
use RonPortal\UrovenPredavani;
use RonPortal\User;
use function Ron\ronText;

class Hodnoceni extends Dotaznik
{
    /** @var  Hodnotitel[] */
    public $hodnotitele = [];

    public $fHodnocenyJmeno = '';
    public $fHodnocenyPrijmeni = '';

    public $fPocatekObdobi = null;
    public $fKonecObdobi = null;

    /** @var string     Prijmeni osoby s ID z tabulky PripraveneTemaHodnoceni.HodnotilRC */
    public $hodnoticiPrijmeni = '';
    /** @var string     Jmeno osoby s ID z tabulky PripraveneTemaHodnoceni.HodnotilRC */
    public $hodnoticiJmeno = '';

    // id zaznamu s vysledky hodnoceni (tabulka VysledekHodnoceni)
    public $idVysledku;

    // predavaci proces
    /** @var string     Kod predavaciho procesu */
    public $kodProcesu = '';
    /** @var string     Pocatecni uroven predavaciho procesu */
    public $pocatecniUroven = '';
    /** @var int        Aktualni poradi predavani v predavacim procesu */
    public $poradiPredavani = 0;

    /** @var string textovy popis vysledku v aktualni urovni - pole HodnoceniOsoby.DetailVysledku */
    public $detailVysledku;


    /**
     * @param int $id
     * @throws DBException
     */
    protected function nactiHlavicku($id)
    {
        parent::nactiHlavicku($id);

        // Nacte dalsi udaje k hodnoceni (hodnoceni zamestnance pripadne hodnoceni skoleni)
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT HodnoceniOsoby.IdVysledku,' .
            '   Hodnoceny.Prijmeni, Hodnoceny.Jmeno,' . // 1, 2
            '   Hodnotici.Prijmeni, Hodnotici.Jmeno,' . // 3, 4
            '   VysledekHodnoceni.PocatekObdobi, VysledekHodnoceni.KonecObdobi,' . // 5, 6
            '   HodnoceniOsoby.KodProcesu, HodnoceniOsoby.PocatecniUroven, HodnoceniOsoby.PoradiPredavani,' . // 7, 8, 9
            '   HodnoceniOsoby.DetailVysledku' . // 10

            '  FROM ( ( ( ( PripraveneTemaHodnoceni LEFT OUTER JOIN Osoba Hodnoceny ' .
            '               ON Hodnoceny.RC = PripraveneTemaHodnoceni.HodnocenyRC' .
            '             ) ' .
            '             LEFT OUTER JOIN Osoba Hodnotici ' .
            '             ON Hodnotici.RC = PripraveneTemaHodnoceni.HodnotilRC ' .
            '           )' .
            '           LEFT OUTER JOIN HodnoceniOsoby' .
            '           ON HodnoceniOsoby.Id = PripraveneTemaHodnoceni.Id' .
            '         )' .
            '         LEFT OUTER JOIN VysledekHodnoceni ' .
            '         ON VysledekHodnoceni.Id = HodnoceniOsoby.IdVysledku' .
            '       )' .
            '       INNER JOIN TemaHodnoceni' .
            '       ON PripraveneTemaHodnoceni.DruhHodnoceni = TemaHodnoceni.DruhHodnoceni' .
            '       AND PripraveneTemaHodnoceni.TemaCislo = TemaHodnoceni.TemaCislo' .

            ' WHERE PripraveneTemaHodnoceni.Id = ' . $params ->addParameter($this ->fId);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $this->idVysledku = $row[0];

            $this->fHodnocenyPrijmeni = $row[1];
            $this->fHodnocenyJmeno = $row[2];

            $this->hodnoticiPrijmeni = $row[3];
            $this->hodnoticiJmeno = $row[4];

            $this->fPocatekObdobi = $this->database->stringToTimestamp($row[5]);
            $this->fKonecObdobi = $this->database->stringToTimestamp($row[6]);

            $this->kodProcesu = $row[7];
            $this->pocatecniUroven = $row[8];
            $this->poradiPredavani = $row[9];

            $this->detailVysledku = $row[10];
        }
    }
    
    
    /**
     * @param $id
     * @throws Exception
     */
    function nactiPripravenyDotaznik($id)
    {
        parent::nactiPripravenyDotaznik($id);

        $this ->hodnotitele = Hodnotitel::nactiHodnotitele($this ->database, $this ->idVysledku, $this ->temaCislo);
    }


    /**
     * Vraci true, pokud neni dotaznik na nejnizsi urovni predavani v predavacim procesu
     * @return bool
     */
    function existujeNizsiUroven()
    {
        return UrovenPredavani::existujeNizsiUrovenPredavani($this ->database,
            $this->kodProcesu, $this->pocatecniUroven, $this->poradiPredavani);
    }



    // Vraci true, pokud neni dotaznik na posledni urovni predavani v predavacim procesu
    function existujeVyssiUroven()
    {
        return UrovenPredavani::existujeVyssiUrovenPredavani($this ->database,
            $this->kodProcesu, $this->pocatecniUroven, $this->poradiPredavani);
    }


    function createXMLForm($hodnoceniAction)
    {
        $xmlForm = parent::createXMLForm($hodnoceniAction);

        $xmlForm->predavaciProces = $this->kodProcesu;
        $xmlForm->pocatecniUroven = $this->pocatecniUroven;
        $xmlForm->poradiPredavani = (int)$this->poradiPredavani;

        $xmlForm->rc = $this->hodnocenyRC;
        $xmlForm->pocatekObdobi = $this->fPocatekObdobi;
        $xmlForm->konecObdobi = $this->fKonecObdobi;

        return $xmlForm;
    }


    function loadPost()
    {
        $result = parent::loadPost();

        try {
            Csrf::validateCSRFPostParam('hodnoceni_' . $this ->fId);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
            return false;
        }

        return $result;
    }


    /**
     * Vyplni u zadaneho dotazniku pole HodnotilRC a Hodnotil (uzivatelske jmeno)
     * Pokud uz je to vyplneno a je tam nekdo jiny nez aktualni uzivatel, tak zobrazi dotaz, jestli chce uzivatel tento dotaznik prevzit
     * @param bool $prevzit   Pokud je true, je povoleno prevzeti
     * @return bool
     * @throws Exception
     */
    function nastavHodnotitele($prevzit)
    {
        // TODO: upravit ! ! !
        if (!$this->fId) {
            throw new Exception(StringUtils::fromUTF8('Chyba pri nastavovani hodnotitele - neni nacten dotaznik!'));
        }

        if ( $this->fXMLSablona) {
            // v XML sablone je mozne zabranit vyplneni (napr. uzivatelska kontrola prav)
            if (!$this->zobrazDotaznikXML(false, false, 'nastavhodnotitele')) {
                return false;
            }
        }

        $params = $this->database->createParameters();

        $sql =
            "UPDATE PripraveneTemaHodnoceni " .
            "  SET HodnotilRC = " . $params->addParameter(User::getRC()) . ", " .
            "   Hodnotici = " . $params->addParameter(RonPortalUtils::getLogUserName(80)) . ", " . //  User::getLoginName() SE 3.2.2025
            '   DatumHodnoceni = ' . $params->addTimestampParameter(time()) . ', ' .
            '   Stav = 1 ' . // = probiha
            '  WHERE Id = ' . $params->addParameter($this ->fId);

        if (!$prevzit) {
            // neumozni prevzit dotaznik, ktery uz hodnoti nekdo jiny.
            $sql .=
                " AND (HodnotilRC IS NULL OR (HodnotilRC = " . $params->addParameter(User::getRC()) . " AND IndividualniPoradi = 1 AND Stav IN (0,4)))"; //28.5.2015, 15.6.2015 pavel.rudol
            //  25.5.2015, 15.6.2015 Pokud hodnotí aktuálně přihlášený hodnotitel a stav ještě nebyl změněn z 0 nebo 4 provede změnu stavu.
        }

        $this ->database ->sql($sql, $params);

        $params = $this->database->createParameters();

        // Overi, jestli je aktualni uzivatel zapsany jako resitel
        $sql =
            'SELECT PripraveneTemaHodnoceni.HodnotilRC, Osoba.Prijmeni, Osoba.Jmeno, ' .
            '       PripraveneTemaHodnoceni.Hodnotici, PripraveneTemaHodnoceni.DatumHodnoceni, ' .
            '       PripraveneTemaHodnoceni.Stav ' .
            '  FROM PripraveneTemaHodnoceni, Osoba ' .
            ' WHERE Id = ' . $params->addParameter($this->fId) .
            '   AND Osoba.RC = PripraveneTemaHodnoceni.HodnotilRC ';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if (!$row) {
            throw new Exception(StringUtils::fromUTF8("Chyba při nastavování hodnotitele - zadaný záznam neexistuje!"));
        }

        // updatne hodnoty
        $this->hodnoticiPrijmeni = $row[1];
        $this->hodnoticiJmeno = $row[2];
        $this->hodnotici = $row[3];
        $this->fDatumHodnoceni = $this->database->stringToTimestamp($row[4]);
        $this->fStav = $row[5];

        if ($row[0] == User::getRC()) {
            // OK, uzivatel je nastaveny jako hodnotitel
            return true;
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ZAKAZAT_PREVZIT_HODNOCENI)) {
            throw new Exception(StringUtils::fromUTF8('Hodnocení má přiřazeno jiného hodnotitele!'));
        }


        return false;
    }


    function zobrazHlavicku()
    {
        require('ronportal/hodnoceni/view_hlavicka_hodnoceni.php');
    }


    function vratHTMLTlacitka($editace, $ulozitstr = null, $odeslatstr = null, $vratitstr = null, $uzavritstr = null)
    {
        if ($this->fPravo != 2) return ''; // pokud nemuze editovat, nic nezobrazi

        $result = '<DIV class="tlacitkahodnoceni">';

        // Tlacitka Odeslat a Upravit
        if ($editace) { //  || $postvalues) {
            if ($ulozitstr != 'null') {
                // tlacitko "odeslat" (ulozi odpovedi, ale nezmeni stav)
                if (!$ulozitstr) $ulozitstr = ronText('form.Store');

                if ($this->fId) $hlpstr = '';
                else $hlpstr = 'disabled';

                $prepocitatstr = ronText('Prepocitat');

                $result .= "<INPUT type='submit' class='submitbutton' $hlpstr value='$ulozitstr' />\n";
                $result .= "<INPUT type='submit' name='prepocitat' class='submitbutton' $hlpstr value='$prepocitatstr' />\n";
            }
        } else {
            if (($this->fPravo == 2) && // ma pravo editovat
                ($this->jeEditovatelnyStav())
            ) // neuzavreno
            {
                // moznost upravit ? ? ? - jen pokud mam pravo a neni uzavren
                //echo '<SPAN style="float:right"> ble </SPAN>';
                $result .= '<A class="button" href="?menuid=hodnoceni&#38;hodnoceniaction=editujhodnoceni&#38;record=' . $this->fId . '">' . ronText('Upravit_hodnoceni') . "</A>\n";
            }
        }


        // Tlacitka Uzavrit a Vratit
        if ($this->jeEditovatelnyStav()) // v reseni   // 0=pripraveno, 1=probiha, 2=ukonceno, 3=ukonceno-vraceno, 4=pripraveno (vraceno)
        {
            $result .= '<SPAN class="float-right">';

            $editace = (int)$editace; // prevede prazdnou hodnotu na 0 - jinak je prazdny parametr javascriptu a nefachci to

            if ($vratitstr != 'null') {
                if ($this->existujeNizsiUroven()) {
                    if (!$vratitstr) {
                        $vratitstr = ronText('Vratit_hodnoceni');
                    }

                    $result .= '<INPUT type="submit" name="vratit" class="submitbutton" data-ron-confirm-id="Opravdu_vratit_hodnoceni" value="' . $vratitstr . '" />';
                }
            }

            // pokud neni vyplneno vse, nepovoli uzavrit
            // pokud se ale jedna o editaci, tak uzavreni povoli, protoze uzivatel mohl chybejici odpovedi doplnit
            if ($this->fVyplnenoVse || $editace) {
                $btntext = $uzavritstr ?: ronText('Uzavrit_hodnoceni');
                $msgId = 'Opravdu_uzavrit_hodnoceni';

                if ($this->existujeVyssiUroven()) {
                    // pokud se dotaznik predava dale, zobrazi se "Odeslat" misto "Uzavrit"
                    $msgId = 'Opravdu_odeslat_hodnoceni';
                    $btntext = $odeslatstr ?: ronText('Odeslat_odpovedi');
                }

                if ($msgId != 'null') {
                    $result .= '<INPUT type="submit" name="uzavrit" data-ron-confirm-id="' . $msgId . '" class="submitbutton" value="' . $btntext . '" />';
                }
            } else {
                $result .= ronText('Chybi_odpovedi');
            }

            $result .= '</SPAN>';
        }

        $result .= "</DIV>\n";

        return $result;
    }


    /**
     * @param bool $editace
     */
    function zobrazTlacitka($editace)
    {
        echo $this->vratHTMLTlacitka($editace);
    }

//    function vratAkciDotazniku()
//    {
//        return "?menuid=hodnoceni&record={$this->fId}";
//    }

//    /**
//     * @param bool $editace
//     * @param bool $postValues
//     * @throws Exception
//     */
//    function zobrazDotaznik($editace, $postValues)
//    {
//        $formAction = $this->vratAkciDotazniku();
//
//        echo "<FORM name='dotaznik' class='inputform' method='post' action='$formAction'>\n";
//
//        if ($editace) {
//            $action = 'ulozdotaznik';
//        } else if ($this->jeEditovatelnyStav()) {
//            $action = 'uzavridotaznik';
//        } else {
//            $action = '';
//        }
//
//        echo Csrf::csrfHiddenInput('hodnoceni_' . $this ->fId);
//
//        echo "<INPUT type='hidden' name='hodnoceniaction' value='$action'>\n";
//
//        if ($this->fXMLSablona) {
//            $this->zobrazDotaznikXML($editace, $postValues);
//        } else {
//            $this->zobrazHlavicku();
//
//            $this->zobrazObsahDotazniku($editace);
//        }
//
//        echo "</FORM>\n";
//    }

}
