<?php
/**
 * stanislav.elbl, 26.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DB\DBException;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NastaveniUzivatele;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class DenJidelnicku
{
    var $datum;

    /** @var PolozkaJidelnicku[] */
    var $polozky = [];

    /**
     * DenJidelnicku constructor.
     * @param $datum
     */
    public function __construct($datum)
    {
        $this->datum = $datum;
    }

}


class Jidelnicek
{
    /** @var  CenovaKategorie */
    var $cenovaKategorie;

    var $datumOd;
    var $datumDo;

    /** @var  DenJidelnicku[] */
    var $data;

    var $jeFotografie;


    /**
     * Jidelnicek constructor.
     * @param CenovaKategorie $cenovaKategorie
     */
    public function __construct($cenovaKategorie)
    {
        $this ->cenovaKategorie = $cenovaKategorie;
        $this ->data = [];
    }


    /**
     * @param Database $database
     * @param PolozkaJidelnicku $polozka
     * @throws DBException
     */
    function nastavZobrazenouCenu($database, $polozka)
    {
        $polozka ->zobrazenaCena = '';

        if ($polozka->f_cenajidlaA) {
            $zobrazenaCena = $this ->cenovaKategorie
                ? $this ->cenovaKategorie->typZobrazeneCenyProTypJidla($polozka->f_typdennihojidla)
                : 1; // neni cenova kategorie (frame) => jen plna cena

            switch ($zobrazenaCena) {
                case 0 :
                    break; // nezobrazovat cenu

                case 1 :
                    $polozka ->zobrazenaCena = StringUtils::format('curr', $polozka->f_cenajidlaA);
                    break; // plna cena

                /** @noinspection PhpMissingBreakStatementInspection */
                case 3 :
                    $polozka ->zobrazenaCena = StringUtils::format('curr', $polozka->f_cenajidlaA) . ' / '; // NE BREAK! plna cena i dotovana
                case 2 :
                    $polozka ->zobrazenaCena .= StringUtils::format('curr', $this ->cenovaKategorie ->vratDotovanouCenuProTypJidla($polozka ->f_typdennihojidla, $polozka-> f_cenajidlaA));
            }
        }
    }

    /**
     * @param Database $database
     * @param string $jidelna
     * @param int $datum
     * @param PracovniPomer $pp
     * @param string[] $vybraneTypy
     * @return DenJidelnicku
     * @throws DBException
     */
    private function nactiDenJidelnicku($database, $jidelna, $datum, $pp, $vybraneTypy)
    {
        if ($this ->cenovaKategorie) {
            $this->cenovaKategorie->overNacteniVlastnostiKDatu($database, $datum);
        }

        $den = new DenJidelnicku($datum);

//        $vybraneTypy = '';
//        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
//            $vybraneTypy = NastaveniUzivatele::getUserSettingsItem($database, 'jidelnicek_vybrane_typy');
//        }

        $sql = PolozkaJidelnicku::sqlProPolozkyJidelnicku($database, 0,
            $datum, $jidelna, $this ->cenovaKategorie ? $this ->cenovaKategorie ->f_kod : null,
            $vybraneTypy);

        $cursor = $database ->getSqlResult($sql);

        while ($row = $cursor ->fetch()) {
            $polozka = new PolozkaJidelnicku();
            $polozka->nactiPolozkuZRadku($database, $row);

            if ($polozka ->kodFotografie) {
                $this ->jeFotografie = true;
            }

            $polozka->nactiPocetObjednavekCelkem($database);  // celkovy pocet objednavek - nacita se, jen kdyz je zapnuto omezeni

            if ( RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA) ) {
                $polozka->nactiInfoZBurzy($database, $pp); // toto by se dalo optimalizovat (nacitat hromadne)!
            }

            $this ->nastavZobrazenouCenu($database, $polozka);

            $den ->polozky[] = $polozka;
        }


        return $den;
    }


//    /**
//     * Nacte pocty objednavek osoby pro vsechny polozky jidelnicku v obdobi
//     * @param $database
//     */
//    function doplnPoctyObjednavekOsoby($database)
//    {
//        /** @var array $poctyObjednavekOsoby Pocty objednavek jednotlivych polozek jidelnicku - nacitaji se hromadne kvuli rychlosti */
//        $poctyObjednavekOsoby = JidelnaUtils::nactiPoctyObjednavekOsoby($database, $this ->datumOd, $this ->datumDo);
//
//        foreach ($this ->data as $denJidenicku) {
//            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
//                $polozkaJidelnicku ->f_pocetobjednavekosoby = isset($poctyObjednavekOsoby[$polozkaJidelnicku ->f_id])
//                    ? $poctyObjednavekOsoby[$polozkaJidelnicku ->f_id]
//                    : 0;
//            }
//        }
//    }

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     */
    function nastavObjednavani($database, $pp)
    {
        if (!$this ->cenovaKategorie) {
            return;
        }

        $manager = new ObjednavkyManager($database);
        $manager ->nactiData($pp,
            $this ->datumOd, $this ->datumDo);

        // nejdrive doplni pocty objednavek ...
        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);
            }
        }

        // ... teprve pak nastavi objednavani
        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $manager ->nactiStavObjednavani($polozkaJidelnicku);
            }
        }
    }


    /**
     * Nacte hodnoceni polozek pro vsechny polozky jidelnicku v obdobi
     * @param Database $database
     * @throws Exception
     */
    private function doplnHodnoceni($database)
    {
        $hodnoceni = PolozkaJidelnicku::nactiHodnoceniPolozek($database, $this ->datumOd, $this ->datumDo);

        foreach ($this ->data as $denJidenicku) {
            foreach ($denJidenicku ->polozky as $polozkaJidelnicku) {
                $hodnoceniPolozky = $hodnoceni[$polozkaJidelnicku->f_id] ?? null;

                if ($hodnoceniPolozky) {
                    $polozkaJidelnicku ->hodnoceniExistuje = true;
                    $polozkaJidelnicku ->hodnoceni = $hodnoceniPolozky ->hodnoceni;
                    $polozkaJidelnicku ->hodnoceniPoznamka = $hodnoceniPolozky ->poznamka;
                }
            }
        }
    }


    /**
     * @param $database
     * @throws Exception
     */
    private function doplnAlergeny($database)
    {
        $seznamAlergenu = PolozkaJidelnicku::nactiAlergenyPolozek($database, $this ->datumOd, $this ->datumDo);

        foreach ($this ->data as $denJidelnicku) {
            foreach ($denJidelnicku ->polozky as $polozkaJidelnicku) {
                $polozkaJidelnicku ->alergeny = $seznamAlergenu[$polozkaJidelnicku->f_id] ?? [];
            }
        }
    }


    /**
     * @param Database $database
     * @param string $jidelna
     * @param int $datumOd
     * @param int $datumDo
     * @param PracovniPomer|null $pp
     * @param string[] $vybraneTypy
     * @throws Exception
     */
    function load(Database $database, $jidelna, $datumOd, $datumDo, $pp, $vybraneTypy)
    {
        $this ->datumOd = $datumOd;
        $this ->datumDo = $datumDo;

        $this ->data = [];
        $this ->jeFotografie = false;

        if (!isset($jidelna)) {
            MessageManager::addWarning(TextManager::_('Vyberte_jidelnu'));
            return;
        }

        JidelnaUtils::nastavCacheObjednavek($datumOd, $datumDo, $jidelna);

        $params = $database ->createParameters();
        $sql =
            'SELECT Den FROM Jidelnicek WHERE Potvrzeno = 1 ' .
            ' AND Den >= ' . $params ->addDateParameter($datumOd) .
            ' AND Den <= ' . $params ->addDateParameter($datumDo);

        if ($jidelna) {
            $sql .=
                ' AND KodTypuRozdeleni = 8 AND jidelna =' . $params->addParameter($jidelna);
        }

        $sql .=
            ' ORDER BY Den';

        $cursor = $database ->select($sql, $params);
        while ($row = $cursor ->fetch()) {
            $this ->data[] = $this ->nactiDenJidelnicku($database, $jidelna, $database ->stringToTimestamp($row[0]), $pp, $vybraneTypy);
        }
        $cursor ->close();

        if ($pp && Prava::vratPrava('objednani') ->jsouNejakePrava()) {
            //$this->doplnPoctyObjednavekOsoby($database);
            $this->nastavObjednavani($database, $pp);  // az po doplnPoctyObjednavekOsoby!
        }

        $this ->doplnHodnoceni($database);
        $this ->doplnAlergeny($database);
    }
}
