<?php
/**
 * stanislav.elbl, 26.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\HttpResponse;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDny;
use RonPortal\Controls\FilterFormFilterSwitchLinks;
use RonPortal\Controls\FilterFormFilterTyden;
use RonPortal\Controls\PodrobnostiDnuSqlBuilder;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use Tracy\Debugger;


class TypyJidelFilter extends FilterFormFilterSwitchLinks
{
    private function getTypyJidelSQL($database)
    {
        $params = $database->createParameters();

        if ($this->filterForm -> getFilterValue('osoba')) {
            // nacte jen typy jidel povolene v cenove kategorii prihlaseneho zamestnance
            $sql =
                'SELECT DISTINCT TypDennihoJidla.Kod, TypDennihoJidla.Popis, TypDennihoJidla.Poradi' .
                ' FROM TypDennihoJidla, PravidlaCenoveKategorie, RozdeleniOsob, VlastnostCenoveKategorie' .
                " WHERE TypDennihoJidla.Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                // odfiltruje typy jidel, ktere se namaji zobrazovat
                '  AND TypDennihoJidla.Kod = PravidlaCenoveKategorie.TypDennihoJidla' .
                '  AND PravidlaCenoveKategorie.PovolitObjednavky = 1' .
                // jen pravidla platna v zadanem rozsahu dnu ...
                '  AND PravidlaCenoveKategorie.KodTypuRozdeleni = VlastnostCenoveKategorie.KodTypuRozdeleni' .
                '  AND PravidlaCenoveKategorie.CenovaKategorie = VlastnostCenoveKategorie.CenovaKategorie' .
                '  AND PravidlaCenoveKategorie.PlatnostOd = VlastnostCenoveKategorie.PlatnostOd' .
                '   AND VlastnostCenoveKategorie.PlatnostOd <= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiDo()) .
                '   AND (VlastnostCenoveKategorie.PlatnostDo >= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiOd()) . ' OR VlastnostCenoveKategorie.PlatnostDo IS NULL) ' .
                // ... prirazena aktualne vybrane osobe
                '   AND VlastnostCenoveKategorie.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '   AND VlastnostCenoveKategorie.CenovaKategorie = RozdeleniOsob.Rozdeleni ' .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this->filterForm ->getFilterValue('osoba', 'rc')) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiDo()) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiOd()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                // seradit podle poradi
                ' ORDER BY TypDennihoJidla.Poradi';
        } else {
            $sql =
                'SELECT Kod, Popis FROM TypDennihoJidla' .
                " WHERE Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                ' ORDER BY Poradi';
        }

        return new SqlStatement($sql, $params);
    }

    public function load(Database $database)
    {
        $this ->setOptionsSql($this ->getTypyJidelSQL($database));

        parent::load($database);
    }
}


class JidelnaFilter extends FilterFormFilter
{
    public function __construct()
    {
        parent::__construct('vyberjidelny');

        // sql dotaz se nastavi az dodatecne
        $this
            ->setOptionsSqlOrder('Popis')
            ->setOptionsSqlFilter('1 = 1');
    }

    /**
     * Zjisti, zda ma zamestnanec nastavenou jednu hlavni jidelnu. Pokud ano, tak vraci jeji kod.
     * Pokud nema zadnou hlavni nebo ma vice hlavnich, vraci null.
     * POZOR! Uz musi byt nastaven zamestnanec!
     * @return string|null
     * @throws DBException
     */
    private function vratHlavniJidelnuZamestnance($database)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis, RozdeleniOsob.Hlavni, RozdeleniOsob.PlatnostOd ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this ->filterForm ->getOmezitObdobiDo()) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            '   AND RozdeleniOsob.Hlavni = 1' .
            '   AND (' . $this ->optionsSqlFilter  . ')' .
            ' ORDER BY RozdeleniOsob.Hlavni DESC, RozdeleniOsob.PlatnostOd DESC';

        $cursor = $database ->getSqlResult($sql, $params);

        $pouzitHlavni = true;
        $hlavniJidelna = null;

        while ($pouzitHlavni  &&  $row = $cursor->fetch()) {
            if (!isset($hlavniJidelna)) {
                // prvni hlavni jidelna
                $hlavniJidelna = $row[0];
            } else {
                // je vice hlavnich jidelen
                $pouzitHlavni = false;
                $hlavniJidelna = null;
            }
        }

        $cursor ->close();

        return $hlavniJidelna;
    }


    public function load(Database $database)
    {
        parent::load($database);

        if (!isset($this ->value)  &&  $this ->filterForm ->getFilterValue('osoba')) {
            $this ->value = $this ->vratHlavniJidelnuZamestnance($database) ?? $_COOKIE['jidelna_jidelna'] ?? null;
        }

        if (($_GET[$this ->name] ?? null) === $this ->value) {
            // cookie nastavi jen v pripade, ze lze vybrat jidelnu, kterou zvolil uzivatel
            HttpResponse::setCookie('jidelna_jidelna', $this ->value, time() + 30000000);
        }
    }


    public function loadOptions(Database $database): FilterFormFilter
    {
        $optionsLoaded = false;

        if (!$this ->optionsSql  &&  $this ->filterForm ->getFilterValue('osoba')) {
            $params = $database->createParameters();
            $this->optionsSql = new SqlStatement(
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.Hlavni ' .
                ' FROM Rozdeleni, RozdeleniOsob ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
                '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiDo()) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiOd()) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this->filterForm ->getOmezitObdobiOd()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                '   AND (%FILTER%)' .
                ' ORDER BY %ORDER%',
                $params
            );

            parent::loadOptions($database);
            $optionsLoaded = true;
        }
        if (!$optionsLoaded  ||  !$this ->items) {
            if ($optionsLoaded  ||  !$this ->optionsSql) {
                $params = $database->createParameters();
                $this->optionsSql = new SqlStatement(
                    'SELECT Rozdeleni.Kod, Rozdeleni.Popis, 0 ' .
                    ' FROM Rozdeleni ' .
                    ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
                    '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this->filterForm->getOmezitObdobiOd()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                    '   AND (%FILTER%)' .
                    ' ORDER BY %ORDER%',
                    $params
                );
            }

            parent::loadOptions($database);
        }

        return $this;
    }
}


class JidelnicekFilterForm extends RonPortalFilterForm implements PodrobnostiDnuSqlBuilder
{
    /** @var FilterFormFilterTyden */
    public $datumJidelnickuFilter;

//    public $jidelny = [];

//    /** @var int */
//    public $datumJidelnickuOd;
//    /** @var int */
//    public $datumJidelnickuDo;

//    /** @var array Seznam typu jidel: key = kod typu jidla, value = popis typu jidla */
//    private $typyJidel = [];
//    /** @var array vybrane typy, jen kody */
//    private $vybraneTypy = [];

    private $podrobnostiSqlSelect = 'COUNT(*)';

//    public $vybranaJidelna = null;

    /***
     * @param Database $database
     * @param bool $zobrazitObjednavky
     */
    function __construct($database, $zobrazitObjednavky)
    {
        parent::__construct($database, 'jidelnicek', true);

        $this ->getAction = $_SERVER['SCRIPT_NAME']; // kvuli odstraneni #..., jinak by nebylo potreba

        if ($zobrazitObjednavky) {
            $prava = Prava::vratPrava('objednani');

//            if ($prava->jsouNejakePrava()) {
//                $this->nastavOsoby(true, $prava, time(), time(), true); // vcetne externich = true
//            }
            $this ->addFilterPP(false, $prava)
                ->setVcetneExternich(true)
                ->setSessionName('jidelnicek_vybranaosoba');
                //->setSessionName('');

            $this ->addFilter(new StavUctuFilter());
        } else {
            $this ->skrytTlacitkoNastavit = true;
        }

        $this ->addFilter(new JidelnaFilter());


        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
            $this->addFilter((new TypyJidelFilter('jidelnicek_typ'))
                ->setSessionName('jidelnicek_typ')
                //->setUserSettingsName('jidelnicek_typ')
                ->addItem('', TextManager::_('Vse'))
            );
        }


        if ($_GET['frame'] ?? false) {
            $this ->addFilter(
                (new FilterFormFilter('frame', FilterFormFilter::TYPE_HIDDEN))
                    -> setDefaultValue('1')
            );
        }

        $this ->addFilterObdobi('JIDELNA')
            ->setEnabled(RonPortalSettings::getIntValue('jidelnicek_zobrazit_po_dnech'));

        $this ->addFilter(
            (new FilterFormFilterDny($this, 'den'))
                ->setSqlPodrobnostiDnu($this)
                ->setEnabled(RonPortalSettings::getIntValue('jidelnicek_zobrazit_po_dnech'))
        );

        $this ->datumJidelnickuFilter = (new FilterFormFilterTyden())
            ->setSessionName('datumJidelnicku')
            ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
            ->setEnabled(!RonPortalSettings::getIntValue('jidelnicek_zobrazit_po_dnech'));

        $this ->addFilter($this ->datumJidelnickuFilter);
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key == 'podrobnostiSqlSelect') {
            $this ->podrobnostiSqlSelect = $valueText;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    /**
     * Sestavi dotaz pro nacteni podrobnosti ke dnum
     * @param Database $database
     * @return SqlStatement
     */
    public function getSqlPodrobnostiDnu(Database $database)
    {
        if ($this ->podrobnostiSqlSelect  &&  $this->getFilterValue('osoba')) {
            $params = $this->database->createParameters();
            return new SqlStatement(
                "SELECT PolozkaJidelnicku.Den, {$this ->podrobnostiSqlSelect}" .
                //"SELECT PolozkaJidelnicku.Den, STRING_AGG(PolozkaJidelnicku.TypDennihoJidla, ', ')" .
                " FROM PolozkaJidelnicku, ObjednaneJidlo" .
                " WHERE ObjednaneJidlo.IDPolozkyJidelnicku = PolozkaJidelnicku.Id" .
                " AND PolozkaJidelnicku.Den >= " . $params->addDateParameter($this->getOmezitObdobiOd()) .
                " AND PolozkaJidelnicku.Den <= " . $params->addDateParameter($this->getOmezitObdobiDo()) .
                " AND PolozkaJidelnicku.Jidelna = " . $params ->addParameter($this ->getFilterValue('vyberjidelny')) .
                " AND ObjednaneJidlo.RC = " . $params->addParameter($this->getFilterValue('osoba', 'rc')) .
                " AND ObjednaneJidlo.OsobniCislo = " . $params->addParameter($this->getFilterValue('osoba', 'osobniCislo')) .
                " AND ObjednaneJidlo.DatumNastupu = " . $params->addDateParameter($this->getFilterValue('osoba', 'datumNastupu')) .
                " AND ObjednaneJidlo.DatumCasZruseni IS NULL" .
                " GROUP BY PolozkaJidelnicku.Den",
                $params
            );
        }

        return null;
    }


//    /**
//     * @throws DBException
//     */
//    private function loadTypyJidel()
//    {
//        $params = $this ->database ->createParameters();
//
//        if ($this ->getFilterValue('osoba')) {
//            // nacte jen typy jidel povolene v cenove kategorii
//            $sql =
//                'SELECT DISTINCT TypDennihoJidla.Kod, TypDennihoJidla.Popis, TypDennihoJidla.Poradi' .
//                ' FROM TypDennihoJidla, PravidlaCenoveKategorie, RozdeleniOsob, VlastnostCenoveKategorie' .
//                " WHERE TypDennihoJidla.Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
//                // odfiltruje typy jidel, ktere se namaji zobrazovat
//                '  AND TypDennihoJidla.Kod = PravidlaCenoveKategorie.TypDennihoJidla' .
//                '  AND PravidlaCenoveKategorie.PovolitObjednavky = 1' .
//                // jen pravidla platna v zadanem rozsahu dnu ...
//                '  AND PravidlaCenoveKategorie.KodTypuRozdeleni = VlastnostCenoveKategorie.KodTypuRozdeleni' .
//                '  AND PravidlaCenoveKategorie.CenovaKategorie = VlastnostCenoveKategorie.CenovaKategorie' .
//                '  AND PravidlaCenoveKategorie.PlatnostOd = VlastnostCenoveKategorie.PlatnostOd' .
//                '   AND VlastnostCenoveKategorie.PlatnostOd <= ' . $params ->addDateParameter($this ->getOmezitObdobiDo()) .
//                '   AND (VlastnostCenoveKategorie.PlatnostDo >= ' . $params ->addDateParameter($this ->getOmezitObdobiOd()) . ' OR VlastnostCenoveKategorie.PlatnostDo IS NULL) ' .
//                // ... prirazena aktualne vybrane osobe
//                '   AND VlastnostCenoveKategorie.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
//                '   AND VlastnostCenoveKategorie.CenovaKategorie = RozdeleniOsob.Rozdeleni ' .
//                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->getFilterValue('osoba', 'rc')) .
//                '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->getFilterValue('osoba', 'osobniCislo')) .
//                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->getFilterValue('osoba', 'datumNastupu')) .
//                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->getOmezitObdobiDo()) .
//                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->getOmezitObdobiOd()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
//                // seradit podle poradi
//                ' ORDER BY TypDennihoJidla.Poradi';
//        } else {
//            $sql =
//                'SELECT Kod, Popis FROM TypDennihoJidla' .
//                " WHERE Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
//                ' ORDER BY Poradi';
//        }
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        //nutne pro porovnani pri ukladani uz. nastaveni, v poli musi byt kody
//        $vsechnyTypy = [];
//        $ulozitFiltry = false;
//
//        while ($row = $cursor->fetch()) {
//            if (isset($_GET['jidelnicek_typ_' . $row[0]])) {
//                $this->vybraneTypy[] = $row[0];
//                $ulozitFiltry = true;
//            }
//            $vsechnyTypy[] = $row[0];
//
//            $this->typyJidel[$row[0]] = $row[1];
//        }
//
//        //nastaveni se ulozi jen pri odeslani filtru
//        if ($ulozitFiltry) {
//            NastaveniUzivatele::setUserSettingsItem($this->database, 'jidelnicek_vybrane_typy', implode(';', $this->vybraneTypy), implode(';', $vsechnyTypy));
//        }
//        //jinak se vycte z uzivatelskeho nastaveni
//        else {
//            $nastaveniTypu = NastaveniUzivatele::getUserSettingsItem($this->database, 'jidelnicek_vybrane_typy');
//            if ($nastaveniTypu != '') {
//                $this->vybraneTypy = explode(';', $nastaveniTypu);
//            }
//        }
//    }



//    function loadData()
//    {
//        $this ->jidelny = [];
//
//        parent::loadData();
//
//        $this->loadTypyJidel();
//    }



//    private function renderTypyJidel()
//    {
//        echo '<div class="filterrow">';
//        echo '<SPAN class="filtercaption"></SPAN>';
//        echo '<SPAN class="filterformtext">';
//
//        foreach ($this->typyJidel as $kodTypu => $popisTypu) {
//            //pokud je uz. nastaveni prazdne jsou vybrane vsechny typy
//            GuiUtils::renderCheckbox('jidelnicek_typ_' . $kodTypu, $popisTypu, empty($this->vybraneTypy) ? true : in_array($kodTypu, $this->vybraneTypy), true);
//        }
//
//        echo '</SPAN>';
//        echo "</DIV>\n";
//    }

//    public function renderFilters()
//    {
//        parent::renderFilters();
//
//        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
//            $this->renderTypyJidel();
//        }
//    }

}
