<?php
/**
 * stanislav.elbl, 1.6.2016
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Message;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\Rozdeleni;
use RonPortal\System\AgentManager;


class Objednavka
{
    public $idObjednavky;
    public $idPolozkyJidelnicku;

    public $datum;
    public $typDennihoJidla;

    public $datumCasVydani;

    /**
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param int $datumOd
     * @param int $datumDo
     * @return Objednavka[]
     * @throws DBException
     */
    public static function nactiObjednavkyOsoby($database, $rc, $osobniCislo, $datumNastupu, $datumOd, $datumDo)
    {
        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT ObjednaneJidlo.Id, PolozkaJidelnicku.Id, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla, ObjednaneJidlo.DatumCasVydani' .
            ' FROM ObjednaneJidlo, PolozkaJidelnicku' .
            ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IdPolozkyJidelnicku' .
            '  AND ObjednaneJidlo.DatumCasZruseni IS NULL' .
            '  AND ObjednaneJidlo.RC = ' . $params ->addParameter($rc) .
            '  AND ObjednaneJidlo.OsobniCislo = ' . $params ->addParameter($osobniCislo) .
            '  AND ObjednaneJidlo.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
            '  AND PolozkaJidelnicku.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '  AND PolozkaJidelnicku.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' ORDER BY PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla, PolozkaJidelnicku.Id';

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $objednavka = new Objednavka();
            $objednavka ->idObjednavky = $row[0];
            $objednavka ->idPolozkyJidelnicku = $row[1];
            $objednavka ->datum = $cursor ->stringToTimestamp($row[2]);
            $objednavka ->typDennihoJidla = $row[3];
            $objednavka ->datumCasVydani = $cursor ->stringToTimestamp($row[4]);

            $result[] = $objednavka;
        }
        $cursor ->close();

        return $result;
    }
}


class ObjednavkyManager
{
    /** @var  Database */
    private $database;

    /** @var PracovniPomer */
    private $pp;

    /** @var  Objednavka[] */
    private $objednavky;

    /** @var CenovaKategorie */
    public $cenovaKategorie;

    /**
     * ObjednavkyManager constructor.
     * @param $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     */
    //public function nactiData($rc, $osobniCislo, $datumNastupu, $idPP, $datumOd, $datumDo)
    public function nactiData($pp, $datumOd, $datumDo)
    {
        $this->pp = $pp;

        if ($pp) {
            $this->cenovaKategorie = new CenovaKategorie($pp->rc, $pp->osobniCislo, $pp->datumNastupu);
            $this ->objednavky = Objednavka::nactiObjednavkyOsoby($this ->database, $pp->rc, $pp->osobniCislo, $pp->datumNastupu, $datumOd, $datumDo);
        }
    }


    /**
     * @param int  $id
     * @param bool $pouzeNevydane
     * @return int
     */
    public function pocetObjednavekOsobyProPolozku($id, $pouzeNevydane = false)
    {
        $result = 0;

        foreach ($this ->objednavky as $objednavka) {
            if ($objednavka ->idPolozkyJidelnicku == $id  &&  (!$pouzeNevydane  ||  !$objednavka ->datumCasVydani)) {
                $result++;
            }
        }

        return $result;
    }


    /**
     * @param int $datum
     * @return int
     */
    public function pocetObjednavekOsobyNaDen($datum)
    {
        $result = 0;

        foreach ($this ->objednavky as $objednavka) {
            if ($objednavka ->datum == $datum) {
                $result++;
            }
        }

        return $result;
    }

    /**
     * @param string $typ
     * @param int $datum
     * @return int
     */
    public function pocetObjednavekOsobyProTypJidla($typ, $datum)
    {
        $result = 0;

        foreach ($this ->objednavky as $objednavka) {
            if ($objednavka ->datum == $datum  &&  !strcasecmp($objednavka ->typDennihoJidla, $typ)) {
                $result++;
            }
        }

        return $result;
    }


    /**
     * Vraci true, pokud pocet dennich objednavek stravnika nepresahnul denni limit
     * @param $datum
     * @return bool
     * @throws DBException
     */
    function zkontrolujPocetObjednavekZaDen($datum)
    {
        if (!$this ->cenovaKategorie ->overNacteniVlastnostiKDatu($this ->database, $datum)) {
            return false;
        }

        if (!$this->cenovaKategorie ->f_Omezit_PocetObjDenCelkem) {
            return true;
        }

        $pocet = $this ->pocetObjednavekOsobyNaDen($datum);

        return $pocet < $this->cenovaKategorie ->f_MaxPocetObjNaDenCelkem;
    }


    /**
     * @param string $typDennihoJidla
     * @param int $datum
     * @return bool
     * @throws Exception
     */
    function zkontrolujPocetObjednavekZaTyp($typDennihoJidla, $datum)
    {
//        try {
//            if (!$this->cenovaKategorie ->overNacteniPravidelProTyp($this ->database, $typDennihoJidla)) {
//                return false;
//            }
//        } catch (DBException $e) {
//            throw $e;
//        } catch (Exception $e) {
//            // pripadnou vyjimku spolkne
//            MessageManager::addDebugMessage($e ->getMessage());
//            return false;
//        }

        $pravidla = $this->cenovaKategorie ->pravidlaProTypy[$typDennihoJidla];
        if (!$pravidla) return false;

        if (!$pravidla->omezit_PocetObjDen) return true;

        $pocet = $this ->pocetObjednavekOsobyProTypJidla($typDennihoJidla, $datum);

        return $pocet < $pravidla->maxPocetObjNaDen;
    }


    /**
     * @param int $den
     * @param string $typDennihoJidla
     * @param float $cenaJidla
     * @return bool
     * @throws Exception
     * @internal param CenovaKategorie $cenovaKategorie
     */
    function zkontrolujLimityOsoby($den, $typDennihoJidla, $cenaJidla)
    {
        if (!$this->zkontrolujPocetObjednavekZaDen($den)) {
            // omezen celkovy pocet objednavek za den
            return false;
        }

        if (!$this->zkontrolujPocetObjednavekZaTyp($typDennihoJidla, $den)) {
            // omezen pocet objednavek daneho typu jidla
            return false;
        }

        // overi, jestli je dostatecny kredit (kontroluje dotovanou cenu)
        $cenajidla = $this ->cenovaKategorie->vratDotovanouCenuProTypJidla($typDennihoJidla, $cenaJidla);
        if (!$this ->cenovaKategorie->zkontrolujKreditStravnika($this ->database, $cenajidla, $den)) {
            return false;
        }

        return true;
    }

    // TODO: Presunout sem zkontrolujKreditStravnika

    /**
     * Pro zadany typ denniho jidla zkontroluje, jestli je objednan dostatecny pocet "hlavnich" jidel
     * @param string $typDennihoJidla   Kontrolovany typ jidla (napriklad polevka)
     * @param int $den  Datum
     * @return bool
     */
    function zkontrolujOstatniObjednavky($typDennihoJidla, $den)
    {
        if (!$this ->cenovaKategorie ->overNacteniVlastnostiKDatu($this ->database, $den)) {
            return false;
        }

        $pravidla = $this->cenovaKategorie->pravidlaProTypy[$typDennihoJidla] ?? null;
        if (!$pravidla  ||  !$pravidla ->objednatJenSTypemJidla) {
            // OK, objednavani neni podmineno objednavkou jineho typu jidla
            return true;
        }

        // spocita pocet objednavek nastavenych (hlavnich) typu jidel (napr. obedy)
        $typy = explode(';', $pravidla ->objednatJenSTypemJidla);
        $pocetOstatni = 0;
        foreach ($typy as $typ) {
            $pocetOstatni += $this ->pocetObjednavekOsobyProTypJidla($typ, $den);
        }

        // spocita pocet objednavek objednavaneho (podrizeneho) typu jidla (napr. polevka)
        $pocet = $this ->pocetObjednavekOsobyProTypJidla($typDennihoJidla, $den);

        // lze objednat, jen kdyz je "hlavnich" objednavek vice nez "podrizenych"
        return $pocetOstatni > $pocet;
    }


    function zkontrolujJineTypyJidel($typDennihoJidla, $den)
    {
        if (!$this ->cenovaKategorie ->overNacteniVlastnostiKDatu($this ->database, $den)) {
            return false;
        }

        $pravidla = $this->cenovaKategorie->pravidlaProTypy[$typDennihoJidla] ?? null;
        if (!$pravidla  ||  !$pravidla ->neobjednatSTypemJidla) {
            // OK, objednavani neni omezeno objednavkou jineho typu jidla
            return true;
        }

        $typy = explode(';', $pravidla ->neobjednatSTypemJidla);
        foreach ($typy as $typ) {
            if ($this ->pocetObjednavekOsobyProTypJidla($typ, $den)) {
                // je objednane jine jidlo, ktere nesmi byt objednane spolecne s timto jidlem
                return false;
            }
        }

        return true;
    }


    /**
     * Vraci true pokud si stravnik muze objednat tuto polozku jidelnicku
     * @param PolozkaJidelnicku $polozkaJidelnicku      V pripade, ze je true, vyhodi vyjimku pokud nelze polozka objednat
     * @param bool $throwException
     * @return bool
     * @throws Exception
     */
    function lzeObjednatPolozku($polozkaJidelnicku, $throwException = false)
    {
        if ($polozkaJidelnicku->stavObjednavani($this ->database) != 1) {
            // nelze objednavat (napr. kvuli casovemu omezeni)
            if ($throwException) {
                throw new Exception(TextManager::_('Jiz_nelze_objednat'));
            }
            return false;
        }

        //12.8.2015 RON-PR doplneno pro kontrolu max. poctu obj. osoby
        if ($polozkaJidelnicku->f_omezitPocetNaOsobu && ($polozkaJidelnicku->f_pocetobjednavekosoby >= $polozkaJidelnicku->f_maxPocetNaOsobu)) {
            if ($throwException) {
                throw new Exception(TextManager::_('Jiz_nelze_objednat'));
            }
            return false;
        }

        //11.8.2015 RON-PR kontrola celkoveho poctu objednavek polozky
        if ($polozkaJidelnicku->f_maxpocetporciA && ($polozkaJidelnicku->f_pocetobjednavekcelkem >= $polozkaJidelnicku->f_maxpocetporciA)) {
            if ($throwException) {
                throw new Exception(TextManager::_('Jiz_nelze_objednat'));
            }
            return false;
        }

        if (!$this ->zkontrolujOstatniObjednavky($polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_den)) {
            if ($throwException) {
                throw new Exception(TextManager::_('Neni_objednano_hlavni_jidlo'));
            }
            return false;
        }

        // zkontroluje, jestli neni objednane jine jidlo, ktere zakazuje objednavku tohoto jidla (nastaveni NeobjednatSTypemJidla)
        if (!$this ->zkontrolujJineTypyJidel($polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_den)) {
            if ($throwException) {
                throw new Exception(TextManager::_('Je_objednane_jine_jidlo'));
            }
            return false;
        }

        if (!$this->zkontrolujLimityOsoby($polozkaJidelnicku ->f_den, $polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_cenajidlaA)) {
            // dosazen maximalni pocet objednavek za den nebo za typ, presazen kredit
            if ($throwException) {
                throw new Exception(TextManager::_('Jiz_nelze_objednat'));
            }
            return false;
        }

        return true;
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @return bool
     * @throws Exception
     */
    function lzeVyzvednoutPolozkuZBurzy($polozkaJidelnicku)
    {
        if ($polozkaJidelnicku->stavObjednavani($this ->database) != 2) {
            // nelze kseftovat v burze
            return false;
        }

        if (JidelnaUtils::pocetVBurze_Osoba($this ->database, $polozkaJidelnicku->f_id, $this ->pp)) {
            // svou polozku muze vyzvednout kdykoliv
            return true;
        }

        if (!JidelnaUtils::pocetVBurzeCelkem($this ->database, $polozkaJidelnicku->f_id)) {
            // nic neni v burze
            return false;
        }

        if (!$this ->zkontrolujOstatniObjednavky($polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_den)) {
            return false;
        }

        // zkontroluje, jestli neni objednane jine jidlo, ktere zakazuje objednavku tohoto jidla (nastaveni NeobjednatSTypemJidla)
        if (!$this ->zkontrolujJineTypyJidel($polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_den)) {
            return false;
        }


        return $this->zkontrolujLimityOsoby($polozkaJidelnicku ->f_den, $polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_cenajidlaA);
    }


    /**
     * Pro zadany typ denniho jidla zkontroluje, jestli neni objednan "podrizeny" typ jidla (napr. pro obed zkontroluje, jestli neni objednana polevka)
     * @param string $typDennihoJidla   Kontrolovany typ jidla (napriklad obed)
     * @param int $den  Datum
     * @return bool
     */
    function zkontrolujHlavniTypyJidel($typDennihoJidla, $den)
    {
        $pravidla = $this ->cenovaKategorie ->pravidlaProTypy[$typDennihoJidla];
        if (!$pravidla  ||  !$pravidla ->podminujeObjednavkuTypuJidla) {
            // OK, objednavani nepodminuje objednavku jineho typu jidla
            return true;
        }

        $typy = explode(';', $pravidla ->podminujeObjednavkuTypuJidla);

        foreach ($typy as $podminenyTyp) {
            if (!$this ->zkontrolujOstatniObjednavky($podminenyTyp, $den)) {
                return false;
            }
        }

        return true;
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @param bool $throwException
     * @return bool
     * @throws Exception
     */
    function lzeZrusitObjednavku($polozkaJidelnicku, $throwException = false)
    {
        if ($polozkaJidelnicku->stavObjednavani($this ->database, true) != 1) {
            // nelze objednavat (napr. kvuli casovemu omezeni)
            if ($throwException) {
                throw new Exception(TextManager::_('Jiz_nelze_zrusit_objednavku'));
            }
            return false;
        }

        if (!$this ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku->f_id, true)) { // pouzeNevydane = true
            if ($throwException) {
                throw new Exception('Nenalezena platna objednavka polozky jidelnicku!');
            }
            return false;
        }

        if (!$this ->zkontrolujHlavniTypyJidel($polozkaJidelnicku ->f_typdennihojidla, $polozkaJidelnicku ->f_den)) {
            if ($throwException) {
                throw new Exception(TextManager::_('Nejdrive_zruste_podrizenou_objednavku'));
            }
            return false;
        }

        return true;
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @return bool
     */
    function lzePredatObjednavkuDoBurzy($polozkaJidelnicku)
    {
        if ($polozkaJidelnicku->stavObjednavani($this ->database, true) != 2) {
            // nelze kseftovat v burze
            return false;
        }

        if ($polozkaJidelnicku->f_pocetobjednavekosoby <= $polozkaJidelnicku->pocetVBurzeMych) {
            // uz jsou vsechny v burze
            return false;
        }

        $pocetNevydanych = $this ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku->f_id, true);
        return !!$pocetNevydanych;
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @throws Exception
     */
    function nactiStavObjednavani($polozkaJidelnicku)
    {
        $polozkaJidelnicku ->lzeObjednat = $this ->lzeObjednatPolozku($polozkaJidelnicku);
        $polozkaJidelnicku ->lzeVyzvednoutZBurzy = !$polozkaJidelnicku ->lzeObjednat  &&  $this ->lzeVyzvednoutPolozkuZBurzy($polozkaJidelnicku);

        $polozkaJidelnicku ->lzeZrusitObjednavku = $this ->lzeZrusitObjednavku($polozkaJidelnicku);
        $polozkaJidelnicku ->lzePredatObjednavkuDoBurzy = !$polozkaJidelnicku ->lzeZrusitObjednavku  &&  $this ->lzePredatObjednavkuDoBurzy($polozkaJidelnicku);
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @throws Exception
     * @throws DBException
     */
    function objednejJidlo($polozkaJidelnicku)
    {
        if (!$this ->cenovaKategorie ->overNacteniVlastnostiKDatu($this ->database, $polozkaJidelnicku ->f_den)) {
            throw new Exception('Cenova kategorie nema pro tento den nastavene vlastnosti!');
        }

        $polozkaJidelnicku ->f_pocetobjednavekosoby = $this ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if ($polozkaJidelnicku->f_pocetobjednavekosoby < 0) {
            throw new Exception('Chyba pri zjistovani poctu objednanych jidel.');
        }

        //if (!$polozkaJidelnicku->lzeObjednat($this->database, $cenovakategorie)) {
        try {
            $this->lzeObjednatPolozku($polozkaJidelnicku, true);
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage(), null, 'alertMessage');
            return;
        }

        if (!Rozdeleni::jeVRozdeleni($this ->database, null, $this ->pp ->idPP, 8, $polozkaJidelnicku ->f_jidelna, 0, $polozkaJidelnicku ->f_den, $polozkaJidelnicku ->f_den)) {
            throw new Exception( 'Chyba - jidlo nelze objednat, protoze nejste zarazen do teto jidelny!' );
        }

        $rec = new RonPortalDBRecord($this->database, 'ObjednaneJidlo', $this);

        $rec->addField('IDPolozkyJidelnicku', 'int', false, 0, $polozkaJidelnicku ->f_id);
        $rec->addLogText(
            '   ' . $polozkaJidelnicku->f_typdennihojidla . ' ' . $polozkaJidelnicku->f_cislonabidky .
            ' (' . $polozkaJidelnicku->f_popisnabidky . ') ' . date('j.n.Y', $polozkaJidelnicku->f_den));
        $rec->addField('RC', '', false, 15, $this ->pp ->rc);
        $rec->addLogText('   ' . $this ->pp ->jmenoPrijmeni);
        $rec->addField('OsobniCislo', '', false, 15, $this ->pp ->osobniCislo);
        $rec->addField('DatumNastupu', 'timestamp', false, 0, $this ->pp ->datumNastupu);
        $rec->addField('DatumCasObjednani', 'timestamp', false, 0, time());
        $rec->addField('Uzivatel', '', false, 25, RonPortalUtils::getLogUserName());

        $newPoradi = $polozkaJidelnicku->novePoradiVydani($this->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu);
        $rec->addField('PoradiVydani', 'int', false, 0, $newPoradi);

        $newId = SQLUtils::newSerialNumber($this->database, 'ObjednaneJidlo', 'ID');
        $rec->addField('ID', 'int', false, 0, $newId);

        $rec->insertRecord();

        $this->provedVyuctovani();
    }


    /**
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @throws Exception
     * @throws DBException
     */
    function zrusObjednavku($polozkaJidelnicku)
    {
        if (!$this ->cenovaKategorie ->overNacteniVlastnostiKDatu($this ->database, $polozkaJidelnicku ->f_den)) {
            throw new Exception('Cenova kategorie nema pro tento den nastavene vlastnosti!');
        }

        $polozkaJidelnicku ->f_pocetobjednavekosoby = $this ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        try {
            $this->lzeZrusitObjednavku($polozkaJidelnicku, true);
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage(), null, 'alertMessage');
            return;
        }

        // Nacte seznam objednavek k zadane polozce
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT ObjednaneJidlo.Id, ObjednaneJidlo.PoradiVydani, ' .
            ' PolozkaJidelnicku.Jidelna, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla ' .
            ' FROM ObjednaneJidlo, PolozkaJidelnicku ' .
            ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
            '   AND IDPolozkyJidelnicku = ' . $params ->addParameter($polozkaJidelnicku ->f_id) .
            '   AND RC = ' . $params ->addParameter($this ->pp ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($this ->pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this->database->dateToParam($this ->pp ->datumNastupu)) .
            '   AND DatumCasZruseni IS NULL ' .
            ' ORDER BY PoradiVydani DESC';

        $cursor = $this->database->getSqlResult($sql, $params);
        $radek_obj = $cursor->fetch();
        $cursor ->close();

        if (!$radek_obj) {
            throw new Exception(TextManager::_('Objednavka_nebyla_nalezena'));
        }

        $params = $this ->database ->createParameters();
        $sql =
            'UPDATE ObjednaneJidlo SET DatumCasZruseni = ' . $params ->addParameter($this->database->timestampToParam(time())) .
            ' WHERE Id = ' . $params ->addParameter((int)$radek_obj[0]);

        $this->database->sql($sql, $params);

        // zaloguje zruseni objednavky
        Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'UPDATE', $this,
            'Zruseni objednavky: RC=' . $this ->pp ->rc . ', PolozkaJidelnicku=' . $polozkaJidelnicku ->f_id . "\n   " .
            $this ->pp->jmenoPrijmeni . "\n   " .
            date('j.n.Y', $polozkaJidelnicku->f_den) . ', ' . $polozkaJidelnicku->f_typdennihojidla . ' ' . $polozkaJidelnicku->f_cislonabidky . ' - ' . $polozkaJidelnicku->f_popisnabidky .
            "\n\n" .
            $sql);

        $params = $this->database->createParameters();

        try {
            // Musi opravit poradi vydeje ostatnich polozek pro stejny typ jidla,
            // jinak by to delalo problem u vydeje pri pridelovani priloh
            // POZOR! Nefunguje s driverem ODBC "SQL Server" a taky se starym SQL serverem (2008)
            $sql =
                'UPDATE ObjednaneJidlo ' .
                ' SET PoradiVydani = (PoradiVydani - 1) ' .
                " WHERE (RC = " . $params->addParameter($this->pp ->rc) . ") " .
                "   AND (OsobniCislo = " . $params->addParameter($this->pp ->osobniCislo) . ") " .
                '   AND (DatumNastupu = ' . $params->addParameter($this->database->timestampToParam($this->pp ->datumNastupu)) . ') ' .
                '   AND (DatumCasZruseni IS NULL) ' .
                '   AND (DatumCasVydani IS NULL) ' .
                '   AND (PoradiVydani >= ' . $params->addParameter($radek_obj[1]) . ') ' .
                '   AND (EXISTS(SELECT ID ' .
                '        FROM PolozkaJidelnicku ' .
                '        WHERE (PolozkaJidelnicku.ID = ObjednaneJidlo.IDPolozkyJidelnicku) ' .
                '          AND (PolozkaJidelnicku.KodTypuRozdeleni = 8) ' . // typ rozdeleni jidelna
                "          AND (PolozkaJidelnicku.Jidelna = " . $params->addParameter($radek_obj[2]) . ") " .
                '          AND (PolozkaJidelnicku.Den = ' . $params->addParameter($this->database->timestampToParam($this->database->stringToTimestamp($radek_obj[3]))) . ') ' .
                "          AND (PolozkaJidelnicku.TypDennihoJidla = " . $params->addParameter($radek_obj[4]) . ") " .
                '       ))';

            $this->database->sql($sql, $params);
        } catch (Exception $e) {
            //MessageManager::addDbErrorMessage($this->database, $sql, $e ->getMessage());
            MessageManager::getInstance() ->addMessage(
                'Chyba pri zmene poradi vydeje: ' . $e->getMessage() . (Prava::vratPrava('admin') ->jsouNejakePrava() ? "\n" . $sql : ''),
                Message::LOG);
            Log::addRecordToDBLog($this->database, 'ObjednaneJidlo', 'ERROR', $this,
                'Chyba pri oprave poradi vydeje objednanych jidel!' . "\n{$e ->getMessage()}\n\n$sql");

        }

        $this->provedVyuctovani();
    }


    /**
     * @throws Exception
     */
    function provedVyuctovani()
    {
        AgentManager::vyuctovaniJidelny($this ->pp ->idPP);
    }
}
