<?php
/**
 * stanislav.elbl, 24.05.2017
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Controls\FormReportsListView;
use RonPortal\Dochazka\Smena;
use RonPortal\Dochazka\OperaceDochazky;
use RonPortal\Kalendar;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\User;


/**
 * Class SloupecKapacitDleSmen
 * @package Ron\Kapacity
 */
class SloupecKapacitDleSmen
{
    /** @var Smena */
    public $smena;

    /** @var OperaceDochazky */
    public $absence;
    /** @var int    Cas absence od - pouze pro zobrazeni dat za aktualni den */
    public $casOd;
    /** @var int    Cas absence do - pouze pro zobrazeni dat za aktualni den */
    public $casDo;

    /** @var bool   Priznak, ze je smena posledni ve skupine a ma za ni byt zobrazen oddelovac */
    public $posledniVeSkupine = false;

    /** @var int    Index v nactenych datech - pouzije se pro data aktualniho dne po serazeni sloupcu podle casu. */
    public $index;
}


/**
 * Class KapacityDleSmenView
 * @package Ron\Kapacity
 */
class KapacityDleSmenView implements ViewObject
{
    /** @var \Ron\DB\Database */
    var $database;

    var $nastaveni;
    /** @var bool|string */
    var $zobrazitOsobySAbsenci = false;

    /** @var  KapacityFilterForm */
    var $filterForm;

    /** @var  PlanovaniKapacit */
    var $data;

    /** @var SloupecKapacitDleSmen[] */
    var $sloupce;
    /** @var string[][] */
    var $dataProDny;

    /** @var SloupecKapacitDleSmen[] */
    var $sloupceProAktualniDen;
    /** @var string[] */
    var $dataProAktualniDen = [];

    /** @var OperaceDochazky[]  Seznam operaci (asociatini pole indexovane kodem) */
    var $operace = [];

    var $reportList;

    /**
     * @param \Ron\DB\Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * @param PlanovaniKapacit_Osoba $osoba
     * @param PlanovaniKapacit_PlanovanaSmena $planovanaSmena
     * @return bool
     */
    public function zobrazitPlanovanouSmenu($osoba, $planovanaSmena)
    {
        $uroven = isset($this ->nastaveni['jenschvalenesmeny'])
            ? (int)$this ->nastaveni['jenschvalenesmeny']
            : 0;

        if (!$uroven) {
            // schvaleni se vubec nekontroluje
            return true;
        }

        if ($uroven >= 11) {
            // podle schvaleni dodatecnych uprav
            if ($osoba->schvaleni3 >= $planovanaSmena->datum) {
                return true;
            }
            if ($uroven <= 12 && $osoba->schvaleni2 >= $planovanaSmena->datum) {
                return true;
            }
            if ($uroven === 11 && $osoba->schvaleni1 >= $planovanaSmena->datum) {
                return true;
            }

            return false;
        }

        // podle schvaleni puvodniho planu
        if ($osoba->schvaleniPuvodnihoPlanu3 >= $planovanaSmena->datum) {
            return true;
        }
        if ($uroven <= 2 && $osoba->schvaleniPuvodnihoPlanu2 >= $planovanaSmena->datum) {
            return true;
        }
        if ($uroven <= 1 && $osoba->schvaleniPuvodnihoPlanu1 >= $planovanaSmena->datum) {
            return true;
        }

        return false;
    }


    /**
     * @throws DBException
     */
    public function nactiSeznamNastavenychSmen()
    {
        $today = DateUtils::today();

        // zjisti vsechny pouzite smeny
        $kody = [];
        $kodyDnes = []; // kody smen pouzite pro aktualni den
        foreach ($this ->data ->seznamOsob as $osoba) {
            foreach ($osoba ->seznamSmen as $planovanaSmena) {
                if ($this ->zobrazitPlanovanouSmenu($osoba, $planovanaSmena)) {
                    $kody[$planovanaSmena ->kod] = $planovanaSmena ->kod;

                    if ($planovanaSmena ->datum === $today) {
                        $kodyDnes[$planovanaSmena ->kod] = $planovanaSmena ->kod;
                    }
                }
            }
        }

        // nacte seznam pouzitych smen
        $params = $this ->database ->createParameters();
        $filter = '';
        foreach ($kody as $kodSmeny) {
            $filter .= ($filter ? ', ' : '') . $params ->addParameter($kodSmeny);
        }
        if ($filter) {
            $filter = 'Smena.Kod IN (' . $filter . ')';
        } else {
            $filter = '(0 = 1)';
        }

        $smeny = Smena::nactiSmeny($this ->database, $filter, $params, 'PocatekSmeny, KonecSmeny');

        foreach ($smeny as $smena) {
            $sloupec = new SloupecKapacitDleSmen();
            $sloupec ->smena = $smena;

            $this ->sloupce[] = $sloupec;

            if (isset($kodyDnes[$smena ->kod])) {
                $this ->sloupceProAktualniDen[] = $sloupec;
            }
        }

        $strOrder = isset($this ->nastaveni['smena'][0]['order']) ? $this ->nastaveni['smena'][0]['order'] : null;
        if ($strOrder && $this ->sloupce) {
            // U tagu <smena> je zadan atribut order - urcuje poradi zobrazovanych smen (kody oddelene strednikem)
            // Pokud je zadan prazdny kod, tak tam bude oddeleni carou.
            // Presklada poradi sloupcu aby odpovidalo zadanemu poradi

            $order = explode(';', $strOrder);
            /** @var SloupecKapacitDleSmen[] $sloupce */
            $sloupce = [];
            foreach ($order as $index => $kod) {
                if ($kod === '') {
                    // oddelovac - oznaci posledni sloupec
                    if ($sloupce) {
                        $sloupce[count($sloupce) - 1] ->posledniVeSkupine = true;
                    }
                } else {
                    $found = false;
                    for ($i = 0; !$found && $i < count($this->sloupce); $i++) {
                        if ($this->sloupce[$i]  &&  strtolower($this->sloupce[$i]->smena->kod) === strtolower($kod)) {
                            $found = true;
                            $sloupce[] = $this->sloupce[$i];
                            $this->sloupce[$i] = null;
                        }
                    }
                }
            }

            // nakonec doplni smeny, ktere jsou v datech, ale nejsou v nastaveni "order"
            foreach ($this ->sloupce as $sloupec) {
                if ($sloupec) {
                    $sloupce[] = $sloupec;
                }
            }

            $this ->sloupce = $sloupce;
        }
    }


    /**
     * @throws DBException
     */
    public function nactiSeznamNastavenychAbsenci()
    {
        if (!isset($this ->nastaveni['absence']) || !$this ->nastaveni['absence']) {
            return;
        }

        $kody = [];
        if ($this ->nastaveni['absence'][0]['order']) {
            // jsou nastavene jen vybrane absence
            $kody = explode(';', $this ->nastaveni['absence'][0]['order']);
        } else {
            // zjisti vsechny pouzite operace
            foreach ($this ->data ->seznamOsob as $osoba) {
                foreach ($osoba ->seznamAbsenci as $planovanaAbsence) {
                    $kody[$planovanaAbsence ->kod] = $planovanaAbsence ->kod;
                }
            }
        }

        if (!$kody) {
            return;
        }

        // nacte seznam pouzitych absenci
        $params = $this ->database ->createParameters();
        $filter = '';
        foreach ($kody as $kodOperace) {
            $filter .= ($filter ? ', ' : '') . $params ->addParameter($kodOperace);
        }

        $seznamOperaci = OperaceDochazky::nactiOperace($this ->database,
            $filter ? 'OperaceDochazky.Kod IN (' . $filter . ')' : null, $filter ? $params : null);

        foreach ($seznamOperaci as $operace) {
            $sloupec = new SloupecKapacitDleSmen();
            $sloupec ->absence = $operace;

            $this ->sloupce[] = $sloupec;

            $this ->operace[$operace ->kod] = $operace; // slouzi pro zpracovani dat aktualniho dne
        }
    }


    /**
     * Sestavi seznam sloupcu podle skutecne pouzitych smen
     * @throws DBException
     */
    public function sestavSeznamSloupcu()
    {
        $this ->sloupce = [];
        $this ->sloupceProAktualniDen = [];

        $this ->nactiSeznamNastavenychSmen();

        if ($this ->sloupce) {
            $this ->sloupce[count($this ->sloupce) - 1] ->posledniVeSkupine = true;
        }

        $this ->nactiSeznamNastavenychAbsenci();
    }


    /**
     * @param SloupecKapacitDleSmen[] $sloupce
     * @param PlanovaniKapacit_PlanovanaSmena $planovanaSmena
     * @return int
     */
    public function najdiSmenu($sloupce, $planovanaSmena)
    {
        foreach($sloupce as $i => $sloupec) {
            if ($sloupec ->smena  &&  !strcasecmp($sloupec ->smena ->kod, $planovanaSmena ->kod ?? '')) {
                return $i;
            }
        }

        return null; // nemelo by nastat
    }


    /**
     * @param SloupecKapacitDleSmen[] $sloupce
     * @param string $kod
     * @param int $casOd
     * @param int $casDo
     * @param bool $pridat  Urcuje, jestli se ma pridat novy sloupec, pokud jeste neexistuje
     * @return int
     */
    public function najdiSloupecAbsence(&$sloupce, $kod, $casOd = null, $casDo = null, $pridat = false)
    {
        foreach($sloupce as $i => $sloupec) {
            if ($sloupec ->absence  &&  !strcasecmp($sloupec ->absence ->kod, $kod ?? '')) {
                if (isset($casOd)  &&  isset($casDo)) {
                    if ($sloupec ->casOd === $casOd  &&  $sloupec ->casDo === $casDo) {
                        return $i;
                    }
                } else {
                    return $i;
                }
            }
        }

        if ($pridat) {
            $sloupec = new SloupecKapacitDleSmen();
            $sloupec ->absence = $this ->operace[$kod];
            $sloupec ->casOd = $casOd;
            $sloupec ->casDo = $casDo;

            $sloupce[] = $sloupec;
            return count($sloupce) - 1;
        }

        return null;
    }


    /**
     * @param string $maska
     * @param PlanovaniKapacit_Osoba $osoba
     * @return string
     */
    public function vratTextOsoby($maska, $osoba)
    {
        // doplni kod, popis, zkratku
        $maska = str_replace(
            ['%JMENO%', '%PRIJMENI%', '%JMENO_PRIJMENI%', '%OSOBNI_CISLO%'],
            [
                StringUtils::htmlTextConvert($osoba ->jmeno),
                StringUtils::htmlTextConvert($osoba ->prijmeni),
                StringUtils::htmlTextConvert($osoba ->jmenoPrijmeni),
                StringUtils::htmlTextConvert($osoba ->osobniCislo)
            ],
            $maska);

        // uzivatelske
        for ($i = 1;  $i <= 20; $i++) {
            $maska = str_replace("%UZIVATELSKA$i%", StringUtils::htmlTextConvert($osoba->uzivatelske[$i - 1]), $maska);
        }

        // uzivatelske PP
        for ($i = 1;  $i <= 20; $i++) {
            $maska = str_replace("%UZIVATELSKA_PP$i%", StringUtils::htmlTextConvert($osoba->uzivatelskePP[$i - 1]), $maska);
        }

        return $maska;
    }


    /**
     * @param PlanovaniKapacit_Osoba $osoba
     */
    public function zpracujSmenyOsoby($osoba)
    {
        $strText = isset($this ->nastaveni['bunka'][0]['text']) ? $this ->nastaveni['bunka'][0]['text'] : null;
        $strTitle = isset($this ->nastaveni['bunka'][0]['title']) ? $this ->nastaveni['bunka'][0]['title'] : null;
        $separator = isset($this ->nastaveni['bunka'][0]['separator']) ? $this ->nastaveni['bunka'][0]['separator'] : '';

        foreach ($osoba ->seznamSmen as $planovanaSmena) {
            if ($this ->zobrazitPlanovanouSmenu($osoba, $planovanaSmena)) {
                // zjisti, jestli je v dany den absence
                $jeNezobrazovanaAbsence = false;

                $classAbsence = '';
                foreach ($osoba->seznamAbsenci as $planovanaAbsence) {
                    if ($planovanaAbsence->datumOd <= $planovanaSmena->datum && $planovanaAbsence->datumDo >= $planovanaSmena->datum  &&  $planovanaAbsence ->kod != '') {
                        $classAbsence .= ' absence_' . $planovanaAbsence->kod;
                        if ( is_array($this ->zobrazitOsobySAbsenci) ) {
                            if (!in_array($planovanaAbsence ->kod, $this ->zobrazitOsobySAbsenci)) {
                                $jeNezobrazovanaAbsence = true;
                            }
                        }
                    }
                }
                if ($classAbsence) {
                    $classAbsence = ' absence' . $classAbsence;
                }

                if ($this->zobrazitOsobySAbsenci === true || !$jeNezobrazovanaAbsence) {
                    $indexDne = date('j', $planovanaSmena->datum) - date('j', $this->data->datumOd);
                    $indexSmeny = $this->najdiSmenu($this ->sloupce, $planovanaSmena);

                    if (isset($indexSmeny)) {
                        if (!isset($this->dataProDny[$indexDne][$indexSmeny])) {
                            $this->dataProDny[$indexDne][$indexSmeny] = '';
                        }

                        $casyStr = '';
                        if ($planovanaSmena->variabilni &&
                            ($planovanaSmena->pocatek !== $this->sloupce[$indexSmeny]->smena->pocatekSmeny || $planovanaSmena->konec !== $this->sloupce[$indexSmeny]->smena->konecSmeny)
                        ) {
                            // variabilni smena, zamestnanec ma zadan jiny cas nez je defaultni
                            $casyStr = ' (' . StringUtils::format('minuty', $planovanaSmena->pocatek / 60) . ' - ' . StringUtils::format('minuty', $planovanaSmena->konec / 60) . ')';
                        }

                        $className =
                            'planovani_osoba' .
                            $classAbsence .
                            ((User::isAuthenticated()  &&  ($osoba->rc === User::getRC())) ? ' vybrana_osoba' : '');

                        if ($strText) {
                            $text = $this->vratTextOsoby($strText, $osoba);
                            $text = str_replace('%SMENA_INFO%', $casyStr, $text);
                        } else {
                            $text = StringUtils::htmlTextConvert($osoba->jmenoPrijmeni) . $casyStr;
                        }

                        if ($separator && $this->dataProDny[$indexDne][$indexSmeny]) {
                            $this->dataProDny[$indexDne][$indexSmeny] .= $separator;
                        }

                        $title = $strTitle ? 'title="' . $this->vratTextOsoby($strTitle, $osoba) . '"' : '';

                        $this->dataProDny[$indexDne][$indexSmeny] .=
                            "<div data-ron-id='{$osoba ->idPP}' class='$className' $title>" .
                            $text .
                            '</div>';

                        if ($planovanaSmena ->datum === DateUtils::today()) {
                            $indexDnes = $this->najdiSmenu($this ->sloupceProAktualniDen, $planovanaSmena);
                            $this->dataProAktualniDen[$indexDnes] =
                                (isset($this->dataProAktualniDen[$indexDnes]) ? $this->dataProAktualniDen[$indexDnes] . $separator : '') .
                                "<div data-ron-id='{$osoba ->idPP}' class='$className' $title>" .
                                $text .
                                '</div>';
                        }
                    }
                }
            }
        }
    }


    /**
     * @param PlanovaniKapacit_Osoba $osoba
     */
    public function zpracujAbsenceOsoby($osoba)
    {
        $today = DateUtils::today();

        $strText = isset($this ->nastaveni['bunka'][0]['text']) ? $this ->nastaveni['bunka'][0]['text'] : null;
        $strTitle = isset($this ->nastaveni['bunka'][0]['title']) ? $this ->nastaveni['bunka'][0]['title'] : null;
        $separator = isset($this ->nastaveni['bunka'][0]['separator']) ? $this ->nastaveni['bunka'][0]['separator'] : '';

        foreach ($osoba ->seznamAbsenci as $absence) {
            $indexSloupec = $this ->najdiSloupecAbsence($this ->sloupce, $absence ->kod);

            if (isset($indexSloupec)) {
                $datumOd = $absence ->datumOd;
                if ($datumOd < $this ->data ->datumOd) {
                    $datumOd = $this ->data ->datumOd;
                }

                $datumDo = $absence ->datumDo;
                if ($datumDo > $this ->data ->datumDo) {
                    $datumDo = $this ->data ->datumDo;
                }

                $casyStr = '';
                if ($absence->delka === 3 || $absence ->delka === 4) {
                    // variabilni smena, zamestnanec ma zadan jiny cas nez je defaultni
                    $casyStr = ' (' . DateUtils::minutesToString($absence->casOd * 60) . ' - ' . DateUtils::minutesToString($absence->casDo * 60) . ')';
                }

                if ($strText) {
                    $text = $this->vratTextOsoby($strText, $osoba);
                    $text = str_replace('%SMENA_INFO%', $casyStr, $text);
                } else {
                    $text = StringUtils::htmlTextConvert($osoba->jmenoPrijmeni);
                }


                $indexDne = date('j', $datumOd) - date('j', $this->data->datumOd);

                for ($datum = $datumOd;  $datum <= $datumDo;  $datum = DateUtils::incDayStart($datum)) {
                    if (!isset($this->dataProDny[$indexDne][$indexSloupec])) {
                        $this->dataProDny[$indexDne][$indexSloupec] = '';
                    }

                    $className =
                        'planovani_osoba absence absence_' . $absence ->kod .
                        ((User::isAuthenticated()  &&  ($osoba->rc === User::getRC())) ? ' vybrana_osoba' : '');

                    if ($separator && $this->dataProDny[$indexDne][$indexSloupec]) {
                        $this->dataProDny[$indexDne][$indexSloupec] .= $separator;
                    }

                    $title = $strTitle ? 'title="' . $this->vratTextOsoby($strTitle, $osoba) . '"' : '';

                    $this->dataProDny[$indexDne][$indexSloupec] .=
                        "<div data-ron-id='{$osoba ->idPP}' class='$className' $title>" .
                        $text .
                        '</div>';

                    if ($datum === $today) {
                        $indexDnes = $this ->najdiSloupecAbsence($this ->sloupceProAktualniDen, $absence ->kod, $absence ->casOd, $absence ->casDo, true);
                        $this ->dataProAktualniDen[$indexDnes] =
                            ($this ->dataProAktualniDen[$indexDnes] ? $this ->dataProAktualniDen[$indexDnes] . $separator : '') .
                            "<div data-ron-id='{$osoba ->idPP}' class='$className' $title>" .
                            $text .
                            '</div>';
                    }

                    $indexDne++;
                }
            }
        }
    }


    /**
     * @throws DBException
     */
    public function zpracujData()
    {
        $this ->sestavSeznamSloupcu();

        $this ->dataProDny = [];
        foreach ($this ->data ->dny as $den) {
            $this ->dataProDny[] = [];
        }

        foreach ($this ->data ->seznamOsob as $osoba) {
            $this ->zpracujSmenyOsoby($osoba);
            $this ->zpracujAbsenceOsoby($osoba);
        }
    }


    private static function porovnejSloupce($sloupec1, $sloupec2)
    {
        /**
         * @var SloupecKapacitDleSmen $sloupec1
         * @var SloupecKapacitDleSmen $sloupec2
         */
        $cas1 = $sloupec1 ->smena
            ? $sloupec1 ->smena ->pocatekSmeny / 60
            : $sloupec1 ->casOd * 60;

        $cas2 = $sloupec2 ->smena
            ? $sloupec2 ->smena ->pocatekSmeny / 60
            : $sloupec2 ->casOd * 60;

        if (isset($cas1, $cas2)) {
            if ($cas1 < $cas2) {
                return -1;
            }
            if ($cas2 < $cas1) {
                return 1;
            }
        } else if (isset($cas1)) {
            return -1;
        } else if (isset($cas2)) {
            return 1;
        }

        $cas1 = $sloupec1 ->smena
            ? $sloupec1 ->smena ->konecSmeny / 60
            : $sloupec1 ->casDo * 60;

        $cas2 = $sloupec2 ->smena
            ? $sloupec2 ->smena ->konecSmeny / 60
            : $sloupec2 ->casDo * 60;

        if (isset($cas1, $cas2)) {
            if ($cas1 < $cas2) {
                return -1;
            }
            if ($cas2 < $cas1) {
                return 1;
            }
        } else if (isset($cas1)) {
            return -1;
        } else if (isset($cas2)) {
            return 1;
        }

        return 0;
    }

    public function zpracujDataProAktualniDen()
    {
        if (!isset($this ->nastaveni['aktualniden'])  ||  !$this ->nastaveni['aktualniden']) {
            return;
        }

        // zapamatuje si puvodni poradi sloupce
        foreach ($this ->sloupceProAktualniDen as $i => $sloupec) {
            $sloupec ->index = $i;
        }

        // seradi sloupce
        //usort($this ->sloupceProAktualniDen, function($sloupec1, $sloupec2) { return KapacityDleSmenView::porovnejSloupce);
        usort($this ->sloupceProAktualniDen, [KapacityDleSmenView::class, 'porovnejSloupce']);
    }


    /**
     * @throws \Exception
     */
    public function load()
    {
        $this ->filterForm = new KapacityFilterForm($this ->database, 'kapacitydlesmen');
        $this ->filterForm ->nastavProPlanovaniObdobi();
        $this ->filterForm ->loadData();

        // nacte nastaveni
        $settingsXml = DBUtils::getSqlValue($this ->database, "SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = 'RonPortal_PK_Dle_Smen'");
        if ($settingsXml) {
            $this ->nastaveni = XmlLoader::loadXmlToArray($settingsXml, [XmlLoader::PARAM_CONVERT_FROM_UTF8 => true, XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true]);
            // priznak, jestli ma zobrazit osoby s absenci - 0=ne, 1=ano, nebo muze obsahovat seznam kodu operaci
            $this ->zobrazitOsobySAbsenci = $this ->nastaveni['zobrazitosobysabsenci'];

            if ($this ->zobrazitOsobySAbsenci == '0') {
                $this ->zobrazitOsobySAbsenci = false;
            }
            else if ($this ->zobrazitOsobySAbsenci == '1') {
                $this ->zobrazitOsobySAbsenci = true;
            }
            else {
                $this ->zobrazitOsobySAbsenci = explode(';', $this ->zobrazitOsobySAbsenci);
            }
        }


        // nacte zakladni data
        $this ->data = new PlanovaniKapacit($this ->database);
        $this ->data ->nactiData(
            $this->filterForm->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(),
            $this->filterForm->getFilterValue('rozdeleni'), $this->filterForm->getFilterValue('rozdeleni', 'vcetnePodrizenych'),
            $this->filterForm->getFilterValue('rozdeleni2'), $this->filterForm->getFilterValue('rozdeleni2', 'vcetnePodrizenych'),
//            $this ->filterForm ->kodRozdeleni, $this ->filterForm ->vcetnePodrizenychRozdeleni,  // RonPortalSession::rozdeleni(), RonPortalSession::rozdeleniVcetnePodrizenych(),
//            $this ->filterForm ->kodRozdeleni2, $this ->filterForm ->vcetnePodrizenychRozdeleni2,  // RonPortalSession::rozdeleni2(), RonPortalSession::rozdeleni2VcetnePodrizenych(),
            true); // $rezimUprav = true

        $this ->zpracujData();
        $this ->zpracujDataProAktualniDen();

//        if (RonPortalSettings::getIntValue(RonPortalSettings::KAPACITY_EDITACE) && isset($_POST['ulozitschvaleni'])) {
//            $this ->data ->provedSchvaleni();
//        }

        $this ->reportList = Report::loadFormReports($this ->database, 'PrehledKapacit', 'PlanovaniKapacitP'); // Jen sestavy pro pasivni. Treti parametr z duvodu kompatibility
    }


    public function renderFiltr()
    {
        $this->filterForm ->render();

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_EDITACE) && Prava::vratHodnotuPrava('kapacity_edit')) {
            // tlacitko pro editaci
            echo '<input type="button" class="button clickHref" data-ron-href="?menuid=planovanikapacit&obdobi=' . DateUtils::dateToUrlString($this ->data ->datumOd) . '" value="' . \Ron\ronText('Planovat') . '"/>';
        }
    }


    /**
     * @param string $maska
     * @param Smena $smena
     * @return string
     */
    public function vratTextSmeny($maska, $smena)
    {
        // doplni kod, popis, zkratku
        $maska = str_replace('%KOD%', StringUtils::htmlTextConvert($smena ->kod), $maska);
        $maska = str_replace('%POPIS%', StringUtils::htmlTextConvert($smena ->popis), $maska);
        $maska = str_replace('%ZKRATKA%', StringUtils::htmlTextConvert($smena ->zkratka), $maska);
        // doplni casy
        $maska = str_replace('%CAS_OD%', StringUtils::format('minuty', $smena->pocatekSmeny / 60), $maska);
        $maska = str_replace('%CAS_DO%', StringUtils::format('minuty', $smena->konecSmeny / 60), $maska);
        // delka
        $maska = str_replace('%DELKA%', StringUtils::format('minuty', $smena->delkaSmeny / 60), $maska);

        return $maska;
    }


    /**
     * @param string $maska
     * @param SloupecKapacitDleSmen $sloupec
     * @return string
     */
    public function vratTextAbsence($maska, $sloupec)
    {
        // doplni kod, popis, zkratku
        $maska = str_replace('%KOD%', StringUtils::htmlTextConvert($sloupec ->absence ->kod), $maska);
        $maska = str_replace('%POPIS%', StringUtils::htmlTextConvert($sloupec ->absence ->popis), $maska);
        $maska = str_replace('%POPIS_PRO_DOCHAZKU%', StringUtils::htmlTextConvert($sloupec ->absence ->popisProDochazku), $maska);

        $maska = str_replace('%CAS_OD%', isset($sloupec ->casOd) ? StringUtils::format('minuty', $sloupec->casOd * 60) : '', $maska);
        $maska = str_replace('%CAS_DO%', isset($sloupec ->casDo) ? StringUtils::format('minuty', $sloupec->casDo * 60) : '', $maska);

        return $maska;
    }


    /**
     * @param SloupecKapacitDleSmen[] $sloupce
     * @param bool $zobrazitDatum
     * @param string $textAbsence
     * @param string $titleAbsence
     */
    public function renderTableHeader($sloupce, $zobrazitDatum, $textAbsence, $titleAbsence)
    {
        echo '<thead><tr class="last-section-row">';

        if ($zobrazitDatum) {
            echo '<th class="last-section-col">' . \Ron\ronText('Date') . '</th>';
        }


        foreach ($sloupce as $sloupec) {
            $text = '';
            $title = '';

            if ($sloupec ->smena) {
                $strText = $this ->nastaveni['smena'][0]['text'];
                $strTitle = $this ->nastaveni['smena'][0]['title'];

                $title = $strTitle
                    ? $this ->vratTextSmeny($strTitle, $sloupec ->smena)
                    : StringUtils::htmlTextConvert($sloupec ->smena ->popis);

                if ($strText) {
                    $text = $this ->vratTextSmeny($strText, $sloupec ->smena);
                } else {
                    $text =
                        StringUtils::htmlTextConvert($sloupec->smena->zkratka ?: $sloupec->smena->kod) .
                        ' <span class="casy_smeny">' . '(' .
                        StringUtils::format('minuty', $sloupec->smena->pocatekSmeny / 60) .
                        ' - ' .
                        StringUtils::format('minuty', $sloupec->smena->konecSmeny / 60) .
                        ')' .
                        '</span>';
                }
            } else if ($sloupec ->absence) {
                $strText = $textAbsence;    // $this ->nastaveni['absence'][0]['text'];
                $strTitle = $titleAbsence;  // $this ->nastaveni['absence'][0]['title'];

                $title = $strTitle
                    ? $this ->vratTextAbsence($strTitle, $sloupec)
                    : '';

                $text = $strText
                    ? $this ->vratTextAbsence($strText, $sloupec)
                    : StringUtils::htmlTextConvert($sloupec->absence->popisProDochazku ?: $sloupec ->absence ->popis);
            }

            echo
                '<th title="' . $title . '"' . ($zobrazitDatum && $sloupec ->posledniVeSkupine ? ' class="last-section-col"' : '') . '>' .
                $text .
                '</th>';
        }

        echo "</tr></thead>\n";
    }


    public function renderData()
    {
        foreach ($this ->data ->dny as $index => $den) {
            $className = Kalendar::getStyleForDate($this ->database, $den ->datum);
            $classStr = $className ? "class='$className'" : '';
            echo "<tr $classStr>";

            // zobrazi datum
            $className = 'last-section-col';
            if ($den ->typ & Kalendar::VIKEND) {
                $className .= ' planovani_vikend';
            }
            if ($den ->typ & Kalendar::SVATEK) {
                $className .= ' planovani_svatek';
            }

            if ($den ->className) $className .= ' ' . $den ->className;

            $styleStr = $den ->barva
                ? 'style="background-color: ' . GuiUtils::getHtmlColor($den ->barva) . '"'
                : '';

            $titleStr = $den ->hint
                ? 'title="' . StringUtils::htmlTextConvert($den ->hint) . '"'
                : '';

            echo "<th class='$className' $styleStr $titleStr>" . date('j.n.', $den ->datum) . '</th>';

            // zobrazi data
            foreach ($this ->sloupce as $s => $sloupec) {
                echo
                    '<td' . ($sloupec ->posledniVeSkupine ? ' class="last-section-col"' : '') . '>' .
                    (isset($this ->dataProDny[$index][$s]) ? $this ->dataProDny[$index][$s] : '') .
                    '</td>';
            }


            echo '</tr>';
        }
    }


    public function renderTable()
    {
        if (isset($this ->nastaveni['style'][0])) {
            echo '<style>' . $this ->nastaveni['style'][0]['#text'] . '</style>';
        }

        echo '<div class="planovani_htmldiv">';
        echo '<table class="planovanidlesmen rowOver rowSelect">';

        echo '<thead>';
        $this ->renderTableHeader($this ->sloupce, true,
            isset($this ->nastaveni['absence'][0]['text']) ? $this ->nastaveni['absence'][0]['text'] : '',
            isset($this ->nastaveni['absence'][0]['title']) ? $this ->nastaveni['absence'][0]['title'] : '');
        echo '</thead>';

        $this ->renderData();

        echo "</table>\n";
        echo '</div>';
    }


    private function renderTodayData()
    {
        echo "<tr>";

        // zobrazi data
        foreach ($this ->sloupceProAktualniDen as $sloupec) {
            echo '<td>' . $this ->dataProAktualniDen[$sloupec ->index] . '</td>';
        }

        echo '</tr>';
    }


    public function renderToday()
    {
        if (!isset($this ->nastaveni['aktualniden'])  ||  !$this ->nastaveni['aktualniden']) {
            return;
        }
        $date = DateUtils::today();
        if ($this ->data ->datumOd > $date  ||  $this ->data ->datumDo < $date) {
            // nejsou zobrazena aktualni data
            return;
        }

        echo '<div class="planovani_htmldiv margin-bottom-2">';
        echo '<table class="planovanidlesmen planovanidlesmen-dnes">';

        echo '<thead>';
        echo
            '<tr class="last-section-row">' .
            '<th colspan="' . count($this ->sloupceProAktualniDen) . '">' . \Ron\ronText('Today') . '</th>' .
            '</tr>';
        $this ->renderTableHeader($this->sloupceProAktualniDen, false,
            $this ->nastaveni['aktualniden'][0]['textAbsence'], $this ->nastaveni['aktualniden'][0]['titleAbsence']);
        echo '</thead>';

        $this ->renderTodayData();

        echo "</table>\n";
        echo '</div>';

        //var_dump($this ->sloupceProAktualniDen);
    }


    public function render()
    {
        echo "<h1>" . \Ron\ronText('Prehled_planovanych_kapacit') . "</h1>\n";

        $this ->renderFiltr();

        $this ->renderToday();

        $this ->renderTable();

        // reporty
        if (User::isAuthenticated()) {
            if ($this->reportList && count($this->reportList)) {
                echo '<div class="planovani_html_reports">';
                FormReportsListView::renderReportList($this->reportList,
                    '?menuid=kapacity&action=report&obdobi=' . date('j.n.Y', $this->data->datumOd) .
                    '&rozdeleni=' . StringUtils::htmlTextConvert($this->data->rozdeleni) .
                    (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP) && $this->data->rozdeleni2 ? '&rozdeleni2=' . StringUtils::htmlTextConvert($this->data->rozdeleni2) : '')
                );

                echo '</div>';
            }
        }

        echo
            '<div id="planovanipodlesmen_paticka" data-text-id="Paticka_planovani_podle_smen">' .
            TextManager::_('Paticka_planovani_podle_smen') .
            '</div>';

        // formular schvaleni
        if (User::isAuthenticated()  &&  RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_EDITACE)) {
            $form = new SchvaleniKapacitForm($this->database, false);
            $form->load($this->data);
            $form->render();
        }
    }
}
