<?php
/**
 * stanislav.elbl, 07.11.2019
 */

namespace RonPortal\Kapacity;

use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterOsoba;
use RonPortal\Controls\FilterFormFilterPP;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;


class KapacityFilterForm extends RonPortalFilterForm
{
    /**
     * KapacityFilterFrom constructor.
     * @param Database $database
     * @param string $action
     */
    public function __construct($database, $action, $zobrazitFiltrOsoba = false)
    {
        parent::__construct($database, $action, true);  // useGet = true

        // nastaveni rozdeleni je pres if ve dvou vetvich - jinak si neprenasi vybrane rozdeleni do jinych seznamu (napr. do zpracovane dochazky)
        $typRozdeleni = KapacitySettings::typRozdeleniProVyber();

        if ($typRozdeleni !== RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
            // planovani kapacit ma jiny typ rozdeleni nez je standardni typ
            $this ->addFilterRozdeleni(
                (new FilterFormFilterRozdeleni($this, 'rozdeleni'))
                    ->setTypRozdeleni($typRozdeleni)
                    ->setSessionName('ads_rozdeleni_' . $typRozdeleni)
                    ->setUserSettingsName('ads_rozdeleni_' . $typRozdeleni)
                    ->setPrava(Prava::vratPrava('kapacity'))
            );
        } else {
            $this ->addFilterRozdeleni1(null, Prava::vratPrava('kapacity'));
        }

        if (User::isAuthenticated()) {
            if ($zobrazitFiltrOsoba) {
//                $this->addFilter(
//                    (new FilterFormFilterOsoba($this, 'osoba'))
//                        ->addItem('', TextManager::_('_vsichni_'))
//                        ->setSessionName('filter_osoba_pk')
//                        ->setCaption(TextManager::_('Osoba'))
//                        ->setPrava(Prava::vratPrava('kapacity'))
//                        ->setDefaultValue('')
//                );
                $this ->addFilter(
                    (new FilterFormFilterPP($this, 'osoba'))
                        ->setPovolitVsechnyOsoby(true)
                        ->setSessionName('filter_osoba_pk')
                        ->setPrava(Prava::vratPrava('kapacity'))
                        ->setDefaultValue('')
                );
            }

            // filtr pro rezim zobrazeni
            if (RonPortalSettings::getIntValue(KapacitySettings::POVOLIT_REZIM_UPRAV)) {
                $data = [
                    0 => TextManager::_('Puvodni_plan'),
                    1 => TextManager::_('Upraveny_plan'),
                    2 => TextManager::_('Kombinovany_plan'),
                    3 => TextManager::_('Kapacity_dle_smen')
                ];
            } else {
                $data = [
                    0 => TextManager::_('Puvodni_plan'),
                    3 => TextManager::_('Kapacity_dle_smen')
                ];
            }

            if (RonPortalSettings::getIntValue('kapacity_povolitsloty')) {
                $data[4] = TextManager::_('Prehled_podle_slotu');
            }

            if (RonPortalSettings::getIntValue('planovanipodminek')) {
                $data[5] = TextManager::_('Planovani_podminek');
            }

            $this->addFilter(
                (new FilterFormFilter('rezim'))
                    ->setItems($data)
                    ->setUrl('?menuid=kapacity&rezim=')
                    ->setUserSettingsName('kapacity_rezimuprav')
                    ->setSessionName(null)  // hodnotu bere z usersettings
            );
        }
    }


    protected function afterLoadFilter($filter)
    {
        if ($filter ->name === 'rezim') {
            // v nekterych rezimech nezobrazuje link na zobrazeni skutecnych absenci
            $filterSkutecne = $this ->getFilter('skutecneabsence');

            if ($filterSkutecne) {
                $zobrazitLinkSkutecneAbsence =
                    $this ->getMenuId === 'kapacity'  &&  $filter ->getValue() < 3;
                //NastaveniUzivatele::getUserSettingsItem($this ->database, 'kapacity_rezimuprav', RonPortalSettings::getIntValue('kapacity_rezim')) < 3;

                $filterSkutecne->setEnabled($zobrazitLinkSkutecneAbsence);
            }
        }

        parent::afterLoadFilter($filter);
    }


    public function nastavProPlanovaniObdobi()
    {
        $this ->addFilterObdobi();

//        $zobrazitLinkSkutecneAbsence =
//            $this ->getMenuId === 'kapacity'  &&
//            NastaveniUzivatele::getUserSettingsItem($this ->database, 'kapacity_rezimuprav', RonPortalSettings::getIntValue('kapacity_rezim')) < 3;

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
            $this ->addFilterRozdeleni2(null, Prava::vratPrava('kapacity'));
            //$this ->nastavRozdeleni2(true);
        }

//        if ($zobrazitLinkSkutecneAbsence) {
            $this ->addFilter(
                (new FilterFormFilter('skutecneabsence'))
                    ->addItem(0, TextManager::_('Skryt_skutecne_absence'))
                    ->addItem(1, TextManager::_('Zobrazit_skutecne_absence'))
                    ->setDefaultValue(0)
                    ->setShowSelectedItem(false)
            );
//        }
    }
}
