<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\Smena;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniOsob;
use RonPortal\User;

class KapacityUtils
{
    /**
     * Vraci typ rozdeleni, ktery bude ve vyberu
     * @return int
     * @deprecated
     */
    public static function typRozdeleniProVyber()
    {
        return KapacitySettings::typRozdeleniProVyber();
    }


    /**
     * @param $database
     * @return int
     * @deprecated
     */
    public static function sirkaBunky($database)
    {
        return KapacitySettings::sirkaBunky($database);
    }

    /**
     * @param $database
     * @return int
     * @deprecated
     */
    public static function vyskaBunky($database)
    {
        return KapacitySettings::vyskaBunky($database);
    }

//    /**
//     * @return int
//     * @deprecated
//     */
//    public static function pouzivatKontrolyPlanovani()
//    {
//        return KapacitySettings::pouzivatKontrolyPlanovani();
//    }

    /**
     * @param Database $database
     * @param Parameters $params
     * @param int $typRozdeleni
     * @param int $datumOd
     * @param int $datumDo
     * @return string Vraci filtr pro zadany typ rozdeleni podle prav uzivatele. Pokud jsou prava nastavena podle jineho typu rozdeleni, tak se nefiltruje!
     */
    public static function filtrRozdeleniProPlanovani($database, $params, $typRozdeleni, $datumOd, $datumDo)
    {
        // prida filtr podle prav
        $pravazobrazeni = Prava::vratPrava('kapacity');
        $pravaeditace = Prava::vratPrava('kapacity_edit');

        // VSE - vraci prazdny filtr
        if ($pravazobrazeni->prava == Prava::PRAVA_VSE || $pravaeditace->prava == Prava::PRAVA_VSE) {
            return '';
        }

        $datumDo = $datumDo ?? $datumOd;

        // VLASTNI ROZDELENI VCETNE PODRIZENYCH
        $typr = null;

        if ($pravazobrazeni->prava >= Prava::VLASTNI_A_PODRIZENA_ROZDELENI && $pravazobrazeni->prava <= Prava::PODRIZENA_ROZDELENI_A_SEBE && $typRozdeleni == $pravazobrazeni->typRozdeleni) {
            $typr = $pravazobrazeni->typRozdeleni;
        } else if ($pravaeditace->prava >= Prava::VLASTNI_A_PODRIZENA_ROZDELENI && $pravaeditace->prava <= Prava::PODRIZENA_ROZDELENI_A_SEBE && $typRozdeleni == $pravaeditace->typRozdeleni) {
            $typr = $pravaeditace->typRozdeleni;
        }

        if ($typr) {
            // zobrazeni nebo editace nastaveno na "vlastni a podrizena" nebo podobne
            return
                'EXISTS (' .
                'SELECT * FROM RozdeleniOsob RU, HierarchieRozdeleni H ' .
                ' WHERE RU.KodTypuRozdeleni = ' . (int)$typr .
                '   AND RU.RC = ' . $params ->addParameter(User::getRC()) .
                '   AND (RU.PlatnostOd <= ' . $params->addDateParameter($datumDo) . ')' .
                '   AND (RU.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR RU.PlatnostDo IS NULL) ' .
                '   AND H.TypNadrazene = RU.KodTypuRozdeleni ' .
                '   AND H.KodNadrazene = RU.Rozdeleni ' .
                '   AND H.KodPodrizene = Rozdeleni.Kod ' .
                ')';
        }


        // VLASTNI ROZDELENI (BEZ PODRIZENYCH)
        if ($pravazobrazeni->prava == Prava::VLASTNI_ROZDELENI || ($pravazobrazeni->prava == Prava::VLASTNI_ROZDELENI_KROME_SEBE && $typRozdeleni == $pravazobrazeni->typRozdeleni)) {
            $typr = $pravazobrazeni->typRozdeleni;
        } else if ($pravaeditace->prava == Prava::VLASTNI_ROZDELENI || ($pravaeditace->prava == Prava::VLASTNI_ROZDELENI_KROME_SEBE && $typRozdeleni == $pravaeditace->typRozdeleni)) {
            $typr = $pravaeditace->typRozdeleni;
        }

        if ($typr) {
            // jen vlastni (muze jich byt i vice)
            return
                'EXISTS (' .
                'SELECT * FROM RozdeleniOsob RU ' .
                ' WHERE RU.KodTypuRozdeleni = ' . (int)$typr .
                '   AND RU.RC = ' . $params ->addParameter(User::getRC()) .
                '   AND (RU.PlatnostOd <= ' . $params->addDateParameter($datumDo) . ')' .
                '   AND (RU.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR RU.PlatnostDo IS NULL) ' .
                '   AND RU.Rozdeleni  = Rozdeleni.Kod ' .
                ')';
        }



        /*
        - nastaveni podle pristupovych skupin - skupiny nefiltruje, protoze vubec nemusi byt stejne jako rozdeleni
        - navic pak nejdou planovat "vybrane osoby", protoze nemusi byt v prislusnem rozdeleni

        $filtr = '';

        // prida skupiny pro zobrazovani
        if ($pravazobrazeni -> f_prava == 2  ||  $pravazobrazeni -> f_prava == 4) // vybrane osoby a skupiny
        {
        $poma = explode(';', $pravazobrazeni -> f_skupiny);

        if ($poma)
          foreach ($poma as $skupina)
          {
            if ($filtr) $filtr .= ' OR ';
            $filtr .= "HierarchieRozdeleni.KodNadrazene = '$skupina'";
          }
        }

        // prida skupiny pro editaci
        if ($pravaeditace -> f_prava == 2  ||  $pravaeditace -> f_prava == 4) // vybrane osoby a skupiny
        {
        $poma = explode(';', $pravaeditace -> f_skupiny);
        foreach ($poma as $skupina)
        {
          if ($skupina)
          {
            if ($filtr) $filtr .= ' OR ';
            $filtr .= "HierarchieRozdeleni.KodNadrazene = '$skupina'";
          }
        }
        }
        if ($filtr) {
            $filtr =
                'EXISTS (SELECT * FROM HierarchieRozdeleni WHERE' .
                ' HierarchieRozdeleni.TypNadrazene = Rozdeleni.KodTypuRozdeleni' .
                " AND ($filtr)" . // kod nadrazene - podle seznamu
                ' AND HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                ')';
        } else {
            // - $filtr = '(0 = 1)';
            // jinak vidi ve filtru vsechna rozdeleni a nejak se s tim musi poprat
        }

        return $filtr;
        */

        return '';
    }


    /**
     * Zkontroluje, jestli nebude po naplanovani smeny v zadane delce v intervalu $denOd-$denDo prekrocen maximalni povoleny pocet hodin.
     * (ten zjisti z mesicniho vysledku)
     * Interval muze presahovat do dalsiho mesice - pak kontroluje kazde obdobi zvlast
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $denOd Datum, od ktereho se planuje nova smena
     * @param int $denDo Datum, do ktereho se planuje nova smena
     * @param $delkaNoveSmeny
     * @param $rezimUprav
     * @throws \Exception
     */
    public static function zkontrolujMaximalniPocetHodin($database, $pp, $denOd, $denDo, $delkaNoveSmeny, $rezimUprav)
    {
        if (!RonPortalSettings::getValue('kapacity_cs_soucet')) return;

        $obdobiOd = DateUtils::firstDayOfMonth($denOd);
        $obdobiDo = DateUtils::lastDayOfMonth($denDo);

        $smeny = PlanovaniKapacit_PlanovanaSmena::nactiSmeny($database, $pp->idPP, $obdobiOd, $obdobiDo, $rezimUprav);
        //throw new \Exception(count($smeny));
        $index = 0;
        while ($obdobiOd < $denDo) {
            $aktualniObdobiDo = DateUtils::lastDayOfMonth($obdobiOd);

            // spocita soucet smen v aktualnim obdobi ($obdobiOd)
            $soucet = 0;
            while ($index < count($smeny) && $smeny[$index]->datum <= $aktualniObdobiDo) {
                if ($smeny[$index]->datum < $denOd || $smeny[$index]->datum > $denDo) { // nepocita smeny, ktere spadaji do planovaneho intervalu
                    $soucet += $smeny[$index]->delka;
                }

                $index++;
            }

            $mesicniVysledek = MesicniVysledek::vratMesicniVysledek($database, $pp->idPP, $obdobiOd, RonPortalSettings::getValue('kapacity_cs_soucet'));

            if ($mesicniVysledek && $mesicniVysledek->hodiny) {
                $datumOd = ($obdobiOd < $denOd) ? $denOd : $obdobiOd;
                $datumDo = ($aktualniObdobiDo > $denDo) ? $denDo : $aktualniObdobiDo;

                // pricte nove naplanovane hodiny v aktualnim obdobi
                $soucet += $delkaNoveSmeny * (DateUtils::daysBetween($datumOd, $datumDo) + 1);

                // a overi, jestli neni prekrocen limit
                if ($soucet > (int)($mesicniVysledek->hodiny * 3600)) {
                    throw new \Exception(\Ron\ronText('Prekrocen_povoleny_limit_planovanych_hodin'));
                }
            }

            $obdobiOd = DateUtils::incMonth($obdobiOd);
        }
    }


    /**
     * Nacte schvaleni planu pro zadanou osobu - podle nastaveni Portalu nacita bud schvaleni pro osobu nebo pro jeji rozdeleni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $kodRozdeleni
     * @return SchvaleniKapacit
     * @throws \Exception
     */
    public static function vratSchvaleni($database, $pp, $den, $kodRozdeleni = null)
    {
        if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            $typRozdeleni = KapacitySettings::typRozdeleniProVyber();

            if (!$kodRozdeleni) {
                $rozdeleniOsoby = RozdeleniOsob::nactiRozdeleniOsoby($database, $pp, $typRozdeleni, $den, $den);

                if (!$rozdeleniOsoby) {
                    // nemelo by nastat - zamestnanec neni v zadnem rozdeleni
                    return null;
                }

                $kodRozdeleni = $rozdeleniOsoby[0] ->kod;
            }

            // TODO: zkontrolovat vsechna rozdeleni a vracet max? Asi neni treba - zamestnanec by mel mit jen jedno.
            return SchvaleniKapacitRozdeleni::nactiSchvaleni($database, DateUtils::firstDayOfMonth($den), $typRozdeleni, $kodRozdeleni);
        } else {
            return SchvaleniKapacitPP::nactiSchvaleni($database, DateUtils::firstDayOfMonth($den), $pp->idPP);
        }
    }


    /**
     * Zkontroluje, jestli zadany den jeste neni schvaleny a je mozne jej editovat.
     * V rezimu uprav navic zkontroluje, jestli je puvodni plan schvalen pozadovanou urovni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param bool $rezimUprav
     * @param string $kodRozdeleni
     * @throws \Exception
     */
    public static function zkontrolujSchvaleni($database, $pp, $den, $rezimUprav, $kodRozdeleni = null)
    {
        // nacte schvaleni - podle nastaveni bud pro PP nebo rozdeleni
        $schvaleni = self::vratSchvaleni($database, $pp, $den, $kodRozdeleni);

        if (!$schvaleni) {
            return; // teoreticky muze nastat, pokud se schvaluje cele rozdeleni a zamestnanec nema prirazene rozdeleni
        }

        if ($schvaleni ->lzeEditovatKapacity($den, $rezimUprav)) {
            return;
        }

        if ($rezimUprav && RonPortalSettings::getIntValue(KapacitySettings::MIN_SCHVALENI_PRO_UPRAVY)) {
            if ($schvaleni->getSchvaleniPuvodnihoPlanu() < $den) {
                // nelze editovat v rezimu uprav - puvodni plan jeste neni schvalen
                throw new \Exception(\Ron\ronText('Puvodni_plan_jeste_neni_schvalen'));
            }
        }

        $moznostEditovatSchvalene = PlanovaniKapacit_Osoba::nactiPovoleniEditovatSchvalene($database, $pp ->idPP, $den, $den);
        if ($moznostEditovatSchvalene) {
            // zamestnanec je zarazen v rozdeleni, ktere opravnuje uzivatele k editaci ve schvalenych dnech (lze pouzit napr. pro editaci novych zamestnancu po jejich nastupu)
            return;
        }

        throw new \Exception( \Ron\ronText('Plan_jiz_je_schvalen') );
    }


    /**
     * Zkontroluje, jestli zadany den jeste neni schvaleny a je mozne jej editovat.
     * V rezimu uprav navic zkontroluje, jestli je puvodni plan schvalen pozadovanou urovni
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @return SchvaleniKapacitRozdeleni|void
     * @throws \Exception
     */
    public static function zkontrolujSchvaleniProObdobi($database, $pp, $datumOd, $datumDo, $rezimUprav)   {
        if (RonPortalSettings::getIntValue(KapacitySettings::ZPUSOB_SCHVALOVANI)) {
            // schvaluje se po rozdelenich - musim pro kazdy den zkontrolovat schvaleni rozdeleni, na kterem je zamestnanec
            $params = $database ->createParameters();
            $sql =
                'SELECT Rozdeleni, Datum' .
                ' FROM PlanovaniKapacit' .
                ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
                '   AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
                '   AND Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
                '   AND Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
                ' ORDER BY Datum';

            $cursor = $database ->getSqlResult($sql, $params);
            try {
                $zkontrolovanaRozdeleni = []; // pamatuje si rozdeleni, ktera uz kontroloval

                $row = $cursor ->fetch();
                $datumZaznamu = $row ? $database ->stringToTimestamp($row[1]) : null;

                for ($datum = $datumOd;  $datum <= $datumDo;  $datum = DateUtils::incDayStart($datum)) {
                    $rozdeleni = null;
                    if ($row  &&  $datumZaznamu == $datum) {
                        $rozdeleni = $row[0];
                        $row = $cursor ->fetch();
                        $datumZaznamu = $row ? $database ->stringToTimestamp($row[1]) : null;
                    }

                    // vlastni rozdeleni kontroluje vzdy (muze se menit)
                    if (!$rozdeleni  ||  !isset($zkontrolovanaRozdeleni[$rozdeleni])) {
                        self::zkontrolujSchvaleni($database, $pp, $datum, $rezimUprav, $rozdeleni);

                        if ($rozdeleni) {
                            $zkontrolovanaRozdeleni[$rozdeleni] = 1;
                        }
                    }
                }

                $cursor ->close();
            } catch (\Exception $e) {
                $cursor ->close();
                throw $e;
            }
        } else {
            // schvaluje se po osobach - staci zkontrolovat schvaleni prvniho dne
            self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);
        }
    }


    /**
     * Zapise naplanovane smeny
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kodSmeny
     * @param bool $rezimUprav
     * @param string $editovaneRozdeleni
     * @param object|string $logDescription
     * @throws \Exception
     * @throws NeniPravoException
     */
    public static function zapisPlanovanouSmenu($database, $pp, $datumOd, $datumDo, $kodSmeny, $rezimUprav, $editovaneRozdeleni = null, $logDescription = null)
    {
        if (!$datumOd || !$datumDo) {
            throw new \Exception('Chyba - spatne zadane datum!');
        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        if (!$kodSmeny) {
            $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
            $rec->smazNaplanovaneSmeny($datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni);
            return;
        }

        $smena = Smena::nactiSmenu($database, $kodSmeny);

        // zjisti, jestli je smena variabilni a pripadne jeji preddefinovany pocatek, konec a delku
        if (!$smena) {
            throw new \Exception('Chyba - nelze nacist udaje o smene - zadana smena (' . $kodSmeny . ') neexistuje!');
        }

        $pocatek = null;
        $konec = null;
        $delka = null;
        $preruseniOd = null;
        $preruseniDo = null;

        if ($smena->delena) {
            // delena smena - nastavi prestavku od-do
            if (isset($_POST['preruseniOd'])) {
                $preruseniOd = DateUtils::timeStringToTimestamp($_POST['preruseniOd'], 0);
                $preruseniDo = DateUtils::timeStringToTimestamp($_POST['preruseniDo'], 0);
            } else {
                $preruseniOd = $smena->preruseniOd;
                $preruseniDo = $smena->preruseniDo;
            }
        }

        if ($smena->variabilni) {
            // variabilni smena
            if (isset($_POST['pocatek'])) {
                $pocatek = DateUtils::timeStringToTimestamp($_POST['pocatek'], 0);
                $konec = DateUtils::timeStringToTimestamp($_POST['konec'], 0, true); // $enable_24_00 = true

                if ($konec == 86400) {
                    $konec = 86399;  // 24:00 prevede na 23:59:59
                }
//                if ($konec < $pocatek) {
//                    $konec = DateUtils::incDay($konec, 1);
//                }
                if ($smena ->vzdyDopocitavatDelkuKonec  ||  !isset($_POST['delka'])) {
                    $delka = $smena->vypocitejDelkuSmeny($database, $pocatek, $konec, $preruseniOd, $preruseniDo);
                } else {
                    $delka = DateUtils::timeStringToTimestamp($_POST['delka'], 0, true);
                }
            } else {
                $pocatek = $smena->pocatekSmeny; // isset($row[2]) ? $this ->database ->stringToTimestamp($row[2]) : null;
                $konec = $smena->konecSmeny; // isset($row[3]) ? $this ->database ->stringToTimestamp($row[3]) : null;
                $delka = $smena ->delkaSmeny;
            }
        } else {
            $delka = $smena->delkaSmeny;
        }

        $rezimUprav &= KapacitySettings::povolitRezimuUprav();

        // zkontrolovat maximalni pocet hodin
        self::zkontrolujMaximalniPocetHodin($database, $pp, $datumOd, $datumDo, $delka, $rezimUprav);

        $den = $datumOd;
        while ($den <= $datumDo) {
            $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
            $rec->fHarmonogram->setNewValue(null);
            $rec->zapisSmenu($den, $kodSmeny, $rezimUprav, $editovaneRozdeleni, $pocatek, $konec, $delka, $preruseniOd, $preruseniDo);
            $den = DateUtils::incDayStart($den, 1);
        }
    }


    /**
     * @param Database $database
     * @param SmenaProDen[] $smeny
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param string $editovaneRozdeleni
     * @param string|object $logDescription
     * @throws \Exception
     * @throws NeniPravoException
     */
    private static function naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni, $logDescription)
    {
        $datum = $datumOd;
        $indexSmeny = 0;
        while ($datum <= $datumDo) {
            // preskoci zaznamy pro predchozi dny - ty tam mohou byt, protoze v kalendari muze byt vice smen pro jeden den
            while ($indexSmeny < count($smeny)  &&  $smeny[$indexSmeny] ->datum < $datum) {
                $indexSmeny ++;
            }

            if ($indexSmeny < count($smeny)  &&  $smeny[$indexSmeny] ->datum == $datum) {
                // nasel smenu pro aktualni den => naplanuje ji
                $pocatek = null;
                $konec = null;
                $delkaSmeny = $smeny[$indexSmeny]->delkaSmeny;
                $preruseniOd = null;
                $preruseniDo = null;

                if ($smeny[$indexSmeny]->variabilni) { // variabilni
                    $pocatek = $smeny[$indexSmeny]->pocatekSmeny;
                    $konec = $smeny[$indexSmeny]->konecSmeny;
                }

                if ($smeny[$indexSmeny] ->delena) { // delena smena - doplni prestavku
                    $preruseniOd = $smeny[$indexSmeny]->preruseniOd;
                    $preruseniDo = $smeny[$indexSmeny]->preruseniDo;
                }

                if ($delkaSmeny) {
                    KapacityUtils::zkontrolujMaximalniPocetHodin($database, $pp, $datum, $datum, $delkaSmeny, $rezimUprav);
                }

                $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
                $rec->logInfoMessage = 'Naplanovani smen podle kalendare ' . date('j.n.Y', $datumOd) . ' - ' . date('j.n.Y', $datumDo);
                $rec->zapisSmenu($datum, $smeny[$indexSmeny]->kod, $rezimUprav, $editovaneRozdeleni, $pocatek, $konec, $delkaSmeny, $preruseniOd, $preruseniDo);
            } else {
                // pro aktualni den neni smena v kalendari => smaze naplanovanou smenu
                $rec = new PlanovaniKapacitRecord($database, $pp, $logDescription);
                $rec->smazNaplanovaneSmeny($datum, $datum, $rezimUprav, $editovaneRozdeleni);
            }

            $datum = DateUtils::incDayStart($datum);
        }
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param string $editovaneRozdeleni
     * @param string|object $logDescription
     * @throws \Exception
     */
    public static function naplanujPodleKalendare($database, $idPP, $datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni, $logDescription)
    {
        $pp = PracovniPomer::nactiPP($database, $idPP);

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);

        $smeny = SmenaProDen::nactiSmenyPodleKalendare($database, $pp, $datumOd, $datumDo);

        self::naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni, $logDescription);
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param string $editovaneRozdeleni
     * @param string|object $logDescription
     * @throws NeniPravoException
     * @throws DBException
     */
    public static function naplanujPodleKalendareRozdeleni($database, $idPP, $datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni, $logDescription)
    {
        $typRozdeleni = RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI);
        if (!$typRozdeleni) {
            throw new \Exception('Chyba - nelze naplanovat podle kalendare rozdeleni - prerazovani zamestnancu neni povoleno!');
        }

        $pp = PracovniPomer::nactiPP($database, $idPP);

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        self::zkontrolujSchvaleni($database, $pp, $datumOd, $rezimUprav);

        // nacte smeny podle kalendare rozdeleni prirazeneho osobe ...
        $smeny1 = SmenaProDen::nactiSmenyPodleKalendareRozdeleni($database, $typRozdeleni, $pp, $datumOd, $datumDo);
        // ... a podle kalandare rozdeleni zmeneneho pro jednotlive dny
        $smeny2 = SmenaProDen::nactiSmenyPodleZmenenehoRozdeleni($database, $pp, $datumOd, $datumDo, KapacitySettings::povolitRezimuUprav() && !$rezimUprav);

        // slouci zaznamy z $smeny2 a $smeny1 do jednoho pole. Pokud je pro nejaky den zaznam v $smeny2 i $smeny1,
        // tak ma prednost $smeny2 (tj. rozdeleni do ktereho je zamestnanec prerazen)
        $smeny = [];
        $index1 = 0;
        $index2 = 0;

        while ($index1 < count($smeny1)  ||  $index2 < count($smeny2)) {
            if ($index2 < count($smeny2)) {
                // jeste jsou nejake zaznamy pro jednotlive dny

                // doplni zaznamy pro predchozi dny
                while ($index1 < count($smeny1)  &&  $smeny1[$index1] ->datum < $smeny2[$index2] ->datum) {
                    $smeny[] = $smeny1[$index1];
                    $index1 ++;
                }

                // prida aktualni zaznam z $smeny2
                $smeny[] = $smeny2[$index2];

                // preskoci zaznamy z $smeny1 pro aktualni den (protoze nasel zaznam v $smeny2)
                while ($index1 < count($smeny1)  &&  $smeny1[$index1] ->datum = $smeny2[$index2] ->datum) {
                    $index1 ++;
                }

                $index2++;
            } else {
                // uz nejsou zaznamy pro jednotlive dny - doplni zaznamy podle rozdeleni osoby
                while ($index1 < count($smeny1)) {
                    $smeny[] = $smeny1[$index1];
                    $index1 ++;
                }
            }
        }


        self::naplanujSmeny($database, $smeny, $pp, $datumOd, $datumDo, $rezimUprav, $editovaneRozdeleni, $logDescription);
    }
}
