<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Kapacity;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\DenniZaznamyBrowser;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;

class PlanovaneSmenyBrowserField extends BrowserField
{
    var $fieldIndex;
    var $fieldIndexVariabilni;

    /**
     * @param Database $database
     * @param int    $fieldIndex
     * @param int    $fieldIndexVariabilni
     */
    function __construct($database, $fieldIndex, $fieldIndexVariabilni)
    {
        parent::__construct($database);

        $this->align = 'right';

        $this->fieldIndex = $fieldIndex;
        $this->fieldIndexVariabilni = $fieldIndexVariabilni;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData  &&  count($rowData) > 1) {
            $index = $rowData['col-variabilni']
                ? $this->fieldIndexVariabilni // variabilni smena
                : $this->fieldIndex;

            return $rowData['@' . $index];

//            if ($rowData[$index]) {
//                return DateUtils::minutesToString($this->database->stringToTimeOfDay($rowData[$index]) * 60);
//            }
        }

        return '';
    }
}


class PreruseniBrowserField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->header = TextManager::_('Preruseni_smeny');
        $this->align = 'center';
        $this ->sqlFieldIndex2Separator = ' - ';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData  &&  count($rowData) > 1  &&  $rowData['col-smena-delena']) {
            return
                $rowData['@col-preruseni-od'] .
                $this ->sqlFieldIndex2Separator .
                $rowData['@col-preruseni-do'];
        }

        return '';
    }
}


class PlanovaneSmenyBrowser extends DenniZaznamyBrowser
{
    /** @var int */
    protected $zobrazitAzPoSchvaleniUrovni = 0;

    /** @var int|null */
    protected $schvaleniPlanuDo = null;

    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'prehledsmen');

        $this
            ->setTitle('Prehled_planovanych_smen')
            ->setCode('planovanesmeny')
            ->setAction('prehledsmen')
            ->setCssClass('browser grid');

        $this->sqlSelectPart =
            'PlanovaniKapacit.Datum'; // 0 - musi byt prvni!

        $this->sqlFromPart =
            '(PlanovaniKapacit LEFT OUTER JOIN Smena ON PlanovaniKapacit.Smena = Smena.Kod)' .
            ' LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni' .
            '  AND Rozdeleni.Kod = PlanovaniKapacit.Rozdeleni' .
            ' LEFT OUTER JOIN Rozdeleni Rozdeleni2 ' .
            '   ON Rozdeleni2.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni2' .
            '  AND Rozdeleni2.Kod = PlanovaniKapacit.Rozdeleni2' .
            ' INNER JOIN PracovniPomer' .
            '   ON PracovniPomer.RC = PlanovaniKapacit.RC' .
            '  AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '   ON Osoba.RC = PlanovaniKapacit.RC';

        $this->sqlWherePart = '';

        $this ->addField()
            ->setSqlFieldName('PlanovaniKapacit.Datum', 0)
            ->setCode('col-datum')
            ->setHeaderText('Date') ->setFormat('timestamp') ->setClassName('browser_rowheader');

        $this->addField()
            ->setSqlFieldName('Smena.Kod')
            ->setCode('col-kod');

        $this->addField()
            ->setSqlFieldName('Smena.Popis')
            ->setCode('col-popis')
            ->setHeaderText('Smena');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Smena.Variabilni')
            ->setCode('col-variabilni');

        // casy pevne smeny (ze zaznamu smeny)
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Smena.PocatekSmeny')
            ->setCode('col-smena-pocatek')
            ->setFormat('timeofday');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Smena.KonecSmeny')
            ->setCode('col-smena-konec')
            ->setFormat('timeofday');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Smena.DelkaSmeny')
            ->setCode('col-smena-delka')
            ->setFormat('timeofday');

        // casy variabilni smeny (ze zaznamu planovani kapacit
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaniKapacit.Smena_Pocatek')
            ->setCode('col-variabilni-pocatek')
            ->setFormat('timeofday');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaniKapacit.Smena_Konec')
            ->setCode('col-variabilni-konec')
            ->setFormat('timeofday');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaniKapacit.Smena_Delka')
            ->setCode('col-variabilni-delka')
            ->setFormat('timeofday');

        $this->addField(new PlanovaneSmenyBrowserField($database, 'col-smena-pocatek', 'col-variabilni-pocatek'))  // pocatek smeny
            ->setHeaderText('Pocatek') ->setCode('col-pocatek');

        $this->addField(new PlanovaneSmenyBrowserField($database, 'col-smena-konec', 'col-variabilni-konec')) // konec smeny
            ->setHeaderText('Konec') ->setCode('col-konec');

        $this->addField(new PlanovaneSmenyBrowserField($database, 'col-smena-delka', 'col-variabilni-delka')) // delka smeny
            ->setHeaderText('Delka') ->setCode('col-delka');


        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Smena.Delena')
            ->setCode('col-smena-delena');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaniKapacit.Smena_PreruseniOd')
            ->setCode('col-preruseni-od')
            ->setFormat('timeofday');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaniKapacit.Smena_PreruseniDo')
            ->setCode('col-preruseni-do')
            ->setFormat('timeofday');

        $this->addField(new PreruseniBrowserField($database));

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2));

            $this->addField()
                ->setSqlFieldName('PlanovaniKapacit.Rozdeleni2')
                ->setCode('col-rozdeleni2')
                ->setHeader($typRozdeleni ? StringUtils::htmlTextConvert($typRozdeleni ->popis) : '', 2);

            $this->addField()
                ->setSqlFieldName('Rozdeleni2.Popis')
                ->setCode('col-popis-rozdeleni2');
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));

            $this->addField()
                ->setSqlFieldName('PlanovaniKapacit.Rozdeleni')
                ->setCode('col-rozdeleni')
                ->setHeader($typRozdeleni ? StringUtils::htmlTextConvert($typRozdeleni ->popis) : '', 2);

            $this->addField()
                ->setSqlFieldName('Rozdeleni.Popis')
                ->setCode('col-popis-rozdeleni');
        }


        $this->addField()
            ->setSqlFieldName('PlanovaniKapacit.Harmonogram')
            ->setCode('col-harmonogram') ->setVisible(false)
            ->setHeaderText('Harmonogram');

        $this->addField()
            ->setSqlFieldName('Smena.PocatekPevneCasti') ->setFormat('timeofday')
            ->setCode('col-pocatek-pevne-casti') ->setVisible(false)
            ->setHeaderText('Pocatek_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.KonecPevneCasti') ->setFormat('timeofday')
            ->setCode('col-konec-pevne-casti') ->setVisible(false)
            ->setHeaderText('Konec_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.DelkaPevneCasti') ->setFormat('timeofday')
            ->setCode('col-delka-pevne-casti') ->setVisible(false)
            ->setHeaderText('Delka_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.ZkratkaProPlanovani')
            ->setCode('col-zkratka') ->setVisible(false)
            ->setHeaderText('Zkratka');

        $this->defaultSort = 0;
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        // pokud ma uzivatel pravo na prohlizeni planovanych kapacit, tak se pouzije toto pravo
        // a navic se z nej pouzije nastaveni "Povolit zobrazeni az po schvaleni planu zadanou urovni"
        $kodPrava = 'prohlizeni';

        $prava = Prava::vratPrava('kapacity');
        if ($prava ->jsouNejakePrava()) {
            $kodPrava = 'kapacity';
            $this ->zobrazitAzPoSchvaleniUrovni = $prava ->priznak2;
        }

        $this ->filterForm = $this->createFilterForm(
            'ADS', $kodPrava,
            false, RonPortalBrowser::OSOBY_JEN_S_DOCHAZKOU);
    }


    public function getRowAttributes($values, $className, $rowNumber)
    {
        $result = parent::getRowAttributes($values, $className, $rowNumber);

        if (isset($this ->schvaleniPlanuDo)) {
            if ($values[0] > $this ->schvaleniPlanuDo) {
                $result['class'] = ($result['class'] ?? '') . ' neschvaleny-den';
            }
        }

        return $result;
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $sql = $this ->filterForm ->buildSqlFilter($params);

        $datumOd = $this ->filterForm ->getOmezitObdobiOd();
        $datumDo = $this ->filterForm ->getOmezitObdobiDo();

        if ($this ->zobrazitAzPoSchvaleniUrovni) {
            $pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->getFilterValue('osoba'));
            $schvaleni = KapacityUtils::vratSchvaleni($this ->database, $pp, $datumOd);

            $datumDo = match ((int)$this ->zobrazitAzPoSchvaleniUrovni) {
                1 => $schvaleni ->schvaleni1,
                2 => $schvaleni ->schvaleni2,
                3 => $schvaleni ->schvaleni3,
                default => $datumDo
            };

            $this ->schvaleniPlanuDo = $datumDo ?? 0;
        }

        return new SqlStatement(
            $sql . ($sql ? ' AND ' : '') .
            'PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($datumOd) .
            ' AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($datumDo) .
            ' AND PlanovaniKapacit.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND PlanovaniKapacit.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }

    public function renderHeader()
    {
        parent::renderHeader();

        if (isset($this ->schvaleniPlanuDo)) {
            if (!$this ->schvaleniPlanuDo) {
                echo '<div class="warningmsg">' . TextManager::_('Plan_smen_jeste_neni_schvalen') . "</div>\n";
            } else {
                $className = $this ->schvaleniPlanuDo === $this ->filterForm ->getOmezitObdobiDo()
                    ? 'successmsg'
                    : 'warningmsg';

                echo "<div class='$className'>" . sprintf(TextManager::_('Plan_smen_je_schvalen_do'), StringUtils::format('date', $this ->schvaleniPlanuDo)) . "</div>\n";
            }
        }
    }


}
