<?php

namespace RonPortal\Kapacity;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Prava;

class PlanovaniKapacit_SluzebniCesta
{
    /** @var int */
    public $idPP;
    /** @var int */
    public $id;
    /** @var int */
    public $datumOd;
    /** @var int */
    public $datumDo;
    /** @var string */
    public $popis;


    /**
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param int[] $seznamPP
     * @return PlanovaniKapacit_SluzebniCesta[]
     */
    public static function nactiSluzebniCestyZDB($database, $datumOd, $datumDo, $seznamPP)
    {
        $prava = Prava::vratPrava('kapacity');
        $pravaEdit = Prava::vratPrava('kapacity_edit');

        $params = $database ->createParameters();
        $sql =
            'SELECT PracovniPomer.Id, SluzebniCesta.Id, SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo, SluzebniCesta.Popis' .
            ' FROM SluzebniCesta, PracovniPomer' .
            ' WHERE SluzebniCesta.RC = PracovniPomer.RC' .
            ' AND SluzebniCesta.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND SluzebniCesta.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND SluzebniCesta.DatumCasDo >= ' . $params ->addDateParameter($datumOd) .
            ' AND SluzebniCesta.DatumCasOd < ' . $params ->addDateParameter(DateUtils::incDayStart($datumDo, 1)); // SC muze mit nastaveny i cas zacatku!

        $sql .= ' AND (0 = 1';
        foreach ($seznamPP as $idPP) {
            $sql .= ' OR PracovniPomer.Id = ' . $params ->addParameter((int)$idPP);
        }
        $sql .= ')';

        $result = [];

        try {
            $cursor = $database->getSqlResult($sql, $params, 'Chyba pri nacitani sluzebnich cest');
            while ($row = $cursor->fetch()) {
                if (Prava::lzeZobrazitPP([$prava, $pravaEdit], $row[0], $datumOd, $datumDo)) {
                    $sc = new self();
                    $sc->idPP = (int)$row[0];
                    $sc->id = (int)$row[1];
                    $sc->datumOd = $cursor->stringToTimestamp($row[2]);
                    $sc->datumDo = $cursor->stringToTimestamp($row[3]);
                    $sc->popis = $row[4];

                    $result[] = $sc;
                }
            }
            $cursor->close();
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        return $result;
    }


    /**
     * @return array
     */
    public function getJson() {
        return [
            'idPP' => $this ->idPP,
            'id' => $this ->id,
            'datumOdStr' => StringUtils::jsonDate($this ->datumOd),
            'datumDoStr' => StringUtils::jsonDate($this ->datumDo),
            'popis' => StringUtils::jsonText($this ->popis)
        ];
    }
}
