<?php
/**
 * stanislav.elbl, 29.08.2019
 * Formular pro planovani kapacit podle definovanych podminek.
 * Podminky obsahuji seznam pozic (rozdeleni), ktere je potreba na lince/stredisku obsadit. K temto pozicim
 * se prirazuji pracovnici, kteri musi mit tuto pozici povolenou (prirazenou).
 * V zaznamu planovani kapacit se pozice priradi v poli rozdeleni2.
 *
 * ROZDELENI XY
 * POZICE       |    SMENA 1     |     SMENA 2     |    SMENA   3
 * Pozice 1     |    Karel       |     Pepa        |     ...
 * Pozice 2     |    Frantisek   |     Honza       |
 */

namespace RonPortal\Kapacity;

//class ZaznamPlanovaniPodminek
//{
//    /** @var string */
//    var $kodSmeny;
//    /** @var Smena */
//    var $smena;
//
//    /** @var int */
//    var $idPP;
//
//    /** @var string     Kod pozice (rozdeleni 2)*/
//    var $pozice;
//
//}


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\BaseViewObject;
use RonPortal\Dochazka\Smena;
use RonPortal\Obdobi;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;

class RadekPlanovaniPodminek
{
    /** @var int        Kod typu rozdeleni 2 (z podminky planovani) */
    var $kodTypuRozdeleni2;
    /** @var string     Kod rozdeleni */
    var $kodRozdeleni2;
    /** @var string     Popis rozdeleni */
    var $popisRozdeleni2;

    /** @var PodminkaPlanovaniKapacit[]    Podminky souvisejici s timto radkem. Indexovano kodem smeny. */
    var $podminky;


    function getPopis()
    {
        return $this ->popisRozdeleni2
            ? $this ->popisRozdeleni2 . '(' . $this ->kodRozdeleni2 . ')'
            : $this ->kodRozdeleni2;
    }
}


class RozdeleniProPlanovaniPodminek
{
    /** @var Rozdeleni */
    var $rozdeleni;

    /** @var SchvaleniKapacitRozdeleni */
    var $schvaleni;

    /** @var Smena[]        Seznam smen (sloupce tabulky) */
    var $smeny = [];

    /** @var RadekPlanovaniPodminek[] */
    var $radky = [];

    /**
     * @param PodminkaPlanovaniKapacit $podminka
     * @return RadekPlanovaniPodminek
     */
    private function najdiRadekProPodminku($podminka)
    {
        foreach ($this ->radky as $radek) {
            // pokud je radek pro stejne rozdeleni, tak je to ten hledany
            if ($podminka ->kodTypuRozdeleni == $radek ->kodTypuRozdeleni2  &&  $podminka ->rozdeleni == $radek ->kodRozdeleni2) {
                //$radek ->podminky[] = $podminka;
                return $radek;
            }
        }

        $radek = new RadekPlanovaniPodminek();
        $radek ->kodTypuRozdeleni2 = $podminka ->kodTypuRozdeleni;
        $radek ->kodRozdeleni2 = $podminka ->rozdeleni;
        $radek ->podminky = [];
        //$radek ->podminky = [$podminka];

        $this ->radky[] = $radek;

        return $radek;
    }


    /**
     * @param Database $database
     * @param SadaPodminekPlanovani[] $sadyPodminek
     */
    public function vytvorRadkyZPodminek($database, $sadyPodminek)
    {
        foreach ($sadyPodminek as $sada) {
            foreach ($sada ->podminky as $podminka) {
                $radek = $this ->najdiRadekProPodminku($podminka);
                $smena = null;


                if ($podminka ->typPodminky == PodminkaPlanovaniKapacit::TYP_SMENA  &&  $podminka ->smena) {
                    if (isset($this ->smeny[$podminka ->smena])) {
                        $smena = $this ->smeny[$podminka ->smena];
                    } else {
                        $smena = Smena::vratSmenu($database, $podminka ->smena);
                        $this ->smeny[$podminka ->smena] = $smena;
                    }
                } else if ($podminka ->typPodminky == PodminkaPlanovaniKapacit::TYP_CAS_OD_DO) {
                    $kodSmeny =
                        '#' . DateUtils::minutesToString($podminka ->casOd / 60) .
                        '-' . DateUtils::minutesToString($podminka ->casDo / 60);
                    if (isset($this ->smeny[$kodSmeny])) {
                        $smena = $this ->smeny[$kodSmeny];
                    } else {
                        $smena = new Smena();
                        $smena ->kod = $kodSmeny;
                        $smena ->popis = $podminka ->getPopis();
                        $smena ->pocatekSmeny = $podminka ->casOd;
                        $smena ->konecSmeny = $podminka ->casDo;
                        $this ->smeny[$kodSmeny] = $smena;
                    }
                }

                if ($smena) {
                    $radek->podminky[$smena->kod] = $podminka;
                }
            }
        }
    }


    /**
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     */
    public function loadData($database, $rozdeleni, $datum)
    {
        $this ->rozdeleni = $rozdeleni;

        $this ->radky = [];

        $sadyPodminek = SadaPodminekPlanovani::nactiSadyPodminekProDen($database, $this ->rozdeleni ->typ, $this ->rozdeleni ->kod, $datum);
        $this ->vytvorRadkyZPodminek($database, $sadyPodminek);

        // seradi smeny podle pocatku
        $smeny = [];
        foreach ($this ->smeny as $smena) {
            $smeny[] = $smena;
        }

        usort($smeny, function($smena1, $smena2) {
            /** @var Smena $smena1 */
            /** @var Smena $smena2 */
            if ($smena1 ->pocatekSmeny < $smena2 ->pocatekSmeny) {
                return -1;
            }

            if ($smena1 ->pocatekSmeny > $smena2 ->pocatekSmeny) {
                return 1;
            }

            return 0;
        });

        $this ->smeny = $smeny;
    }
}


class PlanovaniPodminekFormular extends  BaseViewObject
{
    /** @var Database */
    var $database;

    /** @var Obdobi */
    var $obdobi;

    var $rezimUprav = false;

    /** @var TypRozdeleni */
    var $typRozdeleniLinka;
    /** @var string */
    var $typRozdeleniPozice;

    // TODO: umoznit planovani pro vice rozdeleni !
    /** @var Rozdeleni      Hlavni rozdeleni - mohou byt zobrazena podrizena rozdeleni */
    var $rozdeleni;
    /** @var bool */
    var $vcetnePodrizenych = true;
    /** @var int            Zobrazeny den */
    var $datum;

    /** @var RozdeleniProPlanovaniPodminek[]    Seznam nactenych rozdeleni a jejich dat */
    var $seznamRozdeleni;

    /** @var string[]   Asociativni pole barev smen - pro generovani css */
    var $barvySmen = [];

    /** @var Smena[]        Seznam smen (sloupce tabulky) */
    //var $smeny = [];

    /** @var RadekPlanovaniPodminek[] */
    //var $radky = [];

    var $pocetDnuDetailu = 9;
    /** @var int            Prvni zobrazeny den v detailu osoby */
    var $prvniDenDetailu;


    /**
     * PlanovaniPodminekFormular constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('ronportal/kapacity/view_planovani_podminek.php');
        $this->database = $database;
    }


    /**
     * @param Smena[] $smeny
     */
    private function doplnSmeny($smeny)
    {
        if (!$smeny) {
            return;
        }

        foreach ($smeny as $smena) {
            if ($smena ->barva) {
                $this ->barvySmen[$smena ->kod] = $smena ->barva;
            }
        }
    }


    /**
     * @param Rozdeleni $rozdeleni
     * @param int   $datum
     * @param bool  $vcetnePodrizenych
     */
    public function loadData($rozdeleni, $datum, $vcetnePodrizenych)
    {
        $this ->rozdeleni = $rozdeleni;
        $this ->vcetnePodrizenych = $vcetnePodrizenych;
        $this ->datum = $datum;

        $this ->prvniDenDetailu = DateUtils::incDayStart($this ->datum, -(int)($this ->pocetDnuDetailu / 2));

        $this ->typRozdeleniLinka = TypRozdeleni::getTypRozdeleni($this ->database, $this ->rozdeleni ->typ);
        $this ->typRozdeleniPozice = RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2);

        $this ->obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $this ->datum);

        $this ->seznamRozdeleni = [];

        if ($vcetnePodrizenych) {
            $params = $this ->database ->createParameters();
            $cursor = $this ->database ->getSqlResult(
                'SELECT Rozdeleni.KodTypuRozdeleni, Rozdeleni.Kod, Rozdeleni.Popis,' .
                '  SchvaleniPlanuRozdeleni.Schvaleni1, SchvaleniPlanuRozdeleni.Schvaleni2, SchvaleniPlanuRozdeleni.Schvaleni3' .
                ' FROM HierarchieRozdeleni INNER JOIN Rozdeleni' .
                '  ON Rozdeleni.KodTypuRozdeleni = HierarchieRozdeleni.TypPodrizene  AND  Rozdeleni.Kod = HierarchieRozdeleni.KodPodrizene' .
                " INNER JOIN Obdobi ON Obdobi.Aplikace = 'ADS'" .    // pripojim i obdobi, aby nemusely byt parametry v ON a fungovalo to na starsich SQL
                ' LEFT OUTER JOIN SchvaleniPlanuRozdeleni' .
                '  ON SchvaleniPlanuRozdeleni.Aplikace = Obdobi.Aplikace' .
                '  AND SchvaleniPlanuRozdeleni.PocatekObdobi = Obdobi.PocatekObdobi' .
                '  AND SchvaleniPlanuRozdeleni.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '  AND SchvaleniPlanuRozdeleni.Rozdeleni = Rozdeleni.Kod' .
                ' WHERE Obdobi.PocatekObdobi = ' . $params ->addParameter($this ->database ->dateToParam($this ->obdobi ->zacatek)) .
                '  AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam($datum)) .
                '  AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($datum)) . ' OR Rozdeleni.DatumZruseni IS NULL)' .
                '  AND HierarchieRozdeleni.TypNadrazene = ' . $params ->addParameter($rozdeleni ->typ) .
                '  AND HierarchieRozdeleni.KodNadrazene = ' . $params ->addParameter($rozdeleni ->kod) .
                ' ORDER BY Rozdeleni.Popis',
                $params
            );

            while ($row = $cursor ->fetch()) {
                $r = new Rozdeleni();
                $r ->typ = (int)$row[0];
                $r ->kod = $row[1];
                $r ->popis = $row[2];

                $rozdeleniProPlanovani = new RozdeleniProPlanovaniPodminek();
                $rozdeleniProPlanovani->loadData($this ->database, $r, $datum);

                $rozdeleniProPlanovani->schvaleni = new SchvaleniKapacitRozdeleni();
                $rozdeleniProPlanovani->schvaleni->schvaleni1 = $this ->database ->stringToTimestamp($row[3]);
                $rozdeleniProPlanovani->schvaleni->schvaleni2 = $this ->database ->stringToTimestamp($row[4]);
                $rozdeleniProPlanovani->schvaleni->schvaleni3 = $this ->database ->stringToTimestamp($row[5]);

                if ($rozdeleniProPlanovani ->smeny) {
                    $this->seznamRozdeleni[] = $rozdeleniProPlanovani;
                    $this ->doplnSmeny($rozdeleniProPlanovani ->smeny);
                }
            }
        } else {
            $rozdeleniProPlanovani = new RozdeleniProPlanovaniPodminek();
            $rozdeleniProPlanovani->loadData($this->database, $rozdeleni, $datum);
            $rozdeleniProPlanovani->schvaleni = SchvaleniKapacitRozdeleni::nactiSchvaleni($this ->database, $this ->obdobi ->zacatek, $rozdeleni ->typ, $rozdeleni ->kod);
            $this->seznamRozdeleni[] = $rozdeleniProPlanovani;
            $this ->doplnSmeny($rozdeleniProPlanovani ->smeny);
        }
    }
}
