<?php
/**
 * stanislav.elbl, 13.11.2019
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;

class PodminkaPlanovaniRecord extends RonPortalDBRecord
{
    /** @var DBRecordField */
    public $fId;
    /** @var DBRecordField */
    public $fIdSady;
    /** @var DBRecordField */
    public $fPocetMin;

    /** @var DBRecordField */
    public $fUpdateUserName;
    /** @var DBRecordField */
    public $fUpdateTimestamp;

    /**
     * PodminkaPlanovaniRecord constructor.
     * @param Database $database
     * @param object|string $logDescription
     */
    public function __construct($database, $logDescription)
    {
        parent::__construct($database, 'PodminkaPlanovaniKapacit', $logDescription);

        $this ->beforePostEvent = 'PodminkaPlanovaniKapacit_BEFORE_POST';
        $this ->afterPostEvent = 'PodminkaPlanovaniKapacit_AFTER_POST';

        $this ->fId = $this->addField('Id', 'sequence', true);
        $this ->fIdSady = $this->addField('IdSady', 'integer', true);

        $this ->fPocetMin = $this->addField('PocetMin', 'integer', true);
        $this->addField('PocetMax', 'integer', true);
        $this->addField('TypKontroly', 'string', true, 10);

        $this ->addField('KodTypuRozdeleni', 'integer', true);
        $this ->addField('Rozdeleni', 'string', true, 30);

        $this ->addField('KodDovednosti', 'string', true, 40);
        $this ->addField('KodUrovne', 'integer', true);

        $this ->addField('Smena', 'string', true, 30);
        $this ->addField('CasOd', 'time', true);
        $this ->addField('CasDo', 'time', true);

        $this ->addField('PriraditOsobeSplneniPodm', 'integer', true);
        $this ->addField('TypPodminky', 'integer', true);

        $this ->addField('TextHlaseniMin', 'memo', true);
        $this ->addField('TextHlaseniMax', 'memo', true);

        $this ->addField('SlotProPlanovani', 'integer', true);
        $this ->addField('LzeEditovat', 'integer', true);

        //$this ->addField('Popis', 'string', true, 40); // TODO: doplnit nove pole

        $this ->fUpdateUserName = $this ->addField('Update_UserName', 'string', true, 25);
        $this ->fUpdateUserName ->setUpdateLog(true);
        $this ->fUpdateTimestamp = $this ->addField('Update_Timestamp', 'timestamp', true);
        $this ->fUpdateTimestamp ->setUpdateLog(true);
    }


    /**
     * @param Database $database
     * @param int $id
     * @param object|string $logDescription
     * @return PodminkaPlanovaniRecord
     * @throws DBException
     */
    public static function loadId($database, $id, $logDescription)
    {
        $rec = new static($database, $logDescription);
        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($id), $params);

        $rec ->loadRecord();
        return $rec;
    }

    public function setUpdateUser()
    {
        $this ->fUpdateUserName ->setNewValue(RonPortalUtils::getLogUserName());
        $this ->fUpdateTimestamp ->setNewValue(time());
    }
}
