<?php

namespace  RonPortal\Kapacity;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBRecordField;
use Ron\StringUtils;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\System\AgentManager;

abstract class SchvaleniKapacitAbstractRecord extends RonPortalDBRecord
{
    /**
     * @var int
     */
    var $obdobi;


    /** @var DBRecordField */
    var $fSchvaleni1;
    /** @var DBRecordField */
    var $fSchvalil1;
    /** @var DBRecordField */
    var $fCas_Schvaleni1;

    /** @var DBRecordField */
    var $fSchvaleni2;
    /** @var DBRecordField */
    var $fSchvalil2;
    /** @var DBRecordField */
    var $fCas_Schvaleni2;

    /** @var DBRecordField */
    var $fSchvaleni3;
    /** @var DBRecordField */
    var $fSchvalil3;
    /** @var DBRecordField */
    var $fCas_Schvaleni3;

    /** @var DBRecordField */
    var $fSchvaleniUprav1;
    /** @var DBRecordField */
    var $UpravySchvalil1;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav1;

    /** @var DBRecordField */
    var $fSchvaleniUprav2;
    /** @var DBRecordField */
    var $UpravySchvalil2;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav2;

    /** @var DBRecordField */
    var $fSchvaleniUprav3;
    /** @var DBRecordField */
    var $UpravySchvalil3;
    /** @var DBRecordField */
    var $Cas_SchvaleniUprav3;

    /** @var DBRecordField */
    var $fPoznamka;

    /**
     * @param Database $database
     * @param string $tableName
     * @param int $obdobi
     * @param string|object $logDescription
     */
    public function __construct($database, $tableName, $obdobi, $logDescription = 'Planovani kapacit')
    {
        parent::__construct($database, $tableName, $logDescription);

        $this ->obdobi = $obdobi;

        $this ->addField('Aplikace', '', true, 0, 'ADS');
        $this ->addField('PocatekObdobi', 'date', true, 0, $obdobi);
    }

    /**
     * @param PravaNaOsoby $pravo
     * @return bool
     */
    abstract public function zkontrolujPravo($pravo);

    /**
     * @param int $uroven
     * @param int $datumDo
     */
    abstract public function overKontroly($uroven, $datumDo);

    /**
     * Zkontroluje, jestli ma uzivatel pravo na schvaleni. Zaznam uz musi byt nacten!
     * @param int  $uroven
     * @param int  $hodnota
     * @param int  $hodnotaOriginal   Puvodni hodnota - pro kontrolu prava na snizeni
     * @throws Exception
     */
    public function zkontrolujPravoNaSchvaleni($uroven, $hodnota, $hodnotaOriginal)
    {
        $prava = Prava::vratPrava('kapac_schval' . $uroven);
        if (!$this->zkontrolujPravo($prava)) {
            // vubec nema pravo
            throw new NeniPravoException();
        }


        if ((int)$hodnota < (int)$hodnotaOriginal) {
            // jedna se o snizeni - overi jestli muze snizit datum
            if ($prava ->priznak) {
                // uzivatel nesmi snizit tuto uroven
                throw new Exception(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
            }

            if ($uroven < 3 && RonPortalSettings::getIntValue(KapacitySettings::ZAKAZAT_SNIZIT_SCHVALENI)) {
                // uzivatel snizuje - provede se kontrola, jestli se nejedna o nejvyssi uroven, na kterou ma pravo
                $prava = Prava::vratPrava('kapac_schval3');
                if ($this->zkontrolujPravo($prava)) {
                    // ma pravo na uroven 3 => muze snizovat
                    return;
                }

                if ($uroven === 1) {
                    $prava = Prava::vratPrava('kapac_schval2');
                    if ($this->zkontrolujPravo($prava)) {
                        // ma pravo na uroven 2 => muze snizovat uroven 1
                        return;
                    }
                }

                throw new Exception(\Ron\ronText('Nelze_snizit_schvaleni_PK'));
            }
        }
    }


    /**
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $datumDo
     * @return string
     */
    public function getParametrSkriptu($uroven, $rezimUprav, $datumDo)
    {
        $datumDoStr = $datumDo ? date('j.n.Y', $datumDo) : '0';
        return
            "uroven=$uroven;obdobi=" . date('j.n.Y', $this ->obdobi) .
            ';datum=' . $datumDoStr .
            ';rezimuprav=' . ($rezimUprav ? '1' : '0');
    }


    public function getAgentParameters($uroven, $rezimUprav, $hodnota)
    {
        //$hodnotaStr = date('Y-n-j', $hodnota ?: DateUtils::incDay($this ->obdobi, -1));
        $hodnotaStr = date('Y-n-j', $hodnota ?: $this ->obdobi);
        return
            ' -period:' . date('Y-n-j', $this ->obdobi) .
            ' -apprLevel:' . $uroven .
            ($rezimUprav ? ' -modificationMode' : '') .
            ' -dateTo:' . $hodnotaStr;
    }

    /**
     * @param Database $database
     * @param int $uroven
     * @param bool $rezimUprav
     * @param int $hodnota  Datum, do kdy se ma schvalit
     * @throws Exception
     */
    public function schvalKapacityAgentem($database, $uroven, $rezimUprav, $hodnota)
    {
        if (!RonPortalSettings::getSpousteniADS()) {
            throw new Exception('Chyba ve funkci SchvaleniKapacitDBRecord::schvalKapacityAgentem - neni povoleno spousteni agentu!');
        }

        $parametrskriptu = $this ->getParametrSkriptu($uroven, $rezimUprav, $hodnota);

        AgentManager::executeScriptEvents($database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_BEFORE, $parametrskriptu);

        // prepocet kontrol planovani
        AgentManager::executeADS(
            'ADS_Agent_CapacityPlanning.exe', $this ->getAgentParameters($uroven, $rezimUprav, $hodnota));
    }


    /**
     * @param int  $uroven
     * @param bool $rezimUprav
     * @param int  $hodnota
     * @throws Exception
     */
    public function zapisSchvaleni($uroven, $rezimUprav, $hodnota)
    {
        // nacte existujici zaznam
        $exists = $this ->loadRecord();
        $hodnotaOriginal = 0;
        $xmlContext = null;

        switch ($rezimUprav ? $uroven + 10 : $uroven) {
            case 1 :
                $this ->fSchvaleni1 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni1 ->originalValue;
                $this ->fSchvalil1 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni1 ->setNewValue( time() );
                $xmlContext = 'schvaleni1';
                break;

            case 2 :
                $this ->fSchvaleni2 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni2 ->originalValue;
                $this ->fSchvalil2 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni2 ->setNewValue( time() );
                $xmlContext = 'schvaleni2';
                break;

            case 3 :
                $this ->fSchvaleni3 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleni3 ->originalValue;
                $this ->fSchvalil3 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->fCas_Schvaleni3 ->setNewValue( time() );
                $xmlContext = 'schvaleni3';
                break;

            case 11 :
                $this ->fSchvaleniUprav1 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav1 ->originalValue;
                $this ->UpravySchvalil1 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav1 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav1';
                break;

            case 12 :
                $this ->fSchvaleniUprav2 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav2 ->originalValue;
                $this ->UpravySchvalil2 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav2 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav2';
                break;

            case 13 :
                $this ->fSchvaleniUprav3 ->setNewValue($hodnota);
                $hodnotaOriginal = $this ->fSchvaleniUprav3 ->originalValue;
                $this ->UpravySchvalil3 ->setNewValue( RonPortalUtils::getLogUserName() );
                $this ->Cas_SchvaleniUprav3 ->setNewValue( time() );
                $xmlContext = 'schvaleniuprav3';
                break;
        }

        // zkontroluje prava
        $this ->zkontrolujPravoNaSchvaleni($uroven, $hodnota, $hodnotaOriginal);

        if (!$this ->isChanged()) {
            // nedoslo ke zmene
            return;
        }

        $parametrskriptu = $this ->getParametrSkriptu($uroven, $rezimUprav, $hodnota);

        $this ->executeXmlEvents(RonPortalXMLEvents::KAPACITY_SCHVALENI_BEFORE, $xmlContext);
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_BEFORE, $parametrskriptu);

        $pouzitAgenta = (KapacitySettings::pouzivatKontrolyPlanovani()  &&  $hodnota); // agent (zatim) neumi "odschvalit" //  X nepodporuje rezim uprav - uz neplati X
//        if (!$pouzitAgenta) {
//            // schvaleni celeho rozdeleni se vzdy provede agentem
//            $pouzitAgenta = !!RonPortalSettings::getIntValue(KapacityUtils::SETTINGS_ZPUSOB_SCHVALOVANI);
//        }

        if ($pouzitAgenta) {
            $this ->schvalKapacityAgentem($this ->database, $uroven, $rezimUprav, $hodnota);
        } else {
            if ($hodnota) {
                $this->overKontroly($uroven, $hodnota);
            }

            if ($exists) {
                $this->updateRecord();
            } else {
                $this->insertRecord();
            }
        }


        $this ->executeXmlEvents(RonPortalXMLEvents::KAPACITY_SCHVALENI_AFTER, $xmlContext);
        AgentManager::executeScriptEvents($this ->database, RonPortalScriptEvents::CATEGORY_KAPACITY, RonPortalScriptEvents::KAPACITY_SCHVALENI_AFTER, $parametrskriptu);
    }
}
