<?php
/**
 * User: pavel.rudol
 * Date: 29.4.2016
 * Time: 15:43
 */

namespace RonPortal\Kapacity;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\Controls\UserLoginFormField;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;

class ZastupyFiltrRozdeleniField extends FormField
{
    var $typrozdeleni;

    private $vybraneRozdeleni;
    private $url;

    /**
     * ZastupyFiltrRozdeleniField constructor.
     * @param int $typRozdeleni
     * @param string $fieldname Nazev pole pro html a v databazi
     * @param string $captionText Kod retezce v ini
     * @param string $vybraneRozdeleni
     * @param string $url
     */
    function __construct($typRozdeleni, $fieldname, $captionText, $vybraneRozdeleni, $url)
    {
        parent::__construct();
        $this->typrozdeleni = $typRozdeleni;
        $this->vybraneRozdeleni = $vybraneRozdeleni;
        $this->url = $url;

        $this->fieldType = "rozdeleni";
        $this->fieldName = $fieldname;
        //$this->sqlFieldName = $fieldname;
        $this->maxLength = 30;

        $this->caption = TextManager::_($captionText);
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {

            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass '.
                ' FROM Rozdeleni'.
                ' WHERE Rozdeleni.KodTypuRozdeleni = '.(int)$this->typrozdeleni.
                ' AND (Rozdeleni.DatumZruseni >= '.$database->timestampToString(time()).' OR Rozdeleni.DatumZruseni IS NULL)' ;

            $sql .= " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            MessageManager::addDebugLog('Combo filtr rozdeleni: ' . $sql);

            echo '<div class="row">';

            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', $this->vybraneRozdeleni);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col-4 changeHref form-control';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' data-ron-select2='" . $this->fieldName . "_nazev' data-ron-href='" . $this->url . "' data-ron-working-info='1'";
            $combo ->render();

            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->cssClassIndex = 2;
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', $this->vybraneRozdeleni);
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col-8 changeHref form-control';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' data-ron-select2='$this->fieldName' data-ron-href='" . $this->url . "' data-ron-working-info='1'";
            $combo ->render();

            echo '</div>';

        }
    }
}

class ZastupyVyberKategorieFiled extends FormField
{
    private $typKategorie;

    /**
     * ZastupyVyberKategorieFiled constructor.
     * @param string $typKategorie
     */
    public function __construct($typKategorie)
    {
        parent::__construct();

        $this->typKategorie = $typKategorie;

        $this->fieldType = FormField::FIELD_TYPE_SELECT;
        $this->maxLength = 30;
    }

    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     * @throws DBException
     */
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($canEdit) {

            $sql =
                "SELECT DISTINCT Kategorie.Kod, Kategorie.Popis ".
                " FROM Kategorie".
                " WHERE Kategorie.TypKategorie = '" . $database->addSlashes($this->typKategorie) . "'";


            MessageManager::addDebugLog('Combo vyber kategorie: ' . $sql);

            $combo = new ComboBox($this->fieldName);
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->captionIndex = 0;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Kod', '');
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col1';
            $combo ->htmlParams = "id='{$this->fieldName}_kod' data-ron-select2='" . $this->fieldName . "_nazev'";
            $combo ->render();

            $combo = new ComboBox($this->fieldName.'_nazev');
            $combo ->alwaysShowSelected = isset($value) && $value != '';
            $combo ->captionIndex = 1;
            $combo ->keyIndex = 0;
            $combo ->loadData($database, $sql.' ORDER BY Popis', '');
            $combo ->emptyRow = (isset($value) && $value != ''  &&  $this->required) ? null : ' ';
            $combo ->className .= ' col23';
            $combo ->htmlParams = "id='{$this->fieldName}_popis' data-ron-select2='$this->fieldName'";
            $combo ->render();

        } else {
            $popisKategorie = DBUtils::getSqlValue($database,
                "SELECT Popis FROM Kategorie WHERE Kod = '" . $database->addSlashes($rowData[$this->sqlFieldIndex]) . "'" .
                " AND TypKategorie = '" . $database->addSlashes($this->typKategorie) . "'");

            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . $rowData[$this->sqlFieldIndex] . '" />';
            echo StringUtils::htmlTextConvert($popisKategorie) . "\n";
        }
    }
}


class ZastupyForm extends RonPortalForm
{
    /** @var PravaNaOsoby */
    private $prava;
    /** @var  PravaNaOsoby */
    private $pravaSchvalit;
    /** @var PravaNaOsoby */
    private $pravaPriraditZamestnance;

    private $prirazeniZastupu = false;

    private $zastup;

    public $idZaznamu;

    /**
     * ZastupyForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param bool $prirazeniZastupu
     * @param Zastup $zastup
     * @param string $rozdeleniFiltr
     * @throws Exception
     */
    public function __construct($database, $formAction, $prirazeniZastupu = false, $zastup = null, $rozdeleniFiltr = '')
    {
        parent::__construct($database, $formAction);

        $this->prava = Prava::vratPrava('zastupy_vlozeni');
        $this->pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $this->pravaPriraditZamestnance = Prava::vratPrava('zastupy_prideleni');

        $this->prirazeniZastupu = $prirazeniZastupu;
        $this->zastup = $zastup;

        $this ->htmlAction = '?menuid=zastup' . ($prirazeniZastupu ?
                '&action=priradit'  // . (isset($_GET{'record'}) ? '&formaction=' . $this->formAction . '&record=' . $_GET['record'] : '')
                : '');
        $this ->stornoAction = '?menuid=zastup';

        //$this->formCode = 'ZadostOZastup'; - zbytecne - slouzi pro rozliseni vice formularu na strance, ale tu je jen jeden
        $this ->code = 'formzastupy';

        $this ->title = $this->formAction == 'new'
            ? TextManager::_('Zastup_novy')
            : TextManager::_($prirazeniZastupu ? 'Zastup_prirazeni' : 'Zastup_editace');

        $this ->sqlTableName = 'ZadostOZastup';

        $this->sqlSelectPart =
            'ID, Datum, CasOd, CasDo, KodTypuRozdeleni, Rozdeleni, KodTypuRozdeleni2, Rozdeleni2, ' . // 0 - 7
            'RC, OsobniCislo, DatumNastupu, Schvaleni, Schvalil, Poznamka, Insert_UserName, Insert_Timestamp, Update_UserName, Update_Timestamp, ' .  // 8 - 17
            'DuvodZadosti, KodKategorie'; //18, 19

        $this->defaultRowClass = 'row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('ID') ->setPrimary(true) ->setSqlFieldIndex(0)
            ->setCode('field-id');


        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('Datum') ->setSqlFieldIndex(1) ->setRequired(true) ->setReadOnly(true)
            ->setCode('field-datum')
            ->setCaption('Zastup_datum_od')
            ->setDefaultValue(StringUtils::format('date',DateUtils::startOfTheDay(time())))
            ->setOtherHtmlParams(['data-roncalendar-next' => 'Datum_do']);

        if ($this->isInsert()) {
            $this ->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('*Datum_do') ->setDefaultValue(StringUtils::format('date', DateUtils::startOfTheDay(time())))
                ->setCode('field-datumdo')
                ->setCaption('Zastup_datum_do');
        }

        $this->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('CasOd')->setCaption('Zastup_cas_od')->setReadOnly($prirazeniZastupu)
            ->setDataColumnClass('col-12 col-sm-1')
            ->setFieldCssClass('inputTime')
            ->setOtherHtmlParams(['size' => 5, 'maxlength' => 5])
            ->setCode('field-casod');
        $this->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('CasDo')->setCaption('Do')->setReadOnly($prirazeniZastupu)
            ->setNewRow(false)
            ->setCaptionColumnClass('caption')
            ->setDataColumnClass('col-12 col-sm-1')
            ->setFieldCssClass('inputTime')
            ->setOtherHtmlParams(['size' => 5, 'maxlength' => 5])
            ->setCode('field-casdo');

        //rozdeleni na ktere se zada zastup - dle nastaveni planovani kapacit
        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('KodTypuRozdeleni')
            ->setCode('field-kodtypurozdeleni')
            ->setDefaultValue(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));

        ///$rozdeleni = new ZastupyVyberRozdeleniField(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI), $this->prava);
        $rozdeleni = new RozdeleniFormField(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));
        $rozdeleni ->prava = $this ->prava;

        $this->addField($rozdeleni)
            ->setFieldName('Rozdeleni') ->setSqlFieldIndex(5) ->setRequired(true) ->setReadOnly($prirazeniZastupu)
            ->setCode('field-rozdeleni')
            ->setCaption('Zastup_rozdeleni');

        //pokud neni zadana kategorie rozdeleni, pro vyber pozice se pouzije nastaveny typ rozdeleni
        if ((RonPortalSettings::getValue(KapacitySettings::TYP_KATEGORIE_POZICE) == '') ||
            //nebo se jedna o prirazeni zastupu a zastup byl pozadovan na pozici dle rozdeleni (v nastaveni jiz je ale pozadovana kategorie)
            ($prirazeniZastupu && $zastup->pozice)) {
            //pozice na kterou se zada zastup - dle nastaveni pro zastupy
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('KodTypuRozdeleni2')
                ->setCode('field-kodtypurozdeleni2')
                ->setDefaultValue(RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE,-1));

//            $rozdeleni2 = new ZastupyVyberRozdeleniField(
//                RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1),
//                null,
//                RonPortalSettings::getValue(KapacitySettings::POZICE_VYCET));
            $rozdeleni2 = new RozdeleniFormField(RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1));
            $rozdeleni2 ->setVycetRozdeleni(RonPortalSettings::getValue(KapacitySettings::POZICE_VYCET));

            $this->addField($rozdeleni2)
                ->setFieldName('Rozdeleni2') ->setSqlFieldIndex(7) ->setRequired(false) ->setReadOnly($prirazeniZastupu)
                ->setCode('field-rozdeleni2')
                ->setCaption('Zastup_pozice');
        }
        //v nastaveni byl zvolen typ kategorie pro vyber pozice, zobrazi se vyber kategorii
        else {
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('TypKategorie')
                ->setCode('field-typkategorie')
                ->setDefaultValue(RonPortalSettings::getValue(KapacitySettings::TYP_KATEGORIE_POZICE));

            $kategorie = new ZastupyVyberKategorieFiled(
                RonPortalSettings::getValue(KapacitySettings::TYP_KATEGORIE_POZICE));
            $this->addField($kategorie)
                ->setFieldName('KodKategorie') -> setSqlFieldIndex(19) ->setReadOnly($prirazeniZastupu)
                ->setCaption('Zastup_kategorie_pozice');
        }

        if ($prirazeniZastupu) {
            //pokud je zadan typ rozdeleni pro filtrovani osob, zobrazi vyber rozdeleni
            if (RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_FILTR) != 0) {
                $comboRozdeleni = new ZastupyFiltrRozdeleniField(RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_FILTR), 'RozdeleniFiltr', 'Zastup_rozdeleni_filtr',
                    $rozdeleniFiltr, '?menuid=zastup&action=priradit&formaction=edit&record=' . $zastup->id . '&rozdeleni=');
                $this->addField($comboRozdeleni);
            }

            $jePreferovanaOsoba = false;

            //pokud neni zadana kategorie rozdeleni, pro vyber pozice se pouzije nastaveny typ rozdeleni
            if ((RonPortalSettings::getValue(KapacitySettings::TYP_KATEGORIE_POZICE) == '') ||
                //nebo se jedna o prirazeni zastupu a zastup byl pozadovan na pozici dle rozdeleni (v nastaveni jiz je ale pozadovana kategorie)
                ($prirazeniZastupu && $zastup->pozice)) {
                $filtrPozice =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_POZICE, -1) .
                    "          AND RozdeleniOsob.Rozdeleni = '" . $database->addSlashes($zastup->pozice) . "'" .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }
            //v nastaveni byl zvolen typ kategorie pro vyber pozice, zobrazi se vyber kategorii
            else {
                $filtrPozice =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        JOIN KategorieRozdeleni ON (KategorieRozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND KategorieRozdeleni.Rozdeleni = RozdeleniOsob.Rozdeleni)' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .

                    "          AND KategorieRozdeleni.TypKategorie = '" . $this ->database ->addSlashes(RonPortalSettings::getValue(KapacitySettings::TYP_KATEGORIE_POZICE)) . "'" .
                    "          AND KategorieRozdeleni.KodKategorie = '" . $database->addSlashes($zastup->kategoriePozice) . "'" .
                    ')';
            }

            $filtrRozdeleni = '';
            //filtrovani podle vybraneho rozdeleni
            if (RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_FILTR) != 0) {
                $filtrRozdeleni =
                    ' EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '        WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '          AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '          AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '          AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(KapacitySettings::TYP_ROZDELENI_FILTR) .
                    "          AND RozdeleniOsob.Rozdeleni = '" . $database->addSlashes($rozdeleniFiltr) . "'" .
                    '          AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '          AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')';
            }

            if (RonPortalSettings::getIntValue(KapacitySettings::PREFEROVANY_TYP_ROZDELENI) != 0) {
                $osoba1 = new OsobaFormField('zastupy_prideleni', time(), true);
                $osoba1
                    ->setFieldName('PreferovanaOsoba')->setSqlFieldName(null)
                    ->setCode('field-preferovanaosoba')
                    ->setCaption('Zastup_preferovana_osoba') ->setCaption2('Zastup_zapsat_preferovanou_osobu')
                    ->setRowId('zastup_osoba1');

                $filtr =
                    //21.7.2016 je mozne priradit i zamestnance, ktery jiz ma naplanovanou smenu, jine rozdeleni, pri vkladani se pak zobrazi hlaska ze je zaznam prepsan
                    /*'NOT EXISTS(SELECT * FROM PlanovaniKapacit ' .
                    '           WHERE PlanovaniKapacit.RC = PracovniPomer.RC ' .
                    '             AND PlanovaniKapacit.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '             AND PlanovaniKapacit.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '             AND PlanovaniKapacit.Datum = ' . $this->database->timestampToString($zastup->datum) .
                    ' )'.
                    'AND ' .*/
                    '    EXISTS(SELECT * FROM RozdeleniOsob ' .
                    '           WHERE RozdeleniOsob.RC = PracovniPomer.RC ' .
                    '             AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo ' .
                    '             AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu ' .
                    '             AND RozdeleniOsob.KodTypuRozdeleni = ' . (int)RonPortalSettings::getIntValue(KapacitySettings::PREFEROVANY_TYP_ROZDELENI) .
                    "             AND RozdeleniOsob.Rozdeleni = '" . $this->database->addSlashes(RonPortalSettings::getValue(KapacitySettings::PREFEROVANE_ROZDELENI)) . "'" .
                    '             AND RozdeleniOsob.PlatnostOd <= ' . $this->database->timestampToString($zastup->datum) .
                    '             AND (RozdeleniOsob.PlatnostDo >= ' . $this->database->timestampToString($zastup->datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
                    ')' ;



                $osoba1->filtr = $filtr . ' AND ' . $filtrPozice . ($filtrRozdeleni ? ' AND ' . $filtrRozdeleni : '');

                $jePreferovanaOsoba = DBUtils::getSqlValue($this->database,
                    'SELECT Osoba.Prijmeni FROM Osoba JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC)' .
                    ' WHERE ' . $osoba1->filtr);

                if ($jePreferovanaOsoba) { // prida pole jen pokud nejaka osoba existuje
                    $this->addField($osoba1);
                }
            }

            $osoba = new OsobaFormField('zastupy_prideleni', time());
            $osoba ->setCaption('Zastup_osoba');

            $osoba->setRequired(!$jePreferovanaOsoba);

            if ($jePreferovanaOsoba) {//neni vybran preferovany typ, zobrazi se vzdy
                $osoba->setRowId('zastup_osoba2'); //defaultne skryto, pokud se v predchozim vybere prazdny zaznam, je mozno vybirat zde
                $osoba->setRowClass('form-row d-none');
            }



            $filtr = '';

            $osoba->filtr = $filtr . $filtrPozice. ($filtrRozdeleni ? ' AND ' . $filtrRozdeleni : '');
            MessageManager::addDebugLog($osoba->filtr);
            $this->addField($osoba);

            $this->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-link-kalendar')
                ->setText(
                    '<a id="zastup_kalendar_link" href="#" target="_blank" data-ronlink="?menuid=prehledsmen&obdobi=' . DateUtils::dateToUrlString(DateUtils::firstDayOfMonth($zastup->datum)) .
                    '&osoba=">'. TextManager::_('Zastup_zobraz_kalendar').'</a>');
        }

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DuvodZadosti') ->setSqlFieldIndex(18) ->setReadOnly($prirazeniZastupu)
            ->setCode('field-duvodzadosti')
            ->setSelectSql('SELECT Kod, Popis FROM DuvodZastupu WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setCaption('Zastup_duvod');

        $this->addField(FormField::FIELD_TYPE_EDIT)
            ->setFieldName('Poznamka') ->setSqlFieldIndex(13)
            ->setCaption('Zastup_poznamka');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp') ->setReadOnly(false)
            ->setCode('field-update_timestamp');
        $this ->addField(new UserLoginFormField(false));

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp') ->setReadOnly(true)
            ->setCode('field-insert_timestamp');
        $this ->addField(new UserLoginFormField(true));
    }


    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterInsertRecord($rec)
    {
        return $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_AFTER_POST, 'insert', null, false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
        $this->idZaznamu = $rec['ID'];

        //ve formulari se prirazuje zastup, zapise se do planovani kapacit
        if ($this->prirazeniZastupu && $rec['RC']) {
            try {
                ZastupyUtils::vlozPlanovaniKapacit($this->database, $rec);
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }
        
        return $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_AFTER_POST, 'update', null, false); 
    }
    
    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     */
    function beforeInsertRecord($rec)
    {
        $result = $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_BEFORE_POST, 'insert', null, false);

        if ($this->pravaSchvalit->jePravoNaRozdeleni($rec['KodTypuRozdeleni'],$rec['Rozdeleni'],$rec['Datum'],$rec['Datum'])) {
            $rec->addIntField('TypSchvaleni', true, 1);
            $rec->addTimestampField('Schvaleni', true, time());
            $rec->addStringField('Schvalil', true, 25, RonPortalUtils::getLogUserName());
        }

        return $result;
    }




    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeUpdateRecord($rec)
    {
        if (!parent::beforeUpdateRecord($rec)) {
            return false;
        }

        return $rec->executeXmlEvents(RonPortalXMLEvents::ZASTUP_BEFORE_POST, 'update', null, false);
    }



    function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);

        if (!$this ->isInsert()) {
            // prirazeni osoby pro zastup
            $osobaID = null;

            if (isset($_POST['PreferovanaOsoba'])  &&  $_POST['PreferovanaOsoba']) {
                $osobaID = $_POST['PreferovanaOsoba'];
            } else if (isset($_POST['osoba'])  &&  $_POST['osoba']) {
                $osobaID = $_POST['osoba'];
            } else if (isset($_POST['PreferovanaOsoba']) || isset($_POST['osoba'])) {
                //neni vybrana zadna osoba a formular je v rezimu prirazeni osoby, vypise se chyba
                throw new Exception(TextManager::_('Zastup_nevybrana_osoba'));
            }

            if ($osobaID) {
                if (!$this->pravaPriraditZamestnance->lzeZobrazitPP($osobaID)) {
                    throw new NeniPravoException();
                }

                $pracovniPomer = PracovniPomer::nactiPP($this->database, $osobaID);

                $rec->addField('RC', '', false, 15, $pracovniPomer->rc);
                $rec->addField('OsobniCislo', '', false, 15, $pracovniPomer->osobniCislo);
                $rec->addField('DatumNastupu', 'timestamp', false, 0, $pracovniPomer->datumNastupu);
            }
        }
    }


    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $viceDnu = false;
        $finished = false;
        /** @var int $date */
        $date = null;
        $datumDo = (isset($_POST['Datum_do']) && ($_POST['Datum_do'] != ''))
            ? DateUtils::dateStringToTimestamp($_POST['Datum_do'])
            : null;

        $idZaznamu = null;

        // vlozi samostatny zaznam pro kazdy den v zadanem intervalu datumu
        while (!$finished) {
            $rec = $this->createDBRecord();
            $this->createDBRecordFields($rec, true);

            if (!$date) {
                $date = $rec['Datum'];
            } else {
                $viceDnu = true;
                $rec['Datum'] = $date;
            }

            // kontrola prava
            if (!$this ->prava ->jePravoNaRozdeleni(
                RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI),
                $rec['Rozdeleni'], $date)) {
                throw new NeniPravoException();
            }

            if (!$this->beforeInsertRecord($rec)) {
                return false;
            }

            $rec->insertRecord();

            if (!$this->afterInsertRecord($rec)) {
                return false;
            }

            $idZaznamu = $rec['Id'];

            // prejde na dalsi datum
            $date = DateUtils::incDay($date, 1);
            if (!$datumDo  ||  $date > $datumDo) {
                $finished = true;
            }
        }

        if (!$viceDnu) {
            $this ->idZaznamu = $idZaznamu;
        }

        return true;
    }


    function renderCommonFields()
    {
        //datum pro pouziti v javascriptu
        if ($this->prirazeniZastupu) {
            echo
                "<input type='hidden' name='DatumZastupu' value='" . $this->zastup->datum . "'>";
        }

        parent::renderCommonFields();
    }

}
