<?php
/**
 * stanislav.elbl, 10.06.2021 23:36
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\Smena;
use RonPortal\Log;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;

class ZastupyUtils
{
    /**
     * Vytvori zaznam v planovani kapacit podle zadosti o zastup. Pokud zaznam pro dany den jiz existuje vypise upozorneni.
     * @param Database $database
     * @param RonPortalDBRecord $rec
     * @throws DBException
     * @throws \Exception
     */
    public static function vlozPlanovaniKapacit($database, $rec)
    {
        $planKapacitRec = new RonPortalDBRecord($database,'PlanovaniKapacit', $rec ->logDescription);
        $planKapacitRec->logText = StringUtils::fromUTF8("Přepsáno plánování kapacit pro zaměstnance: %s\n");

        $planKapacitRec->addField('RC', 'string', true, 15, $rec['RC']);
        $planKapacitRec->addField('OsobniCislo', 'string', true, 15, $rec['OsobniCislo']);
        $planKapacitRec->addField('DatumNastupu', 'timestamp', true, 0, $rec['DatumNastupu']);
        $planKapacitRec->addField('Datum', 'timestamp', true, 0, $rec['Datum']);

        $planKapacitRec->addField('Smena', 'string', false, 10, RonPortalSettings::getValue(KapacitySettings::SMENA_PRO_PLANOVANI));
        $planKapacitRec->addField('Smena_Pocatek', 'time', false, 0, $rec['CasOd']);
        $planKapacitRec->addField('Smena_Konec', 'time', false, 0, $rec['CasDo']);
        //7.10.2016 zjistovani delky smeny podle nastaveni smeny
        $planKapacitRec->addField('Smena_Delka', 'time', false, 0,
            Smena::vratDelkuSmeny($database, $rec['CasOd'], $rec['CasDo'], RonPortalSettings::getValue(KapacitySettings::SMENA_PRO_PLANOVANI)));
        $planKapacitRec->addField('KodTypuRozdeleni', 'int', false, 0, $rec['KodTypuRozdeleni']);
        $planKapacitRec->addField('Rozdeleni', 'string', false, 0, $rec['Rozdeleni']);

        $userLogin = new DBRecordField('Update_UserName', 'string', false, 25);
        $userLogin->updateLog = false;
        $userLogin->setNewValue(RonPortalUtils::getLogUserName());
        $planKapacitRec->addDBRecordField($userLogin);

        $updateTimestamp = new DBRecordField('Update_TimeStamp', 'timestamp');
        $updateTimestamp->updateLog = false;
        $updateTimestamp->setNewValue(time());
        $planKapacitRec->addDBRecordField($updateTimestamp);

        $params = $database ->createParameters();
        $planKapacitRec->setWhere(
            ' RC = ' . $params->addParameter($rec['RC']) .
            ' AND OsobniCislo = ' . $params->addParameter($rec['OsobniCislo']) .
            ' AND DatumNastupu = ' . $params ->addParameter($database->dateToParam($rec['DatumNastupu'])) .
            ' AND Datum = ' . $params ->addParameter($database->timestampToParam($rec['Datum'])),
            $params
        );

        if ($planKapacitRec->loadRecord()) {
            //21.7.2016 je mozne priradit i zamestnance, ktery jiz ma naplanovanou smenu, jine rozdeleni, pri vkladani se pak zobrazi hlaska ze je zaznam prepsan

            $pracPomer = new PracovniPomer();
            $pracPomer->nactiPPPodleRC($database, $rec['RC'], $rec['OsobniCislo'], $rec['DatumNastupu']);

            MessageManager::addInfoMessage(sprintf(TextManager::_('Zastup_prepsani_planu'), $pracPomer->jmenoPrijmeniKratke, StringUtils::format('date', $rec['Datum'])));

            if ($planKapacitRec->updateRecord() > 0) {
                Log::addRecordToLog($database, StringUtils::fromUTF8('Přepsání plánu kapacit zaměstnance: ') . $pracPomer->jmenoPrijmeniKratke, $rec->logDescription,
                    sprintf(StringUtils::fromUTF8(
                        "Přepsáno plánování kapacit pro zaměstnance: %s\n" .
                        "RČ: %s\n" .
                        "Osobní číslo: %s\n" .
                        "Datum nástupu: %s\n" .
                        "Datum: %s\n" .
                        "Původní směna: %s\n" .
                        "Původní začátek směny: %s\n" .
                        "Původní konec směny: %s\n" .
                        "Původní délka směny: %s\n" .
                        "Původní kód typu rozdělení: %s\n" .
                        "Původní rozdělení: %s"),
                        $pracPomer->jmenoPrijmeni, $pracPomer->rc, $pracPomer->osobniCislo, StringUtils::format('date', $pracPomer->datumNastupu),
                        StringUtils::format('date', $planKapacitRec['Datum']), $planKapacitRec->findFieldByName('Smena')->originalValue,
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Pocatek')->originalValue),
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Konec')->originalValue),
                        StringUtils::format('time', $planKapacitRec->findFieldByName('Smena_Delka')->originalValue),
                        $planKapacitRec->findFieldByName('KodTypuRozdeleni')->originalValue, $planKapacitRec->findFieldByName('Rozdeleni')->originalValue
                    )
                );
                return;
            }

            //throw new \Exception('Pro vybraný den má již zaměstnanec přiřazen směnu!');
        } else {
            $planKapacitRec->insertRecord();
        }
    }


    /** Smaže záznam v plánování kapacit pro zvolenou osobu a den.
     * Vraci true, pokud byl smazan, false pokud nebyl nalezen a v pripade chyby vyhodi vyjimku
     * @param Database $database
     * @param int $idPP
     * @param int $datum
     * @param string $logDescription Ve výchozím stavu informace o zrušení žádosti
     * @return bool
     * @throws DBException
     */
    public static function zrusPlanovaniKapacit($database, $idPP, $datum, $logDescription = 'Zrušení žádosti o zástup')
    {
        $pracPomer = PracovniPomer::nactiPP($database, $idPP);

        $planKapacitRec = new RonPortalDBRecord($database,'PlanovaniKapacit', $logDescription);

        $planKapacitRec->addField('RC', '', true);
        $planKapacitRec->addField('OsobniCislo', '', true);
        $planKapacitRec->addField('DatumNastupu', 'timestamp', true);
        $planKapacitRec->addField('Datum', 'timestamp', true);
        $planKapacitRec->addField('Smena', '', true);
        $planKapacitRec->addField('Smena_Pocatek', 'time', true);
        $planKapacitRec->addField('Smena_Konec', 'time', true);
        $planKapacitRec->addField('Smena_Delka', 'time', true);
        $planKapacitRec->addField('KodTypuRozdeleni', '', true);
        $planKapacitRec->addField('Rozdeleni', '', true);

        $params = $database ->createParameters();
        $planKapacitRec->setWhere(
            ' RC = ' . $params->addParameter($pracPomer->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pracPomer->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database->dateToParam($pracPomer->datumNastupu)) .
            ' AND Datum = ' . $params ->addParameter($database->timestampToParam($datum)),
            $params
        );

        return $planKapacitRec->deleteRecord();
    }

}
