<?php
/**
 * pavel.rudol, 19.12.2024 15:01
 */

namespace RonPortal\Mzdy;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;

class DavkyNZController extends JsonController
{
    private Database $database;

    private $pravaProhlizeni;
    private $pravaZapis;

    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        $this->pravaProhlizeni = Prava::vratPrava('davkynz_prohl');
        $this->pravaZapis = Prava::vratPrava('davkynz_zapis');
        Prava::nastavObdobiProPrava(DateUtils::today());
    }

    public function browserDavky()
    {
        $b = new DavkyNZBrowser($this->database);
        $b->load();
        return $b;
    }

    public function formDavky()
    {
        $f = new DavkyNZForm($this->database);

        if ($f->handleAction()) {
            if ($f->isInsert()) {
                return new Redirect(null, '?menuid=davkynz&formaction=edit&record=' . $f->id);
            }
            return new Redirect(null, '?menuid=davkynz');
        }

        return $f;
    }

    /**
     * @param JsonOutputObject $output
     * @return void
     * @throws NeniPravoException
     * @throws \Ron\DB\DBException
     */
    private function getFamilyMembers($output)
    {
        $pracPomer = PracovniPomer::nactiPP($this->database);
        $rc = $pracPomer->rc;

        if (!($this->pravaZapis->lzeZobrazitRC($rc)) ) {
            throw new NeniPravoException();
        }

        $cursor = $this->database->select(DavkyNZForm::getSQLRodinnyPrislusnik($this->database, $rc, false, false));

        $members = array();

        $member['kod'] = '';
        $member['popis'] = '';

        $members[] = $member;

        while ($row = $cursor->fetch()) {
            $member['kod'] = StringUtils::jsonText($row[0]);
            $member['popis'] = StringUtils::jsonText($row[1]);

            $members[] = $member;
        }

        $member['kod'] = StringUtils::jsonText('NEW');
        $member['popis'] = StringUtils::jsonText(TextManager::_('DavkyNZ_PridatPrislunika'));

        $members[] = $member;

        $output->addData('members', $members);
    }

    /**
     * @inheritDoc
     */
    function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'clenoverodiny' :
                $this->getFamilyMembers($output);
                break;
        }
    }

    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('mzdy.ini');
        TextManager::getInstance()->loadStrings('person.ini');
        HtmlHeaders::getInstance()->addScript('script/mzdy.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=davkynz&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);
        HtmlHeaders::getInstance()->setAppTitle('DavkyNZ_oznameni');

        if (!($this->pravaProhlizeni->jsouNejakePrava() || $this->pravaZapis->jsouNejakePrava()) ) {
            throw new NeniPravoException();
        }

        if (isset($_GET['formaction']) || isset($_POST['formaction'])) {
            return $this->formDavky();
        }

        //json - seznam clenu rodiny
        if (isset($_GET['action']) && $_GET['action'] == 'clenoverodiny' ) {
            return parent::handleAction($_GET['action']);
        }

        return $this->browserDavky();
    }
}