<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Mzdy;

use Ron\DB\Database;
use Ron\DB\DBException;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class MzdyUtils
{

    /**
     * Vraci kod typu osoby pro zadany pracovni pomer
     * @param Database $database
     * @param PracovniPomer $pp
     * @return int
     * @throws DBException
     */
    public static function getTypOsoby($database, $pp)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Zamestnanec FROM PracovniPomer ' .
            ' WHERE RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu));

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) {
            return (int)$row[0];
        }

        return false;
    }

    /**
     * Zkontroluje, jestli pro zadanou osobu a zamestnance existuje zaznam mzdy
     * Pokud ne, tak jej vytvori a vraci true
     * Pokud ano, tak zkontroluje, jestli uz neni mzda uzamcena - v tom pripade vraci false
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $obdobi
     * @param object|string $dbLogDescription
     * @return bool
     * @throws DBException
     */
    public static function zkontrolujZaznamMzdy($database, $pp, $obdobi, $dbLogDescription)
    {
        $rec = new RonPortalDBRecord($database, 'Mzda', $dbLogDescription);

        $rec->addField('RC', '', true, 15, $pp ->rc); // readonly
        $rec->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $rec->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);
        $rec->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $rec->addField('Obdobi', 'date', true, 0, $obdobi);

        $rec->addField('VytvorenoVDochazce', 'int', true, 0, 1);
        $field_vemzde = $rec->addField('VytvorenoVeMzde', 'int', true, 0, 0);
        $field_schvaleno = $rec->addField('Schvaleno', 'int', true, 0, 0);
        $rec->addField('NemocenskyPojisten', 'int', true, 0, 1);
        $rec->addField('UrcenoPojisteniRucne', 'int', true, 0, 0);
        $rec->addField('DoplatekDoMinZakladu', 'int', true, 0, 0);
        $rec->addField('PojistneZaPracovniVolno', 'int', true, 0, 2);
        $rec->addField('PojistneZaOmlAbsenci', 'int', true, 0, 0);
        $rec->addField('PojistneZaNeomlAbsenci', 'int', true, 0, 0);

        $field_aktualizovat = $rec->addField('AktualizovatVypocet', 'int', false, 0, 0); // readonly = false

        $params = $database ->createParameters();
        $rec->setWhere(
            ' RC = ' . $params->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database->dateToParam($pp ->datumNastupu)) .
            ' AND Obdobi = ' . $params ->addParameter($database ->dateToParam($obdobi)),
            $params);

        if ($rec->loadRecord()) {
            // zaznam nalezen - provede Update
            if ($field_schvaleno->value && $field_vemzde->value) {
                return false;
            }

            if (self::getTypOsoby($database, $pp) === 1) {
                $field_aktualizovat->value = 1;

                return $rec->updateRecord();
            }

            return true;
        }

        // provede insert
        $rec->insertRecord();
        return true;
    }

}
