<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 12.02.2019
 * Pavel.Rudol RON Software 01/2023
 */
namespace RonPortal\Mzdy;

use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controller\DownloadController;
use RonPortal\Controls\FormReportsListView;
use RonPortal\RonPortalSettings;

$model = Model::getInstance();

/** @var VyuctovaniDaneForm $rocniVyuctovaniForm */
$rocniVyuctovaniForm = $model->getData('rocnivyuctovaniform');
$rok = (int)date("Y", $rocniVyuctovaniForm['rok']);
$disabled = $rocniVyuctovaniForm->lzeEditovatVyuctovani() ? "" : "disabled";

/** @var NastaveniOdpoctu $nastaveniOdpoctu */
$nastaveniOdpoctu = $model->getData('nastaveniOdpoctu');
$reportlist = $model->getData('reportlist');
$jednoduchyVzhled = $model->getData('jednoduchyVzhled');

if ($jednoduchyVzhled) {
    $classStr = 'class="dane_info_skryte"';
} else {
    $classStr = 'class="padding-bottom-2"';
}

?>

<?php MessageManager::getInstance() ->showMessages(); ?>

<?php if ($rocniVyuctovaniForm['ZadostPortalStav'] == RocniVyuctovaniDaneRecord::ZADOST_VICE_PP)  { ?>

    <div class="form">
        <div class="form-content">
            <?=   TextManager::_('formular_nelze_vyplnit') ?>
        </div>
        <div class="buttons">
            <input type="button" class="button clickHref" data-ron-href="?menuid=dane&action=zadost" value="<?= TextManager::_('zpet')?>">
        </div>
    </div>

<?php } else { ?>

    <?php
        if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
            echo
            "<div id='novydokument' title='" . TextManager::_('Vlozit_novy_dokument') . "' class='rondialog'>
            <div>
                <div class='row'>
                    <div class='col col-12 col-sm-4 caption'>" . TextManager::_('Nahrat_soubor') . "</div>
                    <div class='col col-12 col-sm-8' id='input-file'><input type='file' name='soubor'></div>
                </div>
            </div>
            <div class='buttons'>
                <div>
                    <input id='vlozitDokumentBtn' type='button' class='button' value='" . TextManager::_('form.Insert') . "' />
                    <input id='stornoDokumentBtn' type='button' class='button cancel' value='" . TextManager::_('Cancel') . "' />
                </div>
            </div>
            
            </div>";
        }
    ?>

    <form method="post" name="formularzadosti" enctype="multipart/form-data" action="?menuid=dane&action=ulozitzadost">
        <input type="hidden" name="rok" value="<?= $rok ?>">
        <input type="hidden" name="osoba" value="<?= $rocniVyuctovaniForm->pracovniPomer->idPP ?>">
        <input type="hidden" name="skolkovne_max" value="<?= $nastaveniOdpoctu->maxSkolkovne ?? 0 ?>">
        <?= Csrf::csrfHiddenInput('vyuctovani_' . $rocniVyuctovaniForm->pracovniPomer->idPP)?>

        <div class='form large-form zadost-zuctovani'>
            <div class="form-content">

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h1> <?= $rocniVyuctovaniForm->nadpisVyuctovani() ?> </h1>
                    </div>
                </div>

                 <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h3><?= TextManager::_("zadost_za_zdanovaci_obdobi")?>
                            <label>
                                <?php //"Žádost a o roční zúčtování záloh" je za předchozí rok ?>
                                <?= $rok; ?>
                            </label>
                        </h3>
                    </div>
                </div>

                <?php if ($rocniVyuctovaniForm['ZadostPortalStav'] == 2) { ?>
                    <div class="row">
                        <div class="col col-12 col-md-12 warning">
                            <?= TextManager::_('zadost_poznamka_ucetni') . StringUtils::htmlTextConvert($rocniVyuctovaniForm['PoznamkaProZamZadost'] ?? TextManager::_('Zadost_vychozi_poznamka_vraceni')) ?>
                        </div>
                    </div>
                <?php } ?>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-12 font-weight-bold">
                            <?= TextManager::_("identifikace_poplatnika")?>
                        </div>
                    </div>

                    <div class="row">
                        <div class="col col-12 col-md-4 ">
                            <?= TextManager::_('prijmeni'). ": " ?><span class="font-weight-bold"><?=  StringUtils::htmlTextConvert($rocniVyuctovaniForm->pracovniPomer->prijmeni) ?></span>
                        </div>

                        <div class="col col-12 col-md-4">
                            <?= TextManager::_('Prohlaseni_jmeno_ditete'). ": " ?><span class="font-weight-bold"><?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->pracovniPomer->jmeno) ?></span>
                        </div>

                        <div class="col col-12 col-md-4">
                            <?= TextManager::_('rodne_cislo'). ": " ?><span class="font-weight-bold"><?=  StringUtils::htmlTextConvert($rocniVyuctovaniForm['RodneCislo']) ?></span>
                        </div>
                    </div>

                    <?php if ($rocniVyuctovaniForm['DanovyNerezident']) { //25.2.2025 doplneno info, ze je zamestanec danovy nerezident, pokud si formular zobrazi ?>
                        <div class="row">
                            <div class="col col-12 center font-weight-bold ">
                                <?= TextManager::_('zadost_danovy_nerezident'). ": " ?>
                            </div>
                        </div>
                    <?php } ?>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 center clickLink toggleClass" data-toggle-class="d-none" data-selector=".dodatacne-slevy">
                        <div class="font-weight-bold font-size-2 margin-all-3"><?= TextManager::_("zadost_dodatecne_slevy") ?>&nbsp;<i class="inline-glyph glyph-button fa glyph-edit"></i></div>
                    </div>
                </div>

                <?php //7.1.2025 TODO zatim skryto - chybi schvaleni v app:
                    if (!$rocniVyuctovaniForm['DanovyNerezident']) {
                        if ($GLOBALS['debug_dane'] ?? false)  include('view_dodatecneslevy.php');
                } ?>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte1"></div>
                <?php } ?>

                <div id="dane_info_skryte1" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("nejsem_povinen_podat")?></div>
                </div>

                <?php if (!$rocniVyuctovaniForm['DanovyNerezident']) { ?>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold">
                        <?= TextManager::_("zadost_uvadim_prijmy_za_obdobi")?>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-3 center">
                            <?= TextManager::_("v_kalendarnich_mesicich")?>
                        </div>
                        <div class="col col-12 col-md-9 center">
                            <?= TextManager::_("zadost_identifikace_predchozich_platcu")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <?php $predchoziPlatceDane = RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3);
                    if (!is_null($rocniVyuctovaniForm['PredchoziPrijmyKdyOdKoho'])) {
                        $predchoziPrijmy = $rocniVyuctovaniForm->parseMemoToArray($rocniVyuctovaniForm['PredchoziPrijmyKdyOdKoho']);
                    } else {
                        $predchoziPrijmy = [];
                    }
                    for($i = 0; $i < $predchoziPlatceDane; $i++) {
                        $mesiceOdDo = [0 => 0, 1 => 0];
                        if (isset($predchoziPrijmy[$i][0]) && $predchoziPrijmy[$i][0] != '') {
                            $mesiceOdDo = explode('-', $predchoziPrijmy[$i][0]);
                        }
                    ?>
                        <div class="row">
                             <div class="col col-12 col-md-3 center">
<!--                                <label title="--><?php //= TextManager::_('')?><!--">-->
<!--                                    <input type="text" class="pole-zadosti filter-zadosti-od-do"-->
<!--                                            name="predchoziplatcedanemesice_--><?php //= $i ?><!--"-->
<!--                                            placeholder="mm-mm" --><?php //= $disabled ?>
<!--                                            value="--><?php //= $_POST["predchoziplatcedanemesice_$i"] ?? $predchoziPrijmy[$i][0] ?? '' ?><!--">-->
<!--                                </label>-->
                                 <?php
                                    $rocniVyuctovaniForm->renderComboMesice("predchoziplatcedanemesice_od_$i", $_POST["predchoziplatcedanemesice_od_$i"] ?? $mesiceOdDo[0]);
                                    echo " - ";
                                    $rocniVyuctovaniForm->renderComboMesice("predchoziplatcedanemesice_do_$i", $_POST["predchoziplatcedanemesice_do_$i"] ?? $mesiceOdDo[1]);
                                ?>
                            </div>

                            <div class="col col-12 col-md-9">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti"
                                           name="predchoziplatcedaneadresa_<?= $i ?>"
                                           placeholder="<?php echo TextManager::_('zadost_nazev_platce_dane') . ",    " .
                                              TextManager::_('zadost_ico_platce_dane') . ",    " .
                                              TextManager::_('adresa') . ",     "
                                              ?>" <?= $disabled ?>
                                            value="<?= $_POST["predchoziplatcedaneadresa_$i"] ?? $predchoziPrijmy[$i][1] ?? '' ?>">
                                </label>
                            </div>
                        </div>

                    <?php }?>
                    <input type="hidden" name="pocetpredchozichplatcudane" value="<?= $i?>">

                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte2"></div>
                <?php } ?>

                <div id="dane_info_skryte2" <?= $classStr ?>>
                    <div class="info"><sup>2)&nbsp;</sup><?= TextManager::_("zadost_za_prve")?></div>
                </div>

                <?php
                DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'predchozizamestnavatel', $rocniVyuctovaniForm->lzeEditovatVyuctovani());
                ?>


                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h2><?= TextManager::_("zadost_nezdanitelne_casti")?></h2>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("zadost_casti_zakladu_dane_uplatnuji")?><sup>3)</sup>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_uplatnuji_nezdanitelne_casti_zakladu")?>
                        </div>

                        <!--<div class="col col-12 col-md-2 ">
                            <?php /*= TextManager::_("zadost_nezdanitelne_casti_napoctena_castka") */?>
                        </div>-->

                        <div class="col col-12 col-md-3 ">
                            <?= TextManager::_("zadost_nezdanitelne_casti_uplatnovana_castka")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostdaryzadam', $rocniVyuctovaniForm['OpravaHodnotDaru'] > 0, false, ['id' => 'daryzadam']);?>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_nezdanitelne_bezuplatne")?>
                        </div>

                        <div class="col col-12 col-md-1 ">
                            <?= StringUtils::format('number0', $rocniVyuctovaniForm['HodnotaDaru']) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" data-selector="#daryzadam" name="zadostdaryvyse" min="0" value="<?= $_POST['zadostdaryvyse'] ?? $rocniVyuctovaniForm['OpravaHodnotDaru'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'dary');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadosturokyzadam', ($rocniVyuctovaniForm['OpravaUrokuZeStavebSpor']) > 0, false, ['id' => 'urokyzadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_uroky_pred_2021")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" data-selector="#urokyzadam, #zadostbytovepotrebypred2021" name="zadosturokyvyse" min="0" value="<?= $_POST['zadosturokyvyse'] ?? $rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'uroky');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadosturoky2021zadam', ($rocniVyuctovaniForm['UrokZeStavebSporPo2021']) > 0, false, ['id' => 'uroky2021zadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_uroky_2021")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" data-selector="#uroky2021zadam, #zadostbytovepotrebypo2021" name="zadosturoky2021vyse" min="0" value="<?= $_POST['zadosturoky2021vyse'] ?? $rocniVyuctovaniForm['UrokZeStavebSporPo2021'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'uroky2021');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <div class="row">
                        <div class="col col-12 col-md-2 ">

                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_danove_produkty_na_stari")?>
                        </div>

                        <div class="col col-12 col-md-3 ">
                            <?= TextManager::_("zadost_nezdanitelne_casti_uplatnovana_castka")?><sup>3a)</sup>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <div class="mobile-only"><br></div> <?php //Penzijni pripojisteni ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpenzijnizadam', $rocniVyuctovaniForm['OpravaPenzijnihoPrip'] > 0, false, ['id' => 'penzijnizadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_penzijni_pripojisteni")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-soucet-max="48000" data-selector="#penzijnizadam" name="zadostpenzijnivyse" min="0" value="<?= $_POST['zadostpenzijnivyse'] ?? $rocniVyuctovaniForm['OpravaPenzijnihoPrip'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'penzijni');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div> <?php //Doplnkove penzijni sporeni ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostdpszadam', $rocniVyuctovaniForm['OdpocetNaPenzijniSporeni'] > 0, false, ['id' => 'dpszadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_dps")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-selector="#dpszadam" name="zadostdpsvyse" min="0" value="<?= $_POST['zadostdpsvyse'] ?? $rocniVyuctovaniForm['OdpocetNaPenzijniSporeni'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'dps');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div> <?php //Penzijni pojisteni u instit. penzijniho pojisteni ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpenzpojzadam', $rocniVyuctovaniForm['OdpocetNaPenzijniPojisten'] > 0, false, ['id' => 'penzpojzadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_penzijni_pojisteni")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-selector="#penzpojzadam" name="zadostpenzpojvyse" min="0" value="<?= $_POST['zadostpenzpojvyse'] ?? $rocniVyuctovaniForm['OdpocetNaPenzijniPojisten'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'penzpoj');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div> <?php //Zivotni pojisteni ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpojistnezadam', $rocniVyuctovaniForm['ZivotniPojisteni'] > 0, false, ['id' => 'pojistnezadam']);?>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_nezdanitelne_zivotni")?>
                        </div>

                        <div class="col col-12 col-md-1 ">
                            <?= StringUtils::format('number0', $rocniVyuctovaniForm['UplatneneZivotniPojisteni']) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-selector="#pojistnezadam" name="zadostpojistnevyse" min="0" value="<?=$_POST['zadostpojistnevyse'] ?? $rocniVyuctovaniForm['ZivotniPojisteni'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'zivotni');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div> <?php //DIP ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostdipzadam', $rocniVyuctovaniForm['OdpocetDIP'] > 0, false, ['id' => 'dipzadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_dip")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-selector="#dipzadam" name="zadostdipvyse" min="0" value="<?= $_POST['zadostdipvyse'] ?? $rocniVyuctovaniForm['OdpocetDIP'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'dip');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div> <?php //Pojisteni dlouhodobe pece ?>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpecezadam', $rocniVyuctovaniForm['PojisteniDlouhodobePece'] > 0, false, ['id' => 'pecezadam']); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_dlouhodoba_pece")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky kontrola-souctu" data-selector="#pecezadam" name="zadostpeceyvyse" min="0" value="<?= $_POST['zadostpeceyvyse'] ?? $rocniVyuctovaniForm['PojisteniDlouhodobePece'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'pece');
                                } ?>
                            </label>
                        </div>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte4"></div>
                <?php } ?>

                <div id="dane_info_skryte4" <?= $classStr ?>>
                    <div class="info"><sup>3)</sup>
                        <?= TextManager::_("zadost_za_ctvrte")?></div>
                    <div class="info"><sup>3a)</sup>
                        <?= TextManager::_("zadost_za_treti_a_2024")?></div>
                </div>

                <?php
                DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', $rocniVyuctovaniForm->lzeEditovatVyuctovani(),true,
                    ['dary' => DaneController::vratPopisDokumentuDleSekce('dary'),
                        'uroky' => DaneController::vratPopisDokumentuDleSekce('uroky'),
                        'uroky2021' => DaneController::vratPopisDokumentuDleSekce('uroky2021'),
                        'penzijni' => DaneController::vratPopisDokumentuDleSekce('penzijni'),
                        'dps' => DaneController::vratPopisDokumentuDleSekce('dps'),
                        'penzpoj' => DaneController::vratPopisDokumentuDleSekce('penzpoj'),
                        'zivotni' => DaneController::vratPopisDokumentuDleSekce('zivotni'),
                        'dip' => DaneController::vratPopisDokumentuDleSekce('dip'),
                        'pece' => DaneController::vratPopisDokumentuDleSekce('pece')]);
                ?>

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h3><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti_nadpis")?></h3>
                    </div>
                </div>

                <div class="info"><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti")?></div>
                <div class="info"><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti_info_2023")?>
                    <?php $rocniVyuctovaniForm->renderComboBytovePotreby('bytovepotrebyuplatnuje', (isset($_POST['bytovepotrebyuplatnuje']) && $_POST['bytovepotrebyuplatnuje'] == 1) || ($rocniVyuctovaniForm['OsobyUplatnujiciUver'] != ''),
                        $rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] + $rocniVyuctovaniForm['UrokZeStavebSporPo2021'] > 0,
                        ['class' => 'bytove_potreby daneToggleDisabled',
                         'data-selector' => '.bytove_potreby_dalsi_osoba'
                        ]);
                    ?>
                    <sup>4)</sup>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("Prohlaseni_jmeno_ditete")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("prijmeni")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("rc")?><sup>4a)</sup>
                        </div>

                        <div class="col col-12 col-md-6 center">
                            <?= TextManager::_("zadost_adresa_trvaleho_bydliste")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <?php $pocetJinychOsobUplatnujicichOdpocet = 3;

                    if (!is_null($rocniVyuctovaniForm['OsobyUplatnujiciUver'])) {
                        $osobyUveru = $rocniVyuctovaniForm->parseMemoToArray($rocniVyuctovaniForm['OsobyUplatnujiciUver']);
                    } else {
                        $osobyUveru = [];
                    }

                    $osobyBytDisabled = $disabled;
                    if (empty($osobyUveru)) {
                        $osobyBytDisabled = 'disabled';
                    }

                    for($i = 0; $i < $pocetJinychOsobUplatnujicichOdpocet; $i++) { ?>
                        <div class="mobile-only"><?= TextManager::_('zadost_osoba') ?></div>
                        <div class="row">
                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby_dalsi_osoba"
                                           name="jinaosobauplatnujeodpocetjmeno_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetjmeno_$i"] ?? $osobyUveru[$i][0] ?? '' ?>" <?= $osobyBytDisabled ?>>
                                </label>
                            </div>

                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby_dalsi_osoba"
                                           name="jinaosobauplatnujeodpocetprijmeni_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetprijmeni_$i"] ?? $osobyUveru[$i][1] ?? '' ?>" <?= $osobyBytDisabled ?>>
                                </label>
                            </div>


                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti ronRC bytove_potreby_dalsi_osoba"
                                           name="jinaosobauplatnujeodpocetrc_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetrc_$i"] ?? $osobyUveru[$i][2] ?? '' ?>" <?= $osobyBytDisabled ?>>
                                </label>
                            </div>

                            <div class="col col-12 col-md-6">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby_dalsi_osoba"
                                           name="jinaosobauplatnujeodpocetadresa_<?= $i ?>"
                                           placeholder="<?= TextManager::_('obec') . ": " .
                                           TextManager::_('mesto') . ": " .
                                           TextManager::_('ulice') . ":  " .
                                           TextManager::_('psc') . ":" ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetadresa_$i"] ?? $osobyUveru[$i][3] ?? '' ?>" <?= $osobyBytDisabled ?>>
                                </label>
                            </div>
                        </div>

                    <?php } ?>
                    <input type="hidden" name="pocetjinychosobuplatnujicichodpocet" value="<?= $pocetJinychOsobUplatnujicichOdpocet?>">

                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte6"></div>
                <?php } ?>

                <div id="dane_info_skryte6" <?= $classStr ?>>
                    <div class="info"><sup>4)&nbsp;</sup><?= TextManager::_("zadost_za_ctvrte_b")?></div>
                    <div class="info"><sup>4a)&nbsp;</sup><?= TextManager::_("zadost_rodne_cislo_info")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-2 ">
                        <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostbytovepotrebypred2021',
                            isset($_POST['zadostbytovepotrebypred2021']) || ($rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] > 0),
                            false, ['class' => 'inactive', 'id' => 'zadostbytovepotrebypred2021']); ?><sup>4b)&nbsp;</sup>
                    </div>

                    <div class="col col-12 col-md-10 font-weight-bold">
                        <?= TextManager::_('zadost_bytove_potreby_pred_2021') ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte8"></div>
                <?php } ?>

                <div id="dane_info_skryte8" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_pred_2021_info")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-2 ">
                        <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostbytovepotrebypo2021',
                            isset($_POST['zadostbytovepotrebypo2021']) || ($rocniVyuctovaniForm['UrokZeStavebSporPo2021'] > 0),
                            false, ['class' => 'inactive', 'id' => 'zadostbytovepotrebypo2021']); ?><sup>4b)&nbsp;</sup>
                    </div>

                    <div class="col col-12 col-md-10 font-weight-bold">
                        <?= TextManager::_('zadost_bytove_potreby_po_2021') ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte9"></div>
                <?php } ?>

                <div id="dane_info_skryte9" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_po_2021_info")?></div>
                    <div class="info"><sup>4b)&nbsp;</sup><?= TextManager::_("zadost_za_ctvrte_c")?></div>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_upozorneni")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold">
                        <?= TextManager::_("zadost_misto_bytove_potreby_2023")?>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-12 ">
                            <input type="hidden" id="adresabytovepotrebyuveru" value="<?= TextManager::_('obec') . ":" .  $rocniVyuctovaniForm->bydliste->obecPopis . ", \n" .
                            TextManager::_('mesto') . ":" . $rocniVyuctovaniForm->bydliste->mesto . ", \n" .
                            TextManager::_('ulice') . ":" . $rocniVyuctovaniForm->bydliste->ulice . ", \n" .
                            TextManager::_('psc') . ":" . $rocniVyuctovaniForm->bydliste->psc ?>"  >

                            <label title="<?= TextManager::_('zadost_pouzit_aktualni_adresu')?>">
                            <textarea name="zadostbytovepotrebyuveruadresa" class="bytove_potreby"
                                      id="zadostbytovepotrebyuveruadresa" rows="3" cols="115"
                                      placeholder="<?php echo TextManager::_('obec') . ":     \n" .
                                          TextManager::_('mesto') . ":     \n" .
                                          TextManager::_('ulice') . ":      \n" .
                                          TextManager::_('psc') . ":    " ?>" <?= $disabled ?>><?= StringUtils::htmlTextConvert($_POST["zadostbytovepotrebyuveruadresa"] ?? $rocniVyuctovaniForm['AdresaBytovePotrebyUveru']) ?>
                            </textarea>
                            </label>
                        </div>
                    </div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h2><?= TextManager::_("zadost_slevy_na_dani")?></h2>
                    </div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold info">
                        <?= TextManager::_("zadost_uplatnuji_narok_na")?>
                    </div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <?= TextManager::_("zadost_sleva_za_exekuci")?>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-10 align-center">
                            <?= TextManager::_("zadost_nahrada_exekuce")?><sup>5)&nbsp;</sup>
                        </div>
                        <div class="col col-12 col-md-2 ">
                            <label title="<?= TextManager::_('')?>">
                                <input type="number" class="pole-zadosti"
                                       name="nahrada_exekuce"
                                       value="<?= $_POST["nahrada_exekuce"] ?? $rocniVyuctovaniForm['SlevaZaZastavenouExekuci'] ?? '' ?>" <?= $disabled ?>>
                            </label>
                        </div>
                    </div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <sup>5)&nbsp;</sup><?= TextManager::_("zadost_nahrada_exekuce_info")?>
                    </div>
                </div>

                <?php
                DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'slevaexekuce', $rocniVyuctovaniForm->lzeEditovatVyuctovani());
                ?>

                <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <?= TextManager::_("zadost_sleva_na_manzelku_2024")?>
                    </div>
                </div>


                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <span class="font-weight-bold"><?= TextManager::_("zadost_udaje_manzel")?></span><br/>
                            <?= TextManager::_("Jmeno")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("Prijmeni")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("rc")?><sup>6)&nbsp;</sup>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("zadost_narok_splnen_za")?><sup>6a)</sup>
                        </div>

                        <div class="col col-12 col-md-1 center">
                            <?= TextManager::_("ZTP/P")?>
                        </div>

                        <div class="col col-12 col-md-3 center">
                            <?= TextManager::_("zadost_narok_splnen_za_ztp")?><sup>6b)</sup>
                        </div>

                    </div>

                    <hr class="oddelovac">

                    <div class="row">
                        <?php if (isset($rocniVyuctovaniForm->partnerPoplatnika)) {
                            if (!is_null($rocniVyuctovaniForm['SlevaPartnerObdobi'])) {
                                $partnerObdobi = explode(";", $rocniVyuctovaniForm['SlevaPartnerObdobi']);
                            } else {
                                $partnerObdobi = [];
                            }
                            ?>
                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->jmeno) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->prijmeni) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->rodneCislo) ?>
                        </div>

                        <div class="col col-12 col-md-2 zadost_mesice">
                            <?php $rocniVyuctovaniForm->renderMesiceCheckboxes('slevanapartnera', '', $partnerObdobi[0] ?? str_repeat('0', 12),false) ?>
                        </div>

                        <div class="col col-12 col-md-1 center">
                            <?php
                                if ($rocniVyuctovaniForm->partnerPoplatnika->ztpp) {
                                    GuiUtils::renderCheckbox('zttp', '', true, ['disabled' => 'disabled']);
                                }
                            ?>
                        </div>

                        <div class="col col-12 col-md-3 zadost_mesice">
                            <?php $rocniVyuctovaniForm->renderMesiceCheckboxes('slevanapartneraproztp', '', $partnerObdobi[1] ?? str_repeat('0', 12), !$rocniVyuctovaniForm->partnerPoplatnika->ztpp) ?>
                        </div>

                        <?php } ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte3"></div>
                <?php } ?>

                <div id="dane_info_skryte3" <?= $classStr ?>>
                    <div class="info"><sup>6)&nbsp;</sup><?= TextManager::_("zadost_rodne_cislo_info")?></div>
                    <div class="info"><sup>6a)&nbsp;</sup><?= TextManager::_("zadost_rozsah_mesicu_2024")?></div>
                    <div class="info"><sup>6b)&nbsp;</sup><?= TextManager::_("zadost_rozsah_mesicu_ZTP")?></div>
                </div>

                <?php
                    DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'slevanapartnera', $rocniVyuctovaniForm->lzeEditovatVyuctovani(),
                        isset($rocniVyuctovaniForm->partnerPoplatnika) /*&& (!empty($partnerObdobi[0]))*/); //vyplnene obdobi se pro zvyrazneni povinnosti nekontroluje, ne
                ?>

                <?php } //if ! $danovyNerezident ?>

            </div>


            <div class="form-content ram">
                <div class="row">
                    <div class="col col-12 col-md-3 ">
                        <span class="font-weight-bold"><?= TextManager::_("zadost_udaje_dite")?></span><br/>
                        <?= TextManager::_("Jmeno")?>
                    </div>
                    <div class="col col-12 col-md-3 ">
                        <?= TextManager::_("Prijmeni")?>
                    </div>
                    <div class="col col-12 col-md-3 ">
                        <?= TextManager::_("Datum_narozeni")?>
                    </div>
                </div>

                <hr class="oddelovac">

                <?php
                    if (!is_null($rocniVyuctovaniForm->deti)) {
                        foreach ($rocniVyuctovaniForm->deti as $dite) {
                            if ($dite->vekNaKonciRoku->y <= 3) {
                ?>

                            <div class="row">
                                <div class="col col-12 col-md-3 ">
                                    <?= StringUtils::htmlTextConvert($dite->jmeno) ?>
                                </div>
                                <div class="col col-12 col-md-3 ">
                                    <?= StringUtils::htmlTextConvert($dite->prijmeni) ?>
                                </div>
                                <div class="col col-12 col-md-3 ">
                                    <?= StringUtils::format('date', $dite->datumNarozeni) ?>
                                </div>
                            </div>
                <?php
                            }
                        }
                    }
                ?>
            </div>

            <br />

            <?php if ($jednoduchyVzhled) { ?>
                <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte7"></div>
            <?php } ?>

            <div id="dane_info_skryte7" <?= $classStr ?>>
                <div class="info"><?= TextManager::_("zadost_neopravnene_vyplaceni")?></div>
            </div>

            <div class="form-content ram">
                <div class="row">
                    <div class="col col-12 col-md-4 center">
                        <?= TextManager::_("") ?>
                    </div>

                    <div class="col col-12 col-md-4 center">
                        <?php echo TextManager::_("Mzdy_prokazatelne_podana_zadost_poplatnikem") ?>
                    </div>

                    <div class="col col-12 col-md-4 center">
                        <?php echo sprintf(TextManager::_("Mzdy_overeni_platcem_dane"), '') ?>
                    </div>
                </div>

                <hr class="oddelovac">

                <div class="row align-center">
                    <div class="col col-12 col-md-4">
                        <div><?= TextManager::_('Mzdy_na_uvedene_zdanovaci_obdobi') ?> </div>
                    </div>

                    <div class="col col-12 col-md-4">
                        <div> <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm['ElektronickyPodpisZadost']) ?> </div>
                    </div>

                    <div class="col col-12 col-md-4">
                        <div> <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm['ElektronickyPodpisUcetniZ']) ?>  </div>
                    </div>
                </div>
            </div>

            <div class="buttons">
                <input type="button" class="button clickHref" data-ron-href="?menuid=dane" value="<?= TextManager::_('zpet')?>">
                <?php if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) { ?>
                    <button type="submit" class="button" name="ulozit" value="1"><?= TextManager::_('zadost_ulozit')?></button>
                    <button type="submit" class="button confirmQuestion"  data-ron-confirm-text="<?= TextManager::_('zadost_opravdu_podepsat'); ?>" name="podepsatzadost" value="1"><?= TextManager::_('zadost_potvrzuji_pravdivost_zadosti')?></button>
                <?php } ?>
            </div>
        </div>

        <br>

    </form>

<?php } ?>

<?php
    FormReportsListView::renderReportList($reportlist, '?menuid=dane&action=report&typ=vyuctovani&idpp=' . $rocniVyuctovaniForm->pracovniPomer->idPP . '&rok=' . $rok);
?>