<?php

namespace RonPortal\Personalistika;

use Ron\Controls\BrowserField;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\RozdeleniUtils;
use RonPortal\User;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\PravaNaOsoby;


class PersonalniProcesyBrowser extends RonPortalBrowser
{
    public $pravaProhlizeni;
    public $pravaEditovat;

    public function __construct(Database $database, PravaNaOsoby $pravaProhlizeni, PravaNaOsoby $pravaEditovat)
    {
        parent::__construct($database, 'personalniprocesy');

        $this->pravaProhlizeni = $pravaProhlizeni;
        $this->pravaEditovat = $pravaEditovat;

        $this->title = TextManager::_('Pers_procesy');

        $this->code = 'browserpersonalniprocesy';
        $this->browserUrl = '?menuid=personalniprocesy';

        $this->sqlSelectPart =
            'PersProces.ID'; // 0

        $this->sqlFromPart =
            'PersProces JOIN TypPersProcesu ON (TypPersProcesu.Kod = PersProces.KodTypuProcesu)' .
            ' LEFT OUTER JOIN Osoba O_Zalozil ON (O_Zalozil.RC = PersProces.ZalozilRC)' .
            ' LEFT OUTER JOIN Osoba O_Ukoncil ON (O_Ukoncil.RC = PersProces.UkoncilRC)' .
            ' LEFT OUTER JOIN Osoba O_Procesu ON (O_Procesu.RC = PersProces.RC)';

        $this->primaryField = 0;

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PersProces.ID', 0)
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('TypPersProcesu.Popis')
            ->setHeaderText('Pers_procesy_typ')
            ->setCode('col-popis-procesu')
            ->setOrdering(true)
            ->setLink('?menuid=personalniprocesy');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Procesu.Jmeno')
            ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Procesu.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Procesu.Titul')
            ->setCode('col-titul');

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 'col-jmeno', 'col-prijmeni', 'col-titul'))
            ->setHeaderText('Pers_procesy_osoba')
            ->setCode('col-osoba')
            ->setOrdering(true)
            ->setSortSQL('O_Procesu.Prijmeni, O_Procesu.Jmeno, O_Procesu.Titul', 'O_Procesu.Prijmeni DESC, O_Procesu.Jmeno DESC, O_Procesu.Titul DESC');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Zalozil.Jmeno')
            ->setCode('col-zalozil-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Zalozil.Prijmeni')
            ->setCode('col-zalozil-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Zalozil.Titul')
            ->setCode('col-zalozil-titul');

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 'col-zalozil-jmeno', 'col-zalozil-prijmeni', 'col-zalozil-titul'))
            ->setHeaderText('Pers_procesy_zalozil')
            ->setCode('col-zalozil')
            ->setOrdering(true)->setSortSQL('O_Zalozil.Prijmeni, O_Zalozil.Jmeno, O_Zalozil.Titul', 'O_Zalozil.Prijmeni DESC, O_Zalozil.Jmeno DESC, O_Zalozil.Titul DESC');

        $this->addField()
            ->setSqlFieldName('PersProces.ZalozilDatum')
            ->setHeaderText('Pers_procesy_zalozeno_datum')
            ->setFormat('datetime')
            ->setCode('col-zalozeno-datum-cas')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('PersProces.Stav')
            ->setHeaderText('Pers_procesy_stav')
            ->setCode('col-stav')
            ->setOrdering(true)
            ->setTranslationPrefix('Pers_procesy_stav_');

        $this->addField()
            ->setSqlFieldName('PersProces.DatumUkonceniPozadovany')
            ->setHeaderText('Pers_procesy_datum_ukonceni_pozadovane')
            ->setFormat('date')
            ->setCode('col-datum-ukonceni-pozadavek')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Ukoncil.Jmeno')
            ->setCode('col-ukoncil-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Ukoncil.Prijmeni')
            ->setCode('col-ukoncil-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('O_Ukoncil.Titul')
            ->setCode('col-ukoncil-titul');

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 'col-ukoncil-jmeno', 'col-ukoncil-prijmeni', 'col-ukoncil-titul'))
            ->setHeaderText('Pers_procesy_ukoncil')
            ->setCode('col-ukoncil')
            ->setOrdering(true)
            ->setSortSQL('O_Ukoncil.Prijmeni, O_Ukoncil.Jmeno, O_Ukoncil.Titul', 'O_Ukoncil.Prijmeni DESC, O_Ukoncil.Jmeno DESC, O_Ukoncil.Titul DESC');

        $this->addField()
            ->setSqlFieldName('PersProces.DatumUkonceniSkutecny')
            ->setHeaderText('Pers_procesy_datum_ukonceni_skutecne')
            ->setFormat('date')
            ->setCode('col-datum-ukonceni-skutecnost')
            ->setOrdering(true);
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $prava = $this->pravaProhlizeni;

        $zobrazeni = (int)($_GET['zobrazeni'] ?? $_SESSION['pers_procesy_zobrazeni'] ?? 0);

        if ($zobrazeni) {
            // procesy k memu vyjadreni => zobrazi i vyber rozdeleni a osob
            if ($prava ->zobrazitVyberRozdeleni()) {
                $this->filterForm->addFilterRozdeleni1(null, $prava);
                $this->filterForm->addFilterRozdeleni2(null, $prava);
            }

            $this ->filterForm ->addFilterPP(true, $prava);
            //$this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich);
            //$this ->filterForm ->povolitVsechyOsoby = true;
        }

//        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
//            $this ->filterForm ->nastavRozdeleni(true);
//            $this ->filterForm ->f_pravarozdeleni = $prava;
//            $this->filterForm->vcetnePodrizenychRozdeleni = true;
//
//            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
//                $this ->filterForm ->nastavRozdeleni2(true);
//                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
//            }
//        }

//        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
//        $this->filterForm->povolitVsechyOsoby = true;
//        $this->filterForm->setUseGet(true, 'personalniprocesy');

//        $this ->filterForm ->addFilter(
//            (new FilterFormFilter('zobrazeni2', FilterFormFilter::TYPE_HIDDEN))
//                ->setHtmlAttributes(['name' => 'zobrazeni'])
//                ->setDefaultValue($zobrazeni)
//        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('zobrazeni'))
                //->setUrl('?menuid=personalniprocesy&zobrazeni=')    // nedavat sem - aby vygeneroval i hidden pole
                ->setSessionName('pers_procesy_zobrazeni')
                ->addItem(
                    (new FilterFormFilterItem(0, TextManager::_('Pers_procesy_moje')))
                        ->setUrl('?menuid=personalniprocesy&zobrazeni=0')
                        //->setGroup('zobrazeni')
                    )
                ->addItem(
                    (new FilterFormFilterItem(1, TextManager::_('Pers_procesy_vyplneni')))
                        ->setUrl('?menuid=personalniprocesy&zobrazeni=1')
                        //->setGroup('zobrazeni')
                    )
                ->setPosition(FilterFormFilter::POSITION_ABOVE_FORM)
                ->setDefaultValue(0)
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('typprocesu', FilterFormFilter::TYPE_OPTIONS))
                ->setSessionName('pers_procesy_proces')
                ->setCaption(TextManager::_('Pers_procesy_typ'))
                ->addItem('', TextManager::_('_vse_'))
                ->setAddClass('ronselect')
                ->setOptionsSql(
                    "SELECT Kod, Popis FROM TypPersProcesu WHERE %FILTER% ORDER BY %ORDER%",
                    '1 = 1',
                    'TypPersProcesu.Popis')
                ->setSqlFilter('PersProces.KodTypuProcesu = ' ),
            true
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('stav'))
                ->setSessionName('pers_procesy_stav')
                ->addItem(2, TextManager::_('Pers_procesy_stav_2'))
                ->addItem(11, TextManager::_('Pers_procesy_stav_11'))
                ->addItem(12, TextManager::_('Pers_procesy_stav_12'))
                ->addItem(13, TextManager::_('Pers_procesy_stav_13'))
                ->addItem(14, TextManager::_('Pers_procesy_stav_14'))
                ->addItem(128, TextManager::_('Pers_procesy_stav_128'))
                ->setDefaultValue(2)
                ->setCaption(false)
                ->setRowClass('margin-top-2')
        );

    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql = $this ->filterForm ->buildSqlFilter($params);

        switch ($this->filterForm->getFilterValue('zobrazeni')) {
            case 0:
            default:
                $sql .=
                    ($sql ? ' AND ' : '') .
                    'PersProces.RC = ' . $params->addParameter(User::getRC());
                break;

            case 1:
                if ($this->filterForm->getFilterValue('osoba')) {
                    $sql .=
                        ($sql ? ' AND ' : '') .
                        "PersProces.RC = " . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc')) .
                        " AND PersProces.OsobniCislo = " . $params->addParameter($this->filterForm->getFilterValue('osoba', 'osobniCislo')) .
                        " AND PersProces.DatumNastupu = " . $params->addDateParameter($this->filterForm->getFilterValue('osoba', 'datumNastupu'));
                } else {
                    $sql .=
                        ($sql ? ' AND ' : '') .
                        "(" . $this->pravaProhlizeni->sqlFiltr('PersProces', DateUtils::today(), DateUtils::today(), $params) .
                        " OR " .
                        $this->pravaEditovat->sqlFiltr('PersProces', DateUtils::today(), DateUtils::today(), $params) . ")";

                    $sql .= " AND ";

                    if ($this->pravaProhlizeni->priznak || $this->pravaEditovat->priznak) {
                        $sql .= "1 = 1";
                    } else {
                        $sql .= PersonalniProcesyController::getSQLFilterProcesy($params);
                    }

                    foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                        if ($rozdeleniFilter ->getValue()) {
                            $sql .=
                                ($sql ? ' AND ' : '') .
                                RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PersProces',
                                    $rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue(),
                                    DateUtils::today(), DateUtils::today(),
                                    $rozdeleniFilter->getValue('vcetnePodrizenych'),
                                    $params);
                        }
                    }

//                    if ($this->filterForm->kodRozdeleni) {
//                        $sql .= " AND " .
//                            RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PersProces', $this->filterForm->f_typrozdeleni, $this->filterForm->kodRozdeleni, DateUtils::today(), DateUtils::today(),
//                                $this->filterForm->vcetnePodrizenychRozdeleni, $params);
//                    }
//                    if ($this->filterForm->kodRozdeleni2) {
//                        $sql .= " AND " .
//                            RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PersProces', $this->filterForm->f_typrozdeleni2, $this->filterForm->kodRozdeleni2, DateUtils::today(), DateUtils::today(),
//                                $this->filterForm->vcetnePodrizenychRozdeleni2, $params);
//                    }
                }
                break;
        }

        if ($this->filterForm->getFilterValue('stav')) {
            $sql .=
                ($sql ? ' AND ' : '') .
                'PersProces.Stav = ' . $params->addParameter($this->filterForm->getFilterValue('stav'));
        } else {
            $sql .=
                ($sql ? ' AND ' : '') .
                'PersProces.Stav > ' . $params->addParameter(1);
        }

//        if ($this->filterForm->typProcesu) {
//            $sql .=
//                " AND PersProces.KodTypuProcesu = " . $params->addParameter($this->filterForm->typProcesu);
//        }

        return new SqlStatement($sql, $params);
    }


}
