<?php

namespace RonPortal\Personalistika;

use Ron\DB\Database;
use Ron\DB\DBException;

class ZdravotniStavOsoby
{
    public $rc;
    public $platnostOd;
    public $platnostDo;

    /** @var int 0 = zdravy, 1 = castecne invalidni, 2 = invalidni (III. stupen) */
    public $stav;
    public $stavPopis;
    public $drzitelPrukazuZTTP;
    public $osetrujiciLekar;
    public $osetrujiciLekarJmenoPrijmeni;
    public $zps;
    public $zps_tzp;

    /**
     * @param Database $database
     * @param $rc
     * @param $datumOd
     * @param $datumDo
     * @return static[]
     * @throws DBException
     */
    public static function nactiStavy($database, $rc, $datumOd, $datumDo)
    {
        if (!isset($datumDo)) {
            $datumDo = $datumOd;
        }

        $params = $database->createParameters();

        $sql =
            'SELECT PlatnostOd, PlatnostDo, ZdravotniStav, S_ZdravotniStav.Popis, DrzitelPrukazuZTTP, ZdravotniStavOsoby.OsetrujiciLekar, ' . //0 - 5
            '  OsetrujiciLekar.JmenoLekare, OsetrujiciLekar.Prijmeni, ZPS, ZPS_TZP ' . //6 - 9
            ' FROM ZdravotniStavOsoby JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = ZdravotniStavOsoby.ZdravotniStav) ' .
            '   LEFT JOIN OsetrujiciLekar ON (OsetrujiciLekar.Kod = ZdravotniStavOsoby.OsetrujiciLekar) ' .
            ' WHERE ZdravotniStavOsoby.RC = ' . $params->addParameter($rc) .
            ($datumOd
                ? ' AND ZdravotniStavOsoby.PlatnostOd <= ' . $params->addDateParameter($datumDo)
                : '') .
            ($datumDo
                ? ' AND (ZdravotniStavOsoby.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR ZdravotniStavOsoby.PlatnostDo IS NULL)'
                : '') .

            ' ORDER BY PlatnostOd DESC'; // nejnovejsi bude na zacatku

        $cursor = $database->getSqlResult($sql, $params);

        $seznamStavu = [];

        while ($row = $cursor->fetch()) {
            $zdravotniStav = new static();
            $zdravotniStav->rc = $rc;
            $zdravotniStav->nactiStavZRadku($database, $row);

            $seznamStavu[] = $zdravotniStav;
        }

        $cursor ->close();

//        if (!is_null($seznamStavu) && count($seznamStavu) == 1) {
//            return $seznamStavu[0];
//        }

        return $seznamStavu;
    }


    /**
     * Nacte zdravotni stav osoby platny k zadanemu datu
     * @param Database $database
     * @param string $rc
     * @param int $datum
     * @return static|null
     * @throws DBException
     */
    public static function nactiStav($database, $rc, $datum)
    {
        $seznam = self::nactiStavy($database, $rc, $datum, $datum);
        return $seznam ? $seznam[0] : null;
    }


    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiStavZRadku($database, $row)
    {
        $this->platnostOd = $database->stringToTimestamp($row[0]);
        $this->platnostDo = $database->stringToTimestamp($row[1]);

        $this->stav = $row[2];
        $this->stavPopis = $row[3];

        $this->drzitelPrukazuZTTP = $row[4];
        $this->osetrujiciLekar = $row[5];
        $this->osetrujiciLekarJmenoPrijmeni = $row[6] . ' ' . $row[7];
        $this->zps = $row[8];
        $this->zps_tzp = $row[9];
    }
}
