<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 10:12
 */

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RozdeleniBrowserField;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use function Ron\ronText;

class SchvaleniZaznamuPKField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->align = 'left';

        $this->header = ronText('PoklK_schvaleni');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hint = '';

        if ($rowData['col-datum-schvaleni4']) {
            $date = $rowData['col-datum-schvaleni4'];
            $name = $rowData['col-schvalil4'];
            $class = 'textschvaleni4';
        } elseif ($rowData['col-datum-schvaleni3']) {
            $date = $rowData['col-datum-schvaleni3'];
            $name = $rowData['col-schvalil3'];
            $class = 'textschvaleni3';
        } else if ($rowData['col-datum-schvaleni2']) {
            $date = $rowData['col-datum-schvaleni2'];
            $name = $rowData['col-schvalil2'];
            $class = 'textschvaleni2';
        } else if ($rowData['col-datum-schvaleni1']) {
            $date = $rowData['col-datum-schvaleni1'];
            $name = $rowData['col-schvalil1'];
            $class = 'textschvaleni1';
        }  else {
            return '';
        }

        $result = "<SPAN class='$class' title='$hint'>";
        $result .= DBUtils::getFormatedValue($this ->database, 'date', $date);
        if ($name) $result .= " ($name)";
        $result .= "</SPAN>\n";

        return $result;
    }
}

class SchvaleniZaznamuPKEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    public $pravaStr;
    public $indexDatum;
    public $indexUzivatel;
    public $indexTypPolozky;
    public $htmlIndex;

    /** @var  PokladniKnihaBrowser */
    public $browser;

    public $existsEdit;


    /**
     * @param Browser $browser
     * @param string $pravaStr
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param int $htmlIndex
     * @param string $caption
     */
    function __construct($browser, $pravaStr, $indexDatum, $indexUzivatel, $indexTypPolozky, $htmlIndex, $caption)
    {
        parent::__construct($browser ->database);
        $this ->align = "center";

        $this ->header =
            "<span id='schvaleniPKMenu_$htmlIndex''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->htmlIndex = $htmlIndex;
        $this ->pravaStr = $pravaStr;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
        $this ->indexTypPolozky = $indexTypPolozky;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection DuplicatedCode  */
        $schvaleno = $rowData[$this ->indexDatum];

        $prava = Prava::vratPrava($this->pravaStr, $this->pravaStr . '_' . $rowData[$this->indexTypPolozky]);

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
        }

        if ($prava ->lzeZobrazitPP($rowData['col-idpp'], $this->database->stringToTimestamp($rowData['col-datum-pohybu']), $this->database->stringToTimestamp($rowData['col-datum-pohybu']))) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData['col-id']}_{$this ->htmlIndex}' value='1' />" .
                "<input type='checkbox' data-ronschvalenipoklk_index='{$this ->browser ->jsonRowIndex}' name='Schvaleni_{$rowData['col-id']}_{$this ->htmlIndex}' " .
                    "id='Schvaleni_{$rowData['col-id']}_{$this ->htmlIndex}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . ronText($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldIndex
     * @param string $url
     * @return string
     */
    function getFooterText($fieldIndex, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL_{$this ->htmlIndex}' id='schvaleni_ALL_{$this ->htmlIndex}' title='' />\n";
        }
        else return '';
    }

}

class ZruseniPKField extends BrowserField
{
    /** @var  string */
    public $pravaStr;
    public $indexTypPolozky;

    /**
     * @param Database $database
     * @param string $pravaStr
     */
    function __construct($database, $pravaStr, $indexTypPolozky)
    {
        parent::__construct($database);
        $this->pravaStr = $pravaStr;
        $this->indexTypPolozky = $indexTypPolozky;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datum-schvaleni1'] || $rowData['col-datum-schvaleni2'] || $rowData['col-datum-schvaleni3'] || $rowData['col-datum-schvaleni4'] || $rowData['col-zauctovano']) return ''; // jiz je schvaleno, nejde smazat

        $prava = Prava::vratPrava($this->pravaStr, $this->pravaStr . '_' . $rowData[$this->indexTypPolozky]);

        // zkontroluje prava na editaci zastupu
        if ($prava->lzeZobrazitPP($rowData['col-idpp'], $this ->database ->stringToTimestamp($rowData['col-datum-pohybu']), $this ->database ->stringToTimestamp($rowData['col-datum-pohybu']))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('PoklK_zrusit_zaznam') . '"></i>', ronText('PoklK_opravdu_zrusit'),
                '?menuid=pokladnikniha&action=zrusitzaznam&record=' . $rowData['col-id'] . Csrf::csrfUrlParameter());
        }

        return '';
    }
}

class DokumentyPKField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-pocet-dokumentu']) {
            return
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . ronText('Pocet_priloh') . ': ' . $rowData['col-pocet-dokumentu'] . '"></i>';
        }

        return '';
    }
}

class PokladniKnihaBrowser extends RonPortalBrowser
{
    private $schvalovani;
    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;
    private $pravaSchvalit4;

    public $jsonRowIndex;
    public $jsonObject;
    public $hiddenFields;

    /**
     * PokladniKnihaBrowser constructor.
     * @param $database
     * @param $cisloBrowseru
     * @param bool $schvalovani
     * @throws DBException
     */
    public function __construct($database, $cisloBrowseru, $schvalovani = false)
    {
        parent::__construct($database, 'pokladnikniha');

        $this->seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $cisloBrowseru);

        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz');

        //nactou se obecna prava bez filtru na zpusob pohybu (aby fungoval filtr)
        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl');
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1');
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2');
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3');
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4');

        $this->title = ronText('pokladni_kniha' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''));

        $this->browserUrl = "?menuid=pokladnikniha&browser=$cisloBrowseru&";
        $this->code = 'browserpokladnikniha' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : '');
        $this->tableId = 'seznampokladniknihy';

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

//        $this->sqlSelectPart =
//            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
//            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //6 - 10
//            'PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, ' . //11, 12
//            'PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //13, 14
//            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //15, 16
//            'PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //17, 18
//            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' . //19 - 23
//            'Osoba.Jmeno, Osoba.Prijmeni, TypPolozkyPokladniKnihy.Popis, PracovniPomer.ID, ' . //24 - 27
//            '(SELECT COUNT(*) FROM DokumentPokladniKnihy WHERE IDPokladniKnihy = PokladniKniha.ID), ' . //28
//            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, ' .  //29 - 34
//            'Rozdeleni.Popis, R2.Popis, PokladniKniha.Rozdeleni3, R3.Popis, PokladniKniha.Schvaleni4, PokladniKniha.Schvalil4, ' . //35 - 40
//            'PokladniKniha.Zauctovano, PokladniKniha.Zauctovano_Uzivatel, PokladniKniha.Zauctovano_DatumCas'; // 41 - 43
//
//        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
//            $this->sqlSelectPart .=
//                ', PokladniKniha.Rozdeleni4, R4.Popis'; //44 - 45
//        }

        $this->sqlFromPart =
            'PokladniKniha  ' .
            'JOIN Osoba ON (Osoba.RC = PokladniKniha.RC) ' .
            'JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky) ' .
            'JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ' .
            'LEFT OUTER JOIN Rozdeleni ON Rozdeleni.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni AND Rozdeleni.Kod = PokladniKniha.Rozdeleni '.
            'LEFT OUTER JOIN Rozdeleni R2 ON R2.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni2 AND R2.Kod = PokladniKniha.Rozdeleni2 ' .
            'LEFT OUTER JOIN Rozdeleni R3 ON R3.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni3 AND R3.Kod = PokladniKniha.Rozdeleni3 ';

        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
            $this->sqlFromPart .=
                'LEFT OUTER JOIN Rozdeleni R4 ON R4.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni4 AND R4.Kod = PokladniKniha.Rozdeleni4';
        }

        $this->primaryField = 0;
        $this->defaultSort = 'col-datum-pohybu';

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.ID')->setCode('col-id');
        
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.ID')->setCode('col-idpp');

        $f = $this->addField()
            ->setHeaderText('Prijmeni')->setSqlFieldName('Osoba.Prijmeni')
            ->setAlign('left')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-prijmeni');

        if ($this ->schvalovani) {
            $f ->setFooter(TextManager::_('Schvalit_hromadne'));
        }

        $this->addField()
            ->setHeaderText('Jmeno')->setSqlFieldName('Osoba.Jmeno')
            ->setAlign('left')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-jmeno');

        $this->addField()
            ->setHeaderText('datum_pohybu')->setSqlFieldName('PokladniKniha.DatumPohybu')
            ->setFormat('date')->setAlign('right')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-datum-pohybu');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.TypPolozky')->setCode('col-typ-polozky-kod');

        $this->addField()
            ->setHeaderText('typ_polozky')->setSqlFieldName('TypPolozkyPokladniKnihy.Popis')->setAlign('left')
            ->setCode('col-typ-polozky')->setTranslationPrefix('db_typpolozkypokladniknihy_popis_', 'col-typ-polozky-kod');

        $this->addField()
            ->setHeaderText('popis' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))
            ->setSqlFieldName('PokladniKniha.Popis')->setAlign('left')
            ->setCode('col-popis' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''));

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Kod')
                ->setCode('col-rozdeleni-kod');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Popis')
                ->setCode('col-rozdeleni-popis');

            $this->addField(new RozdeleniBrowserField($this->database, 'col-rozdeleni-kod', 'col-rozdeleni-popis'))
                ->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni');
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R2.Kod')
                ->setCode('col-rozdeleni2-kod');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R2.Popis')
                ->setCode('col-rozdeleni2-popis');

            $this->addField(new RozdeleniBrowserField($this->database, 'col-rozdeleni2-kod', 'col-rozdeleni2-popis'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni2')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R2.Kod' : 'R2.Popis');
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R3.Kod')
                ->setCode('col-rozdeleni3-kod');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R3.Popis')
                ->setCode('col-rozdeleni3-popis');

            $this->addField(new RozdeleniBrowserField($this->database, 'col-rozdeleni3-kod', 'col-rozdeleni3-popis'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni3')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R3.Kod' : 'R3.Popis');
        }
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R4.Kod')
                ->setCode('col-rozdeleni4-kod');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('R4.Popis')
                ->setCode('col-rozdeleni4-popis');

            $this->addField(new RozdeleniBrowserField($this->database, 'col-rozdeleni4-kod', 'col-rozdeleni4-popis'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni4')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R4.Kod' : 'R4.Popis');
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField()
                ->setHeaderText('castkaBezDPH')->setSqlFieldName('PokladniKniha.CastkaBezDPH')
                ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka-bez-dph');
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField()
                ->setHeaderText('castkaDPH')->setSqlFieldName('PokladniKniha.CastkaDPH')
                ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka-dph');
        }

        $this->addField()
            ->setHeaderText('castka')->setSqlFieldName('PokladniKniha.Castka')
            ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka');

        $this->addField()
            ->setHeaderText('mena')->setSqlFieldName('PokladniKniha.Mena')->setAlign('left')->setCode('col-mena');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('(SELECT COUNT(*) FROM DokumentPokladniKnihy WHERE IDPokladniKnihy = PokladniKniha.ID)')
            ->setCode('col-pocet-dokumentu');  
        
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvaleni1')
            ->setCode('col-datum-schvaleni1');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvalil1')
            ->setCode('col-schvalil1');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvaleni2')
            ->setCode('col-datum-schvaleni2');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvalil2')
            ->setCode('col-schvalil2');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvaleni3')
            ->setCode('col-datum-schvaleni3');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvalil3')
            ->setCode('col-schvalil3');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvaleni4')
            ->setCode('col-datum-schvaleni4');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Schvalil4')
            ->setCode('col-schvalil4');
        
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PokladniKniha.Zauctovano')
            ->setCode('col-zauctovano');
        
        if ($this->schvalovani) {
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval1', 'col-datum-schvaleni1', 'col-schvalil1','col-typ-polozky-kod','1',
                ronText('PoklK_schvaleni_uroven1' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven1');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval2', 'col-datum-schvaleni2', 'col-schvalil2','col-typ-polozky-kod', '2',
                ronText('PoklK_schvaleni_uroven2' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven2');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval3', 'col-datum-schvaleni3', 'col-schvalil3', 'col-typ-polozky-kod', '3',
                ronText('PoklK_schvaleni_uroven3' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven3');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval4', 'col-datum-schvaleni4', 'col-schvalil4', 'col-typ-polozky-kod', '4',
                ronText('PoklK_schvaleni_uroven4' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven4');
        } else {
            $this->addField(new SchvaleniZaznamuPKField($this ->database))->setCode('col-schvaleni');
        }

        $this->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(duvodNavstevy)}') ->setHeader(TextManager::_('duvod_navstevy') . ': ')
            ->setCode('col-duvodnavstevy')


            ->setSqlFieldName('PokladniKniha.PoznamkaKVraceni')
            ->setHeader('<strong>' . ronText('PoklK_zaznam_vracen') . '</strong>: ')
            ->setCode('col-poznamka-vraceni')
            ->setClassName('browserpoznamka') ->setHeaderColSpan(-1);

        $this->addField(new DokumentyPKField())->setCode('col-dokumenty');
        $this->addField(new ZruseniPKField($this->database, 'pkniha_vloz', 'col-typ-polozky-kod'))->setCode('col-zruseni');
    }

    public function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this ->database, ["menuid" => "pokladnikniha", "browser" => $this->seznamPolozekPokladniKnihy->cislo], true);
//        $this->filterForm->nastavOsoby(true, [$this->pravaProhlizet, $this->pravaVlozit, $this->pravaSchvalit1, $this->pravaSchvalit2, $this->pravaSchvalit3, $this->pravaSchvalit4], time(),time());
        $this->filterForm->addFilterPP(true, [$this->pravaProhlizet, $this->pravaVlozit, $this->pravaSchvalit1, $this->pravaSchvalit2, $this->pravaSchvalit3, $this->pravaSchvalit4]);
//        $this->filterForm->povolitVsechyOsoby = true;

        $this->filterForm->addFilter(new FilterFormFilterDatumOdDo($this->filterForm), true);

        $this->filterForm->addFilterRozdeleni1();
        $this->filterForm->addFilterRozdeleni2();

        $params = $this->database->createParameters();

        // filtr typ polozky
        $filtrTyp = new FilterFormFilter('typPolozkyPK', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('typ_polozky'))
            ->setRowId('filterrow_typpolozky')
            ->setSessionName('filterform_typ_polozky_poklK')
            ->setOptionsSql(
                new SqlStatement('SELECT Kod, Popis FROM TypPolozkyPokladniKnihy WHERE ' .
                    (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL) ? (SQLUtils::getINCondition($this->database,'Kod',$this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params)  . ' AND ') : '') .
                    ' %FILTER% ORDER BY %ORDER%', $params),
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('PokladniKniha.TypPolozky = ');
        $this ->filterForm ->addFilter($filtrTyp);

        $sqlFilterKeSchvaleni = 'PokladniKniha.Schvaleni4 IS NULL ';
        // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")

        if (!$this->pravaSchvalit4->jsouNejakePrava()) {
            $sqlFilterKeSchvaleni .= ' AND PokladniKniha.Schvaleni3 IS NULL ';
            // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje

            if (!$this->pravaSchvalit3->jsouNejakePrava()) {
                $sqlFilterKeSchvaleni .= ' AND PokladniKniha.Schvaleni2 IS NULL ';
                // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje

                if (!$this->pravaSchvalit2->jsouNejakePrava()) {
                    $sqlFilterKeSchvaleni .= ' AND PokladniKniha.Schvaleni1 IS NULL ';
                    // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
                }
            }
        }

        // filtr na schvaleni
        $filtrStav = new FilterFormFilter('urovenschvaleni');
        $filtrStav
            ->setCaption(TextManager::_('PoklK_uroven_schvaleni'))
            ->setSessionName('filterform_uroven_schvaleni_PoklK')
            ->addItem('0', TextManager::_('Vse'))
            ->addItem( (new FilterFormFilterItem('-1', TextManager::_('PoklK_ke_schvaleni')))
                ->setSqlFilter($sqlFilterKeSchvaleni)
            )
            ->addItem((new FilterFormFilterItem('-2', TextManager::_('PoklK_bez_schvaleni')))
                ->setSqlFilter('PokladniKniha.Schvaleni1 IS NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL  AND  PokladniKniha.Schvaleni4 IS NULL ')
            )
            ->addItem((new FilterFormFilterItem('1', TextManager::_('PoklK_schvaleni_uroven1')))
                ->setSqlFilter('PokladniKniha.Schvaleni1 IS NOT NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL AND PokladniKniha.Schvaleni4 IS NULL ')
            )
            ->addItem((new FilterFormFilterItem('2', TextManager::_('PoklK_schvaleni_uroven2')))
                ->setSqlFilter('PokladniKniha.Schvaleni2 IS NOT NULL  AND  PokladniKniha.Schvaleni3 IS NULL AND  PokladniKniha.Schvaleni4 IS NULL ')
            )
            ->addItem((new FilterFormFilterItem('3', TextManager::_('PoklK_schvaleni_uroven3')))
                ->setSqlFilter('PokladniKniha.Schvaleni3 IS NOT NULL AND PokladniKniha.Schvaleni4 IS NULL ')
            )
            ->addItem((new FilterFormFilterItem('4', TextManager::_('PoklK_schvaleni_uroven4')))
                ->setSqlFilter('PokladniKniha.Schvaleni4 IS NOT NULL ')
            )
            ->setDefaultValue('0');

        $this ->filterForm ->addFilter($filtrStav);
    }

    function renderFilter()
    {
        $this->filterForm->render();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava())
               $this->renderNewRecord();

            echo "<div id='poklk_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . ronText('PoklK_Schvalit_hromadne') . "'/></div>";
        }
    }

    function renderNewRecord()
    {
        echo "<div class='normaltext poklk_links'><a href='?menuid=pokladnikniha&browser=" . $this->seznamPolozekPokladniKnihy->cislo . "&formaction=new'>" . ronText('New_record') . "</a></div>\n";
    }

    function getSQLFilter()
    {
        $result = '(1 = 1 ) ';

        $params = $this->database->createParameters();

        if (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL)) {
            $result .= ' AND ' . SQLUtils::getINCondition($this->database, 'PokladniKniha.TypPolozky', $this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params);
        }

        $filter = $this->filterForm->buildSqlFilter($params);

        if ($filter) {
            $result .= ' AND ' . $filter;
        }

        $datumOd = DateUtils::today();
        $datumDo = $datumOd;

        if ($this->filterForm->getFilterValue('datumod')) {
            $datumOd = $this->filterForm ->getFilterValue('datumod');
            $datumDo = $this->filterForm ->getFilterValue('datumod', 'datumDo');

            $result .=
                ' AND PokladniKniha.DatumPohybu >= ' . $params->addTimestampParameter($datumOd) .
                ' AND PokladniKniha.DatumPohybu <= ' . $params->addTimestampParameter($datumDo);
        }

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $result .=
                " AND PokladniKniha.RC = " . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc')) .
                " AND PokladniKniha.OsobniCislo = " . $params->addParameter($this->filterForm->getFilterValue('osoba', 'osobniCislo')) .
                " AND PokladniKniha.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->filterForm->getFilterValue('osoba', 'datumNastupu')));
        } else {
            foreach ($this->filterForm->getRozdeleniFilters() as $rozdeleniFilter) {
                if ($rozdeleniFilter->getValue() !== null) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PokladniKniha',
                            $rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue(),
                            $datumOd, $datumDo,
                            $rozdeleniFilter->getValue('vcetnePodrizenych'),
                            $params);
                }
            }
        }

        return new SqlStatement($result, $params);
    }

    public function nactiPrava($suffix)
    {
        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' . $suffix);
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $suffix);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' . $suffix);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $suffix);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $suffix);
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4', 'pkniha_schval4_' . $suffix);
    }
    
    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function poklkEnableLine($values, $datumod, $datumdo)
    {
        $this ->jsonRowIndex = null;

        if ($this->filterForm->getFilterValue('urovenschvaleni') == -1) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values['col-datum-schvaleni4']) return false;
            // jiz je schvaleno nejvyssi urovni = neceka na nikoho

            if ($values['col-datum-schvaleni3']) return $this->pravaSchvalit4->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo);
            // je schvaleno 3, ceka na 4 => pokud mam pravo, ceka na me

            if ($values['col-datum-schvaleni2']) return $this->pravaSchvalit3->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo);
            // je schvaleno 2, ceka na 3 => pokud mam pravo, ceka na me

            if ($values['col-datum-schvaleni1']) return $this->pravaSchvalit2->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo);

            // je schvaleno 1, ceka na 2 => pokud mam pravo, ceka na me

            return $this->pravaSchvalit1->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo na 1, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;
            if ($this->pravaProhlizet->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;
        }

        if ($this->pravaSchvalit1->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit2->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit3->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit4->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo)) return true;

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $this->nactiPrava($values['col-typ-polozky-kod']);
        
        $datumod = $this ->database ->stringToTimestamp($values['col-datum-pohybu']);
        $datumdo = $this ->database ->stringToTimestamp($values['col-datum-pohybu']);

        $result = $this ->poklkEnableLine($values, $datumod, $datumdo);
        if (!$result) return false;

        if ($this ->schvalovani) {

            $this ->jsonRowIndex = count($this ->jsonObject['items']);

            $this ->hiddenFields .= "<input type='hidden' name='idPoklK_{$this ->jsonRowIndex}' value='{$values['col-id']}'/>";

            $this ->jsonObject['items'][] = [
                'id' => $values['col-id'],
                'pravo_1' => $this ->pravaSchvalit1 ->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo) ? 1 : 0,
                'pravo_2' => $this ->pravaSchvalit2 ->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo) ? 1 : 0,
                'pravo_3' => $this ->pravaSchvalit3 ->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo) ? 1 : 0,
                'pravo_4' => $this ->pravaSchvalit4 ->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo) ? 1 : 0,
                'schvaleno_1' => $values['col-datum-schvaleni1'] ? 1 : 0,
                'schvaleno_2' => $values['col-datum-schvaleni2'] ? 1 : 0,
                'schvaleno_3' => $values['col-datum-schvaleni3'] ? 1 : 0,
                'schvaleno_4' => $values['col-datum-schvaleni4'] ? 1 : 0
            ];

        } else {
            $this ->jsonRowIndex = null;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values['col-datum-schvaleni4']) $className .= ' poklk_schvalena4';
        else if ($values['col-datum-schvaleni3']) $className .= ' poklk_schvalena3';
        else if ($values['col-datum-schvaleni2']) $className .= ' poklk_schvalena2';
        else if ($values['col-datum-schvaleni1']) $className .= ' poklk_schvalena1';

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="poklk_row_' . $values['col-id'] . '"';

        return $result;
    }

    function renderContent()
    {
        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        $this ->hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniPoklK" action="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>' .
                Csrf::csrfHiddenInput();
        }

        echo
            '<ul class="popupmenu"  id="schvaleniPoklKMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . ronText('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . ronText('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . ronText('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->hiddenFields) echo $this ->hiddenFields;
        echo '<input type="hidden" name="pocet" value="' . count($this ->jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . ronText('Provest') . "' id='provestHromadneSchvaleniPoklK' />" .
                '<input type="button" class="button" value="' . ronText('Cancel') . '" id="stornoHromadneSchvaleniPoklK" 
                    data-browserlink="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '" />' .
                '</div>'.
                "</form>\n";

            /** @noinspection BadExpressionStatementJS */
            echo
                '<script type="text/javascript">' .
                'ron.PokladniKniha.hromadneSchvaleni.init(' . json_encode($this ->jsonObject) . ')' .
                '</script>';
        } else {
            $pravo = Prava::vratHodnotuPrava('pkniha_vloz');
            if ($pravo) $this->renderNewRecord();
        }
    }

}