<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 9:24
 */

namespace RonPortal\PokladniKniha;

require_once('ronportal/pokladnikniha/pokladniknihautils.php');

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DebugException;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\RozdeleniOsob;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use function Ron\ronText;

class PokladniKnihaController extends JsonController
{
    const POKLADNI_KNIHA_POVOLENA = 'povolitpokladniknihu';
    
    /** @var Database */
    private $database;

    private $cisloBrowseru;
    
    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;
    private $pravaSchvalit4;

    /**
     * PokladniKnihaController constructor.
     * @param $database
     * @throws Exception
     */
    public function __construct($database)
    {
        $this->database = $database; 
        TextManager::getInstance() ->loadStrings('pokladnikniha.ini');

        HtmlHeaders::getInstance() ->setTitle(ronText('RON_Portal') . ' - ' . ronText('pokladni_kniha'));
        HtmlHeaders::getInstance()->addScript('script/pokladnikniha.js');

        $this->cisloBrowseru =
            isset($_POST['browser']) ? (int)$_POST['browser'] :
            (isset($_GET['browser']) ? (int)$_GET['browser'] : (isset($_SESSION['pokladniKnihaCisloBrowseru']) ? $_SESSION['pokladniKnihaCisloBrowseru'] : 1));

        if (is_null($this->cisloBrowseru)) {
            $this->cisloBrowseru = 1;
        } else if (abs($this ->cisloBrowseru) > 32767) {
            // parametr mimo povoleny rozsah databazoveho pole
            throw new NeniPravoException();
        }

        /* 20.10.2021 kontrola prav bude nove mozna odlisit typem polozky misto browseru
        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' .$this->cisloBrowseru);
        ...
        */
        Prava::nastavObdobiProPrava(time(),time());
    }

    public static function zobrazitRozdeleni4($database)
    {
        if (!isset($_SESSION['pokladnikniha_rozdeleni4'])) {
            $_SESSION['pokladnikniha_rozdeleni4'] = RonPortalUtils::jeNainstalovanaAplikace($database, 'ADS') >= DateUtils::dateStringToTimestamp('27.1.2022');
        }
        return $_SESSION['pokladnikniha_rozdeleni4'];
    }

    public static function existujeDatumVraceni($database)
    {
        if (!isset($_SESSION['pokladnikniha_datumvraceni'])) {
            $_SESSION['pokladnikniha_datumvraceni'] = RonPortalUtils::jeNainstalovanaAplikace($database, 'ADS') >= DateUtils::dateStringToTimestamp('7.4.2022');
        }
        return $_SESSION['pokladnikniha_datumvraceni'];
    }

    /** Pokud ma jakekoliv pravo na zaznam, vrati true.
     * @param int $idZaznamu id zaznamu pokladni knihy
     * @param bool $vratZaznam pokud je true vrati i nacteny zaznam, jinak vrati jen true
     * @return bool|PokladniKniha pokud je pravo na zaznam PK vrati ho, jinak vypise chybu
     * @throws Exception
     */
    private function jePravoNaZaznam($idZaznamu, $vratZaznam = false)
    {
        $zaznam = PokladniKniha::nactiZaznam($this->database, $idZaznamu);

        if ($zaznam === false) {
            throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy nebyl nalezen.'));
        }

        /* 1.6.2016 kontrola prav se lisi jen browserem, ne typem polozky
        $suffixPrava = $this->cisloBrowseru != 1 ? ('_' . $this->cisloBrowseru) : '';

        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1','pkniha_schval1' . $suffixPrava . '_'.$zaznam->typPolozky);
        ;*/

        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' . $zaznam->typPolozkyKod);
        $this->pravaVlozit    = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $zaznam->typPolozkyKod);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' . $zaznam->typPolozkyKod);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $zaznam->typPolozkyKod);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $zaznam->typPolozkyKod);
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4', 'pkniha_schval4_' . $zaznam->typPolozkyKod);
        Prava::nastavObdobiProPrava($zaznam->datumPohybu, $zaznam->datumPohybu);

        if (!(
            $this->pravaProhlizet->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaVlozit->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit1->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit2->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit3->lzeZobrazitPP($zaznam->idPP) ||
            $this->pravaSchvalit4->lzeZobrazitPP($zaznam->idPP)
        )) {
            throw new NeniPravoException();
        }

        $this->cisloBrowseru = $zaznam->cisloBrowseru;

        if ($vratZaznam) {
            return $zaznam;
        }
        return true;
    }

    /**
     * @param int $idDokumentu
     * @param int $idZaznamu
     * @return DokumentOutputObject
     * @throws Exception
     * @throws DBException
     */
    private function downloadDokument($idDokumentu, $idZaznamu)
    {
        Csrf::validateCSRFGetParam('dokument_poklk_' . $idZaznamu);

        if (!$this->jePravoNaZaznam($idZaznamu)) {
            throw new NeniPravoException();
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT IDDokumentu FROM PokladniKniha JOIN DokumentPokladniKnihy ON (DokumentPokladniKnihy.IDPokladniKnihy = PokladniKniha.ID) " .
            " JOIN Dokument ON (Dokument.id = DokumentPokladniKnihy.ID) ".
            " JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.typ)" .
            "WHERE PokladniKniha.ID = " . $params->addParameter((int)$idZaznamu) .
            " AND Dokument.ID = " . $params->addParameter((int)$idDokumentu) .
            " AND (Dokument.zobrazitNaIntrawebu = 1 OR (Dokument.zobrazitNaIntrawebu = 2 AND TypDokumentu.zobrazitNaIntrawebu = 1))";

        $cursor = $this->database->getSqlResult($sql,$params);

        $row = $cursor->fetch();

        if (!$row) {
            throw new NeniPravoException();
        }

        return new DokumentOutputObject($this->database,$idDokumentu);
    }

    /** Nahraje dokument a priradi ho zaznamu pokladni knihy
     * @param JsonOutputObject $output
     */
    function uploadDokument($output)
    {
        try {
            // zkontroluje prava
            $idOsoby = (int)$_POST['idosoby'];
            $idPokladniKnihy = (int)$_POST['id'];

            $zaznam = $this->jePravoNaZaznam($idPokladniKnihy, true);

            $zaznam->lzeEditovat();

            if (!$this->pravaVlozit->lzeZobrazitPP($idOsoby)) {
                throw new NeniPravoException();
            }

            Csrf::validateCSRFPostParam('dokument_poklk_' . $idPokladniKnihy);

            $id = null;
            if (!is_uploaded_file($_FILES['soubor']['tmp_name'] ?? null)  &&  $_POST['pripojitdokument'] ?? null) {
                $id = (int)$_POST['pripojitdokument'];
            } else {
                DokumentyUtils::handleUploadError('soubor');
                DokumentyUtils::verifyFileType('soubor');
            }

            $cisloRadku = isset($_POST['CisloRadku']) ? (int)$_POST['CisloRadku'] : null;

            // Vlozi id dokumentu k zaznamu pokladni knihy
            $rec = new RonPortalDBRecord($this ->database, 'DokumentPokladniKnihy');
            $rec ->setXmlEvents(RonPortalXMLEvents::POKLK_DOKUMENT_BEFORE_POST, RonPortalXMLEvents::POKLK_DOKUMENT_AFTER_POST);
            $fId = $rec ->addField('ID', 'int', true, 0, $id);

            $rec->addField('IDPokladniKnihy', 'int', true, 0, $idPokladniKnihy);
            if ($cisloRadku) {
                $rec->addField('CisloRadku', 'int', false, 0, $cisloRadku);
            }

            // vola udalost BEFORE_POST jeste pred zapsanim dokumentu (napr. pro moznost zabraneni zapisu)
            $rec ->executeXmlEvents($rec ->beforePostEvent, 'insert', $rec ->eventsParameters);

            if (!isset($id)) {
                // vlozi do databaze obsah dokumentu
                $id = DokumentyUtils::vlozDokument($this->database, 'soubor', RonPortalSettings::getIntValue('poklk_typ_dokumentu'), $_POST['popis'], StringUtils::fromUTF8('Pokladní kniha'), true);
                $fId ->setNewValue($id);
            }

            $rec ->insertRecord();
        } catch (Exception $e) {
            $output->addError($e);
        }
    }

    /**
     * @param int $idZaznamu
     * @return ReportOutputObject
     * @throws DBException
     * @throws NeniPravoException
     */
    function report($idZaznamu)
    {
        $zaznam = $this->jePravoNaZaznam($idZaznamu, true);

        if (!($zaznam instanceof PokladniKniha)) {
            throw new Exception(StringUtils::fromUTF8('Chyba při načítání záznamu pokladní knihy'));
        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, ['PokladniKniha', "PokladniKniha_$zaznam->cisloBrowseru"]);

        $result->nastavPP(PracovniPomer::nactiPP($this->database, $zaznam->idPP));
        $result->nastavObdobi(3, $zaznam->datumPohybu, $zaznam->datumPohybu);

        if ($result ->report ->appReportName) {
            $result->integerParams['IdPokladniKnihy'] = $idZaznamu;
        }

        return $result;
    }

    /**
     * @param int $idZaznamu id zaznamu pokladni knihy
     * @param bool $vraceni
     * @return BaseViewObject
     * @throws Exception
     */
    private function zobrazDetailZaznamu($idZaznamu, $vraceni = false)
    {
        $zaznam = $this->jePravoNaZaznam($idZaznamu, true);

        if (!($zaznam instanceof PokladniKniha)) {
            throw new Exception(StringUtils::fromUTF8('Chyba při načítání záznamu pokladní knihy'));
        }

        $dokumentForm = new NovyDokumentPokladniKnihyForm($this->database, $idZaznamu, $zaznam->idPP);

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $zaznam->cisloBrowseru);

        Model::addData('pokladnikniha',$zaznam);
        Model::addData('vraceni',$vraceni);
        Model::addData('pravaVlozit', $this->pravaVlozit);
        Model::addData('pravaSchvalit1', $this->pravaSchvalit1);
        Model::addData('pravaSchvalit2', $this->pravaSchvalit2);
        Model::addData('pravaSchvalit3', $this->pravaSchvalit3);
        Model::addData('pravaSchvalit4', $this->pravaSchvalit4);
        Model::addData('dokumentForm', $dokumentForm);
        Model::addData('seznamPolozekPoklK', $seznamPolozekPokladniKnihy);

        $dokumentyBrowser = new DokumentyPokladniKnihyBrowser($this->database, $idZaznamu);
        $dokumentyBrowser->load();

        $knihaaction = $_GET['action'] ?? '';

        $radkyBrowser = new RadekPokladniKnihyBrowser($this->database, $zaznam, $seznamPolozekPokladniKnihy, $knihaaction);
        $radkyBrowser->load();

        $dialog = new RadekPokladniknihyDialog($this->database, $zaznam, $seznamPolozekPokladniKnihy);

        Model::addData('dokumentyBrowser', $dokumentyBrowser);
        Model::addData('radkyBrowser', $radkyBrowser);
        Model::addData('dialogRadku', $dialog);

        Model::addData('reports', Report::loadFormReports($this->database, ['PokladniKniha', "PokladniKniha_$zaznam->cisloBrowseru"]));

        return new BaseViewObject('ronportal/pokladnikniha/pokladnikniha_detail.php');
    }

    /**
     * @param $formAction
     * @return Redirect|PokladniKnihaForm
     * @throws Exception
     */
    private function pokladniKnihaFormular($formAction)
    {
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz');

        if (!$this->pravaVlozit->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $this->cisloBrowseru);
        
        $zaznam = null;
        
        if (($formAction == 'edit') || ($formAction == 'save')) {
            $idZaznamu = isset($_POST['record']) ? $_POST['record'] : $_GET['record'];

            $zaznam = PokladniKniha::nactiZaznam($this->database, $idZaznamu);

            if ($zaznam === false) {
                throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy nebyl nalezen.'));
            }

            if (!$zaznam->lzeEditovat()) {
                throw new NeniPravoException();
            }
        }

        $form = new PokladniKnihaForm($this->database, $formAction, $seznamPolozekPokladniKnihy);

        if ($form->handleAction()) {
            return $this->redirectToView($form->id, $this->cisloBrowseru);
        }

        return $form;
    }

    /**
     * @param int $id
     * @param int $uroven
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function schvaleni($id, $uroven)
    {
        Csrf::validateCSRFGetParam();

        $rec = new PokladniKnihaRecord($this ->database, $id);

        try {
            $rec ->schval($uroven);
        } catch (Exception $exception) {
//            return $this ->zobrazDetailZaznamu($id);
        }

        return $this ->redirectToView($id);
    }

    /**
     * @param int $id
     * @param int $uroven
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function zrusSchvaleni($id, $uroven)
    {
        Csrf::validateCSRFGetParam();

        $rec = new PokladniKnihaRecord($this ->database, $id);
        try {
            if ($rec->zrusSchvaleni((int)$uroven)) {
                return $this->redirectToView($id);
            } else {
                return $this->zobrazDetailZaznamu($id);
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
            return $this->zobrazDetailZaznamu($id);
        }
    }

    /**
     * @param int $id
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function vratSchvaleni($id)
    {
        Csrf::validateCSRFPostParam();

        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba - není zadán ID záznamu pokladní knihy!'));

        if (!isset($_POST['poznamkakvraceni']) || !$_POST['poznamkakvraceni']) {
            return $this ->zobrazDetailZaznamu($id, true);
        }

        $uroven = (int)$_POST['urovenvraceni'];
        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při vrácení - není zadána úroveň schválení!'));
            return $this ->zobrazDetailZaznamu($id, true);
        }

        $rec = new PokladniKnihaRecord($this ->database, $id);
        if ($rec ->vratSchvaleni($uroven, $_POST['poznamkakvraceni'])) {
            return $this ->redirectToView($id);
        } else {
            return $this ->zobrazDetailZaznamu($id);
        }
    }

    /** Nastavi/zrusi zauctovani zaznamu pokladni knihy
     * @param int $id id zaznamu pokladni knihy
     * @param $zauctovano
     * @return Redirect
     * @throws Exception
     */
    function nastavZauctovani($id, $zauctovano)
    {
        Csrf::validateCSRFGetParam();

        $rec = new PokladniKnihaRecord($this->database, $id);
        $rec->nastavZauctovani($zauctovano);

        return $this->redirectToView($id);
    }

    /** Smaze zaznam pokladni knihy, pokud neni schvalen.
     * @param int $id id zaznamu pokladni knihy
     * @return Redirect
     * @throws Exception
     */
    private function smazZaznam($id)
    {
        Csrf::validateCSRFGetParam();

        $zaznam = $this->jePravoNaZaznam($id, true);

        $browser = $zaznam->cisloBrowseru;

        try {
            $zaznam->lzeEditovat();

            $seznamDokumentuCursor = PokladniKniha::nactiPripojeneDokumenty($this->database, $id);

            while ($row = $seznamDokumentuCursor->fetch()) {
                try {
                    $this->smazDokument($row[1]);
                }
                catch (Exception $e) {
                    MessageManager::addErrorMessage($e);
                }
            }

            $rec = new PokladniKnihaRecord($this->database, $id);
            if ($rec->deleteRecord()) {
                MessageManager::addInfoMessage(ronText('PoklK_zruseni_ok'));
            } else {
                MessageManager::addDbErrorMessage($this->database, null, StringUtils::fromUTF8('Při rušení záznamu došlo k chybě!'));
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return new Redirect('','?menuid=pokladnikniha&browser=' . $browser);
    }

    /** Smaze dokument prirazeny k zaznamu pokladni knihy, pokud neni schvalena.
     * @param int $id id zaznamu pokladni knihy
     * @param int $idDokumentu
     * @return Redirect
     * @throws Exception
     */
    private function smazPripojenyDokument($id, $idDokumentu)
    {
        Csrf::validateCSRFGetParam('dokument_poklk_' . $id);

        $zaznam = $this->jePravoNaZaznam($id, true);

        try {
            $zaznam->lzeEditovat();

            $seznamDokumentuCursor = PokladniKniha::nactiPripojeneDokumenty($this->database, $id);

            $dokumentPripojen = false;

            while ($row = $seznamDokumentuCursor->fetch()) {
                if ($row[1] == $idDokumentu) {
                    $dokumentPripojen = true;

                    try {
                        if ($this->smazDokument($idDokumentu)) {
                            MessageManager::addInfoMessage(ronText('PoklK_zruseni_dokumentu_ok'));
                        }
                    }
                    catch (Exception $e) {
                        MessageManager::addErrorMessage($e);
                    }
                }
            }

            if (!$dokumentPripojen) {
                throw new Exception('PoklK_dokument_nepripojen');
            }

        } catch (Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToView($id);
    }

    /** Smaze dokument z databaze.
     * @param int $id
     * @return bool Vraci true, pokud byl smazan, false pokud nebyl nalezen a v pripade chyby vyhodi vyjimku
     * @throws Exception
     * @throws DBException
     */
    private function smazDokument($id)
    {
        $rec = new RonPortalDBRecord($this->database,'DokumentPokladniKnihy');

        $rec->setXmlEvents(RonPortalXMLEvents::POKLK_DOKUMENT_BEFORE_POST, RonPortalXMLEvents::POKLK_DOKUMENT_AFTER_POST);

        $rec->logDescription = StringUtils::fromUTF8('Pokladní kniha');

        $rec->addField('ID', 'int', true);
        $rec->addField('IDPokladniKnihy', 'int', true);

        $rec->where = 'ID = ' . (int)$id;
        if ($rec->deleteRecord()) {

            $recDokument = new RonPortalDBRecord($this->database, 'Dokument');
            $recDokument->logDescription = 'Pokladní kniha';

            $recDokument->addField('ID', 'int', true);

            $recDokument->where = 'ID = ' . (int)$id;

            return $recDokument->deleteRecord();
        }
        
        return false;
    }

    /** Smaze radek pokladni knihy
     * @param int $id ID pokladni knihy
     * @return Redirect
     * @throws Exception
     */
    private function smazRadekPokladniKnihy($id)
    {
        Csrf::validateCSRFGetParam();

        $zaznam = $this->jePravoNaZaznam($id, true);

        $zaznam->lzeEditovat();

        $rec = new RadekPokladniKnihyRecord($this->database, $id, (int)$_GET['radek'], $this);
        $rec->createFields((int)$_GET['radek']);

        try {
            $rec->deleteRecord();

            MessageManager::addInfoMessage(TextManager::_('PoklK_Zruseni_radku_OK'));

        } catch (Exception $exception) {
            throw new Exception($exception);
        }

        return $this->redirectToView($id, $zaznam->cisloBrowseru);
    }

    /**
     * @param int $id
     * @param int|null $cisloBrowseru
     * @return Redirect
     */
    private function redirectToView($id, $cisloBrowseru = null)
    {
        return new Redirect(null, '?menuid=pokladnikniha&action=zaznam&record=' . $id . ($cisloBrowseru ? '&browser=' . $cisloBrowseru : ''));
    }

    /**
     * Nacte data dialogu pro editaci radku
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function nactiRadek($output)
    {
        $zaznam = $this->jePravoNaZaznam((int)$_GET['idpokl'], true);

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $this->cisloBrowseru);

        //nemelo by nastavat (jen v pripade, ze uzivatel otevre jiny browser a znovu v jinem panelu nacte zaznam z jineho browseru)
        if (is_null($seznamPolozekPokladniKnihy->vratTypPolozky($zaznam->typPolozkyKod)))
            throw new Exception('Chyba - zvoleny zaznam pokladni knihy patri do jineho seznamu!');


        $f = new RadekPokladniknihyDialog($this ->database, $zaznam, $seznamPolozekPokladniKnihy);

        $output ->addData('data', $f ->getJsonData());
    }

    /**
     * Nacte data dialogu pro editaci radku
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function zapisRadek($output)
    {
        $zaznam = $this->jePravoNaZaznam((int)$_POST['IDPokladniKnihy'], true);

        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $zaznam->cisloBrowseru);

        //nemelo by nastavat (jen v pripade, ze uzivatel otevre jiny browser a znovu v jinem panelu nacte zaznam z jineho browseru)
        if (is_null($seznamPolozekPokladniKnihy->vratTypPolozky($zaznam->typPolozkyKod))) {
            throw new Exception('Chyba - zvoleny zaznam pokladni knihy patri do jineho seznamu!');
        }
        if (!$seznamPolozekPokladniKnihy->vratTypPolozky($zaznam->typPolozkyKod)->editovatRadky) {
            throw new Exception('Chyba - zvoleny zaznam pokladni knihy nema povolenou editaci radku!');
        }

        $f = new RadekPokladniknihyDialog($this ->database, $zaznam, $seznamPolozekPokladniKnihy);
        $f->utf8 = true;

        if (!$f ->handleAction()) {
            $output ->addMessages(MessageManager::getInstance());
        }
    }

    /**
     * Nacte data do formulare pokladni knihy (zatim seznam pridelenych rozdeleni, pokud je nastaveno, ze se ma nacist prirazene)
     * @param $output
     * @return void
     * @throws DBException
     * @throws NeniPravoException
     * @throws DebugException
     */
    private function nactiDataProOsobu($output)
    {
        Prava::nastavObdobiProPrava(time(), time());

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);

        if (!Prava::lzeZobrazitPP('pkniha_vloz', $pp ->idPP)) {
            throw new NeniPravoException();
        }

        for ($i = 1; $i <= 5; $i++) {
            $index = $i == 1 ? '' : $i;
            // nacte vychozi hodnotu pro rozdeleni
            if ((RonPortalSettings::getIntValue("typrozdelenipoklk$index")) && RonPortalSettings::getIntValue("rozdelenipoklk{$index}_vychozihodnota")) {

                $r = RozdeleniOsob::nactiRozdeleniOsoby($this->database,
                    $pp, RonPortalSettings::getIntValue("typrozdelenipoklk$index"),
                    time(), time(),
                    (RonPortalSettings::getIntValue("rozdelenipoklk{$index}_vychozihodnota") == 2) ? 'Hlavni = 1' : null
                );

                if ($r) {
                    $output->addData("rozdeleni$i", StringUtils::jsonText($r[0]->kod));
                }
            }
        }
    }

    /** Zpracovani dat pro dialog radku pokladni knihy
     * @param $action
     * @param $output
     * @return void
     * @throws Exception
     */
    public function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'radek' :
                $this->nactiRadek($output);
                break;

            case 'zapisradek' :
                $this->zapisRadek($output);
                break;
            case 'dataproosobu' :
                $this->nactiDataProOsobu($output);
                break;

            case 'dokumentupload' :
                $this ->uploadDokument($output);
                break;
        }
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|PokladniKnihaBrowser|PokladniKnihaForm|string
     * @throws Exception
     * @throws DBException
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(self::POKLADNI_KNIHA_POVOLENA)) {
            throw new Exception(ronText('Nemate_pravo'));
        }

        $knihaaction = isset($_GET['action']) ? $_GET['action'] : '';

        //js udalosti, musi byt pred form action, protoze dialog se odesila take s form action a nezpracovalo by se spravne ulozeni z dialogu
        if (($knihaaction == 'radek') || ($knihaaction == 'zapisradek') || ($knihaaction == 'dataproosobu') || ($knihaaction == 'dokumentupload')) {
            return parent::handleAction($knihaaction);
        }

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            // formular (editace, insert)
            return $this ->pokladniKnihaFormular($formaction);
        }

        if (isset($_POST['hromadneSchvaleni'])  &&  $_POST['hromadneSchvaleni']) {
            if (PokladniKnihaRecord::hromadneSchvaleni($this ->database))
                return new Redirect(null, '?menuid=pokladnikniha&browser=' . (int)$_GET['browser']);
        }

        switch ($knihaaction) {
            case 'zaznam':
                return $this->zobrazDetailZaznamu($_GET['record']);

            case 'zrusitradekpokladniknihy' :
                return $this->smazRadekPokladniKnihy((int)$_GET['record']);

            case 'dokumentdownload':
                return $this->downloadDokument((int)$_GET['record'], (int)$_GET['id']);

            case 'dokumentdelete' :
                return $this->smazPripojenyDokument((int)$_GET['id'], (int)$_GET['record']);
            
            case 'zrusitzaznam' :
                return $this->smazZaznam((int)$_GET['record']);

            case 'report' :
                return $this->report($_GET['record']);

            case 'schval1' :
                return $this ->schvaleni((int)$_GET['record'], 1);
            case 'schval2' :
                return $this ->schvaleni((int)$_GET['record'], 2);
            case 'schval3' :
                return $this ->schvaleni((int)$_GET['record'], 3);
            case 'schval4' :
                return $this ->schvaleni((int)$_GET['record'], 4);

            case 'zrusschvaleni1' :
                return $this ->zrusSchvaleni((int)$_GET['record'], 1);
            case 'zrusschvaleni2' :
                return $this ->zrusSchvaleni($_GET['record'], 2);
            case 'zrusschvaleni3' :
                return $this ->zrusSchvaleni($_GET['record'], 3);
            case 'zrusschvaleni4' :
                return $this ->zrusSchvaleni($_GET['record'], 4);

            case 'vrat' :
                return $this ->vratSchvaleni($_GET['record']);

            // zauctovani
            case 'zauctovat' :
                return $this ->nastavZauctovani($_GET['record'], 1);

            case 'zrusitzauctovani' :
                return $this ->nastavZauctovani($_GET['record'], 0);
            
            default:
                if (!isset($_GET['browser'])) {
                    throw new Exception(StringUtils::fromUTF8('Chyba v pokladní kniha controller: není zadáno číslo browseru pro zobrazení!'));
                }
                //pro spravnou funkcnost "vratit zpet" z detailu zaznamu (predevsim pri presmerovani po schvaleni atd.)
                $_SESSION['pokladniKnihaCisloBrowseru'] = (int)$_GET['browser'];

                $seznamPolozek = SeznamPolozekPokladniKnihy::vratBrowser($this->database, (int)$_GET['browser']);
                if (!$seznamPolozek->jePravoNaBrowser())
                    throw new NeniPravoException();

                //$browser = new PokladniKnihaBrowser($this->database, (int)$_GET['browser'], isset($_GET['schvalovani'])  &&  $_GET['schvalovani']);
                $browser = new PokladniKnihaBrowser($this->database, (int)$_GET['browser'], isset($_GET['schvalovani'])  &&  $_GET['schvalovani']);
                $browser->load();
                return $browser;
        }
    }

}