<?php

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RozdeleniBrowserField;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;
use function Ron\ronText;

class RadekPokladniKnihyBrowserField extends BrowserField
{
    /** @var PokladniKniha */
    public $pokladniKniha;

    public function __construct($pokladniKniha)
    {
        parent::__construct();
        $this ->pokladniKniha = $pokladniKniha;
    }

    public function lzeZobrazitRadek($rowData)
    {
        $pravo = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $rowData[RadekPokladniKnihyBrowser::TYP_POHYBU_INDEX]);
        if (!$pravo ->lzeZobrazitPP($this->pokladniKniha->idPP, $this->pokladniKniha->datumPohybu, $this->pokladniKniha->datumPohybu)) {
            return false;
        }

        return true;
    }

}

class EditaceRadkuPKField extends RadekPokladniKnihyBrowserField
{
    /**
     * @param PokladniKniha $pokladniKniha
     */
    function __construct($pokladniKniha)
    {
        parent::__construct($pokladniKniha);
        $this ->code = 'col-edit';
        $this ->align = 'left';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->lzeZobrazitRadek($rowData)) {
            return '';
        }

        return
            '<i class="inline-glyph glyph-button fa glyph-edit open-dialog"' .
            ' data-dialog="#radek_poklk_dialog" data-ronform-url="?menuid=pokladnikniha&action=radek&idpokl=' . $this->pokladniKniha->vratID() . '&record="' .
            ' data-ronform-record="' . $rowData[RadekPokladniKnihyBrowser::CISLO_RADKU_INDEX] . '"' . // id radku
            ' title="' . ronText('Editovat_zaznam') . '">';
    }
}

class KopieRadkuPKField extends RadekPokladniKnihyBrowserField
{
    /**
     * @param PokladniKniha $pokladniKniha
     */
    function __construct($pokladniKniha)
    {
        parent::__construct($pokladniKniha);
        $this ->code = 'col-copy';
        $this ->align = 'left';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->lzeZobrazitRadek($rowData)) {
            return '';
        }

        return
            '<i class="inline-glyph glyph-button fa glyph-copy open-dialog"' .
            ' data-dialog="#radek_poklk_dialog" data-ronform-url="?menuid=pokladnikniha&action=radek&idpokl=' . $this->pokladniKniha->vratID() . '&record=' . $rowData[RadekPokladniKnihyBrowser::CISLO_RADKU_INDEX] . '"' .
            ' title="' . ronText('Kopirovat_zaznam') . '">';
    }
}


class ZruseniRadkuPKField extends RadekPokladniKnihyBrowserField
{
    /**
     * @param PokladniKniha $pokladniKniha
     */
    function __construct($pokladniKniha)
    {
        parent::__construct($pokladniKniha);
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this->lzeZobrazitRadek($rowData)) {
            return '';
        }

        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
            TextManager::_('PoklK_Opravdu_zrusit_radek'),
            "?menuid=pokladnikniha&action=zrusitradekpokladniknihy&record=" . $this->pokladniKniha->vratID() . "&radek=" . $rowData[RadekPokladniKnihyBrowser::CISLO_RADKU_INDEX] . Csrf::csrfUrlParameter(),
            'data-ron-send-position="1"');
    }
}


class DokumentyRadkuPKField extends BrowserField
{
    var $lzeEditovat;

    /**
     * @param Database $database
     * @param bool $lzeEditovat
     */
    function __construct($database, $lzeEditovat)
    {
        parent::__construct($database);
        $this->code = 'col-dokumenty';
        $this->align = 'center';

        $this ->lzeEditovat = $lzeEditovat;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT DokumentPokladniKnihy.Id, Dokument.Popis' .
            '  FROM DokumentPokladniKnihy JOIN Dokument ON Dokument.ID = DokumentPokladniKnihy.ID' .
            ' WHERE IDPokladniKnihy = ' . $params->addParameter($rowData[RadekPokladniKnihyBrowser::ID_POKLADNI_KNIHY_INDEX]) .
            '   AND CisloRadku = ' . $params->addParameter($rowData[RadekPokladniKnihyBrowser::CISLO_RADKU_INDEX]);

        $result = '';

        $cursor = $this->database->getSqlResult($sql, $params);
        while ($row = $cursor->fetch()) {
            $result .=
                '<a target="_blank" href="?menuid=pokladnikniha&action=dokumentdownload&id=' . (int)$rowData[RadekPokladniKnihyBrowser::ID_POKLADNI_KNIHY_INDEX] . '&record=' . $row[0] .
                    Csrf::csrfUrlParameter('dokument_poklk_' . $rowData[RadekPokladniKnihyBrowser::ID_POKLADNI_KNIHY_INDEX]) . '">' .
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . StringUtils::htmlTextConvert($row[1]) . '"></i>' .
                '</a>';

        }
        $cursor ->close();

        if ($this ->lzeEditovat) {
            $result .=
                '<a href="#" title="' . TextManager::_('Vlozit_novy_dokument') . '">' .
                '<i class="inline-glyph fa glyph-upload show-dokument-dialog" data-fields="CisloRadku" data-field-cisloradku="' . $rowData[RadekPokladniKnihyBrowser::CISLO_RADKU_INDEX] . '"></i>' .
                '</a>';
        }

        return $result;
    }
}

class RadekPokladniKnihyBrowser extends RonPortalBrowser
{
    const CISLO_RADKU_INDEX = 0;
    const TYP_POHYBU_INDEX = 5;
    const ID_POKLADNI_KNIHY_INDEX = 19;

    public $zobrazitNovyRadek;

    public $pokladniKniha;
    public $idPokladniKnihy;
    public $datum;
    public $editovatRadek;
    public $kopirovatRadek;

    public $formaction;

    public $seznamPolozekPokladniKnihy;

    /**
     * @param Database $database
     * @param PokladniKniha $pokladniKniha
     * @param SeznamPolozekPokladniKnihy $seznamPolozek
     * @param string $akce
     * @throws Exception
     */
    public function __construct($database, PokladniKniha $pokladniKniha, SeznamPolozekPokladniKnihy $seznamPolozek, string $akce)
    {
        parent::__construct($database, 'pokladnikniha');

        switch ($akce) {
            case 'editovatradekpokladniknihy' :
                $this->formaction = 'ulozradekpokladniknihy';
                if (isset($_POST['CisloRadku'])) $this->editovatRadek = $_POST['CisloRadku'];
                else $this->editovatRadek = $_GET['radek'] ?? null;
                break;

            case 'kopirovatradekpokladniknihy' :
                $this->formaction = 'vlozradekpokladniknihy';
                $this->kopirovatRadek = $_GET['radek'] ?? null;
                break;


            case 'pridatradek' :
                $this->formaction = 'vlozradekpokladniknihy';
                $this->zobrazitNovyRadek = true;
                break;
        }

        $this->pokladniKniha = $pokladniKniha;
        $this->idPokladniKnihy = $pokladniKniha->vratID();
        $this->seznamPolozekPokladniKnihy = $seznamPolozek;

        $this->code = 'browserradekpokladniknihy' . ($seznamPolozek->cislo != 1 ? "_$seznamPolozek->cislo" : '');
        $this->tableId = 'radekpokladniknihy';

        $this->showMessages = false;

        $this->sqlSelectPart =
            'RadekPokladniKnihy.CisloRadku, RadekPokladniKnihy.DatumPohybu, RadekPokladniKnihy.Castka, RadekPokladniKnihy.CastkaBezDPH, RadekPokladniKnihy.CastkaDPH, ' . //0 - 4
            'RadekPokladniKnihy.TypPolozky, RadekPokladniKnihy.Popis, RadekPokladniKnihy.Poznamka, ZpusobPlatbyPK.Popis, TypPolozkyPokladniKnihy.Popis, ' . //5 - 9
            'RadekPokladniKnihy.KodTypuRozdeleni, RadekPokladniKnihy.Rozdeleni, RadekPokladniKnihy.KodTypuRozdeleni2, RadekPokladniKnihy.Rozdeleni2, ' .  //10 - 13
            'Rozdeleni.Popis, R2.Popis, RadekPokladniKnihy.Rozdeleni3, R3.Popis, RadekPokladniKnihy.ZpusobPlatby, RadekPokladniKnihy.IDPokladniKnihy'; // 14 - 19

        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
            $this->sqlSelectPart .=
                ' , R4.Kod, R4.Popis '; // 20 - 21
        }

        $this->sqlFromPart =
            'RadekPokladniKnihy  ' .
            'JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = RadekPokladniKnihy.TypPolozky) ' .
            'JOIN ZpusobPlatbyPK ON (ZpusobPlatbyPK.Kod = RadekPokladniKnihy.ZpusobPlatby) ' .
            'LEFT OUTER JOIN Rozdeleni ON Rozdeleni.KodTypuRozdeleni = RadekPokladniKnihy.KodTypuRozdeleni AND Rozdeleni.Kod = RadekPokladniKnihy.Rozdeleni ' .
            'LEFT OUTER JOIN Rozdeleni R2 ON R2.KodTypuRozdeleni = RadekPokladniKnihy.KodTypuRozdeleni2 AND R2.Kod = RadekPokladniKnihy.Rozdeleni2 ' .
            'LEFT OUTER JOIN Rozdeleni R3 ON R3.KodTypuRozdeleni = RadekPokladniKnihy.KodTypuRozdeleni3 AND R3.Kod = RadekPokladniKnihy.Rozdeleni3 ';

        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
            $this->sqlFromPart .=
                'LEFT OUTER JOIN Rozdeleni R4 ON R4.KodTypuRozdeleni = RadekPokladniKnihy.KodTypuRozdeleni4 AND R4.Kod = RadekPokladniKnihy.Rozdeleni4 ';
        }

        $this->sqlWherePart =
            'RadekPokladniKnihy.IDPokladniKnihy = ' . $this->idPokladniKnihy;

        $this->primaryField = 0;

        $this->addField()
            ->setHeaderText('datum_pohybu')->setSqlFieldIndex(1)->setSqlFieldName('RadekPokladniKnihy.DatumPohybu')
            ->setFormat('date')->setAlign('right')->setCode('col-datum-pohybu');

        $this->addField()
            ->setHeaderText('typ_polozky')->setSqlFieldIndex(9)->setSqlFieldName('TypPolozkyPokladniKnihy.Popis')
            ->setAlign('left')->setCode('col-typ-polozky')->setTranslationPrefix('db_typpolozkypokladniknihy_popis_', 5);

        $this->addField()
            ->setHeaderText('popis')->setSqlFieldIndex(6)->setSqlFieldName('RadekPokladniKnihy.Popis')
            ->setAlign('left')->setCode('col-popis');

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            $this->addField(new RozdeleniBrowserField($this->database, 11, 14))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni')
                ->setSqlFieldName('RadekPokladniKnihy.Rozdeleni')->setOrdering(false);
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            $this->addField(new RozdeleniBrowserField($this->database, 13, 15, 'col-rozdeleni2'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni2')
                ->setSqlFieldName('RadekPokladniKnihy.Rozdeleni2')->setOrdering(false);
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            $this->addField(new RozdeleniBrowserField($this->database, 16, 17, 'col-rozdeleni3'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni3')
                ->setSqlFieldName('RadekPokladniKnihy.Rozdeleni3')->setOrdering(false);
        }
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            $this->addField(new RozdeleniBrowserField($this->database, 20, 21, 'col-rozdeleni4'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni4')
                ->setSqlFieldName('RadekPokladniKnihy.Rozdeleni4')->setOrdering(false);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField()
                ->setHeaderText('castkaBezDPH')->setSqlFieldIndex(3)->setSqlFieldName('RadekPokladniKnihy.CastkaBezDPH')
                ->setFormat('number2')->setAlign('right')->setCode('col-castka-bez-dph');
        }
        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
        $this->addField()
            ->setHeaderText('castkaDPH')->setSqlFieldIndex(4)->setSqlFieldName('RadekPokladniKnihy.CastkaDPH')
            ->setFormat('number2')->setAlign('right')->setCode('col-castka-dph');
        }
        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastku()) {
            $this->addField()
                ->setHeaderText('castka')->setSqlFieldIndex(2)->setSqlFieldName('RadekPokladniKnihy.Castka')
                ->setFormat('number2')->setAlign('right')->setCode('col-castka');
        }

        $this->addField()
            ->setHeaderText('PoklK_zpusobplatby')->setSqlFieldIndex(8)->setSqlFieldName('ZpusobPlatbyPK.Popis')
            ->setAlign('left')->setCode('col-zpusob-platby')->setTranslationPrefix('db_zpusobplatbypk_popis_', 18);

        $this->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('RadekPokladniKnihy.Poznamka')
            ->setCode('col-poznamka')
            ->setClassName('poznamka')
            ->setHeaderColSpan(-1);

        if ($this->pokladniKniha->lzeEditovat(false) &&
            $akce != 'editovatradekpokladniknihy' &&
            $akce != 'kopirovatradekpokladniknihy' &&
            $akce != 'pridatradek' &&
            $akce != 'ulozuctovanisluzebky'
        ) {
            $this ->addField(new EditaceRadkuPKField($this->pokladniKniha));
            $this ->addField(new KopieRadkuPKField($this->pokladniKniha));
            $this ->addField(new ZruseniRadkuPKField($this->pokladniKniha));

        }

        $this ->addField(new DokumentyRadkuPKField($database, $this->pokladniKniha->lzeEditovat(false)));

        // schvaleni
        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText('<i class="inline-glyph fa glyph-check" title="' . TextManager::_('PoklK_radek_zkontrolovan') . '">')
            ->setCode('col-zkontrolovano')
            ->setFilterValue(true)
            ->setSqlFieldName('Zkontrolovano');
    }

    function setFilterForm()
    {
        $this->filterForm = null;
    }

    public function render()
    {
        parent::render();

        if ($this->pokladniKniha->lzeEditovat(false)) {
            echo
                "<div class='text_link' id='radekPoklKBrowserLinks'><ul class='linkslist'>\n" .
                "<li><a href='#' class='open-dialog' data-dialog='#radek_poklk_dialog'>" . ronText('Novy_radek') . "</a></li>\n" .
                "</ul></div>\n";
        }
    }
}