<?php
/**
 * stanislav.elbl, 09.06.2020
 * Formular pro zmenu velikosti pomucek zamestnance
 */

namespace RonPortal\Pomucky;


use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\SQLUtils;
use Ron\MVC\BaseViewObject;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;


class VelikostiPomucekItem
{
    public $kodTypuVelikosti;
    public $popisTypuVelikosti;

    /** @var Option[] */
    public $seznamVelikosti = [];

    /** @var string[]   Vybrane hodnoty - muze jich byt i vice */
    public $aktualniHodnoty = [];
}



class VelikostiPomucekForm extends BaseViewObject
{
    /** @var Database */
    public $database;

    /** @var PracovniPomer */
    public $pp;

    /** @var VelikostiPomucekItem[] */
    public $items = [];

    /**
     * VelikostiPomucekForm constructor.
     * @param Database $database
     * @param PracovniPomer $pp
     */
    public function __construct($database, $pp)
    {
        parent::__construct('ronportal/pomucky/view_velikosti_pomucek_form.php');

        $this->database = $database;
        $this->pp = $pp;
    }


    /**
     * nacte seznam typu velikosti vcetne moznych hodnot
     * @throws \Ron\DB\DBException
     */
    private function nactiVelikosti()
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT TypVelikosti.Kod, TypVelikosti.Popis, VelikostPPomucky.Velikost' .
            ' FROM TypVelikosti, VelikostPPomucky' .
            ' WHERE VelikostPPomucky.TypVelikosti = TypVelikosti.Kod' .
            ' ORDER BY TypVelikosti.Popis, VelikostPPomucky.Velikost';

        $cursor = $this->database->getSqlResult($sql, $params);
        /** @var VelikostiPomucekItem $lastItem */
        $lastItem = null;
        while ($row = $cursor->fetch()) {
            if (!$lastItem || $lastItem->kodTypuVelikosti != $row[0]) {
                $lastItem = new VelikostiPomucekItem();
                $lastItem->kodTypuVelikosti = $row[0];
                $lastItem->popisTypuVelikosti = $row[1];

                $lastItem->seznamVelikosti = [
                    new Option(null, \Ron\ronText('nevybrano'))
                ];

                $this->items[] = $lastItem;
            }

            $lastItem->seznamVelikosti[] = new Option($row[2], $row[2]);
        }
        $cursor->close();
    }


    /**
     * Nastavi aktualniHodnotu v seznamu velikosti na zadanou $velikost
     * @param $kod
     * @param $velikost
     */
    private function nastavVelikostOsoby($kod, $velikost)
    {
        foreach ($this->items as $item) {
            if ($item->kodTypuVelikosti == $kod) {
                $item->aktualniHodnoty[] = $velikost;
                return;
            }
        }
    }


    private function nactiVelikostiOsoby()
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT VelikostPPOsoby.TypVelikosti, VelikostPPOsoby.Velikost' .
            ' FROM VelikostPPOsoby' .
            ' WHERE RCOsoby = ' . $params->addParameter($this->pp->rc);

        $cursor = $this->database->getSqlResult($sql, $params);
        while ($row = $cursor->fetch()) {
            $this->nastavVelikostOsoby($row[0], $row[1]);
        }
        $cursor->close();
    }


    public function load()
    {
        $this->nactiVelikosti();
        $this->nactiVelikostiOsoby();
    }


    private function zapisNahlaseneZmeny($nahlaseneHodnoty)
    {
        $rec = new RonPortalDBRecord($this ->database, 'NahlasenaZmena', $this);
        $rec ->addStringField('TypZmeny', false, 20, 'velikostpomucek');
        $rec ->addStringField('RC', false, 15, $this ->pp->rc);
        $rec ->addTimestampField('DatumCas', false, time());
        $rec ->addField('Hodnoty', 'memo', false, 0, $nahlaseneHodnoty);
        $rec ->addStringField('Nahlasil', false, 15, User::getRC());
        $rec ->addIntField('Stav', false, 1);   // zmena se zapise rovnou jako zapsana
        $rec ->addTimestampField('DatumCasZpracovani', false, time());
        $rec ->addStringField('Zpracoval', false, 15, User::getRC());
        $rec ->addField('Poznamka', 'memo', false, 0, \Ron\ronText('zmena_velikosti_pomucky_uzivatelem'));

        $id = SQLUtils::newSerialNumber($this ->database, 'NahlasenaZmena', 'Id');
        $rec ->addIntField('Id', false, $id);

        $rec ->insertRecord();
    }


    private function updateRec($item, $puvodniHodnota, $novaHodnota)
    {
        if (!$puvodniHodnota  &&  !$novaHodnota) {
            return;
        }

        $rec = new RonPortalDBRecord($this ->database, 'VelikostPPOsoby', $this);
        $rec ->addStringField('RCOsoby', true, 15, $this ->pp ->rc);
        $rec ->addStringField('TypVelikosti', true, 10, $item ->kodTypuVelikosti);
        $rec ->addStringField('Velikost', false, 20, $novaHodnota);

        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'RCOsoby = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND TypVelikosti = ' . $params ->addParameter($item ->kodTypuVelikosti) .
            ' AND Velikost = ' . $params ->addParameter($puvodniHodnota ?: $novaHodnota),
            $params);

        if ($novaHodnota != '') {
            $rec ->updateOrInsert();
        } else {
            if ($rec ->loadRecord()) {
                $rec ->deleteRecord();
            }
        }
    }

    public function process()
    {
        $textZmeny = '';

        foreach ($this ->items as $item) {
            $noveHodnoty = [];

            foreach ($item ->aktualniHodnoty as $hodnota) {
                // zkontroluje, jestli doslo k editaci hodnoty
                $htmlName = 'velikost_' . str_replace(' ', '_', $item ->kodTypuVelikosti) . '_' . str_replace(' ', '_', $hodnota);
                if (isset($_POST[$htmlName])) {
                    $novaHodnota = $_POST[$htmlName];

                    if ($novaHodnota != $hodnota) {
                        // doslo k editaci existujici hodnoty
                        $this ->updateRec($item, $hodnota, $novaHodnota);

                        $textZmeny .=
                            ($textZmeny ? ', ' : '') .
                            $item ->popisTypuVelikosti . ' (' . $item ->kodTypuVelikosti . ') ' .
                            \Ron\ronText('puvodni_velikost') . ': ' . $hodnota . ', ' .
                            \Ron\ronText('nova_velikost') . ': ' . $novaHodnota;

                        // doslo ke zmene
//                    $this ->zapisNahlasenouZmenu($item);
                    }

                    $noveHodnoty[] = $novaHodnota;
                }
            }

            // a jeste doplni pripadnou nove zadanou hodnotu
            $htmlName = 'velikost_' . str_replace(' ', '_', $item ->kodTypuVelikosti);

            if (isset($_POST[$htmlName])) {
                $novaHodnota = $_POST[$htmlName];
                if ($novaHodnota != '') {
                    $this ->updateRec($item, null, $_POST[$htmlName]);

                    $textZmeny .=
                        ($textZmeny ? ', ' : '') .
                        $item ->popisTypuVelikosti . ' (' . $item ->kodTypuVelikosti . ') ' .
                        \Ron\ronText('nova_velikost') . ': ' . $novaHodnota;

                    $noveHodnoty[] = $novaHodnota;

                    // doslo ke zmene
//                    $this ->zapisNahlasenouZmenu($item);
                }
            }

            $item ->aktualniHodnoty = $noveHodnoty;
        }

        if ($textZmeny) {
            $this->zapisNahlaseneZmeny(\Ron\ronText('zmena_velikosti') . ' ' . $textZmeny);
        }
    }
}
