<?php
/**
 * User: stanislav.elbl
 * Date: 12.05.2021
 */

namespace RonPortal\Pomucky;

use Ron\Controls\FormField;
use Ron\Controls\FormFieldMessage;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\UserLoginFormField;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;


/**
 * Formular pro hromadny vydej narokovanych pomucek. Nejdrive uzivatel vybere zamestnance, pak se zobrazi
 * seznam naroku a uzivatel zaklikava, ktere vydal
 */
class VydejNarokuForm extends RonPortalForm
{
    /** @var PracovniPomer */
    private $pp;


    /** @var array      Seznam nevydanych naroku */
    private $naroky;

    /**
     * @param Database $database
     * @param string $formAction
     */
    function __construct($database, $formAction = 'new')
    {
        parent::__construct($database, $formAction);

        $this ->title = TextManager::_('Hromadny_vydej_naroku_pomucek');
        $this ->code = 'formvydejpomucek';

        $this ->stornoAction = '?menuid=pracovnipomucky';

        $idpp = (int)($_POST['osoba'] ?? $_GET['osoba'] ?? 0);

        if ($idpp) {
            try {
                $this->pp = PracovniPomer::nactiPP($this->database, $idpp);

                $this ->nactiNaroky();
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }

        if ($this ->pp  &&  $this ->naroky) {
            // uz je zadan PP (v GET nebo v predchozim POST)
            // vlozi id PP
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*osoba') ->setDefaultValue($this ->pp ->idPP)
                ->setCode('field-jmenoosoby');

            // zobrazi jmeno osoby
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCaption('Osoba')
                ->setCode('field-jmenoosoby')
                ->setText($this ->pp ->jmenoPrijmeni);

            $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                ->setCaption('nevydane_pomucky');

            foreach ($this ->naroky as $narok) {
                $this ->addField(FormField::FIELD_TYPE_CHECK)
                    ->setFieldName('*' . $this ->jmenoPoleProZaznam($narok))
                    ->setCode('field-pomucka_' . $narok['kod'])
                    ->setCaption(StringUtils::htmlTextConvert($narok['popis']))
                    ->setCaption2($narok['velikost']
                        ? \Ron\ronText('doporucena_velikost') . ': ' . StringUtils::htmlTextConvert($narok['velikost'])
                        : '');
            }

            $this ->addField(FormField::FIELD_TYPE_ACTUAL_DATE)
                ->setFieldName('DatumVydani')
                ->setCode('field-datumvydani');

            $this ->addLogFields(false, true);
//            // logovaci pole
//            $this ->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
//                ->setFieldName('Update_TimeStamp') ->setUpdateLog(true)
//                ->setCode('field-update_timestamp');
//
//            $this ->addField(new UserLoginFormField(false))
//                ->setCode('field-update_username');
        } else {
            // osoba neni zadana v URL ani v POST => nejdrive zobrazi vyber osoby
            $this ->addField(new OsobaFormField('pomucky_vydej', time(), $this ->pp))
                ->setCode('field-osoba')
                ->setCaption('Osoba');

            if ($this ->pp && !$this ->naroky) {
                $this ->addField(new FormFieldMessage())
                    ->setText( TextManager::_('Zadna_pomucka_k_vydani'));
            }
        }


    }


    protected function jmenoPoleProZaznam($narok)
    {
        return StringUtils::htmlTextConvert($narok['kod']) . '#' . $narok['datumNaroku'] . '#' . $narok['poradi'];
    }


    public function nactiNaroky()
    {
        $this ->naroky = [];

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PrirazenaPracPomucka.PracovniPomucka, PrirazenaPracPomucka.DatumNaroku, PrirazenaPracPomucka.Poradi,' .
            '  PracovniPomucka.Popis, MAX(VelikostPPOsoby.Velikost), TypPomucky.Kod' .
            ' FROM PrirazenaPracPomucka INNER JOIN PracovniPomucka' .
            '  ON PracovniPomucka.Kod = PrirazenaPracPomucka.PracovniPomucka' .
            ' INNER JOIN TypPomucky' .
            '  ON TypPomucky.Kod = PracovniPomucka.TypPomucky' .
            ' LEFT OUTER JOIN VelikostPPOsoby' .
            '  ON VelikostPPOsoby.RCOsoby = PrirazenaPracPomucka.RC' .
            '  AND VelikostPPOsoby.TypVelikosti = TypPomucky.TypVelikosti' .
            ' WHERE PrirazenaPracPomucka.DatumVydani IS NULL' .
            '  AND PrirazenaPracPomucka.RC = ' . $params ->addParameter($this ->pp ->rc) .
            '  AND PrirazenaPracPomucka.NarokNerealizovat = 0' .
            ' GROUP BY PrirazenaPracPomucka.PracovniPomucka, PrirazenaPracPomucka.DatumNaroku, PrirazenaPracPomucka.Poradi,' .
            '  PracovniPomucka.Popis, TypPomucky.Kod' .
            ' ORDER BY PracovniPomucka.Popis';

        $pravo = Prava::vratPrava('pomucky_vydej');

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            if ($pravo ->priznak3ObsahujeKod($row[5])) { // kontrola, jestli uzivatel muze vydavat tento typ pomucky
                $this->naroky[] =
                    [
                        'kod' => $row[0],
                        'datumNaroku' => $this->database->stringToTimestamp($row[1]),
                        'poradi' => (int)$row[2],
                        'popis' => $row[3],
                        'velikost' => $row[4]
                    ];
            }
        }
    }

    public function renderCommonFields()
    {
        if ($this ->pp) {
            echo '<input type="hidden" name="vydejpomucek" value="1" />';
        }

        parent::renderCommonFields();
    }

    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        // pokud se zobrazuje jen vyber osoby, nastavi text tlacitka pro odeslani na "Vybrat"
        if (!$this ->pp  &&  isset($result['submit'])) {
            $result['submit']['value'] = TextManager::_('Vybrat');
        }

        return $result;
    }


    public function editRecord()
    {
        return false; // formular se nepouziva pro editaci
    }


    /**
     * Zapise vydej jedne pomucky. $narok musi obsahovat pole se zaznamem nactene metodou nactiNaroky()
     * @param array $narok
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    private function zapisZaznam($narok)
    {
        $nazevPole = $this ->jmenoPoleProZaznam($narok);
        if (!isset($_POST[$nazevPole]) || !(int)$_POST[$nazevPole]) {
            return;
        }

        $rec = new RonPortalDBRecord($this ->database, 'PrirazenaPracPomucka', $this);

        $rec->setXmlEvents('PRACOVNI_POMUCKY_BEFORE_POST', 'PRACOVNI_POMUCKY_AFTER_POST');

        $rec ->addStringField('RC', true);
        $rec ->addStringField('OsobniCislo', true);
        $rec ->addDateField('DatumNastupu', true);
        $rec ->addLogText('   ' . $this ->pp ->jmenoPrijmeni);
        $rec ->addStringField('PracovniPomucka', true);
        $rec ->addDateField('DatumNaroku', true);
        $rec ->addIntField('Poradi', true);

        $this ->createDBRecordFields($rec, true);

        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'RC = ' . $params ->addParameter($this ->pp ->rc) .
            ' AND PracovniPomucka = ' . $params ->addParameter($narok['kod']) .
            ' AND DatumNaroku = ' . $params ->addTimestampParameter($narok['datumNaroku']) .
            ' AND Poradi = ' . $params ->addParameter((int)$narok['poradi']),
            $params
        );

        if ($rec ->loadRecord()) {
            $rec ->updateRecord();
        };
    }


    public function insertRecord()
    {
        if (!$this ->pp) {
            return false;
        }

        if (!isset($_POST['vydejpomucek'])) {
            return false;
        }

        foreach ($this ->naroky as $narok) {
            $this ->zapisZaznam($narok);
        }

        return true;
    }
}
