<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;

class PredavaciProcesyUtils
{
    /**
     * vraci true, pokud je aktualni uzivatel v nejakem predavacim procesu
     *     X hodnota se nacita z databaze jen poprve, pak se nacita ze session
     * @param Database $database
     * @return bool
     */
    public static function jeVNejakemProcesu($database)
    {
        //if (isset($_SESSION['jevpredavacimprocesu'])) return $_SESSION['jevpredavacimprocesu'];

        $params = $database->createParameters();

        if (User::jeZastup()) {
            // Uzivatel zastupuje jineho uzivatele - jen procesy, ktere mu predal
            $sql =
                'SELECT DISTINCT PoradiPredavani.Poradi' .
                ' FROM RozdeleniOsob, PMProUrovne, PoradiPredavani, ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele' .
                // PM zastupovaneho uzivatele
                ' WHERE RozdeleniOsob.RC = ' . $params ->addParameter(User::getRC()) .
                '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                '  AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni' .
                '  AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni' .
                // ale jen pro procesy, ktere byly predany zastupujicimu uzivateli
                '  AND PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne' .
                '  AND PoradiPredavani.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';

        } else {
            $sql =
                'SELECT DISTINCT PoradiPredavani.Poradi ' .
                ' FROM PoradiPredavani, PMProUrovne, RozdeleniOsob  ' .
                ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($database->dateToParam(time())) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($database->dateToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
                '   AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni ' .
                '   AND PoradiPredavani.KodUrovne = PMProUrovne.KodUrovne';
        }

        try {
            $cursor = $database->select($sql, $params);
        } catch (\Exception $e) {
            return false;
        }

        $row = $cursor ->fetch();
        $cursor ->close();

        return !!$row;
    }


    /**
     * Vraci true, pokud je aktualni uzivatel na zadane urovni predavaciho procesu
     * @param Database $database
     * @param string $uroven
     * @return bool
     * @throws \Exception
     */
    public static function overUrovenPredavani($database, $uroven)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PMProUrovne.KodUrovne ' .
            ' FROM RozdeleniOsob, PMProUrovne ' .
            (User::jeZastup() ? ', PoradiPredavani, ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele ' : '') .  // uzivatel je prihlasen "jako" => proces musi byt predan

            ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($database->timestampToParam(time())) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($database->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND PMProUrovne.KodUrovne = ' . $params->addParameter($uroven);


        if (User::jeZastup()) {
            // Uzivatel je jen zastup - doplni filtr na povolene procesy
            $sql .=
                '  AND PoradiPredavani.KodUrovne = PMProUrovne.KodUrovne' .
                '  AND PoradiPredavani.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return false;
        }

        return true; // je to ok
    }


    /**
     * Vraci true, pokud aktualni uzivatel muze hromadne schvalovat hodnoceni
     * @param string[] $urovneUzivatele seznam urovni, do kterych uzivatel patri (funkce seznamUrovniPredavaniUzivatele)
     * @return bool
     */
    public static function muzeHromadneSchvalovatHodnoceni($urovneUzivatele)
    {
        $urovneHromadne = RonPortalSettings::getValue('urovneprohromadnehodnoceni');

        if (!isset($urovneHromadne)) {
            return false;
        }

        $povoleneurovne = explode(';', $urovneHromadne);

        foreach ($povoleneurovne as $u) {
            if (in_array($u, $urovneUzivatele)) {
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci pole - seznam urovni predavani aktualniho uzivatele (podle jeho pracovnich mist)
     * @param Database $database
     * @return string[]
     */
    public static function seznamUrovniPredavaniUzivatele($database)
    {
        $result = [];

        $params = $database ->createParameters();

        if (User::jeZastup()) {
            // zastup - prebira jen vybrane predavaci procesy
            $sql =
                'SELECT DISTINCT PMProUrovne.KodUrovne' .
                ' FROM RozdeleniOsob, PMProUrovne, PoradiPredavani, ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele' .
                // PM zastupovaneho uzivatele
                ' WHERE RozdeleniOsob.RC = ' . $params ->addParameter(User::getRC()) .
                '  AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                '  AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni' .
                '  AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni' .
                // ale jen pro procesy, ktere byly predany zastupujicimu uzivateli
                '  AND PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne' .
                '  AND PoradiPredavani.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';
        } else {
            $sql =
                'SELECT PMProUrovne.KodUrovne ' .
                ' FROM RozdeleniOsob, PMProUrovne ' .

                ' WHERE RozdeleniOsob.RC = ' . $params->addParameter(User::getRC()) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($database->dateToParam(time())) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($database->dateToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
                '   AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
                '   AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni ';
        }

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $result[] = $row[0];
        }

        $cursor ->close();
        return $result;
    }


    /**
     * Vraci filtr pro urovne predavani aktualniho uzivatele
     * tj. vsechny urovne predavani, do kterych je zarazen (prostrednictvim pracovniho mista)
     * @param Database $database
     * @param string $jmenoPole
     * @param Parameters $params
     * @return string
     */
    public static function filtrUrovnePredavani($database, $jmenoPole, $params = null)
    {
        $seznamUrovni = self::seznamUrovniPredavaniUzivatele($database);

        $result = '';

        foreach ($seznamUrovni as $uroven) {
            $result .=
                ($result ? ' OR ' : '') .
                ($params
                    ? "$jmenoPole = " . $params->addParameter($uroven)
                    : "$jmenoPole = '" . $database->addSlashes($uroven) . "'"
                );
        }

        return $result;
    }


    /**
     * Vraci seznam procesu, pro ktere je aktualni uzivatel na prvnim miste predavani
     * @param Database $database
     * @param bool $jenPrvniUroven
     * @return string[]
     */
    public static function seznamProcesuUzivatele($database, $jenPrvniUroven)
    {
        $result = [];

        $params = $database ->createParameters();
        $filtrUrovne = self::filtrUrovnePredavani($database, 'P1.KodUrovne', $params);

        if (!$filtrUrovne) {
            // nema zadnou uroven predavani
            return $result;
        }

        $sql =
            'SELECT DISTINCT KodProcesu FROM PoradiPredavani P1' .
            ' WHERE (' . $filtrUrovne . ')';

        if ($jenPrvniUroven)
            $sql .=
                ' AND NOT EXISTS ( ' .
                '  SELECT * FROM PoradiPredavani P2 ' .
                '  WHERE P2.KodProcesu = P1.KodProcesu ' .
                '    AND P2.PocatecniUroven = P1.PocatecniUroven ' .
                '    AND P2.Poradi < P1.Poradi ' .
                ' ) ';

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $result[] = $row[0];
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Vraci true, pokud je aktualni uzivatel v zadanem predavacim procesu pro danou vstupni uroven
     * @param Database $database
     * @param string $kodProcesu
     * @param string $vstupniUroven
     * @return bool
     * @throws \Exception
     */
    public static function jeVPredavacimProcesu($database, $kodProcesu, $vstupniUroven)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PoradiPredavani.Poradi ' .
            ' FROM PoradiPredavani, PMProUrovne, RozdeleniOsob ' .
            (User::jeZastup() ? ', ZastupProPredavaciProces, VybraneRoleProZastup, ZastupUzivatele' : '') .  // uzivatel je prihlasen "jako" => proces musi byt predan

            ' WHERE RozdeleniOsob.RC = ' . $params ->addParameter(User::getRC()) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->timestampToParam(time())) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->timestampToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni ' .
            '   AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni ' .
            '   AND PoradiPredavani.KodUrovne = PMProUrovne.KodUrovne ' .
            '   AND PoradiPredavani.KodProcesu = ' . $params ->addParameter($kodProcesu) .
            '   AND PoradiPredavani.PocatecniUroven = ' . $params ->addParameter($vstupniUroven);

        if (User::jeZastup()) {
            // Uzivatel je jen zastup - doplni filtr na povolene procesy
            $sql .=
                '  AND PoradiPredavani.KodProcesu = ZastupProPredavaciProces.KodProcesu' .
                '  AND ZastupProPredavaciProces.KodRole = VybraneRoleProZastup.KodRole' .
                '  AND VybraneRoleProZastup.IdZastupu = ZastupUzivatele.Id' .
                '  AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC()) .
                '  AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRealUserRC()) .
                '  AND ZastupUzivatele.Potvrzeni <= 1' . // jen stav "nepotvrzeno" nebo "potvrzeno" (vynecha zamitnuto a zruseno)
                '  AND ZastupUzivatele.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam(time())) .
                '  AND (ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return (bool)$row;
    }


    // Vraci SQL dotaz pro vyber osob - podle prav aktualniho uzivatele a podle vstupni urovne zadaneho procesu
    /**
     * @param Database $database
     * @param string $proces
     * @param int    $kodTypuRozdeleni
     * @return string
     * @throws \Exception
     */
    static function sqlVyberOsobyProPredavaciProces($database, $proces, $kodTypuRozdeleni)
    {
        if (!$kodTypuRozdeleni) {
            $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);
        }

        $where = '';

        if ($kodTypuRozdeleni) { // jen osoby z meho a nebo podrizenych oddeleni
            // Vybere osoby, na ktere mam pravo (podle hierarchie)
            $filtrRozdeleni =
                'EXISTS ' .
                '( SELECT * ' .        // jsem ve stejnem nebo nadrazenem rozdeleni
                '  FROM RozdeleniOsob R_JA, HierarchieRozdeleni H, RozdeleniOsob R_Hodnoceny ' .
                "  WHERE R_Ja.RC = '" . $database ->addSlashes(User::getRC()) . "'\n" .
                '    AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                '    AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                '    AND H.TypPodrizene = R_Hodnoceny.KodTypuRozdeleni ' .
                '    AND H.KodPodrizene = R_Hodnoceny.Rozdeleni ' .
                '    AND R_JA.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                '    AND R_Hodnoceny.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
                '    AND R_Hodnoceny.RC = Osoba.RC' .
                '    AND R_Ja.PlatnostOd <= ' . $database ->timestampToString(time()) .
                '    AND (R_Ja.PlatnostDo >= ' . $database ->timestampToString(time()) . "  OR  R_Ja.PlatnostDo IS NULL)" .
                '    AND R_Hodnoceny.PlatnostOd <= ' . $database ->timestampToString(time()) .
                '    AND (R_Hodnoceny.PlatnostDo >= ' . $database ->timestampToString(time()) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)' .
                ')';
        } else {
            $filtrRozdeleni = '( 0 = 1)';
        }

        if ($proces) {
            /** @var UrovenPredavani[] $urovne */
            $urovne = UrovenPredavani::nactiVstupniUrovnePredavani($database, $proces);

            $vlastni = false;
            $filtrBezHierarchie = '';
            $filtrHierarchie = '';

            /** @var UrovenPredavani $uroven */
            foreach ($urovne as $uroven) {
                if ($uroven->vlastni) {
                    $vlastni = true;
                } else if ($uroven->nekontrolovatHierarchii) {
                    $filtrBezHierarchie  .=
                        ($filtrBezHierarchie ? ' OR ' : '') .
                        "PMU.KodUrovne = '" . $database ->addSlashes($uroven ->pocatecniUroven) . "'";
                } else {
                    $filtrHierarchie  .=
                        ($filtrHierarchie ? ' OR ' : '') .
                        "PMU.KodUrovne = '" . $database ->addSlashes($uroven ->pocatecniUroven) . "'";
                }
            }

            // na nektere z urovni je zaskrtnuto "vlastni" => vidi i sebe
            if ($vlastni) $where .= "Osoba.RC = '" . $database ->addSlashes(User::getRC()) . "'";

            // urovne, na kterych je zaskrtnuto "nekontrolovat hierarchii"
            if ($filtrBezHierarchie) {
                $where .=
                    ($where ? ' OR ' : '') .
                    'EXISTS ' .
                    ' ( SELECT * FROM RozdeleniOsob RO_PM, PMProUrovne PMU, PracovniPomer PP ' .
                    '   WHERE RO_PM.KodTypuRozdeleni = PMU.KodTypuRozdeleni ' .
                    '     AND RO_PM.Rozdeleni = PMU.KodRozdeleni ' .
                    '     AND RO_PM.RC = Osoba.RC ' .
                    '     AND RO_PM.PlatnostOd <= ' . $database ->timestampToString(time()) .
                    '     AND (RO_PM.PlatnostDo >= ' . $database ->timestampToString(time()) . '  OR  RO_PM.PlatnostDo IS NULL)' .
                    '     AND PP.RC = RO_PM.RC ' .
                    '     AND PP.OsobniCislo = RO_PM.OsobniCislo ' .
                    '     AND PP.DatumNastupu = RO_PM.DatumNastupu ' .
                    '     AND PP.DatumNastupu <= ' . $database ->timestampToString(time()) .
                    '     AND (PP.DatumUkonceni >= ' . $database ->timestampToString(time()) . '  OR  PP.DatumUkonceni IS NULL)' .
                    "     AND ($filtrBezHierarchie) " .
                    " )\n";
            }

            // urovne, na kterych neni zaskrtnuto "nekontrolovat hierarchii" (coz je defaultni stav)
            if ($filtrHierarchie) {
                $where .=
                    ($where ? ' OR ' : '') .
                    '(' .
                    'EXISTS ' .
                    ' ( SELECT * FROM RozdeleniOsob RO_PM, PMProUrovne PMU, PracovniPomer PP' .
                    '   WHERE RO_PM.KodTypuRozdeleni = PMU.KodTypuRozdeleni ' .
                    '     AND RO_PM.Rozdeleni = PMU.KodRozdeleni ' .
                    '     AND RO_PM.RC = Osoba.RC ' .
                    '     AND RO_PM.PlatnostOd <= ' . $database ->timestampToString(time()) .
                    '     AND (RO_PM.PlatnostDo >= ' . $database ->timestampToString(time()) . '  OR  RO_PM.PlatnostDo IS NULL)' .
                    '     AND PP.RC = RO_PM.RC ' .
                    '     AND PP.OsobniCislo = RO_PM.OsobniCislo ' .
                    '     AND PP.DatumNastupu = RO_PM.DatumNastupu ' .
                    '     AND PP.DatumNastupu <= ' . $database ->timestampToString(time()) .
                    '     AND (PP.DatumUkonceni >= ' . $database ->timestampToString(time()) . '  OR  PP.DatumUkonceni IS NULL)' .
                    "     AND ($filtrHierarchie) " .
                    " )\n" .
                    '   AND  ' .
                    $filtrRozdeleni .
                    ")\n";
            }

        } else {
            $where = $filtrRozdeleni;
        }

        return
            'SELECT RC, Prijmeni, Jmeno, Titul FROM Osoba ' .
            ($where ? ' WHERE ' . $where : '') .
            ' ORDER BY Prijmeni, Jmeno';
    }


    /**
     * Vraci SQL dotaz pro vyber osob - podle prav aktualniho uzivatele a podle vstupni urovne zadaneho procesu
     * @param Database $database
     * @param string $proces
     * @param int $kodTypuRozdeleni
     * @param int $pouzeVlastni (0=vlastni i podrizena rozdeleni, 1=jen vlastni, 2=jen podrizena rozdeleni) - doplneno 08-2015
     * @param null|string $filtr filtr, ktery se doplni do dotazu - doplneno 07-2020
     * @return string
     */
    public static function sqlVyberPPProPredavaciProces($database, $proces, $kodTypuRozdeleni, $pouzeVlastni = 0, $filtr = null)
    {
        if (!$kodTypuRozdeleni) $kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);

        $sql =
            'SELECT DISTINCT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.IdFotografie, ' . // 0 - 4
            ' PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID, ' . // 5 - 7
            ' PracovniPomer.DatumUkonceni '; // 7

        // prida uzivatelske pole z nastaveni intrawebu
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) $sql .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1);  // (9)
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) $sql .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2);  // (9 nebo 10)

        $sql .=
            ' FROM Osoba, PracovniPomer, ' .
            '   RozdeleniOsob PM_ZAM, PMProUrovne PMU_ZAM, UrovenPredavani U_ZAM, ' . // pracovni misto a uroven zamestnance
            '   PoradiPredavani PORADI, PMProUrovne PMU_JA, RozdeleniOsob PM_JA ' . // moje pracovni misto a uroven
//        '   RozdeleniOsob R_ZAM, HierarchieRozdeleni H, RozdeleniOsob R_JA ' . // hierarchie rozdeleni

            ' WHERE Osoba.RC = PracovniPomer.RC ' .
            // jen aktualne platne PP
            '   AND PracovniPomer.DatumNastupu <= ' . $database ->timestampToString(time()) .
            '   AND (PracovniPomer.DatumUkonceni >= ' . $database ->dateToString(time()) . '  OR  PracovniPomer.DatumUkonceni IS NULL)' .

            // pracovni mista odpovidajici vstupni urovni procesu
            '   AND PM_ZAM.RC = Osoba.RC ' .
            '   AND PM_ZAM.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND PM_ZAM.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND PM_ZAM.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (PM_ZAM.PlatnostDo >= ' . $database ->dateToString(time()) . '  OR  PM_ZAM.PlatnostDo IS NULL)' .
            '   AND PM_ZAM.KodTypuRozdeleni = PMU_ZAM.KodTypuRozdeleni ' .
            '   AND PM_ZAM.Rozdeleni = PMU_ZAM.KodRozdeleni ' .
            '   AND U_ZAM.Kod = PMU_ZAM.KodUrovne ' .

            // vstupni uroven predavani
            "   AND (PORADI.KodProcesu = '" . $database ->addSlashes($proces) . "') " .
            '   AND (PORADI.PocatecniUroven = U_ZAM.Kod) ' .
            '   AND NOT EXISTS ( ' .

            '  SELECT * FROM PoradiPredavani P2 ' .
            "  WHERE P2.KodProcesu = '" . $database ->addSlashes($proces) . "' " .
            '    AND P2.PocatecniUroven = PORADI.PocatecniUroven ' .
            '    AND P2.Poradi < PORADI.Poradi ' .
            '   )' .

            '   AND PMU_JA.KodUrovne = PORADI.KodUrovne ' .
            '   AND PM_JA.KodTypuRozdeleni = PMU_JA.KodTypuRozdeleni ' .
            '   AND PM_JA.Rozdeleni = PMU_JA.KodRozdeleni ' .
            "   AND PM_JA.RC = '" . $database ->addSlashes(User::getRC()) . "'" .
            '   AND PM_JA.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (PM_JA.PlatnostDo >= ' . $database ->dateToString(time()) . '  OR  PM_JA.PlatnostDo IS NULL) ' .

            "   AND (PORADI.Vlastni = 0  OR  Osoba.RC = '" . $database ->addSlashes(User::getRC()) . "') " .

            // Hierarchie rozdeleni
            '   AND ( ' .
            ' (PORADI.NekontrolovatHierarchii = 1) ' .
            ' OR  '.
            'EXISTS ( ' .
            'SELECT * ' .
            ' FROM RozdeleniOsob R_ZAM, HierarchieRozdeleni H, RozdeleniOsob R_JA' .
            "   WHERE R_JA.RC = '" . $database ->addSlashes(User::getRC()) . "'" .
            '   AND R_JA.OsobniCislo = PM_JA.OsobniCislo ' .
            '   AND R_JA.DatumNastupu = PM_JA.DatumNastupu ' .
            '   AND R_JA.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (R_JA.PlatnostDo >= ' . $database ->dateToString(time()) . '  OR  R_JA.PlatnostDo IS NULL)' .
            '   AND R_JA.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
            '   AND H.TypNadrazene = R_JA.KodTypuRozdeleni ' .
            '   AND H.KodNadrazene = R_JA.Rozdeleni ' .
            '   AND H.TypPodrizene = R_ZAM.KodTypuRozdeleni ' .
            '   AND H.KodPodrizene = R_ZAM.Rozdeleni ' .
            (($pouzeVlastni == 1) ? '   AND H.KodNadrazene = H.KodPodrizene' : '' ) .  // jen vlastni
            (($pouzeVlastni == 2) ? '   AND H.KodNadrazene <> H.KodPodrizene' : '' ) . // jen podrizene
            '   AND R_ZAM.KodTypuRozdeleni = ' . (int)$kodTypuRozdeleni .
            '   AND R_ZAM.RC = Osoba.RC' .
            '   AND R_ZAM.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND R_ZAM.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND R_ZAM.PlatnostOd <= ' . $database ->timestampToString(time()) .
            '   AND (R_ZAM.PlatnostDo >= ' . $database ->dateToString(time()) . '  OR  R_ZAM.PlatnostDo IS NULL)' .
            ')' .
            '   )';

        if (!is_null($filtr) && !empty($filtr)) {
            $sql .= "\n AND (" . $filtr . ")";
        }

        $sql .=
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo';

        return $sql;
    }

    /**
     * @param Database $database
     * @param string $predProces kod predavaciho procesu
     * @param string $pocatecniUroven kod pocatecni urovne
     * @param int $poradi poradi predavani
     * @param int $typRozdeleni kod typu rozdeleni
     * @param string $rozdeleni kod rozdeleni
     * @return SqlStatement
     */
    public static function sqlOsobyNaUrovniProcesuDleRozdeleni($database, $predProces, $pocatecniUroven, $poradi, $typRozdeleni, $rozdeleni)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, RozdeleniOsob.RC, RozdeleniOsob.OsobniCislo, RozdeleniOsob.DatumNastupu, Osoba.Jmeno, Osoba.Prijmeni
            FROM PoradiPredavani
                JOIN PMProUrovne ON (PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne)
                JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni)
                JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)
            WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL)
              AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RozdeleniOsob.PlatnostDo IS NULL)
              AND (PoradiPredavani.NekontrolovatHierarchii = 1
                    OR EXISTS(SELECT 1 FROM RozdeleniOsob RO JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typNadrazene = RO.KodTypuRozdeleni AND HierarchieRozdeleni.kodNadrazene = RO.Rozdeleni)
                              WHERE RO.RC = RozdeleniOsob.RC
                                AND RO.OsobniCislo = RozdeleniOsob.OsobniCislo
                                AND RO.DatumNastupu = RozdeleniOsob.DatumNastupu
                                AND RO.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
                                AND (RO.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RO.PlatnostDo IS NULL)
                                AND HierarchieRozdeleni.typPodrizene = " . $params->addParameter($typRozdeleni) . "
                                AND HierarchieRozdeleni.kodPodrizene = " . $params->addParameter($rozdeleni) . "
                            )
                    )
              AND PoradiPredavani.KodProcesu = " . $params->addParameter($predProces) . "
              AND PoradiPredavani.PocatecniUroven = " . $params->addParameter($pocatecniUroven) . "
              AND PoradiPredavani.Poradi = " . $params->addParameter($poradi);

        return new SqlStatement($sql, $params);
    }

    /**
     * @param Database $database
     * @param string $predProces kod predavaciho procesu
     * @param string $pocatecniUroven kod pocatecni urovne
     * @param int $poradi poradi predavani
     * @param int $typRozdeleni kod typu rozdeleni pro hierarchii
     * @param string $rc RC osoby
     * @return SqlStatement
     */
    public static function sqlOsobyNaUrovniProcesuDleOsoby($database, $predProces, $pocatecniUroven, $poradi, $typRozdeleni, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, RozdeleniOsob.RC, RozdeleniOsob.OsobniCislo, RozdeleniOsob.DatumNastupu, Osoba.Jmeno, Osoba.Prijmeni
            FROM PoradiPredavani
                JOIN PMProUrovne ON (PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne)
                JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni)
                JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)
            WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL)
              AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RozdeleniOsob.PlatnostDo IS NULL)
              AND (PoradiPredavani.NekontrolovatHierarchii = 1
                   OR EXISTS(SELECT 1 FROM RozdeleniOsob RO JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typNadrazene = RO.KodTypuRozdeleni AND HierarchieRozdeleni.kodNadrazene = RO.Rozdeleni)
                                    JOIN RozdeleniOsob R_Podrizeny ON (R_Podrizeny.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene AND R_Podrizeny.Rozdeleni = HierarchieRozdeleni.kodPodrizene)
                                    JOIN PracovniPomer ON (PracovniPomer.RC = R_Podrizeny.RC AND PracovniPomer.OsobniCislo = R_Podrizeny.OsobniCislo AND PracovniPomer.DatumNastupu = R_Podrizeny.DatumNastupu)
                              WHERE RO.RC = RozdeleniOsob.RC
                                AND RO.OsobniCislo = RozdeleniOsob.OsobniCislo
                                AND RO.DatumNastupu = RozdeleniOsob.DatumNastupu
                                AND RO.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
                                AND (RO.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RO.PlatnostDo IS NULL)
                                AND PracovniPomer.DatumNastupu <= " . $params->addParameter($database->dateToParam(time())) . "
                                AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL)
                                AND PracovniPomer.RC = " . $params->addParameter($rc) . "
                                AND R_Podrizeny.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) . "
                                AND R_Podrizeny.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
                                AND (R_Podrizeny.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR R_Podrizeny.PlatnostDo IS NULL)
                             )
                    )
              AND PoradiPredavani.KodProcesu = " . $params->addParameter($predProces) . "
              AND PoradiPredavani.PocatecniUroven = " . $params->addParameter($pocatecniUroven) . "
              AND PoradiPredavani.Poradi = " . $params->addParameter($poradi);

        return new SqlStatement($sql, $params);
    }

    /**
     * @param Database $database
     * @param string $predProces kod predavaciho procesu
     * @param string $pocatecniUroven kod pocatecni urovne
     * @param int $poradi poradi predavani
     * @param int $typRozdeleni kod typu rozdeleni pro hierarchii
     * @param string $rc RC pracovniho pomeru
     * @param string $osobniCislo osobni cislo pracovniho pomeru
     * @param int $datumNastupu datum nastupu pracovniho pomeru
     * @return SqlStatement
     */
    public static function sqlOsobyNaUrovniProcesuDlePracPomeru($database,  $predProces, $pocatecniUroven, $poradi, $typRozdeleni, $rc, $osobniCislo, $datumNastupu)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, RozdeleniOsob.RC, RozdeleniOsob.OsobniCislo, RozdeleniOsob.DatumNastupu, Osoba.Jmeno, Osoba.Prijmeni
            FROM PoradiPredavani
                JOIN PMProUrovne ON (PMProUrovne.KodUrovne = PoradiPredavani.KodUrovne)
                JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PMProUrovne.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PMProUrovne.KodRozdeleni)
                JOIN PracovniPomer ON (PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)
                JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)
            WHERE PracovniPomer.DatumNastupu <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (PracovniPomer.DatumUkonceni >= " . $params->addParameter($database->dateToParam(time())) . " OR PracovniPomer.DatumUkonceni IS NULL)
              AND RozdeleniOsob.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
              AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RozdeleniOsob.PlatnostDo IS NULL)
              AND (PoradiPredavani.NekontrolovatHierarchii = 1
                   OR EXISTS(SELECT 1 FROM RozdeleniOsob RO JOIN HierarchieRozdeleni ON (HierarchieRozdeleni.typNadrazene = RO.KodTypuRozdeleni AND HierarchieRozdeleni.kodNadrazene = RO.Rozdeleni)
						JOIN RozdeleniOsob R_Podrizeny ON (R_Podrizeny.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene AND R_Podrizeny.Rozdeleni = HierarchieRozdeleni.kodPodrizene)
				  WHERE RO.RC = RozdeleniOsob.RC
				    AND RO.OsobniCislo = RozdeleniOsob.OsobniCislo
					AND RO.DatumNastupu = RozdeleniOsob.DatumNastupu
					AND RO.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
					AND (RO.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR RO.PlatnostDo IS NULL)
					AND R_Podrizeny.RC = " . $params->addParameter($rc) . "
					AND R_Podrizeny.OsobniCislo = " . $params->addParameter($osobniCislo) . "
					AND R_Podrizeny.DatumNastupu = " . $params->addParameter($database->dateToParam($datumNastupu)) . "
					AND R_Podrizeny.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) . "
					AND R_Podrizeny.PlatnostOd <= " . $params->addParameter($database->dateToParam(time())) . "
					AND (R_Podrizeny.PlatnostDo >= " . $params->addParameter($database->dateToParam(time())) . " OR R_Podrizeny.PlatnostDo IS NULL)
                            )
                    )
              AND PoradiPredavani.KodProcesu = " . $params->addParameter($predProces) . "
              AND PoradiPredavani.PocatecniUroven = " . $params->addParameter($pocatecniUroven) . "
              AND PoradiPredavani.Poradi = " . $params->addParameter($poradi);

        return new SqlStatement($sql, $params);
    }
}
