<?php

/**
 * Menu ronportalu
 */

namespace RonPortal;

use Ron\Controls\AppMenu;
use Ron\Controls\AppMenuItem;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Dokumenty\DokumentyUtils;
use function Ron\ronText;


class XmlMenuItem
{
    var $action;
    var $text;
    var $htmlId;

    /**
     * XmlMenuItem constructor.
     * @param $url
     * @param $text
     * @param $htmlId
     */
    public function __construct($url, $text, $htmlId)
    {
        $this->action = $url;
        $this->text = $text;
        $this->htmlId = $htmlId;
    }
}


class RonPortalMenu extends AppMenu
{
    const SETTINGS_MENU_XML = 'menuxmlformulare';

    /** @var XmlMenuItem[][]    seznam polozek menu pro xml sablony */
    var $xmlSablony;

    private static $instance;


    function __construct()
    {
        parent::__construct();
        $this ->className = 'mainmenu force-visible';
    }


    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new static();
        }

        return self::$instance;
    }


    /**
     * Nacte uzivatelske menu ze session nebo z databaze
     * @param Database $database
     */
    function loadUserMenu($database)
    {
        global $_allwaysloadsettings;

        if (!isset($_SESSION['ronportal_user_menu']) ||  $_allwaysloadsettings) {
            $menuText = null;

            try {
                if (file_exists('user/user_menu.xml')) {
                    $menuText = file_get_contents('user/user_menu.xml');
                } else {

                    // nacte uzivatelske menu z databaze
                    $cursor = $database->select("SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = 'RonPortal_Menu'");
                    $row = $cursor->fetch();
                    $menuText = $row ? $cursor->stringToBinary($row[0]) : null;
                    if ($menuText) {
                        global $db_target_charset;
                        if ($db_target_charset) {
                            $charset = TextManager::getInstance() ->getCharSet();
                            if ($db_target_charset != $charset) {
                                $menuText = iconv($db_target_charset, $charset, $menuText);
                            }
                        }
                    }
                    $cursor->close();
                }

                if ($menuText) {
                    require_once('ron/xmlloader.php');

                    $loader = new XmlLoader();
                    $loader->includeAttributes = true;
                    $loader->prefixToSubarray = true;
                    $loader->ignorePrefixes = true;
                    $loader->convertFromUtf8 = true;
                    $menuArray = $loader->readXml($menuText);
                    $_SESSION['ronportal_user_menu'] = $menuArray;
                } else {
                    $_SESSION['ronportal_user_menu'] = false;
                }
            } catch (\Exception $e) {
                MessageManager::addErrorMessage('Pri nacitani uzivatelskeho menu doslo k chybe! ' . $e->getMessage());
                $_SESSION['ronportal_user_menu'] = false;
            }
        }
    }


    /**
     * @param Database $database
     * @throws \Exception
     */
    function load($database)
    {
        $this ->loadUserMenu($database);

        $this->nactiXMLSablony($database);

        if ($_SESSION['ronportal_user_menu']) {
            $this ->loadUserSettingsMenu($database, $_SESSION['ronportal_user_menu']);
        } else {
            $this->enableMultipleSections = RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYBER_APLIKACE) == 1;
            $this->nastavSekce();

            if (file_exists('user_menu.php')) {
                // uzivatelsky definovane menu
                require_once('adsapp/usermenuutils.php');
                require_once('adsapp/nastaveni.php'); // pro kompatibilitu starych usermenu - pouzivaji funkce S_Get...

                require('user_menu.php');
            } else {
                $this->loadDefaultMenu($database);
            }
        }

        if ( !$this ->enableMultipleSections ) {
            $this ->setActiveSection($_SESSION['menusection']);
        }
    }


    /**
     * Pomocna funkce - sestavi html parametry zadane v XML atributech
     * @param $menuItemSettings
     * @return string
     */
    function buildHtmlParams($menuItemSettings)
    {
        $htmlParams = '';

        if (isset($menuItemSettings['html'])  &&  $menuItemSettings['html']) {
            foreach ($menuItemSettings['html'] as $paramName => $paramValue) {
                $htmlParams .= "$paramName='$paramValue' ";
            }
        }

        return $htmlParams;
    }


    /**
     * Nacte uzivatelske menu ze zadaneho pole
     * @param $database
     * @param $menuArray
     * @throws \Exception
     */
    function loadUserSettingsMenu($database, $menuArray)
    {
        if (isset($menuArray['section'])  &&  $menuArray['section']) {
            $this->enableMultipleSections = false;

            foreach ($menuArray['section'] as $sectionSettings) {
                $zobrazit = true;

                if ($sectionSettings['settings']) {
                    if (!RonPortalSettings::getValue($sectionSettings['settings'])) $zobrazit = false;
                }

                if ($zobrazit) {
                    $pravaStr = $sectionSettings['prava'];
                    if ($pravaStr  &&  !Prava::jsouNejakePrava($pravaStr)) $zobrazit = false;
                }

                if ($zobrazit) {
                    if ($sectionSettings['id']) {
                        $action = isset($sectionSettings['defaultaction']) ? '&menuid=' . $sectionSettings['defaultaction'] : null;
                        $this->addSection(\Ron\ronText($sectionSettings['text']), $sectionSettings['id'], true, $action, '', $sectionSettings['glyph']); //, $sectionSettings['htmlid']);
                    } else {
                        $newItem = new AppMenuItem(\Ron\ronText($sectionSettings['text']), $sectionSettings['defaultaction']);
                        $newItem ->htmlParams = $this ->buildHtmlParams($sectionSettings);
                        $newItem ->glyph = $sectionSettings['glyph'];
                        $this ->addSectionItem( $newItem );
                    }
                }
            }
        } else {
            $this ->nastavSekceRonPortal();
        }

        $activeItem = null;
        $defaultMenuId = RonPortalApp::getDefaultPortalAction();

        if (isset($menuArray['menuitem'])  &&  $menuArray['menuitem']) {
            foreach ($menuArray['menuitem'] as $menuItemSettings) {
                // ['action' => '...', 'text' => '...', 'section' => '...', 'settings' => '...', 'prava' => '...', 'html' => [...]
                $zobrazit = true;

                if ($menuItemSettings['settings']) {
                    if ($menuItemSettings['settingstable']) {
                        if (!$this ->existujeNastaveni($database, $menuItemSettings['settings'], $menuItemSettings['settingstable'])) {
                            $zobrazit = false;
                        }
                    } else {
                        if (!RonPortalSettings::getValue($menuItemSettings['settings'])) {
                            $zobrazit = false;
                        }
                    }
                }

                if ($zobrazit) {
                    $pravaStr = $menuItemSettings['prava'];
                    if ($pravaStr  &&  !Prava::jsouNejakePrava($pravaStr)) $zobrazit = false;
                }

                if ($zobrazit) {
                    $profily = $menuItemSettings['profil'];
                    if ($profily && !in_array(User::getProfil(), explode(';', $menuItemSettings['profil']))) {
                        $zobrazit = false;
                    }
                }

                if ($zobrazit) {
                    if (isset($menuItemSettings['xmlindex'])) {
                        $this ->zobrazXMLSablony((int)$menuItemSettings['xmlindex'], $menuItemSettings['section']);
                    } else if ($menuItemSettings['text']) {
                        //$newItem = new AppMenuItem($itemtext, $action, '', $htmlid);
                        $newItem = new AppMenuItem(\Ron\ronText($menuItemSettings['text']), $menuItemSettings['action'], $menuItemSettings['class'], $menuItemSettings['htmlid']);
                        $newItem ->sectionId = $menuItemSettings['section'];

                        if (isset($_GET['menuid'])  &&  $_GET['menuid'] === $newItem ->menuId) {
                            $activeItem = $newItem;
                        } else if (!$activeItem  &&  $newItem ->menuId === $defaultMenuId) {
                            $activeItem = $newItem;
                        }
//                        $htmlParams = '';
//                        if (isset($menuItemSettings['html'])  &&  is_array($menuItemSettings['html'])) {
//                            foreach ($menuItemSettings['html'] as $paramName => $paramValue) {
//                                $htmlParams .= "$paramName='$paramValue' ";
//                            }
//                        }
                        $newItem ->htmlParams = $this ->buildHtmlParams($menuItemSettings);
                        $this ->addMenuItem($newItem);
                    } else {
                        $this->addEmptyRow($menuItemSettings['section']);
                    }
                }
            }

            // po prihlaseni zapne sekci, ktera je prirazena vychozi menuid
            if ($activeItem  &&  $activeItem ->sectionId  && !isset($_SESSION['menusection'])) {
                $this ->setActiveSection($activeItem ->sectionId);
            }
        } else {
            $this ->loadDefaultMenu($database);
        }
    }

    /**
     * @param Database $database
     * @throws \Exception
     */
    function loadDefaultMenu($database)
    {

        if (!User::getVynutitZmenuHesla()) // pokud musi zmenit heslo, tak neni nic jineho zobrazeno
        {
            $this->pridejUzivatelskePolozky();

            $this->zobrazXMLSablony(0); // nahore

            // Dashboard
            if (Prava::jsouNejakePrava('dashboard')) {
                $this ->addItem(TextManager::_('MI_Dashboard'), 'dashboard', 'mi_dashboard');
            }

            if (RonPortalSettings::getIntValue('povolitankety')) {
                $this ->addItem(TextManager::_('Ankety'), 'anketa', 'mi_anketa');
            }

            if (Prava::vratHodnotuPrava('zamest_prohl') != 0) {
                $this->addItem(\Ron\ronText('MI_Seznam_PP'), 'zamestnanec', 'mi_seznampp');
            }

            $this->addEmptyRow();

            // Spolecne
            if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_OSOBNI)) {
                $this->addItem(\Ron\ronText('MI_Osobni_udaje'), 'osobniudaje', 'mi_osobniudaje');
            }

            if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VZKAZY)) {
                $this ->addItem(\Ron\ronText('MI_Vzkazy'), 'vzkazy', 'mi_vzkazy');
            }

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PERSONALISTIKU)) {
                $this->pridejPersonalistiku($database);
            }
            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PERSONALISTIKU)) {
                $this->pridejMzdy($database);
            }

            if (RonPortalSettings::getValue(self::SETTINGS_MENU_XML)) {
                if (Prava::vratHodnotuPrava('xml') == 3 || PredavaciProcesyUtils::jeVNejakemProcesu($database)) {
                    $this->addItem(RonPortalSettings::getValue(self::SETTINGS_MENU_XML), 'seznamxmlformularu', 'mi_xmlformulare');
                    $this->addEmptyRow();
                }
            }

            $this->zobrazXMLSablony(2); // nad dochazkou
            $this->addEmptyRow();

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DOCHAZKU)) {
                $this->pridejDochazku($database);
            }

            global $reportsmenuitem; // deprecated, jen pro kompatibilitu!
            global $showReports;
            if ($showReports  ||  $reportsmenuitem) {
                global $reportserver;

                if ($reportserver) {
                    $this ->addItem( \Ron\ronText($reportsmenuitem ?: 'MI_Tisk'), 'reports', 'mi_reportlist');
                    $this ->addEmptyRow('sec_dochazka');
                }
            }


            $this->zobrazXMLSablony(5); // za dochazkou
            $this->addEmptyRow();

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_JIDELNU)) {
                $this->pridejJidelnu();
            }

            $this->addEmptyRow();
            $this->zobrazXMLSablony(6); // za jidelnou
            $this->addEmptyRow();

            if (Prava::vratHodnotuPrava('prohl_terminaly') != 0)
            {
                $this->addItem(\Ron\ronText('MI_Terminaly'), 'terminaly', 'mi_terminaly', 'sec_dochazka');
                $this->addItem(\Ron\ronText('MI_ZaznamySnimacu'), 'zaznamysnimacu', 'mi_zaznamysnimacu', 'sec_dochazka');
                $this->addEmptyRow();
            }

            if (RonPortalSettings::getIntValue('vyroba')) {
                if (Prava::jsouNejakePrava('vyroba')) {
                    $this ->addItem(TextManager::_('MI_Puvodni_vyroba'), 'puvodnivyroba', 'mi_puvodnivyroba', 'sec_vyroba');
                }
                if (Prava::vratHodnotuPrava('editacevyroby') != 0) {
                    $this->addItem(\Ron\ronText('MI_Vyroba_edit'), 'vyroba', 'mi_vyroba', 'sec_vyroba');
                }

                if (Prava::vratHodnotuPrava('vyroba')) {
                    $this->addItem(\Ron\ronText('MI_Monitorovani_vyroby'), 'monitorovaniv', 'mi_monitorovaniv', 'sec_vyroba');
                    $this->addItem(ronText('MI_Vyrobni_cinnosti'), 'vyrobnicinnosti', 'mi_vyrobnicinnosti', 'sec_vyroba');
                }

                $this->addEmptyRow();
            }


            if ((Prava::vratHodnotuPrava('objednavka_vydana_prohl') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_vlozeni') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_faktura') != 0) ||
                (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni1') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni2') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni3') != 0)) {
                    $this->addItem(\Ron\ronText('MI_Objednavky_vydane'), 'objednavky', 'mi_objednavkyvydane', 'sec_sklad');
                    if ((Prava::vratHodnotuPrava('objednavka_vydana_vlozeni') != 0))
                        $this->addItem(\Ron\ronText('MI_Objednavka_novy_zaznam'), 'novaobjednavka', 'mi_objednavkyvydane', 'sec_sklad');
                if (Prava::vratHodnotuPrava('objednavka_vydana_prohl') != 0) {
                    $this->addItem(\Ron\ronText('MI_Seznam_firem'), 'firma', 'mi_firmy', 'sec_sklad');
                }
                $this->addEmptyRow();
            }


            if (User::isAuthenticated()  &&  !User::isLoggedAs()  &&  RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_ICAL)) {
                // vlastni nastaveni
                $this->addItem(\Ron\ronText('MI_Nastaveni'), 'nastaveni', 'mi_nastaveni', '*sec_user_menu');
            }

            if (Prava::vratHodnotuPrava('blokovani') != 0) {
                $this->addItem(\Ron\ronText('MI_Blokovani_karty'), 'blokovani', 'mi_blokovani', '*sec_user_menu');
            }

            if (Prava::vratHodnotuPrava('odblokovani') != 0) {
                $this->addItem(\Ron\ronText('MI_Odblokovani_karty'), 'odblokovani', 'mi_odblokovani', '*sec_user_menu');
            }
        }

        if (Prava::vratPrava('zast_uz_komu') ->jsouNejakePrava()  ||  Prava::vratPrava('zast_uz_koho') ->jsouNejakePrava()) {
            $this ->addItem(\Ron\ronText('MI_Zastupy_uzivatelu'), 'zastupuzivatele', 'mi_zastupuzivatele');
        }

        if (Prava::vratPrava('loginas')->jsouNejakePrava()) {
            $this->addItem(\Ron\ronText('MI_Login_as'), 'loginas', 'mi_loginas', '*sec_user_menu');
        }

        if (Prava::vratHodnotuPrava('admin') == 3) {
            $this ->addItem( \Ron\ronText('MI_Nastaveni_portalu'), 'nastaveniportalu', 'mi_nastaveniportalu');
            $this ->addItem( \Ron\ronText('MI_Novinky'), 'novinky', 'mi_novinky');
        }

        if (User::isPasswordChangeEnabled()) { // napr. pri overovani pomoci LDAP nelze menit heslo
            if (Prava::vratHodnotuPrava('zmenahesla') != 0) $this->addItem(\Ron\ronText('MI_Zmena_hesla'), 'changepassword', 'mi_changepassword');
        }

        $this->addItem(\Ron\ronText(User::isAuthenticated() ? 'Logout' : 'MI_Login'), 'logout', 'mi_logout');

        $this->addEmptyRow();
        $this->zobrazXMLSablony(7); // konec

        $this ->setSectionsDefaultActions();
    }


    function nastavSekceRonPortal()
    {
        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DOCHAZKU)) {
            $this ->addSection(\Ron\ronText('MI_APP_Dochazka'), 'sec_dochazka', RonPortalSession::menuSectionIsVisible('sec_dochazka'), null, '', 'fal fa-user-clock');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PERSONALISTIKU)) {
            $this ->addSection(\Ron\ronText('MI_APP_Personalistika'), 'sec_personalistika', RonPortalSession::menuSectionIsVisible('sec_personalistika'), null, '', 'fal fa-user-tag');
        }
        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PERSONALISTIKU)) {
            $this ->addSection(\Ron\ronText('MI_APP_Mzdy'), 'sec_mzdy', RonPortalSession::menuSectionIsVisible('sec_mzdy'), null, '', 'fal fa-coins');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_JIDELNU)) {
            $this ->addSection(\Ron\ronText('MI_APP_Jidelna'), 'sec_jidelna', RonPortalSession::menuSectionIsVisible('sec_jidelna'), '&menuid=jidelnicek', '', 'fal fa-utensils');
        }

        if (RonPortalSettings::getIntValue('vyroba')) {
            $this->addSection(\Ron\ronText('MI_APP_Vyroba'), 'sec_vyroba', RonPortalSession::menuSectionIsVisible('sec_vyroba'), null, '', 'fal fa-hammer');
        }

        if ((Prava::vratHodnotuPrava('objednavka_vydana_prohl') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_vlozeni') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_faktura') != 0) ||
            (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni1') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni2') != 0) || (Prava::vratHodnotuPrava('objednavka_vydana_schvaleni3') != 0)) {
            $this->addSection(\Ron\ronText('MI_APP_Sklad'), 'sec_sklad', RonPortalSession::menuSectionIsVisible('sec_sklad'), null, '', 'fal fa-warehouse-alt');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ELEARNING_POVOLEN)  &&  Prava::jsouNejakePrava('editacekurzu') ) {
            $this ->addSectionItem( new AppMenuItem(\Ron\ronText('E_Learning'), 'kurzy', '', 'sec_elearning', 'fal fa-book-open'));
            //$this ->addSection(\Ron\ronText('E_Learning'), true, '?menuid=kurzy');
            //echo "<li class='active'><a href='?menuid=kurzy'>" . \Ron\ronText('E_Learning') . "</a></li>\n";
        }
    }


    function nastavSekce()
    {
        if ( file_exists('user_sectionmenu.php') ) {
            // uzivatelsky definovane sekce
            require_once('adsapp/usermenuutils.php');
            require_once('user_sectionmenu.php');

            $this ->enableMultipleSections = false; // uzivatelske section menu => prepinani modulu
        } else {
            if ( RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_VYBER_APLIKACE) ) {
                $this->nastavSekceRonPortal();
            }
        }
    }


    /**
     * @param Database $database
     * @return bool
     */
    function nactiXMLSablony($database)
    {
        $this->xmlSablony = [];

        if (!RonPortalSettings::getInstance() ->getIntValue(RonPortalSettings::NACIST_XML_DO_MENU)) {
            return false;
        }

        // Vytvori pole, napr.
        // [0] =>  ( 'pol1' => 'A b c',  'pol2' => 'X yz' )
        // [1] =>  ( ...
        // ...

        $params = $database->createParameters();

        $sql =
            'SELECT MenuPozice, MenuId, MenuText, TypPrava, Id FROM SablonaXMLFormulare ' .
            " WHERE MenuText IS NOT NULL  AND MenuText IS NOT NULL  AND  MenuPozice IS NOT NULL " . // na oracle nefunguje test MenuText <> '' !
            '   AND (PlatnostOd <= ' . $params->addDateParameter(time()) . ' OR PlatnostOd IS NULL) ' .
            '   AND (PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR PlatnostDo IS NULL) ' .
            ' ORDER BY MenuPozice, MenuId, Id';

        try {
            $cursor = $database->select($sql, $params);
        } catch (\Exception $e) {
            // chybu pri nacitani XML sablon ignoruje
            return false;
        }

        $last_menupozice = null;
        $last_menuid = null;

        while ($row = $cursor ->fetch()) {

            $zobrazit = true;

            // pokud jsou nastavena prava, probehne kontrola
            if ($row[3]) {
                // pokud nema zamestnanec zadna prava, tak danou polozku nezobrazuje
                $prava = Prava::vratPrava($row[3]);
                if (!$prava->jsouNejakePrava()) $zobrazit = false;
            }

            if ($zobrazit) {
                $pozice = $row[0];

                // pokud jeste neni vytvoreno pole polozek pro danou pozici, tak ho vytvori
                if (!isset($this->xmlSablony[$pozice])) $this->xmlSablony[$pozice] = [];

                if ($row[1]) {
                    // je zadane MenuId => bude to odkaz na seznam XML sablon
                    if ($last_menupozice !== $pozice || $last_menuid !== $row[1]) {
                        //$this->xmlSablony[$row[0]] ['seznamxmlformularu&xmlmenuid=' . $row[1]] = $row[2];
                        $this->xmlSablony[$pozice]['#' . $row[1]] = new XmlMenuItem('seznamxmlformularu&xmlmenuid=' . $row[1], \Ron\ronText($row[2]), 'mi_seznamxml_' . $row[1]);
                    }
                } else {
                    // neni zadane MenuId => bude to odkaz primo na vyplneni/zobrazeni sablony
                    $this->xmlSablony[$pozice][] = new XmlMenuItem('xml&record=' . $row[4], \Ron\ronText($row[2]), 'mi_xml_' . $row[4]);
                }
            }
        }

        $cursor ->close();

        return true;
    }


    /**
     * Zobrazi polozky menu pro XML sablony zarazene na zadanou pozici
     * @param int $pozice
     * @param string $section
     * @return bool
     */
    function zobrazXMLSablony($pozice, $section = '')
    {
        if (!$this->xmlSablony) return false;
        if (!isset($this->xmlSablony[$pozice])) return false;

//        foreach ($this->xmlSablony[$pozice] as $url => $text) {
//            $this->addItem($text, $url);
//        }

        foreach ($this->xmlSablony[$pozice] as $xmlMenuItem) {
            $newItem = new AppMenuItem($xmlMenuItem ->text, $xmlMenuItem ->action, '', $xmlMenuItem ->htmlId);
            $newItem ->sectionId = $section;
            $this ->addMenuItem($newItem);
        }

        return true;
    }



    // --------------------------------------------------------------------
    // Uzivatelske polozky
    function pridejUzivatelskePolozky()
    {
        $jeuzivatelska = false;
        $uzivatelske = explode(';', RonPortalSettings::getValue(RonPortalSettings::UZIVATELSKE_STRANKY) ?? '');

        foreach ($uzivatelske as $polozka) {
            $pomarray = explode(':', $polozka);
            if (isset($pomarray[1]) && $pomarray[1]) {
                $jeuzivatelska = true;
                $this->addItem($pomarray[1], $pomarray[0], 'mi_user_' . (strstr($pomarray[0], '&', true) ?: $pomarray[0]));
            }
        }

        // Za uzivatelske prida prazdny radek
        if ($jeuzivatelska) $this->addEmptyRow();
    }


    /** Doplni do menu odkazy pro personalistiku
     * @param Database $database
     * @throws \Exception
     */
    function pridejPersonalistiku($database)
    {
        if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOKUMENTY)) {
            if (Prava::jsouNejakePrava([Prava::vratPrava('prohl_osobni'), Prava::vratPrava('dokumenty_prohl'), Prava::vratPrava('dokumenty_seznam')])) {
                $this->addItem(\Ron\ronText('Dokumenty'), 'dokumenty', 'mi_dokumenty', 'sec_personalistika');
            }
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_SKOLENI) &&
            ((Prava::vratHodnotuPrava('skoleni_prohlizeni') != 0) || (Prava::vratHodnotuPrava('skoleni_prihlasit') != 0) || (Prava::vratHodnotuPrava('skoleni_plan') != 0) ||
            (Prava::vratHodnotuPrava('skoleni_vyslat') != 0) || (Prava::vratHodnotuPrava('skoleni_absolvovani') != 0) )
        ) {
            $this ->addItem(\Ron\ronText('MI_Skoleni'), 'skoleni', 'mi_skoleni', 'sec_personalistika');
        }

        $prava = Prava::vratPrava('ukoly'); // prava na prohlizeni a "splneni"
        $pravaVlozeni = Prava::vratPrava('ukoly_vlozeni'); // prava na vlozeni
        $pravaVedouci = Prava::vratPrava('ukoly_nast_resitele'); // prava na prirazeni ukolu resiteli
        if ($prava ->jsouNejakePrava() || $pravaVlozeni->jsouNejakePrava() || $pravaVedouci->jsouNejakePrava()) {
            $this ->addItem(\Ron\ronText('MI_Ukoly'), 'ukoly', 'mi_ukoly', 'sec_personalistika');
        }

        // pokud jsou prava pridej polozku Nahlasene zmeny (nezobrazuje se zamestnancum, kteri maji pouze vlastni nahlaseni a nemaji schvalovani)
        $prava = Prava::vratPrava('ou_zmeny_nahlaseni');
        if (($prava ->jsouNejakePrava()  &&  !$prava ->pouzeSebe()) || (Prava::vratHodnotuPrava('ou_zmeny_schval') != 0 )) {
            $this->addItem(\Ron\ronText('MI_Nahlasene_zmeny'), 'nahlasenezmeny', 'mi_nahlasene_zmeny', 'sec_personalistika');
        }

        $this ->zobrazXMLSablony(1); // 1 = za zakladnimi

        $this ->addEmptyRow('sec_personalistika');

        if (RonPortalSettings::getIntValue('povolitpersonalniprocesy') &&
            ((Prava::vratHodnotuPrava('pers_procesy_prohl') != 0) || (Prava::vratHodnotuPrava('pers_procesy_vlozeni') != 0))) {
            $this ->addItem(\Ron\ronText('MI_Personalni_procesy'), 'personalniprocesy', 'mi_personalniprocesy', 'sec_personalistika');
            $this ->addEmptyRow('sec_personalistika');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_HODNOCENI)) {
            global $MI_Hodnoceni; // kompat
            global $MI_VysledkyHodnoceni; // kompat

            if (PredavaciProcesyUtils::jeVNejakemProcesu($database)) $this->addItem($MI_Hodnoceni ?: \Ron\ronText('MI_Hodnoceni'), 'hodnoceni', 'mi_seznamhodnoceni', 'sec_personalistika');
            if (Prava::vratHodnotuPrava('hodnoceni') != 0) $this->addItem(\Ron\ronText('MI_Hodnoceni_osob'), 'hodnoceniosob', 'mi_hodnoceniosob', 'sec_personalistika');
            $this ->addItem($MI_VysledkyHodnoceni ?: \Ron\ronText('MI_Vysledky_hodnoceni'), 'vysledkyhodnoceni', 'mi_vysledkyhodnoceni', 'sec_personalistika');
        }

        $this ->addEmptyRow('sec_personalistika');

        if ((Prava::vratHodnotuPrava('pr_denik_write') != 0) || (Prava::vratHodnotuPrava('pr_denik_read') != 0)) {
            $this ->addItem(\Ron\ronText('MI_Provozni_denik'), 'provoznidenik', 'mi_provoznidenik', 'sec_personalistika');
        }

        $this ->addEmptyRow('sec_personalistika');

        if ( (Prava::vratHodnotuPrava('lekarske_prohl') != 0) && RonPortalSettings::getIntValue('povolitprohlidky')) {
            $this->addItem(\Ron\ronText('MI_Zdravotni_prohlidky'), 'lekarskeprohlidky', 'mi_lekarskeprohlidky', 'sec_personalistika');
        }

        if ((Prava::vratHodnotuPrava('lekarske_prohl') != 0)) {
            $this->addItem(\Ron\ronText('MI_Pracovni_omezeni'), 'pracovniomezeniosoby', 'mi_pracovniomezeniosoby', 'sec_personalistika');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOVEDNOSTI))
        {
            if ((Prava::vratHodnotuPrava('doved_ziskane') != 0) || (Prava::vratHodnotuPrava('doved_edit') != 0))
                $this ->addItem(\Ron\ronText('MI_Dovednosti'), 'maticedovednosti', 'mi_dovednosti', 'sec_personalistika');
        }

        if (((Prava::vratHodnotuPrava('pomucky_prohl') != 0) || (Prava::vratHodnotuPrava('pomucky_vydej') != 0) || (Prava::vratHodnotuPrava('pomucky_potvrzeni') != 0))
            && RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_POMUCKY))
        {
            $this->addItem(\Ron\ronText('MI_Pracovni_pomucky'), 'pracovnipomucky', 'mi_pracovnipomucky', 'sec_personalistika');
        }

        //pokud jsou prava pridej polozku Benefity
        if ((Prava::vratHodnotuPrava('benefit_rezervace') != 0) || (Prava::vratHodnotuPrava('benefit_schvaleni') != 0) || (Prava::vratHodnotuPrava('benefit_vydani') != 0) || (Prava::vratHodnotuPrava('benefit_vznik_bodu') != 0)) {
            $this->addItem(\Ron\ronText('MI_Benefity'), 'benefity', 'mi_benefity', 'sec_personalistika');
        }

        $this ->addEmptyRow('sec_personalistika');

        if (   ((Prava::vratHodnotuPrava('uchazeci_prohl') != 0) || (Prava::vratHodnotuPrava('uchazeci_komunikace') != 0))
                && RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_UCHAZECE))
        {
            $this->addItem(\Ron\ronText('MI_Seznam_uchazecu'), 'uchazeci', 'mi_uchazeci', 'sec_personalistika');
        }

        $this ->addEmptyRow('sec_personalistika');


        if ((Prava::vratHodnotuPrava('napady') != 0 || Prava::vratHodnotuPrava('napady_admin') != 0)) {
            $this->addItem(\Ron\ronText('MI_Napady'), 'napady', 'mi_napady', 'sec_personalistika');
        }

        $this ->addEmptyRow('sec_personalistika');


    }

    function pridejMzdy($database)
    {
        if (Prava::vratHodnotuPrava('prohl_mzdy') != 0) {
            $this ->addItem(\Ron\ronText('MI_Mzdove_udaje'), 'mzdoveudaje', 'mi_mzdoveudaje', 'sec_mzdy');
        }

        if ($this->lzeZobrazitVyplatniListky($database)) {
            $this->addItem(\Ron\ronText('MI_Vyplatni_listky'), 'vyplatnilistky', 'mi_vyplatnilistky', 'sec_mzdy');
        }

        $this ->addEmptyRow('sec_mzdy');

        //pokud je pravo pridej polozku - Dane - prohlaseni poplatnika dane
        if ((Prava::vratHodnotuPrava('dane_prohlaseni') != 0) || (Prava::vratHodnotuPrava('dane_vyuctovani') != 0) ) {
            $this->addItem(\Ron\ronText('MI_Dane'), 'dane', 'mi_dane', 'sec_mzdy');
        }

        $this ->addEmptyRow('sec_mzdy');

        //pokud je pravo pridej polozku - Oznameni k davkam NP
        if ((Prava::vratHodnotuPrava('davkynz_prohl') != 0) || (Prava::vratHodnotuPrava('davkynz_zapis') != 0) ) {
            $this->addItem(\Ron\ronText('MI_DavkyNZ'), 'davkynz', 'mi_davkynz', 'sec_mzdy');
        }

        $this ->addEmptyRow('sec_mzdy');
    }

    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     */
    function lzeZobrazitVyplatniListky($database)
    {
        if (User::isLoggedAs()) {
            return false;
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_DOKUMENTY)) {
            return false;
        }

        global $_allwaysloadsettings;

        if (!$_allwaysloadsettings && isset($_SESSION['zobrazit_vyplatni_listky']))
            return $_SESSION['zobrazit_vyplatni_listky'];

        $_SESSION['zobrazit_vyplatni_listky'] = DokumentyUtils::lzeZobrazitVyplatniListky($database);

        return $_SESSION['zobrazit_vyplatni_listky'];
    }


    /**
     * @param int $cisloBrowseru
     * @return bool
     */
    function jePravoNaPokladniKnihu($cisloBrowseru) {
        $typy = $_SESSION['pokladniKnihaSeznamTypu'][$cisloBrowseru];

        foreach ($typy as $typ) {
            if ((Prava::vratHodnotuPrava('pkniha_prohl', 'pkniha_prohl_' . $typ) != 0) ||
                (Prava::vratHodnotuPrava('pkniha_vloz', 'pkniha_vloz_' . $typ) != 0) ||
                (Prava::vratHodnotuPrava('pkniha_schval1', 'pkniha_schval1_' . $typ) != 0) ||
                (Prava::vratHodnotuPrava('pkniha_schval2', 'pkniha_schval2_' . $typ) != 0) ||
                (Prava::vratHodnotuPrava('pkniha_schval3', 'pkniha_schval3_' . $typ) != 0))
            {
                return true;
            }
        }
        return false;
    }


    /**
     * @param $database
     * @return array
     * @throws \Exception
     */
    function nactiPokladniKnihy($database) {
        global $_allwaysloadsettings;
        if ($_allwaysloadsettings  ||  !isset($_SESSION['pokladniKnihaCislaBrowseru'])) {
            $sql = 'SELECT COALESCE(RonPortalBrowser, 1), Kod FROM TypPolozkyPokladniKnihy ORDER BY 1';

            /* @var Database $database */
            $cursor = $database->getSqlResult($sql);
            $browsery = [];
            $typy = [];
            $browser = '';

            while ($row = $cursor->fetch()) {
                $typy[$row[0]][] = $row[1];

                if ($browser !== $row[0]) {
                    $browsery[] = $row[0] != 0 ? $row[0] : 1;
                }
                $browser = $row[0];
            }

            $cursor ->close();

            $_SESSION['pokladniKnihaCislaBrowseru'] = $browsery;
            $_SESSION['pokladniKnihaSeznamTypu'] = $typy;

            return $browsery;
        } else {
            return $_SESSION['pokladniKnihaCislaBrowseru'];
        }
    }


    /**
     * @param Database $database
     * @param string $name
     * @param string $tableName
     * @param mixed $defaultValue
     * @return bool
     */
    function existujeNastaveni($database, $name, $tableName = 'SYS_BVAR_GLOBAL', $defaultValue = null)
    {
        global $_allwaysloadsettings;

        $sessionName = '#ronportal_menu_' . $name;

        if (!$_allwaysloadsettings) {
            if (isset($_SESSION[$sessionName])) {
                return $_SESSION[$sessionName];
            }
        }

        $params = $database ->createParameters();
        try {
            $tableName = preg_replace('/[^\w]/', '', $tableName);
            $cursor = $database->getSqlResult("SELECT Val FROM $tableName WHERE Name = " . $params->addParameter($name), $params);
            $row = $cursor ->fetch();

            if ($row) {
                $result = !!$row[0];
            } else {
                $result = !!$defaultValue;
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
            $result = false;
        }

        $_SESSION[$sessionName] = $result;
        return $result;
    }


    // --------------------------------------------------------------------
    // Dochazka
    /**
     * @param Database $database
     * @throws \Exception
     */
    function pridejDochazku($database)
    {
        // pokud jsou prava na zobrazovani, tak prida polozky pro zobrazovani
        if (Prava::vratHodnotuPrava('prohlizeni') != 0) {
            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PUVODNI)) $this ->addItem(\Ron\ronText('MI_Dochazka_zamestnance'), 'dochazkazamestnance', 'mi_dochazkazamestnance', 'sec_dochazka');
            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_HISTORII_PRUCHODU)) $this ->addItem(\Ron\ronText('MI_Historie_pruchodu'), 'historiepruchodu', 'mi_historiepruchodu', 'sec_dochazka');

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_ZPRACOVANOU)) $this ->addItem(\Ron\ronText('MI_Zpracovana_dochazka'), 'zpracovanadochazka', 'mi_zpracovanadochazka', 'sec_dochazka');
            if ($this ->existujeNastaveni($database, 'RonPortal_Prehled_zpracovane')) {
                $this ->addItem(\Ron\ronText('MI_Prehled_zpracovane'), 'prehledzpracovane', 'mi_prehledzpracovane', 'sec_dochazka');
            }

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DENNI_VYSLEDKY)) $this ->addItem(\Ron\ronText('MI_Denni_vysledky'), 'dennivysledky', 'mi_dennivysledky', 'sec_dochazka');

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_MESICNI_VYSLEKDY)) $this ->addItem(\Ron\ronText('MI_Mesicni_vysledky'), 'mesicnivysledky', 'mi_mesicnivysledky', 'sec_dochazka');
            else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_MESICNI_R)) $this ->addItem(\Ron\ronText('MI_Mesicni_vysledky'), 'mesicnivysledkyr', 'mesicnivysledkyr', 'sec_dochazka');

            if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_VARIABILNI)) $this ->addItem(\Ron\ronText('MI_Variantni_vysledky'), 'variantnivysledky', 'mi_variantnivysledky', 'sec_dochazka');
            else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_VARIABILNI_DENNI)) $this ->addItem(\Ron\ronText('MI_Variantni_vysledky'), 'dennivysledkyvariantni', 'mi_dennivysledkyvariantni', 'sec_dochazka');

            if ( RonPortalSettings::getIntValue(RonPortalSettings::OMEZENI_DOCHAZKY)) $this ->addItem(\Ron\ronText('MI_Omezeni_dochazky'), 'omezenidochazky', 'mi_omezenidochazky', 'sec_dochazka');
        }

        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_MZDOVE_POLOZKY) && Prava::vratHodnotuPrava('zobrazit_mzdpol') != 0) {
            $this ->addItem(\Ron\ronText('MI_Mzdove_polozky'), 'mzdovepolozky', 'mi_mzdovepolozky', 'sec_dochazka');
        }

        $this ->addEmptyRow('sec_dochazka');

        $this ->zobrazXMLSablony(3); // dochazka 1   TODO: pridat parametr section?
        $this ->addEmptyRow();

        // pokud jsou prava, tak prida polozku pro vkladani
        if (Prava::vratHodnotuPrava('vkladani') != 0) {
            $this ->addItem(\Ron\ronText('MI_Vlozeni_dochazky'), 'vlozeni', 'mi_vlozeni', 'sec_dochazka');
        }

        if (trim(RonPortalSettings::getValue(RonPortalSettings::HISTORIE_MENU) ?? '') && RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL) && Prava::vratHodnotuPrava('historie') != 0)
            $this ->addItem(RonPortalSettings::getValue(RonPortalSettings::HISTORIE_MENU), 'historie', 'mi_historie', 'sec_dochazka');

        $this ->addEmptyRow('sec_dochazka');

        // --------------------------------------------------------
        // pokud jsou prava, tak prida polozku pro schvalovani
        if (Prava::vratHodnotuPrava('zpracovani') != 0 ||
            Prava::vratHodnotuPrava('zkontrolovani') != 0 ||
            Prava::vratHodnotuPrava('schvaleni') != 0
        ) {
            global $MI_SchvalovaniDochazky; // kompat
            $this ->addItem($MI_SchvalovaniDochazky ?: \Ron\ronText('MI_Schvalovani_dochazky'), 'schvalovani', 'mi_schvalovani', 'sec_dochazka');
        }


        // editace mzdovych polozek a mesicnich vysledku
        //if (RonPortalSettings::getValue(RonPortalSettings::SLOUPCE_EDITACE_MESICNICH)) {
        if ($this ->existujeNastaveni($database, 'RonPortal_Editace_mesicnich', 'SYS_BVAR_GLOBAL', RonPortalSettings::getValue('editacemesicnich'))) {
            if (Prava::vratHodnotuPrava('vkladani_mzdpol') // )
                || Prava::vratHodnotuPrava('editace_mv')     // predelano 19.5.2016 puvodne tu bylo 'editace'
            )
            {
                $this ->addItem(\Ron\ronText('MI_Editace_mesicnich'), 'editacemesicnich', 'mi_editacemesicnich', 'sec_dochazka');
            }
        }

        $this ->addEmptyRow('sec_dochazka');

        $this ->zobrazXMLSablony(4); // dochazka 2
        $this ->addEmptyRow();

        // Planovani, schvalovani
        if (Prava::vratHodnotuPrava('absence') != 0) {
            $this->addItem(\Ron\ronText('MI_Planovani_absence'), 'planovani', 'mi_planovani', 'sec_dochazka');
        }

        global $bezdochazky;
        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI) && Prava::vratHodnotuPrava('absence_schval') != 0) {
            $this ->addItem(\Ron\ronText('MI_Schvalovani_absence'), 'schvalovaniabsence', 'mi_schvalovaniabsence', 'sec_dochazka');
        } else if ( !$bezdochazky ) {
            // neni zapnuto schvalovani nebo nemuze schvalovat
            $poma = [Prava::vratPrava('planovani'), Prava::vratPrava('prohlizeni'), Prava::vratPrava('absence_prehled') ];
            if (Prava::jsouNejakePrava($poma)) $this ->addItem(\Ron\ronText('MI_Prehled_absenci'), 'prehledabsenci', 'mi_prehledabsenci', 'sec_dochazka');
        }

        if (Prava::vratHodnotuPrava('eneschopenka_prohl')) {
            $this->addItem(\Ron\ronText('MI_E_neschopenka'), 'eneschopenka', 'mi_eneschopenka', 'sec_dochazka');
        }

        if (Prava::vratHodnotuPrava('prohlizeni')) {
            if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_KALENDAR_SMEN)) {
                $this ->addItem( \Ron\ronText('Kalendar'), 'kalendar', 'mi_kalendar', 'sec_dochazka');
            }
        }


        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_KAPACITY)) {
            if (Prava::jsouNejakePrava('prohlizeni;kapacity')) {
                global $MI_PrehledSmen; // kompat
                $this ->addItem($MI_PrehledSmen ?: \Ron\ronText('MI_Prehled_smen'), 'prehledsmen', 'mi_prehledsmen', 'sec_dochazka');
            }

            global $MI_PlanovaniKapacit; // kompat
            if (Prava::vratHodnotuPrava('kapacity') != 0)
                $this ->addItem($MI_PlanovaniKapacit ?: \Ron\ronText('MI_Planovani_kapacit'), 'kapacity', 'mi_kapacity', 'sec_dochazka');

            $poma = [Prava::vratPrava('zastupy_vlozeni'), Prava::vratPrava('zastupy_schvaleni'), Prava::vratPrava('zastupy_prideleni'),
                Prava::vratPrava('zastupy_zruseni'), Prava::vratPrava('zastupy_prohl') ];
            if (Prava::jsouNejakePrava($poma)) {
                $this->addItem(\Ron\ronText('MI_Zastupy'), 'zastup', 'mi_zastup', 'sec_dochazka');
            }
        }

        if (Prava::jsouNejakePrava('poznamkykedni')) {
            $this ->addItem( ronText('MI_Poznamky_ke_dni'), 'poznamkykedni', 'mi_poznamkykedni', 'sec_dochazka');
        }

        $this ->addEmptyRow('sec_dochazka');

        $jeSC = false;
        if (RonPortalSettings::getIntValue('povolitsluzebnicesty')  &&  Prava::jsouNejakePrava('sluzebnicesty;slcesty_prohl')) {
            $this ->addItem(\Ron\ronText('MI_Sluzebni_cesty'), 'sluzebnicesty', 'mi_sluzebnicesty', 'sec_dochazka');
            $jeSC = true;
        }

        if (Prava::jsouNejakePrava('vozidla')) {
            $this ->addItem(ronText('MI_Vozidla'), 'vozidla', 'mi_vozidla', 'sec_dochazka');
            $jeSC = true;
        }

        if ($jeSC) {
            $this ->addEmptyRow('sec_dochazka');
        }

        if (RonPortalSettings::getIntValue('povolitpokladniknihu')) {
            $browsery = $this ->nactiPokladniKnihy($database);

            foreach ($browsery as $cisloBrowseru) {
                if ($this ->jePravoNaPokladniKnihu($cisloBrowseru))
                {
                    $this->addItem(\Ron\ronText('MI_Pokladni_kniha_'.$cisloBrowseru), 'pokladnikniha&browser='.$cisloBrowseru,
                        'mi_pokladnikniha_'.$cisloBrowseru, 'sec_dochazka');
                }
            }

            $this->addEmptyRow('sec_dochazka');
        }

        if (RonPortalSettings::getValue('sloupcepoctulidi') && Prava::vratHodnotuPrava('prohlizeni') != 0) {
            $this ->addItem(\Ron\ronText('MI_Pocty_lidi'), 'pocty', 'mi_pocty', 'sec_dochazka');
        }


        // pokud jsou prava, tak prida polozku pro monitorovani
        if (Prava::vratHodnotuPrava('monitorovani') != 0) {
            $this ->addItem(\Ron\ronText('MI_Monitorovani'), 'monitorovani', 'mi_monitorovani', 'sec_dochazka');
        }

        $this ->addEmptyRow('sec_dochazka');


        // pokud jsou prava, tak prida polozku pro navstevy
        if (Prava::vratHodnotuPrava('navstevy') != 0 && RonPortalSettings::getIntValue('povolitnavstevy')) {
            $this->addItem(\Ron\ronText('MI_Navstevy'), 'navstevy', 'mi_navstevy');
        }
        $this ->addEmptyRow();
    }



    // --------------------------------------------------------------------
    // Jidelna
    function pridejJidelnu()
    {
        if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_JIDELNICEK)) $this ->addItem(\Ron\ronText('MI_Jidelnicek'), 'jidelnicek', 'mi_jidelnicek', 'sec_jidelna');

        if (Prava::vratHodnotuPrava('prohl_jidelna') != 0) {
            $this ->addItem(\Ron\ronText('MI_Pohyby_na_uctu'), 'pohybynauctu', 'mi_pohybynauctu', 'sec_jidelna');
            $this ->addItem(\Ron\ronText('MI_Denni_pohyby'), 'dennipohyby', 'mi_dennipohyby', 'sec_jidelna');
            $this ->addItem(\Ron\ronText('MI_Odbery_jidla'), 'puvodniodber', 'mi_puvodniodber', 'sec_jidelna');
            $this ->addItem(\Ron\ronText('MI_Ucet_stravnika'), 'ucetstravnika', 'mi_ucetstravnika', 'sec_jidelna');

            if (Prava::jsouNejakePrava('jidelna_prevedeni_objednavky;jidelna_zruseni_objednavky;jidelna_do_burzy')) {
                $this->addItem(\Ron\ronText('MI_Jidelna_seznam_objednavek'), 'jidelnaobjednavky', 'mi_jidelnaobjednavky', 'sec_jidelna');
            }


            if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_STRAVENKY)) {
                $this ->addItem(\Ron\ronText('MI_Stravenky'), 'stravenky', 'mi_stravenky', 'sec_jidelna');
            }
        }

        $this ->addEmptyRow('sec_jidelna');
    }


    function isItemActive(AppMenuItem $menuItem): bool
    {
        if (($menuItem ->menuId ?? '') === ($_GET['menuid'] ?? null)) {
            return true;
        }

        $pomStr = 'menuid=' . $menuItem ->menuId;
        parse_str($pomStr, $url);

        if ($menuItem ->menuId  &&  isset($_GET['menuid'])  &&  isset($url['menuid'])  &&  $url['menuid'] == $_GET['menuid']) {
            if ($url['menuid'] == 'seznamxmlformularu') {
                return ($_GET['xmlmenuid'] ?? '') === ($url['xmlmenuid'] ?? null);
            } elseif ($url['menuid'] == 'xml') {
                return $url['record'] == ($_GET['record'] ?? '');
            } elseif ($url['menuid'] == 'pokladnikniha') {
                return ($url['browser'] ?? '') === ($_GET['browser'] ?? null);
            }

            return true;
        }

        return false;
    }
}
