<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Model;
use RonPortal\User;
use Ron\StringUtils;
use RonPortal\Controls\ComboBoxOsoba;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;
use function Ron\ronText;

class VydanaObjednavkaFormular extends BaseViewObject
{
    /** @var Database */
    public $database;
    /** @var VydanaObjednavka */
    public $objednavka;
    public $formAction;

    public $comboStredisko;
    public $comboSchvalovatel1;
    public $comboSchvalovatel2;
    public $comboSchvalovatel3;

    public $comboMena;

    /** @var PravaNaOsoby prava na vkladani*/
    private $prava;

    /**
     * VydanaObjednavkaFormular constructor.
     * @param Database $database
     * @param VydanaObjednavka $objednavka
     * @param $action
     * @param $prava
     */
    public function __construct($database, $objednavka, $action, $prava)
    {
        if ($objednavka->readOnly)
            parent::__construct('ronportal/sklad/view_objednavkadetail.php');
        else
            parent::__construct('ronportal/sklad/view_objednavkaform.php');

        $this->database = $database;
        $this->objednavka = $objednavka;
        $this->formAction = $action;

        $this->comboStredisko = null;
        $this->comboMena = null;

        $this->prava = $prava;
    }

    /**
     * @param int $typRozdeleni
     * @param int $index
     * @param Database $database
     * @param null|string $stredisko
     * @return string
     */
    public static function getSQLRozdeleni($typRozdeleni, $index, $database, $stredisko = null)
    {
        $params = $database->createParameters();

        $kodPrava = RonPortalSettings::getValue("vydanaobjednavka_typrozdeleni{$index}_prava");
        if ($kodPrava)
            $pravaRozdeleni = Prava::vratPrava($kodPrava);

        $sql =
            "SELECT Kod, Popis, cssClass, cssClassObrazku FROM Rozdeleni WHERE DatumVytvoreni <= " . $params->addDateParameter(DateUtils::today()) .
            "  AND (DatumZruseni >= ". $params->addDateParameter(DateUtils::today()) . " OR DatumZruseni IS NULL)" .
            "  AND KodTypuRozdeleni = " . $params->addParameter($typRozdeleni);
        if (isset($pravaRozdeleni))
            $sql .= " AND (" . $pravaRozdeleni->sqlfiltrRozdeleni(DateUtils::today(), DateUtils::today(), $params) . ")";
        if (RonPortalSettings::getIntValue("vydanaobjednavka_rozdeleni{$index}_nejnizsi_uroven"))
            $sql .= " AND NOT EXISTS(SELECT Kod FROM Rozdeleni R WHERE R.RozdeleniRodice = Rozdeleni.KodTypuRozdeleni AND R.Rodic = Rozdeleni.Kod) ";

        if (RonPortalSettings::getIntValue("vydanaobjednavka_rozdeleni{$index}_filtr_stredisko") && $stredisko) {
            $sql .=
                " AND (NOT EXISTS(SELECT 1 FROM VazbaRozdeleni " .
                "       WHERE VazbaRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni " .
                "       AND VazbaRozdeleni.kodNadrazene = " . $params->addParameter($stredisko) .
                "       AND VazbaRozdeleni.PlatnostOd <= " . $params->addDateParameter(DateUtils::today()) .
                "       AND (VazbaRozdeleni.PlatnostDo >= " . $params->addDateParameter(DateUtils::today()) . " OR VazbaRozdeleni.PlatnostDo IS NULL)) " .

                "   OR EXISTS(SELECT 1 FROM HierarchieRozdeleni JOIN VazbaRozdeleni ON VazbaRozdeleni.typPodrizene = HierarchieRozdeleni.typNadrazene AND VazbaRozdeleni.kodPodrizene = HierarchieRozdeleni.kodNadrazene " .
                "       WHERE HierarchieRozdeleni.typNadrazene = Rozdeleni.KodTypuRozdeleni " .
                "       AND HierarchieRozdeleni.typPodrizene = Rozdeleni.KodTypuRozdeleni " .
                "       AND HierarchieRozdeleni.kodPodrizene = Rozdeleni.Kod " .
                "       AND VazbaRozdeleni.kodNadrazene = " . $params->addParameter($stredisko) .
                "       AND VazbaRozdeleni.PlatnostOd <= " . $params->addDateParameter(DateUtils::today()) .
                "       AND (VazbaRozdeleni.PlatnostDo >= " . $params->addDateParameter(DateUtils::today()) . " OR VazbaRozdeleni.PlatnostDo IS NULL)) " .
                ") ";
        }

        $sql .= " ORDER BY Kod";

        return new SqlStatement($sql, $params);
    }

    /**
     * @throws DBException
     */
    public function load()
    {
        $typRozdeleniFiltrStredisek = RonPortalSettings::getIntValue('vydanaobjednavka_filtrovat_stredisko_tr');

        if ($this->prava->jsouNejakePrava()) {
            $pravaStredisko = Prava::vratPrava('objednavka_vydana_stredisko');
            $params = $this->database->createParameters();
            $sql =
                "SELECT kod, Popis FROM Stredisko WHERE EXISTS(SELECT Kod FROM Rozdeleni WHERE Rozdeleni.Kod = Stredisko.kod AND Rozdeleni.KodTypuRozdeleni = " . $params->addParameter($typRozdeleniFiltrStredisek) .
                ' AND ' . $pravaStredisko->sqlfiltrRozdeleni(DateUtils::today(), DateUtils::today(), $params) . ')';

            MessageManager::addDebugLog('Stredisko: ' .$sql);
            $pp = PracovniPomer::nactiPPOsoby($this->database, User::getRC());
            if (count($pp) == 1) {
                /** @var PracovniPomer $pp */
                $pp = $pp[0];
                $rozdeleni = Rozdeleni::vratRozdeleniOsoby($this->database, $pp->idPP, $typRozdeleniFiltrStredisek);
            }

            $stredisko = $this->objednavka->stredisko ?: (isset($rozdeleni) ? $rozdeleni : null);

            $combo = new ComboBox('stredisko');
            $combo->loadData($this->database, $sql . 'ORDER BY Popis', $stredisko, $params);
            $combo->className .= ' objednavka-stredisko form-input ronselect '  . MessageManager::getClass('stredisko');

            $this->comboStredisko = $combo;

            if (isset(ObjednavkyController::$typRozdeleniVydaneObjednavky[1])) {
                $this->loadComboRozdeleni(ObjednavkyController::$typRozdeleniVydaneObjednavky[1], 1, isset($this->objednavka->rozdeleni1) ? $this->objednavka->rozdeleni1->kod : null, $stredisko);
            }
            if (isset(ObjednavkyController::$typRozdeleniVydaneObjednavky[2])) {
                $this->loadComboRozdeleni(ObjednavkyController::$typRozdeleniVydaneObjednavky[2], 2, isset($this->objednavka->rozdeleni2) ? $this->objednavka->rozdeleni2->kod : null, $stredisko);
            }
            if (isset(ObjednavkyController::$typRozdeleniVydaneObjednavky[3])) {
                $this->loadComboRozdeleni(ObjednavkyController::$typRozdeleniVydaneObjednavky[3], 3, isset($this->objednavka->rozdeleni3) ? $this->objednavka->rozdeleni3->kod : null, $stredisko);
            }
            if (isset(ObjednavkyController::$typRozdeleniVydaneObjednavky[4])) {
                $this->loadComboRozdeleni(ObjednavkyController::$typRozdeleniVydaneObjednavky[4], 4, isset($this->objednavka->rozdeleni4) ? $this->objednavka->rozdeleni4->kod : null, $stredisko);
            }
            if (isset(ObjednavkyController::$typRozdeleniVydaneObjednavky[5])) {
                $this->loadComboRozdeleni(ObjednavkyController::$typRozdeleniVydaneObjednavky[5], 5, isset($this->objednavka->rozdeleni5) ? $this->objednavka->rozdeleni5->kod : null, $stredisko);
            }

            $sql =
                "SELECT KodMeny_ISO FROM Mena WHERE Aktivni = 1 ORDER BY KodMeny_ISO";
            $combo = new ComboBox('mena');
            $combo->captionIndex = 0;
            $combo->loadData($this->database, $sql, $this->objednavka->mena);
            $combo->emptyRow = '&nbsp;';
            $combo->className .= ' objednavka-mena ' . MessageManager::getClass('mena');

            $this->comboMena = $combo;
        }


        $params = $this->database->createParameters();
        $sql =
            "SELECT Rodic FROM Rozdeleni WHERE KodTypuRozdeleni = " . $params->addParameter($typRozdeleniFiltrStredisek) . " AND Kod = " . $params->addParameter($this->objednavka->stredisko);

        $rozdeleniSchvalovatel1 = DBUtils::getSqlValue($this->database, $sql, $params);

        if ($rozdeleniSchvalovatel1 === false)
            $rozdeleniSchvalovatel1 = null;

        $this->comboSchvalovatel1 = $this->loadComboSchvalovatel('schvalovatel1', $typRozdeleniFiltrStredisek, $rozdeleniSchvalovatel1, $this->objednavka->schvaleni1RC);

        $params = $this->database->createParameters();
        $sql =
            "SELECT Rodic FROM Rozdeleni WHERE KodTypuRozdeleni = " . $params->addParameter($typRozdeleniFiltrStredisek) . " AND Kod = " . $params->addParameter($rozdeleniSchvalovatel1);

        $rozdeleniSchvalovatel2 = DBUtils::getSqlValue($this->database, $sql, $params);

        $this->comboSchvalovatel2 = $this->loadComboSchvalovatel('schvalovatel2', $typRozdeleniFiltrStredisek, $rozdeleniSchvalovatel2, $this->objednavka->schvaleni2RC);
    }

    /**
     * @param string $varName
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param $selected
     * @return ComboBoxOsoba
     * @throws DBException
     */
    private function loadComboSchvalovatel($varName, $typRozdeleni, $rozdeleni, $selected)
    {
        $combo = new ComboBoxOsoba($varName);
        $sqltext =
            'SELECT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, Osoba.Titul, Osoba.IdFotografie, ' . // 0 - 4
            ' PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID, ' . // 5 - 7
            ' PracovniPomer.DatumUkonceni '; // 8

        // prida uzivatelske pole z nastaveni intrawebu
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) $sqltext .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1); // (9)
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) $sqltext .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2); // (9 nebo 10)

        $sqltext .=
            "\nFROM Osoba, PracovniPomer\n" .
            ' WHERE Osoba.RC = PracovniPomer.RC ';

        $sqltext .= ' AND (PracovniPomer.Zamestnanec = 1  OR  PracovniPomer.Zamestnanec = 2)';

        $sqltext = $sqltext .
            ' AND PracovniPomer.DatumNastupu <= ' . $this->database ->dateToString(DateUtils::today()) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $this->database ->dateToString(DateUtils::today()) . ' OR PracovniPomer.DatumUkonceni IS NULL)';

        $filtr = $combo->vratFiltrRozdeleni($this->database, DateUtils::today(), DateUtils::today(),
            (int)$typRozdeleni, $rozdeleni, 0);

        if ($filtr) $sqltext .= "\nAND (" . $filtr . ')';


        $combo->loadSQLPP($this->database, $sqltext, null, true, $selected, 0, 0);
        $combo->className .= ' ronselect';
        return $combo;
    }

    /**
     * @param int $typRozdeleni
     * @param int $index
     * @param string $selectedValue
     * @param string $stredisko
     * @return void
     * @throws DBException
     * @throws Exception
     */
    private function loadComboRozdeleni($typRozdeleni, $index, $selectedValue, $stredisko)
    {
        $typ = TypRozdeleni::getTypRozdeleni($this->database, $typRozdeleni);
        Model::addData("typRozdeleni$index", $typ->kod);
        Model::addData("typRozdeleni{$index}Popis", $typ->popis);

        if (RonPortalSettings::getIntValue('vydanaobjednavka_dodavatel_dle_rozdeleni') == $index) {
            $rozdeleniParams['data-url-dodavatel'] = '?menuid=objednavky&action=seznamdodavatelu&rozdeleni=';
        }

        if (RonPortalSettings::getIntValue("vydanaobjednavka_rozdeleni{$index}_filtr_stredisko")) {
            $rozdeleniParams['data-url-seznam'] = "?menuid=objednavky&action=seznamrozdeleni&typrozdeleni={$typRozdeleni}&index={$index}&stredisko=";
        }

        $suffix = '';
        if ($index > 1)
            $suffix = $index;

        $combo = new ComboBox("rozdeleni$suffix");
        $combo->loadData($this->database, static::getSQLRozdeleni($typRozdeleni, $index, $this->database, $stredisko), $selectedValue);
        $combo->showCode = true;
        $combo->emptyRow = '&nbsp;';
        $combo->cssClassIndex = 2;
        $combo->className .= ' ronselect form-input ' . MessageManager::getClass("rozdeleni$suffix");

        if (isset($rozdeleniParams))
            $combo->htmlParams = $rozdeleniParams;

        $combo->htmlParams['data-input-class'] = 'form-input ' . MessageManager::getClass("rozdeleni$suffix");

        Model::addData("comboRozdeleni$index", $combo);
    }

    /**
     * @param RadekVydaneObjednavky $radek
     * @param int $index
     */
    public function renderComboZbozi($radek = null, $index = 1)
    {
        $title = ronText('Zbozi');
        $suffix = (is_null($radek) ? 'novy_radek_' . $index  :  $radek->cisloRadku);
        $varName = 'zbozi_' . $suffix;

        $captions = ronText('Kod') . ';' . ronText('Popis') . ';' . ronText('Kod_dodavatele') . ';' . ronText('Cenabezdane') . ';' . ronText('Poznamka');

        echo '<div class="input-group">';
        echo "<input name='{$varName}_view' id='{$varName}_view'  
                                                    data-ron-combo-base-url='?menuid=objednavky&action=zbozidodavatele&dodavatel='
                                                    data-ron-combo-url='?menuid=objednavky&action=zbozidodavatele&dodavatel={$this->objednavka->dodavatel}'
                                                    data-ron-combo-code='{$varName}'
                                                    data-ron-combo-group='objednavka_zbozi'
                                                    data-ron-combo-fields='kod;popis;koddodavatele;cenabezdane;poznamka'
                                                    data-ron-combo-text-field='popis'
                                                    data-ron-combo-row-suffix='{$suffix}'
                                                    data-captions='{$captions}'
                                                    title='{$title}'
                                                    class='objednavka_zbozi ron_zbozi_combo form-input form-select' value=''/>";

        echo "<input type='hidden' name='{$varName}' value='{$radek->kodZbozi}'>";
        echo '</div>';
    }

    /**
     *
     */
    public function renderComboDodavatel()
    {
        $title = ronText('Dodavatel');
        $varName = 'dodavatel';

        $captions = ronText('Nazev1') . ';' . (ObjednavkyController::$indexUzivatelskehoFiltruDodavatele ? ObjednavkyController::$nadpisUzivatelskehoFiltruDodavatele . ';' : '') .
            ronText('Klicova_slova') . ';' . ronText('Poznamka');

        $class = MessageManager::getClass('dodavatel');
        $rozdeleni = isset($this->objednavka->rozdeleni1) ? '&rozdeleni=' . $this->objednavka->rozdeleni1->kod : '';

//        echo '<div class="input-group form-input">';
        echo "<input name='{$varName}_view' id='{$varName}_view' 
                                                    data-ron-combo-spinner='loadingSpinnerDodavatel' 
                                                    data-ron-combo-url='?menuid=objednavky&action=seznamdodavatelu$rozdeleni'
                                                    data-ron-combo-code='{$varName}'
                                                    data-ron-combo-group='objednavka_dodavatel'
                                                    data-ron-combo-fields='nazev1;" . (ObjednavkyController::$indexUzivatelskehoFiltruDodavatele ? 'filtr;' : '') . "klicova-slova;poznamka'
                                                    data-ron-combo-text-field='nazev1'
                                                    data-url-zbozi ='?menuid=objednavky&action=zbozidodavatele&dodavatel='
                                                    data-selected='{$this->objednavka->dodavatel}'
                                                    data-ronselect-class='ronselect form-input {$class}'
                                                    data-captions='{$captions}'
                                                    title='{$title}'                                                    
                                                    class='objednavka_dodavatel ron_dodavatel_combo form-input {$class}' value=''/>";
//        echo "<i id='loadingSpinnerDodavatel' class='fa glyph-spinner fa-spin loadingSpinnerZbozi position-absolute position-top position-right margin-right-2 margin-top-2'></i>";

        echo "<input type='hidden' name='{$varName}' value='{$this->objednavka->dodavatel}'>";
//        echo '</div>';
    }

    /**
     * @param $uroven
     * @param bool $zobrazitOdkazy
     * @throws Exception
     */
    public function renderNastavSchvaleni($uroven, $zobrazitOdkazy = true)
    {
        $schvalovatel = null;
        $datumSchvaleni = null;
        $schvalil = null;
        $stavSchvaleni = null;
        $combo = null;
        switch ($uroven) {
            case 1 :
                $schvalovatel = $this->objednavka->nastavenySchvalovatel1;
                $datumSchvaleni = $this->objednavka->schvaleni1Datum;
                $schvalil = $this->objednavka->schvaleni1Jmeno;
                $stavSchvaleni = $this->objednavka->schvaleni1Stav;
                $combo = $this->comboSchvalovatel1;
                break;
            case 2 :
                $schvalovatel = $this->objednavka->nastavenySchvalovatel2;
                $datumSchvaleni = $this->objednavka->schvaleni2Datum;
                $schvalil = $this->objednavka->schvaleni2Jmeno;
                $stavSchvaleni = $this->objednavka->schvaleni2Stav;
                $combo = $this->comboSchvalovatel2;
                break;
            case 3 :
                $schvalovatel = $this->objednavka->nastavenySchvalovatel3;
                $datumSchvaleni = $this->objednavka->schvaleni3Datum;
                $schvalil = $this->objednavka->schvaleni3Jmeno;
                $stavSchvaleni = $this->objednavka->schvaleni3Stav;
                $combo = $this->comboSchvalovatel3;
                break;
        }

        if (trim($schvalovatel) !== '') {
            echo "<span class='font-weight-bold'>" . StringUtils::htmlTextConvert($schvalovatel) . "</span>" . '&nbsp;';
            //pokud objednavka jeste nebyla schvalena, nebo byla vracena zobrazi odkaz na predani ke schvaleni
            if ($this->objednavka->lzeSchvalovat($uroven))
                echo " <a href='?menuid=objednavky&action=odeslikeschvaleni&record=" . $this->objednavka->cisloObjednavky . "'>" . ronText('Odeslat_ke_schvaleni') . "</a>";
        }
        if ($datumSchvaleni) {
            echo '&nbsp;' . '<span class="font-italic">' . StringUtils::format('date', $datumSchvaleni) . ': ' . StringUtils::htmlTextConvert($schvalil) . '&nbsp;' . '&nbsp;' . VydanaObjednavka::vratPopisStavuSchvaleni($stavSchvaleni) . '</span>';
        }

        if ($zobrazitOdkazy) {
            if ($this->objednavka->lzeSchvalit($uroven)) {
                //pro stav vraceno je mozno znovu schvalit
                echo " <a class='schvaleni_link' data-url='?menuid=objednavky&action=schvalobjednavku{$uroven}&record=" . $this->objednavka->cisloObjednavky . "' 
                    href='?menuid=objednavky&action=schvalobjednavku{$uroven}&record=" . $this->objednavka->cisloObjednavky . "'>" . ronText('Schvalit_vydanou_objednavku') . "</a>";

                echo "&nbsp;<a href='?menuid=objednavky&action=vraceni&record=" . $this->objednavka->cisloObjednavky . "' data-uroven='{$uroven}' data-text-class='poznamka_vraceni' class='vyjadreni_k_objednavce_link'>" . ronText('Vratit_objednavku') . "</a>";
                echo "&nbsp;<a href='?menuid=objednavky&action=zamitnuti&record=" . $this->objednavka->cisloObjednavky . "' data-uroven='{$uroven}' data-text-class='poznamka_zamitnuti' class='vyjadreni_k_objednavce_link'>" . ronText('Zamitnout') . "</a>";
                echo "&nbsp;" . GuiUtils::renderCheckbox('vyzadovat_schvaleni', ronText('Vyzadovat_schvaleni'), $this->objednavka->vyzadovatSchvaleni, 'id="vyzadovat_schvaleni"', "vyzadovat_schvaleni_{$uroven}");
            } elseif ($this->objednavka->lzeNastavitSchvalovatele($uroven) && !is_null($combo)) {
                echo '<form method="post" name="nastavenischvalovatele" action="?menuid=objednavky&action=nastavschvalovatele&record=' . $this->objednavka->cisloObjednavky . '">';
                echo '<input type="hidden" name="uroven" value="' . $uroven . '" />';
                $combo->render();
                echo '<input type="submit" class="submitbutton" value="OK">';

                echo '</form>';
            }
        }
    }

    /**
     * @throws Exception
     */
    public function render()
    {
        $lzeVlozitDokument = false;

        $pravaFakturovat = Prava::vratPrava('objednavka_vydana_faktura');

        if ($this->objednavka->lzeEditovat() && $this->prava->lzeZobrazitRC($this->objednavka->vystavil) && $this->prava->priznak)
            $lzeVlozitDokument = true;
        if ($this->objednavka->lzeFakturovat(false) && $pravaFakturovat->lzeZobrazitRC($this->objednavka->vystavil))
            $lzeVlozitDokument = true;

        Model::addData('database', $this->database);
        Model::addData('objednavka', $this ->objednavka);
        Model::addData('comboStredisko', $this ->comboStredisko);
        Model::addData('comboSchvalovatel1', $this ->comboSchvalovatel1);
        Model::addData('comboSchvalovatel2', $this ->comboSchvalovatel2);
        Model::addData('comboMena', $this->comboMena);
        Model::addData('formAction', $this ->formAction);
        Model::addData('lzeeditovat', $lzeVlozitDokument);

        parent::render();
    }
}

class VyberDodavateleCombo extends ComboBox implements ICreateItem
{
    function __construct($varName = 'dodavatel')
    {
        parent::__construct($varName);

        $this ->noHtmlConvert = true; // htmlTextConvert popisu zajistuje primo tato trida
    }

    function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = StringUtils::htmlTextConvert($row[0]);
        $caption = StringUtils::htmlTextConvert($row[1]);

        $option = new Option($key, $caption);
        $option->params = [
            'data-kod' => StringUtils::htmlTextConvert($row[0]),
            'data-nazev1' => StringUtils::htmlTextConvert($row[1]),
            'data-nazev2' => StringUtils::htmlTextConvert($row[2]),
            'data-klicova-slova' => StringUtils::htmlTextConvert($row[3]),
            'data-poznamka' => StringUtils::htmlTextConvert($row[4])
        ];

        if (ObjednavkyController::$indexUzivatelskehoFiltruDodavatele)
            $option->params['data-filtr'] = StringUtils::htmlTextConvert($row[5]);

        return $option;
    }
}