<?php
/**
 * stanislav.elbl, 14.09.2022 11:05
 */

namespace RonPortal\Skoleni;

use Ron\DateUtils;
use Ron\DB\DBRecordField;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;

class GenerovaneSkoleniRecord extends RonPortalDBRecord
{
    /** @var DBRecordField */
    public $fRC;
    /** @var DBRecordField */
    public $fDruhSkoleni;
    /** @var DBRecordField */
    public $fDatumVznikuPovinnosti;

    /** @var DBRecordField */
    public $fDuvodSkoleni;

    /** @var DBRecordField */
    public $fPotvrzeni;
    /** @var DBRecordField */
    public $fSchvaleniPrihlasky;

    /** @var DBRecordField */
    public $fDatumSkoleni;
    /** @var DBRecordField */
    public $fPozadovaneDatumSkoleni;
    /** @var DBRecordField */
    public $fCisloSkoleni;

    /** @var DBRecordField */
    public $fDatumPrihlasky;
    /** @var DBRecordField */
    public $fPrihlasenoZAplikace;
    /** @var DBRecordField */
    public $fPrihlasil;

    /** @var DBRecordField */
    public $fVyhodnotil;
    /** @var DBRecordField */
    public $fDatumVyhodnoceni;


    /**
     * @param \Ron\DB\Database $database
     * @param object|string $logDescription
     */
    public function __construct($database, $logDescription) // $rc, $druhSkoleni, $datumVznikuPovinnosti,
    {
        parent::__construct($database, 'GenerovaneSkoleni', $logDescription);

        $this ->fRC = $this->addStringField('RC', true, 15);
        //$record->addLogText('  ' . $pp ->jmenoPrijmeni);

        $this ->fDruhSkoleni = $this ->addStringField('DruhSkoleni', true, 10);
        $this ->fDatumVznikuPovinnosti = $this->addTimestampField('DatumVznikuPovinnosti', true, DateUtils::now());

        $this ->fDuvodSkoleni = $this->addIntField('DuvodSkoleni', true, 1); // individualni
        $this ->fPotvrzeni = $this->addIntField('Potvrzeni', true, 0);
        $this ->fSchvaleniPrihlasky = $this->addIntField('SchvaleniPrihlasky', true);

        $this ->fPozadovaneDatumSkoleni = $this->addDateField('PozadovaneDatumSkoleni', true);  // , $skoleni ->datumPocatkuSkoleni
        $this ->fDatumSkoleni = $this->addDateField('DatumSkoleni', true);  // , $skoleni ->datumPocatkuSkoleni
        $this ->fCisloSkoleni = $this->addStringField('CisloSkoleni', true, 40);

        $this ->fDatumPrihlasky = $this->addDateField('DatumPrihlasky', true);  // , time()
        $this ->fPrihlasenoZAplikace = $this->addIntField('PrihlasenoZAplikace', true); // , 1
        $this ->fPrihlasil = $this ->addStringField('Prihlasil', true, 25); // User::getLoginName()

        $this ->fVyhodnotil = $this->addStringField('Vyhodnotil', true, 25);
        $this ->fDatumVyhodnoceni = $this->addDateField('DatumVyhodnoceni', true);

        // dalsi readonly pole
        $this ->addDateField('DatumZapisu', true, DateUtils::today());
        $this ->addStringField('Vlozil', true, 25, User::getLoginName());

        $this ->addInsertLogFields();
        $this ->addUpdateLogFields();
    }


    /**
     * @param string|PracovniPomer $rc
     * @param string $druhSkoleni
     * @param int $datumVznikuPovinnosti
     * @throws \Exception
     */
    public function setWhereSkoleni($rc, $druhSkoleni, $datumVznikuPovinnosti)
    {
        $rcStr = ($rc instanceof PracovniPomer) ? $rc ->rc : $rc;

        $this ->fRC ->setValue($rcStr);
        $this ->fDruhSkoleni ->setValue($druhSkoleni);

        $params = $this ->database ->createParameters();

        if (isset($datumVznikuPovinnosti)) {
            $this ->fDatumVznikuPovinnosti ->setValue($datumVznikuPovinnosti);

            $this->setWhere(
                'RC = ' . $params->addParameter($rcStr) .
                ' AND DruhSkoleni = ' . $params->addParameter($druhSkoleni) .
                ' AND DatumVznikuPovinnosti = ' . $params->addTimestampParameter($datumVznikuPovinnosti),
                $params);
        } else {
            $this->setWhere(
                'RC = ' . $params->addParameter($rcStr) .
                ' AND DruhSkoleni = ' . $params->addParameter($druhSkoleni) .
                ' AND DatumSkoleni IS NULL' .
                ' AND NOT EXISTS(SELECT * FROM GenerovaneSkoleni GS WHERE GS.RC = GenerovaneSkoleni.RC' .
                '  AND GS.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni' .
                '  AND GS.DatumVznikuPovinnosti > GenerovaneSkoleni.DatumVznikuPovinnosti' .
                '  AND GS.CisloSkoleni IS NOT NULL)',
                $params);
        }
    }
}
