<?php
/**
 * stanislav.elbl, 18.08.2022 16:11
 */

namespace RonPortal\Skoleni;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;

class PozadavekNaSkoleniForm extends RonPortalForm
{
    const STAV_NEW = 0;
    const STAV_VYBER_DRUHU = 1;
    const STAV_ALL = 2;

    private $stav = self::STAV_NEW;

    /** @var PracovniPomer  Uzivatelem vybrana osoba */
    //private $pp = null;

    private $druhSkoleni;

    /** @var FormField */
    private $fDruhSkoleni;

    function __construct($database)
    {
        parent::__construct($database);

        $this ->sqlTableName = 'GenerovaneSkoleni';

        $this ->code = 'formgenerovaneskoleni';
        $this ->stornoAction = '?menuid=skoleni';
        $this ->setXmlEventsName('GENEROVANE_SKOLENI');

        $this ->nastavStav();

        // zobrazi vyber osoby
        $this->addField(new OsobaFormField('skoleni_vyslat', DateUtils::now()))
            ->setFieldName('osoba')->setSqlFieldName(null)
            ->setCode('field-osoba')
            ->setCaption('Osoba')
            ->setRequired(true);


        if ($this ->stav === self::STAV_NEW) {
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*stav') ->setDefaultValue(self::STAV_VYBER_DRUHU);

            // vyber typu skoleni zobrazi jen v prvnim kroku
            // v dalsim uz nepujde zmenit, protoze jsou na nej navazana dalsi pole (vyber konkretniho skoleni)
            $this ->fDruhSkoleni = $this ->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('DruhSkoleni') ->setCaption('typ_skoleni')
                ->setCode('field-druhskoleni')
                ->setSelectSql('SELECT DruhSkoleni.Kod, DruhSkoleni.Popis, KategorieSkoleni.Popis 
                            FROM DruhSkoleni LEFT OUTER JOIN KategorieSkoleni ON DruhSkoleni.Kategorie = KategorieSkoleni.Kod 
                            WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'DruhSkoleni.Popis')
                ->setRequired(true)
                ->setFieldCssClass('form-control ronselect')
                ->setItemsAttributes([
                    'data-druh-skoleni' => 0, 'data-druh-skoleni-popis' => 1,
                    'data-kategorie' => 2])
                ->setOtherHtmlParams(['data-columns' => 'druh-skoleni;druh-skoleni-popis;kategorie',
                    'data-captions' => TextManager::_('Kod') . ';' . TextManager::_('Nazev') . ';' . TextManager::_('Skoleni_kategorie')])
                ->setShowEmptyValue(true);
        } else {
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*stav') ->setDefaultValue(self::STAV_ALL);

            // druh skoleni uz je nastaven - prida ostatni pole
            $this ->fDruhSkoleni = $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('DruhSkoleni') ->setDefaultValue($this ->druhSkoleni);

            $params = $this ->database ->createParameters();
            $popisDruhu = DBUtils::getSqlValue($this ->database,
                'SELECT Popis FROM DruhSkoleni WHERE Kod = ' . $params ->addParameter($this ->druhSkoleni),
                $params);
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-popisdruhuskoleni')
                ->setText($popisDruhu) ->setCaption('typ_skoleni') ->setRequired(true);

            $kurzParams = $this ->database ->createParameters();
            $this ->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('CisloSkoleni') ->setCaption('Skoleni_kurz')
                ->setCode('field-cisloskoleni') ->setFieldId('pozadavek_na_skoleni_kurz')
                ->setSelectSql(
                    new SqlStatement(
                        'SELECT CisloSkoleni, Popis FROM Skoleni' .
                        ' WHERE DruhSkoleni = ' . $kurzParams ->addParameter($this ->druhSkoleni) .
                        ' AND %FILTER% ORDER BY %ORDER%',
                        $kurzParams),
                    'StavSkoleni IN (0,1)', 'Popis')
                ->setShowEmptyValue(true) ->setSelectShowCode(true);

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCaption('Skoleni_kapacita') ->setFieldId('pozadavek_na_skoleni_kapacita')
                ->setCode('field-kapacita-info');

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCaption('Skoleni_prihlasenych_kurz') ->setFieldId('pozadavek_na_skoleni_pocet_prihlasenych')
                ->setCode('field-pocet-prihlasenych-info');

            $this ->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('PozadovaneDatumSkoleni') ->setCaption('Pozadovane_datum_skoleni')
                ->setCode('field-pozadovanedatumskoleni') ->setFieldId('pozadavek_na_skoleni_pozadovane_datum')
                ->setRequired(true);

            $this ->addField()
                ->setFieldName('PopisSkoleni') ->setCaption('Skoleni_popis_skoleni')
                ->setCode('field-popisskoleni');

            // dokument
            if (RonPortalSettings::getIntValue('skoleni_typ_dokumentu')) {
                $this ->addField(FormField::FIELD_TYPE_FILE)
                    ->setFieldName('dokument') ->setCaption('Skoleni_dokument_vyslani_na_skoleni')
                    ->setCode('field-dokument');

                $this ->addField()
                    ->setFieldName('*popisDokumentu') ->setCaption('Skoleni_popis_dokument_vyslani_na_skoleni')
                    ->setCode('field-popis-dokumentu');
            }

            // memo poznamka
            $this ->addField(FormField::FIELD_TYPE_MEMO)
                ->setFieldName('Poznamka') ->setCaption('Poznamka');

        }

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('DuvodSkoleni')
            ->setCode('field-duvodskoleni')
            ->setDefaultValue(1)
            ->setReadOnly(true);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Potvrzeni')
            ->setCode('field-potvrzeni')
            ->setDefaultValue(1)
            ->setReadOnly(true);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Generovane')
            ->setCode('field-generovane')
            ->setDefaultValue(0)
            ->setReadOnly(true);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('PrihlasenoZAplikace')
            ->setCode('field-prihlasenozaplikace')
            ->setDefaultValue(1)
            ->setReadOnly(true);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Prihlasil')
            ->setCode('field-prihlasil')
            ->setDefaultValue(RonPortalUtils::getLogUserName())
            ->setReadOnly(true)
            ->setMaxLength(25);
    }


    /**
     * Podle hodnot v POST nastavi stav
     */
    private function nastavStav()
    {
        $this->druhSkoleni = $_POST['DruhSkoleni'] ?? null;

        if (!$this ->druhSkoleni) {
            $this ->stav = self::STAV_NEW;
        } else {
            $this->stav = (int)($_POST['stav'] ?? self::STAV_NEW);
//            switch ((int)($_POST['stav'] ?? 0)) {
//                case self::STAV_VYBER_DRUHU :
//                    $this ->stav = self::STAV_ALL;
//                    break;
//
//                case self::STAV_ALL :
//                case self::STAV_POST :
//                    $this ->stav = self::STAV_POST;
//                    break;
//
//                default :
//                    $this ->stav = self::STAV_VYBER_DRUHU;
//            }
        }


//        if ($this ->stav > self::STAV_VYBER_DRUHU) {
//            // osoba by mela byt v post
//            // - nacte ji a overi, jestli ma uzivatel pravo na vybranou osobu
//            $pp = PracovniPomer::nactiPP($this ->database, $_POST['osoba']);
//
//            if (!Prava::vratPrava('skoleni_vyslat') ->lzeZobrazitRC($pp ->rc)) {
//                $this ->stav = self::STAV_VYBER_OSOBY;
//                MessageManager::addErrorMessage(TextManager::_('Nemate_pravo'));
//                return;
//            }
//
//            $this ->pp = $pp;
//        }

//        if ($this ->stav > self::STAV_VYBER_DRUHU) {
//            if (!isset($_POST['DruhSkoleni'])  ||  trim($_POST['DruhSkoleni']) === '') {
//                $this ->stav = self::STAV_VYBER_DRUHU;
//            }
//        }
    }



    public function createDBRecordFields($rec, $insert)
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        if (!Prava::vratPrava('skoleni_vyslat') ->lzeZobrazitRC($pp ->rc)) {
            throw new NeniPravoException();
        }

        $rec ->addStringField('RC', true, 15, $pp->rc);
        $rec ->addDateField('DatumVznikuPovinnosti', true, DateUtils::today());

        parent::createDBRecordFields($rec, $insert);

        if ($rec['CisloSkoleni']) {
            $skoleni = Skoleni::nactiSkoleni($this ->database, $rec['CisloSkoleni']);

            if (!$skoleni->jeVolnaKapacitaSkoleni($this ->database)) {
                throw new \Exception(sprintf(TextManager::_('SkoleniNaplnenaKapacita'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }

            if (SkoleniUtils::jeOsobaNaSkoleni($this->database, $rec['CisloSkoleni'], $pp->rc)) {
                throw new \Exception(sprintf(TextManager::_('SkoleniJizPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            }
        }
    }


    function beforeInsertRecord($rec)
    {
        if ($rec['PozadovaneDatumSkoleni'] < DateUtils::today()) {
            throw new \Exception( TextManager::_('Pozadovane_datum_skoleni_nesmi_byt_v_minulosti') );
        }

        // TODO: doplnit kontrolu existence zaznamu - mozna obecne ve formulari?

        return parent::beforeInsertRecord($rec);
    }


    function afterInsertRecord($rec)
    {
        if (is_uploaded_file($_FILES['dokument']['tmp_name'])) {
            $idDokumentu = DokumentyUtils::vlozDokument($this->database,
                'dokument', RonPortalSettings::getIntValue('skoleni_typ_dokumentu'),
                $_POST['popisDokumentu'], $this);

            $recDokumentOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this);
            $recDokumentOsoby->addField('RC', '', true, 15, $rec['RC']);
            $recDokumentOsoby->addField('id', 'int', true, 0, $idDokumentu);
            $recDokumentOsoby->addField('GSk_RC', '', true, 15, $rec['RC']);
            $recDokumentOsoby->addField('GSk_DruhSkoleni', '', true, 20, $rec['DruhSkoleni']);
            $recDokumentOsoby->addTimestampField('GSk_Datum', true, $rec['DatumVznikuPovinnosti']);

            $recDokumentOsoby->insertRecord();
        }

        MessageManager::addSuccessMessage(TextManager::_('Pozadavek_na_skoleni_uspesne_zapsan'));
        return parent::afterInsertRecord($rec);
    }


    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);
        if ($this ->stav === self::STAV_NEW) {
            $result['submit']['value'] = TextManager::_('Pokracovat');
        }
        return $result;
    }


    public function checkRequiredFields($edit)
    {
        if ($this ->stav !== self::STAV_ALL) {
            // zkontroluje jen typ skoleni
            if (!$this ->fDruhSkoleni ->checkRequired(false)) {
                MessageManager::getInstance()->addError(TextManager::_('form.Missing_value') . $this ->fDruhSkoleni->caption, $this ->fDruhSkoleni->fieldName);
            }

            return false;
        }

        return parent::checkRequiredFields($edit);
    }


}
