<?php
/**
 * Roman.Szablatura RON Software 19.07.2019
 */

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Prava;


/**
 * Class SchvalovaniOsobNaPlanuKurzuFilterForm
 */
class SchvalovaniOsobNaPlanuKurzuFilterForm extends SkoleniFilterForm
{
//    /** @var string */
//    public $cisloSkoleni;

    /**
     * SchvalovaniOsobNaPlanuKurzuFilterForm constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni', true);

        Prava::nastavObdobiProPrava(time(),time());

        $this->f_classname = 'filtercontainer leftcaptions';
        $this->f_formaction = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=planypodlekurzu";

        $this ->addFilter((new FilterFormFilter('filtrprihlaseninaplanovaneskoleni'))
            ->addItem('planypodleosoby', TextManager::_('plany_podle_osoby'))
            ->addItem('planypodlekurzu', TextManager::_('plany_podle_kurzu'))
            ->setSessionName('filtrprihlaseninaplanovaneskoleni'),
            true // top - na prvnim radku
        );

        $this->addFilter(
            (new FilterFormFilterDatumOdDo($this)),
            true
        );

        $this ->addDruhSkoleniFilter();

        $this ->addFilter(new SchvalovaniOsobCisloSkoleniFilter($this));

        $this ->addFilter((new FilterFormFilter('aktualniplanykurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_aktualni_plany'))
        );

        $this ->addFilter((new FilterFormFilter('action', FilterFormFilter::TYPE_HIDDEN))
            ->setValue('prihlaseninaplanovaneskoleni')
        );

//        $this ->addFilter((new FilterFormFilter('filtrprihlaseninaplanovaneskoleni', FilterFormFilter::TYPE_HIDDEN))
//            ->setValue('planypodlekurzu')
//        );
    }
}


/**
 * Filtr pro vyber Kodu kurzu (CisloSkoleni) - musi byt az za Formularem!
 * Pred nactenim zaznamu upravi SQL dotaz podle filtru "druhskoleni"
 * (pokud je druhskoleni zadan, tak nabidne jen skoleni daneho druhu)
 */
class SchvalovaniOsobCisloSkoleniFilter extends CisloSkoleniFilter
{
    /** @var SchvalovaniOsobNaPlanuKurzuFilterForm */
    public $filterForm;

    /**
     * @param SchvalovaniOsobNaPlanuKurzuFilterForm $filterForm
     */
    public function __construct($filterForm)
    {
        parent::__construct('', TextManager::_('_vse_'), 'Stav <> 2');
        $this ->filterForm = $filterForm;
    }


    public function loadOptions(Database $database): FilterFormFilter
    {
        $params = $database->createParameters();
        $this ->setOptionsSql(
            new SqlStatement(
                'SELECT DISTINCT CisloSkoleni, Popis' .
                ' FROM PlanovaneSkoleni' .
                ' WHERE %FILTER%' .
                ($this ->filterForm ->getFilterValue('druhskoleni')
                    ? ' AND DruhSkoleni = ' . $params ->addParameter($this ->filterForm ->getFilterValue('druhskoleni'))
                    : ''
                ) .
                ' ORDER BY %ORDER%',
                $params
            ),
            'Stav <> 2',
            'Popis'
        );

        $this ->setOptionsSqlOrder2('CisloSkoleni');

        return parent::loadOptions($database);
    }

}



/**
 * Class KurzyPodrizenehoField
 */
class KurzyPodrizenehoField extends BrowserField
{
    function getLinkUrl($recordId, $rowData)
    {
        return parent::getLinkUrl($rowData['col-idpp'], $rowData);  // v URL nepouzije kod kurzu, ale id osoby
    }
}


/**
 * Class SchvalovaniPlanuKurzuBrowser
 */
class SchvalovaniPlanuKurzuBrowser extends PrihlaseniNaPlanovanaBrowser
{
    /**
     * SchvalovaniPodrizenychNaPlanuKurzuBrowser constructor.
     * @param Database $database
     * @param $filtrPrihlaseniNaPlanovaneSkoleni
     * @throws \Exception
     */
    public function __construct($database, $filtrPrihlaseniNaPlanovaneSkoleni)
    {
        parent::__construct($database,
            "skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=$filtrPrihlaseniNaPlanovaneSkoleni");

        $this->code = 'browserschvalovaniosobpodleplanukurzu';
        $this->tableId = 'schvalovaniosobpodleplanukurzu';

        Prava::nastavObdobiProPrava(time(), time());

        $this->sqlFromPart =
            'PracovniPomer' .
            ' INNER JOIN Osoba ON (Osoba.RC = PracovniPomer.RC)' .
            ' CROSS JOIN PlanovaneSkoleni' .
            ' LEFT OUTER JOIN PlanovaneSkoleniOsob ON ((PlanovaneSkoleniOsob.RC = Osoba.RC) AND (PlanovaneSkoleniOsob.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni))';

        $whereParams = $this ->database ->createParameters();
        $this->sqlWherePart = new SqlStatement(
            '(PracovniPomer.Zamestnanec = 1  OR  PracovniPomer.Zamestnanec = 2)' .
            ' AND PracovniPomer.DatumNastupu <= ' . $whereParams ->addDateParameter(time()) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $whereParams ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            ' AND PracovniPomer.ID = (SELECT Max(ID) FROM PracovniPomer PP2 WHERE PP2.RC = PracovniPomer.RC)',
            $whereParams);

        $this->defaultSort = 'col-prijmeni';

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni'))
            ->setCode('col-prijmeni')
            ->setOrdering(true)
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DruhSkoleni') ->setHeaderText('typ_skoleni') ->setOrdering(true)
            ->setCode('col-druhskoleni');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.CisloSkoleni') ->setHeaderText('cislo_skoleni') ->setOrdering(true)
            ->setCode('col-cisloskoleni');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.Popis') ->setHeaderText('Popis') ->setOrdering(true) ->setClassName('font-weight-bold')
            ->setCode('col-popis');

        //Sloupec s proklikem na Schvalovani kurzu podrizeneho
        $this->addField(new KurzyPodrizenehoField($database))
            ->setType(BrowserField::FIELD_TYPE_TEXT)
            ->setLink('?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=planypodleosoby&osoba=')
            ->setText(TextManager::_('prejit_na_kurzy_osoby'));

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumPocatkuSkoleni') ->setHeaderText('datum_pocatku_skoleni') ->setFormat('date')
            ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumpocatkuskoleni');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumUkonceniSkoleni') ->setHeaderText('datum_ukonceni_skoleni') ->setFormat('date')
            ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumukonceniskoleni');

        $this->addField(new PocetOsobNaKurzuField($database));

        $this->addField(new PotvrzeniSkoleniField($database));

        $this->addField(new SchvaleniPrihlaseniOsobyNaKurzuField($database, 1));

        // hidden fields
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno');


        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC')
            ->setCode('col-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleni.PocetOsob')
            ->setCode('col-pocetosob');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob' .
                ' WHERE PlanovaneSkoleniOsob.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni' .
                ' AND PlanovaneSkoleniOsob.Potvrzeni < 2) AS CelkemPrihlasenych')
            ->setCode('col-celkem-prihlasenych');

        // poznamky (zobrazeny na dalsich radcich)
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleniOsob.PoznamkaSchvalovani)}')
            ->setCode('col-poznamkaschvalovani')
            ->setHeader(TextManager::_('Skoleni_poznamka_schvalovani') . ': ', -1)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleni.Poznamka)}') ->setVisible(false)
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('Skoleni_poznamka') . ': ', -1)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');
    }


    function setFilterForm()
    {
        $this->filterForm = new SchvalovaniOsobNaPlanuKurzuFilterForm($this ->database);
        $this->filterForm->loadData();
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');

        //Overeni prav
        $result .=
            ($result ? ' AND ' : '') .
            '(' .
            $pravoPrihlasit->sqlFiltr('PracovniPomer',time(),time()) .
            ' OR ' .
            $pravoSchvalit->sqlFiltr('PracovniPomer',time(),time()) .
            ')';

        $result .=
            ' AND PlanovaneSkoleni.Stav < 2 ';

        if ($this ->filterForm ->getFilterValue('druhskoleni')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PlanovaneSkoleni.DruhSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('druhskoleni'));
        }

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DatumPocatkuSkoleni <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }


        if ($this ->filterForm ->getFilterValue('aktualniplanykurzu')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter(time());
        }

        if ($this ->filterForm ->getFilterValue('cisloskoleni') !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PlanovaneSkoleni.CisloSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('cisloskoleni'));
        }

        return new SqlStatement($result, $params);
    }

}
