<?php
/**
 * Lektorem spravovane kurzy
 * Roman.Szablatura RON Software 30.8.2017
 */

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Class ZaznamenatUcastSkoleniAKurzu
 */
class ZaznamenatUcastSkoleniAKurzu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // zobrazuje se jen u dlouhodobych kurzu
        if (!$rowData['col-dlouhodobykurz']) {
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class ZaznamenatHodnoceniSkoleniAKurzu
 */
class ZaznamenatHodnoceniSkoleniAKurzu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // u dlouhodobych kurzu se nezobrazuje
        if ($rowData['col-dlouhodobykurz']) {
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class SkoleniAKurzyBrowser
 * Seznam lektorem spravovanych kurzu
 */
class SkoleniAKurzyBrowser extends RonPortalBrowser
{
    /** @var  PravaNaOsoby $pravaLektora */
    private $pravaLektora;

    /**
     * VsechnaSkoleniAKurzy constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni');

        $this->pravaLektora = Prava::vratPrava('skoleni_lektor');
        Prava::nastavObdobiProPrava(time(),time());

        $this->code = 'browserskolenikurzy';
        $this->tableId = 'browserskolenikurzy';
        $this->browserUrl = "?menuid=skoleni&action=skolenikurzy";

        $this->sqlSelectPart =
            "CisloSkoleni"; //0

        $this ->primaryField = 0;

        $this->sqlFromPart = "Skoleni " .
            "LEFT JOIN S_StavSkoleni ON (Skoleni.StavSkoleni = S_StavSkoleni.Kod)";

        $this->addField()
            ->setSqlFieldName('CisloSkoleni', 0) ->setHeaderText("Skoleni_CisloSkoleni")
            ->setCode('col-cisloskoleni')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('DruhSkoleni') ->setHeaderText('typ_skoleni')
            ->setCode('col-druhskoleni') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('Skoleni.Popis') ->setHeaderText('Skoleni_Popis')
            ->setCode('col-popis')
            ->setOrdering(true) ->setClassName('popisSkoleniKurzy');

        $this->addField()
            ->setSqlFieldName('S_StavSkoleni.Popis') // puvodne AS S_SP
            ->setHeaderText('Stav_skoleni')
            ->setCode('col-stavskoleni') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('DatumPocatkuSkoleni') ->setHeaderText('Skoleni_DatumZacatku')
            ->setCode('col-datumpocatkuskoleni')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');

        $this->defaultSort = 'col-datumpocatkuskoleni';

        $this->addField()
            ->setSqlFieldName('DatumUkonceniSkoleni') ->setHeaderText('Skoleni_DatumUkonceni')
            ->setCode('col-datumukonceniskoleni')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');

        $this->addField(new ZaznamenatUcastSkoleniAKurzu())
            ->setType(BrowserField::FIELD_TYPE_TEXT)
            ->setCode('col-zaznamenat-ucast')
            ->setAlign('center') ->setHeaderText('ucast_na_skoleni')
            ->setLink('?menuid=skoleni&action=skoleniucast&cisloskoleni=')
            ->setText(TextManager::_('ucast_na_skoleni'));


        $this->addField(new ZaznamenatHodnoceniSkoleniAKurzu())
            ->setType(BrowserField::FIELD_TYPE_TEXT)
            ->setCode('col-zaznamenat-ucast')
            ->setAlign('center') ->setHeaderText('hodnoceni_skoleni')
            ->setLink('?menuid=skoleni&action=skolenihodnoceni&record=')
            ->setText(TextManager::_('hodnoceni_skoleni'));

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Skoleni.DlouhodobyKurz') ->setHeaderText('Skoleni_DlouhodobyKurz')
            ->setCode('col-dlouhodobykurz')
            ->setOrdering(true) ->setAlign('center')
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);

        $this->addField()
            ->setSqlFieldName('Kapacita') ->setHeaderText('Skoleni_Kapacita')
            ->setCode('col-kapacita')
            ->setFormat('number0') ->setOrdering(true) ->setAlign('center');

    }



    function setFilterForm()
    {
        Prava::nastavObdobiProPrava(time(),time());

        $this->filterForm = new SkoleniFilterForm($this ->database, ['menuid' => 'skoleni', 'action' => 'skolenikurzy'], true);

        //Zobrazeni FilterFormu pro nastaveni data od - do
        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setCaption(TextManager::_('skoleni_v_obdobi') . ' ' . TextManager::_('Od'))
        );

        $this ->filterForm ->addDruhSkoleniFilter();

        $this ->filterForm ->addFilter((new FilterFormFilter('stav'))
            ->setSessionName('stavSkoleniCB')
            ->setCaption(TextManager::_('Stav_skoleni'))
            ->addItem('', TextManager::_('Vse'))
            ->setOptionsSql(
                'SELECT DISTINCT Kod, Popis FROM S_StavSkoleni WHERE %FILTER% ORDER BY %ORDER%',
                'S_StavSkoleni.Kod < 2',
                'Popis')
        );

        $this ->filterForm ->addFilter((new FilterFormFilter('dlouhodobe', FilterFormFilter::TYPE_CHECKBOX))
            ->setSessionName('filtrJenDlouhodobeSkoleni')
            ->setCaption2(TextManager::_('jen_dlouhodoby'))
        );
    }


    public function enableLine($values)
    {
        if ($this->pravaLektora->prava == 3 ) {
            return true;
        }

        //Dle priznaku 3 zobrazovat kurzy
        if (trim($this->pravaLektora->priznak3) != '') {
            $povoleneTypyKurzu = explode(';', $this->pravaLektora->priznak3);
            return in_array($values[0], $povoleneTypyKurzu);    // konkretni cislo skoleni
        }

        return false;
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result =
            ($result ? ' AND ' : '') .
            'Skoleni.StavSkoleni < 2';

        //$obdobi = explode('_', $_SESSION['aktualniObdobi']);

        if ($this->filterForm->getFilterValue('druhskoleni')) {
            $result .=
                ($result ? ' AND ' : '') .
                'Skoleni.DruhSkoleni = ' . $params->addParameter($this->filterForm->getFilterValue('druhskoleni'));
        }

        if ($this ->filterForm ->getFilterValue('stav') !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                'Skoleni.StavSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('stav'));
        }


        if ($this ->filterForm ->getFilterValue('dlouhodobe')) {
            $result .=
                ($result ? ' AND ' : '') .
                'Skoleni.DlouhodobyKurz = 1';
        }

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                '(Skoleni.DatumUkonceniSkoleni IS NULL OR Skoleni.DatumUkonceniSkoleni >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod')) . ')';
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' Skoleni.DatumPocatkuSkoleni <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        return new SqlStatement($result, $params);
    }
}
