<?php
/**
 * stanislav.elbl, 06.07.2022 9:47
 */

namespace RonPortal\Skoleni;

use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalFilterForm;


class CisloSkoleniFilter extends FilterFormFilter
{
    /** @var FilterFormFilter */
    public $typSkoleniFilter;

    public function __construct($sql, $emptyRowStr, $optionsFilter)
    {
        parent::__construct('cisloskoleni', FilterFormFilter::TYPE_COMBOBOXES);

        $this
            ->setCaption(TextManager::_('pouze_kod_kurzu'))
            ->setRowId('filterrow_cislo_skoleni')
            ->setAddClass('ronselect')
            ->setHtmlAttributes(
                [
                    'id' => 'cisloSkoleni',
                    'data-ron-select2' => '#cisloSkoleni_kod',
                    'data-ron-autosubmit' => '1'
                ])
            ->setHtmlAttributes2(
                [
                    'id' => 'cisloSkoleni_kod',
                    'data-ron-select2' => '#cisloSkoleni',
                    'data-ron-autosubmit' => '1'
                ]
            )
            ->setOptionsSql($sql, $optionsFilter, 'Popis')
            ->setOptionsSqlOrder2('CisloSkoleni')
        ;

        if ($emptyRowStr) {
            $this
                ->addItem('', $emptyRowStr)
                ->addItem2('', $emptyRowStr);
        }
    }
}


class SkoleniFilterForm extends RonPortalFilterForm
{
    /** @var string */
    public $cisloSkoleni;

    /** @var FilterFormFilter */
    private $filtrDruhSkoleni;


    function addSpecifikaceFilter()
    {
        $filtrSpecifikace = new FilterFormFilter('specifikace', FilterFormFilter::TYPE_OPTIONS);
        $filtrSpecifikace
            ->setSessionName('filtrSpecifikaceKurzu')
            ->setRowId('filterrow_specifikace_kurzu')
            ->setCaption(TextManager::_('Skoleni_specifikace_filtr'))
            ->setAddClass('ronselect')
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql(
                'SELECT Kod, Popis FROM SpecifikaceKurzu WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Popis');

        $this->addFilter($filtrSpecifikace);
    }


    function addKategorieFilter()
    {
        $filtrKategorie = new FilterFormFilter('kategorie', FilterFormFilter::TYPE_OPTIONS);
        $filtrKategorie
            ->setSessionName('filtrKategorieSkoleni')
            ->setRowId('filterrow_kategorie_skoleni')
            ->setCaption(TextManager::_('Skoleni_kategorie_filtr'))
            ->setAddClass('ronselect')
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql(
                'SELECT Kod, Popis FROM KategorieSkoleni WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Popis');

        $this->addFilter($filtrKategorie);
    }

    function addDruhSkoleniFilter()
    {
        $filtrDruh = new FilterFormFilter('druhskoleni', FilterFormFilter::TYPE_COMBOBOXES);
        $filtrDruh
            ->setSessionName('filtrDruhSkoleniAKurzu')
            ->setRowId('filterrow_druh_skoleni')
            ->setCaption(TextManager::_('typ_skoleni'))
            ->setAddClass('ronselect')
            ->addItem('', TextManager::_('_vse_'))
            ->addItem2('', TextManager::_('_vse_'))
            ->setOptionsSql(
                'SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 AND zobrazitNaIntrawebu = 1 AND (%FILTER%) ORDER BY %ORDER%',
                '1 = 1',
                'Popis')
            ->setOptionsSqlOrder2('Kod')
            ->setHtmlAttributes(
                [
                    'id' => 'druhSkoleni',
                    'data-ron-select2' => '#druhSkoleni_kod',
                    'data-ronselect-class' => 'ronselect adsfilterforminput_popis',
                    'data-columns' => 'text;value',
                    'data-ron-autosubmit' => '1'
                ])
            ->setAttrIndexes2(['data-popis' => 1])
            ->setHtmlAttributes2(
                [
                    'id' => 'druhSkoleni_kod',
                    'data-ron-select2' => '#druhSkoleni',
                    'data-ronselect-class' => 'ronselect adsfilterforminput_kod',
                    'data-columns' => 'value;popis',
                    'data-ron-autosubmit' => '1',
                    'data-drop-down-align' => 'right'
                ])
            ;

        $this->filtrDruhSkoleni = $filtrDruh;

        $this ->addFilter($filtrDruh);
    }


    public function addCisloSkoleniFilter($sql, $emptyRowStr, $optionsFilter = '1 = 1')
    {
        $filtrCisloSkoleni = new CisloSkoleniFilter($sql, $emptyRowStr, $optionsFilter);
        $this ->addFilter($filtrCisloSkoleni);

        return $filtrCisloSkoleni;
    }

    protected function afterLoadFilter($filter)
    {
        parent::afterLoadFilter($filter);

        if ($filter ->name === 'kategorie'  &&  $filter ->getValue() !== null) {
            $params = $this->database->createParameters();

            $this->filtrDruhSkoleni->setOptionsSql(
                new SqlStatement('SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 AND zobrazitNaIntrawebu = 1 ' .
                ' AND Kategorie = ' . $params->addParameter($filter->getValue()) .
                ' AND (%FILTER%) ORDER BY %ORDER%', $params),
                '1 = 1',
                'Popis');
        }
    }

}
