<?php
/**
 * Seznam radku sluzebni cesty
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;


class EditaceRSCField extends BrowserField
{
    protected $lzeEditovat;

    public function __construct($lzeEditovat)
    {
        parent::__construct();
        $this ->lzeEditovat = $lzeEditovat;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="#" title="' . TextManager::_($this ->lzeEditovat ? 'Editovat_zaznam' : 'Detail') . '">' .
            '<i class="inline-glyph glyph-button fa ' . ($this ->lzeEditovat ? 'glyph-edit' : 'glyph-detail') . //' open-dialog"' .
            //' data-dialog="#radek_sc_dialog" data-ronform-url="?menuid=sluzebnicestyjs&action=radeksc&idsc=' . $rowData[0] . '&record="' . // id SC
            //' data-ronform-record="' . $rowData[1] . // id radku
            '">' .
            '</a>';
    }
}


class KopieRSCField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="#" title="' . TextManager::_('Kopirovat_zaznam') . '">' .
            '<i class="inline-glyph glyph-button fa glyph-copy gfiles-open-dialog"' .
            //' data-dialog="#radek_sc_dialog" data-ronform-url="?menuid=sluzebnicestyjs&action=radeksc&idsc=' . $rowData[0] . '&copyrecord=' . $rowData[1] . '">' .
            ' data-dialog-append-url="&copyrecord=' . $rowData[1] . '">' .
            '</a>';
    }
}


class ZruseniRSCField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection HtmlUnknownTarget */
        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
            TextManager::_('SC_Opravdu_zrusit_radek'),
            '?menuid=sluzebnicesty&action=zrusitradeksluzebky&record=' . $rowData[0] . '&radek='. $rowData[1] . Csrf::csrfUrlParameter("deleteradeksc-{$rowData[0]}-{$rowData[1]}"),
            'data-ron-send-position="1"'
        );
    }
}



class RadekSluzebniCestyBrowser extends RonPortalBrowser
{
    /** @var SluzebniCesta */
    var $sluzebniCesta;
    var $lzeEditovat;

    var $zobrazitMisto; // zobrazuje se jen kdyz nejake existuje

    private $posledniDatum = null;


    /**
     * @param Database $database
     * @param bool $lzeEditovat
     * @param SluzebniCesta $sluzebniCesta
     */
    function __construct($database, $lzeEditovat, $sluzebniCesta)
    {
        parent::__construct($database, null);

        $this->sluzebniCesta = $sluzebniCesta;
        $this->code = 'browserradeksc';

        $this->browserUrl = '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $sluzebniCesta ->id . '&';

        $this->title = TextManager::_('Seznam_radku_sc');
        $this->tableId = 'tblRadekSluzebniCesty';
        $this ->sqlSelectPart =
            'RadekSluzebniCesty.SluzebniCesta_ID, RadekSluzebniCesty.CisloRadku, RadekSluzebniCesty.DatumCasOd, RadekSluzebniCesty.Poznamka'; // 0..3

        $this->sqlFromPart =
            '( RadekSluzebniCesty LEFT OUTER JOIN OperaceDochazky ' .
            '  ON RadekSluzebniCesty.KodOperace = OperaceDochazky.Kod ' .
            ') ' .
            ' INNER JOIN Stat ON RadekSluzebniCesty.Stat = Stat.Kod' .
            ' LEFT OUTER JOIN MistoSluzebniCesty ON MistoSluzebniCesty.Kod = RadekSluzebniCesty.Misto' .
            ' LEFT OUTER JOIN DopravniProstredek ON RadekSluzebniCesty.DopravniProstredek = DopravniProstredek.Kod';

        $this->sqlWherePart =
            'RadekSluzebniCesty.SluzebniCesta_ID = ' . (int)$sluzebniCesta ->id;

        $this->defaultSort = 'col-datum';
        $this->primaryField = 1;

        $this->cssClass = 'browser minwidth rowOver gfiles-dialog-container';
        $this ->rowCssClass = 'gfiles-open-dialog';
        $this ->tableHtmlParams = [
            'data-dialog-content-url' => '?menuid=sluzebnicestyjs&action=radekscrecord&idsc=' . $sluzebniCesta ->id, // http://localhost/ronportal/profiler.php?menuid=sluzebnicestyjs&action=radeksc&idsc=6718&record=1
            //'data-form-url' => '?menuid=poznamkykedni&formaction=edit'
        ];


        $this->lzeEditovat = $lzeEditovat;

        $this->showMessages = $this ->code; // jen zpravy prirazene browseru

        $this ->zobrazitMisto = false;

        $cursor = $database ->getSqlResult('SELECT COUNT(*) FROM MistoSluzebniCesty WHERE Aktivni = 1');
        if ($row = $cursor ->fetch()) {
            if ($row[0]) {
                $this ->zobrazitMisto = true;
            }
        }

        $this->addField()
            ->setSqlFieldName('RadekSluzebniCesty.DatumCasOd', 2)
            ->setCode('col-datum') ->setHeaderText('Date') ->setFormat('date');

        $this->addField()
            ->setSqlFieldName('RadekSluzebniCesty.DatumCasOd', 2)
            ->setCode('col-cas') ->setHeaderText('Cas') ->setFormat('time');

        // kod operace pro pripadny preklad v nasledujicim poli
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekSluzebniCesty.KodOperace')
            ->setCode('col-kodoperace');

        $this->addField()
            ->setSqlFieldName('OperaceDochazky.Popis') ->setClassName('radekSC_operace')
            ->setCode('col-operace') ->setHeaderText('Operace_sc') ->setVisible(SluzebniCestyController::getZobrazitOperaceSC())
            ->setTranslationPrefix('db_operacedochazky_popis_', 'col-kodoperace');

        // kod mista SC pro pripadny preklad v nasledujicim poli
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekSluzebniCesty.Misto')
            ->setCode('col-kodmista');

        $this->addField()
            ->setSqlFieldName('MistoSluzebniCesty.Popis') ->setClassName('radekSC_misto')
            ->setCode('col-misto') ->setHeaderText('Misto_sc') ->setVisible($this ->zobrazitMisto) // zobrazuje se jen kdyz jsou nejaka mista v databazi
            ->setTranslationPrefix('db_mistosluzebnicesty_popis_', 'col-kodmista');

        $this->addField()
            ->setSqlFieldName('RadekSluzebniCesty.Popis')
            ->setCode('col-popis') ->setHeaderText('Popis_radku_sc') ->setClassName('radekSC_popis');

        // kod statu - pro preklad v poli stat
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekSluzebniCesty.Stat')
            ->setCode('col-kodstatu');

        $this->addField()
            ->setSqlFieldName('Stat.Popis')
            ->setCode('col-stat') ->setHeaderText('Stat') ->setClassName('radekSC_stat')
            ->setTranslationPrefix('db_stat_popis_', 'col-kodstatu');


        // kod dopravniho prostredku pro pripadny preklad v nasledujicim poli
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekSluzebniCesty.DopravniProstredek')
            ->setCode('col-koddopravnihoprostredku');

        $this->addField()
            ->setSqlFieldName('DopravniProstredek.Popis')
            ->setCode('col-dopravniprostredek') ->setHeaderText('Dopravni_prostredek')
            ->setVisible(false)
            ->setTranslationPrefix('db_dopravniprostredek_popis_', 'col-koddopravnihoprostredku');

        $this->addField()
            ->setSqlFieldName('PocetPJidel_S')
            ->setCode('col-snidane') ->setHeaderText('Pocet_snidani') ->setAlign('right') ->setClassName('radekSC_snidane')
        ;
        $this->addField()
            ->setSqlFieldName('PocetPJidel_O')
            ->setCode('col-obedy') ->setHeaderText('Pocet_obedu') ->setAlign('right') ->setClassName('radekSC_obed');

        $this->addField()
            ->setSqlFieldName('PocetPJidel_V')
            ->setCode('col-vecere') ->setHeaderText('Pocet_veceri') ->setAlign('right') ->setClassName('radekSC_vecere');

        if ($lzeEditovat) {
            // TODO: aby slo zobrazit i detail pro needitovatelne radky, je potreba vylepsit podporu readonly zobrazeni v dialogu!
            $this->addField(new EditaceRSCField($lzeEditovat))
                ->setAlign('left')
                ->setCode('col-edit');
        }

        if ($lzeEditovat) {
            $this->addField(new KopieRSCField())
                ->setAlign('left')
                ->setCode('col-copy');

            $this->addField(new ZruseniRSCField())
                ->setAlign('left')
                ->setCode('col-delete');
        }

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('RadekSluzebniCesty.Poznamka')
            ->setCode('col-poznamka')
            ->setHeaderColSpan(-1)
            ->setClassName('poznamka');

       // $this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }

    function setFilterForm()
    {
        // nedela nic - filter form se nepouziva
    }


    public function getRowAttributes($values, $className, $rowNumber)
    {
        $result = parent::getRowAttributes($values, $className, $rowNumber) ?? [];
        $result['data-dialog-append-url'] = '&record=' . $values[1]; // cisloRadkuSc
//        $result['role'] = 'link';
//        $result['tabindex'] = '0';
        return $result;
    }


    // Zobrazeni jednoho radku
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $date = DateUtils::startOfTheDay($this ->database ->stringToTimestamp($values['col-datum']));
        $newDate = $date  &&  $this ->posledniDatum  &&  $this ->posledniDatum !== $date;
        $this ->posledniDatum = $date;

        parent::renderLine($values, $className . ($newDate ? ' border-top border-dark' : ''), $rowNumber, $edit);
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @param bool $lzeEditovat
     * @throws \Exception
     */
    public static function zobrazRadkySluzebniCesty($database, $sluzebniCesta, $lzeEditovat)
    {
        $b = new static($database, $lzeEditovat, $sluzebniCesta);
        $b ->load();
        $b->render();

        if ($lzeEditovat) {
            echo
                "<div class='text_link' id='radekSCBrowserLinks'><ul class='linkslist'>\n" .
                "<li><a href='#' class='gfiles-open-dialog clickLink' data-dialog-content-url='?menuid=sluzebnicestyjs&action=radekscrecord&idsc={$sluzebniCesta ->id}'  data-X-dialog='#radek_sc_dialog'>" . TextManager::_('Novy_radek') . "</a></li>\n" .
                "<li><a href='?menuid=sluzebnicesty&action=editovatradkysluzebky&record={$sluzebniCesta ->id}'>" . TextManager::_('Editovat_radky_sc') . "</a></li>\n" .
                "</ul></div>\n";
        }
    }

}
