<?php
/**
 * Seznam radku uctovani sluzebni cesty
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class EditaceUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;
    public $action;
    public $title;

    public $lzeEditovat;


    /**
     * @param SluzebniCesta $sc
     * @param string $action
     */
    function __construct($sc, $action, $code, $lzeEditovat)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->action = $action;
        $this ->code = $code;
        $this ->align = 'left';

        $this ->lzeEditovat = $lzeEditovat;
    }


    public static function lzeEditovatRadek($rowData, $idPP, $datumScOd, $datumScDo)
    {
        // nektere radky uctovani nejdou editovat (vyuctovani atd.)
        if ($rowData['col-typ'] >= 1 && $rowData['col-typ'] <= 4) {
            return false;
        }

        // kontrola, jestli ma pravo editovat tento radek
        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData['col-typ']);
        return $pravo ->lzeZobrazitPP($idPP, $datumScOd, $datumScDo);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!self::lzeEditovatRadek($rowData, $this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

//        // nektere radky uctovani nejdou editovat (vyuctovani atd.)
//        if ($rowData['col-typ'] >= 1 && $rowData['col-typ'] <= 4) {
//            return '';
//        }
//
//        // kontrola, jestli ma pravo editovat tento radek
//        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData['col-typ']);
//        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
//            return '';
//        }

        return
            ($this ->action == 'editovatuctovanisluzebky'
                ?
                    '<a href="#" title="' . \Ron\ronText($this ->lzeEditovat ? 'Editovat_zaznam' : 'Detail') . '">' .
                    '<i class="inline-glyph glyph-button fa ' . ($this ->lzeEditovat ? 'glyph-edit' : 'glyph-detail') . ' open-dialog"' .
                    ' data-dialog="#radek_uctovani_sc_dialog"' .
                    ' data-ronform-url="?menuid=sluzebnicestyjs&action=radekuctovanisc&idsc=' . $rowData[0] . '&record="' . // id SC
                    ' data-ronform-record="' . $rowData[1] . '">' . // id radku
                    '</a>'
                : '') .

            ($this ->action == 'kopirovatuctovanisluzebky'
                ?
                    '<a href="#" title="' . \Ron\ronText('Kopirovat_zaznam') . '">' .
                    '<i class="inline-glyph glyph-button fa glyph-copy gfiles-open-dialog"' .
                    //' data-dialog="#radek_sc_dialog" data-ronform-url="?menuid=sluzebnicestyjs&action=radeksc&idsc=' . $rowData[0] . '&copyrecord=' . $rowData[1] . '">' .
                    ' data-dialog-append-url="&copyrecord=' . $rowData[1] . '">' .
//                    '<i class="inline-glyph glyph-button fa glyph-copy open-dialog"' .
//                    ' data-dialog="#radek_uctovani_sc_dialog"' .
//                    ' data-ronform-url="?menuid=sluzebnicestyjs&action=radekuctovanisc&idsc=' . $rowData[0] . '&record=' . $rowData[1] . '">' .
                    '</a>'
                : '');
    }
}


class ZruseniUSCField extends BrowserField
{
    /** @var SluzebniCesta */
    public $sc;

    /**
     * @param SluzebniCesta $sc
     */
    function __construct($sc)
    {
        parent::__construct();
        $this ->sc = $sc;
        $this ->code = 'col-delete';
        $this ->align = 'left';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!EditaceUSCField::lzeEditovatRadek($rowData, $this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
            return '';
        }

//        if ($rowData['col-typ'] >= 1 && $rowData['col-typ'] <= 4) {
//            return '';
//        }
//
//        $pravo = Prava::vratPrava('slcesty_ucet', 'slcesty_ucet_' . $rowData['col-typ']);
//        if (!$pravo ->lzeZobrazitPP($this ->sc ->f_PP ->idPP, $this ->sc ->datumCasOd, $this ->sc ->datumCasDo)) {
//            return '';
//        }

        return GuiUtils::changeLocationQuest(
            '<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
            TextManager::_('SC_Opravdu_zrusit_radek'),
            "?menuid=sluzebnicesty&action=zrusituctovanisluzebky&record=" . $rowData[0] . "&radek=" . $rowData[1] . Csrf::csrfUrlParameter("deleteuctovanisc-{$this ->sc ->id}-{$rowData[1]}"),
            'data-ron-send-position="1"');
    }
}


class CastkaUSCField extends BrowserField
{
    function getDataCellClass($value, $rowNumber, $recordId, $rowData)
    {
        switch ($rowData['col-typ-druhpohybu']) {
            // druh pohybu minus nebo plus
            case 1 :
            case 2 :
                // pokud se ma znamenko zobrazovat kladne (nastaveni 0) prevede na ABS
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
                    $value = abs($value);
                }
                break;
        }

        return parent::getDataCellClass($value, $rowNumber, $recordId, $rowData);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ( !isset($value) ) {
            return '';
        }

        switch ($rowData['col-typ-druhpohybu']) {
            // druh pohybu minus nebo plus
            case 1 :
            case 2 :
                // pokud se ma znamenko zobrazovat kladne (nastaveni 0) prevede na ABS
                if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZNAMENKO_UCTOVANI)) {
                    $value = abs($value);
                }
                break;
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


class DokumentyUSCField extends BrowserField
{
    var $lzeEditovat;

    /**
     * @param Database $database
     * @param bool $lzeEditovat
     */
    function __construct($database, $lzeEditovat)
    {
        parent::__construct($database);
        $this->code = 'col-dokumenty';
        $this->align = 'center';

        $this ->lzeEditovat = $lzeEditovat;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $sql =
            'SELECT DokumentSluzebniCesty.Id, Dokument.Popis' .
            '  FROM DokumentSluzebniCesty, Dokument ' .
            ' WHERE SluzebniCesta_ID = ' . (int)$rowData[0] .
            '   AND Dokument.Id = DokumentSluzebniCesty.Id ' .
            '   AND CisloRadku = ' . (int)$rowData[1];

        $result = '';

        $cursor = $this->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $result .=
                '<a target="_blank" href="?menuid=sluzebnicesty&action=download&idslcesty=' . (int)$rowData[0] . '&record=' . $row[0] . '">' .
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . StringUtils::htmlTextConvert($row[1]) . '"></i>' .
                '</a>';

        }
        $cursor ->close();

        if ($this ->lzeEditovat  &&  ($rowData['col-typ'] > 4  ||  $rowData['col-typ'] < 1)) {
            $result .=
                '<a href="#" title="' . TextManager::_('Vlozit_novy_dokument') . '">' .
                '<i class="inline-glyph fa glyph-upload show-dokument-dialog" data-fields="CisloRadku" data-field-cisloradku="' . $rowData[1] . '"></i>' .
                '</a>';
        }

        return $result;
    }
}


class CastkaDMField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return $rowData['col-dm_kodmeny']
            ? parent::getViewText($value, $rowNumber, $recordId, $rowData)
            : '';
    }
}


class KurzDMField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData['col-dm_vlastnikurz']) {
            return '';
        }

        // je zaskrtnuto "vlastni kurz"
        return
            $rowData['@col-dm_kurz_mnozstvi'] . ' ' .
            $rowData['col-mena'] . ' = ' .
            $rowData['@col-dm_kurz'] . ' ' .
            $rowData['col-dm_kodmeny'];
    }
}


class RozdeleniRadkuUctovaniSCField extends BrowserField
{
    private $rozdeleni2;


    public function __construct($rozdeleni2)
    {
        parent::__construct();
        $this ->rozdeleni2 = $rozdeleni2;

        $this ->setType(BrowserField::FIELD_TYPE_COMMENT);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$this ->rozdeleni2  &&  $rowData['col-rozdeleni']) {
            return "{$rowData['col-typrozdeleni-popis']}: {$rowData['col-rozdeleni-popis']} ({$rowData['col-rozdeleni']})";
        }

        if ($this ->rozdeleni2  &&  $rowData['col-rozdeleni2']) {
            return "{$rowData['col-typrozdeleni2-popis']}: {$rowData['col-rozdeleni2-popis']} ({$rowData['col-rozdeleni2']})";
        }

        return '';
    }
}


class NahradaBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData['col-typ-nahrada']) {
            return '';
        }

        return
            // spotreba
            TextManager::_('Spotreba_TP') . ': ' . $rowData['@col-nvv_spotrebavozidla'] . ' ' . TextManager::_('Litru_100') .
            ($rowData['col-nvv_druhphm'] ? ' (' . StringUtils::htmlTextConvert($rowData['col-nvv_druhphm']) . ')' : '') .
            ', ' .
            // pocet km
            "<strong>{$rowData['@col-nvv_pocetujetychkm']} " . TextManager::_('km') . '</strong>' .
            // cena PHM
            (!$rowData['col-nvv_neprokazanacenaphm']
                ?
                ", " . StringUtils::format('number_2', (float)$rowData['col-nvv_cenaphm']) .
                ($rowData['col-mena'] ? ' ' . $rowData['col-mena'] : '') .
                ' ' . TextManager::_('Kc_litr') // bez (float) to zobrazuje napr. 38.20000003
                : '') .
            // zakladani sazba
            ', ' . TextManager::_('Zakladni_sazba') . ': ' .
            TextManager::_($rowData['col-nvv_nepocitatzaklsazbu'] ? 'Str_No' : 'Str_Yes');
    }
}



class RadekUctovaniSluzebniCestyBrowser extends RonPortalBrowser
{
    /** @var SluzebniCesta */
    private $sluzebniCesta;

    var $lzeEditovat;

    /** @var int    Pomocna promenna pro renderovani radku */
    private $posledniDatum = null;

    // datum urcuje hodnotu pro datumu pro novy zaznam - mel by to byt zacatek SC
    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     */
    function __construct($database, $sluzebniCesta)
    {
        parent::__construct($database, null);

        $this ->sluzebniCesta = $sluzebniCesta;

        $this ->code = 'browseructovanisc';
        $this ->title = TextManager::_('Uctovani');
        $this ->tableId = 'tblRadekUctovaniSlCesty';

        $this->browserUrl = '?menuid=sluzebnicesty&action=zobrazslcestu&record=' . $sluzebniCesta ->id . '&';

        $this -> sqlSelectPart =
            'RadekUctovaniSCesty.SluzebniCesta_ID, RadekUctovaniSCesty.CisloRadku'; // 0..1

        $this ->sqlFromPart =
            '(RadekUctovaniSCesty INNER JOIN TypRUctovaniSlCesty ' .
            '  ON RadekUctovaniSCesty.Typ = TypRUctovaniSlCesty.Kod ' .
            ' ) ' .
            ' LEFT OUTER JOIN ZpusobPlatbySC ' .
            ' ON ZpusobPlatbySC.Kod = RadekUctovaniSCesty.ZpusobPlatby ' .
            ' LEFT OUTER JOIN Rozdeleni R1' .
            ' ON R1.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni' .
            ' AND R1.Kod = RadekUctovaniSCesty.Rozdeleni' .
            ' LEFT OUTER JOIN TypRozdeleni TR1' .
            ' ON TR1.Kod = R1.KodTypuRozdeleni' .
            ' LEFT OUTER JOIN Rozdeleni R2' .
            ' ON R2.KodTypuRozdeleni = RadekUctovaniSCesty.KodTypuRozdeleni2' .
            ' AND R2.Kod = RadekUctovaniSCesty.Rozdeleni2' .
            ' LEFT OUTER JOIN TypRozdeleni TR2' .
            ' ON TR2.Kod = R2.KodTypuRozdeleni';

        $this ->sqlWherePart = '';

        $this ->primaryField = 1;

        if ($sluzebniCesta ->lzeEditovat) {
            $prava = Prava::vratPrava('slcesty_ucet');
            $this ->lzeEditovat = $prava ->lzeZobrazitPP($sluzebniCesta ->f_PP ->idPP, $sluzebniCesta ->datumCasOd, $sluzebniCesta ->datumCasDo);
        } else {
            $this->lzeEditovat = false;
        }

//        $lzeSchvalit = Prava::lzeZobrazitPP('slcesty_k_ucet', $this ->sluzebniCesta ->f_PP ->idPP);

        $this->showMessages = 'browseructovanisc';

        $this ->cssClass = 'browser minwidth rowOver gfiles-dialog-container';
        // $this ->rowCssClass = 'gfiles-open-dialog'; - ne, protoze nektere radky nelze editovat
        $this ->tableHtmlParams = [
            'data-dialog-content-url' => '?menuid=sluzebnicestyjs&action=uctovaniscrecord&idsc=' . $sluzebniCesta ->id, // http://localhost/ronportal/profiler.php?menuid=sluzebnicestyjs&action=radeksc&idsc=6718&record=1
        ];

        $this ->defaultSort = 'col-datum';

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Den') ->setHeaderText('Datum')
            ->setCode('col-datum') ->setFormat('date');

        $this ->addField()
            ->setSqlFieldName( 'TypRUctovaniSlCesty.Popis') ->setHeaderText('Typ')
            ->setCode('col-typ-popis')
            ->setTranslationPrefix('db_typructovanislcesty_popis_', 'col-typ');

        $this ->addField( new CastkaUSCField() )
            ->setSqlFieldName('RadekUctovaniSCesty.Castka') -> setHeaderText('Castka')
            ->setCode('col-castka')
            ->setAlign('right') ->setFormat('number2');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.KodMeny') ->setHeaderText('Mena')
            ->setCode('col-mena');

        if (RonPortalSettings::getIntValue('vlastnikurzradkusc')) {
            $this->addField(new KurzDMField())
                ->setFormat('number_3') ->setHeaderText('Kurz')
                ->setCode('col-kurz');

            // vlastni kurz meny
            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('RadekUctovaniSCesty.DM_VlastniKurz')
                ->setCode('col-dm_vlastnikurz');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('RadekUctovaniSCesty.DM_Kurz_Mnozstvi')
                ->setCode('col-dm_kurz_mnozstvi')
                ->setFormat('number_3');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('RadekUctovaniSCesty.DM_Kurz')
                ->setCode('col-dm_kurz')
                ->setFormat('number_3');
        }

        if ( RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH) ) {
            $this ->addField()
                ->setSqlFieldName('RadekUctovaniSCesty.CastkaDPH') ->setHeaderText('SC_DPH')
                ->setCode('col-dph')
                ->setFormat('number2') ->setAlign('right');
        }

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.Typ')
            ->setCode('col-typ');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.ZpusobPlatby')
            ->setCode('col-zpusobplatby');

        $this ->addField()
            ->setSqlFieldName('ZpusobPlatbySC.Popis') ->setHeaderText('SC_Zpusob_platby')
            ->setCode('col-zpusobplatby-popis')
            ->setTranslationPrefix('db_zpusobplatbysc_popis_', 'col-zpusobplatby');

        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Popis') ->setCode('col-popis') ->setHeaderText('Popis_radku_uctovani_sc');

        $this->addField(new CastkaDMField($database))
            ->setSqlFieldName('RadekUctovaniSCesty.DM_Castka') ->setHeaderText('Castka_DM')
            ->setCode('col-castkadm')
            ->setFormat('number2') ->setAlign('right');


        $this ->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.DM_KodMeny') ->setHeaderText('Mena_DM')
            ->setCode('col-dm_kodmeny');

        if ($this ->lzeEditovat) {
            // TODO: aby slo zobrazit i detail pro needitovatelne radky, je potreba vylepsit podporu readonly zobrazeni v dialogu!
            $this->addField(new EditaceUSCField($this->sluzebniCesta, 'editovatuctovanisluzebky', 'col-edit', $this->lzeEditovat));
        }

        if ($this ->lzeEditovat) {
            $this ->addField(new EditaceUSCField($this ->sluzebniCesta, 'kopirovatuctovanisluzebky', 'col-copy', $this ->lzeEditovat));
            $this ->addField(new ZruseniUSCField($this ->sluzebniCesta));
        }

        $this ->addField(new DokumentyUSCField($database, $this ->lzeEditovat));

        // schvaleni
        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText('<i class="inline-glyph fa glyph-check" title="' . TextManager::_('Uctovani_SC_zkontrolovano') . '">')
            ->setCode('col-zkontrolovano')
            ->setFilterValue(true)
            ->setSqlFieldName('Zkontrolovano');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypRUctovaniSlCesty.DruhPohybu')
            ->setCode('col-typ-druhpohybu');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypRUctovaniSlCesty.NahradaZaVlastniVozidlo')
            ->setCode('col-typ-nahrada');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_DruhPHM')
            ->setCode('col-nvv_druhphm');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_NeprokazanaCenaPHM')
            ->setCode('col-nvv_neprokazanacenaphm');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_CenaPHM')
            ->setCode('col-nvv_cenaphm');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_PocetUjetychKM')
            ->setCode('col-nvv_pocetujetychkm')
            ->setFormat('int');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_SpotrebaVozidla')
            ->setCode('col-nvv_spotrebavozidla')
            ->setFormat('number_2');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.NVV_NepocitatZaklSazbu')
            ->setCode('col-nvv_nepocitatzaklsazbu');

        // rozdeleni 1
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.Rozdeleni')
            ->setCode('col-rozdeleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('R1.Popis')
            ->setCode('col-rozdeleni-popis');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TR1.Popis')
            ->setCode('col-typrozdeleni-popis');

        // rozdeleni 2
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('RadekUctovaniSCesty.Rozdeleni2')
            ->setCode('col-rozdeleni2');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('R2.Popis')
            ->setCode('col-rozdeleni2-popis');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TR2.Popis') // typ rozdeleni 2
            ->setCode('col-typrozdeleni2-popis');

        $this ->addField(new RozdeleniRadkuUctovaniSCField(false))
            ->setCode('col-poznamka-rozdeleni')
            ->setHeaderColSpan(-1) ->setClassName('poznamka');

        $this ->addField(new RozdeleniRadkuUctovaniSCField(true))
            ->setCode('col-poznamka-rozdeleni2')
            ->setHeaderColSpan(-1) ->setClassName('poznamka');

        $this ->addField((new NahradaBrowserField())
            ->setType(BrowserField::FIELD_TYPE_COMMENT)
            ->setCode('col-nahrada')
            ->setHeaderColSpan(-1) ->setClassName('poznamka')
        );

        //$this ->loadConfigurationXml(RonPortalBrowser::loadBrowserConfiguration($this ->database, $this ->code));
    }

    function setFilterForm()
    {
        // nedela nic - filtr se nezobrazuje
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        $lzeEditovat = EditaceUSCField::lzeEditovatRadek($values,
            $this ->sluzebniCesta ->f_PP ->idPP,
            $this ->sluzebniCesta ->datumCasOd, $this ->sluzebniCesta ->datumCasDo
        );

        $className .=
            ($lzeEditovat ? ' gfiles-open-dialog' : '') .
            ($values ? (' typradkuuctovani_' . $values['col-typ']) : '');
        $result = parent::getRowAttributes($values, $className, $rowNumber);

        if ($lzeEditovat) {
            $result['data-dialog-append-url'] = '&record=' . $values[1]; // CisloRadku
        }

        return $result;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $date = DateUtils::startOfTheDay($this ->database ->stringToTimestamp($values['col-datum']));
        $newDate = $date  &&  $this ->posledniDatum  &&  $this ->posledniDatum !== $date;
        $this ->posledniDatum = $date;

        parent::renderLine($values, $className . ($newDate ? ' border-top border-dark' : ''), $rowNumber, $edit);
    }


    /**
     * @param Database $database
     * @param SluzebniCesta $sc
     * @param string $akce
     */
    public static function zobrazUctovani($database, SluzebniCesta $sc)
    {
        $b = new static($database, $sc);
        $b ->load("RadekUctovaniSCesty.SluzebniCesta_ID = {$sc ->id}");
        $b ->render();

        if ($b ->lzeEditovat) {
            /** @noinspection HtmlUnknownAnchorTarget */
            echo
                "<ul class='linkslist'>" .
                //"<li><a class='sendPosition' href='?menuid=sluzebnicesty&action=pridatuctovanisluzebky&record={$sc ->id}'>" . TextManager::_('Novy_radek') . '</a></li>' .
                "<li><a href='#' class='gfiles-open-dialog clickLink' data-dialog-content-url='?menuid=sluzebnicestyjs&action=uctovaniscrecord&idsc={$sc ->id}'>" . \Ron\ronText('Novy_radek') . "</a></li>" .
                "</ul>\n";

            if (RonPortalSettings::getSpousteniADS() ) {
                echo
                    "<div id='link_vyuctovani_sc'>" .
                    "<a class='sendPosition' href='?menuid=sluzebnicesty&action=vyuctovatsluzebky&record={$sc ->id}'>" . TextManager::_('Vyuctovani_sc') . '</a>' .
                    "</div>\n";
            }
        }
    }

}
