<?php
/**
 * stanislav.elbl, 29.07.2015
 * Seznam sluzebnich cest
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\ProgressDialog;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FilterFormFilterObdobi;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Controls\RozdeleniBrowserField;
use RonPortal\Controls\UkonceniPracPomeruInfoField;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;

class SluzebniCestyFilterForm extends RonPortalFilterForm
{
    /** @var bool */
    private $schvalovani;
    
//    /** @var array */
//    private $filtrySchvaleniSC;

//    /** @var int    Aktualni hodnota filtru pro urovne schvalovani */
//    public $urovenSchvaleniSC = 0;

    /** @var FilterFormFilter */
    public $filtrUrovenSchvaleni;

    /**
     * @param Database $database
     * @param bool $schvalovani
     */
    function __construct($database, $schvalovani)
    {
        parent::__construct($database, 'sluzebnicesty', true);

        $this ->schvalovani = $schvalovani;

        $this ->addFilter(
            new FilterFormFilterObdobi($this), true
        );

//        $this->nastavObdobi(true, 'ADS', false, false); // !self::getUrovenSchvaleni()
//        $this->nastavDatumOdDo(true, false, TextManager::_('Datum_od'));

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setPovolitPrazdneDatumDo(true)
                ->addLink([
                    'datumOd' => time(),
                    'datumDo' => null,
                    'text' => TextManager::_('SC_aktualni_a_budouci')
                ]),
            true
        );


//        $this ->datumOdDoLinks = [
//            [
//                'datumOd' => time(),
//                'datumDo' => null,
//                'text' => TextManager::_('SC_aktualni_a_budouci')
//            ]
//        ];
//
//        $this ->povolitPrazdneDatumDo = true;

//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)) {
//            $this->nastavRozdeleni(true);
//        }

        $pravaRozdeleni = Prava::vratPrava('slcesty_prohl');
        if (!$pravaRozdeleni ->jsouNejakePrava()) {
            // uzivatel nema nastaveno pravo pro prohlizeni => rozdeleni se nebude filtrovat podle tohoto prava
            // (predpoklad je, ze ma uzivatel nastaveno pravo pro prohlizeni vsech SC, ktere muze editovat  nebo schvalovat)
            $pravaRozdeleni = null;
        }

        $this ->addFilterRozdeleni1(null, $pravaRozdeleni);
        $this ->addFilterRozdeleni2(null, $pravaRozdeleni);

        // $this ->addFilterRozdeleni('rozdeleni_sc_3', 98); - JEN TEST


        //$this->nastavAkci( '?menuid=sluzebnicesty' . ($schvalovani ? '&schvalovani=1' : '') );
        //$this->nastavAkci( '?menuid=sluzebnicesty' . ($schvalovani ? '&schvalovani=1' : '') );

//        if (isset($_GET['jenzahranicnisc'])) {
//            RonPortalSession::setValue('jenzahranicnisc', $_GET['jenzahranicnisc']);
//        }

//        $this ->nastavOsoby(true, [Prava::vratPrava('sluzebnicesty'), Prava::vratPrava('slcesty_prohl')]); // time(), time()
//        $this->povolitVsechyOsoby = true; // zobrazi volbu "vsichni"

        ($this ->addFilterPP(true))
            ->setPrava([Prava::vratPrava('sluzebnicesty'), Prava::vratPrava('slcesty_prohl')]);

        // filtr zahranicni/tuzemske
        $filterZahranicni = new FilterFormFilter('jenzahranicnisc');
        $filterZahranicni
            //->setCaption()
            ->setRowId('filter_zahranicni')
            ->setItems([
                    new FilterFormFilterItem('', TextManager::_('Vse')),
                    new FilterFormFilterItem('1', TextManager::_('Jen_zahranicni')),
                    new FilterFormFilterItem('0', TextManager::_('Jen_tuzemske'))
                ]
            )
            ->setSqlFilter('SluzebniCesta.Zahranicni = ');
        $this ->addFilter($filterZahranicni);

        // filtr zauctovano/nezauctovano
        if (RonPortalSettings::getIntValue('slcesty_filtr_zauctovano') == 2) {
            // ma se zobrazit samostatny filtr pro "zauctovano"
            $filterZauctovani = new FilterFormFilter('zauctovanisc');
            $filterZauctovani
                ->setCaption(TextManager::_('SC_Zauctovano'))
                ->setItems([
                    '' => TextManager::_('Vse'),
                    1 => TextManager::_('Jen_zauctovane_sc'),
                    0 => TextManager::_('Jen_nezauctovane_sc')]
                )
                ->setSqlFilter('SluzebniCesta.Zauctovano = ')
                ->setUserSettingsName(true);
            $this->addFilter($filterZauctovani);
        }

        // filtr na urovne schvaleni
        $this ->nastavFiltryUrovneSchvaleni();

//        if (isset($_GET['urovenschvaleni'])  &&  isset($this ->filtrySchvaleniSC[(int)$_GET['urovenschvaleni']])) {
//            $this ->urovenSchvaleniSC = (int)$_GET['urovenschvaleni'];
//            RonPortalSession::setValue('urovenschvaleni_sc', $this ->urovenSchvaleniSC);
//        } else {
//            $this ->urovenSchvaleniSC = (int)RonPortalSession::getValue('urovenschvaleni_sc');
//        }
//
//        if (!$this ->urovenSchvaleniSC) {
//            // pro filtr na schvalovani "vse" je obdobi nebo datum povinne
//            $this ->obdobiNeboDatumPovinne = true;
//        }
    }


    /**
     * Naplni pole se vsemi moznymi hodnotami pro filtr "Uroven schvaleni"
     * @return void
     */
    private function nastavFiltryUrovneSchvaleni()
    {
        //filtr na schvaleni nadrizenym 2 musi byt povolen v nastaveni
        $pocetUrovni = RonPortalSettings::getValue('slcesty_filtry_nadrizeny');
        if (!isset($pocetUrovni)) {
            // pro kompatibilitu se starym nastavenim
            $pocetUrovni = RonPortalSettings::getIntValue('slcesty_filtr_nadrizeny2') ? 2 : 1;
        }

        $zobrazitZauctovani = RonPortalSettings::getIntValue('slcesty_filtr_zauctovano') === 1;

        $this ->filtrUrovenSchvaleni = new FilterFormFilter('urovenschvaleni');

        $pokracovat = true;
        // Podle prav uzivatele sestavi SQL filtr pro polozku "Ke schvaleni"
        if ($zobrazitZauctovani) {
            // Zauctovani se zobrazuje jako dalsi uroven schvaleni
            $sqlFilterKeSchvaleni =
                'SluzebniCesta.Zauctovano = 0';

            if (!Prava::jsouNejakePrava('slcesty_zauct')  &&  (!Prava::existujePravo('slcesty_zauct_Z')  ||  !Prava::jsouNejakePrava('slcesty_zauct_Z'))) {
                $sqlFilterKeSchvaleni .= ' AND SluzebniCesta.Schvaleni4 IS NULL';
                // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje
            } else {
                $pokracovat = false;
            }
        } else {
            $sqlFilterKeSchvaleni =
                'SluzebniCesta.Schvaleni4 IS NULL';
            // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")
        }

        if ($pokracovat  &&  !Prava::jsouNejakePrava('slcesty_schval4')  &&  (!Prava::existujePravo('slcesty_schval4_Z')  ||  !Prava::jsouNejakePrava('slcesty_schval4_Z'))) {
            $sqlFilterKeSchvaleni .= ' AND SluzebniCesta.Schvaleni3 IS NULL ';
            // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje

            if (!Prava::jsouNejakePrava('slcesty_schval3')  &&  (!Prava::existujePravo('slcesty_schval3_Z')  ||  !Prava::jsouNejakePrava('slcesty_schval3_Z'))) {
                $sqlFilterKeSchvaleni .= ' AND SluzebniCesta.Schvaleni2 IS NULL ';
                // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje

                if (!Prava::jsouNejakePrava('slcesty_schval2')  &&  (!Prava::existujePravo('slcesty_schval2_Z')  ||  !Prava::jsouNejakePrava('slcesty_schval2_Z'))) {
                    $sqlFilterKeSchvaleni .= ' AND SluzebniCesta.Schvaleni1 IS NULL ';
                    // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
                }
            }
        }


        // filtry Vse/Ke schvaleni/Bez schvaleni ...
        $urovneItems = [
            '' => new FilterFormFilterItem('', TextManager::_('Vse')),

            '-1' => (new FilterFormFilterItem('-1', TextManager::_('SC_ke_schvaleni')))
                ->setSqlFilter($sqlFilterKeSchvaleni),

            '-2' => (new FilterFormFilterItem('-2', TextManager::_('SC_bez_schvaleni')))
                ->setSqlFilter(
                    'SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL' .
                    '  AND  SluzebniCesta.Schvaleni3 IS NULL  AND  SluzebniCesta.Schvaleni4 IS NULL' .
                    '  AND  SluzebniCesta.SchvalenoNadrizenym = 0 AND SluzebniCesta.SchvalenoNadrizenym2 = 0' .
                    ($zobrazitZauctovani ? ' AND SluzebniCesta.Zauctovano = 0' : '')
                )
        ];

        // filtry pro povoleni nadrizenymi
        if ($pocetUrovni) {
            $urovneItems['99'] =
                (new FilterFormFilterItem('99', TextManager::_('Povoleno_nadrizenym')))
                ->setSqlFilter(
                    'SluzebniCesta.SchvalenoNadrizenym = 1 AND SluzebniCesta.SchvalenoNadrizenym2 = 0 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');
        }

        if ($pocetUrovni >= 2) {
            $urovneItems['100'] =
                (new FilterFormFilterItem('100', TextManager::_('Povoleno_nadrizenym_2')))
                ->setSqlFilter(
                    'SluzebniCesta.SchvalenoNadrizenym2 = 1 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');
        }

        if ($pocetUrovni >= 3) {
            $urovneItems['101'] =
                (new FilterFormFilterItem('101', TextManager::_('Povoleno_nadrizenym_3')))
                ->setSqlFilter(
                    'SluzebniCesta.SchvalenoNadrizenym3 = 1  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');
        }

        if ($pocetUrovni >= 4) {
            $urovneItems['102'] =
                (new FilterFormFilterItem('102', TextManager::_('Povoleno_nadrizenym_4')))
                ->setSqlFilter(
                    'SluzebniCesta.SchvalenoNadrizenym4 = 1' .
                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');
        }

        // filtry pro schvaleni vyuctovani
        $urovneItems['1'] =
            (new FilterFormFilterItem('1', TextManager::_('Schvaleni_sc_plne_1')))
                ->setSqlFilter(
                    'SluzebniCesta.Schvaleni1 IS NOT NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');

        $urovneItems['2'] =
            (new FilterFormFilterItem('2', TextManager::_('Schvaleni_sc_plne_2')))
                ->setSqlFilter(
                    'SluzebniCesta.Schvaleni2 IS NOT NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL');

        $urovneItems['3'] =
            (new FilterFormFilterItem('3', TextManager::_('Schvaleni_sc_plne_3')))
                ->setSqlFilter(
                    'SluzebniCesta.Schvaleni3 IS NOT NULL  AND  SluzebniCesta.Schvaleni4 IS NULL');

        if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
            $urovneItems['4'] =
                (new FilterFormFilterItem('4', TextManager::_('Schvaleni_sc_plne_4')))
                    ->setSqlFilter(
                        'SluzebniCesta.Schvaleni4 IS NOT NULL');
        }

        // filtr pro zauctovani
        if ($zobrazitZauctovani) {
            // filtr "zauctovano" se ma zobrazit jako dalsi uroven schvaleni
            $urovneItems['80'] =
                (new FilterFormFilterItem('80', TextManager::_('SC_zauctovana')))
                    ->setSqlFilter(
                        'SluzebniCesta.Zauctovano = 1');
        }

        $this ->filtrUrovenSchvaleni
            ->setCaption(TextManager::_('Uroven_schvaleni'))
            ->setItems($urovneItems);

        $this
            ->addFilter($this ->filtrUrovenSchvaleni);

//        $this ->filtrySchvaleniSC = [
//            0 => TextManager::_('Vse'),
//            -1 => TextManager::_('SC_ke_schvaleni'),
//            -2 => TextManager::_('SC_bez_schvaleni')
//        ];
//
//
//        if ($pocetUrovni) {
//            $this ->filtrySchvaleniSC[99] = TextManager::_('Povoleno_nadrizenym');
//        }
//
//        if ($pocetUrovni >= 2) {
//            $this ->filtrySchvaleniSC[100] = TextManager::_('Povoleno_nadrizenym_2');
//        }
//
//        if ($pocetUrovni >= 3) {
//            $this ->filtrySchvaleniSC[101] = TextManager::_('Povoleno_nadrizenym_3');
//        }
//
//        if ($pocetUrovni >= 4) {
//            $this ->filtrySchvaleniSC[102] = TextManager::_('Povoleno_nadrizenym_4');
//        }
//
//        $this ->filtrySchvaleniSC[1] = TextManager::_('Schvaleni_sc_plne_1');
//        $this ->filtrySchvaleniSC[2] = TextManager::_('Schvaleni_sc_plne_2');
//        $this ->filtrySchvaleniSC[3] = TextManager::_('Schvaleni_sc_plne_3');
//
//        // filtr na schvaleni urovni 4 musi byt povolen v nastaveni
//        if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
//            $this ->filtrySchvaleniSC[4] = TextManager::_('Schvaleni_sc_plne_4');
//        }
//
//        // filtr na zauctovano se musi povolit v nastaveni
//        if (RonPortalSettings::getIntValue('slcesty_filtr_zauctovano') == 1) {
//            $this ->filtrySchvaleniSC[80] = TextManager::_('SC_zauctovana');
//        }
        
    }


    public function loadConfiguration()
    {
        // jeste pred nactenim konfigurace nacte filtr pro stav schvaleni, aby mohl zapnout povinne obdobi
        $this ->filtrUrovenSchvaleni ->load($this ->database);
        if (!$this ->filtrUrovenSchvaleni ->getValue()) {
            // pro filtr na schvalovani "vse" je obdobi nebo datum povinne
            $this ->obdobiNeboDatumPovinne = true;
        }

        parent::loadConfiguration();
    }


//    function renderFilterStav()
//    {
//        echo '<div class="filterrow" id="filter_urovenschvaleni">';
//        echo '<span class="filtercaption">' . TextManager::_('Uroven_schvaleni') . '</span>';
//
//        LinksList::renderLinks($this ->filtrySchvaleniSC, $this ->urovenSchvaleniSC, 'urovenschvaleni', '', false, true);
//
//        echo "</div>\n";
//    }


//    function renderFilterZahranicni()
//    {
//        echo '<div class="filterrow" id="filter_zahranicni">';
//        echo '<span class="filtercaption"></span>';
//
//        $data = array(
//            0 => TextManager::_('Vse'),
//            1 => TextManager::_('Jen_zahranicni'),
//            2 => TextManager::_('Jen_tuzemske'),
//        );
//
//        LinksList::renderLinks($data, RonPortalSession::getValue('jenzahranicnisc'), 'jenzahranicnisc', '', false, true); // hiddenField=true
//
//        echo "</div>\n";
//    }


    function renderFilters()
    {
        parent::renderFilters();

        if ($this ->schvalovani) {
            echo '<input type="hidden" name="schvalovani" value="1"/>';
        }

//        $this ->renderFilterZahranicni();   // vse / zahranicni / tuzemske
//        $this ->renderFilterStav();         // vsechno / ke schvaleni / ...
    }
}



class DokumentySCField extends BrowserField
{
    public $sqlIndexIdDokumentu;

    /**
     * DokumentySCField constructor.
     * @param int $sqlFieldIndex
     * @param int $sqlIndexIdDokumentu
     */
    public function __construct($sqlFieldIndex, $sqlIndexIdDokumentu)
    {
        parent::__construct();
        $this ->setSqlFieldIndex($sqlFieldIndex);
        $this->sqlIndexIdDokumentu = $sqlIndexIdDokumentu;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[$this ->sqlFieldIndex]) {
            $text = '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . TextManager::_('Pocet_priloh') . ': ' . $rowData[$this ->sqlFieldIndex] . '"></i>';

            // pokud je jen jeden dokument, tak rovnou zobrazi proklik na stahnuti souboru
            if ($rowData[$this ->sqlFieldIndex] == 1) {
                return "<a target='_blank' href='?menuid=sluzebnicesty&action=download&idslcesty={$rowData[0]}&record={$rowData[$this ->sqlIndexIdDokumentu]}'>$text</a>";
            }

            return $text;
        }

        return '';
    }
}



/**
 * Odkaz na schvaleni - podle prav
 * Class SchvaleniSlCestyField
 */
class SchvaleniSlCestyField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hint = '';

        if ($rowData['col-schvaleni4']) {
            $date = $rowData['col-schvaleni4'];
            $name = $rowData['col-schvalil4'];
            $class = 'textschvaleni4';
        } else if ($rowData['col-schvaleni3']) {
            $date = $rowData['col-schvaleni3'];
            $name = $rowData['col-schvalil3'];
            $class = 'textschvaleni3';
        } else if ($rowData['col-schvaleni2']) {
            $date = $rowData['col-schvaleni2'];
            $name = $rowData['col-schvalil2'];
            $class = 'textschvaleni2';
        } else if ($rowData['col-schvaleni1']) {
            $date = $rowData['col-schvaleni1'];
            $name = $rowData['col-schvalil1'];
            $class = 'textschvaleni1';
        } else if ($rowData['col-schvalenonadrizenym4']) {
            // povoleno/zamitnuto nadrizenym 4
            $date = $rowData['col-sn4_datumcas'];
            $name = $rowData['col-sn4_uzivatel'];

            if ($rowData['col-schvalenonadrizenym4'] == 1) {
                $class = 'textschvaleninadrizenym4';
                $hint = TextManager::_('Povoleno_nadrizenym_4');
            } else {
                $class = 'textzamitnutinadrizenym4';
                $hint = TextManager::_('Zamitnuto_nadrizenym_4');
            }
        } else if ($rowData['col-schvalenonadrizenym3']) {
            // povoleno/zamitnuto nadrizenym 3
            $date = $rowData['col-sn3_datumcas'];
            $name = $rowData['col-sn3_uzivatel'];

            if ($rowData['col-schvalenonadrizenym3'] == 1) {
                $class = 'textschvaleninadrizenym3';
                $hint = TextManager::_('Povoleno_nadrizenym_3');
            } else {
                $class = 'textzamitnutinadrizenym3';
                $hint = TextManager::_('Zamitnuto_nadrizenym_3');
            }
        } else if ($rowData['col-schvalenonadrizenym2']) {
            // povoleno/zamitnuto nadrizenym 2
            $date = $rowData['col-sn2_datumcas'];
            $name = $rowData['col-sn2_uzivatel'];

            if ($rowData['col-schvalenonadrizenym2'] == 1) {
                $class = 'textschvaleninadrizenym2';
                $hint = TextManager::_('Povoleno_nadrizenym_2');
            } else {
                $class = 'textzamitnutinadrizenym2';
                $hint = TextManager::_('Zamitnuto_nadrizenym_2');
            }
        } else if ($rowData['col-schvalenonadrizenym'] == 1) {
            // schvaleno nadrizenym

            $date = $rowData['col-sn_datumcas'];
            $name = $rowData['col-sn_uzivatel'];

            if ($rowData['col-schvalenonadrizenym'] == 1) {
                $class = 'textschvaleninadrizenym';
                $hint = TextManager::_('Povoleno_nadrizenym');
            } else {
                $class = 'textzamitnutinadrizenym';
                $hint = TextManager::_('Zamitnuto_nadrizenym');
            }
        } else if ($rowData['col-schvalenozamestnancem']) {
            return TextManager::_('Schvaleno_zamestnancem');
        } else {
            return TextManager::_('Schvaleni_sc_neschvaleno'); // implicitne prazdny text, lze nastavit v prekladech
        }

        return
            "<SPAN class='$class' title='$hint'>" .
            ($date ? DBUtils::getFormatedValue($this ->database, 'date', $date) : '') .
            ($name ? ' (' . StringUtils::htmlTextConvert($name) . ')' : '') .
            "</SPAN>\n";
    }
}


class SchvaleniSlCestyEditField extends BrowserField
{
    /** @var  string */
    public $kodPrava;
    public $indexSchvaleno;
    public $indexDatum;
    public $indexUzivatel;
    public $htmlIndex;
    /** @var int */
    public $indexPredchoziUroven;

    /** @var  SluzebniCestaBrowser */
    public $browser;

    public $existsEdit;

    /** @var  bool  Urcuje, jestli se ma dodrzovat posloupnost schvalovani */
    public $dodrzovatPosloupnost;


    /**
     * @param Browser $browser
     * @param string $kodPrava
     * @param int $indexSchvaleno
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param int $indexPredchoziUroven     Index pole s predchozi urovni schvalovani (pro kontrolu posloupnosti)
     * @param int $htmlIndex
     * @param string $caption
     */
    function __construct($browser, $kodPrava, $indexSchvaleno, $indexDatum, $indexUzivatel, $indexPredchoziUroven, $htmlIndex, $caption)
    {
        parent::__construct($browser ->database);

        $this ->code = 'col-edit-schvaleni' . $htmlIndex;

        $this ->align = "center";
        $this ->header =
            "<span class='schvaleniSCShowMenu' data-index='{$htmlIndex}' id='schvaleniSCMenu_{$htmlIndex}''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->htmlIndex = $htmlIndex;
        $this ->kodPrava = $kodPrava;
        $this ->indexSchvaleno = $indexSchvaleno;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;

        $this ->indexPredchoziUroven = $indexPredchoziUroven;

        $this ->dodrzovatPosloupnost = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_POSLOUPNOST_SCHVALOVANI);
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $schvaleno = isset($this ->indexSchvaleno) ? $rowData[$this ->indexSchvaleno] : $rowData[$this ->indexDatum];

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) {
                $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
            }
        }

        // pokud je zapnuto dodrzovani posloupnosti a jedna se o uroven 2 a vice, tak overi, jestli je predchozi uroven schvalena
        $predchoziUrovenSchvalena =  !$this ->dodrzovatPosloupnost  ||  !$this ->indexPredchoziUroven  ||  $rowData[$this ->indexPredchoziUroven];

        if ($predchoziUrovenSchvalena  &&  Prava::vratPrava($this ->kodPrava, $rowData['col-zahranicni'] ? $this ->kodPrava . '_Z' : null) ->lzeZobrazitPP((int)$rowData['col-idpp'])) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}_{$this ->htmlIndex}' value='1' />" .
                "<input type='checkbox' class='schvaleni-check  Schvaleni_{$this ->htmlIndex}' data-level='{$this ->htmlIndex}'" .
                    //" data-ronschvalenisc_index='{$this ->browser ->f_jsonRowIndex}'" .
                    " name='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' id='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}'" .
                    " data-original-value='" . ($schvaleno ? '1' : '0') . "'" .
                    ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) .
                " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . TextManager::_($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldpos
     * @param string $url
     * @return string
     */
    function getFooterText($fieldpos, $url = null)
    {
        return $this ->existsEdit
            ? "<input type='checkbox' class='schvaleni_ALL' data-index='{$this ->htmlIndex}' id='schvaleni_ALL_{$this ->htmlIndex}' title='' />\n"
            :'';
    }

}



//class KopieSCField extends BrowserField
//{
//    function __construct()
//    {
//        parent::__construct();
//        $this->align = 'left';
//        $this->code = 'col-kopie';
//    }
//
//
//    function getViewText($value, $rowNumber, $recordId, $rowData)
//    {
//        /** @noinspection HtmlUnknownTarget */
//        return
//            '<a href="?menuid=sluzebnicesty&formaction=newfromcopy&record=' . $rowData[0] . '">' .
//            '<i class="inline-glyph glyph-button fa glyph-copy" title="' . \Ron\ronText('Kopirovat_zaznam') . '"></i></a>';
//            //'" onclick="StopEventPropagation(event)"><i class="inline-glyph glyph-button fa glyph-copy" title="' . \Ron\ronText('Kopirovat_zaznam') . '"></i></a>';
//    }
//}




class ZruseniSCField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-zauctovano']) {
            // zauctovano - nejde smazat, zobrazuje fajfku
            /** @noinspection HtmlUnknownTarget */
            return '<i class="inline-glyph fa glyph-check" title="' . TextManager::_('SC_zauctovana') . '">';
        }

        if ($rowData['col-schvaleni1'] || $rowData['col-schvaleni2'] || $rowData['col-schvaleni3']  ||  $rowData['col-schvaleni4']) {
            // jiz je schvaleno, nejde smazat
            return '';
        }

        $idPP = (int)$rowData['col-idpp'];
        $zahranicni = (int)$rowData['col-zahranicni'];

        // zkontroluje prava na editaci SC
        $prava = Prava::vratPrava('sluzebnicesty', $zahranicni ? 'sluzebnicesty_Z' : null);
        if ($prava ->lzeZobrazitPP($idPP, $this ->database ->stringToTimestamp($rowData['col-datumcasod']), $this ->database ->stringToTimestamp($rowData['col-datumcasdo']))) {
            /** @noinspection HtmlUnknownTarget */
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_sluzebku') . '"></i>', TextManager::_('Opravdu_zrusit_sluzebku'),
                '?menuid=sluzebnicesty&action=zrusitsluzebku&record=' . $rowData[0] . Csrf::csrfUrlParameter('delete_sc_' . $rowData[0]));
        }

        return '';
    }
}



class PodrobnostiSCField extends BrowserField
{

    /**
     * PodrobnostiSCField constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this ->setType(BrowserField::FIELD_TYPE_COMMENT);
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $zacatek = '';
        if ($rowData['col-mistozacatku'] && (!RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY) || $rowData['col-mistozacatku'] != RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK))) {
            $zacatek = $rowData['col-mistozacatku'];
        }
        if ($rowData['col-popismistazacatku']) {
            $zacatek .=
                ($zacatek ? ', ' : '') .
                $rowData['col-popismistazacatku'];
        }

        $konec = '';
        if ($rowData['col-cil'] && (!RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_CASY_AUTOMATICKY) || $rowData['col-cil'] != RonPortalSettings::getValue(SluzebniCesta::SETTINGS_ZACATEK))) {
            $konec = $rowData['col-cil'];
        }
        if ($rowData['col-popiscile']) {
            $konec .=
                ($konec ? ', ' : '') .
                $rowData['col-popiscile'];
        }

        if ($zacatek || $konec || $rowData['col-pres']) {
            return parent::getViewText(
                $zacatek . (($zacatek && $konec) ? ' - ' : '') . $konec .
                ($rowData['col-pres'] ?  ', ' . TextManager::_('Pres') . ': ' . $rowData['col-pres'] : ''), // pres
                $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}


class SluzebniCestaBrowser extends RonPortalBrowser
{
    //nastaveno podle indexu pole v SQL dotazu
//    const INDEX_POCET_DOKUMENTU = 50;
//    const INDEX_ID_DOKUMENTU = 51;
//    const INDEX_ROZDELENI_KOD = 50;
//    const INDEX_ROZDELENI_POPIS = 51;

    /** @var  bool */
    var $schvalovani; // rezim schvalovani

    /** @var string */
    private $hiddenFields;

    private $pocetRadkuKeSchvaleni = 0;

    /** @var SluzebniCestyFilterForm */
    var $filterForm;

    private $filtrovatPodleTypuPrava = null;

    /**
     * @param Database $database
     * @param bool $schvalovani - zapina rezim hromadneho schvalovani
     */
    function __construct($database, $schvalovani)
    {
        parent::__construct($database, 'sluzebnicesty');

        $this->title = TextManager::_('Prehled_sluzebnich_cest');
        $this->tableId = 'seznamsluzebnichcest';

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;
        $this->code = 'browsersluzebnicesty';

        if ($schvalovani) {
            $this ->cssClass .= ' hromadne-schvalovani';
        }

        $this->sqlSelectPart =
            'SluzebniCesta.Id'; // 0

//        $this->sqlGroupByPart = $this->sqlSelectPart;
        $this->sqlGroupByPart = true;

//        $this->sqlSelectPart .=
//            ",\n COUNT(DokumentSluzebniCesty.Id) AS PocetDokumentu " . //je potreba nastavit konstanty s indexem pole !
//            ', MIN(DokumentSluzebniCesty.Id) AS IdDokumentu ';


        $this->sqlFromPart =
            '( ( ( ( SluzebniCesta INNER JOIN Osoba' .
            '  ON Osoba.RC = SluzebniCesta.RC' .
            ")\n" .
            ' INNER JOIN PracovniPomer' .
            ' ON PracovniPomer.RC = SluzebniCesta.RC' .
            '  AND PracovniPomer.OsobniCislo = SluzebniCesta.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = SluzebniCesta.DatumNastupu' .
            ")\n" .
            ' LEFT OUTER JOIN UcelSluzebniCesty' .
            '  ON UcelSluzebniCesty.Kod = SluzebniCesta.Ucel' .
            ")\n" .
            ' LEFT OUTER JOIN RadekUctovaniSCesty' .
            ' ON RadekUctovaniSCesty.SluzebniCesta_Id = SluzebniCesta.Id' .
            '  AND RadekUctovaniSCesty.Typ = 3' . // soucet vyuctovani v domaci mene
            ")\n" .
            ' LEFT OUTER JOIN DokumentSluzebniCesty' .
            " ON DokumentSluzebniCesty.SluzebniCesta_ID = SluzebniCesta.Id\n" .
            ' LEFT OUTER JOIN Stat CilovyStat ON (CilovyStat.Kod = SluzebniCesta.CilovyStat)';

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            //$this->sqlSelectPart .= ', Rozdeleni.Kod, Rozdeleni.Popis';
            //$this->sqlGroupByPart .= ', Rozdeleni.Kod, Rozdeleni.Popis';

            $this->sqlFromPart =
                '( ( ' .
                $this->sqlFromPart .
                ")\n" .
                ' LEFT OUTER JOIN RozdeleniOsob ' .
                ' ON RozdeleniOsob.RC = SluzebniCesta.RC ' .
                ' AND RozdeleniOsob.OsobniCislo = SluzebniCesta.OsobniCislo ' .
                ' AND RozdeleniOsob.DatumNastupu = SluzebniCesta.DatumNastupu ' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND RozdeleniOsob.PlatnostOd <= SluzebniCesta.DatumCasOd ' .
                ' AND (RozdeleniOsob.PlatnostDo >= {fn_EXTRACTDATE(SluzebniCesta.DatumCasOd)}  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ")\n" .
                ' LEFT OUTER JOIN Rozdeleni ' .
                ' ON Rozdeleni.KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP) .
                ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ';
        }

        $this->sqlWherePart = '';

        $this->primaryField = 0; // 'col-id';

        if (!$schvalovani) {
            $this->rowClickLink = '?menuid=sluzebnicesty&action=zobrazslcestu';
        }

        $this->browserUrl = '?menuid=sluzebnicesty&' . ($this ->schvalovani ? 'schvalovani=1&' : '');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.DatumUkonceni') ->setCode('col-datumukonceni-pp')
            ->setFormat('date');

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.Id', 0) ->setHeaderText('Cislo_sc') ->setAlign('center')
            ->setCode('col-id')
            ->setLink('?menuid=sluzebnicesty&action=zobrazslcestu') ->setOrdering(true)
            ->setExcludeFromGroupBy(true); // uz je v $selectPart

        $f = $this->addField(new UkonceniPracPomeruInfoField($this->database, 'col-datumukonceni-pp'))
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setLink('?menuid=sluzebnicesty&action=zobrazslcestu') ->setOrdering(true);

        if ($this ->schvalovani) {
            $f ->setFooter(TextManager::_('Schvalit_hromadne')); // ->setFooterColSpan(4);
        }

        $this->addField(new UkonceniPracPomeruInfoField($this->database, 'col-datumukonceni-pp'))
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            ->setLink('?menuid=sluzebnicesty&action=zobrazslcestu') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.OsobniCislo') ->setVisible(false) ->setHeaderText('Osobni_cislo')
            ->setCode('col-osobnicislo') ->setOrdering(true);

        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_TYP)) {
            // rozdeleni pro zobrazeni (typ rozdeleni je v nastaveni)
            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Kod')
                ->setCode('col-rozdeleni-kod');

            $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
                ->setSqlFieldName('Rozdeleni.Popis')
                ->setCode('col-rozdeleni-popis');

            // col-rozdeleni
            $this->addField(new RozdeleniBrowserField($this ->database, 'col-rozdeleni-kod', 'col-rozdeleni-popis'))
                ->setCode('col-rozdeleni');
        }

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.DatumCasOd') ->setHeaderText('Zacatek') ->setFormat('date')
            ->setCode('col-datumcasod')
            ->setLink('?menuid=sluzebnicesty&action=zobrazslcestu') ->setOrdering(true);

        $this->defaultSort = 'col-datumcasod';

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.DatumCasDo') ->setHeaderText('Konec') ->setFormat('date') ->setOrdering(true)
            ->setCode('col-datumcasdo');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.CilovyStat') ->setHeaderText('Cilovy_stat') ->setOrdering(true)
            ->setCode('col-kodcilovehostatu');

        $this->addField()
            ->setSqlFieldName('CilovyStat.Popis') ->setHeaderText('Cilovy_stat') ->setOrdering(true)
            ->setCode('col-cilovystat')
            ->setVisible(RonPortalSettings::getIntValue('cilovystatcsbrowser'))
            ->setTranslationPrefix('db_stat_popis_', 'col-kodcilovehostatu');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN) // pro moznost jazykoveho prekladu nasledujiciho pole
            ->setSqlFieldName('SluzebniCesta.Ucel') ->setCode('col-koducelu');

        $this->addField()
            ->setSqlFieldName('UcelSluzebniCesty.Popis') ->setHeaderText('Ucel_sc') ->setOrdering(true)
            ->setCode('col-ucel') ->setClassName('col_ucel')
            ->setTranslationPrefix('db_ucelsluzebnicesty_popis_', 'col-koducelu');

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.Popis') ->setVisible(!$this->schvalovani) // pri schvalovani se nezobrazi
            ->setCode('col-popis')
            ->setHeaderText('Popis_sc') ->setOrdering(1) ->setWrap(true);

        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('SluzebniCesta.Zahranicni') ->setHeaderText('Typ_sc')
            ->setCode('col-zahranicni') ->setClassName('seznamsc_typ')
            ->setOptions([TextManager::_('Tuzemska'), TextManager::_('Zahranicni')]);


        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.MistoZacatku') ->setHeaderText('Misto_zacatku') ->setOrdering(true)
            ->setCode('col-mistozacatku');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.PopisMistaZacatku') ->setHeaderText('Popis_mista_zacatku') ->setOrdering(true)
            ->setCode('col-popismistazacatku');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Cil') ->setHeaderText('Cil_sc') ->setOrdering(true)
            ->setCode('col-cil');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.PopisCile') ->setHeaderText('Popis_cile') ->setOrdering(true)
            ->setCode('col-popiscile');


        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Pres') ->setHeaderText('Pres') ->setOrdering(true)
            ->setCode('col-pres');

        $this->addField()
            ->setSqlFieldName('RadekUctovaniSCesty.Castka') ->setHeaderText('Sluzebni_cesta_celkem') ->setFormat('number2')
            ->setCode('col-vyuctovani')
            ->setAlign('right') ->setOrdering(1) ->setClassName('col_vyuctovani');

        for ($i = 1;  $i <= 5;  $i++) {
            //if (RonPortalSettings::getValue('slcesty_uzivatelska' . $i)) {
                $this->addField()
                    ->setSqlFieldName('SluzebniCesta.Uzivatelska' . $i)
                    ->setCode('col-uzivatelska' . $i)->setHeader(RonPortalSettings::getValue('slcesty_uzivatelska' . $i))
                    ->setOrdering(1)->setClassName('uzivatelska' . $i)->setWrap(true)
                    ->setVisible(RonPortalSettings::getValue('slcesty_uzivatelska' . $i));
            //}
        }

        // data pro schvaleni nadrizenym 1
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SchvalenoNadrizenym') ->setCode('col-schvalenonadrizenym');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN_Uzivatel') ->setCode('col-sn_uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN_DatumCas') ->setCode('col-sn_datumcas')
            ->setFormat('datetime');

        // data pro schvaleni nadrizenym 2
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SchvalenoNadrizenym2') ->setCode('col-schvalenonadrizenym2');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN2_Uzivatel') ->setCode('col-sn2_uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN2_DatumCas') ->setCode('col-sn2_datumcas')
            ->setFormat('datetime');

        // data pro schvaleni nadrizenym 3
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SchvalenoNadrizenym3') ->setCode('col-schvalenonadrizenym3');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN3_Uzivatel') ->setCode('col-sn3_uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN3_DatumCas') ->setCode('col-sn3_datumcas')
            ->setFormat('datetime');

        // data pro schvaleni nadrizenym 4
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SchvalenoNadrizenym4') ->setCode('col-schvalenonadrizenym4');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN4_Uzivatel') ->setCode('col-sn4_uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.SN4_DatumCas') ->setCode('col-sn4_datumcas')
            ->setFormat('datetime');

        // data pro schvaleni 1
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvaleni1') ->setCode('col-schvaleni1') ->setFormat('datetime');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvalil1') ->setCode('col-schvalil1');

        // data pro schvaleni 2
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvaleni2') ->setCode('col-schvaleni2');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvalil2') ->setCode('col-schvalil2');

        // data pro schvaleni 3
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvaleni3') ->setCode('col-schvaleni3');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvalil3') ->setCode('col-schvalil3');

        // data pro schvaleni 4
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvaleni4') ->setCode('col-schvaleni4');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Schvalil4') ->setCode('col-schvalil4');

        // data pro schvaleni nadrizenym 4
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Zauctovano') ->setCode('col-zauctovano');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Zauctovano_Uzivatel') ->setCode('col-zauctovano_uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Zauctovano_DatumCas') ->setCode('col-zauctovano_datumcas')
            ->setFormat('datetime');


        if ($this->schvalovani) {
            // rezim hromadneho schvalovani - zobrazi checkboxy
            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_nadriz',
                'col-schvalenonadrizenym', 'col-sn_datumcas', 'col-sn_uzivatel', null,
                'N', TextManager::_('Povoleno_nadrizenym')));

            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_nadriz2',
                'col-schvalenonadrizenym2', 'col-sn2_datumcas', 'col-sn2_uzivatel', null,
                'N2', TextManager::_('Povoleno_nadrizenym_2')))
                ->setVisible(RonPortalSettings::getIntValue('slcesty_filtry_nadrizeny') > 1);

            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_nadriz3',
                'col-schvalenonadrizenym3', 'col-sn3_datumcas', 'col-sn3_uzivatel', null,
                'N3', TextManager::_('Povoleno_nadrizenym_3')))
                ->setVisible(RonPortalSettings::getIntValue('slcesty_filtry_nadrizeny') > 2);

            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_nadriz4',
                'col-schvalenonadrizenym4', 'col-sn4_datumcas', 'col-sn4_uzivatel', null,
                'N4', TextManager::_('Povoleno_nadrizenym_4')))
                ->setVisible(RonPortalSettings::getIntValue('slcesty_filtry_nadrizeny') > 3);


            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_schval1', null, 'col-schvaleni1', 'col-schvalil1', null, '1', TextManager::_('Schvaleni_sc_plne_1')));
            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_schval2', null, 'col-schvaleni2', 'col-schvalil2', 'col-schvaleni1', '2', TextManager::_('Schvaleni_sc_plne_2')));
            $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_schval3', null, 'col-schvaleni3', 'col-schvalil3', 'col-schvaleni2', '3', TextManager::_('Schvaleni_sc_plne_3')));

            if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
                $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_schval4',
                    null, 'col-schvaleni4', 'col-schvalil4', 'col-schvaleni3',    // pole schvaleni4 je datum
                    '4', TextManager::_('Schvaleni_sc_plne_4')));
            }

            $pravoZauctovat = Prava::vratPrava('slcesty_zauct');
            if ($pravoZauctovat ->jsouNejakePrava()) {
                $this->addField(new SchvaleniSlCestyEditField($this, 'slcesty_zauct', 'col-zauctovano', 'col-zauctovano_datumcas', 'col-zauctovano_uzivatel', null, 'Z', TextManager::_('SC_Zauctovano')));
            }
        } else {
            $this->addField()
                ->setSqlFieldName('SluzebniCesta.SchvalenoZamestnancem') ->setFormat('anone')
                ->setCode('col-schvalenozamestnancem') ->setVisible(false)
                ->setAlign('center') ->setHeaderText('Schvaleno_zamestnancem') ->setOrdering(true);

            $this->addField(new SchvaleniSlCestyField($this ->database))
                ->setCode('col-schvaleni') ->setAlign('left') ->setHeaderText('Schvaleni_sc');
        }

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.DopravniProstredek') ->setHeaderText('Dopravni_prostredek') ->setOrdering(true)
            ->setCode('col-dopravniprostredek') ->setVisible(false)
            ->setTranslationPrefix('db_dopravniprostredek_popis_');

        $this->addField()
            ->setSqlFieldName('SluzebniCesta.RegistracniZnacka') ->setHeaderText('Vozidlo') ->setOrdering(true)
            ->setCode('col-registracniznacka') ->setVisible(0);

        // poznamky
        $this ->addField(new PodrobnostiSCField())
            ->setCode('col-podrobnosti')
            ->setClassName('poznamka browserpoznamka padding-left-2') ->setHeaderColSpan(2)
            ->setWrap(true);


        if ($this ->schvalovani) {
            // v rezimu schvalovani je poznamka na dalsim radku
            $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName('SluzebniCesta.Popis')
                ->setCode('col-popis-comment')
                ->setClassName('poznamka browserpoznamka padding-left-2') ->setHeaderColSpan(2)
                ->setWrap(true);
        }

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('SluzebniCesta.PoznamkaKVraceni')
            ->setCode('col-poznamkakvraceni')
            ->setHeader(TextManager::_('SC_Vracena') . ': ') ->setHeaderColSpan(2) ->setHeaderClass('poznamka browserpoznamka align-right')
            ->setClassName('poznamka browserpoznamka padding-left-2')
            ->setWrap(true);


        // ostatni hidden pole
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SluzebniCesta.Zauctovano')
            ->setCode('col-zauctovano');

        // dokumenty
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('COUNT(DokumentSluzebniCesty.Id) AS PocetDokumentu')
            ->setCode('col-pocetdokumentu')
            ->setExcludeFromGroupBy(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('MIN(DokumentSluzebniCesty.Id) AS IdDokumentu')
            ->setCode('col-iddokumentu')
            ->setExcludeFromGroupBy(true);

        $this->addField(new DokumentySCField('col-pocetdokumentu', 'col-iddokumentu'))
            ->setCode('col-dokumenty') ->setAlign('center');

        // odkazy
        if (!$this ->schvalovani  &&  Prava::jsouNejakePrava('sluzebnicesty')) {
            $this->addField(new ZruseniSCField($this ->database))
                ->setCode('col-zrusit') ->setOrdering(true) ->setSqlFieldName('SluzebniCesta.Zauctovano')
                ->setHeader('<span title="' . TextManager::_('SC_zauctovana') . '/' . TextManager::_('Zrusit') . '">' . TextManager::_('Sloupec_zrusit_sc') . '</span>');

            $this->addField(BrowserField::FIELD_TYPE_TEXT)
                ->setText('<i class="inline-glyph glyph-button fa glyph-copy" title="' . TextManager::_('Kopirovat_zaznam') . '"></i>')
                ->setLink('?menuid=sluzebnicesty&formaction=copy&record=')
                ->setCode('col-kopie');
        }
    }


    function setFilterForm()
    {
        $this ->filterForm = new SluzebniCestyFilterForm($this ->database, $this ->schvalovani);
        $this ->filterForm ->htmlId = 'sc_filter_form';
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'filtrovatPodleTypuPrava') {
            $this ->filtrovatPodleTypuPrava = $valueText;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    function renderFilter()
    {
        parent::renderFilter();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if (!$this->schvalovani) {
            $pravo = Prava::vratHodnotuPrava('sluzebnicesty');
            if ($pravo) {
                echo "<div class='normaltext d-inline-block' id='sc_links'><a href='?menuid=sluzebnicesty&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
            }

            echo
                "<div class='d-inline-block padding-left-4' id='sc_hromadneschvalovani'>" .
                "<input type='button' class='button clickHref' name='zobrazithromadne' value='" . TextManager::_('Schvalit_hromadne') . "' data-ron-href='?menuid=sluzebnicesty&schvalovani=1'/>" .
                "</div>";
        }
    }


    function getSQLFilter()
    {
//        // uroven schvaleni (vse, ke schvaleni, ...)
//        switch ($this ->filterForm ->urovenSchvaleniSC) {
//            case -2 : // bez schvaleni
//                $result =
//                    'SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL' .
//                    '  AND  SluzebniCesta.Schvaleni3 IS NULL  AND  SluzebniCesta.Schvaleni4 IS NULL  AND  SluzebniCesta.SchvalenoNadrizenym = 0 AND SluzebniCesta.SchvalenoNadrizenym2 = 0 ';
//                break;
//
//            case 80 : // zauctovano
//                $result = 'SluzebniCesta.Zauctovano = 1 ';
//                break;
//
//            case 99 : // povolene nadrizenym a zaroven nepovolene nadrizenym 2, urovni 1,2,3 ani 4
//                $result =
//                    'SluzebniCesta.SchvalenoNadrizenym = 1 AND SluzebniCesta.SchvalenoNadrizenym2 = 0 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
//                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
//                break;
//
//            case 100 : //povolene nadrizenym 2 a zaroven neschvalene urovni 1,2,3 ani 4
//                $result =
//                    'SluzebniCesta.SchvalenoNadrizenym2 = 1 AND SluzebniCesta.SchvalenoNadrizenym3 = 0  AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
//                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
//                break;
//
//            case 101 : //povolene nadrizenym 3 a zaroven neschvalene urovni 1,2,3 ani 4
//                $result =
//                    'SluzebniCesta.SchvalenoNadrizenym3 = 1 AND SluzebniCesta.SchvalenoNadrizenym4 = 0' .
//                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
//                break;
//
//            case 102 : //povolene nadrizenym 4 a zaroven neschvalene urovni 1,2,3 ani 4
//                $result =
//                    'SluzebniCesta.SchvalenoNadrizenym4 = 1' .
//                    ' AND  SluzebniCesta.Schvaleni1 IS NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL';
//                break;
//
//            case 1 :
//                $result = 'SluzebniCesta.Schvaleni1 IS NOT NULL  AND  SluzebniCesta.Schvaleni2 IS NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
//                break;
//
//            case 2 :
//                $result = 'SluzebniCesta.Schvaleni2 IS NOT NULL  AND  SluzebniCesta.Schvaleni3 IS NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
//                break;
//
//            case 3 :
//                $result = 'SluzebniCesta.Schvaleni3 IS NOT NULL AND SluzebniCesta.Schvaleni4 IS NULL ';
//                break;
//
//            case 4 :
//                $result = 'SluzebniCesta.Schvaleni4 IS NOT NULL ';
//                break;
//
//            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
//                $result = 'SluzebniCesta.Schvaleni4 IS NULL';
//                // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")
//
//                $existujePravoZ = Prava::existujePravo('slcesty_schval4_Z');
//
//                if (!Prava::jsouNejakePrava('slcesty_schval4')  &&  (!$existujePravoZ  ||  !Prava::jsouNejakePrava('slcesty_schval4_Z'))) {
//                    $result .= ' AND SluzebniCesta.Schvaleni3 IS NULL ';
//                    // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje
//
//                    $existujePravoZ = Prava::existujePravo('slcesty_schval3_Z');
//
//                    if (!Prava::jsouNejakePrava('slcesty_schval3')  &&  (!$existujePravoZ  ||  !Prava::jsouNejakePrava('slcesty_schval3_Z'))) {
//                        $result .= ' AND SluzebniCesta.Schvaleni2 IS NULL ';
//                        // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje
//
//                        $existujePravoZ = Prava::existujePravo('slcesty_schval2_Z');
//
//                        if (!Prava::jsouNejakePrava('slcesty_schval2')  &&  (!$existujePravoZ  ||  !Prava::jsouNejakePrava('slcesty_schval2_Z'))) {
//                            $result .= ' AND SluzebniCesta.Schvaleni1 IS NULL ';
//                            // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
//                        }
//                    }
//                }
//
//                break;
//
//            default : // vse
//                $result = '';
//        }

        $result = '';

        $filterFormFilter = $this ->filterForm ->buildSqlFilter();

        if ($filterFormFilter) {
            $params = $filterFormFilter ->parameters;
            $result .=
                ($result ? ' AND ' : '') .
                '(' . $filterFormFilter ->sqlText . ')';
        } else {
            $params = $this->database->createParameters();
        }

        // obdobi
        if ($this ->filterForm ->getFilterValue('obdobi', 'jenObdobi')) {  //  ||  $this ->filterForm ->getFilter('obdobi') ->isRequired()) {
            $result .=
                ($result ? ' AND ' : '') .
                'SluzebniCesta.DatumCasDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
                ' AND SluzebniCesta.DatumCasOd < ' . $params ->addDateParameter(DateUtils::incDay($this ->filterForm ->getFilterValue('obdobi', 'datumDo'), 1));
        } else if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'SluzebniCesta.DatumCasDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                ($this ->filterForm ->getFilterValue('datumod', 'datumDo')
                    ? ' AND SluzebniCesta.DatumCasOd < ' . $params ->addDateParameter(
                        DateUtils::incDay($this ->filterForm ->getFilterValue('datumod', 'datumDo'), 1))
                    : '');
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            $value = $rozdeleniFilter ->getValue();
            if (isset($value)) {
                $filtrR = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'SluzebniCesta',
                    $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                    '{fn_EXTRACTDATE(SluzebniCesta.DatumCasOd)}', 'SluzebniCesta.DatumCasOd',
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params);

                if ($filtrR) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        '(' . $filtrR . ')';
                }
            }
        }

        // jen vybrana osoba
        if ($this ->filterForm ->getFilterValue('osoba') !== null) {
            $result .=
                ($result ? ' AND ' : '') .
                ' SluzebniCesta.RC = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                ' AND SluzebniCesta.OsobniCislo = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                ' AND SluzebniCesta.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu'));
        }

        if ($this ->filtrovatPodleTypuPrava) {
            $prava = Prava::vratPrava($this ->filtrovatPodleTypuPrava);

            $filtrPodlePrav = $prava ->sqlFiltr('SluzebniCesta', time(), time(), $params);

            if ($filtrPodlePrav) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $filtrPodlePrav;
            }
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }


    private function lzeZobrazitSC($kodPrava, $idPP, $zahranicni, $datumOd, $datumDo)
    {
        $prava = Prava::vratPrava($kodPrava, $zahranicni ? $kodPrava . '_Z' : null);
        return $prava ->lzeZobrazitPP($idPP, $datumOd, $datumDo);
//        $this->pravaNadrizeny = Prava::vratPrava('slcesty_nadriz');
//        $this->pravaNadrizeny2 = Prava::vratPrava('slcesty_nadriz2');
//        $this->pravaNadrizeny3 = Prava::vratPrava('slcesty_nadriz3');
//        $this->pravaNadrizeny4 = Prava::vratPrava('slcesty_nadriz4');
    }


    /**
     * @param array $values
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws \Exception
     */
    function scEnableLine($values, $datumOd, $datumDo)
    {
        //$this ->f_jsonRowIndex = null;
        $idPP = (int)$values['col-idpp'];
        $zahranicni = (int)$values['col-zahranicni'];

        //if ($this ->filterForm ->urovenSchvaleniSC == -1) {
        if ($this ->filterForm ->filtrUrovenSchvaleni ->getValue() == -1) {
            // filtr Ke schvaleni - zkontroluje, jestli to ceka na me

            if (RonPortalSettings::getIntValue('slcesty_filtr_zauctovano') === 1) {
                // zauctovani se zobrazuje jako dalsi uroven schvaleni - pokud je zauctovano, nezobrazuji
                if ($values['col-zauctovano']) {
                    return false;
                }

                if ($values['col-schvaleni4']) {
                    //je schvaleno 4, ceka na zauctovani => pokud mam pravo, ceka na me
                    return $this ->lzeZobrazitSC('slcesty_zauct', $idPP, $zahranicni, $datumOd, $datumDo);
                }
            } else {
                // zauctovani se resi samostatne nebo se vubec neresi - uroveni 4 je nejvyssi
                if ($values['col-schvaleni4']) {
                    // jiz je schvaleno nejvyssi urovni = neceka na nikoho
                    return false;
                }
            }

            if ($values['col-schvaleni3']) {
                //je schvaleno 3, ceka na 4 => pokud mam pravo, ceka na me
                return $this ->lzeZobrazitSC('slcesty_schval4', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            if ($values['col-schvaleni2']) {
                // je schvaleno 2, ceka na 3 => pokud mam pravo, ceka na me
                return $this ->lzeZobrazitSC('slcesty_schval3', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            if ($values['col-schvaleni1']) {
                // je schvaleno 1, ceka na 2 => pokud mam pravo, ceka na me
                return $this ->lzeZobrazitSC('slcesty_schval2', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            // neni schvaleno zadnou urovni => pokud mam pravo na 1, ceka na me
            if ($this ->lzeZobrazitSC('slcesty_schval1', $idPP, $zahranicni, $datumOd, $datumDo)) {
                return true;
            }

            // Dale zkontroluje prava na schvalovani pred sluzebni cestou

            if ($values['col-schvalenonadrizenym4']) {
                // je schvaleno nadrizenym 4 => uz neceka na nikoho
                return false;
            }

            if ($values['col-schvalenonadrizenym3']) {
                //je schvaleno nadrizenym 3, ceka na 4 => pokud mam pravo, ceka na me
                return $this->lzeZobrazitSC('slcesty_nadriz4', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            if ($values['col-schvalenonadrizenym2']) {
                //je schvaleno nadrizenym 2, ceka na 3 => pokud mam pravo, ceka na me
                return $this->lzeZobrazitSC('slcesty_nadriz3', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            if ($values['col-schvalenonadrizenym']) {
                //je schvaleno nadrizenym 1, ceka na 2 => pokud mam pravo, ceka na me
                return $this->lzeZobrazitSC('slcesty_nadriz2', $idPP, $zahranicni, $datumOd, $datumDo);
            }

            return $this->lzeZobrazitSC('slcesty_nadriz', $idPP, $zahranicni, $datumOd, $datumDo);
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->lzeZobrazitSC('sluzebnicesty', $idPP, $zahranicni, $datumOd, $datumDo)) {
                return true;
            }
            if ($this->lzeZobrazitSC('slcesty_prohl', $idPP, $zahranicni, $datumOd, $datumDo)) {
                return true;
            }
            if ($this->lzeZobrazitSC('slcesty_zauct', $idPP, $zahranicni, $datumOd, $datumDo)) {
                return true;
            }
        }

        if ($this->lzeZobrazitSC('slcesty_schval1', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }
        if ($this->lzeZobrazitSC('slcesty_schval2', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }
        if ($this->lzeZobrazitSC('slcesty_schval3', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }
        if ($this->lzeZobrazitSC('slcesty_schval4', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }

        if ($this->lzeZobrazitSC('slcesty_nadriz', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }

        if ($this->lzeZobrazitSC('slcesty_nadriz2', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }

        if ($this->lzeZobrazitSC('slcesty_nadriz3', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }

        if ($this->lzeZobrazitSC('slcesty_nadriz4', $idPP, $zahranicni, $datumOd, $datumDo)) {
            return true;
        }

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        $datumOd = $this ->database ->stringToTimestamp($values['col-datumcasod']);
        $datumDo = $this ->database ->stringToTimestamp($values['col-datumcasdo']);

        $result = $this ->scEnableLine($values, $datumOd, $datumDo);
        if (!$result) {
            return false;
        }

        if ($this ->schvalovani) {
            $this ->hiddenFields .= "<input type='hidden' name='idSC_{$this ->pocetRadkuKeSchvaleni}' value='{$values[0]}'/>";
            $this ->pocetRadkuKeSchvaleni++;

//            $this ->f_jsonRowIndex = count($this ->f_jsonObject['items']);
//            $this ->f_hiddenFields .= "<input type='hidden' name='idSC_{$this ->f_jsonRowIndex}' value='{$values[0]}'/>";
//
//            $this ->f_jsonObject['items'][] = [
//                'id' => $values[0],
//                'pravo_n' => $this ->pravaNadrizeny ->lzeZobrazitPP($values['col-idpp'], $datumod, $datumdo) ? 1 : 0,
//                'pravo_n2' => $this ->pravaNadrizeny2 ->lzeZobrazitPP($values[8], $datumod, $datumdo) ? 1 : 0,
//                'pravo_1' => $this ->pravaSchvaleni1 ->lzeZobrazitPP($values[8], $datumod, $datumdo) ? 1 : 0,
//                'pravo_2' => $this ->pravaSchvaleni2 ->lzeZobrazitPP($values[8], $datumod, $datumdo) ? 1 : 0,
//                'pravo_3' => $this ->pravaSchvaleni3 ->lzeZobrazitPP($values[8], $datumod, $datumdo) ? 1 : 0,
//                'pravo_4' => $this ->pravaSchvaleni4 ->lzeZobrazitPP($values[8], $datumod, $datumdo) ? 1 : 0,
//                'schvaleno_n' => $values[23] ? 1 : 0,
//                'schvaleno_n2' => $values[26] ? 1 : 0,
//                'schvaleno_1' => $values[2] ? 1 : 0,
//                'schvaleno_2' => $values[4] ? 1 : 0,
//                'schvaleno_3' => $values[6] ? 1 : 0,
//                'schvaleno_4' => $values[30] ? 1 : 0
//            ];
//
//        } else {
//            $this ->f_jsonRowIndex = null;
        }

        return true;
    }


    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        $className .=
            ' sc_row' .
            ($values['col-zahranicni'] ? ' zahranicni' : '') .
            ($values['col-schvaleni4'] ? ' sc_schvalena4' : '') .
            ($values['col-schvaleni3'] ? ' sc_schvalena3' : '') .
            ($values['col-schvaleni2'] ? ' sc_schvalena2' : '') .
            ($values['col-schvaleni1'] ? ' sc_schvalena1' : '') .
            (($values['col-schvalenonadrizenym4'] == 1) ? ' sc_schvalenanadrizenym4' : '') .
            (($values['col-schvalenonadrizenym4'] == 2) ? ' sc_zamitnutanadrizenym4' : '') .
            (($values['col-schvalenonadrizenym3'] == 1) ? ' sc_schvalenanadrizenym3' : '') .
            (($values['col-schvalenonadrizenym3'] == 2) ? ' sc_zamitnutanadrizenym3' : '') .
            (($values['col-schvalenonadrizenym2'] == 1) ? ' sc_schvalenanadrizenym2' : '') .
            (($values['col-schvalenonadrizenym2'] == 2) ? ' sc_zamitnutanadrizenym2' : '') .
            (($values['col-schvalenonadrizenym'] == 1) ? ' sc_schvalenanadrizenym' : '') .
            (($values['col-schvalenonadrizenym'] == 2) ? ' sc_zamitnutanadrizenym' : '');

        return
            parent::getRowAttributes($values, $className, $rowNumber) .
            ' id="sc_row_' . $values[0] . '"' .
            ' data-id-sc="' . $values[0] . '"' .
            ' data-info-sc="' . StringUtils::htmlTextConvert($values['col-jmeno'] . ' ' . $values['col-prijmeni']) . ', ' .
                StringUtils::format("date", $this ->database ->stringToTimestamp($values['col-datumcasod'])) . '"' .
            ' ' . Csrf::csrfDataAttribute('sc_schvaleni_' . $values[0]);
    }


    function renderContent()
    {
        echo '<div class="position-relative">';

        $this ->hiddenFields = '';

        echo
            '<ul class="popupmenu"  id="schvaleniSCMenu">' .
            "<li data-ron-hodnotaschvaleni='1'>" . TextManager::_('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . TextManager::_('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . TextManager::_('form.Original') . "</li>".
            '</ul>';

//        if ($this ->schvalovani) {
//            echo
//                '<form method="post" name="hromadneSchvalovaniSC" id="hromadneSchvalovaniSCForm" action="?menuid=sluzebnicesty" data-csrfkey="' . Csrf::getCsrfKey() . '">' .
//                '<input type="hidden" name="hromadneSchvaleni" value="1"/>';
//        }


        parent::renderContent();

        if ($this ->hiddenFields) {
            echo $this ->hiddenFields;
        }

        //echo '<input type="hidden" name="pocet" value="' . count($this ->f_jsonObject['items']) . '" />';
        echo '<input type="hidden" name="pocet" value="' . $this ->pocetRadkuKeSchvaleni . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                //"<input type='button' class='submitbutton' value='" . \Ron\ronText('Provest') . "' id='provestHromadneSchvaleniSC' />" .
                '<button class="submitbutton" id="provestHromadneSchvaleniSC">' . TextManager::_('Provest') . '</button>' .
                '<button class="button clickHref" id="stornoHromadneSchvaleniSC" data-ron-href="?menuid=sluzebnicesty">' . TextManager::_('Cancel') . '</button>' .
                '</div>'.
//                "</form>";
                "\n";
        } else {
            if (Prava::vratHodnotuPrava('sluzebnicesty')) {
                echo "<div class='normaltext'><a href='?menuid=sluzebnicesty&formaction=new'>" . TextManager::_('New_record') . "</a></div>\n";
            }
        }

        echo '</div>';

        if ($this ->schvalovani) {
            ProgressDialog::renderProgressDialog(TextManager::_('Schvaleni_sluzebnich_cest'), [
                'caption' => TextManager::_('Probiha_schvalovani_sluzebnich_cest')
            ]);
        }

    }
}
