<?php
/**
 * Formularova pole pro formular sluzebni cesty
 */

namespace RonPortal\SluzebniCesty;


use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\RonPortalSettings;
use Tracy\Dumper;

class OsobaSCFormField extends OsobaFormField
{
    public function zkontrolujPravo($pp)
    {
        // parent::zkontrolujPravo($pp);  - tady nedela nic - kontrola prava se provede az po nacteni vsech poli, protoze muze byt ruzna napr. pro zahranicni SC
    }
}


/**
 * Formularove pole pro formular sluzebni cesty - umoznuje omezit editaci pole jen pro tuzemske/zahranicni sluzebni cesty
 */
class SluzebniCestaFormField extends FormField
{
    const ZAHRANICNI_I_TUZEMSKE = 0;
    const JEN_ZAHRANICNI = 1;
    const JEN_TUZEMSKE = 2;

    public $jenZahranicni;

    public function __construct($fieldType = self::FIELD_TYPE_EDIT, $jenZahranicni = self::ZAHRANICNI_I_TUZEMSKE)
    {
        parent::__construct($fieldType);
        $this
            ->setTableAlias('SluzebniCesta')
            ->setJenZahranicni($jenZahranicni);
    }


    /**
     * @param int|mixed $jenZahranicni
     * @return SluzebniCestaFormField
     */
    public function setJenZahranicni($jenZahranicni)
    {
        $this->jenZahranicni = $jenZahranicni;

        $this ->rowClass = $this ->rowClass
            ? StringUtils::removeSubstring(' ', $this ->rowClass, ['jen-tuzemska-sc', 'jen-zahranicni-sc'])
            : $this ->rowClass;

        if ($jenZahranicni) {
            $this ->rowClass =
                ($this ->rowClass ?: 'row') .
                (($jenZahranicni === self::JEN_TUZEMSKE)
                    ? ' jen-tuzemska-sc'
                    : ' jen-zahranicni-sc');
        }

        return $this;
    }

    protected function loadConfigurationItem($key, $value)
    {
        if ($key === 'jenZahranicni') {
            $this ->setJenZahranicni((int)$value);
        } else {
            parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * Podle nastaveni pole a odeslane hodnoty "zahranicni" urci, jestli je mozne editovat pole pro
     * @return bool
     */
    public static function lzeEditovatProSC($poleJenZahranicni)
    {
        switch ($poleJenZahranicni) {
            case self::JEN_ZAHRANICNI :
                // pole je jen pro zahranicni SC
                return ($_POST['Zahranicni'] ?? false);

            case self::JEN_TUZEMSKE :
                // pole je jen pro tuzemske SC
                return !($_POST['Zahranicni'] ?? false);

            default :
                return true;
        }
    }


    public function getPostValue($utf8 = false)
    {
        if (!self::lzeEditovatProSC($this ->jenZahranicni)) {
            // pole neni editovatelne pro odeslanou sluzebni cestu (podle priznaku "zahranicni"
            return $this ->defaultValue;
        }

        return parent::getPostValue();
    }


    public function createDBRecordField($database, $tableName, $insert)
    {
        $result = parent::createDBRecordField($database, $tableName, $insert);

        if ($result  &&  !self::lzeEditovatProSC($this ->jenZahranicni)) {
            // pole se muze editovat jen pro zahranicni/tuzemske SC => v pripade, ze SC neni/je zahranicni, nastavi pole readonly
            if (is_array($result)) {
                foreach ($result as $recordField) {
                    $recordField ->setReadOnly(true);
                }
            } else {
                $result->setReadOnly(true);
            }
        }

        return $result;
    }
}


/**
 * Pole "Stat"
 * v pripade, ze je prednastaven nebo je nastaven typ rozdeleni pro urceni statu, tak je to konstanta,
 * jinak jej vybira uzivatel
 */
class VychoziStatSCFormField extends SluzebniCestaFormField
{
    public function __construct()
    {
        $statAutomaticky = RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT)  ||  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI_PRO_STAT);

        parent::__construct(
            ($statAutomaticky)
                ? self::FIELD_TYPE_HIDDEN
                : self::FIELD_TYPE_SELECT,
            false);

        $this
            ->setReadOnly($statAutomaticky)
            ->setRequired(!$statAutomaticky);

        if (!$statAutomaticky) {
            $this ->setFieldCssClass('form-control ronselect');
        }

        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT)) {
            $this ->setDefaultValue(RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT));
        }
    }
}


/**
 * Pole pro nastaveni zpusobu prenosu do dochazky. Zapisuje datova pole PrenasetDoDochazky a TypPrenosuDoDochazky
 */
class PrenasetDoDochazkyFormField extends SluzebniCestaFormField
{
    // nastaveni 0=>neprenaset, 1=>podle volby uzivatele, 2=>po smenach, 3=>podle rozpisu
    // pole TypPrenosuDoDochazky 0=>po smenach, 1=>podle rozpisu
    public function __construct()
    {
        // uzivatel muze editovat jen pokud je v nastaveni vychozich hodnot zvoleno "podle volby uzivatele"
        parent::__construct(
            (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) === 1)
                ? FormField::FIELD_TYPE_COMBO
                : FormField::FIELD_TYPE_CONSTANT
        );

        $this
            ->setFieldName('TypPrenosuDoDochazky') ->setCaption('Prenos_do_dochazky')
//            ->setCode('field-typprenosudodochazky')
//            ->setRowId('hlavickasc_prenosdodochazky')
//            ->setVisible(RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1) === 1) // prenaseni do dochazky - nastaveno "podle volby uzivatele"
            ->setOptions([-1 => \Ron\ronText('Neprenaset_do_dochazky'), 0 => \Ron\ronText('Prenaset_po_smenach'), 1 => \Ron\ronText('Prenaset_podle_rozpisu')])
            ->setDataType('int');
        ;

        switch (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_PRENASET_DO_DOCHAZKY, 1)) {
            case 2 : // po smenach
                $this ->setDefaultValue(0);
                break;

            case 3 : // podle rozpisu
                $this ->setDefaultValue(1);
                break;

            default : // neprenaset nebo podle volby uzivatele
                $this ->setDefaultValue(-1); // nastavi na "neprenaset"
        }
    }

    public function getValueFromDBValue($database, $dbValue, $rowData)
    {
        if ($rowData  &&  !$rowData['field-prenasetdodochazky']) {
            // pole "PrenasetDoDochazky" ma hodnotu 0
            return -1; // neprenaset
        }
        return parent::getValueFromDBValue($database, $dbValue, $rowData);
    }


    public function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        // pokud je vybrano "Neprenaset", tak pole PrenasetDoDochazky nastavi na 0,
        // jinak na 1 a podle hodnoty nastavi pole TypPrenosuDoDochazky
        $fPrenaset = new DBRecordField('PrenasetDoDochazky', 'int', $this ->readOnly);
        $fTypPrenosu = new DBRecordField($this ->sqlFieldName, 'int', $this ->readOnly);

        $value = ($this ->fieldType === FormField::FIELD_TYPE_CONSTANT)
            ? (int)$this ->defaultValue
            : (int)parent::getPostValue($utf8);

        $fPrenaset ->setValue($value === -1 ? 0 : 1);
        $fTypPrenosu ->setValue($value === 1 ? 1 : 0);

        return [$fPrenaset, $fTypPrenosu];
    }
}


class RozdeleniSCFormField extends RozdeleniFormField
{
    /** @var int    vse/jen zahranicni/jen tuzemske - viz konstanty SluzebniCestaFormField::...  */
    public $jenZahranicni;

    /** @var bool   Urcuje, jestli se pri ukladani dat maji vzdy vytvaret pole DBRecordu */
    public $alwaysCreateFields;

    /** @var bool   Urcuje, jestli se jedna o editaci pole "rozdeleni2" */
    public $poleRozdeleni2;

    /**
     * @param bool $poleRozdeleni2  Ucruje, jestli se jedna o datove pole Rozdeleni, nebo Rozdeleni2
     * @param int $zahranicni
     */
    public function __construct($poleRozdeleni2, $zahranicni = SluzebniCestaFormField::ZAHRANICNI_I_TUZEMSKE)
    {
        parent::__construct(
            $poleRozdeleni2
                ? ($zahranicni == SluzebniCestaFormField::JEN_ZAHRANICNI
                    ? RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2Z)
                    : RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI2))
                : RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI),
            $poleRozdeleni2
                ? RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI2_POUZE_VLASTNI)
                : RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI_POUZE_VLASTNI));

        $this ->setTypRozdeleniFieldName($poleRozdeleni2 ? 'KodTypuRozdeleni2' : 'KodTypuRozdeleni');

        //$this ->setRowClass('row background-yellow');
        $this ->setJenZahranicni($zahranicni);
        $this ->alwaysCreateFields = !$poleRozdeleni2;
        $this ->poleRozdeleni2 = $poleRozdeleni2;
        $this ->setFieldId('rozdelenisc');
    }

    /**
     * @param int $jenZahranicni
     * @return RozdeleniSCFormField
     */
    public function setJenZahranicni($jenZahranicni)
    {
        $this->jenZahranicni = $jenZahranicni;

        $this ->rowClass = $this ->rowClass
            ? StringUtils::removeSubstring(' ', $this ->rowClass, ['jen-tuzemska-sc', 'jen-zahranicni-sc'])
            : $this ->rowClass;

        if ($jenZahranicni) {
            $this ->rowClass =
                ($this ->rowClass ?: 'row') .
                (($jenZahranicni === SluzebniCestaFormField::JEN_TUZEMSKE)
                    ? ' jen-tuzemska-sc'
                    : ' jen-zahranicni-sc');
        }

        return $this;
    }

    protected function loadConfigurationItem($key, $value)
    {
        if ($key === 'jenZahranicni') {
            $this ->setJenZahranicni((int)$value);
        } else {
            parent::loadConfigurationItem($key, $value);
        }
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (isset($rowData[0])  &&  $this ->poleRozdeleni2  &&  $rowData[0] != $this ->kodTypuRozdeleni) {
            $value = null;
        }

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }


    public function getPostValue($utf8 = false)
    {
        // pokud je pole jen pro zahranicni/tuzemske SC a nelze editovat pro tuto SC - tak pri insert pouzije vychozi hodnotu
        // (pri edit se dodatecne nastavi na readonly)
        return SluzebniCestaFormField::lzeEditovatProSC($this ->jenZahranicni)
            ? parent::getPostValue()
            : $this ->defaultValue;
    }


    public function createDBRecordField($database, $tableName, $insert)
    {
        $lzeEditovatProSC = SluzebniCestaFormField::lzeEditovatProSC($this ->jenZahranicni);
        if (!$lzeEditovatProSC  &&  !$this ->alwaysCreateFields) {
            return null;
        }

        $result = parent::createDBRecordField($database, $tableName, $insert);

        if ($result  &&  !$lzeEditovatProSC) {
            // pole se muze editovat jen pro zahranicni/tuzemske SC => v pripade, ze SC neni/je zahranicni, nastavi pole readonly
            if (is_array($result)) {
                foreach ($result as $recordField) {
                    $recordField ->setReadOnly(true);
                }
            } else {
                $result->setReadOnly(true);
            }
        }

        return $result;
    }
}


class UrceneStravneFormField extends FormField
{
    public function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_TEXT);
    }

    protected function getFieldText($database, $value, $canEdit, $insert, $rowData)
    {
        if (!isset($value)) {
            return '';
        }

        return
            parent::getFieldText($database, $value, $canEdit, $insert, $rowData) .
            ' ' .
            ($rowData ? $rowData['field-urcenedennistravnemena'] : '');
    }
}


class PredpokladanyVydajCastkaFormField extends SluzebniCestaFormField
{
    /** @var PredpokladanyVydajSC */
    private $predpokladanyVydajSC;

    public function __construct($predpokladanyVydajSC)
    {
        parent::__construct(self::FIELD_TYPE_FLOAT, false);
        $this ->predpokladanyVydajSC = $predpokladanyVydajSC;
    }


    public function loadEditValue($database, $usePostValues, $rowData, $canEditRecord)
    {
        if ($usePostValues && $canEditRecord && !$this->readOnly) {
            parent::loadEditValue($database, $usePostValues, $rowData, $canEditRecord);
        } else {
            $this ->value =  self::getValueFromDBValue($database, $this ->predpokladanyVydajSC ->castka, $rowData);
        }
    }
}
