<?php
/**
 * stanislav.elbl, 17.03.2016
 */

namespace RonPortal\SluzebniCesty;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use Ron\DB\Parameters;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\System\AgentManager;
use RonPortal\User;


/**
 * Class SluzebniCestaRecord
 * @package Ron\RonPortal
 */
class SluzebniCestaRecord extends RonPortalDBRecord
{
    /** @var int */
    public $id;

    /** @var  int   id PP pro kontrolu prav */
    public $idPP;

    /** @var DBRecordField */
    public $fDatumCasOd;
    /** @var DBRecordField */
    public $fDatumCasDo;
    /** @var DBRecordField */
    public $fRC;

    /** @var DBRecordField */
    public $fZahranicni;

    /** @var DBRecordField */
    public $fSchvaleniZ;
    /** @var DBRecordField */
    public $fSchvaleniN;
    /** @var DBRecordField */
    public $fSchvaleniN2;
    /** @var DBRecordField */
    public $fSchvaleniN3;
    /** @var DBRecordField */
    public $fSchvaleniN4;
    /** @var DBRecordField */
    public $fSchvaleni1;
    /** @var DBRecordField */
    public $fSchvaleni2;
    /** @var DBRecordField */
    public $fSchvaleni3;
    /** @var DBRecordField */
    public $fSchvaleni4;

    /** @var DBRecordField  */
    public $fHistorieField;


    function __construct($database, $id, $logDescription = 'SluzebniCesty')
    {
        parent::__construct($database, 'SluzebniCesta', $logDescription);
        $this ->id = (int)$id;

        //$this ->where = 'SluzebniCesta.Id = ' . (int)$id;
        $params = $database ->createParameters();
        $where = 'SluzebniCesta.Id = ' . $params ->addParameter((int)$id);
        $this ->setWhere($where, $params);

        // je nutne pridat jmeno tabulky SluzebniCesta., protoze se pripojuje jeste PracovniPomer
        $this ->addField('SluzebniCesta.Id', 'sequence', true); // readonly

        $this ->fRC = $this ->addField('SluzebniCesta.RC', '', true); // readonly
        $this ->addField('SluzebniCesta.OsobniCislo', '', true);
        $this ->addField('SluzebniCesta.DatumNastupu', 'date', true);

        $this ->fDatumCasOd = $this ->addField('SluzebniCesta.DatumCasOd', 'date', true);
        $this ->fDatumCasDo = $this ->addField('SluzebniCesta.DatumCasDo', 'date', true);

        $this ->fZahranicni = $this ->addField('SluzebniCesta.Zahranicni', '', true);

        $this ->addField('SluzebniCesta.MistoZacatku', '', true);
        $this ->addField('SluzebniCesta.PopisMistaZacatku', '', true);

        $this ->addField('SluzebniCesta.Cil', '', true);
        $this ->addField('SluzebniCesta.PopisCile', '', true);
        $this ->addField('SluzebniCesta.CilovyStat', '', true);

        $this ->addField('SluzebniCesta.Popis', '', true);

        $this ->fSchvaleniZ = $this ->addField('SchvalenoZamestnancem', 'int', true);
        $this ->fSchvaleniN = $this ->addField('SluzebniCesta.SchvalenoNadrizenym', 'int', true);
        $this ->fSchvaleniN2 = $this ->addField('SluzebniCesta.SchvalenoNadrizenym2', 'int', true);
        $this ->fSchvaleniN3 = $this ->addField('SluzebniCesta.SchvalenoNadrizenym3', 'int', true);
        $this ->fSchvaleniN4 = $this ->addField('SluzebniCesta.SchvalenoNadrizenym4', 'int', true);
        $this ->fSchvaleni1 = $this ->addField('SluzebniCesta.Schvaleni1', 'datetime', true);
        $this ->fSchvaleni2 = $this ->addField('SluzebniCesta.Schvaleni2', 'datetime', true);
        $this ->fSchvaleni3 = $this ->addField('SluzebniCesta.Schvaleni3', 'datetime', true);
        $this ->fSchvaleni4 = $this ->addField('SluzebniCesta.Schvaleni4', 'datetime', true);

        $this ->addField('SluzebniCesta.KodTypuRozdeleni', 'int', true);
        $this ->addField('SluzebniCesta.Rozdeleni', '', true);

        $this ->addField('SluzebniCesta.KodTypuRozdeleni2', 'int', true);
        $this ->addField('SluzebniCesta.Rozdeleni2', '', true);

        $this ->addField('SluzebniCesta.Poznamka', 'memo', true);

        $this ->fHistorieField = $this ->addField('SluzebniCesta.HistorieSchvalovani', 'memo', false); // readonly = false
        $this ->fHistorieField ->updateLog = true;
    }


    function loadRecordValues($row)
    {
        if (!$this ->id) {
            throw new \Exception('Chyba - neni zadano ID sluzebni cesty!');
        }

        parent::loadRecordValues($row);

        $this ->idPP = $row[count($this ->fields)];

        return true;
    }


    /**
     * @param string $filtr
     * @param Parameters $targetParams
     * @param Parameters $filtrParams
     * @return string
     */
    function getSelectSQL($filtr = '', $targetParams = null, $filtrParams = null)
    {
        $sql =
            'SELECT ' . $this ->getSelectPartSQL() . ', PracovniPomer.Id ' . // PP.Id posledni!
            '  FROM SluzebniCesta, PracovniPomer ' .
            ' WHERE SluzebniCesta.RC = PracovniPomer.RC ' .
            '   AND SluzebniCesta.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND SluzebniCesta.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND (' . $this ->where . ')';

        $targetParams ->appendParameters($this ->whereParams);

        if ($filtr) {
            $sql .= ' AND (' . $filtr . ')';
            if ($filtrParams) {
                $targetParams ->appendParameters($filtrParams);
            }
        }

        return $sql;
    }


    /**
     * Zkontroluje, jestli ma uzivatel zadane pravo na sluzebni cestu
     * @param string $pravaStr  Kod prava, ktere se ma zkontrolovat
     * @param bool $zruseni Urcuje, jestli se jedna o zruseni schvaleni - v tom pripade kontroluje i priznak "zakazat zrusit schvaleni"
     * @throws NeniPravoException
     */
    function zkontrolujPrava($pravaStr, $zruseni = false)
    {
        $prava = Prava::vratPrava($pravaStr, $this ->fZahranicni ->value ? $pravaStr . '_Z' : null); // u zahranicnich je mozne nadefinovat uzivatelske pravo se sufixem _Z
        if ($zruseni  &&  $prava ->priznak) {
            // je zaskrtnuto "zakazat zrusit schvaleni"
            throw new NeniPravoException();
        };

        Prava::nastavObdobiProPrava($this ->fDatumCasOd ->value, $this ->fDatumCasDo ->value);

        if (!$prava ->lzeZobrazitPP($this ->idPP)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param string $eventName
     * @param string $xmlContext
     * @return bool
     */
    function xmlEventsSC($eventName, $xmlContext)
    {
        return $this ->executeXmlEvents($eventName, $xmlContext, ['sluzebnicesta' => $this ->id]);
    }


    /**
     * @param int $uroven
     * @return bool
     * @throws \Exception
     */
    function schvalSlCestu($uroven)
    {
        $pomf = $this ->addField('Schvalil' . $uroven, '', false, 0, RonPortalUtils::getLogUserName());
        $pomf ->updateLog = true;

        for ($i = $uroven; $i <= 4; $i++) {
            $this->addField('DatumVraceni' . $i, 'timestamp', false, 0, null); // vynuluje datumy vraceni
        }
        $this ->addField('PoznamkaKVraceni', '', false, 0, null); // a poznamku

        if ($uroven == 1  &&  RonPortalSettings::getSpousteniADS()) {
            // pred schvalenim prvni urovni provede prepocet
            AgentManager::executeADS('ADS_Agent_BusinessTrips.exe', '0 -id:' . (int)$this ->id . ' -alwaysLog', true);
        }

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana sluzebni cesta nebyla nalezena!');
        }

        // pole s predchozi urovni - pro kontrolu v pripade zapnute kontroly posloupnosti schvalovani
        $predchoziUroven = null;

        // schvaleni nastavuje az po LoadRecord, aby se zbytecne nevolal update, pokud uz bylo schvaleno
        // (neni to totiz priznak, ale datum)
        switch ((int)$uroven) {
            case 1 :
                if ( !$this ->fSchvaleni1 ->value) {
                    $this ->fSchvaleni1 ->setNewValue(time());
                }
                break;

            case 2 :
                $predchoziUroven = $this ->fSchvaleni1;
                if ( !$this ->fSchvaleni2 ->value) {
                    $this ->fSchvaleni2 ->setNewValue(time());
                }
                break;

            case 3 :
                $predchoziUroven = $this ->fSchvaleni2;
                if ( !$this ->fSchvaleni3 ->value) {
                    $this ->fSchvaleni3 ->setNewValue(time());
                }
                break;

            case 4 :
                $predchoziUroven = $this ->fSchvaleni3;
                if ( !$this ->fSchvaleni4 ->value) {
                    $this ->fSchvaleni4 ->setNewValue(time());
                }
                break;
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        if ($predchoziUroven  &&  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_POSLOUPNOST_SCHVALOVANI)) {
            // je zapnuto "dodrzovat posloupnost schvalovani" -> zkontroluje, jestli je schvalena predchozi uroven
            if (!$predchoziUroven ->value) {
                throw new \Exception(sprintf(TextManager::_('Sluzebni_cesta_neni_schvalena_predchozi_urovni'), $this ->id, (int)$uroven));
            }
        }

        $this ->zkontrolujPrava("slcesty_schval$uroven");

        $this ->fHistorieField ->value =
            "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' .
            sprintf(\Ron\ronText('Schvaleni_sc_log'), '' . $uroven) .
            "\n" .
            $this ->fHistorieField ->originalValue; // puvodni obsah nakonec

        $this ->logText = TextManager::_('LOG_schvaleni_SC');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'schvaleni' . $uroven . '_before')) {
            return false;
        }

        $this ->updateRecord();

        return $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'schvaleni' . $uroven . '_after');
    }


    /**
     * @param int $uroven
     * @return bool
     * @throws \Exception
     */
    function zrusSchvaleniSlCesty($uroven)
    {
        switch ($uroven) {
            case '1' :
                $this ->fSchvaleni1 ->setNewValue(null);
                break;

            case '2' :
                $this ->fSchvaleni2 ->setNewValue(null);
                break;

            case '3' :
                $this ->fSchvaleni3 ->setNewValue(null);
                break;

            case '4' :
                $this ->fSchvaleni4 ->setNewValue(null);
                break;
        }

        $pomf = $this ->addField('Schvalil' . $uroven, '', false, 0, null);
        $pomf ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana sluzebni cesta nebyla nalezena!');
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        $this ->zkontrolujPrava("slcesty_schval$uroven", true);

        if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_EDITACE_V_UZAVRENEM)  &&
            Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $this ->fDatumCasOd ->value, $this ->fDatumCasDo ->value)
        ) {
            throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        if ( ( $uroven <= 3  && $this ->fSchvaleni4 ->value )
            || ( $uroven <= 2 && $this ->fSchvaleni3 ->value )
            || ( $uroven == 1 && $this ->fSchvaleni2 ->value )
        ) {
            throw new \Exception(TextManager::_('Sluzebni_cesta_je_jiz_schvalena_vyssi_urovni'));
        }

        $this ->fHistorieField ->value =
            "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' .
            sprintf(\Ron\ronText('Odschvaleni_sc_log'), '' . $uroven) .
            "\n" .
            $this ->fHistorieField ->originalValue; // puvodni obsah nakonec

        $this ->logText = TextManager::_('LOG_zruseno_schvaleni_SC');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'zrusitschvaleni' . $uroven . '_before')) {
            return false;
        }

        $this ->updateRecord();

        return $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'zrusitschvaleni' . $uroven . '_after');
    }


    /**
     * Zkontroluje, jestli uzivatel muze vratit schvaleni zadane urovne
     * @param int $uroven
     * @return bool
     * @throws \Exception
     */
    function lzeVratitSluzebniCesta($uroven)
    {
        // vetsina chyb by nemela vubec nastat
        // - nektere pouze v pripade, ze by 2 uzivatele soubezne pracovali se stejnym zaznamem

        //pokud je zapnuta uroven 4, kontroluje se i ta
        if (RonPortalSettings::getIntValue('slcesty_filtr_uroven4')) {
            if ($this ->fSchvaleni4 ->value) {
                // jiz je schvalena uroven 4 - nelze vratit
                throw new \Exception('Chyba - nelze vratit - schvalovani sluzebni cesty jiz bylo dokonceno!');
            }
        }
        else {
            if ($this->fSchvaleni3->value) {
                // jiz je schvalena uroven 3 - nelze vratit
                throw new \Exception('Chyba - nelze vratit - schvalovani sluzebni cesty jiz bylo dokonceno!');
            }
        }

        switch ($uroven) {
            case 3 :
                if (!$this ->fSchvaleni3 ->value) {
                    throw new \Exception('Chyba - nelze vratit - uroven 3 neni schvalena!');
                }
                break;

            case 2 :
                //zde se kontroluje jen pokud se pouzivaji 4 urovne
                if ($this ->fSchvaleni3 ->value) {
                    throw new \Exception('Chyba - nelze vratit - uroven 3 je jiz schvalena!');
                }

                if (!$this ->fSchvaleni2 ->value) {
                    throw new \Exception('Chyba - nelze vratit - uroven 2 neni schvalena!');
                }
                break;

            case 1 :
                if ($this ->fSchvaleni2 ->value) {
                    throw new \Exception('Chyba - nelze vratit - uroven 2 je jiz schvalena!');
                }

                if (!$this ->fSchvaleni1 ->value) {
                    throw new \Exception('Chyba - nelze vratit - uroven 1 neni schvalena!');
                }
                break;

            default:
                throw new \Exception('Chyba - spatne zadana uroven schvaleni pri vraceni sluzebni cesty! (' . $uroven . ')');
        }


        $this ->zkontrolujPrava('slcesty_schval' . ($uroven + 1));

        return true;
    }


    /**
     * @param int    $uroven
     * @param string $poznamka
     * @return bool
     * @throws \Exception
     */
    function vratSchvaleni($uroven, $poznamka)
    {
        $this ->addField('DatumVraceni' . $uroven, 'timestamp', false, 0, time());
        $this ->addField('PoznamkaKVraceni', '', false, 100, $poznamka);

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba pri vraceni schvaleni - zadana sluzebni cesta nebyla nalezena!');
        }

        if (!$this ->lzeVratitSluzebniCesta($uroven)) {
            return false;
        }

        if ( !RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_EDITACE_V_UZAVRENEM)
            && Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $this ->fDatumCasOd ->value, $this ->fDatumCasDo ->value) ) {
            throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        switch ($uroven) {
            case 1 :
                $this ->fSchvaleni1 ->setNewValue(null);
                break;

            case 2 :
                $this ->fSchvaleni2 ->setNewValue(null);
                break;

            case 3 :
                $this ->fSchvaleni3 ->setNewValue(null);
                break;

            default :
                throw new \Exception('Chyba pri vraceni schvaleni SC - spatne zadana uroven schvaleni!');
        }

        $this ->fHistorieField ->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . sprintf(\Ron\ronText('Vraceni_sc_log'), '' . $uroven) .
            "\n> " . $poznamka .
            "\n" . $this ->fHistorieField ->originalValue // puvodni obsah nakonec
        );

        $this ->logText = TextManager::_('LOG_vraceno_schvaleni_SC');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'vratitschvaleni' . $uroven . '_before')) {
            return false;
        }
        if (!$this ->updateRecord()) {
            return false;
        }

        return $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'vratitschvaleni' . $uroven . '_after');
    }



    /**
     * Schvaleni/zruseni schvaleni SC zamestnancem
     * @param int $hodnota    1=schvalit, 2=zrusit schvaleni
     * @throws \Exception
     */
    function schvalSlCestu_Z($hodnota)
    {
        $this ->fSchvaleniZ ->setNewValue($hodnota);

        $f = $this ->addTimestampField('SZ_DatumCas', false, time());
        $f ->updateLog = true;
        $f = $this ->addStringField('SZ_Uzivatel', false, 25, RonPortalUtils::getLogUserName());
        $f ->updateLog = true;

        $logstr = \Ron\ronText($hodnota ? 'Schvaleni_sc_zam_log' : 'Zruseno_schvaleni_sc_zam_log');

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana sluzebni cesta nebyla nalezena!');
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return;
        }

        if (!SluzebniCestyUtils::lzeSchvalitZamestnancem($this ->fRC ->value, $this ->idPP, $this ->fZahranicni ->value, $this ->fDatumCasOd ->value, $this ->fDatumCasDo ->value)) {
            throw new NeniPravoException();
        }

        $this ->fHistorieField ->value =
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " .
            $logstr .
            "\n" .
            $this ->fHistorieField ->originalValue; // puvodni obsah nakonec

        $this ->logText = TextManager::_($hodnota ? 'LOG_schvaleni_SC' : 'LOG_zruseno_schvaleni_SC');

        $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI,
            $hodnota ? 'schvaleni_zamestnanec_before' : 'zrusitschvaleni_zamestnanec_before');
        $this ->updateRecord();
        $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI,
            $hodnota ? 'schvaleni_zamestnanec_after' : 'zrusitschvaleni_zamestnanec_after');
    }


    /**
     * @param int $hodnota  1=schvalit, 2=zamitnout
     * @param int $level    1 = schvaleni nadrizeneho, 2 = nadrizeneho 2
     * @return bool
     * @throws \Exception
     */
    function schvalSlCestu_N($hodnota, $level = 1)
    {
        $levelStr = ((int)$level == 1)
            ? ''
            : (int)$level;

        switch ($level) {
            case 2 :
                $this->fSchvaleniN2->setNewValue($hodnota);
                break;

            case 3 :
                $this->fSchvaleniN3->setNewValue($hodnota);
                break;

            case 4 :
                $this->fSchvaleniN4->setNewValue($hodnota);
                break;

            default :
                $this->fSchvaleniN->setNewValue($hodnota);
        }

        $f = $this ->addTimestampField("SN{$levelStr}_DatumCas", false, time());
        $f ->updateLog = true;
        $f = $this ->addStringField("SN{$levelStr}_Uzivatel", false, 25, RonPortalUtils::getLogUserName());
        $f ->updateLog = true;

        switch ($hodnota) {
            case 0 :
                $logstr = \Ron\ronText("Zruseno_schvaleni_sc_nad{$levelStr}_log");
                $scriptContext = "zrusitschvaleni_nadrizeny{$levelStr}_";
                break;

            case 1 :
                $logstr = \Ron\ronText("Schvaleni_sc_nad{$levelStr}_log");
                $scriptContext = "schvaleni_nadrizeny{$levelStr}_";
                break;

            case 2 :
                $logstr = \Ron\ronText("Zamitnuti_sc_nad{$levelStr}_log");
                $scriptContext = "zamitnuti_nadrizeny{$levelStr}_";
                break;

            default :
                $scriptContext = null;
                $logstr = '';
        }


        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana sluzebni cesta nebyla nalezena!');
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        $this ->zkontrolujPrava("slcesty_nadriz{$levelStr}");

        $this ->fHistorieField ->value =
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . $logstr .
            "\n" .
            $this ->fHistorieField ->originalValue; // puvodni obsah nakonec

        $this ->logText = TextManager::_('LOG_schvaleni_SC');

        if ($scriptContext  &&  !$this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, $scriptContext . 'before')) {
            return false;
        }

        $this ->updateRecord();

        if (!$scriptContext) {
            return true;
        }

        return $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, $scriptContext . 'after');
    }


    /**
     * @param int $zauctovano
     * @throws \Exception
     */
    function nastavZauctovaniSlCesty($zauctovano)
    {
        $this ->addField('Zauctovano', 'int', false, 0, (int)$zauctovano);

        $pomfield = $this ->addField('Zauctovano_Uzivatel', '', false, 0, RonPortalUtils::getLogUserName());
        $pomfield ->updateLog = true;

        $pomfield = $this ->addField('Zauctovano_DatumCas', 'datetime', false, 0, time());
        $pomfield ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana sluzebni cesta nebyla nalezena!');
        }

        $this ->zkontrolujPrava('slcesty_zauct');

        // zauctovani
        $logstr = \Ron\ronText($zauctovano ? 'Zauctovani_sc_log' : 'Zruseno_zauctovani_sc_log');

        $this ->fHistorieField ->value =
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " .
            $logstr .
            "\n" .
            $this ->fHistorieField ->originalValue;  // puvodni obsah nakonec

        $this ->logText = TextManager::_('LOG_nastaveno_zauctovani_SC');

        if (!$this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'zauctovani_before')) {
            throw new \Exception('Pri nastaveni zauctovani doslo k chybe!');
        }

        if (!$this ->updateRecord()) {
            throw new \Exception('Chyba - pri nastaveni zauctovani sluzebni cesty doslo k chybe!');
        }

        $this ->xmlEventsSC(RonPortalXMLEvents::SC_SCHVALENI, 'zauctovani_after');
    }


//    /**
//     * @param Database $database
//     * @param $logDescription
//     * @return bool
//     */
//    public static function hromadneSchvaleniSC($database, $logDescription)
//    {
//        Csrf::validateCSRFPostParam();
//
//        // uzivatel odeslal hromadne schvaleni SC
//        $pocet = (int)$_POST['pocet'];
//
//        $result = true;
//
//        for ($i = 0;  $i < $pocet;  $i++) {
//            $id = $_POST["idSC_$i"];
//            $rec = new SluzebniCestaRecord($database, $id, $logDescription);
//
//            try {
//                if (isset($_POST["NastavitSchvaleni_{$id}_N"]) && $_POST["NastavitSchvaleni_{$id}_N"]) {
//                    if (!$rec->schvalSlCestu_N($_POST["Schvaleni_{$id}_N"] ? 1 : 0)) {
//                        $result = false;
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_N2"]) && $_POST["NastavitSchvaleni_{$id}_N2"]) {
//                    if (!$rec->schvalSlCestu_N($_POST["Schvaleni_{$id}_N2"] ? 1 : 0, 2)) {
//                        $result = false;
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_N3"]) && $_POST["NastavitSchvaleni_{$id}_N3"]) {
//                    if (!$rec->schvalSlCestu_N($_POST["Schvaleni_{$id}_N3"] ? 1 : 0, 3)) {
//                        $result = false;
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_N4"]) && $_POST["NastavitSchvaleni_{$id}_N4"]) {
//                    if (!$rec->schvalSlCestu_N($_POST["Schvaleni_{$id}_N4"] ? 1 : 0, 3)) {
//                        $result = false;
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_1"]) && $_POST["NastavitSchvaleni_{$id}_1"]) {
//                    if ($_POST["Schvaleni_{$id}_1"]) {
//                        if (!$rec->schvalSlCestu(1)) {
//                            $result = false;
//                        }
//                    } else {
//                        if (!$rec->zrusSchvaleniSlCesty(1)) {
//                            $result = false;
//                        }
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_2"]) && $_POST["NastavitSchvaleni_{$id}_2"]) {
//                    if ($_POST["Schvaleni_{$id}_2"]) {
//                        if (!$rec->schvalSlCestu(2)) {
//                            $result = false;
//                        }
//                    } else {
//                        if (!$rec->zrusSchvaleniSlCesty(2)) {
//                            $result = false;
//                        }
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_3"]) && $_POST["NastavitSchvaleni_{$id}_3"]) {
//                    if ($_POST["Schvaleni_{$id}_3"]) {
//                        if (!$rec->schvalSlCestu(3)) {
//                            $result = false;
//                        }
//                    } else {
//                        if (!$rec->zrusSchvaleniSlCesty(3)) {
//                            $result = false;
//                        }
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_4"]) && $_POST["NastavitSchvaleni_{$id}_4"]) {
//                    if ($_POST["Schvaleni_{$id}_4"]) {
//                        if (!$rec->schvalSlCestu(4)) {
//                            $result = false;
//                        }
//                    } else {
//                        if (!$rec->zrusSchvaleniSlCesty(4)) {
//                            $result = false;
//                        }
//                    }
//                }
//
//                if (isset($_POST["NastavitSchvaleni_{$id}_Z"]) && $_POST["NastavitSchvaleni_{$id}_Z"]) {
//                    $rec ->nastavZauctovaniSlCesty($_POST["Schvaleni_{$id}_Z"] ? 1 : 0);
//                }
//            } catch (\Exception $e) {
//                $result = false;
//                MessageManager::addErrorMessage($e ->getMessage());
//            }
//        }
//
//        if (!$result) {
//            MessageManager::addErrorMessage('Pri hromadnem schvalovani sluzebnich cest doslo k chybe!');
//        }
//
//        return $result;
//    }
}
