<?php
/**
 * stanislav.elbl, 30.11.2016
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\MessageManagerDialogRenderer;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\DialogPartialView;
use Ron\MVC\ExceptionViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\PartialView;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RozdeleniOsobyShiftForm;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalApp;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\RozdeleniOsob;
use RonPortal\User;


/**
 * Obsluha javascriptovych pozadavku ve sluzebnich cestach
 */
class SluzebniCestyJsController extends JsonController
{

    /** @var  Database */
    protected $database;


    /**
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     */
    function vyberSpolucestujicich($output)
    {
        Prava::nastavObdobiProPrava(time(), time());
        //Prava::nactiPravaUzivatele($this->database, null, 'slcesty_spolu');

        $params = $this ->database ->createParameters();
        $filter = '(PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ')';
        $seznamPP = PracovniPomer::nactiSeznamPP($this->database,
            new SqlStatement($filter, $params),
            Prava::vratPrava('slcesty_spolu'),
            time(), time());
        $seznamJs = [];
        foreach ($seznamPP as $pp) {
            $seznamJs[] = $pp->getJsonData();
        }

        $output->addData('pp', $seznamJs);
    }


    /**
     * Volano pri vyberu osoby ve formulari sluzebni cesty - nacte napr. prirazena vozidla
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     */
    function nactiDataProOsobu($output)
    {
        //Prava::nactiPravaUzivatele($this ->database); - vola se z App
        Prava::nastavObdobiProPrava(time(), time());

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);

        if (!Prava::lzeZobrazitPP('sluzebnicesty', $pp ->idPP)) {
            throw new NeniPravoException();
        }

        // nacte prirazena vozidla
        $options = SluzebniCestaFormular::nactiVozidlaOsoby($this ->database, $pp ->idPP);

        $jsonData = [];
        foreach ($options as $option) {
            $jsonData[] = $option ->getJson();
        }

        $output ->addData('vozidla', $jsonData);

        // nacte stat
        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT)  ||  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI_PRO_STAT)) {
            $stat = SluzebniCestyUtils::getVychoziStatOsoby($this ->database, $pp ->idPP, time());
            $output ->addData('stat', $stat);
        }


        // nacte vychozi hodnotu pro rozdeleni
        if ((RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI))  &&  RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI_VYCHOZI)) {

            $r = RozdeleniOsob::nactiRozdeleniOsoby($this ->database,
                $pp, RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI),
                time(), time(),
                (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ROZDELENI_VYCHOZI) == 2) ? 'Hlavni = 1' : null
            );

            if ($r) {
                $output ->addData('rozdeleni', StringUtils::jsonText($r[0] ->kod));
            }
        }

    }

    /**
     * Pridani spolucestujiciho ke sluzebni ceste action=pridejspolucestujiciho&idsc=<id_sluzebni_cesty>&idpp=<id_PP>
     * @param JsonOutputObject $output
     * @throws NeniPravoException
     * @throws \Exception
     */
    function pridejSpolucestujiciho($output)
    {
        $idSC = (int)$_POST['idsc'];
        Csrf::validateCSRFPostParam("sc_spolucestujici-{$idSC}");

        Prava::nastavObdobiProPrava(time(), time());
        //Prava::nactiPravaUzivatele($this->database, null, 'slcesty_spolu');

        $prava = Prava::vratPrava('slcesty_spolu');

        // zkontrolovat pravo na vybrany PP
        $idPP = (int)$_POST['idpp'];
        if (!$idPP) {
            throw new \Exception('Neni vybrana osoba!');
        }

        if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
            throw new NeniPravoException();
        }

        // zkontrolovat pravo na sluzebni cestu, schvaleni, zauctovani
        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
        $sc->lzeEditovat($this->database, true);

        $pp = PracovniPomer::nactiPP($this->database, $idPP);

        $rec = new RonPortalDBRecord($this->database, 'Spolucestujici', $this);
        $rec->addField('SluzebniCesta_ID', 'int', true, 0, $idSC);
        $rec->addField('RC', '', true, 15, $pp->rc);
        $rec->addField('OsobniCislo', '', true, 15, $pp->osobniCislo);
        $rec->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $rec->addField('', '', true, 0, '   ' . $pp->jmenoPrijmeniKratke);

        $params = $this ->database ->createParameters();
        $rec->setWhere(
            'SluzebniCesta_ID = ' . $params ->addParameter($idSC) .
            ' AND RC = ' . $params->addParameter($pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this->database->timestampToParam($pp->datumNastupu)),
            $params);

        if (!$rec->loadRecord()) {
            $rec->insertRecord();
        }
    }


    /**
     * Zkontroluje, jestli zadany PP (spolucestujici) nema sluzebni cestu prekryvajici se se zadanou SC
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function overSpolucestujiciho($output)
    {
        Prava::nastavObdobiProPrava(time(), time());
        //Prava::nactiPravaUzivatele($this->database, null, 'slcesty_spolu');

        $prava = Prava::vratPrava('slcesty_spolu');

        $idPP = (int)$_GET['idpp'];
        if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
            throw new NeniPravoException();
        }

        $idSC = (int)$_GET['idsc'];

        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);

        $pp = PracovniPomer::nactiPP($this->database, $idPP);

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumCasOd, DatumCasDo FROM SluzebniCesta' .
            ' WHERE RC = ' . $params ->addParameter($pp ->rc) . // kontroluje jen RC - predpokladam, ze jedna osoba nemuze na ruzne SC
            ' AND DatumCasOd < ' . $params ->addParameter($this ->database ->timestampToParam($sc ->datumCasDo)) .
            ' AND DatumCasDo > ' . $params ->addParameter($this ->database ->timestampToParam($sc ->datumCasOd)) .
            ' ORDER BY DatumCasOd';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if ($row = $cursor ->fetch()) {
            $textSc =
                StringUtils::format('datetime', $cursor ->stringToTimestamp($row[0])) .
                ' - ' .
                StringUtils::format('datetime', $cursor ->stringToTimestamp($row[1]));

            $output ->addText('infoText', sprintf(\Ron\ronText('Spolucestujici_jiz_ma_cestu'), $textSc) );
            $output ->setResult('warning');
        }
        $cursor ->close();
    }


    /**
     * Vraci max. stravne pro zadany stat
     * @param JsonOutputObject $output
     */
    function maximalniDenniStravneProStat($output)
    {
        // TODO: nefunguje, pokud se urcuje vychozi stat podle rozdeleni
        $cilovyStat = $_GET['cilovystat'];
        $vychoziStat = $_GET['vychozistat'];

        if (RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT)) {
            $vychoziStat = RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT);
        }

        if (!$cilovyStat) {
            $output->addData('text', '');
            return;
        }

        try {
            $datum = DateUtils::dateStringToTimestamp($_GET['datum']);
        } catch (\Exception $e) {
            $output->addError($e->getMessage(), 'DatumCasOd');
            return;
        }

        try {
            $poma = SluzebniCestyUtils::vratMaximalniDenniStravneProStat($this->database, $datum, $cilovyStat, $vychoziStat);
            $output->addText('mena', $poma['mena']);
            $output->addText('text', StringUtils::format('number_2', $poma['sazba']) . ' ' . $poma['mena']);
            $output->addData('sazba', $poma['sazba']);
        } catch (DBException $e) {
            throw $e;
        } catch (\Exception $e) {
            $output ->addData('errorMessage', StringUtils::jsonText($e ->getMessage()));
        }
    }


    /**
     * @param JsonOutputObject $output
     */
    function statProMistoSC($output)
    {
        $misto = StringUtils::fromUTF8($_GET['kod']);
        $params = $this ->database ->createParameters();
        $sql = 'SELECT Stat FROM MistoSluzebniCesty WHERE Kod = ' . $params ->addParameter($misto);
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            $output ->addData('text', StringUtils::jsonText($row[0]));
        }
    }


    /**
     * @param JsonOutputObject $output
     */
    function kapesneProStat($output)
    {
        $stat = StringUtils::fromUTF8($_GET['stat']);
        $params = $this ->database ->createParameters();
        $sql = "SELECT ImplicitniSazbaKapesneho FROM Stat WHERE Kod = " . $params ->addParameter($stat);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) {
            $output ->addData('text', StringUtils::format('float', $row[0]));
        }
    }


    private function kontrolaRadkuUctovani($output)
    {
        $idSluzebniCesty = (int)$_POST['idslcesty'];

        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $idSluzebniCesty);

        $prava = Prava::vratPrava('slcesty_k_ucet');
        if (!$prava ->lzeZobrazitPP($cesta ->f_PP ->idPP)) {
            throw new NeniPravoException();
        }

        throw new \Exception('NENI IMPLEMENTOVANO!');
    }


    /**
     * @param JsonOutputObject $output
     */
    function adresaRozdeleni($output)
    {
        $kod = StringUtils::fromUTF8($_GET['kod']);
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Stat, Ulice, Mesto, PSC FROM Rozdeleni' .
            ' WHERE Kod = ' . $params ->addParameter($kod) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenisc'));
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        if ($row) {
            $output ->addData('stat', StringUtils::jsonText($row[0]));
            $output ->addData('ulice', StringUtils::jsonText($row[1]));
            $output ->addData('mesto', StringUtils::jsonText($row[2]));
            $output ->addData('psc', StringUtils::jsonText($row[3]));
        }
    }


    /**
     * Obslouzi vyber rozdeleni osoby (shiftform)
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    public function rozdeleniOsoby(JsonOutputObject $output)
    {
        if ( !RonPortalSettings::getIntValue('rozdelenisc_povolitprirazeni')  ||  !RonPortalSettings::getIntValue('typrozdelenisc')) {
            throw new \Exception('Chyba! Tato funkce neni povolena!');
        }

        $pp = null;
        if (RonPortalSettings::getIntValue('rozdelenisc_povolitprirazeni') === 2) {
            // Vzdy vybira rozdeleni pro vlastni PP - zjisti ID platneho PP
            $pp = PracovniPomer::najdiPPOsoby($this ->database, User::getRC());
        }

        Prava::nastavObdobiProPrava(time(), time());

        $shiftForm = new RozdeleniOsobyShiftForm($this ->database,
            RonPortalSettings::getIntValue('typrozdelenisc'),
            Prava::vratPrava('sluzebnicesty'),
            $pp ? $pp ->idPP : null
        );

        $data = $shiftForm ->handleAction();

        $output ->addData('data', $data);
    }


    function dokumentUpload()
    {
        //nactiPravaUzivatele($this ->database, S_GetUserRC());
        $idSluzebniCesty = (int)$_GET['idslcesty'];

        $cesta = SluzebniCesta::nactiSluzebniCestu($this->database, $idSluzebniCesty);

        Prava::nastavObdobiProPrava($cesta->datumCasDo, $cesta->datumCasDo);

        if (!$cesta->lzeEditovatSluzebniCestu($this ->database, true)) {
            throw new NeniPravoException();
        }

        Csrf::validateCSRFPostParam('dokument_sc_' . $idSluzebniCesty);

        $cisloRadku = isset($_POST['CisloRadku']) ? (int)$_POST['CisloRadku'] : null;

        $id = null;
        if (!is_uploaded_file($_FILES['soubor']['tmp_name'] ?? null)  &&  $_POST['pripojitdokument'] ?? null) {
            $id = (int)$_POST['pripojitdokument'];
        } else {
            DokumentyUtils::handleUploadError('soubor');
            DokumentyUtils::verifyFileType('soubor');
        }

        $rec = new RonPortalDBRecord($this ->database, 'DokumentSluzebniCesty', $this);
        $rec ->setXmlEvents(RonPortalXMLEvents::SC_DOKUMENT_BEFORE_POST, RonPortalXMLEvents::SC_DOKUMENT_AFTER_POST);

        $fId = $rec ->addField('Id', 'int', false, 0, $id);   // Pozor! Pri vkladani noveho souboru jeste neni nastaveno!
        $rec ->addField('SluzebniCesta_ID', 'int', false, 0, (int)$idSluzebniCesty);

        if ($cisloRadku) {
            $rec->addField('CisloRadku', 'int', false, 0, (int)$cisloRadku);
        }

        // vola udalost BEFORE_POST jeste pred zapsanim dokumentu (napr. pro moznost zabraneni zapisu)
        $rec ->executeXmlEvents($rec ->beforePostEvent, 'insert', $rec ->eventsParameters);

        if (!isset($id)) {
            // vlozi do databaze obsah dokumentu
            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', (int)$_POST['typ'], $_POST['popis'], $this, true);
            $fId ->setNewValue($id);
        }

        $rec ->insertRecord();
    }


    function schvaleniSC($output)
    {
        $idSC = (int)$_POST['id'];
        Csrf::validateCSRFPostParam('sc_schvaleni_' . $idSC);

        $rec = new SluzebniCestaRecord($this ->database, $idSC, $this);

        if (isset($_POST["Schvaleni_N"])) {
            $rec->schvalSlCestu_N((int)$_POST["Schvaleni_N"], 1);
        }

        if (isset($_POST["Schvaleni_N2"])) {
            $rec->schvalSlCestu_N((int)$_POST["Schvaleni_N2"], 2);
        }

        if (isset($_POST["Schvaleni_N3"])) {
            $rec->schvalSlCestu_N((int)$_POST["Schvaleni_N3"], 3);
        }

        if (isset($_POST["Schvaleni_N4"])) {
            $rec->schvalSlCestu_N((int)$_POST["Schvaleni_N4"], 4);
        }

        for ($i = 1;  $i <= 4;  $i++) {
            if (isset($_POST["Schvaleni_$i"])) {
                if ($_POST["Schvaleni_$i"]) {
                    $rec->schvalSlCestu($i);
                } else {
                    $rec->zrusSchvaleniSlCesty($i);
                }
            }
        }

        if (isset($_POST["Schvaleni_Z"])) {
            $rec ->nastavZauctovaniSlCesty($_POST["Schvaleni_Z"] ? 1 : 0);
        }
    }



    private function radekSluzebniCestyRecord()
    {
        $f = null;

        try {
            $idSC = (int)$_GET['idsc'];
            $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
            Prava::nastavObdobiProPrava($sc ->datumCasOd);

            if (!$sc->lzeZobrazitSluzebniCestu()) {
                throw new NeniPravoException();
            }

            $post = $this->getRequest()->isPost();
            $insert = ($_GET['record'] ?? '') === '';
            $formAction = $insert ? ($post ? 'insert' : 'new') : ($post ? 'save' : 'edit');
            if (!$post && $insert && ($_GET['copyrecord'] ?? false)) {
                $formAction = 'copy';
            }

            $f = new RadekSluzebniCestyDialog($this ->database,
                $sc,
                $formAction,
                !$sc ->lzeEditovatSluzebniCestu($this ->database, false)
            );
            //$f ->utf8 = true; - presunuto do InnputForm

            if ($f->handleAction()) {
                return new JsonOutputObject([
                    'action' => 'reload'
                ]);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if ($f) {
            try {
                return new DialogPartialView($f);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }

        return new DialogPartialView(new MessageManagerDialogRenderer());
    }


    private function radekUctovaniSluzebniCestyRecord()
    {
        $f = null;

        try {
            $idSC = (int)$_GET['idsc'];
            $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
            Prava::nastavObdobiProPrava($sc ->datumCasOd);

            if (!$sc->lzeZobrazitSluzebniCestu()) {
                throw new NeniPravoException();
            }

            $post = $this->getRequest()->isPost();
            $insert = ($_GET['record'] ?? '') === '';
            $formAction = $insert ? ($post ? 'insert' : 'new') : ($post ? 'save' : 'edit');
            if (!$post && $insert && ($_GET['copyrecord'] ?? false)) {
                $formAction = 'copy';
            }

            $f = new RadekUctovaniSCDialog($this ->database,
                $sc,
                $formAction,
                !$sc ->lzeEditovatSluzebniCestu($this ->database, false)
            );
            //$f ->utf8 = true; - presunuto do inputform

            if ($f->handleAction()) {
                return new JsonOutputObject([
                    'action' => 'reload'
                ]);
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if ($f) {
            try {
                return new DialogPartialView($f);
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }

        return new DialogPartialView(new MessageManagerDialogRenderer());
    }




//    /**
//     * Nacte data dialogu pro editaci radku
//     * @param JsonOutputObject $output
//     * @throws \Exception
//     */
//    private function nactiRadekSC($output)
//    {
//        $idSC = (int)$_GET['idsc'];
//        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
//        Prava::nastavObdobiProPrava($sc ->datumCasOd);
//        //Prava::nactiPravaUzivatele($this ->database);
//
//        if (!$sc->lzeZobrazitSluzebniCestu()) {
//            throw new NeniPravoException();
//        }
//
//        $f = new RadekSluzebniCestyDialog($this ->database, $sc);
//
//        $output ->addData('data', $f ->getJsonData());
//    }


//    /**
//     * Nacte data dialogu pro editaci radku uctovani
//     * @param JsonOutputObject $output
//     * @throws NeniPravoException
//     */
//    private function nactiRadekUctovaniSC($output)
//    {
//        $idSC = (int)$_GET['idsc'];
//        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
//        Prava::nastavObdobiProPrava($sc ->datumCasOd);
//        //Prava::nactiPravaUzivatele($this ->database);
//
//        if (!$sc->lzeZobrazitSluzebniCestu()) {
//            throw new NeniPravoException();
//        }
//
//        $f = new RadekUctovaniSCDialog($this ->database, $sc);
//
//        $output ->addData('data', $f ->getJsonData());
//    }


//    /**
//     * Zpracuje data dialogu pro editaci radku
//     * @param JsonOutputObject $output
//     * @throws NeniPravoException
//     */
//    private function zapisRadekSC($output)
//    {
//        $idSC = (int)$_POST['SluzebniCesta_ID'];
//        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
//        Prava::nastavObdobiProPrava($sc ->datumCasOd);
//        //Prava::nactiPravaUzivatele($this ->database);
//
//        if (!$sc->lzeEditovatSluzebniCestu($this ->database, true)) {
//            throw new NeniPravoException();
//        }
//
//        $f = new RadekSluzebniCestyDialog($this ->database, $sc);
//        //$f ->utf8 = true; - nastavi se automaticky pro dialogmode=true
//
//        if (!$f ->handleAction()) {
//            $output ->addMessages(MessageManager::getInstance());
//        }
//    }


//    /**
//     * Zpracuje data dialogu pro editaci radku uctovani
//     * @param JsonOutputObject $output
//     * @throws NeniPravoException
//     */
//    private function zapisRadekUctovaniSC($output)
//    {
//        $idSC = (int)$_POST['SluzebniCesta_ID'];
//        $sc = SluzebniCesta::nactiSluzebniCestu($this->database, $idSC);
//        Prava::nastavObdobiProPrava($sc ->datumCasOd);
//        //Prava::nactiPravaUzivatele($this ->database);
//
//        if (!$sc->lzeEditovatSluzebniCestu($this ->database, true)) {
//            throw new NeniPravoException();
//        }
//
//        $f = new RadekUctovaniSCDialog($this ->database, $sc);
//        $f ->utf8 = true;
//
//        if (!$f ->handleAction()) {
//            $output ->addMessages(MessageManager::getInstance());
//        }
//    }


    function handleJsonAction($action, $output)
    {
        switch($_GET['action'] ?? '') {
//            case 'radeksc' :
//                $this ->nactiRadekSC($output);
//                break;

//            case 'radekuctovanisc' :
//                $this ->nactiRadekUctovaniSC($output);
//                break;

//            case 'zapisradeksc' :
//                $this ->zapisRadekSC($output);
//                break;

//            case 'zapisradekuctovanisc' :
//                $this ->zapisRadekUctovaniSC($output);
//                break;

            case 'vyberspolucestujicich' :
                $this ->vyberSpolucestujicich($output);
                break;

            case 'dataproosobu' :
                $this ->nactiDataProOsobu($output);
                break;

            case 'pridejspolucestujiciho' :
                $this ->pridejSpolucestujiciho($output);
                break;

            case 'overspolucestujiciho' :
                $this ->overSpolucestujiciho($output);
                break;

            case 'maxstravneprostat' :
                $this ->maximalniDenniStravneProStat($output);
                break;

            case 'statpromistosc' :
                $this ->statProMistoSC($output);
                break;

            case 'kapesneprostat' :
                $this ->kapesneProStat($output);
                break;

            case 'radekuctovanikontrola' :
                $this ->kontrolaRadkuUctovani($output);
                break;

            case 'adresarozdeleni' :
                $this ->adresaRozdeleni($output);
                break;

            case 'rozdeleniosoby' :
                $this ->rozdeleniOsoby($output);
                break;

            case 'schvalenisc' :
                $this ->schvaleniSC($output);
                break;

            case 'dokumentupload' :
                $this ->dokumentUpload();
                break;

            default :
                throw new \Exception('Chyba v SluzebniCestyJsController::handleJsonAction - neznama akce!');
        }
    }

    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');

        switch($_GET['action'] ?? '') {
            case 'radekscrecord' :
                return $this->radekSluzebniCestyRecord();

            case 'uctovaniscrecord' :
                return $this->radekUctovaniSluzebniCestyRecord();
        }

        return parent::handleAction($action);
    }
}
