<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\System;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HttpResponse;
use Ron\MVC\OutputObject;

class FotoOutputObject implements OutputObject
{
    /** @var  Database */
    protected $database;
    /** @var  int */
    protected $id;
    /** @var string Nazev SQL tabulky */
    protected $tabulka;

    /**
     * FotoOutputObject constructor.
     * @param Database $database
     * @param int $id Nepovinny parametr - id fotografie
     * @param string $tabulka Nepovinny parametr - nazev
     */
    public function __construct(Database $database, $id = null, $tabulka = 'Fotografie')
    {
        $this->database = $database;
        $this->id = $id;

        $this->tabulka = $tabulka;
    }


    public function output()
    {
        $params = $this ->database ->createParameters();

        $sql = 'SELECT typ, foto FROM ' . $this->tabulka . ' WHERE id = ' . $params ->addParameter($this ->id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        if (!$row = $cursor ->fetch()) exit; // zadane foto nenalezeno

        HttpResponse::getInstance() ->getHeaders() ->addHeadersForContent("foto{$this ->id}.{$row[0]}");
        //3.2.2017 RON-PR doplnena podpora cachovani obrazku na 1 mesic
        HttpResponse::getInstance() ->getHeaders() ->addHeader('Expires', date('r', DateUtils::incMonth(time(),1)));
        HttpResponse::getInstance() ->getHeaders() ->addHeader('Cache-control', 'public');
        HttpResponse::renderHeaders();

        print $this ->database ->stringToBinary($row[1]);
        exit;
    }


    /**
     * Ze zadane tabulky (Osoba, P_Uchazec a podobne) doplni IdFotografie
     * @param $rc
     * @param string $tabulka
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public function nactiIdFotkyOsoby($rc, $tabulka = 'Osoba')
    {
        $params = $this ->database ->createParameters();
        $sql = 'SELECT IdFotografie FROM ' . $tabulka . ' WHERE RC = ' . $params ->addParameter($rc);
        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Zadana osoba nebyla nalezena!');
        }

        $this ->id = $row[0];
    }


    public function nactiIdFotkyPracovnihoPomeru($idPP)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.IdFotografie ' .
            '  FROM Osoba, PracovniPomer ' .
            ' WHERE PracovniPomer.Id = ' . $params ->addParameter($idPP) .
            '   AND Osoba.RC = PracovniPomer.RC';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Zadany pracovni pomer nebyl nalezen!');
        }

        $this ->id = $row[0];
    }
}
