<?php

/**
 * Pavel Rudol, RON Software
 *  13.3.2015
 * seznam skupin generovanych do terminalu
 */

namespace RonPortal\Terminaly;

use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Seznam skupin generovanych do terminalu
 *
 * @author pavel.rudol
 */
class SkupinyTerminaluBrowser extends RonPortalBrowser
{
    /** @var  PravaNaOsoby */
    private $pravaZobrazit;
    /** @var string */
    private $terminal;

    /**
     * @param Database $database
     * @param string $terminal
     */
    function __construct($database, $terminal)
    {
        parent::__construct($database, 'terminaly&action=skupiny&record=' . $this ->terminal);
        
        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');
        $this->terminal = $terminal;
        
        $this
            ->setTitle('terminal_skupiny')
            ->setCode('browserskupinyterminalu')
            ->setTableId('browserskupinyterminalu');

        $this->sqlFromPart = 
                'PovoleniPristupuRozdeleni JOIN Rozdeleni ON (Rozdeleni.Kod = PovoleniPristupuRozdeleni.Rozdeleni '.
                '  AND Rozdeleni.KodTypuRozdeleni = PovoleniPristupuRozdeleni.TypRozdeleni)';
        

        $this->defaultSort = '-col-platnostod';
        $this->primaryField = 'col-kod';

        $this->browserUrl = '?menuid=terminaly&action=skupiny&record=' . StringUtils::htmlTextConvert($this->terminal) . '&';
        
        $this->addField()
            ->setSqlFieldName('Rozdeleni.Kod') ->setHeaderText('Kod') ->setOrdering(true)
            ->setCode('col-kod');

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setHeaderText('rozdeleni') ->setOrdering(1)
            ->setCode('col-popis');

        $this->addField()
            ->setSqlFieldName('PovoleniPristupuRozdeleni.PlatnostOd') ->setHeaderText('Platnost_od') ->setOrdering(1)
            ->setCode('col-platnostod')
            ->setFormat('date');

        $this->addField()
            ->setSqlFieldName('PovoleniPristupuRozdeleni.PlatnostDo') ->setHeaderText('Platnost_do') ->setOrdering(1)
            ->setCode('col-platnostdo')
            ->setFormat('date');
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this->database,
            [
                'menuid' => 'terminaly',
                'record' => $this ->terminal,
                'action' => 'skupiny'
            ],
            true);

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setCaption(TextManager::_('terminal_platnost') . ' ' . TextManager::_('Od'))
                ->setDatumOdDoLinks(['today']),
            true
        );
    }


    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            '(PovoleniPristupuRozdeleni.Terminal = ' . $params->addParameter($this->terminal) . ')';

        $datumOd = $this ->filterForm ->getOmezitObdobiOd() ?? time();
        $datumDo = $this ->filterForm ->getOmezitObdobiDo() ?? $datumOd;

        if ($this->pravaZobrazit->jsouNejakePrava()) {
            $result .=
                ' AND PovoleniPristupuRozdeleni.PlatnostOd  <= ' . $params->addDateParameter($datumDo) .
                ' AND (PovoleniPristupuRozdeleni.PlatnostDo >= ' . $params->addDateParameter($datumOd) . ' OR PovoleniPristupuRozdeleni.PlatnostDo IS NULL)';
        } else {
            $result .= ' AND 1 = 0';
        }
                
        return new SqlStatement($result, $params);
    }
}
