<?php

/**
 * Pavel Rudol, RON Software
 *  10.2.2015
 */

namespace RonPortal\Terminaly;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Seznam terminalu, na ktere ma prihlaseny uzivatel pravo podle rozdeleni. 
 * Pokud nelze pro filtrovani pouzit rozdeleni, budou se zobrazovat vsechny terminaly.
 *
 * @author pavel.rudol
 */
class TerminalyBrowser extends RonPortalBrowser
{
    
    /** @var PravaNaOsoby */
    private $pravaZobrazit;

    /**
     * TerminalyBrowser constructor.
     * @param Database $database
     * @param $action
     */
    public function __construct($database)
    {
        parent::__construct($database, 'terminaly');

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('prohl_terminaly');

        $this->title = TextManager::_('Seznam_terminalu');

        $this->code = 'browserterminaly';
        $this->tableId = 'browserterminaly';

        $this->sqlSelectPart = 'Terminal.Kod'; // 0 (primary)

        $this->sqlFromPart =
                'Terminal JOIN S_UrceniTerminalu ON (S_UrceniTerminalu.Kod = Terminal.KodUrceni)'.
                ' LEFT JOIN Umisteni ON (Umisteni.Kod = Terminal.KodUmisteni)'.
                ' LEFT JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = Terminal.KodTypuRozdeleni AND Rozdeleni.Kod = Terminal.KodRozdeleni)'.
                ' LEFT JOIN StavTerminalu ON (StavTerminalu.KodTerminalu = Terminal.Kod)';

        $this->primaryField = 0;

        $this->browserUrl = '?menuid=terminaly&';

        $this->addField()
            ->setSqlFieldName('Terminal.Kod') ->setHeaderText('Kod') ->setOrdering(true)
            ->setCode('col-kod');

        $this->defaultSort = 'col-kod';

        $this->addField()
            ->setSqlFieldName('Terminal.Popis') ->setHeaderText('Popis') ->setOrdering(true)
            ->setCode('col-popis')
            ->setLink('?menuid=terminaly');

        $this->addField()
            ->setSqlFieldName('S_UrceniTerminalu.Popis') ->setHeaderText('terminal_urceni') ->setOrdering(true)
            ->setCode('col-urceni');

        $this->addField()
            ->setSqlFieldName('Umisteni.Popis') ->setHeaderText('umisteni') ->setOrdering(true)
            ->setCode('col-umisteni');

        $this->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setHeaderText('rozdeleni') ->setOrdering(true)
            ->setCode('col-rozdeleni');

        $this->addField()
            ->setSqlFieldName('StavTerminalu.NeuspesnychNastaveni') ->setHeaderText('terminal_pocet_nastaveni_browser')
            ->setAlign('center') ->setOrdering(true)
            ->setCode('col-neuspesnychnastaveni');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName($database ->castWideMemo('Terminal.Poznamka'))
            ->setCode('col-poznamka')
            ->setClassName('browserpoznamka terminalpoznamka') ->setHeaderColSpan(-1);
    }
    

    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database, $this->action, true);

        $this ->filterForm ->addFilterRozdeleni(
            (new FilterFormFilterRozdeleni($this ->filterForm, 'rozdeleni'))
                ->setTypRozdeleni($this ->pravaZobrazit-> typRozdeleni)
                ->setJenVlastniRozdeleni(2)
                ->setShowVcetnePodrizenych(false)
        );

//        $this-> filterForm ->nastavRozdeleni(true, $this ->pravaZobrazit-> typRozdeleni, 2);
//        $this-> filterForm ->showVcetnePodrizenychRozdeleni = false;

        // vyber urceni terminalu
        $urceniFilter = new FilterFormFilter('urceni', FilterFormFilter::TYPE_OPTIONS);
        $urceniFilter
            ->setCaption(TextManager::_('terminal_jen_urceni'))
            ->setRowId('filterrow_urceniterminalu')
            ->setSessionName('filterform_urceni_terminalu')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM S_UrceniTerminalu WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('Terminal.KodUrceni = ');
        $this ->filterForm ->addFilter($urceniFilter);

        // vyber umisteni terminalu
        $umisteniFilter = new FilterFormFilter('umisteni', FilterFormFilter::TYPE_OPTIONS);
        $umisteniFilter
            ->setCaption(TextManager::_('terminal_jen_umisteni'))
            ->setRowId('filterrow_umisteniterminalu')
            ->setSessionName('filterform_umisteni_terminalu')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM Umisteni WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('Terminal.KodUmisteni = ');
        $this ->filterForm ->addFilter($umisteniFilter);

        // zaskrtavatko "jen neuspesne nactene"
        $stavFilter = new FilterFormFilter('jenneuspesne',FilterFormFilter::TYPE_CHECKBOX);
        $stavFilter
            ->setCaption(TextManager::_('terminal_jen_neuspesne_nastavene'))
            ->setRowId('filterrow_stavterminalu')
            ->setSessionName('filterform_jen_neuspesne_nastavene_terminaly')
            ->setSqlFilter('StavTerminalu.NeuspesnychNastaveni > 0');
        $this ->filterForm ->addFilter($stavFilter);
    }
    
    
    /**
     * Vraci cast retezce, ktera se vlozi do WHERE casti dotazu
     * muze vracet napriklad filtr podle zvoleny filtru (viz renderFilter)
     * @return SqlStatement
     */
    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        //$result = '((Rozdeleni.KodTypuRozdeleni = 8) OR (' . $this->f_pravaZobrazit->SQLfiltrRozdeleni(time(), time(), 1) . '))'; //rozdeleni na ktere je pravo vcetne vsech jidelen
        $result .=
            ($result ? ' AND ' : '') .
            '(' . $this->pravaZobrazit->sqlfiltrRozdeleni(time(), time(), $params, 1) . ')'; // zobrazi jen terminaly z rozdeleni na ktere je pravo, ostatni odfiltruje

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    'Terminal.KodTypuRozdeleni = ' . $params ->addParameter((int)$rozdeleniFilter->getTypRozdeleni()) .
                    ' AND Terminal.KodRozdeleni = ' . $params ->addParameter($rozdeleniFilter->getValue());
            }
        }
                
        return new SqlStatement($result, $params);
    }




    function renderLine($values, $classname, $rownumber, $edit = false)
    {
        if ($values['col-neuspesnychnastaveni'] > 0) {
            $classname = $rownumber % 2
                ? 'terminal_neuspesnenastaveni1'
                : 'terminal_neuspesnenastaveni2';
        }
        
        parent::renderLine($values, $classname, $rownumber, $edit);
    }
}
