<?php

namespace RonPortal\Testy;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\System\AgentManager;
use RonPortal\User;
use RonPortal\XML\RonPortalXMLForm;

class Dotaznik
{
    public const STAV_PRIPRAVENO = 0;
    public const STAV_PROBIHA = 1;
    public const STAV_UKONCENO = 2;
    public const STAV_UKONCENO_VRACENO = 3;
    public const STAV_PRIPRAVENO_VRACENO = 4;
    public const STAV_VYRAZENO = 5;

    /** @var Database */
    var $database;

    // zakladni udaje z hlavicky sablony (TemaHodnoceni)
    var $druhHodnoceni;
    var $temaCislo;
    /** @var int 0=hodnoceni, 1=test - urci se podle DruhHodnoceni */
    var $test;
    var $kod;
    var $popisTematu;
    var $poznamkaTematu;
    var $barvaPozadi;
    var $sirkaSloupce;
    /** @var  int id xml sablony pro zobrazeni */
    var $fXMLSablona; //
    var $minPocetBodu;
    var $maxPocetBodu;
    var $stejnaVaha;
    var $editovatPoznamku;

    /** @var int 0=nezobrazovat, 1=oznacit spatne a spravne otazky, 2=oznacit i spravne odpovedi */
    public $zobrazitOdpovediUspesne;
    public $zobrazitOdpovediNeuspesne;

    public $nahodnePoradiOtazek;

    // udaje z hodnoceni konkretniho zamestnance
    var $fId;
    /** @var  int 0 = pripraveno, 1 = probiha, 2 = ukonceno, 3 = ukonceno - vraceno, 4 = pripraveno (vraceno) */
    var $fStav;
    var $fDatumHodnoceni;
    var $poznamkaHodnoceni;

    /** @var string     Nacteno z PripraveneTemaHodnoceni.HodnocenyRC */
    public $hodnocenyRC;
    /** @var string     Nacteno z PripraveneTemaHodnoceni.Hodnotici - jedna se o text - napr. uzivatelske jmeno */
    var $hodnotici;

    // testy
    public $test_body;
    public $test_stav;
    public $test_datumCasPocatku;
    public $test_datumCasUkonceni;

    /** @var  bool priznak, jestli byl dotaznik ulozen v XML pomoci <ulozodpovedihodnoceni> a s jakym vysledkem,  null = nebyl ulozen, true/false = byl ulozen */
    var $fUlozenoVXML;
    /** @var  bool priznak, jestli byl dotaznik prepocten v XML pomoci <prepocitejhodnoceni> */
    var $fPrepoctenoVXML;
    /** @var bool  priznak, jestli jsou vyplnene vsechny odpovedi Nastavi se pri zobrazovani dotazniku a podle toho se pak (ne)zobrazi tlacitko "uzavrit" */
    var $fVyplnenoVse;

    /** @var  int 0 - aktualni uzivatel nema pravo na dotaznik, 1 -    ma jen pravo zobrazit, 2 -    ma pravo editovat */
    var $fPravo;

    /** @var  OtazkaDotazniku[] */
    var $otazky;


    /**
     * Dotaznik constructor.
     * @param $database
     * @param null $id
     * @throws DBException
     */
    function __construct($database, $id = null)
    {
        $this ->database = $database;

        if ($id) {
            $this->nactiPripravenyDotaznik($id);
        }

        $this ->zobrazitOdpovediNeuspesne = 0;
        $this ->zobrazitOdpovediUspesne = 0;
    }


    function getJsonData()
    {
        $obj = [
            'druhHodnoceni' => $this ->druhHodnoceni,
            'temaCislo' => $this ->temaCislo,
            'test' => $this ->test,
            'otazky' => []
        ];

        foreach ($this ->otazky as $otazka) {
            $obj['otazky'][] = $otazka ->getJson();
        }

        return $obj;
    }



    function jeEditovatelnyStav()
    {
        return $this->fStav == 0 || $this->fStav == 1  || $this ->fStav == 4;
    }


    /**
     * @param int $id
     * @throws DBException
     */
    protected function nactiHlavicku($id)
    {
        $id = (int)$id;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PripraveneTemaHodnoceni.Stav, PripraveneTemaHodnoceni.DatumHodnoceni, ' . // 0, 1
            '       PripraveneTemaHodnoceni.HodnocenyRC, PripraveneTemaHodnoceni.Hodnotici,' . // 2, 3
            $this ->database ->castWideMemo('PripraveneTemaHodnoceni.Poznamka') . ', ' . // 4
            '       TemaHodnoceni.DruhHodnoceni, TemaHodnoceni.TemaCislo, ' . // 5, 6
            '       TemaHodnoceni.PopisTematu, ' . $this ->database ->castWideMemo('TemaHodnoceni.Poznamka') . ', ' . // 7, 8
            '       TemaHodnoceni.BarvaPozadi, TemaHodnoceni.SirkaSloupce, TemaHodnoceni.XMLproZobrazeni,' . // 9..11
            '       TemaHodnoceni.EditovatPoznamku, ' . // 12
            '       PripraveneTemaHodnoceni.Test_Body, PripraveneTemaHodnoceni.Test_Stav, PripraveneTemaHodnoceni.Test_DatumCasPocatku, PripraveneTemaHodnoceni.Test_DatumCasUkonceni, ' . // 13..16
            '       TemaHodnoceni.ZobrazitOdpovediUspesne, TemaHodnoceni.ZobrazitOdpovediNeuspesne, TemaHodnoceni.NahodnePoradiOtazek' . // 27..30

            ' FROM PripraveneTemaHodnoceni, TemaHodnoceni' .
            ' WHERE PripraveneTemaHodnoceni.DruhHodnoceni = TemaHodnoceni.DruhHodnoceni' .
            '   AND PripraveneTemaHodnoceni.TemaCislo = TemaHodnoceni.TemaCislo' .

            ' AND PripraveneTemaHodnoceni.Id = ' . $params ->addParameter($id);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $this->fId = $id;
            $this->fStav = $row[0];
            $this->fDatumHodnoceni = $this->database->stringToTimestamp($row[1]);

            $this->hodnocenyRC = $row[2];
            $this->hodnotici = $row[3];

            $this->poznamkaHodnoceni = $row[4];

            $this->druhHodnoceni = (int)$row[5];
            $this->temaCislo = (int)$row[6];

            $this->popisTematu = $row[7];
            $this->poznamkaTematu = $row[8];

            $this->barvaPozadi = $row[9];
            $this->sirkaSloupce = $row[10];
            $this->fXMLSablona = $row[11];

            $this->editovatPoznamku = (int)$row[12];

            $this ->test_body = $row[13];
            $this ->test_stav = $row[14];
            $this ->test_datumCasPocatku = $this ->database ->stringToTimestamp($row[15]);
            $this ->test_datumCasUkonceni = $this ->database ->stringToTimestamp($row[16]);

            $this->zobrazitOdpovediUspesne = (int)$row[17];
            $this->zobrazitOdpovediNeuspesne = (int)$row[18];

            $this->nahodnePoradiOtazek = (int)$row[19];

            $this->fUlozenoVXML = null;
            $this->fPrepoctenoVXML = null;

            $this ->test = ($this ->druhHodnoceni == 6  ||  $this ->druhHodnoceni == 9  ||  $this ->druhHodnoceni == 10)
                ? 1
                : 0;
        }
    }


    /**
     * @param $id
     * @throws DBException
     */
    function nactiPripravenyDotaznik($id)
    {
        $this ->nactiHlavicku($id);
        $this ->otazky = OtazkaDotazniku::nactiOtazky($this ->database, $id, $this ->druhHodnoceni, $this ->temaCislo, false);
    }


    /**
     * nacteni pro zobrazeni bez pripraveneho dotazniku - jen sablona (testovaci zobrazeni z aplikace)
     * @param $druhHodnoceni
     * @param $temaCislo
     * @throws Exception
     */
    private function nactiHlavickuSablony($druhHodnoceni, $temaCislo)
    {
        $sql =
            'SELECT TemaHodnoceni.DruhHodnoceni, TemaHodnoceni.TemaCislo, ' . // 0, 1
            '       TemaHodnoceni.PopisTematu, ' . $this ->database ->castWideMemo('TemaHodnoceni.Poznamka') . ', ' . // 2, 3
            '       TemaHodnoceni.BarvaPozadi, TemaHodnoceni.SirkaSloupce, TemaHodnoceni.XMLproZobrazeni, ' . // 4..6
            '       TemaHodnoceni.Kod, TemaHodnoceni.MinPocetBodu, TemaHodnoceni.MaxPocetBodu, TemaHodnoceni.StejnaVaha, ' . // 7..10
            '       TemaHodnoceni.EditovatPoznamku, TemaHodnoceni.ZobrazitOdpovediUspesne, TemaHodnoceni.ZobrazitOdpovediNeuspesne ' . // 11..13
            '  FROM TemaHodnoceni ' .
            ' WHERE DruhHodnoceni = ' . (int)$druhHodnoceni . ' AND  TemaCislo = ' . (int)$temaCislo;

        $cursor = $this ->database ->getSqlResult($sql);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception('Pozadovany dotaznik nebyl nalezen!');
        }

        $this->druhHodnoceni = $druhHodnoceni;
        $this->temaCislo = $temaCislo;
        $this->popisTematu = $row[2];
        $this->poznamkaTematu = $row[3];
        $this->editovatPoznamku = $row[11];

        $this ->kod = $row[7];
        $this ->minPocetBodu = $row[8];
        $this ->maxPocetBodu = $row[9];
        $this ->stejnaVaha = $row[10];

        $this->barvaPozadi = $row[4];
        $this->sirkaSloupce = $row[5];
        $this->fXMLSablona = $row[6];

        $this->fId = null;
        $this->fStav = 0;
        $this->fDatumHodnoceni = null;
        $this->poznamkaHodnoceni = '';

        $this->hodnocenyRC = null;

        $this->hodnotici = '';

        $this->fUlozenoVXML = null;
        $this->fPrepoctenoVXML = null;

        $this ->zobrazitOdpovediUspesne = (int)$row[12];
        $this ->zobrazitOdpovediNeuspesne = (int)$row[13];

        $this ->test = ($this ->druhHodnoceni == 6  ||  $this ->druhHodnoceni == 9  ||  $this ->druhHodnoceni == 10) ? 1 : 0;
    }


    /**
     * @throws DBException
     */
    function nactiSpravneOdpovedi() {
        if (!$this ->otazky) {
            return;
        }

        foreach ($this ->otazky as $otazka) {
            $otazka ->nactiSpravneOdpovedi($this ->database);
        }
    }


    /**
     * @param $druhHodnoceni
     * @param $temaCislo
     * @param bool $jenVlastniOdpovedi
     * @throws Exception
     */
    function nactiSablonu($druhHodnoceni, $temaCislo, $jenVlastniOdpovedi = false)
    {
        $this ->nactiHlavickuSablony($druhHodnoceni, $temaCislo);
        $this ->otazky = OtazkaDotazniku::nactiOtazky($this ->database, null, $druhHodnoceni, $temaCislo, $jenVlastniOdpovedi);
    }


    function zobrazHlavicku()
    {
    }


    function vratOtazku($kodKriteria)
    {
        foreach ($this ->otazky as $otazka) {
            if ($otazka ->kod == $kodKriteria) {
                return $otazka;
            }
        }

        return null;
    }

    /**
     * @param bool $editace
     * @param int $povolitHTML  0=ne, 1=ano a nahradit \n, 2=ano
     * @param int  $oznacitSpravne  urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     */
    function zobrazOtazkyDotazniku($editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
        $this->fVyplnenoVse = true;
        // pokud nektera povinna otazka nema vyplnenou odpoved, tak priznak nastavi na false

        foreach ($this ->otazky as $otazka) {
            $stylestr = ($otazka->barvaPozadi)
                ? 'style="background-color:' . GuiUtils::getHtmlColor($otazka->barvaPozadi) . '"'
                : '';

            echo "<div class='otazka_{$otazka->kod}' $stylestr>";
            $otazka->zobrazOtazku($editace, $oznacitSpravne, $povolitHTML);
            echo "</div>\n";

            if (!$otazka->odpovedVyplnena  &&  $otazka->povinnaOdpoved) {
                $this->fVyplnenoVse = false;
            }
        }
    }


    /**
     * @param $editace
     * @return string
     */
    function vratHTMLPoznamka($editace)
    {
        $result = '<div class="poznamkahodnoceni margin-all-2 margin-left-4">';

        if ($editace) {
            $result .= '<textarea name="poznamkahodnoceni" class="form-control" rows="8">' . StringUtils::htmlTextConvert($this->poznamkaHodnoceni) . '</textarea>';
        } else {
            $result .= StringUtils::longTextToHtml($this->poznamkaHodnoceni, true);
        }

        $result .= '</DIV>';

        return $result;
    }


    /**
     * @param $editace
     */
    function zobrazPoznamku($editace)
    {
        echo '<DIV class="textotazky">' . TextManager::_('Poznamka') . '</DIV>';

        echo $this->vratHTMLPoznamka($editace);
    }



    /**
     * @param $hodnoceniAction
     * @return RonPortalXMLForm
     * @throws Exception
     */
    function createXMLForm($hodnoceniAction)
    {
        $xmlForm = new RonPortalXMLForm($this ->database);
        $xmlForm->showHeader = false;

        $xmlForm->setVariable('HODNOCENI_Id', (int)$this->fId);

        $xmlForm->setVariable('HODNOCENI_DruhHodnoceni', (int)$this->druhHodnoceni);
        $xmlForm->setVariable('HODNOCENI_TemaCislo', (int)$this->temaCislo);

        $xmlForm->setVariable('HODNOCENI_PopisTematu', $this->popisTematu);
        $xmlForm->setVariable('HODNOCENI_PoznamkaTematu', $this->poznamkaTematu);

        $xmlForm->setVariable('HODNOCENI_Action', $hodnoceniAction); // nahled/editace/prevzit

        $xmlForm->rc = $this->hodnocenyRC;

        $xmlForm->dotaznik = $this;

        return $xmlForm;
    }


    /**
     * @param $editace
     * @param $postValues
     * @param null $hodnoceniAction
     * @return bool
     * @throws Exception
     */
    function zobrazDotaznikXML($editace, $postValues, $hodnoceniAction = null)
    {
        $action = $hodnoceniAction
            ? $hodnoceniAction
            : ($editace ? 'editace' : 'nahled');

        // pokud nektera otazka nema vyplnenou odpoved, tak pri zobrazovani priznak nastavi na false
        $this->fVyplnenoVse = true;

        $xmlForm = $this->createXMLForm($action);
        $xmlForm->readOnly = !$editace;

        $xmlForm ->nactiXMLFormular($this ->fXMLSablona, true);
        $xmlForm ->usePostValues = $postValues;

        if ($hodnoceniAction) {
            $xmlForm ->loadXML();
            $xmlForm ->event();
        } else {
            $html = $xmlForm ->getHtml(); // POZOR! Neni implementovano osetreni passwordProtected formulare (neni potreba, da se chranit cele hodnoceni)
            MessageManager::getInstance() ->showMessages();
            echo $html;
        }

        //$xmlForm->zobrazDotaznikZeSablony($this->fXMLSablona, $postvalues, !!$hodnoceniaction); // pri "nastavhodnotitele" se nic nezobrazuje

        return !$xmlForm ->isError();
    }


    /**
     * @throws Exception
     */
    function vyplnDotaznikXML()
    {
        if (!$this->fXMLSablona) {
            throw new Exception('Chyba ve funkci VyplnDotaznik - neni zadan id sablony!');
        }

        $xmlform = $this->createXMLForm('editace');

        $this->fVyplnenoVse = true;
        // pokud nektera otazka nema vyplnenou odpoved, tak pri zobrazovani priznak nastavi na false

        $xmlform->nactiXMLFormular($this ->fXMLSablona, true); // zeSablony = true
        $xmlform->usePostValues = true;
        $xmlform ->fill();
    }



    /**
     * @param $editace
     * @param int  $oznacitSpravne  urcuje, jestli se maji oznacit spravne a spatne odpovedi (0=ne, 1=jen zaskrtnute, 2=vsechny)
     * @param int $povolitHTML  0=ne, 1=ano a nahradit \n, 2=ano
     */
    function zobrazObsahDotazniku($editace, $oznacitSpravne = 0, $povolitHTML = 0)
    {
//        echo '<div class="row"><h2 class="col col-12 nadpishodnoceni">' .
//            ($povolitHTML ? $this ->popisTematu : StringUtils::htmlTextConvert($this->popisTematu)) .
//            "</h2></div>\n";
//
//        if ($this->poznamkaTematu) {
//            echo
//                '<div class="row"><div class="col col-12 popishodnoceni poznamka margin-bottom-3">' .
//                (($povolitHTML == 2) ? $this ->poznamkaTematu : StringUtils::longTextToHtml($this->poznamkaTematu, !$povolitHTML)) .
//                "</div></div>\n";
//        }

        $this->zobrazOtazkyDotazniku($editace, $oznacitSpravne, $povolitHTML);

        if ($this ->editovatPoznamku) {
            echo "<div class='row'><div class='col col-12'>";
            $this->zobrazPoznamku($editace);
            echo "</div></div>\n";
        }
    }


//    /**
//     * Zobrazi dotaznik (PripraveneTemaHodnoceni)
//     * pokud je $editace = false  i  $postvalues = false, tak se jedna pouze o zobrazeni
//     * pokud je $editace = false a $postvalues = true, tak nemuze menit hodnoty, ale pouze "potvrdit" nebo se vratit ("opravit")
//     * @param $editace    boolean     Pokud je true, muze uzivatel menit hodnoty
//     * @param $postValues boolean  Pokud je true, pouziji se pro zobrazeni hodnoty z POST
//     */
//    abstract function zobrazDotaznik($editace, $postValues);



    // ----- EDITACE DAT --------------------------------------------------------
    /**
     * Nacte data z POST
     * @return bool
     * @throws Exception
     */
    function loadPost()
    {
        $result = true;
        $this->fVyplnenoVse = true;
        // pokud nektera otazka nema vyplnenou odpoved, tak priznak nastavi na false

        foreach ($this ->otazky as $otazka) {
            $result &= $otazka->loadPost();

            if ($otazka ->viceTypuOdpovedi) {
                foreach ($otazka->dalsiOdpovedi as $odpoved) {
                    //$result &=
                    $odpoved->ulozOdpoved($this->database, $this ->fId);
                }
            }

            if (!$otazka->odpovedVyplnena && $otazka->povinnaOdpoved) {
                $this->fVyplnenoVse = false;
            }
        }

        if ($this ->editovatPoznamku) {
            $this->poznamkaHodnoceni = $_POST['poznamkahodnoceni'] ?? null;
        }

        return $result;
    }


    /**
     * @throws Exception
     */
    function ulozOdpovedi()
    {
        $this->fVyplnenoVse = true;
        // pokud nektera otazka nema vyplnenou odpoved, tak priznak nastavi na false

        foreach ($this ->otazky as $otazka) {
            //$result &=
            $otazka->ulozOdpoved($this ->database, $this ->fId);

            if ($otazka ->viceTypuOdpovedi) {
                foreach ($otazka->dalsiOdpovedi as $odpoved) {
                    //$result &=
                    $odpoved->ulozOdpoved($this->database, $this ->fId);
                }
            }

            if (!$otazka->odpovedVyplnena && $otazka->povinnaOdpoved) {
                $this->fVyplnenoVse = false;
            }
        }
    }


    /**
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function ulozVyplnenyDotaznik()
    {
        // TODO: presunout toto do Hodnoceni?
        if (!$this ->loadPost()) {
            return false;
        }

        $this ->ulozOdpovedi();

        // zapise jmeno hodnoticiho a cas
        $rec = new RonPortalDBRecord($this ->database, 'PripraveneTemaHodnoceni', $this);
        $rec ->addField('HodnotilRC', '', false, 15, User::getRC());
        $rec ->addField('Hodnotici', '', false, 80, RonPortalUtils::getLogUserName(80));
        $rec ->addField('DatumHodnoceni', 'timestamp', false, 0, time());
        $rec ->addField('Stav', 'int', false, 0, 1);
        $rec ->addField('Poznamka', 'memo', false, 0, $this ->poznamkaHodnoceni);

        $params = $this ->database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter((int)$this ->fId), $params);

        $rec ->loadRecord();
        $rec ->updateRecord();

        return true;
    }


    /**
     * @return bool
     * @throws Exception
     */
    function prepocitejDotaznik()
    {
        if (!$this->fVyplnenoVse) {
            // prepocet pouze v pripade, ze je vyplneno vse, jinak nema smysl
            MessageManager::addErrorMessage('Chyba! Neni vyplneno vse');
            return false;
        }

        try {
            AgentManager::executePersonAgent("0 -IDHodnoceni:{$this->fId} -alwaysLog");
        } catch (Exception $e) {
            throw new Exception( TextManager::_('Chyba_prepoctu_hodnoceni') . "\n" . $e ->getMessage(), 0, $e );
        }

        return true;
    }

}
