<?php
/**
 * stanislav.elbl, 14.05.2020
 */

namespace RonPortal\Vyroba;

use Ron\Controls\FormField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;

class VkladaniVyrobyFormular extends RonPortalForm
{

    /**
     * VkladaniVyrobyFormular constructor.
     * @param \Ron\DB\Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'new', true); // dialogMode = true

        $this ->cssClass = 'large-form rondialog ronform form-25-75';
        $this ->code = 'formpuvodnivyroba';

        $this ->setXmlEventsName('PUVODNI_VYROBA');

        $this ->stornoAction = "#";

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'vlozeni_vyroby_dialog',
            //'data-ronform-url' => '?menuid=monitorovaniv&action=loaddata',
            'data-ronform-submit-url' => '?menuid=monitorovaniv&action=vlozpuvodni',
            'data-ronform-after-submit' => 'reload',
            'title' => \Ron\ronText('Vlozeni_puvodni_vyroby')
        ]);

        $this ->sqlTableName = 'PuvodniVyroba';

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('*idpp') ->setFieldId('vkladani_idpp') ->setDefaultValue('');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setFieldId('vkladani_jmeno')
            ->setText('...');

        if (Prava::vratPrava('vkladanivyroby') ->priznak) {
            $this ->addField(FormField::FIELD_TYPE_DATEANDTIME)
                ->setFieldName('DatumCas') ->setCaption('Cas')
                ->setCode('field-datumcas') ->setFieldId('vkladani_vyroby_datum_cas')
                ->setDefaultValue([StringUtils::format('date', time()), StringUtils::format('time', time())]);
        }

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodVyrobniCinnosti')
            ->setCode('field-operace')
            ->setCaption('Vyroba_cinnost')
            ->setRequired(true) ->setFieldId('vkladani_vyrobni_cinnost')
            ->setSelectSql(
                'SELECT Kod, Popis,' .
                ' T_ZadavatVyrobek, T_ZadavatVyrobniCislo, T_ZadavatZakazku, T_ZadavatMnozstviDobre, T_ZadavatMnozstviSpatne, T_ZadavatCas, T_TypZadavaniCasu' .
                ' FROM VyrobniCinnost WHERE Aktivni = 1 ORDER BY Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setItemsAttributes([
                'data-zadavat-vyrobek' => 2, 'data-zadavat-vyrobni-cislo' => 3,
                'data-zadavat-zakazku' => 4, 'data-zadavat-mnozstvi-dobre' => 5, 'data-zadavat-mnozstvi-spatne' => 6,
                'data-zadavat-cas' => 7, 'data-typ-zadavani-casu' => 8]);

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VyrobniZakazka')
            ->setCode('field-vyrobnizakazka')
            ->setCaption('Vyroba_zakazka')
            ->setFieldId('vkladani_vyrobni_zakazka')
            ->setSelectSql(
                new SqlStatement(
                    'SELECT CisloZakazky, Popis FROM VyrobniZakazka' .
                    ' WHERE PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
                    ' AND (PlatnostDo >= ' .$params ->addParameter($this ->database ->dateToParam(time())) . ' OR PlatnostDo IS NULL)',
                    $params
                ))
            ->setFieldCssClass('form-control ronselect')
            ->setShowEmptyValue(true);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Vyrobek')
            ->setCode('field-vyrobek')
            ->setCaption('Vyroba_vyrobek')
            ->setFieldId('vkladani_vyrobek')
            ->setSelectSql('SELECT Kod, Popis FROM Zbozi WHERE Neaktivni = 0')
            ->setFieldCssClass('form-control ronselect')
            ->setShowEmptyValue(true);

        $this ->addField()
            ->setFieldName('VyrobniCislo')
            ->setCode('field-vyrobnicislo')
            ->setFieldId('vkladani_vyrobni_cislo')
            ->setCaption('Vyroba_vyrobni_cislo')
            ->setMaxLength(30);

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('MnozstviDobre')
            ->setCode('field-mnozstvidobre')
            ->setFieldId('vkladani_mnozstvi-dobre')
            ->setCaption('Vyroba_mnozstvi_dobre');

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('MnozstviSpatne')
            ->setCode('field-mnozstvispatne')
            ->setFieldId('vkladani_mnozstvi-spatne')
            ->setCaption('Vyroba_mnozstvi_spatne');

    }


    function createDBRecordFields($rec, $insert)
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['idpp']);

        if (!Prava::lzeZobrazitPP('vkladanivyroby', $pp ->idPP, time())) {
            throw new NeniPravoException();
        }

        // osoba
        $rec ->addStringField('RC', true, 15, $pp ->rc);
        $rec ->addStringField('OsobniCislo', true, 15, $pp ->osobniCislo);
        $rec ->addTimestampField('DatumNastupu', true, $pp ->datumNastupu);

        // cas (jen pokud neni vlozen ve formulari - tj. uzivatel ho nemuze sam zadat)
        if (!Prava::vratPrava('vkladanivyroby') ->priznak) {
            $rec->addTimestampField('DatumCas', true, time());
        }

        // dalsi pole
        parent::createDBRecordFields($rec, $insert);
    }
}
