<?php

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\DB\Database;
use RonPortal\System\AgentManager;

class VyrobaUtils
{
    /** Vraci datum a cas zpracovani vyroby
     * @param Database $database
     * @return int
     * @throws \Exception
     */
    public static function datumCasZpracovaniVyroby($database)
    {
        $sql = "SELECT val FROM SYS_VAR_GLOBAL WHERE (Name = 'DatumZpracovaniVyroby')";
        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani zpracovani vyroby!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row  ||  !$row[0]) return 0;

        try {
            $result = DateUtils::datetimeStringToTimestamp($row[0]); // Pozor! Hodnota v db ulozena jako text!
        } catch (\Exception $e) {
            throw new \Exception("Chyba pri zjistovani posledniho zpracovnani dochazky: {$row[0]}");
        }

        return $result;
    }


    /**
     * Spusti agenta pro prepocet vysledku vyroby pro zadane IdPP (jedno nebo seznam) a zadane obdobi
     * @param int[]|int|string $idPPList    Id pracovniho pomeru, seznam ID, nebo seznam ID oddelenych strednikem
     * @param int $denOd
     * @param int $denDo
     */
    public static function prepocetVysledku($idPPList, $denOd, $denDo = null)
    {
        $denDo = $denDo ?: $denOd;

        $idPPStr = '';
        if (is_array($idPPList)) {
            foreach ($idPPList as $idPP) {
                $idPPStr .= ($idPPStr ? ';' : '') . $idPP;
            }
        } else {
            $idPPStr = $idPPList;
        }

        AgentManager::executeVyroba('_Agent_Production',
            '3 -interval:' . date('Y-m-d', $denOd) . ';' . date('Y-m-d', $denDo) .
            ' -IDPP:' . $idPPStr, true);
    }

    /**
     * Spusti agenta pro prepocet vyroby podle puvodni pro zadane IdPP (jedno nebo seznam) a zadane obdobi
     * @param int[]|int $idPPList
     * @param int $denOd
     * @param int $denDo
     */
    public static function znovuZpracovaniHromadne($idPPList, $denOd, $denDo = null)
    {
        $denDo = $denDo ?: $denOd;

        $idPPStr = '';
        if (is_array($idPPList)) {
            foreach ($idPPList as $idPP) {
                $idPPStr .= ($idPPStr ? ';' : '') . $idPP;
            }
        } else {
            $idPPStr = $idPPList;
        }

        if ($idPPStr) {
            // nesmi se volat pro prazdny seznam - jinak agent zpracuje vsechny osoby!
            AgentManager::executeVyroba('_Agent_Production',
                '4 -interval:' . date('Y-m-d', $denOd) . ';' . date('Y-m-d', $denDo) .
                ' -IDPP:' . $idPPStr, true);
        }
    }
}
