<?php
/**
 * stanislav.elbl, 07.09.2018
 */

namespace RonPortal\Vyroba;

use Ron\DB\Database;
use Ron\DB\Parameters;
use RonPortal\PracovniPomer;


class VyrobniZakazka
{
    public $cisloZakazky;
    public $popis;

    /** @var int */
    public $platnostOd;
    /** @var int */
    public $platnostDo;
    /** @var int */
    public $datumPrijeti;


    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return VyrobniZakazka[]
     */
    private static function nactiZakazkySQL($database, $sql, $params)
    {
        $cursor = $database ->getSqlResult($sql, $params);

        $result = [];
        while ($row = $cursor ->fetch()) {
            $zakazka = new static();
            $zakazka ->cisloZakazky = $row[0];
            $zakazka ->popis = $row[1];
            $zakazka ->platnostOd = $database ->stringToTimestamp($row[2]);
            $zakazka ->platnostDo = $database ->stringToTimestamp($row[3]);
            $zakazka ->datumPrijeti = $database ->stringToTimestamp($row[4]);

            $result[] = $zakazka;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Vraci vyrobni zakazku se zadanym cislem
     * @param Database $database
     * @param string $cislo
     * @return VyrobniZakazka
     * @throws \Exception
     */
    public static function vratVyrobniZakazku($database, $cislo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT VyrobniZakazka.CisloZakazky, VyrobniZakazka.Popis, VyrobniZakazka.PlatnostOd, VyrobniZakazka.PlatnostDo, VyrobniZakazka.DatumPrijeti' .
            ' FROM VyrobniZakazka' .
            ' WHERE VyrobniZakazka.CisloZakazky = ' . $params ->addParameter($cislo);

        $list = static::nactiZakazkySQL($database, $sql, $params);
        if (!$list  ||  !count($list)) {
            throw new \Exception('Zakazka se zadanym cislem ' . $cislo . ' nenexistuje!');
        }

        return $list[0];
    }


    /**
     * Vraci seznam vyrobnich zakazek platnych v zadanem obdobi
     * @param Database $database
     * @param int $platnostOd
     * @param int $platnostDo
     * @return VyrobniZakazka[]
     */
    public static function nactiVyrobniZakazky($database, $platnostOd, $platnostDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT VyrobniZakazka.CisloZakazky, VyrobniZakazka.Popis, VyrobniZakazka.PlatnostOd, VyrobniZakazka.PlatnostDo, VyrobniZakazka.DatumPrijeti' .
            ' FROM VyrobniZakazka' .
            ' WHERE (VyrobniZakazka.PlatnostOd IS NULL  OR  VyrobniZakazka.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($platnostDo)) . ')' .
            ' AND (VyrobniZakazka.PlatnostDo IS NULL  OR  VyrobniZakazka.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($platnostOd)) . ')' .
            ' ORDER by Popis';

        return static::nactiZakazkySQL($database, $sql, $params);
    }


    /**
     * Nacte seznam vyrobnich zakazek prirazenych zadane osobe ($pp) pres tabulky CinnostVyrobniZakazky a PracovnikCinnostiZakazky
     * @param Database $database
     * @param int $platnostOd
     * @param int $platnostDo
     * @param PracovniPomer $pp
     * @return VyrobniZakazka[]
     */
    public static function nactiVyrobniZakazkyOsoby($database, $platnostOd, $platnostDo, $pp)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT DISTINCT VyrobniZakazka.CisloZakazky, VyrobniZakazka.Popis, VyrobniZakazka.PlatnostOd, VyrobniZakazka.PlatnostDo, VyrobniZakazka.DatumPrijeti' .
            ' FROM VyrobniZakazka, CinnostVyrobniZakazky, PracovnikCinnostiZakazky' .
            ' WHERE VyrobniZakazka.CisloZakazky = CinnostVyrobniZakazky.CisloZakazky' .
            ' AND CinnostVyrobniZakazky.Id = PracovnikCinnostiZakazky.CinnostVyrobniZakazky_ID' .
            ' AND PracovnikCinnostiZakazky.RC = ' . $params ->addParameter($pp->rc) .
            ' AND PracovnikCinnostiZakazky.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            ' AND PracovnikCinnostiZakazky.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)) .
            ' AND (VyrobniZakazka.PlatnostOd IS NULL  OR  VyrobniZakazka.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($platnostDo)) . ')' .
            ' AND (VyrobniZakazka.PlatnostDo IS NULL  OR  VyrobniZakazka.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($platnostOd)) . ')' .
            ' ORDER by Popis';

        return static::nactiZakazkySQL($database, $sql, $params);
    }
}

