<?php
/**
 * User: pavel.rudol
 * Date: 16.5.2018
 * Time: 8:41
 * (presunuto z f_dennivyroba)
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilterRozdeleni;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\PracovniPomer;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\RozdeleniUtils;
use RonPortal\System\AgentManager;

class ZaznamEditaceVyroby extends ZaznamVyroby
{
    public $jmenoOsoby;

    public $vyrobekError = false;
    public $cinnostError = false;
    public $zakazkaError = false;
    public $casOdError = false;
    public $casDoError = false;
    public $jeChyba = false;

    /** @var DenniVysledek[] */
    public $denniVysledky;
    /** @var MesicniVysledek[] */
    public $mesicniVysledky;

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param VyrobaFilterForm $filterForm
     * @param string|string[] $typCSDenni
     * @param string|string[] $typCSMesicni
     * @param bool $nactiVysledkyVyroby
     * @return ZaznamEditaceVyroby[]
     * @throws DBException
     */
    public static function nactiOsobyProVyrobu($database, $den, $prava, $filterForm, $typCSDenni = null, $typCSMesicni = null, $nactiVysledkyVyroby = false)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Vyroba.serial, PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, Osoba.Prijmeni, Osoba.Jmeno, Vyroba.den, Vyroba.vyrobek, Vyroba.cinnost, Vyroba.hodiny, ". // 0 .. 10
            " Vyroba.mnozstviDobre, Vyroba.mnozstviSpatne, Vyroba.Premie, Vyroba.cisloObjednavkyPrijate, Vyroba.casOd, Vyroba.casDo, Vyroba.DatumCasPuvodniVyroby, " . // 11 .. 17
            " Vyroba.popis, {fn_CASTWIDEMEMO(Vyroba.rozsirujiciPopis)}, Vyroba.VyrobniZakazka, Vyroba.KodTypuRozdeleni, Vyroba.Rozdeleni, Vyroba.DatumSchvaleni, Vyroba.Zpracoval, Vyroba.kodFirmy, " . // 18 .. 25
            " Vyroba.Hmotnost, Vyroba.JednotekVCyklu, Vyroba.PocetCyklu, Vyroba.Insert_UserName, Vyroba.Insert_Timestamp, Vyroba.Update_UserName, Vyroba.Update_Timestamp \n" . // 26 .. 32
            "FROM PracovniPomer " .
            " INNER JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) \n".
            " LEFT OUTER JOIN Vyroba ON (PracovniPomer.RC = Vyroba.RC AND PracovniPomer.OsobniCislo = Vyroba.OsobniCislo AND PracovniPomer.DatumNastupu = Vyroba.DatumNastupu " .
            "   AND den = " . $params->addParameter($database->dateToParam($den)) . ") \n" .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $filterForm, $den);

        $sql .=
            " ORDER BY Osoba.Prijmeni, Osoba.Jmeno, Vyroba.serial";

        MessageManager::addDebugLog('Seznam osob: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $zaznamy = [];

        if (!empty($typCSDenni))
            $denniVysledky = static::nactiDenniVysledky($database, $den, $prava, $filterForm, $typCSDenni);
        if (!empty($typCSMesicni))
            $mesicniVysledky = static::nactiMesicniVysledky($database, DateUtils::firstDayOfMonth($den), $prava, $filterForm, $typCSMesicni);

        if ($nactiVysledkyVyroby)
            $vysledkyVyroby = VysledekVyroby::nactiVysledkyVyrobyProDenniEditaci($database, $den, $prava, $filterForm);

        while ($row = $cursor->fetch()) {
            $vyroba = new ZaznamEditaceVyroby();
            $vyroba->serialVyroba = (int)$row[0];
            $vyroba->idPP = $row[1];
            $vyroba->rc = $row[2];
            $vyroba->osobniCislo = $row[3];
            $vyroba->datumNastupu = $database->stringToTimestamp($row[4]);
            $vyroba->jmenoOsoby = $row[5] . ' ' . $row[6];
            $vyroba->den = $database->stringToTimestamp($row[7]);
            $vyroba->vyrobek = $row[8];
            $vyroba->cinnost = $row[9];
            $vyroba->hodiny = $row[10];
            $vyroba->mnozstviDobre = $row[11];
            $vyroba->mnozstviSpatne = $row[12];
            $vyroba->casOd = $database->stringToTimestamp($row[15]);
            $vyroba->casDo = $database->stringToTimestamp($row[16]);
            $vyroba->zakazka = $row[20];
            $vyroba->popis = $row[18];
            $vyroba->rozsirujiciPopis = $row[19];

            if (!empty($denniVysledky))
                $vyroba->denniVysledky = isset($denniVysledky[$row[1]]) ? $denniVysledky[$row[1]] : null;
            if (!empty($mesicniVysledky))
                $vyroba->mesicniVysledky = $mesicniVysledky[$row[1]];
            if (!empty($vysledkyVyroby))
                $vyroba->vysledkyVyroby = $vysledkyVyroby[$row[0]] ?? null;

            $zaznamy[] = $vyroba;
        }

        return $zaznamy;
    }

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param int $prepocetVyroby 0 = neprepocitavat, 1 = prepocitat vzdy, 2 = prepocitat jen zmenene
     * @param int $datumCasZpracovaniVyroby
     * @param int $prepocetDochazky 0 = neprepocitavat, 1 = prepocitat vzdy, 2 = prepocitat jen zmenene
     * @param bool $kontrolaDochazky
     * @param array $nastaveni
     * @param VyrobaFilterForm $filterForm
     * @return ZaznamEditaceVyroby[]|null
     * @throws DBException
     */
    public static function zpracujPost($database, $den, $prava, $prepocetVyroby, $datumCasZpracovaniVyroby, $prepocetDochazky, $kontrolaDochazky, $nastaveni, $filterForm)
    {
        if ($den > $datumCasZpracovaniVyroby)
            throw new \Exception(TextManager::_('Vyroba_nesmi_prekrocit_cas_zpracovani'));

        $zaznamyVyroby = static::nactiOsobyProVyrobu($database, $den, $prava, $filterForm);

        $vychoziCinnost = $_POST['vyroba_vychozi_cinnost'] ?? null;

        /** @var $chybneRadky ZaznamEditaceVyroby[]|array  */
        $chybneRadky = [];

        /** @var $prepocitatZaznamy ZaznamEditaceVyroby[]|array  */
        $prepocitatZaznamy = [];
        /** @var $prepocitatZaznamyDochazky ZaznamEditaceVyroby[]|array  */
        $prepocitatZaznamyDochazky = [];

        $pocetCyklu = 1;
        if (isset($_POST["pocet_chybnych_zaznamu"])) {
            $pocetCyklu = 1 + $_POST["pocet_chybnych_zaznamu"];
        }

        foreach ($zaznamyVyroby as $zaznamVyroby) {
            $idZaznamu = $zaznamVyroby->serialVyroba ?: $zaznamVyroby->idPP;
            $maxIndexZaznamu = $_POST["{$idZaznamu}_max_index"];

            for ($n = 1; $n <= $pocetCyklu; $n++) {
                //pro puvodne zadavany zaznam u ktereho doslo pri poslednim zapisu k chybe
                if ($n > 1) {
                    $zaznamVyroby->serialVyroba = null;

                    $indexChyby = $n - 1;
                    $idZaznamu = "vyroba_chyba_{$indexChyby}";
                    $maxIndexZaznamu = $_POST["{$idZaznamu}_max_index"];

                    $idpp = $_POST["vyroba_idpp_{$idZaznamu}"];
                    $pracPomer = new PracovniPomer();
                    $pracPomer->nactiPPPodleId($database, $idpp);

                    $zaznamVyroby->rc = $pracPomer->rc;
                    $zaznamVyroby->osobniCislo = $pracPomer->osobniCislo;
                    $zaznamVyroby->datumNastupu = $pracPomer->datumNastupu;

                    if ($n == $pocetCyklu)
                        $pocetCyklu = 1;
                }

                for ($i = 1; $i <= $maxIndexZaznamu; $i++) {
                    $idRadku = "{$idZaznamu}_{$i}";

                    $vyrobek = $_POST["vyroba_vyrobek_{$idRadku}"];
                    $cinnost = $_POST["vyroba_cinnost_{$idRadku}"] ?? null; //cinnost nemusi byt editovatelna, muze byt nastavena vychozi
                    $zakazka = $_POST["vyroba_zakazka_{$idRadku}"];

                    $pocetKs = null;
                    if (!empty($_POST["vyroba_pocetks_{$idRadku}"])) {
                        $pocetKs = StringUtils::stringToFloat($_POST["vyroba_pocetks_{$idRadku}"]);
                    }

                    $pocetKsSpatne = null;
                    if (!empty($_POST["vyroba_pocetks_spatne_{$idRadku}"])) {
                        $pocetKsSpatne = StringUtils::stringToFloat($_POST["vyroba_pocetks_spatne_{$idRadku}"]);
                    }

                    $popis = null;
                    if (!empty($_POST["vyroba_popis_{$idRadku}"])) {
                        $popis = $_POST["vyroba_popis_{$idRadku}"];
                    }

                    $rozsirenyPopis = null;
                    if (!empty($_POST["vyroba_rozsirenypopis_{$idRadku}"])) {
                        $rozsirenyPopis = $_POST["vyroba_rozsirenypopis_{$idRadku}"];
                    }

                    $hodiny = null; 
                    if (!empty($_POST["vyroba_hodiny_{$idRadku}"])) {
                        $vkladatCas = true;
                        foreach ($nastaveni['sloupeceditace'] as $sloupecEditace) {
                            if ($sloupecEditace['typ'] == 'HODINY') {
                                if (isset($sloupecEditace['vkladatcas']) && $sloupecEditace['vkladatcas'] == 0) {
                                    $vkladatCas = false;
                                }
                            }
                        }
                        if ($vkladatCas) {
                            $hodiny = DateUtils::timeStringToFloat($_POST["vyroba_hodiny_{$idRadku}"], true);
                        } else {
                            $hodiny = StringUtils::stringToFloat($_POST["vyroba_hodiny_{$idRadku}"]);
                        }
                    }

                    $datumOd = $zaznamVyroby->den ?: $den;
                    //standardne se nastavi na den zaznamu vyroby, pripadne zvoleny den ve formulari, pokud vsak je nastavene i zadavani datumu, nacte se z POSTu
                    if (!empty($_POST["vyroba_datumod_{$idRadku}"]))
                        $datumOd = DateUtils::dateStringToTimestamp($_POST["vyroba_datumod_{$idRadku}"]);
                    $casOd = null;
                    if (!empty($_POST["vyroba_casod_{$idRadku}"]))
                        $casOd = DateUtils::timeStringToTimestamp($_POST["vyroba_casod_{$idRadku}"], $datumOd);

                    $datumDo = $zaznamVyroby->den ?: $den;
                    //standardne se nastavi na den zaznamu vyroby, pripadne zvoleny den ve formulari, pokud vsak je nastavene i zadavani datumu, nacte se z POSTu
                    if (!empty($_POST["vyroba_datumdo_{$idRadku}"]))
                        $datumDo = DateUtils::dateStringToTimestamp($_POST["vyroba_datumdo_{$idRadku}"]);
                    $casDo = null;
                    if (!empty($_POST["vyroba_casdo_{$idRadku}"]))
                        $casDo = DateUtils::timeStringToTimestamp($_POST["vyroba_casdo_{$idRadku}"], $datumDo);

                    if (is_null($hodiny) && !is_null($casOd) && !is_null($casDo))
                        $hodiny = ($casDo - $casOd) / 3600;

                    if (($vyrobek != '') || ($cinnost != '') || ($zakazka != '') || ($_POST["vyroba_pocetks_{$idRadku}"] != '')) {
                        $vlozit = true;

                        if (!SQLUtils::existsMasterRecord($database, $vyrobek, 'Zbozi', 'Kod')) {
                            MessageManager::addErrorText('Vyroba_vyrobek_neexistuje');
                            $zaznamVyroby->vyrobekError = true;
                            $vlozit = false;
                        }
                        if (!($vychoziCinnost || SQLUtils::existsMasterRecord($database, $cinnost, 'VyrobniCinnost', 'kod'))) {
                            MessageManager::addErrorText('Vyroba_cinnost_neexistuje');
                            $zaznamVyroby->cinnostError = true;
                            $vlozit = false;
                        }
                        if (!SQLUtils::existsMasterRecord($database, $zakazka, 'VyrobniZakazka', 'cisloZakazky')) {
                            MessageManager::addErrorText('Vyroba_zakazka_neexistuje');
                            $zaznamVyroby->zakazkaError = true;
                            $vlozit = false;
                        }

                        if ((!is_null($casOd) && ($casOd > $datumCasZpracovaniVyroby)) || (!is_null($casDo) && ($casDo > $datumCasZpracovaniVyroby))) {
                            MessageManager::addErrorText('Vyroba_nesmi_prekrocit_cas_zpracovani');

                            if (!is_null($casOd) && ($casOd > $datumCasZpracovaniVyroby))
                                $zaznamVyroby->casOdError = true;
                            if (!is_null($casDo) && ($casDo > $datumCasZpracovaniVyroby))
                                $zaznamVyroby->casDoError = true;

                            $vlozit = false;
                        }

                        if ($vlozit) {
                            $vyrobaRecord = new RonPortalDBRecord($database, 'Vyroba', 'Editace vyroby');
                            $vyrobaRecord->setXmlEvents(RonPortalXMLEvents::VYROBA_BEFORE_POST, RonPortalXMLEvents::VYROBA_AFTER_POST);
                            $vyrobaRecord->addField('serial', 'sequence', true);
                            $vyrobaRecord->addField('RC', 'string', true, 15, $zaznamVyroby->rc);
                            $vyrobaRecord->addField('OsobniCislo', 'string', true, 15, $zaznamVyroby->osobniCislo);
                            $vyrobaRecord->addField('DatumNastupu', 'timestamp', true, 0, $zaznamVyroby->datumNastupu);
                            $vyrobaRecord->addField('den', 'timestamp', true, 0, $zaznamVyroby->den ?: $den);

                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'VYROBEK')) {
                                $vyrobaRecord->addField('vyrobek', 'string', false, 30, $vyrobek);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CINNOST') || isset($vychoziCinnost)) {
                                $fCinnost = $vyrobaRecord->addField('cinnost', 'string', true, 30, $vychoziCinnost);
                                if ($cinnost) {
                                    $fCinnost->setNewValue($cinnost);
                                }
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'ZAKAZKA')) {
                                $vyrobaRecord->addField('VyrobniZakazka', 'string', false, 30, $zakazka);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'MNOZSTVI')) {
                                $vyrobaRecord->addField('mnozstviDobre', 'float', false, 0, $pocetKs);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'MNOZSTVISPATNE')) {
                                $vyrobaRecord->addField('mnozstviSpatne', 'float', false, 0, $pocetKsSpatne);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'HODINY')) {
                                $vyrobaRecord->addField('hodiny', 'float', false, 0, $hodiny);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CASOD')) {
                                $vyrobaRecord->addField('casOd', 'timestamp', false, 0, $casOd);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'CASDO')) {
                                $vyrobaRecord->addField('casDo', 'timestamp', false, 0, $casDo);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'POPIS')) {
                                $vyrobaRecord->addStringField('popis', false, 80, $popis);
                            }
                            if (VyrobaController::existujeSloupecNastaveni($nastaveni, 'ROZSIRENYPOPIS')) {
                                $vyrobaRecord->addMemoField('rozsirujiciPopis', false, $rozsirenyPopis);
                            }

                            $vyrobaRecord->addField('Zpracoval', '', false, 25, RonPortalUtils::getLogUserName())
                                ->updateLog = true;

                            $vyrobaRecord ->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
                            $vyrobaRecord ->addField('Insert_Timestamp', 'timestamp', true, 0, time());
                            $vyrobaRecord ->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
                                ->updateLog = true;
                            $vyrobaRecord ->addField('Update_Timestamp', 'timestamp', false, 0, time())
                                ->updateLog = true;

                            $prepocitat = false;
                            $prepocitatDochazku = false;
                            if (($zaznamVyroby->serialVyroba > 0) && ($i == 1)) {
                                $params = $database->createParameters();
                                $vyrobaRecord->setWhere('serial = ' . $params->addParameter($zaznamVyroby->serialVyroba), $params);
                                $vyrobaRecord->loadAndUpdateRecord();
                                if (($prepocetVyroby == 1) || ($prepocetVyroby == 2 && $vyrobaRecord->isChanged())) {
                                    $prepocitat = true;
                                }
                                if (($prepocetDochazky == 1) || ($prepocetDochazky == 2 && $vyrobaRecord->isChanged())) {
                                    $prepocitatDochazku = true;
                                }
                            } else {
                                $vyrobaRecord->insertRecord();
                                if ($prepocetVyroby > 0) {
                                    $prepocitat = true;
                                    $zaznamVyroby->den = $den;
                                }
                                if ($prepocetDochazky > 0) {
                                    $prepocitatDochazku = true;
                                    $zaznamVyroby->den = $den;
                                }
                            }
                            if ($prepocitat) {
                                if (!array_key_exists($zaznamVyroby->idPP, $prepocitatZaznamy)) {
                                    $prepocitatZaznamy[$zaznamVyroby->idPP] = $zaznamVyroby;
                                    $prepocitatZaznamy[$zaznamVyroby->idPP]->casOd = $vyrobaRecord['casOd'];
                                    $prepocitatZaznamy[$zaznamVyroby->idPP]->casDo = $vyrobaRecord['casDo'];
                                }
                            }
                            if ($prepocitatDochazku) {
                                if (!array_key_exists($zaznamVyroby->idPP, $prepocitatZaznamyDochazky)) {
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP] = $zaznamVyroby;
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP]->casOd = $vyrobaRecord['casOd'];
                                    $prepocitatZaznamyDochazky[$zaznamVyroby->idPP]->casDo = $vyrobaRecord['casDo'];
                                }
                            }
                        } else {
                            $chybnyZaznam = clone $zaznamVyroby;

                            $chybnyZaznam->serialVyroba = null;
                            $chybnyZaznam->vyrobek = $vyrobek;
                            $chybnyZaznam->cinnost = $vychoziCinnost ?: $cinnost;
                            $chybnyZaznam->zakazka = $zakazka;
                            $chybnyZaznam->mnozstviDobre = $pocetKs;
                            $chybnyZaznam->mnozstviSpatne = $pocetKsSpatne;
                            $chybnyZaznam->hodiny = $hodiny;
                            $chybnyZaznam->casOd = $casOd;
                            $chybnyZaznam->casDo = $casDo;
                            $chybnyZaznam->jeChyba = true;
                            $chybneRadky[] = $chybnyZaznam;
                        }
                    }
                }
            }
        }

        if ($prepocetVyroby > 0 && !empty($prepocitatZaznamy)) {
            $idPPList = [];
            $minDatumOd = 0;
            $maxDatumDo = 0;
            foreach ($prepocitatZaznamy as $zaznamVyroby) {
                $idPPList[] = $zaznamVyroby->idPP;

                // pokud se needituje cas od-do, pouzije se den
                if (!$minDatumOd) $minDatumOd = $zaznamVyroby->casOd ?: $zaznamVyroby ->den;
                if (!$maxDatumDo) $maxDatumDo = $zaznamVyroby->casDo ?: $zaznamVyroby ->den;

                $minDatumOd = min($minDatumOd, $zaznamVyroby->casOd ?: $zaznamVyroby ->den);
                $maxDatumDo = max($maxDatumDo, $zaznamVyroby->casDo ?: $zaznamVyroby ->den);
            }

            VyrobaController::prepocetVysledkuHromadne($idPPList, DateUtils::startOfTheDay($minDatumOd), DateUtils::endOfTheDay($maxDatumDo));
        }

        if ($prepocetDochazky > 0 && !empty($prepocitatZaznamyDochazky)) {
            $idPPStr = '';
            $minDatumOd = 0;
            $maxDatumDo = 0;
            foreach ($prepocitatZaznamyDochazky as $zaznamVyroby) {
                if ($idPPStr != '') $idPPStr .= ';';
                $idPPStr .= $zaznamVyroby->idPP;

                // pokud se needituje cas od-do, pouzije se den
                if (!$minDatumOd) $minDatumOd = $zaznamVyroby->casOd ?: $zaznamVyroby ->den;
                if (!$maxDatumDo) $maxDatumDo = $zaznamVyroby->casDo ?: $zaznamVyroby ->den;

                $minDatumOd = min($minDatumOd, $zaznamVyroby->casOd ?: $zaznamVyroby ->den);
                $maxDatumDo = max($maxDatumDo, $zaznamVyroby->casDo ?: $zaznamVyroby ->den);
            }
            try {
                AgentManager::prepocetDochazky($idPPStr, DateUtils::startOfTheDay($minDatumOd), DateUtils::endOfTheDay($maxDatumDo), false, $kontrolaDochazky);
            } catch (\Exception $exception) {
                MessageManager::addErrorMessage($exception);
            }
        }

        if (empty($chybneRadky)) return null;

        return $chybneRadky;
    }

    /**
     * @param Database $database
     * @param Parameters $params
     * @param PravaNaOsoby $prava
     * @param VyrobaFilterForm $filterForm
     * @param int $datumOd
     * @param int|null $datumDo
     * @return string
     * @throws DBException
     */
    public static function sqlFiltrOsoby($database, $params, $prava, $filterForm, $datumOd, $datumDo = null)
    {
        if (is_null($datumDo)) {
            $datumDo = $datumOd;
        }

        $sql = $prava->sqlFiltr('PracovniPomer', $datumOd, DateUtils::lastDayOfMonth($datumDo), $params) ;

        foreach ($filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if (null !== $rozdeleniFilter ->getValue()) {
                $sql .= ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($database, 'PracovniPomer',
                        $rozdeleniFilter->getValue(FilterFormFilterRozdeleni::SPEC_TYP_ROZDELENI),
                        $rozdeleniFilter->getValue(),
                        $datumOd, $datumDo,
                        $rozdeleniFilter->getValue(FilterFormFilterRozdeleni::SPEC_VCETNE_PODRIZENYCH),
                        $params);
            }
        }

//        if ($filterForm->getFilterValue('rozdeleni')) {
//            $sql .= ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($database, 'PracovniPomer', $filterForm->kodTypuRozdeleni, $filterForm->kodRozdeleni, $datumOd, $datumDo, $filterForm->vcetnePodrizenychRozdeleni, $params);
//        }
//
//        if (null !== $filterForm->getFilterValue('rozdeleni2')) {
//            $sql .= ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($database, 'PracovniPomer',
//                    $filterForm->getFilterValue('rozdeleni2', FilterFormFilterRozdeleni::SPEC_TYP_ROZDELENI),
//                    $filterForm->getFilterValue('rozdeleni2'),
//                    $datumOd, $datumDo,
//                    $filterForm->getFilterValue('rozdeleni2', FilterFormFilterRozdeleni::SPEC_VCETNE_PODRIZENYCH),
//                    $params);
//        }

        if ($filterForm->getFilterValue('osoba')) {
            $sql .=
                " AND PracovniPomer.RC = " . $params->addParameter($filterForm->getFilterValue('osoba', 'rc')) .
                " AND PracovniPomer.OsobniCislo = " . $params->addParameter($filterForm->getFilterValue('osoba', 'osobniCislo')) .
                " AND PracovniPomer.DatumNastupu = " . $params->addDateParameter($filterForm->getFilterValue('osoba', 'datumNastupu'));
        }

        return $sql;
    }

    /**
     * @param Database $database
     * @param int $den
     * @param PravaNaOsoby $prava
     * @param VyrobaFilterForm $filterForm
     * @param string|string[] $typCS
     * @return array
     * @throws DBException
     */
    private static function nactiDenniVysledky($database, $den, $prava, $filterForm, $typCS)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, ". // 0 .. 4
            " DenniVysledekVypoctu.Hodiny, DenniVysledekVypoctu.Dny, DenniVysledekVypoctu.KalendarniDny, DenniVysledekVypoctu.Poznamka " . // 5 .. 8
            "FROM PracovniPomer " .
            " LEFT OUTER JOIN DenniVysledekVypoctu ON (PracovniPomer.RC = DenniVysledekVypoctu.RC AND PracovniPomer.OsobniCislo = DenniVysledekVypoctu.OsobniCislo AND PracovniPomer.DatumNastupu = DenniVysledekVypoctu.DatumNastupu " .
            "   AND den = " . $params->addParameter($database->dateToParam($den)) . ") " .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $filterForm, $den);

        $sql .= " AND " . SQLUtils::getINCondition($database, 'TypCasoveSlozkyMzdy', $typCS, $params);

        $sql .=
            " ORDER BY PracovniPomer.ID, TypCasoveSlozkyMzdy";

        MessageManager::addDebugLog('Denni vysledky: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $vysledkyPP = [];
        $vysledky = [];
        $idPP = 0;

        while ($row = $cursor->fetch()) {
            if ($idPP  &&  $idPP != $row[0]) {
                $vysledkyPP[$idPP] = $vysledky;
                $vysledky = [];
            }

            $vysledek = new DenniVysledek();
            $vysledek->den = $den;
            $vysledek->typCasoveSlozkyMzdy = $row[4];
            $vysledek->hodiny = (float)$row[5];
            $vysledek->dny = (float)$row[6];
            $vysledek->kalendarniDny = (float)$row[7];
            $vysledek->poznamka = $row[8];

            $vysledky[$row[4]] = $vysledek;

            $idPP = $row[0];
        }

        if (!empty($vysledky))
            $vysledkyPP[$idPP] = $vysledky;

        return $vysledkyPP;
    }

    /**
     * @param Database $database
     * @param int $mesic
     * @param PravaNaOsoby $prava
     * @param VyrobaFilterForm $filterForm
     * @param string|string[] $typCS
     * @return array
     * @throws DBException
     */
    private static function nactiMesicniVysledky($database, $mesic, $prava, $filterForm, $typCS)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT PracovniPomer.ID, PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, MesicniVysledekVypoctu.TypCasoveSlozkyMzdy, ". // 0 .. 4
            " MesicniVysledekVypoctu.Hodiny, MesicniVysledekVypoctu.Dny, MesicniVysledekVypoctu.KalendarniDny, MesicniVysledekVypoctu.Poznamka " . // 5 .. 8
            "FROM PracovniPomer " .
            " LEFT OUTER JOIN MesicniVysledekVypoctu ON (PracovniPomer.RC = MesicniVysledekVypoctu.RC AND PracovniPomer.OsobniCislo = MesicniVysledekVypoctu.OsobniCislo AND PracovniPomer.DatumNastupu = MesicniVysledekVypoctu.DatumNastupu " .
            "   AND Mesic = " . $params->addParameter($database->dateToParam($mesic)) . ") " .
            " WHERE 1 = 1" .

            " AND " . static::sqlFiltrOsoby($database, $params, $prava, $filterForm, $mesic, DateUtils::lastDayOfMonth($mesic));

        $sql .= " AND " . SQLUtils::getINCondition($database, 'TypCasoveSlozkyMzdy', $typCS, $params);

        $sql .=
            " ORDER BY PracovniPomer.ID, TypCasoveSlozkyMzdy";

        MessageManager::addDebugLog('Mesicni vysledky: ' . $sql);

        $cursor = $database->getSqlResult($sql, $params);

        $vysledkyPP = [];
        $vysledky = [];
        $idPP = 0;

        while ($row = $cursor->fetch()) {
            if ($idPP != $row[0]) {
                $vysledkyPP[$row[0]] = $vysledky;
                $vysledky = [];
            }

            $vysledek = new MesicniVysledek();
            $vysledek->mesic = $mesic;
            $vysledek->typCasoveSlozkyMzdy = $row[4];
            $vysledek->hodiny = (float)$row[5];
            $vysledek->dny = (float)$row[6];
            $vysledek->kalendarniDny = (float)$row[7];
            $vysledek->poznamka = $row[8];

            $vysledky[$row[4]] = $vysledek;

            $idPP = $row[0];
        }

        if (!empty($vysledky))
            $vysledkyPP[$idPP] = $vysledky;

        return $vysledkyPP;
    }


}
