<?php
/**
 * stanislav.elbl
 * 22.05.2021 0:22
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\UrovenPredavani;
use RonPortal\XML\XMLFormElement;

/**
 * Class XMLElementAction
 * @package RonPortal\XML\Classes
 * Implementace tagu <action> a <button>
 */
class XMLElementAction extends XMLFormElement
{
    function getHTMLOutput()
    {
        // TODO: pokud formular pouziva POST, provest akci
        if ($this ->xmlForm ->readOnly) {
            return '';
        }

        $urovne = $this ->elementName == 'button'
            ? '#all'
            : $this ->getAttributeValue('urovne');

        if ($urovne) {
            // jsou zadane urovne predavani, na kterych je akce zobrazena - zkontroluje, jestli je v nich aktualni uroven
            if ($urovne != '#all') {
                // #all = akce je povolena vzdy
                if (strripos(';' . $urovne . ';', ';' . $this->xmlForm->kodUrovne . ';') === false) {
                    return '';
                }
            }
        } else {
            // urovne nejsou zadane - pokud neni na posledni urovni predavani, akce neni povolena
            if ($this ->xmlForm ->viceSekci) {
                // dokument jeste neni vyplnen, akce se nezobrazi
                return '';
            }

            if (!$this ->xmlForm ->pocatecniUroven) {
                return '';
            }

            if (UrovenPredavani::existujeVyssiUrovenPredavani($this ->xmlForm ->database,
                $this ->xmlForm ->predavaciProces, $this ->xmlForm ->pocatecniUroven,
                $this ->xmlForm ->poradiPredavani))
            {
                return '';
            }
        }

        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));
        $text = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('text')));

        $attrs = [
            'type' => 'button',
            'name' => StringUtils::htmlTextConvert($name),
            'value' => StringUtils::htmlTextConvert($text),
            'class' => 'button xmlaction',
            'data-ron-xmlaction' => StringUtils::htmlTextConvert($name)
        ];

        // dotaz pri kliknuti
        $questionId = $this ->getAttributeValue('questionid');
        if ($questionId) {
            $question = \Ron\ronText($questionId);
        } else {
            $question = StringUtils::fromUTF8($this->getAttributeValue('question'));
        }

        if ($question) {
            $question = $this ->nahradPromenne($question);
        }

        if ($question) {
            $attrs['data-ron-confirm-text'] = StringUtils::htmlTextConvert($question);
        }

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        return
            "<input $attrs_str/>";

        //$confirmquestion = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('question')));
        //$onclickstr = "XMLAction(this, '$confirmquestion', '$name')";
        //return
        //'<input type="button" class="button" OnClick="' . $onclickstr . '" value="'.$text.'" />';
    }


    function writeXMLOutput($xmlWriter)
    {
        if ($this ->xmlForm ->readOnly  ||  $this ->xmlForm ->isError()) {
            parent::copyXML($xmlWriter);
        }

        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));
        if ($name  &&  $name == $this ->xmlForm ->xmlAction) {
            parent::writeXMLOutput($xmlWriter);
        } else {
            parent::copyXML($xmlWriter);
        }
    }


    function process()
    {
        if ($this ->xmlForm ->readOnly) {
            return;
        }

        $name = $this ->nahradPromenne($this ->getAttributeValue('name'));
        if ($name  &&  $name == $this ->xmlForm ->xmlAction) {
            if (!$this ->xmlForm->isError()) {
                $this->xmlForm->addDebugMessage('XML - spustena akce "' . $name . '"');
                $this->xmlForm->addLogData('', StringUtils::fromUTF8('Spuštěna akce ') . $name);
                // provede akci
                parent::process();
            }
        }
    }

}

/*
//DOC
PAGE action
<pre>
PRE
  <action name="jmeno_akce" text="OK" questionid="Opravdu_odeslat" question="Opravdu chcete ...?" urovne="U1;U2">
      ... obsah akce ...
  </action>
PREEND
</pre>

PAGE button
<pre>
PRE
  <button name="jmeno_akce" text="OK" questionid="Opravdu_odeslat" question="Opravdu chcete ...?" urovne="U1;U2">
      ... obsah akce ...
  </button>
PREEND
</pre>
//END
*/