<?php

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElementInput;


/**
 * Implementace tagu <checkbox>
 * Class XMLFormElementCheckbox
 * @package RonPortal\XML\Classes
 */
class XMLElementCheckbox extends XMLFormElementInput
{
    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);
    }


    public function getHTMLOutput()
    {
        $name = $this ->nahradPromenne($this ->name);
        $editing = self::getBoolValue($this ->nahradPromenne($this->getAttributeValue('editing')), true); // yes/no

//        if ($this ->xmlForm ->dataMode()  &&  !$editing  &&  $this ->variableValue($this ->name) !== null) {
//            // datove XML, needitovatelne pole, uz vyplnene
//            return '<span class="dotaznik_odpoved">' . \Ron\ronText($this ->variableValue($this ->name) ? 'Str_Yes' : 'Str_No' )  . '</span>';
//        }

        $attrs = [
            'type' => 'checkbox',
            'name' => $name
        ];

        if ($this ->xmlForm ->readOnly) {
            // ukonceny formular - pole musi byt readonly
            $attrs['disabled'] = 'disabled';
        }
        else if ($this ->xmlForm ->dataMode()  &&  !$editing  &&  $this ->variableValue($this ->name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            $attrs['disabled'] = 'disabled';
        }

        if ($this ->xmlForm ->usePostValues) {
            $checked = (isset($_POST[$name]) && $_POST[$name]) ? $_POST[$name] : 0;
        } else {
            $checked = null;

            if ($this ->xmlForm ->dataMode()) {
                $checked = $this ->variableValue($name);
            }

            if (!isset($checked)) {
                $checked = $this ->nahradPromenne($this ->getAttributeValue('checked'));
            }
        }

        if ($checked) {
            $attrs['checked'] = 'checked';
            $this -> setVariableValue($name, 1);
        } else {
            $this -> setVariableValue($name, 0);
        }

        $attrs = $this ->setHTMLAttributes($attrs);
        $attrs_str = $this ->getHTMLAttributes($attrs);

        $htmlParams = $this->nahradPromenne($this->htmlParams); // u formularu s nastavenym mode se vubec nenacita, takze nelze pouzit!

        return "<input $attrs_str $htmlParams/>";
    }


    public function writeXMLOutput($xmlWriter)
    {
        $name = $this ->nahradPromenne($this ->name);
        $editing = self::getBoolValue($this ->nahradPromenne($this->getAttributeValue('editing')), true); // yes/no

        $checked = isset($_POST[$name])  &&  $_POST[$name];
        if ($checked) {
            $this ->setVariableValue($name, 1);
        } else {
            $this ->setVariableValue($name, 0);
        }

        if ($editing) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter, ['checked' => ($checked ? '1' : '0')]);
            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $checked ? '1' : '0');

            $xmlWriter->startElement('span');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text( StringUtils::toUTF8(\Ron\ronText($checked ? 'Str_Yes' : 'Str_No')) );
            $xmlWriter->endElement();
        }
    }


    public function process()
    {
        $name = $this->nahradPromenne($this->name);
        $editing = self::getBoolValue($this ->nahradPromenne($this->getAttributeValue('editing')), true); // yes/no

        if ($this ->xmlForm ->dataMode()  &&  !$editing  &&  $this ->variableValue($this ->name) !== null) {
            // readonly checkbox - nedela nic
            return;
        }

        $value = (isset($_POST[$name]) && $_POST[$name]) ? 1 : 0;
        $oldValue = $this ->variableValue($name);

        $this ->setVariableValue($name, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }

}


/*
//DOC
PAGE checkbox
<pre>
PRE
  <checkbox name="jmeno" editing="yes" datavariable="1" checked="0" />
PREEND
</pre>

//END
*/
