<?php

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Implementace XML tagu <graphdata> - nacita z databaze data pro graf
 */
class XMLElementGraphData extends XMLFormElementCommand
{
    protected $sql;

    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this -> sql = $this ->getTextContent(false);
    }


    public static function formatLabel($value, $format)
    {

    }

    public function doCommand()
    {
        $this ->processElements();  // zpracuje vnorene elementy (<graphdataset>)

        /** @var XMLElementGraphDataset[] $datasetElements */
        $datasetElements = $this ->findChildrenByName('graphdataset');

        $labelsIndex = $this ->getAttributeValue('labelsindex');

        if (!$datasetElements) {
            // neexistuje zadny podrizeny element <graphdataset> => vytvori ho a nastavi podle hlavniho tagu <graphdata>
            $datasetElement = new XMLElementGraphDataset($this ->xmlForm, $this);
            $datasetElement ->doCommand();
            //$datasetElement ->graphLabelsFormat = $this ->getAttributeValue('graphlabelsformat', 'plain');

            $datasetElements = [$datasetElement];
        }

        $sqlTextVariable = $this ->getAttributeValue('sqltextvariable');

        $sql = $sqlTextVariable ?
            $this -> variableValue($sqlTextVariable) : // convert = false
            $this -> sql;

        $sql = $this -> nahradPromenneSQL($sql);

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql, null, 'TAG graphdata: chyba v dotazu!');

        $graphLabelsFormat = $this ->getAttributeValue('graphlabelsformat', 'plain');
        switch ($graphLabelsFormat) {
            case 'plain' :
            case 'none' :
                $jsLabelsFormat = $graphLabelsFormat;
                break;

            default :
                $jsLabelsFormat = 'ronlabels'; // seznam nadpisu bude predan primo do JSON v polozce data.datasets.0.ronlabels
        }

//        $data = [];    // hodnoty
        $labels = [];  // nadpisy (napr. pro legendu kolacoveho grafu)
//        $graphLabels = [];  // nadpisy primo v jednotlivych oblastech grafu

        while ($row = $cursor ->fetch()) {
            foreach ($datasetElements as $datasetElement) {
                $datasetElement ->loadDataRow($row);
            }

            if (isset($labelsIndex)  &&  count($row) > $labelsIndex) {
                //$labels[] = StringUtils::jsonText($row[$labelsIndex]);
                $labels[] = StringUtils::jsonText($row[$labelsIndex]);
            }
        }

        $graphData = [
            'type' => $this ->nahradPromenne($this ->getAttributeValue('type', 'pie')),
            'data' => [
                'datasets' => [
//                    0 => ['data' => $data, 'ronlabels' => $graphLabels]
                ],
                'labels' => $labels
            ],
            'graphlabelsformat' => $jsLabelsFormat
        ];

        foreach ($datasetElements as $datasetElement) {
            $graphData['data']['datasets'][] = $datasetElement ->getJsonData();
        }

        $graphTitle = $this ->nahradPromenne($this ->getAttributeValue('graphtitle', ''));
        if ($graphTitle !== '') {
            $graphData['options'] = [
                'responsive' => 1,
                'maintainAspectRatio' => false,
                'plugins' => [
                    'title' => [
                        'display' => 1,
                        'text' => $graphTitle
                    ]
                ]
            ];
        }

        $variableName = $this ->nahradPromenne($this ->getAttributeValue('name'));
        $this ->xmlForm ->setVariable($variableName, $graphData);
    }
}


/**
 * Implementace tagu <graphdataset> - muze byt jedine uvnitr tagu graphdata
 */
class XMLElementGraphDataset extends XMLFormElementCommand
{
    public $dataIndex = 0;

    /** @var string     Format textoveho zobrazeni hodnot */
    public $graphLabelsFormat = 'plain';

    /** @var string     Format textoveho zobrazeni hodnot odeslany do js */
    public $jsLabelsFormat = 'plain';

    /** @var array  Data pro graf - mela by to byt cisla */
    public $data;

    /** @var array  Data pro textove zobrazeni hodnot v grafu */
    public $ronLabels;


    public function doCommand()
    {
        $this ->graphLabelsFormat = $this ->getAttributeValue('graphlabelsformat');
        if ($this ->graphLabelsFormat === null) {
            $this ->graphLabelsFormat = $this ->parent ->getAttributeValue('graphlabelsformat', 'plain');
        }

        $this ->dataIndex = (int)$this ->getAttributeValue('dataindex', 0);

        $this ->data = [];
        $this ->ronLabels = [];

        switch ($this ->graphLabelsFormat) {
            case 'plain' :
            case 'none' :
                $this ->jsLabelsFormat = $this ->graphLabelsFormat;
                break;

            default :
                $this ->jsLabelsFormat = 'ronlabels'; // seznam nadpisu bude predan primo do JSON v polozce data.datasets.0.ronlabels
        }

        // label
        // data
        // graphlabelsformat
        // graphLabels
        // backgroundColor
    }

    public function loadDataRow($row)
    {
        $this ->data[] = $row[$this ->dataIndex];

        if ($this ->jsLabelsFormat === 'ronlabels') {
            $this ->ronLabels[] = StringUtils::format($this ->graphLabelsFormat, $row[$this ->dataIndex]);
        }

    }

    public function getJsonData()
    {
        $result = [
            'data' => $this ->data,
            'ronlabels' => $this ->ronLabels
        ];

        // Nadpis datasetu (zobrazi se napr. v legende sloupcoveho grafu)
        $attrs = [];

        $label = $this ->nahradPromenne($this ->getAttributeValue('label', ''));
        if ($label) {
            $attrs['label'] = StringUtils::fromUTF8($label);
        }

        $attrs = $this ->setHTMLAttributes($attrs);

        foreach ($attrs as $attrName => $attrValue) {
            $result[$attrName] = StringUtils::jsonText($attrValue);
        }

        return $result;
    }
}

/*
//DOC
PAGE graphdata
<pre>
PRE
  <graphdata name="jmeno_promenne" type="typ grafu" sqltextvariable="jmeno_promenne_s_sql_dotazem" graphtitle="titulek grafu" graphlabelsformat="format_nadpisu" labelsindex="index_nadpisu_v_sql_dotazu">
    <graphdataset label="Nadpis datasetu" dataindex="index_dat_v_sql_dotazu" graphlabelsformat="format_nadpisu"/>
    ...
  </graphdata>
PREEND
</pre>

//END
*/
