<?php
/**
 * stanislav.elbl, 22.06.2021 13:30
 */

namespace RonPortal\XML\Classes;

use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\XML\XMLFormElementCommand;
use RonPortal\XML\XmlFormException;

class XMLElementMessage extends XMLFormElementCommand
{
    var $fieldName;
    var $text;


    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this ->fieldName = $this ->getAttributeValue('fieldname');
        $this ->text = $this ->getAttributeValue('text');

        if (!isset($this ->text)) {
            $this ->text = $this ->getTextContent(true);
        } else {
            $this ->text = StringUtils::fromUTF8($this ->text);
        }
    }


    function doCommand()
    {
        $fieldName = $this ->nahradPromenne($this ->fieldName);
        $text = $this ->nahradPromenne($this ->text);

        switch ($this ->elementName) {
            case 'exception' :
                throw new XmlFormException($text, $fieldName);

            case 'error' :
            case 'errormsg' :
            case 'errormessage' :
                $this ->xmlForm ->addErrorMsg( $text, $fieldName, $this ->nahradPromenne($this ->getAttributeValue('id')) );
                break;

            case 'infomsg' :
            case 'infomessage' :
                MessageManager::addInfoMessage($text);
                break;

            case 'successmsg' :
                MessageManager::addSuccessMessage($text);
                break;

            case 'warning' :
                MessageManager::addWarning($text, $fieldName);
                $lzePredat = $this ->nahradPromenne($this ->getAttributeValue('lzepredat'));
                if ($lzePredat != '') {
                    $this ->xmlForm ->lzePredat = (int)$lzePredat;
                }
                break;

            case 'debugmsg' :
            case 'debugmessage' :
                $this ->xmlForm ->addDebugMessage($text);
                break;

            case 'debuglog' :
                $this ->xmlForm ->addDebugMessage($text, true);
                break;
        }
    }
}

/*
//DOC
PAGE error
<pre>
PRE
  <error fieldname="jmeno_formularoveho_pole" text="..." id="id_chyby">Text ...</error>
PREEND
</pre>

PAGE exception
<pre>
PRE
  <exception fieldname="jmeno_formularoveho_pole" text="...">Text ...</exception>
PREEND
</pre>

PAGE infomsg
<pre>
PRE
  <infomsg fieldname="jmeno_formularoveho_pole" text="..." id="id_chyby">Text ...</infomsg>
PREEND
</pre>

PAGE warning
<pre>
PRE
  <warning fieldname="jmeno_formularoveho_pole" text="..." lzepredat="1">Text ...</warning>
PREEND
</pre>

PAGE debugmsg
<pre>
PRE
  <debugmsg fieldname="jmeno_formularoveho_pole" text="...">Text ...</debugmsg>
PREEND
</pre>

PAGE debuglog
<pre>
PRE
  <debuglog fieldname="jmeno_formularoveho_pole" text="...">Text ...</debuglog>
PREEND
</pre>


//END
*/