<?php
/**
 * stanislav.elbl, 23.06.2021 17:45
 */

namespace RonPortal\XML\Classes;

use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\XML\XMLFormElementLoader;
use RonPortal\Zamestnanci\NovyZamestnanecForm;

class NovyZamestnanecFormXML extends NovyZamestnanecForm
{
    public $usePostValuesFromXML;

    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
//echo " loaduju data -{$usePostValues}- ";
        parent::loadFieldValues($insert, $values, $this ->usePostValuesFromXML ?: $usePostValues, $canEditRecord);
    }
}


class XMLElementNastupniFormular extends XMLFormElementLoader
{
    /** @var NovyZamestnanecFormXML */
    private static $form = null;

    /**
     * Vytvori formular
     * @return NovyZamestnanecFormXML
     * @throws \Exception
     */
    private function createForm()
    {
        TextManager::getInstance() ->loadStrings('zamestnanci.ini');
        HtmlHeaders::getInstance() ->addCss('css/person.css');
        HtmlHeaders::getInstance() ->addScript('script/personscripts.js');

        $idPP = (int)$this ->nahradPromenne($this ->getAttributeValue('idpp'));
        $pp = $idPP
            ? PracovniPomer::nactiPP($this ->xmlForm ->database, $idPP)
            : '';

        $form = new NovyZamestnanecFormXML($this ->xmlForm ->database,
            $pp ? 'edit' : 'new',
            $pp ? $pp ->idPP : null,
            true);  // zobrazeno z XML = true

        $form ->title = ''; // zobrazuje se nadpis z XML formulare
        if ($this ->xmlForm ->readOnly) {
            $form ->readOnly = true;
        }
        $form ->setNovyZamestnanecConfiguration($this ->replaceContentVariables());

        if ($this ->xmlForm ->usePostValues) {
//echo " nastavuji usePostValuesFromXML ";
            $form ->usePostValuesFromXML = $this ->xmlForm ->usePostValues;
        }

        return $form;
    }


    private function createVariables()
    {
        $name = $this ->getAttributeValue('name');

        if (!$name) {
            return;
        }

        $this ->setVariableValue($name . '_status', self::$form ->getStatus()); // 'rc' nebo 'all'
        $this ->setVariableValue($name . '_status_' . self::$form ->getStatus(), 1);
    }


    function getHTMLOutput()
    {
//        HtmlHeaders::getInstance()
//            ->addScript('script/vyroba.js')
//            ->addCss('css/dochazka.css');
        if (!self::$form) {
//echo " getHTMLOutput v XML nastupaku ";
            self::$form = $this->createForm();
            self::$form ->handleAction();

            $this ->createVariables();
        }

        $this ->xmlForm ->viewObjects[] = self::$form;

        return $this ->xmlForm ->viewObjectPlaceholder;
    }


    function process()
    {
        self::$form = $this ->createForm();
        $result = self::$form ->handleAction();

        if (self::$form ->noveDokumenty) {
            array_push($this ->xmlForm ->noveDokumenty, ...self::$form ->noveDokumenty);
        }

        $this->createVariables();

        if (!$result) {
            $this ->xmlForm ->addErrorMsg(''); // aby pokracoval dale - formular jeste nebyl kompletne vyplnen
        } else {
            try {
                if (MessageManager::getInstance() ->isError()) {
                    // Pravdepodobne chybi vyplnene povinne udaje (s required="2"). Ulozeni probehlo, ale formular se nesmi odeslat dale
                    if ($this ->xmlForm ->lzePredat) {
                        $this->xmlForm->lzePredat = 2; // jde jen vratit
                    }
                }
                //if (!$this ->xmlForm ->pp_RC  &&  self::$form ->getIdPP()) {  // byl vlozen novy zaznam
                if (self::$form->getIdPP()) {
                    // PP nastavuje i pri editaci, aby fungovala editace osobniho cisla
                    $this->xmlForm->nastavPracovniPomer('NASTUPNI_FORMULAR_PP', self::$form->getIdPP());
                }
            } catch (\Exception $e) {
                $this ->xmlForm ->addErrorMsg($e);
            }
        }
    }
}

/*
//DOC
PAGE nastupniformular
<pre>
PRE
  <nastupniformular />
PREEND
</pre>

//END
*/
