<?php
/**
 * stanislav.elbl, 24.05.2021 20:20
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLForm;
use RonPortal\XML\XMLFormElementInput;

class XMLElementRadio extends XMLFormElementInput
{

    var $value;
    var $checked;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->value = $this->getAttributeValue('value');
        $this->checked = $this->getAttributeValue('checked');
    }


    function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);

        $attrs = [
            'type' => 'radio',
            'name' => $name
        ];

        $value = $this->nahradPromenne(StringUtils::fromUTF8($this->value));
        $checked = $this->nahradPromenne($this->checked);

        $attrs['value'] = $value;

        $dataValueExists =
            $this ->xmlForm ->mode == XMLForm::XML_MODE_DATA  &&
            isset($this ->xmlForm ->dataVariables[$name]);

        $editing = $this ->nahradPromenne($this->getAttributeValue('editing'));
        $readOnly = $dataValueExists  &&  ($editing == 'no' || $editing == '0' || $editing == 'false' || $editing == 'remove');

        if ($readOnly  ||  $this->xmlForm->readOnly) {
            $attrs['disabled'] = true;
        }

        if (!$readOnly  &&  $this->xmlForm->usePostValues && isset($_POST[$name])) {
            $checked = ($_POST[$name] == $value) ? 1 : 0;
        } else if ($this ->xmlForm ->dataMode()) {
            // datove xml
            $selectedValue = $this ->variableValue($name);
            if (isset($selectedValue)) {
                $checked = $selectedValue == $value;
            }
        }

        if ($dataValueExists  &&  $editing == 'remove'  &&  !$checked) {
            return '';
        }

        if ($checked) {
            $this->setVariableValue($name, $value);
            $attrs['checked'] = 'checked';
        }

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        $htmlParams = $this->nahradPromenne($this->htmlParams); // u formular s nastavenym mode neni podporovan (vubec se nenacita)

        $result =
            "<input $attrs_str $htmlParams/>" .
            $this ->getElementsHTMLOutput(); // text k radiobuttonku muze byt vnoreny (umoznuje to zadat editing="remove")

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        // editing="yes", "no", "remove" - nahradi se jen textem
        $name = $this->nahradPromenne($this->name);
        $value = $this->nahradPromenne(StringUtils::fromUTF8($this->value));
        $checked = (isset($_POST[$name])  &&  ($_POST[$name] == $value)) ? 1 : 0;
        if ($checked) {
            $this->setVariableValue($name, $value);
        }

        $editing = $this ->nahradPromenne($this->getAttributeValue('editing'));

        if ($editing == 'remove') {
            // checkboxy se maji odstranit
            if ($checked) {
                $this ->writeXmlVariable($xmlWriter, $name, $value);

                $xmlWriter->startElement('span');
                $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
                $this->writeXMLElements($xmlWriter);
                $xmlWriter->endElement();
            }

        } else {
            $attrs = ['checked' => ($checked ? '1' : '0')];

            if ($editing == 'no' || $editing == '0' || $editing == 'false') {
                if (!$checked) {
                    $attrs['html:disabled'] = 'disabled';
                }
            }

            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter, $attrs);
            $this->writeXMLElements($xmlWriter);
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $name = $this->nahradPromenne($this->name);

        $dataValueExists =
            $this ->xmlForm ->mode == XMLForm::XML_MODE_DATA  &&
            isset($this ->xmlForm ->dataVariables[$name]);

        $editing = $this ->nahradPromenne($this->getAttributeValue('editing'));

        if ($dataValueExists  &&  ($editing == 'no' || $editing == '0' || $editing == 'false' || $editing == 'remove')) {
            // nedela nic - hodnota uz byla vyplnena v predchozim kroku formulare
        } else if (isset($_POST[$name]) &&  ($_POST[$name] == $this ->value)) {
            // zaskrtnuta tato volba
            $oldValue = $this->variableValue($name);

            $this->setVariableValue($name, $this ->value);
            if ($this ->dataVariable) {
                $this->xmlForm->addDataVariable($name);
                if ($this ->value != $oldValue) {
                    // zaloguje zmenu
                    $this->xmlForm->addLogData($name, $this ->value);
                }
            }
        }
    }
}

/*
//DOC
PAGE radio
<pre>
PRE
  <radio name="jmeno" value="hodnota" editing="yes" datavariable="1" checked="0" />
PREEND
</pre>

//END
*/
