<?php
/**
 * stanislav.elbl, 16.06.2021 15:48
 */

namespace RonPortal\XML\Classes;

use Ron\DB\DBRecord;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\XML\XMLFormElementCommand;

class XMLElementRecord extends XMLFormElementCommand
{
    /** @var RonPortalDBRecord */
    var $dbRecord;

    public function __construct($xmlForm, $parent)
    {
        parent::__construct($xmlForm, $parent);
        $this ->processNestedElements = true;
    }


    function doCommand()
    {
        $name = $this ->getAttributeValue('name');
        $logDescription = $this->getAttributeValue('logdescription');
        $tableName = $this->getAttributeValue('tablename');

        if ($name  &&  !$tableName) {
            // nacte jiz existujici DBRecord
            $this ->dbRecord = $this ->variableValue($name);
            if (!($this ->dbRecord instanceof DBRecord)) {
                throw new \Exception("Chyba v tagu record! Promenna $name neexistuje nebo neobsahuje DBRecord!");
            }
        } else {
            $this->dbRecord = new RonPortalDBRecord($this->xmlForm->database,
                $tableName,
                $logDescription ?: 'XML ' . $this->xmlForm->idSablony);

            $this->dbRecord ->checkMultiple = $this ->getBoolAttribute('checkmultiple', true);

            // pokud je zadano jmeno promenne, nastavi do ni dbrecord
            if ($name) {
                $this->setVariableValue($name, $this->dbRecord);
            }
        }
        //parent::elementsEvent();
    }
}



class XMLElementRecordPart extends XMLFormElementCommand
{
    var $wherePart;
    var $valueText;

    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        switch ($this->elementName) {
            case 'recordwhere' :
                $this->wherePart = $this->getTextContent(false);
                break;

            case 'recordfield' :
                $this->valueText = $this->getAttributeValue('value');

                if (!isset($this ->valueText)) {
                    $this ->valueText = $this->getTextContent(false);
                } else {
                    $this ->valueText = StringUtils::fromUTF8($this ->valueText);
                }
        }
    }


    function doCommand()
    {
        /* @var  XMLElementRecord $recordElement */
        $recordElement = $this->findParentByName('record');

        if (!$recordElement) {
            MessageManager::addDebugMessage('Chyba v tagu ' . $this ->elementName . ': nenalezen nadrazeny tag record!');
            return;
        }

        switch ($this->elementName) {
            case 'recordfield' :
                $readOnly = self::getBoolValue($this ->getAttributeValue('readonly'));
                $value = $this->nahradPromenne($this ->valueText);
                $name = $this->nahradPromenne($this ->getAttributeValue('name'));

                $fieldFormat = $this->getAttributeValue('format');

                switch ($fieldFormat) {
                    case 'float' :
                    case 'double' :
                        $value = is_string($value)
                            ? ($value === '' ? null : StringUtils::stringToFloat($value))
                            : (float)$value;
                        break;

                    case 'int':
                    case 'integer':
                        $value = ($value === '' ? null : (int)$value);
                        break;
                }

                $field = null;

                if ($this ->getBoolAttribute("updateexistingfield")) {
                    // tag ma nastaveny atribut updateexisting, takze zkontroluje, jestli uz pole existuje
                    $field = $recordElement->dbRecord->findFieldByName($name);
                    if ($field) {
                        if ($this ->getAttributeValue('readonly') !== null) {
                            $field ->readOnly = $readOnly;
                        }

                        if ($this ->getAttributeValue('value') !== null) {
                            $field ->setValue($value);
                        }

                        if ($this ->getAttributeValue('size') !== null) {
                            $field ->setSize((int)$this ->getAttributeValue('size'));
                        }
                    }
                }

                if ($fieldFormat === 'serial') {
                    $fieldFormat = 'autoserial'; // automaticky pouzije sekvenci, pokud existuje
                } else if ($fieldFormat === 'serial!') {
                    // pokud by nekde bylo potreba pouzit serial pro jine nez primarni pole v tabulce, ktera ma sekvenci
                    $fieldFormat = 'serial';
                }

                if (!$field) {
                    $field = $recordElement->dbRecord->addField(
                        $name,
                        $fieldFormat,
                        $readOnly,
                        (int)$this->getAttributeValue('size'),
                        $value);
                }

                // priznaky pro logovaci pole (napr. Update_UserName)
                $field ->updateLog = $this ->getAttributeValue('updatelog');

                break;

            case 'recordwhere' :
                $where = $this ->nahradPromenneSQL($this ->wherePart);
                $recordElement ->dbRecord ->where = $where;
                break;

            case 'loadrecord' :
                $varName = $this ->getAttributeValue('name');

                $this ->setVariableValue($varName, $recordElement ->dbRecord);

                $recordElement ->dbRecord ->loadRecord();
                foreach ( $recordElement ->dbRecord ->fields as $field ) {
                    $this ->setVariableValue($varName . '_' . ($field ->fieldName), $field ->originalValue);
                }
                break;

            case 'insertrecord' :
                $recordElement ->dbRecord ->insertRecord();
                break;

            case 'updateorinsertrecord' :
                $recordElement ->dbRecord ->updateOrInsert();
                break;

            case 'deleterecord' :
                $recordElement ->dbRecord ->deleteRecord();
                break;

            case 'updaterecord' :
            case 'editrecord' :
                if ($recordElement->dbRecord->loadRecord()) {
                    $recordElement->dbRecord->updateRecord();
                } else {
                    if ($recordElement ->dbRecord ->checkMultiple) {
                        // TODO: vyhodit vyjimku? Umoznit potlacit hlasku?
                        MessageManager::addErrorMessage('Chyba pri nacitani puvodniho zaznamu!');
                    }
                }
        }
    }
}


/*
//DOC
PAGE record
<pre>
PRE
  <record tablename="JMENOTABULKY" logdescription="Popis pro zapis do logu" name="jmeno promenne">
    <recordfield name="JMENOPOLE1" format="FORMAT_POLE1" size="DELKA_TEXTOVEHO_POLE" readonly="0" updatelog="0" checkmultiple="1" value="HODNOTA">JINAK_ZADANA_HODNOTA</recordfield>
    ...

    <recordwhere>SQL podminka pro SELECT a UPDATE</recordwhere>

    <insertrecord />
    <updaterecord />
    <updateorinsertrecord />

    <deleterecord />

    <loadrecord name="JMENO_PROMENNE" />

  </record>
PREEND
</pre>
//END
*/
