<?php
/**
 * stanislav.elbl, 22.06.2021 23:09
 */

namespace RonPortal\XML\Classes;

use Ron\MVC\Redirect;
use RonPortal\XML\XMLFormElementCommand;

class XMLElementRedirect extends XMLFormElementCommand
{
    /*
     * <redirect url="..."> - redirect na zadanou url
     * <redirect urlparams="..."> - obecny redirect na jinou stranku v portalu, napr. urlparams="menuid=dochazkazamestnance"
     * <redirect sablona="..." params="...">  - na zadanou sablonu
     * <redirect xmlid="..." params="...">  - na zadane vyplnene xml
     * <redirect> - sam na sebe
     * <redirect params="....">  - sam na sebe (plati i pro sablonu i pro vyplneny dotaznik) + zadane parametry
     */
    function getRedirect()
    {
        if ($value = $this ->getAttributeValue('value') != null) {
            if (!$value) {
                return null;
            }
        }

        $urlParams = $this ->getAttributeValue('urlparams');
        if ($urlParams) {
            //$urlParams = $this ->nahradPromenne($urlParams, 'html'); // kdyz se pouzije format html, tak nelze vlozit &
            $urlParams = $this ->nahradPromenne($urlParams, 'text'); // TODO: udelat format 'url' ?
        }

        if ($url = $this ->getAttributeValue('url')) {
            // obecny redirect kamkoliv
            $url = $this->nahradPromenne($url, 'html');
            return new Redirect($url, $urlParams);
        }

        if ($urlParams) {
            // redirect na jakoukoliv stranku portalu
            return new Redirect(null, $urlParams);
        }

        $idSablony = $this ->xmlForm ->idSablony;

        if ($xmlId = $this ->getAttributeValue('xmlid')) {
            // redirect na zadane xml
        } else if ($hlpId = $this ->getAttributeValue('sablona')) {
            // redirect na zadanou sablonu
            $idSablony = $hlpId;
        } else if ($xmlId = $this ->xmlForm -> id) {
            // vyplneny dotaznik - redirect na sebe
        }

        $params = $this ->nahradPromenne($this ->getAttributeValue('params'), 'html');
        $params = $params ? '&' . $params : '';

        if ($params) {
            if ($xmlId) {
                // uz vyplneny formular
                return new Redirect(null, '?menuid=editovatdotaznik&record=' . $xmlId . $params);
            }

            return new Redirect(null, '?menuid=xml&record=' . $idSablony . $params);
        }

        // pri prvnim vyplnovani predavaci sablony jeste neni prirazen Id vyplneneho formulare - nelze jeste vytvorit Redirect
        return null;
    }


    function doCommand()
    {
        $redirect = $this ->getRedirect();

        if ($redirect) {
            $this->xmlForm->redirect = $redirect;
        } else {
            $this->xmlForm->pokracovat = true;
        }
//        switch ($this ->elementName) {
//            case 'pokracovat' :
//                $value = $this->getAttributeValue('value');
//                if (isset($value)) $value = (int)$this->nahradPromenne($value);
//                else $value = 1;
//                $this ->xmlForm ->pokracovat = $value;
//                break;
//
//            case 'redirect' :
//                $this ->xmlForm ->redirect = $this;
//                break;
//
//            default :
//                throw new \Exception("Element {$this ->elementName} neni implementovan!");
//        }
    }
}

/*
//DOC
PAGE redirect
<pre>
PRE
  <redirect sablona="id_xml_sablony" xmlid="id_vyplneneho_formulare" params="url_parametry" url="url" urlparams="url_parametry" value="0"/>
PREEND
</pre>
//END
*/
