<?php
/**
 * User: stanislav.elbl
 * Date: 20.05.2021 16:53
 */

namespace RonPortal\XML\Classes;

use Ron\Http\Csrf;
use RonPortal\User;
use RonPortal\XML\XMLForm;
use RonPortal\XML\XMLFormElement;

/**
 * Korenovy tag
 *      ronform - generuje HTML <form>
 *      ronxml  - negeneruje HTML <form>
 *      ronxmldata  - obsah vyplneneho datoveho formulare
 *      xmlform - musi byt uvnitr formulare (uvnitr <ronxml>)
 */
class XMLElementRonForm extends XMLFormElement
{
    var $isForm; // true pro ronform, jinak false
    var $mode;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->mode = $this->getAttributeValue('mode');

        if ($this->elementName === 'xmlform') {
            // tento tag muze byt pouze uvnitr formulare
            $this->isForm = true;
            if (!$this->parent) {
                throw new \Exception('Tag xmlform muze byt pouze uvnitr formulare!');
            }
        } else if ($this ->mode === 'include') {
            if (!$this->parent) {
                throw new \Exception('Formular s mode="include" nemuze byt pouzivan samostatne!');
            }
        } else {
            $this->isForm = $this->elementName === 'ronform';

            $this->xmlForm->neukladat = (int)$this->getAttributeValue('neukladat'); // zastarale - nahrazeno mode="show"
            $this->xmlForm->verze = $this->getAttributeValue('verze'); // zastarale, formulare s mode="..." maji automaticky nastaveno verze="2"

            $this ->xmlForm ->showHeader = $this ->getAttributeValue('showheader') ?? $this ->xmlForm ->showHeader;

            $this->xmlForm->mode = $this->mode;
            if ($this->xmlForm->mode || $this->elementName === 'ronxmldata')  {
                $this->xmlForm->verze = 2;
            }

            $this->xmlForm->enableJson = $this->getBoolAttribute('enablejson');

            $this->xmlForm->passwordProtected = $this ->getBoolAttribute('passwordprotected');
        }
    }


    private function getCsrfRequiestId()
    {
        return $this ->xmlForm ->id
            ? 'xml_form_' . $this ->xmlForm ->id
            : 'xml_sablona_' . $this ->xmlForm ->idSablony;
    }


    function getHTMLOutput()
    {
        if (!$this->isForm) {
            return $this->getElementsHTMLOutput();
        }


        if ($this ->xmlForm ->neukladat == 2  && isset($_POST['xmlformpost'])  &&  (int)$_POST['xmlformpost']) {
            // u nevyplnovaciho formulare, ktery ma neukladat="2" pouzije post, pokud byl formular odeslan
            $this ->xmlForm ->usePostValues = true;
        }

        if (!User::userAuthenticated()) {
            if ($this->xmlForm->mode != XMLForm::XML_MODE_PROCESS && $this->xmlForm->mode != XMLForm::XML_MODE_SHOW) {
                throw new \Exception('Formular login_page musi mit mod PROCESS nebo SHOW!');
            }
            $action = '?menuid=xmllogin&action=process&record=' . $this->xmlForm->idSablony;
        } else if ($this ->xmlForm ->formAction) {
            $action = "?menuid={$this ->xmlForm ->formAction}&record={$this->xmlForm->idSablony}";
        } else if ($this ->xmlForm ->mode == XMLForm::XML_MODE_PROCESS) {
            //$action = '?menuid=processxml&record=' . $this->xmlForm->idSablony;
            $action = null; // odesila na stejnou URL
        } else if ($this ->xmlForm ->mode == XMLForm::XML_MODE_SHOW  ||  $this->xmlForm->neukladat) {
            //$action = '?menuid=xml&record=' . $this->xmlForm->idSablony;
            $action = null; // odesila na stejnou URL
        } else {
            $action = null; // odesila na stejnou URL
        }


        $attrs = [
            'method' => 'post',
            'enctype' => 'multipart/form-data',
            'class' => 'submitWorkingInfo'
        ];

        if (isset($action)) {
            $attrs['action'] = $action;
        }
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        $result = "<form $attrs_str>";

        if (!$this->xmlForm->readOnly) {
            $result .=
                "<input type='hidden' name='ronxmlform' value='1' />" . // doplneno v nove verzi - jen pro testovani ? ? ?
                "<input type='hidden' name='id' value='{$this -> xmlForm -> id}'/>" .
                "<input type='hidden' name='idsablony' value='{$this -> xmlForm -> idSablony}'/>" .
                //"<input type='hidden' name='typdotazniku' value='{$this -> xmlForm -> f_typ}'/>" .
                "<input type='hidden' name='xmlformpost' value='1'/>". // priznak, ze byl formular odeslan POSTem
                "<input type='hidden' name='formaction' value=''/>" .
                "<input type='hidden' name='xmlaction' value='submit'/>" . // xml akce - naplni se pri stisku buttonu. Vychozi je submit, aby fungoval na odeslani enterem
                Csrf::csrfHiddenInput($this ->getCsrfRequiestId());

            if (!empty($this->xmlForm->hiddenFields)) {
                foreach ($this->xmlForm->hiddenFields as $key => $value) {
                    $result .= "<input type='hidden' name='{$key}' value='{$value}'>";
                }
            }

            if (!$this ->xmlForm ->mode  &&  $this->xmlForm->neukladat) {
                // pro kompatibilitu starych xml s neukladat="..." bez nastaveneho mode
                $result .=
                    "<input type='hidden' name='neukladat' value='1'/>";
            }
        }

        $result .= $this->getElementsHTMLOutput();

        $result .= '</form>';

        return $result;
    }


    function process()
    {
        if ($this ->isForm) {
            Csrf::validateCSRFPostParam($this->getCsrfRequiestId());
        }
        parent::process();
    }


}
