<?php

namespace RonPortal\XML\Classes;

use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\XML\XMLFormElementInput;

/**
 * Implementace tagu <sqlselect>
 * Class XMLFormElementSQLSelect
 * @package RonPortal\XML\Classes
 */
class XMLElementSQLSelect extends XMLFormElementInput
{
    protected $sql;
    protected $sqlTextVariable;
    protected $format;
    protected $value;
    protected $showCode;

    public function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this -> value = $this ->getAttributeValue('value');
        $this -> sqlTextVariable = $this ->getAttributeValue('sqltextvariable');
        $this -> format = $this ->getAttributeValue('format');
        $this -> showCode = self::getBoolValue($this ->getAttributeValue('showcode'));

        $this -> sql = $this ->getTextContent(false);
    }


    /**
     * @throws DBException
     */
    private function loadSQLData($name)
    {
        $sql = $this -> sqlTextVariable ?
            $this -> variableValue($this -> sqlTextVariable) : // convert = false
            $this -> sql;

        $sql = $this -> nahradPromenneSQL($sql);
        $this ->xmlForm ->addDebugMessage("tag sqlselect, name: $name, sql: $sql", true);

        return $this ->xmlForm ->database ->getSqlResult($sql, null, 'TAG sqlselect: chyba v dotazu!');
    }


    /**
     * @throws DBException
     * @throws Exception
     */
    public function getHTMLOutput()
    {
        $name = $this ->nahradPromenne($this ->name);
        $editing = self::getBoolValue($this ->nahradPromenne($this ->getAttributeValue('editing')));

        $attrs = ['name' => $name];

        if ($this->xmlForm->readOnly) {
            $attrs['disabled'] = true;
        }

        if ( $this -> xmlForm -> usePostValues  &&  isset($_POST[$name]) ) {
            $value = $_POST[$name];
        } else if ($this ->xmlForm ->dataMode()  &&  $this ->variableValue($name) !== null) {
            // datove xml - hodnota uz je ulozena v datech
            $value = $this ->variableValue($name);

            if (!$editing) {
                // needitovatelny, uz vyplneny select v datovem formulari
                return '<span class="dotaznik_odpoved">' . $this ->variableValue($name . '_text') . '</span>';
            }
        } else {
            // vychozi hodnota zadana ve formulari nebo drive vyplnena hodnota
            $value = $this -> nahradPromenne(StringUtils::fromUTF8($this -> value));
        }

        if ( $this ->xmlForm ->isError($name)) {
            $attrs['class'] = 'dotaznik_errinput';
        }

        $cursor = $this ->loadSQLData($name);

        $htmlParams = $this -> nahradPromenne($this -> htmlParams);

        $attrs = $this ->setHTMLAttributes($attrs);
        $attrs_str = $this ->getHTMLAttributes($attrs);

        $result = "<select $attrs_str $htmlParams>";

        if ( !isset($value)  ||  !$this -> required ) { // pokud jeste nebyla zadana hodnota nebo pole neni povinne, prida prazdnou hodnotu
            $selectedstr = $value ? '' : 'selected="selected"';
            $result .= "<option value='' $selectedstr>&nbsp;</option> ";
        }

        while ($row = $cursor ->fetch()) {
            $rowval = $this ->formatSqlValue($this -> format, $row[0]);
            // formatovatni (format timestamp na cislo, ostatni standardne na retezec)

            if ( $rowval == $value ) {
                $selectedstr = 'selected="selected"';
                $this -> setVariableValue($name, $value);
                $this -> setVariableValue($name.'_text', $row[1]);
            } else {
                $selectedstr = '';
            }

            $text = StringUtils::htmlTextConvert($row[1]);
            if ($this -> showCode) $text .= ' ('.$rowval.')';

            $result .= "<option value='$rowval' $selectedstr>$text</option> ";
        }

        $cursor ->close();

        $result .= "</select>\n";

        return $result;
    }


    public function writeXMLOutput($xmlWriter)
    {
        $name = $this ->nahradPromenne($this ->name);

        if ($this ->required && $_POST[$name] == '') {
            $this ->xmlForm->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $cursor = $this ->loadSQLData($name);

        $found = false;

        $kod = null;
        $popis = null;
        while (!$found && $row = $cursor ->fetch()) {
            $rowval = $this ->formatSqlValue($this -> format, $row[0]);

            if ($rowval == StringUtils::htmlTextConvert($_POST[$name])) {
                $kod = $rowval;
                $popis = $row[1];
                $found = true;
            }
        }
        $cursor ->close();

        $this -> setVariableValue($name, $kod);
        $this -> setVariableValue($name.'_text', $popis);

        if (self::getBoolValue($this ->nahradPromenne($this ->getAttributeValue('editing')))) {
            $xmlWriter->startElement(
                ($this->prefix ? $this->prefix . ':' : '') .
                $this->elementName
            );

            $this->writeXMLAttributes($xmlWriter, ['value' => $kod]);

            $this->writeXMLElements($xmlWriter);

            $xmlWriter->endElement();
        } else {
            $this ->writeXmlVariable($xmlWriter, $name, $kod);
            $this ->writeXmlVariable($xmlWriter, $name . '_text', $popis);
            $xmlWriter ->startElement('span');
            $xmlWriter ->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter ->text( StringUtils::toUTF8($popis) );
            $xmlWriter ->endElement();
        }
    }


    public function process()
    {
        $name = $this ->nahradPromenne($this ->name);
        $editing = self::getBoolValue($this ->nahradPromenne($this ->getAttributeValue('editing')));

        if (!$editing  &&  $this ->variableValue($name) !== null) {
            // neni editovatelny a uz ma vyplnenou hodnotu
            return;
        }

        $oldValue = $this ->variableValue($name);

        if ($this ->required  &&  $_POST[$name] == '') {
            $this ->xmlForm ->addErrorMsg(\Ron\ronText('Chybi_povinna_hodnota'), $name);
        }

        $cursor = $this ->loadSQLData($name);

        $found = false;
        $value = null;
        $valueText = null;

        while (!$found  &&  $row = $cursor ->fetch()) {
            $rowVal = $this ->formatSqlValue($this ->format, $row[0]);

            if ($rowVal == $_POST[$name]) {
                $value = $rowVal;
                $valueText = $row[1];

                $found = true;
            }
        }

        $cursor ->close();

        $this ->setVariableValue($name, $value);
        $this ->setVariableValue($name . '_text', $valueText);

        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
            $this->xmlForm->addDataVariable($name . '_text');

            if ($value != $oldValue) {
                $this->xmlForm->addLogData($name, $value . ' (' . $valueText . ')');
            }
        }
    }
}

/*
//DOC
PAGE sqlselect
<pre>
PRE
  <sqlselect name="jmeno" format="" required="0" editing="yes" datavariable="1" value="" showcode="0" sqltextvariable="jmeno_promenne">SQL dotaz SELECT ...</sqlselect>
PREEND
</pre>

//END
*/
