<?php
/**
 * stanislav.elbl
 * 21.05.2021 20:31
 */

namespace RonPortal\XML\Classes;


use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\UrovenPredavani;
use RonPortal\XML\XMLForm;
use RonPortal\XML\XMLFormElement;


class XMLElementSubmit extends XMLFormElement
{
    private function getInputHtml($xmlAction, $defaultText)
    {
        // text tlacitka
        $textId = $this ->getAttributeValue('textid');
        $text = $textId
            ? TextManager::_($this ->nahradPromenne($textId))
            : $this ->nahradPromenne(StringUtils::fromUTF8($this->getAttributeValue('text')));

        if (!$text  &&  $defaultText) {
            $text = $defaultText;
        }

        $attrs = [
            'type' => 'button',
            'value' => StringUtils::htmlTextConvert($text),
            'class' => 'button xmlaction',
            'data-ron-xmlaction' => $xmlAction
        ];

        // dotaz pri kliknuti
        $questionId = $this ->getAttributeValue('questionid');
        $question = $questionId
            ? TextManager::_($this ->nahradPromenne($questionId))
            : $this ->nahradPromenne(StringUtils::fromUTF8($this->getAttributeValue('question')));

        if ($question) {
            $attrs['data-ron-confirm-text'] = StringUtils::htmlTextConvert($question);
        }

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        return
            "<input $attrs_str/>";
    }


    function getHTMLOutput()
    {
        if ( $this ->xmlForm -> readOnly ) {
            return '';
        }

        switch ($this ->elementName) {
            case 'ulozit' :
                return $this ->getInputHtml('submit', TextManager::_('form.Store'));

            case 'predatdale' :
                if ($this ->xmlForm -> viceSekci) {
                    // dokument jeste neni vyplnen, tlacitko se nezobrazi
                    return '';
                }

                // Pokud neni kam predat (je na posledni urovni), tak tlacitko nezobrazi
                if ( $this ->xmlForm -> pocatecniUroven ) {
                    // pokud jeste neni prirazena uroven, je tlacitko zobrazeno
                    if ( !UrovenPredavani::existujeVyssiUrovenPredavani($this ->xmlForm ->database,
                        $this ->xmlForm -> predavaciProces, $this ->xmlForm -> pocatecniUroven,
                        $this ->xmlForm -> poradiPredavani ) )
                    {
                        return '';
                    }
                }

                return $this ->getInputHtml('submitnext', TextManager::_('Odeslat_xml'));

            case 'vratitzpet' :
                if ($this ->xmlForm -> viceSekci) {
                    // dokument jeste neni vyplnen, tlacitko se nezobrazi
                    return '';
                }

                // Pokud neni kam predat (je na prvni urovni), tak tlacitko nezobrazi
                if ( !UrovenPredavani::existujeNizsiUrovenPredavani($this ->xmlForm ->database,
                    $this ->xmlForm -> predavaciProces, $this ->xmlForm -> pocatecniUroven,
                    $this ->xmlForm -> poradiPredavani ) )
                {
                    return '';
                }

                return $this ->getInputHtml('submitprev', TextManager::_('Vratit_xml'));

            default :
                return $this ->elementName;
        }
    }


    private function getActionName()
    {
        switch ($this ->elementName) {
            case 'ulozit' :
                return 'submit';

            case 'predatdale' :
                return 'submitnext';

            case 'vratitzpet' :
                return 'submitprev';

            default :
                throw new \Exception('Button ' . $this ->elementName . ' - process neni implementovano!');
        }
    }


    private function predejFormular()
    {
        switch ($this ->elementName) {
            case 'predatdale' :
                $this ->xmlForm ->addDebugMessage('XML - zvolena akce "predat dale"');
                if (!$this ->xmlForm->isError()  &&  $this ->xmlForm ->lzePredat === XMLForm::XML_LZE_PREDAT) {
                    $this ->xmlForm->predej(1);
                    $this ->xmlForm ->addLogData('', sprintf(TextManager::_('Formular_predan_na_uroven'), $this ->xmlForm ->kodUrovne));
                    return true;
                } else if ($this ->xmlForm ->lzePredat !== XMLForm::XML_LZE_PREDAT) {
                    // formular nelze predat, ale neni v nem vygenerovana zadna chyba
                    $this ->xmlForm ->addErrorMsg(TextManager::_('Formular_nelze_predat_dale'));
                }
                return false;

            case 'vratitzpet' :
                $this ->xmlForm ->addDebugMessage('XML - zvolena akce "vratit zpet"');
                if (!$this ->xmlForm->isError()  &&  $this ->xmlForm ->lzePredat) {
                    $zacatek = self::getBoolValue($this->getAttributeValue('zacatek'));
                    $this ->xmlForm->predej($zacatek ? -999 : -1);
                    $this ->xmlForm ->addLogData('', sprintf(TextManager::_('Formular_vracen_na_uroven'), $this ->xmlForm ->kodUrovne));
                    return true;
                } else if (!$this ->xmlForm ->lzePredat) {
                    $this ->xmlForm ->addErrorMsg(TextManager::_('Formular_nelze_vratit_zpet'));
                }
                return false;

            default :
                // tag ulozit
                return true;
        }
    }


    function writeXMLOutput($xmlWriter)
    {
        if ( $this ->xmlForm -> readOnly ) {
            throw new \Exception('Formular je pouze pro cteni!');
        }

        $actionName = $this->getActionName();
        if ($actionName  &&  $actionName == $this ->xmlForm ->xmlAction) {
            $this ->predejFormular();
            parent::writeXMLOutput($xmlWriter);
        } else {
            // akce se nema provadet - jen kopiruje
            $this ->copyXML($xmlWriter);
        }
    }


    function process()
    {
        if ( $this ->xmlForm -> readOnly ) {
            return;
        }

        $actionName = $this->getActionName();

        if ($actionName  &&  $actionName == $this ->xmlForm ->xmlAction) {
            if ($this ->predejFormular()) {
                // provede vnorene akce - ale jen v pripade, ze predani uspesne probehlo
                parent::process();
            }
        }
    }
}

/*
//DOC
PAGE ulozit
<pre>
PRE
  <ulozit textid="Odeslat" text="Odeslat" questionid="Opravdu_odeslat" question="Opravdu chcete ...?">
      ... obsah akce ...
  </ulozit>
PREEND
</pre>

PAGE predatdale
<pre>
PRE
  <predatdale textid="Predat_dale" text="Odeslat" questionid="Opravdu_odeslat" question="Opravdu chcete ...?">
      ... obsah akce ...
  </predatdale>
PREEND
</pre>

PAGE vratitzpet
<pre>
PRE
  <vratitzpet textid="Predat_dale" text="Odeslat" questionid="Opravdu_odeslat" question="Opravdu chcete ...?" zacatek="0">
      ... obsah akce ...
  </vratitzpet>
PREEND
</pre>

//END
*/