<?php
/**
 * User: stanislav.elbl
 */

namespace RonPortal\XML\Classes;

use Ron\DB\DBUtils;
use Ron\StringUtils;
use RonPortal\PredavaciProcesyUtils;
use RonPortal\XML\XMLFormElementRonPortal;

class XMLElementVyberPP extends XMLFormElementRonPortal
{
    protected $name;
    protected $editing;
    protected $pouzeVlastni;
    protected $sqlFilter;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->editing = self::getBoolValue($this ->getAttributeValue('editing'));
        $this ->pouzeVlastni = (int)($this ->getAttributeValue('pouzevlastni'));

        // TODO: pro implementaci writeXmlOutput bude asi potreba predelat! (jako u <variable>
        $this ->sqlFilter = trim(StringUtils::fromUTF8($this ->loadXMLValue($xmlReader)));
    }


    function getHTMLOutput()
    {
        $value = null;
        if ($this ->xmlForm ->dataMode()) {
            $value = $this ->variableValue($this ->name);
        }

        if (!isset($value)) {
            $value = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('value')));
        } else if ($this ->xmlForm ->dataMode()  &&  !$this ->editing) {
            // datove XML, uz vyplneny needitovatelny vyber
            return '<span class="dotaznik_odpoved">' . $this ->variableValue($this ->name . '_jmenoprijmeni') . '</span>';
        }

        $sqlFiltr = $this->nahradPromenneSQL($this->sqlFilter);

        $sql = PredavaciProcesyUtils::sqlVyberPPProPredavaciProces($this ->xmlForm ->database,
            $this ->ronPortalXmlForm->predavaciProces, $this ->ronPortalXmlForm->typRozdeleniProcesu,
            $this ->pouzeVlastni,
            $sqlFiltr);

        $this ->xmlForm ->addDebugMessage('vyberpracovnihopomeru, sql: ' . $sql, true);

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql);

        $attrs = [];
        $attrs['name'] = $this ->name;

        $attrs = $this->setHTMLAttributes($attrs);

        $result = '<SELECT ' . $this->getHTMLAttributes($attrs) . '>';

        while ($row = $cursor ->fetch()) {
            $selectedstr = ($row[7] == (int)$value) ? 'selected="selected"' : '';

            $pp_str = StringUtils::htmlTextConvert($row[1]). " " . StringUtils::htmlTextConvert($row[2]) . ($row[3] ? ", " . StringUtils::htmlTextConvert($row[3]) : '') . " - ";

            $pp_str .=
                \Ron\ronText('Osobni_cislo') . ' ' . StringUtils::htmlTextConvert($row[5]) . ', ' .
                \Ron\ronText('Datum_nastupu') . ' ' . DBUtils::getFormatedValue($this ->xmlForm ->database, 'date', $row[6]);

            $result .= "<OPTION value='{$row[7]}' $selectedstr>$pp_str</OPTION>\n";
        }

        $result .= "</SELECT>\n";

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        $value = (int)$_POST[$this ->name];

        $this ->xmlForm -> nastavPracovniPomer($this ->name, $value);

        if ($this ->editing) {
            $xmlWriter->startElement(
                ($this->prefix ? $this->prefix . ':' : '') .
                $this->elementName
            );

            $this->writeXMLAttributes($xmlWriter, ['value' => $value]);

            $this->writeXMLElements($xmlWriter);

            $xmlWriter->endElement();

        } else {
            $this ->writeXmlVariable($xmlWriter, $this ->name, $value);
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_id', $value);
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_RC', $this ->variableValue($this ->name . '_RC'));

            $this ->writeXmlVariable($xmlWriter, $this ->name . '_jmeno', $this ->variableValue($this ->name . '_jmeno'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_prijmeni', $this ->variableValue($this ->name . '_prijmeni'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_jmenoprijmeni', $this ->variableValue($this ->name . '_jmenoprijmeni'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_titul', $this ->variableValue($this ->name . '_titul'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_osobnicislo', $this ->variableValue($this ->name . '_osobnicislo'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_datumnastupu', $this ->variableValue($this ->name . '_datumnastupu'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_datumnastupu_str', $this ->variableValue($this ->name . '_datumnastupu_str'));
            $this ->writeXmlVariable($xmlWriter, $this ->name . '_popis', $this ->variableValue($this ->name . '_popis'));

            $xmlWriter->startElement('span');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text( StringUtils::toUTF8($this ->variableValue($this ->name . '_popis')) );
            $xmlWriter->endElement();
        }
    }


    function process()
    {
        $originalValue = $this ->variableValue($this ->name);

        if (isset($originalValue)  &&  $this ->xmlForm ->dataMode()  &&  !$this ->editing) {
            // hodnota uz byla nastavena drive a vyber neni editovatelny
            return;
        }

        if (isset($_POST[$this ->name])) {
            $this->xmlForm->nastavPracovniPomer($this->name, (int)$_POST[$this->name]);

            if ($originalValue != $this->variableValue($this->name)) {
                // zaloguje zmenu
                $this->xmlForm->addLogData($this->name, $_POST[$this->name] . ' (' . $this->variableValue($this->name . '_popis') . ')');
            }

            $this->xmlForm->addDataVariable($this->name);
            $this->xmlForm->addDataVariable($this->name . '_id');
            $this->xmlForm->addDataVariable($this->name . '_RC');
            $this->xmlForm->addDataVariable($this->name . '_jmeno');
            $this->xmlForm->addDataVariable($this->name . '_prijmeni');
            $this->xmlForm->addDataVariable($this->name . '_jmenoprijmeni');
            $this->xmlForm->addDataVariable($this->name . '_titul');
            $this->xmlForm->addDataVariable($this->name . '_osobnicislo');
            $this->xmlForm->addDataVariable($this->name . '_datumnastupu');
            $this->xmlForm->addDataVariable($this->name . '_datumnastupu_str');
            $this->xmlForm->addDataVariable($this->name . '_popis');
        }
    }

}


/*
//DOC
PAGE vyberpracovnihopomeru
<pre>
PRE
  <vyberpracovnihopomeru name="PP" editing="0" pouzevlastni="0" value="id_pp">SQL filtr</vyberpracovnihopomeru>
PREEND
</pre>
//END
*/
