<?php

namespace RonPortal\XML;

use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\User;
use XMLWriter;

/**
 * Obecny predek pro vstupni elementy (edit, checkbox, select, ...)
 * Class XMLFormElementInput
 */
class XMLFormElementInput extends XMLFormElement
{
    protected $required;
    protected $name;

    protected $htmlParams; // DEPRECATED!

    /** @var bool Urcuje, jestli se ma v datovem formulari vytvaret datova promenna (default je true).
     *            Musi to byt implementovano v potomkovi! */
    protected $dataVariable;

    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this->required = self::getBoolValue($this->getAttributeValue('required'));
        $this->dataVariable = self::getBoolValue($this ->getAttributeValue('datavariable', 1));

        $this->name = $this->getAttributeValue('name');
        if (!$this ->xmlForm ->mode) {
            // deprecated parametr htmlparams uz neni u novych formularu podporovan
            MessageManager::addDebugMessage('Parametr "htmlparams" jiz neni podporovan! Pouzivejte misto nej prefix html:!');
            $this->htmlParams = $this->getAttributeValue('htmlparams');
        }
    }


    /**
     * @param XMLWriter $xmlWriter
     * @param string $value
     * @param bool $inline Pokud je true, zabali se do <span>, jinak do <div>
     * @return void  Vlozi html text s posledni odpovedi (tag <posledniodpoved>)
     */
    protected function posledniOdpovedXML($xmlWriter, $value, $inline = true)
    {
        $xmlWriter->startElement('posledniodpoved');

        $xmlWriter->startElement($inline ? 'span' : 'div');
        $xmlWriter->writeAttribute('class', 'dotaznik_jmeno');
        $xmlWriter->text(date('j.n.Y G:i', time()) . ' - ' . StringUtils::toUTF8(StringUtils::xmlTextConvert(User::getUserName())) . ': ');
        $xmlWriter->endElement(); // span/div

        $xmlWriter->text( StringUtils::toUTF8($value) );

        $xmlWriter->endElement(); // posledniodpoved
    }
}
