<?php
/**
 * Roman.Szablatura RON Software 08.08.2019
 */

// TODO: zrusit - nepouziva se

namespace RonPortal\Zamestnanci;


use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;


/**
 * Class EditovanaZmenaDriveNahlaseneZmenyField
 * @package Ron\RonPortal
 */
class EditovanaZmenaDriveNahlaseneZmenyField extends BrowserField
{
    public $idEditovaneZmeny;


    /**
     * EditovanaZmenaDriveNahlaseneZmenyField constructor.
     * @param $database
     * @param $idEditovaneZmeny
     */
    public function __construct($database, $idEditovaneZmeny)
    {
        parent::__construct($database);
        $this->idEditovaneZmeny = $idEditovaneZmeny;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        if ($rowData[0] == $this->idEditovaneZmeny) {
            if ($rowData['col-zpracoval-rc'] != '') {
                $result .= "<div class='font-weight-bold align-center'>" . \Ron\ronText('nyni_prohlizite') . "</div>";
            } else {
                if ($rowData[1] == 0) { // stav
                    $result .= "<div class='font-weight-bold align-center'>" . \Ron\ronText('nyni_editovano') . "</div>";
                }
            }
        }

        return $result;
    }
}


/**
 * Class PocetDokumentuDriveNahlaseneZmenyField
 * @package Ron\RonPortal
 */
class PocetDokumentuDriveNahlaseneZmenyField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return $value
            ? $this ->getLink('<i class="fa inline-glyph glyph-attachment" title="' . \Ron\ronText('Pocet_priloh') . ': ' . $value . '">', $rowData[0], $rowData)
            : '';
    }
}


/**
 * Class DriveNahlaseneZmenyBrowser
 * @package Ron\RonPortal
 */
class DriveNahlaseneZmenyBrowser extends RonPortalBrowser
{
    /** @var  Database $database */
    public $database;
    private $id;
    private $typZmeny;
    public $idEditovaneZmeny = "";


    /**
     * DriveNahlaseneZmenyBrowser constructor.
     * @param Database $database
     * @param string $id
     * @param $typZmeny
     * @param null $idEditovaneZmeny
     * @param string $backMenuId
     */
    function __construct($database, $id, $typZmeny, $idEditovaneZmeny=null, $backMenuId='osobniudaje')
    {
        parent::__construct($database, null);

        $this->id = $id;
        $this->typZmeny = $typZmeny;

        $this->idEditovaneZmeny = $idEditovaneZmeny;

        $this->code = 'browsernahlasenezmeny';

        $this ->title = \Ron\ronText('nahlasene_zmeny');

        $this->sqlSelectPart =
            'NahlasenaZmena.ID, NahlasenaZmena.Stav';  //0..1

        $this->sqlFromPart =
            'NahlasenaZmena LEFT JOIN TypNahlaseneZmeny ON (NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod)' .
            ' LEFT JOIN Osoba OsobaZmeny ON (NahlasenaZmena.RC = OsobaZmeny.RC) ' .
            ' LEFT JOIN Osoba Zpracovatel ON (NahlasenaZmena.Zpracoval = Zpracovatel.RC)' .
            ' LEFT JOIN Osoba Nahlasil ON (NahlasenaZmena.Nahlasil = Nahlasil.RC)' .
            ' LEFT JOIN PracovniPomer ON (NahlasenaZmena.RC = PracovniPomer.RC)';

        $this->sqlOrderPart = 'NahlasenaZmena.DatumCas DESC';

        if ($idEditovaneZmeny) {
            $this->addField(new EditovanaZmenaDriveNahlaseneZmenyField($this->database, $idEditovaneZmeny))
                ->setHeaderText('uprava_zmeny')
                ->setAlign('center');
        }

        $this ->primaryField = 0;

        if ($backMenuId != 'osobniudaje') {
            $backMenuId = null;
        }

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCas') ->setFormat('date')
            ->setCode('col-datum')
            ->setHeaderText('datum_hlaseni')
            ->setLink('?menuid=nahlasenezmeny' . ($backMenuId ? '&back=' . $backMenuId : ''));


        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('NahlasenaZmena.Stav', 1)
            ->setCode('col-stavzmeny')
            ->setHeaderText('stav_zmeny') ->setClassName('potvrzeni_zmeny')
            ->setLink('?menuid=nahlasenezmeny' . ($backMenuId ? '&back=' . $backMenuId : ''))
            ->setOptions([
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);


        // osoba zmeny
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Prijmeni') ->setCode('col-osoba-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Jmeno') ->setCode('col-osoba-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Titul') ->setCode('col-osoba-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-osoba-jmeno', 'col-osoba-prijmeni', null, 'col-osoba-titul'))
            ->setCode('col-osoba')
            ->setSortSQL('OsobaZmeny.Prijmeni, OsobaZmeny.Jmeno, OsobaZmeny.Titul')
            ->setSortSQLDesc('OsobaZmeny.Prijmeni DESC, OsobaZmeny.Jmeno DESC, OsobaZmeny.Titul DESC')
            ->setHeaderText('Osoba')
            ->setOrdering(true);

        // nahlasil
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Prijmeni') ->setCode('col-nahlasil-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Jmeno') ->setCode('col-nahlasil-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Titul') ->setCode('col-nahlasil-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-nahlasil-jmeno', 'col-nahlasil-prijmeni', null, 'col-nahlasil-titul'))
            ->setCode('col-nahlasil')
            ->setSortSQL('Nahlasil.Prijmeni, Nahlasil.Jmeno, Nahlasil.Titul')
            ->setSortSQLDesc('Nahlasil.Prijmeni DESC, Nahlasil.Jmeno DESC, Nahlasil.Titul DESC')
            ->setHeaderText('zmenu_nahlasil') ->setClassName('poznamka')
            ->setOrdering(true);

        // zpracoval
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Prijmeni') ->setCode('col-zpracoval-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Jmeno') ->setCode('col-zpracoval-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Titul') ->setCode('col-zpracoval-titul');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('NahlasenaZmena.Zpracoval') ->setCode('col-zpracoval-rc');

        $this ->addField($f = new JmenoPrijmeniBrowserField($this ->database, 'col-zpracoval-jmeno', 'col-zpracoval-prijmeni', null, 'col-zpracoval-titul'))
            ->setCode('col-zpracoval')
            ->setSortSQL('Zpracovatel.Prijmeni, Zpracovatel.Jmeno, Zpracovatel.Titul')
            ->setSortSQLDesc('Zpracovatel.Prijmeni DESC, Zpracovatel.Jmeno DESC, Zpracovatel.Titul DESC')
            ->setHeaderText('zpracoval') ->setClassName('poznamka')
            ->setOrdering(true);
        $f ->setIndexIfNull('col-zpracoval-rc'); // v tabulce neni vazba - pokud nenajde osobu podle RC, tak zobrazi primo hodnotu pole "zpracoval"

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCasZpracovani')
            ->setCode('col-datumcaszpracovani')
            ->setHeaderText('datum_zpracovani') ->setFormat('date')
            ->setOrdering(true);

        $this->addField(new PocetDokumentuDriveNahlaseneZmenyField())
            ->setCode('col-pocet-dokumentu')
            ->setSqlFieldName('(SELECT COUNT(*) FROM DokumentZmeny WHERE DokumentZmeny.IDZmeny = NahlasenaZmena.ID)')
            ->setAlign('center');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(NahlasenaZmena.Poznamka)}')
            ->setCode('col-poznamka');
    }

    function setFilterForm()
    {
        return null;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        switch ($values[1]) { // stav
            case 1 :
                $className .= ' zpracovana-zmena';
                break;
            case 2 :
                $className .= ' odmitnuta-zmena';
                break;
            default :
                $className .= ' nezpracovana-zmena';
                break;
        }

        return parent::getRowAttributes($values, $className, $rowNumber);
    }


    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     */
    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($this->idEditovaneZmeny == $values[0]){
            echo "<tr class='font-weight-bold $className'>";
        } else {
            echo "<tr class='$className'>";
        }

        echo "<td></td>";
        $poznamka = StringUtils::htmlTextConvert($values['col-poznamka']);
        echo
            "<td class='poznamka' title='$poznamka' colspan='" . ($this->visibleFieldsCount() - 1) . "' >" .
            $poznamka .
            '</td>';

        echo "</tr>\n";

        $this ->renderSeparator();
    }


    /**
     * @return SqlStatement|string
     */
    public function getSQLFilter()
    {
        $result = '';
        $params = $this ->database ->createParameters();
        $result .= ' PracovniPomer.ID = ' . $params->addParameter($this->id);

        if ($this->typZmeny == 'rodinnyprislusnik'){
            $result .=
                ' AND (NahlasenaZmena.TypZmeny = ' . $params->addParameter($this->typZmeny) .
                ' OR NahlasenaZmena.TypZmeny = ' . $params->addParameter('novyclenrodiny') .
                ')';

        } else {
            $result .= " AND NahlasenaZmena.TypZmeny = " . $params->addParameter($this->typZmeny);
        }

        return new SqlStatement($result, $params);
    }
}
