<?php

/**
 * stanislav.elbl, 02.03.2021
 */

namespace RonPortal\Zamestnanci;

use Exception;
use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\RonPortal\Zamestnanci\NovyZamestnanecOsobniCisloField;
use Ron\RonPortal\Zamestnanci\NovyZamestnanecPlatnostOdField;
use Ron\RonPortal\Zamestnanci\NovyZamestnanecRCErrorField;
use Ron\RonPortal\Zamestnanci\NovyZamestnanecRCField;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\XML\RonPortalXMLForm;


require_once 'ronportal/zamestnanci/novyzamestnanecfields.php';

//class NovyZamestnanecFormTable
//{
//    public $sqlTableName;
//    public $alias = null;
//
//    public $enabled; // ? ?
//
//    public $foreignKeyFields; // RC nebo RC;OsobniCislo;DatumNastupu
//
//    public $platnostOdField = null;
//    public $platnostDoField = null;
//}





class NovyZamestnanecForm extends RonPortalForm
{
    const STATUS_RC = 'rc';
    const STATUS_ALL = 'all';

//    /**
//     * @var string     Mod formulare:
//     *      rc  Nejdrive se zadava RC a pak teprve zbytek. Po zadani RC se provede kontrola existence a pripadne nacteni udaju osoby
//     */
//    protected $mode = 'rc';

    /**
     * @var string      Aktualni stav formulare
     *      rc      Uzivatel zadava jen RC. Po jeho predani se prepne do dalsiho kroku
     *      all     Uzivatel zadava vsechny dalsi udaje
     */
    protected $status = self::STATUS_RC;


    /** @var bool   Priznak, ze je nainstalovana personalistika nebo mzdy */
    protected $jePerson;

    /** @var string[]   Seznam podrizenych tabulek napojenych na tabulku Osoba (musi obsahovat pole RC) */
    protected $tablesOsoba = [
        'Osoba', 'OsobniUdaj'
    ];

    protected $tablesPracovniPomer = [
        'PrirazeniKalendare'
    ];

    /** @var string[]   Seznam podrizenych tabulek napojenych na tabulku PracovniPomer (musi obsahovat RC, OsobniCislo, DatumNastupu */
    protected $tablesPP = [
        //'PracovniPomer'
    ];

    /** @var NovyZamestnanecRCField */
    protected $fRC;
    /** @var NovyZamestnanecRCErrorField  Pole informujici o pripadne chybe v rodnem cisle */
    protected $fRCError;
    /** @var FormField */
    protected $fDatumNarozeni;
    /** @var FormField */
    protected $fPohlavi;
    /** @var FormField */
    protected $fCisloPojistence;
    /** @var FormField */
    protected $fSerioveCisloUctu;

    /** @var RozdeleniZamestnanceField[] */
    protected $rozdeleniFields = [];

    /** @var PrukazOsobyField[] */
    protected $prukazyFields = [];

    /** @var DokumentOsobyField[] */
    protected $dokumentyFields = [];

    /** @var string     Id osoby, pro kterou se vytvari novy PP (Osoba.RC). Pouzije se v pripade zadavani noveho PP pro existujici osobu */
    protected $osobaRC = null;
    /** @var bool       Nastavi se na true, pokud se jedna o osobu, ktera ma jeste dalsi aktivni pracovni pomer */
    protected $existujeAktivniPP = false;
    /** @var null       Platnost Do posledniho ukonceneho pracovniho pomeru */
    protected $posledniPlatnyPPDo = null;
    /** @var int        Id editovaneho PP - pouzije se pri editaci z XML formulare. Nastavi se take po uspesnem zapisu zaznamu do databaze. */
    protected $idPP = null;
    /** @var PracovniPomer  Editovany pracovni pomer nacteny podle $idPP */
    protected $pp = null;
    /** @var int            Celkovy pocet existujicich pracovnich pomeru osoby (nastavi se v pripade editace) */
    protected $pocetPP = 0;

    protected $zobrazenoZXML = false;

    protected $novyZamestnanecConfiguration = null;

    protected $configurationLoaded = false;

    /** @var int[]      Seznam id nove vlozenych dokumentu pri ulozeni formulare */
    public $noveDokumenty = [];

    /**
     * NovyZamestnanecForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param int $idPP Editovany pracovni pomer - pro pouziti v XML formulari
     * @param bool $zobrazenoZXML
     */
    public function __construct($database, $formAction = 'new', $idPP = null, $zobrazenoZXML = false)
    {
        parent::__construct($database, $formAction);

        $this->idPP = $idPP;   // editovany pracovni pomer - pouzije se pri editaci z XML formulare

        if ($this->idPP) {
            $this->status = self::STATUS_ALL;

            $this->pp = PracovniPomer::nactiPP($database, $this->idPP);
            $this->pocetPP = ZamestnanciUtils::pocetPPOsoby($database, $this->pp->rc);
        }

        $this->title = TextManager::_('Novy_zamestnanec');
        $this->code = 'formnovyzamestnanec';
        $this->htmlParams['id'] = 'formnovyzamestnanec';

        $this->defaultRowClass = 'row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this->zobrazenoZXML = $zobrazenoZXML;
        if ($this->zobrazenoZXML) {
            $this->renderHtmlForm = false; // negeneruje html tag <form>, protoze je soucasti XML formulare
        }

        $this->jePerson = RonPortalUtils::jeNainstalovanaAplikace($database, ['MZDY', 'PERSON']);

        if ($this->jePerson) {
            array_push($this->tablesOsoba,
                'TrvaleBydliste', 'PrechodneBydliste',
                'ZdravotniStavOsoby', 'PojisteniOsobyU_ZP',
                'VzdelaniOsoby', 'RodinnyStav', 'Duchod'
            );

            array_push($this ->tablesPracovniPomer,
                'PlatovyVymer', 'EvidencniStav'
            );
        }

        // Jednotliva editovaci pole prida az po nacteni zakladni konfigurace
    }

    /**
     * @return int
     */
    public function getIdPP()
    {
        return $this->idPP;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }


    /**
     * Nastavi dodatecnou konfiguraci formulare - pouziva se z XML tagu <nastupniformular>
     * @param array $novyZamestnanecConfiguration
     */
    public function setNovyZamestnanecConfiguration($novyZamestnanecConfiguration)
    {
        $this->novyZamestnanecConfiguration = $novyZamestnanecConfiguration;
    }

    /**
     * Prida pole pro zadani rodneho cisla a jeho kontrolu
     */
    private function addFieldsRodneCislo()
    {
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id', 'PracovniPomer')->setPrimary(true)
            ->setReadOnly(true)
            ->setCode('field-idpp');

        $this->fRC = new NovyZamestnanecRCField();
        $this->fRC
            ->setFieldName('RodneCislo', 'OsobniUdaj')->setCaption('RC')->setRequired($this->jePerson)
            ->setCode('field-rodnecislo')->setFieldId('osoba_rodnecislo')->setMaxLength(15)
            ->setFieldCssClass('form-control keyFilter')
            ->setOtherHtmlParams([
                    'data-allowed-chars' => '01123456789', // jen cisla
                    'autocomplete' => 'off']
            )
            ->setVisible($this->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField($this->fRC);

        $this->fRCError = new NovyZamestnanecRCErrorField();
        $this->fRCError
            ->setCode('field-rc-error')
            ->setVisible(false);    // zobrazi se jen pri chybe v RC
        $this->addField($this->fRCError);

        if (isset($_POST['OsobniUdaj#RodneCislo'])) {
            $this->fRCError->analyzujRC(trim($_POST['OsobniUdaj#RodneCislo']));
        }
    }


    private function addNovyZamestnanecFields()
    {
        $this->fRC->showReadonly = true;
        $this->fRCError->setReadOnly(true);

        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zakladniudaje')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zakladni_udaje');

        // zakladni udaje
        $this->addField()
            ->setFieldName('Titul', 'Osoba')->setCaption('Titul')
            ->setCode('field-titul')
            ->setFieldId('osoba_titul')
            ->setMaxLength(20);

        // TODO: podle nastaveni udelat editovatelne
        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('RC', 'Osoba')->setCaption('Id_osoby')->setRequired(false)// nastavuje se automaticky az tesne pred POST
            ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-osoba-rc');

        $this->addField()
            ->setFieldName('Jmeno', 'Osoba')->setCaption('Jmeno')->setRequired(true)
            ->setMaxLength(30)
            ->setCode('field-jmeno');

        $this->addField()
            ->setFieldName('Prijmeni', 'Osoba')->setCaption('Prijmeni')->setRequired(true)
            ->setMaxLength(30)
            ->setCode('field-prijmeni');

        $this->addField()
            ->setFieldName('TitulZa', 'Osoba')->setCaption('Titul_za')
            ->setMaxLength(20)
            ->setCode('field-titulza');

        // osobni udaje
        $this->addFieldsOsobniUdaje();

        $this->addFieldsKontakty();

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka', 'Osoba')->setCaption('Poznamka')
            ->setCode('field-poznamka');

        $this->addUserLogFields('Osoba');

        // TODO:
        // OsobniUdaj.RC_Zahranicni 15

        if ($this->jePerson) {
            // Trvale a prechodne bydliste
            $this->addFieldsAdresa('TrvaleBydliste', 'Trvale_bydliste', 1);
            $this->addFieldsAdresa('PrechodneBydliste', 'Prechodne_bydliste', 0);

            // Zdravotni pojisteni
            $this->addFieldsZP();
            // Zdravotni stav
            $this->addFieldsZdravotniStav();
            // duchod
            $this->addFieldsDuchod();
            // vzdelani
            $this->addFieldsVzdelani();
            // rodinny stav
            $this->addFieldsRodinnyStav();
        }

        // Rodinni prislusnici
        // - zadavani
        // + najit rodinne prislusniky

        // Pracovni pomer
        $this->addFieldsPracovniPomer();

        $this->addFieldsEvidencniStav();
        $this->addFieldsKalendar();

        if ($this->jePerson) {
            $this->addFieldsVyplaceniMzdy();
            $this->addFieldsPlatovyVymer();
        }

        // Medium
        // Login, Profil
    }


    private function addFieldsOsobniUdaje()
    {
        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'OsobniUdaj')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-osobniudaj-rc');

        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osobniudaje')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('osobni_udaje');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-osobniudaje')
            ->setCaption('Osobni_udaje');

        $this->fDatumNarozeni = $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumNarozeni', 'OsobniUdaj')->setCaption('Datum_narozeni')
            ->setFieldId('osoba_datumnarozeni')
            ->setCode('field-datumnarozeni');

        $this->addField()
            ->setFieldName('RodneJmeno', 'OsobniUdaj')->setCaption('Rodne_jmeno')
            ->setMaxLength(15)
            ->setCode('field-rodneprijmeni');

        $this->fPohlavi = $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Pohlavi', 'OsobniUdaj')->setDataType('int')->setCaption('Pohlavi')
            ->setCode('field-pohlavi')
            ->setFieldId('osoba_pohlavi')
            ->setSelectSql('SELECT Kod, Popis FROM S_Pohlavi ORDER BY Kod');

        // narodnost atd.
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osobniudaje2')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('osobni_udaje2');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Narodnost', 'OsobniUdaj')->setDataType('int')->setCaption('Narodnost')
            ->setCode('field-narodnost')
            ->setFieldId('osoba_narodnost')
            ->setSelectSql('SELECT Kod, Popis FROM Narodnost WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues([1, 2])// 1=ceska, 2=slovenska
            ->setVisible($this->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('StatniPrislusnost', 'OsobniUdaj')->setCaption('Statni_prislusnost')
            ->setCode('field-statniprislusnost')
            ->setFieldId('osoba_statniprislusnost')
            ->setSelectSql('SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setPreferredValues(['CZ', 'SK'])
            ->setVisible($this->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField()
            ->setFieldName('MistoNarozeni', 'OsobniUdaj')->setCaption('Misto_narozeni')
            ->setMaxLength(30)
            ->setCode('field-mistonarozeni')
            ->setVisible($this->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OkresNarozeni', 'OsobniUdaj')->setCaption('Okres_narozeni')
            ->setCode('field-okresnarozeni')
            ->setFieldId('osoba_okresnarozeni')
            ->setSelectSql('SELECT Kod, Popis FROM Okres WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setVisible($this->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField(FormField::FIELD_TYPE_INT)
            ->setFieldName('PocetDetiProDuchod', 'Osoba')->setCaption('Pocet_deti_pro_duchod')
            ->setCode('field-pocetdetiproduchod')
            ->setFieldId('osoba_pocetdetiproduchod')
            ->setVisible($this->jePerson);
    }

    /**
     * Prida pole pro editaci zakladnich kontaktu
     */
    private function addFieldsKontakty()
    {
        // zakladni kontakty (email, telefon)
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-kontakty')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('kontakty');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-kontakty')
            ->setCaption('Kontakty');

        $this->addField()
            ->setFieldName('EMail', 'Osoba')->setCaption('E_mail')
            ->setMaxLength(60)
            ->setCode('field-email');

        $this->addField()
            ->setFieldName('EMailProVyplatniListek', 'Osoba')->setCaption('E_mail_pro_vyplatni_listek')
            ->setMaxLength(60)
            ->setCode('field-emailprovyplatnilistek');

        $this->addField()
            ->setFieldName('Mobil', 'Osoba')->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode('field-mobil');

        $this->addField()
            ->setFieldName('Telefon', 'Osoba')->setCaption('Telefon')
            ->setMaxLength(60)
            ->setCode('field-telefon');
    }


    /**
     * Pridani poli pro editaci trvale nebo prechodne adresy
     * @param string $tableAlias Alias tabulky v SQL dotazu
     * @param string $header
     * @param bool $trvale
     */
    private function addFieldsAdresa($tableAlias, $header, $trvale)
    {
        $code = strtolower($tableAlias);

        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode($trvale ? 'group-trvalebydliste' : 'group-prechodnebydliste')
            ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId($code);

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-{$code}")
            ->setCaption($header);

        if (!$trvale) {
            $this->addField(FormField::FIELD_TYPE_NOT_NULL_FLAG)
                ->setCode("field-{$code}-exists")//->setReadOnly(true)
                ->setSqlFieldName('PlatnostOd', 'AdresaOsoby', $tableAlias)->setFieldName("{$code}_existuje")
                ->setFieldId("osoba_{$code}_existuje")
                ->setCaption2('Prechodne_bydliste_je_rozdilne')
                ->setFieldCssClass('toggle-class toggle-on-load')
                ->setAppendFormClass('je-prechodne-bydliste')
                ->setOtherHtmlParams([
                    'data-selector' => '#formnovyzamestnanec',
                    'data-check-class' => 'je-prechodne-bydliste'
                ]);
        }

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('KontaktniAdresa', 'AdresaOsoby', $tableAlias)->setCaption("Kontaktni_adresa")
            ->setCode("field-{$code}-kontaktniadresa")
            ->setFieldId("osoba_{$code}_kontaktniadresa")
            ->setDefaultValue($trvale ? 1 : 0);

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'AdresaOsoby', $tableAlias)->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-{$code}-rc");

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'AdresaOsoby', $tableAlias)
            ->setCode("field-{$code}-platnostod")
            ->setDataType('date')
            ->setDefaultValue(DateUtils::today());

        // priznak "Trvale bydliste"
        $this->addField(FormField::FIELD_TYPE_CONSTANT)->setReadOnly(true)
            ->setCode("field-{$code}-trvale")
            ->setSqlFieldName('TrvaleBydliste', 'AdresaOsoby', $tableAlias)
            ->setDefaultValue($trvale);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stat', 'AdresaOsoby', $tableAlias)->setCaption('Stat')
            ->setCode("field-{$code}-stat")
            ->setFieldId("osoba_{$code}_stat")
            ->setSelectSql('SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues(['CZ', 'SK'])
            ->setFieldCssClass('form-control ronselect');

        $this->addField()
            ->setFieldName('Ulice', 'AdresaOsoby', $tableAlias)->setCaption('ulice')
            ->setMaxLength(60)
            ->setCode("field-{$code}-ulice")
            ->setFieldId("osoba_{$code}_ulice");

        $this->addField()
            ->setFieldName('PSC', 'AdresaOsoby', $tableAlias)->setCaption('psc')
            ->setMaxLength(10)
            ->setCode("field-{$code}-psc")
            ->setFieldId("osoba_{$code}_psc")
            ->setFieldCssClass('form-control psc-input')
            ->setOtherHtmlParams([
                "data-mesto-input-selector" => "#osoba_{$code}_mesto"
            ]);


        $this->addField()
            ->setFieldName('Mesto', 'AdresaOsoby', $tableAlias)->setCaption('mesto')
            ->setMaxLength(40)
            ->setCode("field-{$code}-mesto")
            ->setFieldId("osoba_{$code}_mesto")
            ->setFieldCssClass('form-control ronselect mesto-input')
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-ronselect-class' => 'form-control',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznammest=1',
                'data-enable-url-cache' => '1',
                'data-columns' => 'mesto;psc;okresPopis',
                'data-filter-columns' => 'mesto;psc',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('PSC') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'mesto',
                'data-text-column' => 'mesto',
                'data-only-autocomplete' => '1',        // umoznuje zadat jakykoliv text
                // napojeni na dalsi inputy
                "data-psc-input-selector" => "#osoba_{$code}_psc",
                "data-obec-input-selector" => "#osoba_{$code}_obec"
            ]);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Obec', 'AdresaOsoby', $tableAlias)->setCaption('obec')
            ->setMaxLength(40)
            ->setCode("field-{$code}-obec")
            ->setFieldId("osoba_{$code}_obec")
            ->setFieldCssClass('form-control ronselect obec-input')->setRowClass('row margin-bottom-2')
            ->setSelectSql('SELECT Kod, Popis FROM Obec WHERE 0 = 1')
            ->setShowEmptyValue(true)
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznamobci=1&showemptyvalue=1',
                'data-enable-url-cache' => '1',
                'data-columns' => 'popis;okresPopis',
                'data-filter-columns' => 'popis',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'kod',
                'data-value-column' => 'kod',
                'data-text-column' => 'popis',
                // napojeni na dalsi inputy
                "data-mesto-input-selector" => "#osoba_{$code}_mesto"
            ]);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZpusobDopravy', 'AdresaOsoby', $tableAlias)->setCaption('Zpusob_dopravy_do_zamestnani')->setDataType('int')
            ->setCode("field-{$code}-zpusobdopravy")
            ->setFieldId("osoba_{$code}_zpusobdopravy")->setShowEmptyValue(false)
            ->setSelectSql('SELECT Kod, Popis FROM DopravaDoZamestnani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setDefaultValue(0)
            ->setPreferredValues([0]);

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('VzdalenostDoZamestnani', 'AdresaOsoby', $tableAlias)->setCaption('Vzdalenost_do_zamestnani')
            ->setCode("field-{$code}-vzdalenostdozamestnani")
            ->setFieldId("osoba_{$code}_vzdalenostdozamestnani")
            ->setFieldCssClass('inputFloat margin-right-2')
            ->setCaption2('km');

        $this->addField()
            ->setFieldName('E_Mail', 'AdresaOsoby', $tableAlias)->setCaption('E_Mail')
            ->setMaxLength(60)
            ->setCode("field-{$code}-email")
            ->setFieldId("osoba_{$code}_email");

        $this->addField()
            ->setFieldName('Mobil', 'AdresaOsoby', $tableAlias)->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode("field-{$code}-mobil")
            ->setFieldId("osoba_{$code}_mobil");

        $this->addField()
            ->setFieldName('Telefon1', 'AdresaOsoby', $tableAlias)->setCaption('Telefon')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon")
            ->setFieldId("osoba_{$code}_telefon");

        $this->addField()
            ->setFieldName('Telefon2', 'AdresaOsoby', $tableAlias)->setCaption('Telefon_2')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon2")
            ->setFieldId("osoba_{$code}_telefon2");
    }


    private function addFieldsPracovniPomer()
    {
        // -!Typ PP, !Osobni cislo, !Datum nastupu, Poradove cislo,
        // - cislo smlouvy, datum podpisu, duvod vzniku PP atd.
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-pracovnipomer')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('pracovnipomer');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-pracovnipomer')
            ->setCaption('Pracovni_pomer');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// doplni se podle RC osoby
        ->setFieldName('RC', 'PracovniPomer')
            ->setReadOnly(true)
            ->setCode('field-pp-rc')->setMaxLength(15);

        //$this ->addField()
        $this->addField(new NovyZamestnanecOsobniCisloField($this->database, !$this->idPP))
            ->setFieldName('OsobniCislo', 'PracovniPomer')->setCaption('Osobni_cislo')
            ->setCode('field-osobnicislo')
            ->setMaxLength(15)->setRequired(true);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumNastupu', 'PracovniPomer')->setCaption('Datum_nastupu')
            ->setCode('field-datumnastupu')->setDefaultValue(StringUtils::format('date', DateUtils::today()))
            ->setFieldId('pracovnipomer_datumnastupu')
            ->setRequired(true);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceni', 'PracovniPomer')->setCaption('Datum_ukonceni')
            ->setCode('field-datumukonceni');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Zamestnanec', 'PracovniPomer')->setCaption('Typ_osoby')->setDataType('int')
            ->setCode('field-typosoby')
            ->setRequired(true)->setDefaultValue(101)
            ->setSelectSql('SELECT Kod, Popis FROM S_TypOsoby WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypPracovnihoPomeru', 'PracovniPomer')->setCaption('Typ_pracovniho_pomeru')
            ->setCode('field-typpracovnihopomeru')->setDefaultValue('HPP')
            ->setRequired(true)
            ->setSelectSql('SELECT Kod, Popis FROM TypPracovnihoPomeru WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DuvodVznikuPP', 'PracovniPomer')->setCaption('Duvod_vzniku_pp')
            ->setCode('field-duvodvznikupp')
            ->setSelectSql('SELECT Kod, Popis FROM DuvodVznikuPP WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setVisible($this->jePerson);

        $notifikaceCount = DBUtils::getSqlValue($this->database, 'SELECT COUNT(*) FROM ProfilNotifikace');
        if ($notifikaceCount) {
            $this->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('ProfilNotifikace', 'PracovniPomer')->setCaption('Profil_notifikace')
                ->setCode('field-profilnotifikace')
                ->setSelectSql('SELECT Kod, Popis FROM ProfilNotifikace WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');
        }

        $this->addUserLogFields('PracovniPomer');
        // Zkusebni doba
        // - mesicu / do, vypoctena
    }


    /**
     * Prida pole z tabulky PojisteniOsobyU_ZP
     */
    private function addFieldsZP()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zdravotnipojisteni')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zdravotnipojisteni');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-zdravotnipojisteni")
            ->setCaption(TextManager::_('Zdravotni_pojisteni'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'PojisteniOsobyU_ZP')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-zdravotnipojisteni-rc');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodZdravotniPojistovny', 'PojisteniOsobyU_ZP')->setCaption('Zdravotni_pojistovna')
            ->setCode('field-kodzdravotnipojistovny')
            ->setSelectSql('SELECT Kod, Popis FROM ZdravotniPojistovna WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectShowCode(true);

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'PojisteniOsobyU_ZP')->setCaption('Platnost_od')
            ->setCode('field-zdravotnipojisteni-platnostod');

        $this->fCisloPojistence = $this->addField()
            ->setFieldName('CisloPojistence', 'PojisteniOsobyU_ZP')->setCaption('Cislo_pojistence')
            ->setCode('field-cislopojistence')->setMaxLength(15);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('CisloUctarny', 'PojisteniOsobyU_ZP')->setCaption('Cislo_uctarny')
            ->setCode('field-cislouctarny')
            ->setSelectSql('SELECT Kod, Popis FROM Uctarna WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypZamestnance', 'PojisteniOsobyU_ZP')->setDataType('int')->setCaption('Typ_zamestnance_ZP')
            ->setCode('field-zdravotnipojisteni-typzamestnance')
            ->setSelectSql('SELECT Kod, Popis FROM S_TypZamestnanceProZP WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

    }

    /**
     * Prida pole z tabulky ZdravotniStavOsoby
     */
    private function addFieldsZdravotniStav()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zdravotnistav')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zdravotnistav');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-zdravotnistav")
            ->setCaption(TextManager::_('Zdravotni_stav'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'ZdravotniStavOsoby')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-zdravotnistav-rc");

//        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
//            ->setFieldName('PlatnostOd', 'ZdravotniStavOsoby') ->setReadOnly(true)
//            ->setCode('field-zdravotnistav-platnostod')
//            ->setDataType('date')
//            ->setDefaultValue(DateUtils::today());

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZdravotniStav', 'ZdravotniStavOsoby')->setDataType('int')->setCaption('Zdravotni_stav')
            ->setCode('field-zdravotnistav')
            ->setSelectSql('SELECT Kod, Popis FROM S_ZdravotniStav WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'ZdravotniStavOsoby')->setCaption('Platnost_od')
            ->setCode('field-zdravotnistav-platnostod');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('DrzitelPrukazuZTTP', 'ZdravotniStavOsoby')->setDataType('int')->setCaption('Drzitel_prukazu_ZTP')
            ->setCode('field-zdravotnistav-drzitelprukazuztp');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ZPS', 'ZdravotniStavOsoby')->setDataType('int')->setCaption('ZPS')
            ->setCode('field-zdravotnistav-zps');

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ZPS_TZP', 'ZdravotniStavOsoby')->setDataType('int')->setCaption('ZPS_TZP')
            ->setCode('field-zdravotnistav-zps_tzp');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OsetrujiciLekar', 'ZdravotniStavOsoby')->setCaption('Osetrujici_lekar')->setVisible(false)
            ->setCode('field-osetrujicilekar')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, JmenoLekare FROM OsetrujiciLekar WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'JmenoLekare');
    }


    /**
     * Prida pole z tabulky Duchod
     */
    private function addFieldsDuchod()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-duchod')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('duchod');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-duchod")
            ->setCaption(TextManager::_('Duchod'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'Duchod')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-duchod-rc");

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypDuchodu', 'Duchod')->setDataType('int')->setCaption('Typ_duchodu')
            ->setCode('field-typduchodu')
            ->setSelectSql('SELECT Kod, Popis FROM TypDuchodu WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumPriznani', 'Duchod')->setCaption('Datum_priznani_duchodu')
            ->setRequiredInDatabase(true)
            ->setCode('field-duchod-datumpriznani');

        $this->addField()
            ->setFieldName('PlatceDuchodu', 'Duchod')->setCaption('Platce_duchodu')->setMaxLength(160)
            ->setCode('field-platceduchodu');
    }


    private function addFieldsVzdelani()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-vzdelani')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('vzdelani');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-vzdelani")
            ->setCaption(TextManager::_('Nejvyssi_dosazene_vzdelani'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'VzdelaniOsoby')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-vzdelani-rc");

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodVzdelani', 'VzdelaniOsoby')->setDataType('int')->setCaption('Vzdelani')
            ->setCode('field-kodvzdelani')
            ->setFieldId('kodVzdelani')
            ->setSelectSql('SELECT Kod, Popis, KodProCSU FROM TypVzdelani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod')
            ->setItemsAttributes(['kodcsu' => 2])
            ->setFieldCssClass('form-control toggle-class toggle-on-load')
            ->setAppendFormClass('je-vzdelani')
            ->setOtherHtmlParams([
                'data-selector' => '#formnovyzamestnanec',
                'data-not-empty-class' => 'je-vzdelani'
            ]);


        $this->addField(FormField::FIELD_TYPE_COMBO)
            ->setFieldName('MistoStudia', 'VzdelaniOsoby')->setDataType('int')->setCaption('Misto_studia')
            ->setCode('field-mistostudia')
            ->setFieldId('mistoStudia')
            ->setOptions([0 => TextManager::_('Vzdelani_CR'), 1 => TextManager::_('Vzdelani_zahranici')])
            ->setDefaultValue(0);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OborVzdelani', 'VzdelaniOsoby')->setCaption('Obor_vzdelani')
            ->setCode('field-oborvzdelani')
            ->setFieldId('oborVzdelani')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, Popis FROM OborVzdelani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues(['0000']);

        $this->addField()
            ->setFieldName('Zamereni', 'VzdelaniOsoby')->setCaption('Zamereni_vzdelani')
            ->setMaxLength(40)
            ->setCode('field-vzdelani-zamereni');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceni', 'VzdelaniOsoby')->setCaption('Datum_ukonceni_vzdelani')
            ->setCode('field-vzdelani-datumukonceni')
            ->setFieldId('datumUkonceniStudia');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VzdelaniUkonceno', 'VzdelaniOsoby')->setDataType('int')->setCaption('Zpusob_ukonceni_vzdelani')
            ->setCode('field-vzdelaniukonceno')
            ->setFieldId('vzdelaniUkonceno')
            ->setSelectSql('SELECT Kod, Popis FROM S_ZpusobUkonceniSkoly WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Skola', 'VzdelaniOsoby')->setCaption('Skola')
            ->setCode('field-skola')
            ->setSelectSql('SELECT Kod, Nazev FROM Skola WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Nazev')
            ->setVisible(false);    // implicitne skryto, protoze ve vychozim stavu je ciselnik prazdny

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumOd', 'VzdelaniOsoby')->setCaption('Datum_vzdelani_od')
            ->setCode('field-vzdelani-datumod')
            ->setDataColumnClass('col-12 col-sm-3');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumDo', 'VzdelaniOsoby')->setCaption('Datum_vzdelani_do')
            ->setNewRow(false)
            ->setCode('field-vzdelani-datumdo')
            ->setCaptionColumnClass('col-12 col-sm-1 caption')
            ->setDataColumnClass('col-12 col-sm-5');

        $this->addField()
            ->setFieldName('OborVzdelaniISPV', 'VzdelaniOsoby')->setCaption('Obor_vzdelani_dle_ISPV')
            ->setMaxLength(40)
            ->setCode('field-oborvzdelaniispv')
            ->setFieldId('oborVzdelaniISPV')
            ->setDataColumnClass('col12 col-sm-7');

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('field-vyhledatoborvzdelani')
            ->setNewRow(false)->setText(TextManager::_('Vyhledat_obor_ISPV'))
            ->setFieldId('vyhledatVzdelaniISPV')
            ->setFieldCssClass('vyhledat-obor-vzdelani')
            ->setCaptionColumnClass('d-none')->setDataColumnClass('col12 col-sm-2 align-right js')
            ->setOtherHtmlParams([
                'data-misto-studia-selector' => '#mistoStudia',
                'data-kod-vzdelani-selector' => '#kodVzdelani',
                'data-datum-ukonceni-selector' => '#datumUkonceniStudia',
                'data-obor-vzdelani-ispv-selector' => '#oborVzdelaniISPV'
            ]);
    }

    private function addFieldsRodinnyStav()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-rodinnystav')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('rodinnystav');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-rodinnystav")
            ->setCaption(TextManager::_('Rodinny_stav'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'RodinnyStav')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-rodinnystav-rc");

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodRodinnehoStavu', 'RodinnyStav')->setDataType('int')->setCaption('Rodinny_stav')
            ->setCode('field-kodrodinnehostavu')
            ->setSelectSql('SELECT Kod, Popis FROM S_RodinnyStav WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod')
            ->setFieldId('kodRodinnehoStavu');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'RodinnyStav')->setCaption('Platnost_od')
            ->setCode('field-rodinnystav-platnostod')
            ->setFieldId('platnostRodinnehoStavuOd');
    }


    private function addFieldsVyplaceniMzdy()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-vyplacenimzdy')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('vyplacenimzdy');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-vyplacenimzdy')
            ->setCaption(TextManager::_('Vyplaceni_mzdy'));
        /*
        if ($this ->osobaRC  ||  $this ->idPP) {
            $params = $this ->database ->createParameters();

            $selectSql = $this ->osobaRC
                ? new SqlStatement(
                    'SELECT SerioveCislo, CisloUctu, KodBanky FROM BankovniUcet' .
                    ' WHERE RC = ' . $params ->addParameter($this ->osobaRC) .
                    ' AND (%FILTER%) ORDER BY %ORDER%',
                    $params)
                :  new SqlStatement(
                    'SELECT SerioveCislo, CisloUctu, KodBanky FROM BankovniUcet, PracovniPomer' .
                    ' WHERE PracovniPomer.RC = BankovniUcet.RC' .
                    ' AND PracovniPomer.Id = ' . $params ->addParameter($this ->idPP) .
                    ' AND (%FILTER%) ORDER BY %ORDER%',
                    $params);

            $this ->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('SerioveCisloUctuVyplaty', 'VyplaceniMzdy') ->setDataType('int') ->setCaption('Ucet')
                ->setCode('field-seriovecislouctuvyplaty')
                ->setSelectSql($selectSql, '1 = 1', 'CisloUctu')
                ->setAttrIndexes(['kodbanky' => 2]);
        } else {
        */
        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('SerioveCisloUctuVyplaty', 'VyplaceniMzdy')->setDataType('int')->setReadOnly(true)
            ->setCode('field-noveseriovecislouctuvyplaty');
        //}

//        $this ->addField() ->setFieldName('ZpusobPlatbyVyplaty', 'VyplaceniMzdy');
        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'VyplaceniMzdy')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-vyplacenimzdy-rc');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('OsobniCislo', 'VyplaceniMzdy')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-vyplacenimzdy-osobnicislo');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('DatumNastupu', 'VyplaceniMzdy')->setReadOnly(true)->setDataType('date')
            ->setCode('field-vyplacenimzdy-datumnastupu');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_DLE_PP, true))// podle zacatku mesice
        ->setFieldName('PlatnostOd', 'VyplaceniMzdy')
            ->setCode('field-vyplaceni-platnostod');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('ZpusobPlatbyVyplaty', 'VyplaceniMzdy')->setDataType('int')->setDefaultValue(1)// na ucet
            ->setCode('field-vyplacenimzdy-zpusobplatbyvyplaty');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('ZpusobPlatbyZalohy', 'VyplaceniMzdy')->setDataType('int')->setDefaultValue(0)
            ->setCode('field-vyplacenimzdy-zpusobplatbyzalohy');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('DefinovatCastkuHotove', 'VyplaceniMzdy')->setDataType('int')->setDefaultValue(0)
            ->setCode("field-vyplacenimzdy-definovatcastkuhotove");

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('SerioveCislo', 'BankovniUcet')->setDataType('int')->setReadOnly(true);

        $this->addField()
            ->setFieldName('CisloUctu', 'BankovniUcet')->setMaxLength(20)->setCaption('Cislo_uctu')
            ->setCode('field-bankonvniucet-cislouctu')
            ->setDataColumnClass('col-12 col-sm-3')
            ->addFieldCssClass('keyFilter')
            ->setOtherHtmlParams(['autocomplete' => 'off', 'data-allowed-chars' => '0123456789-']);


        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodBanky', 'BankovniUcet')->setCaption('Kod_banky')
            ->setNewRow(false)
            ->setCode('field-bankovniucet-kodbanky')
            ->setCaptionColumnClass('col-12 col-sm-2 caption')
            ->setDataColumnClass('col-12 col-sm-4')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, Nazev FROM Banka WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Nazev')
            ->setSelectShowCode(true);

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se automaticky (hodnota 1 = ucet zamestnance)
        ->setFieldName('TypUctu', 'BankovniUcet')->setDataType('int')->setDefaultValue(1)
            ->setCode('field-bankovniucet-typuctu');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'BankovniUcet')->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-bankovniucet-rc");

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se automaticky
        ->setFieldName('JmenoUctu', 'BankovniUcet')->setMaxLength(40)->setReadOnly(true)
            ->setCode('field-bankovniucet-jmenouctu')
            ->setDefaultValue(TextManager::_('Maska_jmena_uctu_vyplaty'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se automaticky
        ->setFieldName('ZahranicniPlatba', 'BankovniUcet')->setDataType('int')->setDefaultValue(0)->setReadOnly(true)
            ->setCode('field-bankovniucet-zahranicniplatba');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se automaticky
        ->setFieldName('KonstantniSymbol', 'BankovniUcet')->setDefaultValue('0138')->setReadOnly(true)
            ->setCode('field-bankovniucet-konstantnisymbol');

        $this->addUserLogFields('BankovniUcet');

        // 0 hotove, 1 prevodem
        // RC, Os, DatumNastupu, PlatnostOd
        // ZpusobPlatbyVyplaty: 0=hotove, 1=na ucet
        // ZpusobPlatbyZalohy=0, DefinovatCastkuHotove=0
        // ----- BankovniUcet ----
        // SerioveCislo
        // JmenoUctu[40],KodBanky[10],CisloUctu[20]
        // TypUctu: 1=ucty zamestnancu
        // RC[15]
        // SpecifickySymbol[20], KonstantniSymbol[10], VariabilniSymbol[20]
        // Insert_UserName, Insert_Timestamp
        // ZahranicniPlatba=0

    }


    private function addFieldsPlatovyVymer()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-platovyvymer')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('platovyvymer');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-platovyvymer')
            ->setCaption(TextManager::_('Platovy_vymer'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'PlatovyVymer')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-platovyvymer-rc');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('OsobniCislo', 'PlatovyVymer')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-platovyvymer-osobnicislo');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('DatumNastupu', 'PlatovyVymer')->setReadOnly(true)->setDataType('date')
            ->setCode('field-platovyvymer-datumnastupu');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_DLE_PP))// podle zacatku mesice
        ->setFieldName('PlatnostOd', 'PlatovyVymer')
            ->setCode('field-platovyvymer-platnostod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypMzdy', 'PlatovyVymer')->setCaption('Typ_mzdy')
            ->setCode('field-platovyvymer-typmzdy')
            ->setDataType('int')
            ->setRequiredInDatabase(true)
//            ->setCaptionColumnClass('col-12 col-sm-2 caption')
//            ->setDataColumnClass('col-12 col-sm-4')
            ->setFieldCssClass('form-control')
            ->setSelectSql('SELECT Kod, Popis FROM S_TypMzdy WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Kategorie', 'PlatovyVymer')->setCaption('Kategorie_zamestnance')
            ->setCode('field-platovyvymer-kategorie')
            ->setRequiredInDatabase(true)
            ->setDataType('int')
            ->setFieldCssClass('form-control')
            ->setSelectSql('SELECT Kod, Popis FROM S_KategorieZamestnance WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('PlatovyVymer', 'PlatovyVymer')->setCaption('Mzdovy_vymer')
            ->setCode("field-platovyvymer");
    }


    private function addFieldsEvidencniStav()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-evidencnistav')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('evidencnistav');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-evidencnistav')
            ->setCaption(TextManager::_('Evidencni_stav'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'EvidencniStav')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-evidencnistav-rc');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('OsobniCislo', 'EvidencniStav')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-evidencnistav-osobnicislo');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('DatumNastupu', 'EvidencniStav')->setReadOnly(true)->setDataType('date')
            ->setCode('field-evidencnistav-datumnastupu');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_DLE_PP))// podle zacatku mesice
        ->setFieldName('PlatnostOd', 'EvidencniStav')
            ->setCode('field-evidencnistav-platnostod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodEvidencnihoStavu', 'EvidencniStav')->setCaption('Evidencni_stav')
            ->setCode('field-kodevidencnihostavu')
            ->setFieldCssClass('form-control')
            ->setSelectSql('SELECT Kod, Popis FROM TypEvidencnihoStavu WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');
    }


    private function addFieldsKalendar()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-kalendar')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('kalendar');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-kalendar')
            ->setCaption(TextManager::_('Kalendar'));

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle Osoba.RC
        ->setFieldName('RC', 'PrirazeniKalendare')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-kalendar-rc');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('OsobniCislo', 'PrirazeniKalendare')->setReadOnly(true)->setMaxLength(15)
            ->setCode('field-kalendar-osobnicislo');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)// nastavi se podle PP
        ->setFieldName('DatumNastupu', 'PrirazeniKalendare')->setReadOnly(true)->setDataType('date')
            ->setCode('field-kalendar-datumnastupu');

        $this->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_DLE_PP))// podle zacatku mesice
        ->setFieldName('PlatnostOd', 'PrirazeniKalendare')
            ->setCode('field-kalendar-platnostod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Kalendar', 'PrirazeniKalendare')->setCaption('Kalendar')
            ->setCode('field-kalendar')
            ->setSelectShowCode(true)
            ->setFieldCssClass('form-control')
            ->setSelectSql('SELECT Kod, Popis FROM Kalendar WHERE (%FILTER%) ORDER BY %ORDER%', ' Aktivni = 1', 'Popis');
    }


    private function addUserLogFields($tableName)
    {
        $code = strtolower($tableName);

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Insert_UserName', $tableName)->setReadOnly(true)
            ->setCode("field-{$code}-insert_username")
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp', $tableName)->setReadOnly(true)
            ->setCode("field-{$code}-insert_timestamp");

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Update_UserName', $tableName)->setUpdateLog(true)
            ->setCode("field-{$code}-update_username")
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp', $tableName)->setUpdateLog(true)
            ->setCode("field-{$code}-update_timestamp");
    }


    protected function createNewFieldFromConfiguration($fieldSettings)
    {
        // pripadna nova pole pridava pouze v pripade, ze je zobrazen cely formular
        if ($this->status === self::STATUS_RC) {
            return null;
        }

        switch ($fieldSettings['fieldType'] ?? '') {
            case 'rozdeleni' :
                $f = new RozdeleniZamestnanceField();
                $this->rozdeleniFields[] = $f;
                return $f;

            case 'prukaz' :
                $f = new PrukazOsobyField();
                $this->prukazyFields[] = $f;
                return $f;

            case 'dokument' :
                $f = new DokumentOsobyField();
                $this->dokumentyFields[] = $f;
                return $f;
        }

        return parent::createNewFieldFromConfiguration($fieldSettings);
    }


    /**
     * Najde posledni zaznam vzdelani
     * @return array
     */
    protected function nactiPosledniVzdelani()
    {
        $paramsVzdelani = $this->database->createParameters();
        $sql =
            'SELECT KodVzdelani, OborVzdelani, DatumUkonceni FROM VzdelaniOsoby' .
            ' WHERE RC = ' . $paramsVzdelani->addParameter($this->pp ? $this->pp->rc : $this->osobaRC);

        return DBUtils::getSqlValue($this->database, $sql, $paramsVzdelani, true);
    }

    public function getSqlFromPart()
    {
        // datum, ke kteremu se nacitaji platne podrizene zaznamy
        $platnostTimestamp = $this->pp
            ? $this->pp->datumNastupu
            : time();

        $rowVzdelani = $this->nactiPosledniVzdelani();

        // Osoba
        $sql =
            'Osoba';

        // Pracovni pomer
        if ($this ->idPP) {
            // editace uz zapsaneho pracovniho pomeru
            $sql .=
                ' INNER JOIN PracovniPomer ON PracovniPomer.RC = Osoba.RC';
        } else {
            // jeste nezapsany PP
            $sql .=
                ' LEFT OUTER JOIN PracovniPomer ON 0 = 1';
        }

        // OsobniUdaj
        $sql .=
            ' LEFT OUTER JOIN OsobniUdaj ON OsobniUdaj.RC = Osoba.RC';

        $params = $this->database->createParameters();

        if ($this->jePerson) {
            $sql .=
                // Trvale bydliste
                ' LEFT OUTER JOIN AdresaOsoby TrvaleBydliste' .
                '  ON TrvaleBydliste.RC = Osoba.RC AND TrvaleBydliste.TrvaleBydliste = 1' .
                '  AND (TrvaleBydliste.PlatnostDo IS NULL OR TrvaleBydliste.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND TrvaleBydliste.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Prechodne bydliste
                ' LEFT OUTER JOIN AdresaOsoby PrechodneBydliste' .
                '  ON PrechodneBydliste.RC = Osoba.RC AND PrechodneBydliste.TrvaleBydliste = 0' .
                '  AND (PrechodneBydliste.PlatnostDo IS NULL OR PrechodneBydliste.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PrechodneBydliste.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Zdravotni stav
                ' LEFT OUTER JOIN ZdravotniStavOsoby' .
                '  ON ZdravotniStavOsoby.RC = Osoba.RC' .
                '  AND (ZdravotniStavOsoby.PlatnostDo IS NULL OR ZdravotniStavOsoby.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND ZdravotniStavOsoby.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Zdravotni pojisteni
                ' LEFT OUTER JOIN PojisteniOsobyU_ZP' .
                '  ON PojisteniOsobyU_ZP.RC = Osoba.RC' .
                '  AND (PojisteniOsobyU_ZP.PlatnostDo IS NULL OR PojisteniOsobyU_ZP.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PojisteniOsobyU_ZP.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Duchod
                ' LEFT OUTER JOIN Duchod' .
                '  ON Duchod.RC = Osoba.RC' .
                '  AND (Duchod.DatumOdebrani IS NULL OR Duchod.DatumOdebrani >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND Duchod.DatumPriznani <= ' . $params->addDateParameter($platnostTimestamp) .
                // Rodinny stav
                ' LEFT OUTER JOIN RodinnyStav' .
                '  ON RodinnyStav.RC = Osoba.RC' .
                '  AND (RodinnyStav.PlatnostDo IS NULL OR RodinnyStav.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND RodinnyStav.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Evidencni stav
                ' LEFT OUTER JOIN EvidencniStav' .
                '  ON EvidencniStav.RC = PracovniPomer.RC ' .
                '  AND EvidencniStav.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND EvidencniStav.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND (EvidencniStav.PlatnostDo IS NULL OR EvidencniStav.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND EvidencniStav.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Kalendar
                ' LEFT OUTER JOIN PrirazeniKalendare' .
                '  ON PrirazeniKalendare.RC = PracovniPomer.RC ' .
                '  AND PrirazeniKalendare.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND PrirazeniKalendare.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND (PrirazeniKalendare.PlatnostDo IS NULL OR PrirazeniKalendare.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PrirazeniKalendare.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // PlatovyVymer ...
                ' LEFT OUTER JOIN PlatovyVymer' .
                '  ON PlatovyVymer.RC = PracovniPomer.RC ' .
                '  AND PlatovyVymer.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND PlatovyVymer.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND (PlatovyVymer.PlatnostDo IS NULL OR PlatovyVymer.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PlatovyVymer.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // VyplaceniMzdy ...
                ' LEFT OUTER JOIN VyplaceniMzdy' .
                '  ON VyplaceniMzdy.RC = PracovniPomer.RC ' .
                '  AND VyplaceniMzdy.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND VyplaceniMzdy.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND (VyplaceniMzdy.PlatnostDo IS NULL OR VyplaceniMzdy.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND VyplaceniMzdy.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // ... a prislusny bankovni ucet
                ' LEFT OUTER JOIN BankovniUcet ON BankovniUcet.SerioveCislo = VyplaceniMzdy.SerioveCisloUctuVyplaty' .
                // Vzdelani osoby
                ' LEFT OUTER JOIN VzdelaniOsoby' .
                '  ON VzdelaniOsoby.RC = Osoba.RC' .
                ($rowVzdelani
                    ? '  AND VzdelaniOsoby.KodVzdelani = ' . $params->addParameter($rowVzdelani[0]) .
                    '  AND VzdelaniOsoby.OborVzdelani = ' . $params->addParameter($rowVzdelani[1]) .
                    '  AND VzdelaniOsoby.DatumUkonceni = ' . $params->addTimestampParameter($this->database->stringToTimestamp($rowVzdelani[2]))
                    : '  AND 0 = 1'
                );  // TODO: nacist posledni zaznam vzdelani
        }

        return new SqlStatement($sql, $params);
    }


    public function getWhereCondition()
    {
        $params = $this->database->createParameters();

        if ($this->idPP) {
            return new SqlStatement(
                'PracovniPomer.Id = ' . $params->addParameter($this->idPP),
                $params
            );
        } else if ($this->osobaRC) {
            return new SqlStatement(
                'Osoba.RC = ' . $params->addParameter($this->osobaRC),
                $params
            );
        }

        return '';
    }

    protected function renderPrimaryFields()
    {
        // Negeneruje nic, jedna se vzdy o novy zaznam
        // a pokud to je pres XML, kde je umoznena i editace, tak se editovany PP zadava v constructoru.
        // Navic implicitni hidden pole "record" se pouziva v xml
    }


    public function renderCommonFields()
    {
        parent::renderCommonFields();

        echo "<input type='hidden' name='novyZamestnanecFormStatus' value='{$this ->status}'/>";
    }


    public function renderMessages()
    {
        if (!$this->zobrazenoZXML) {
            parent::renderMessages();
        }
    }


    public function renderFooter($insert)
    {
        if (!$this->zobrazenoZXML) { // v XML tlacitka nezobrazuje
            parent::renderFooter($insert);
        }
    }


    /**
     * Nastavi pole ze zadanych tabulek jako readonly
     * @param string[] $tableNames
     */
    protected function nastavReadonly($tableNames)
    {
        /** @var FormField $field */
        foreach ($this->fields as $field) {
            if (in_array($field->tableName, $tableNames) || in_array($field->tableAlias, $tableNames)) {
                // jedna se o data k osobe s aktivnim pracovnim pomerem => nastavi je readonly
                //$field ->otherHtmlParams['disabled'] = 'disabled';
                //$field ->otherHtmlParams['readonly'] = 'readonly';
                if ($field->fieldName !== 'OsobniUdaj#RodneCislo') { // rodne cislo je osetreno samostatne (pouziva se jako hlavni identifikator nove osoby)
                    $field->setReadOnly(2); // kvuli kontroly required poli
                }
            }
        }
    }


    protected function loadRozdeleni()
    {
        foreach ($this->rozdeleniFields as $field) {
            $field->nactiRozdeleniOsoby($this->database, $this->idPP);
        }
    }


    protected function loadPrukazy()
    {
        $rc = $this->pp
            ? $this->pp->rc
            : $this->osobaRC;

        if ($rc) {
            foreach ($this->prukazyFields as $field) {
                $field->nactiPrukazOsoby($this->database, $rc, $this->pp ? $this->pp->datumNastupu : time());
            }
        }
    }


    protected function loadDokumenty()
    {
        $rc = $this->pp
            ? $this->pp->rc
            : $this->osobaRC;

        $idPP = $this->pp ? $this->pp->idPP : null;
        if (!$idPP && $rc) {
            // TODO: nacist (nejake) idpp osoby
        }

        if ($rc) {
            foreach ($this->dokumentyFields as $field) {
                $field->nactiDokumentyOsoby($this->database, $rc, $idPP);
            }
        }
    }

//    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
//    {
//        foreach ($this->fields as $field) {
//            if ($field->visible) {
//                if ($insert) {
//                    $field ->loadInsertValue($this ->database, $usePostValues, $values);
//                } else {
//                    $field ->loadEditValue($this->database, $usePostValues, $values, $canEditRecord);
//                }
//            }
//        }
//    }

    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
        if ($usePostValues && $_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // Bylo odeslano jen RC. Pro vsechna ostatni pole pouzije defaultValue.
            parent::loadFieldValues($insert, $values, false, $canEditRecord);
            // ... jen RC preplacne odeslanou hodnotou
            $this->fRC->loadInsertValue($this->database, $usePostValues);
            //$this ->status = self::STATUS_ALL;

            $this->loadPrukazy();
        } else {
            //parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);
            foreach ($this->fields as $field) {
                if ($field->visible) {
                    if ($insert && !$field->readOnly) {  // readonly pole nejdou editovat ani pri insertu
                        $field->loadInsertValue($this->database, $usePostValues, $values);
                    } else {
                        $field->loadEditValue($this->database, $usePostValues, $values, $canEditRecord);
                    }
                }
            }

            if ($values && !$usePostValues) {
                $this->loadRozdeleni();
                $this->loadDokumenty();
                $this->loadPrukazy();
                $this->fRCError->analyzujRC($this->fRC->value);
            }
        }
    }


    /**
     * Podle zadaneho RC zjisti, jestli uz osoba v databazi existuje
     * @param $rc
     * @return null|InfoPodleRC
     * @throws Exception
     */
    private function overRC($rc)
    {
        // nacte informace o osobe podle zadaneho rodneho cisla
        $infoPodleRC = NovyZamestnanecUtils::nactiInfoPodleRC($this->database, $rc, $this->idPP);

        if ($this->isInsert() && $infoPodleRC && $infoPodleRC->idAktivnihoPP) {
            // osoba ma aktivni pracovni pomer
            $prava = Prava::vratPrava('zamest_novy');
            if (!$prava->priznak) {
                // uzivatel nema pravo pridavat PP pro aktivni osoby
                throw new Exception(TextManager::_('Chyba_novy_zamestnanec_osoba_ma_aktivni_pracovni_pomer'));
            }
        }

        return $infoPodleRC;
    }

    /**
     * Doplni hodnoty podle zadaneho rc
     * @param string $rc
     * @return bool|int
     */
    private function handleActionRC($rc)
    {
        if ($this->fRCError->analyzaRC->datumNarozeni) {
            $this->fDatumNarozeni->defaultValue = StringUtils::format('date', $this->fRCError->analyzaRC->datumNarozeni);
        }

        if ($this->fRCError->analyzaRC->pohlavi) {
            $this->fPohlavi->defaultValue = $this->fRCError->analyzaRC->pohlavi;
        }

        if ($this->fCisloPojistence && isset($this->fRCError->analyzaRC)) {
            if (!$this->fRCError->analyzaRC->ecp) {
                $this->fCisloPojistence->defaultValue = $rc;
            }
        }
    }

    public function loadFieldsConfiguration($fieldsConfArray)
    {
        parent::loadFieldsConfiguration($fieldsConfArray);

        if ($this->status === self::STATUS_ALL && $this->existujeAktivniPP) {
            // osoba ma aktivni pracovni pomer
            $this->nastavReadonly($this->tablesOsoba);
//        } else if ($this ->status === self::STATUS_ALL  &&  $this ->posledniPlatnyPPDo) {
//            // osoba ma neaktivni pracovni pomer - zakaze editaci poli z tabulky Osoba a OsobniUdaj
//            $this->nastavReadonly(['Osoba', 'OsobniUdaj']);
        }
    }

    function loadConfiguration()
    {
        if ($this->configurationLoaded) {
            return;
        }

        parent::loadConfiguration();

        // nacte navic konfiguraci z XML formulare
        if ($this->novyZamestnanecConfiguration) {
            $this->objectConfiguration ->loadConfigurationArray($this->novyZamestnanecConfiguration);
        }

        $this->configurationLoaded = true;
    }


    private function zpracujInfoPodleRC($infoPodleRC)
    {
        if ($infoPodleRC) {
            $this->osobaRC = $infoPodleRC ->rc;
            $this->posledniPlatnyPPDo = $infoPodleRC ->posledniPlatnyPPDo;

            if ($infoPodleRC ->dalsiPP  ||  ($infoPodleRC ->idAktivnihoPP  && $infoPodleRC ->idAktivnihoPP != $this ->idPP)) {
                $this ->existujeAktivniPP = true;
                MessageManager::addWarning(TextManager::_('Existuje_aktivni_pp_nelze_editovat_osobni'), null, 'existuje_aktivni_pp'); // id zabrani nasobnemu zobrazeni hlasky
            } else if ($infoPodleRC ->posledniPlatnyPPDo) {
                MessageManager::addWarning(TextManager::_('Existuje_neaktivni_pp'), null, 'existuje_neaktivni_pp');
            }
        }
    }


    public function handleAction()
    {
        $this ->addFieldsRodneCislo();

        try {

            if (isset($_POST) && $_POST) {
                $this->status = $_POST['novyZamestnanecFormStatus'] ?? self::STATUS_RC;

                // overi zadane RC
                $rc = isset($_POST['OsobniUdaj#RodneCislo'])
                    ? trim($_POST['OsobniUdaj#RodneCislo'])
                    : null;

                // overi existenci osoby se zadanym RC v databazi
                $infoPodleRC = $this->overRC($rc);

                //if ($this->status === self::STATUS_ALL  ||  $this ->fRCError ->lzePokracovat()) {
                if ($this ->fRCError ->lzePokracovat()) {
                    $this ->zpracujInfoPodleRC($infoPodleRC);

                    $this->addNovyZamestnanecFields();

                    if ($this->status === self::STATUS_RC) {
                        // v predchozim kroku bylo odeslano jen RC - doplni vychozi datum narozeni atd.
                        $this->handleActionRC($rc);
                    }

                    // nastavi status na "vse" (=zobrazeni celeho formulare)
                    $this ->status = self::STATUS_ALL;

                    // pred nactenim zaznamu musi nacist konfiguraci, aby doplnil uzivatelsky definovana pole
                    $this ->loadConfiguration();

                    if ($infoPodleRC) {
                        $data = $this->loadRecord();
                        $this->data = $data
                            ? new FormRowArray($this ->fields, $data)
                            : null;
                    }
                }
            } else if ($this ->idPP) {
                $rodneCislo = NovyZamestnanecUtils::vratRodneCisloPracovnihoPomeru($this ->database, $this ->idPP);
                if ($rodneCislo) {
                    // overi existenci osoby se zadanym RC v databazi a nacte jeho pripadne dalsi PP
                    $infoPodleRC = $this->overRC($rodneCislo);
                    $this->zpracujInfoPodleRC($infoPodleRC);
                }

                $this->addNovyZamestnanecFields();
            }

            return parent::handleAction();
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }
    }


    /**
     * Vytvori DBRecords pro tabulku VyplaceniMzdy a pripadne BankovniUcet
     * @param PracovniPomer $pp
     * @return array
     */
    private function createUcetVyplatyRecords($pp)
    {
        $jeZadanUcet = isset($_POST['BankovniUcet#CisloUctu'])  &&  $_POST['BankovniUcet#CisloUctu'] != '';

        $result = [];

        if ($jeZadanUcet) {
            $serioveCislo = null;

            $osobaRC = $this ->osobaRC;

            if ($pp) {
                $aktualniSerioveCislo = ZamestnanciUtils::najdiBankovniUcetPP($this ->database, $pp);
                $pocetPouzitiUctu = $aktualniSerioveCislo
                    ? ZamestnanciUtils::pocetPouzitiBankovnihoUctu($this ->database, $aktualniSerioveCislo)
                    : 0;

                if ($aktualniSerioveCislo  &&  $pocetPouzitiUctu <= 1) {
                    // Ucet pro vyplatu uz je prirazen => je potreba provest aktualizaci
                    // Ucet je pouzit jen u editovaneho PP => muze se provest update
                    $recBankovniUcet = $this->createDBRecord('BankovniUcet');
                    $this->createDBRecordFieldsForTable($recBankovniUcet, 'BankovniUcet', true, true);
                    $params = $this->database->createParameters();
                    $recBankovniUcet->setWhere('SerioveCislo = ' . $params->addParameter($aktualniSerioveCislo), $params);
                    $result['BankovniUcet'] = $recBankovniUcet;

                    return $result;
                } else {
                    // Ucet jeste neni vubec prirazen, nebo je aktualni ucet pouzit i u jineho PP ($pocetPouzitiUctu > 1)
                    $osobaRC = $pp ->rc;
                }
            }

            // zjisti, jestli uz k zamestnanci existuje stejny ucet (pri zadavani noveho PP k existujici osobe)
            if ($osobaRC) {
                $serioveCislo = ZamestnanciUtils::najdiBankovniUcetOsoby($this ->database, $osobaRC, $_POST['BankovniUcet#CisloUctu'], $_POST['BankovniUcet#KodBanky']);
            }

            if (!$serioveCislo) {
                // nenasel existujici ucet => vytvori novy
                $recBankovniUcet = $this->createDBRecord('BankovniUcet');
                $this->createDBRecordFieldsForTable($recBankovniUcet, 'BankovniUcet', true, true);
                $recBankovniUcet ->setWhere('0 = 1'); // aby sel record vlozit i pri editaci
                $result['BankovniUcet'] = $recBankovniUcet;
            }

            $recVyplaceni = $this ->createDBRecord('VyplaceniMzdy');
            $this ->createDBRecordFieldsForTable($recVyplaceni, 'VyplaceniMzdy', true, true);
            if ($pp) {
                $params = $this ->database ->createParameters();
                $recVyplaceni->setWhere(
                    'RC = ' . $params ->addParameter($pp ->rc) .
                    ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                    ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu),    // platnost by nemela byt potreba resit - jedna se o novy pp => prepoklada se, ze bude jen jeden zaznam
                    $params);
            }
            if ($serioveCislo) {
                // uzivatel zadal ucet, ktery uz ma osoba prirazen u jineho PP
                $recVyplaceni['SerioveCisloUctuVyplaty'] = $serioveCislo;
            }
            $result['VyplaceniMzdy'] = $recVyplaceni;
        } else {
            // TODO: smazat, pokud byl uz zadan drive
        }

        return $result;
    }


    /**
     * @param PracovniPomer $pp
     * @return \Ron\DB\DBRecord[]
     */
    private function createZamestnanecRecords($pp = null)
    {
        $result = [];

        if (!$this ->existujeAktivniPP) {
            $recOsoba = $this->createDBRecord('Osoba');
            $this->createDBRecordFieldsForTable($recOsoba, 'Osoba', true, true);
            if ($pp) {
                $params = $this->database->createParameters();
                $recOsoba->setWhere('RC = ' . $params->addParameter($pp->rc), $params);
            } else if ($this ->osobaRC) {
                $params = $this->database->createParameters();
                $recOsoba->setWhere('RC = ' . $params->addParameter($this ->osobaRC), $params);
            }

            $result['Osoba'] = $recOsoba;
        }

        $recPP = $this ->createDBRecord('PracovniPomer');
        $this ->createDBRecordFieldsForTable($recPP, 'PracovniPomer', true, true);
        if ($pp) {
            $params = $this ->database ->createParameters();
            $recPP ->setWhere(
                'RC = ' . $params ->addParameter($pp ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu),
                $params);
        }

        $result['PracovniPomer'] = $recPP;

        $datumNastupu = $recPP['DatumNastupu'];

        foreach ($this ->fields as $formField) {
            if ($formField instanceof NovyZamestnanecPlatnostOdField) {
                $formField ->nastavPodleDatumuNastupu($datumNastupu);
            }
        }

        $platnostTimestamp = $pp
            ? $pp ->datumNastupu
            : time();

        $tables = array_merge($this ->tablesOsoba, $this ->tablesPracovniPomer);

//        $recOsobniUdaj = $this->createDBRecord('OsobniUdaj');
//        $this ->createDBRecordFieldsForTable($recOsobniUdaj, 'OsobniUdaj', true);
        foreach ($tables as $tableName) {
            $sqlTableName = null;
            $platnostDoFieldName = 'PlatnostDo';

            $whereParams = $this->database->createParameters();
            $whereSql = '';

            if (!$this->existujeAktivniPP) {
                switch ($tableName) {
                    case 'Osoba' :
                    case 'PracovniPomer' :
                        // uz bylo vytvoreno
                        break;

                    case 'OsobniUdaj' :
                        $sqlTableName = $tableName;
                        $platnostDoFieldName = null;
                        break;


                    case 'TrvaleBydliste' :
                        $sqlTableName = 'AdresaOsoby';
                        $whereSql = 'TrvaleBydliste = 1';
                        break;

                    case 'PrechodneBydliste' :
                        if ($this->pp || (isset($_POST['prechodnebydliste_existuje']) && $_POST['prechodnebydliste_existuje'])) {
                            // prechodne bydliste - pri vkladani jen kdyz je zaskrtnuto "... je rozdilne"
                            // pri editaci vzdy - pokud neni zaskrtnuto, tak musi byt smazano nebo ukonceno
                            $sqlTableName = 'AdresaOsoby';
                            $whereSql = 'TrvaleBydliste = 0';
                        }
                        break;

                    case 'ZdravotniStavOsoby' :
                        if (isset($_POST['ZdravotniStavOsoby#ZdravotniStav']) && $_POST['ZdravotniStavOsoby#ZdravotniStav'] !== '') {
                            // zdravotni stav - jen kdyz je vyplneno Zdravotni_stav
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'Duchod' :
                        if (isset($_POST['Duchod#TypDuchodu']) && $_POST['Duchod#TypDuchodu'] !== '') {
                            // duchod - jen kdyz je vybran typ duchodu
                            $sqlTableName = $tableName;
                            $platnostDoFieldName = 'DatumOdebrani';
                        }
                        break;


                    case 'PojisteniOsobyU_ZP' :
                        if (isset($_POST['PojisteniOsobyU_ZP#KodZdravotniPojistovny']) && $_POST['PojisteniOsobyU_ZP#KodZdravotniPojistovny']) {
                            // zdravotni pojistovna - jen kdyz je vybrana pojistovna
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'VzdelaniOsoby' :
                        if (isset($_POST['VzdelaniOsoby#KodVzdelani']) && $_POST['VzdelaniOsoby#KodVzdelani']) {
                            // vzdelani osoby - jen kdyz je vybran kod vzdelani
                            $sqlTableName = $tableName;
                            $platnostDoFieldName = null;
                        }
                        break;

                    case 'RodinnyStav' :
                        if (isset($_POST['RodinnyStav#KodRodinnehoStavu']) && $_POST['RodinnyStav#KodRodinnehoStavu'] !== '') {
                            // zdravotni pojistovna - jen kdyz je zadan kod stavu
                            $sqlTableName = $tableName;
                        }
                        break;
                }
            }

            switch ($tableName) {
                case 'PlatovyVymer' :
                    if (isset($_POST['PlatovyVymer#TypMzdy']) && $_POST['PlatovyVymer#TypMzdy'] !== '') {
                        $sqlTableName = $tableName;
                    }
                    break;

                case 'PrirazeniKalendare' :
                    if (isset($_POST['PrirazeniKalendare#Kalendar']) && $_POST['PrirazeniKalendare#Kalendar'] !== '') {
                        $sqlTableName = $tableName;
                    }
                    break;

                case 'EvidencniStav' :
                    if (isset($_POST['EvidencniStav#KodEvidencnihoStavu']) && $_POST['EvidencniStav#KodEvidencnihoStavu'] !== '') {
                        $sqlTableName = $tableName;
                    }
                    break;

                case 'BankovniUcet' :
                case 'VyplaceniMzdy' :
                    // vytvari se samostatne
                    break;
            }


            if ($sqlTableName) {
                //echo " Vytvorim pole pro $sqlTableName $tableName <br/> ";
                $rec = $this->createDBRecord($sqlTableName);
                $this->createDBRecordFieldsForTable($rec, $tableName, true);

                $rc = $pp ? $pp->rc : $this->osobaRC;

                if ($rc) {
                    // filtr pro osobu
                    $whereSql .=
                        ($whereSql ? ' AND ' : '') .
                        'RC = ' . $whereParams->addParameter($rc);

                    // filtr pro pracovni pomer
                    $fieldOsobni = $rec->findFieldByName('OsobniCislo');
                    if ($fieldOsobni && $pp) {
                        $whereSql .=
                            ' AND OsobniCislo = ' . $whereParams->addParameter($pp->osobniCislo) .
                            ' AND DatumNastupu = ' . $whereParams->addDateParameter($pp->datumNastupu);
                    }

                    // filtr podle platnosti
                    if ($platnostDoFieldName) {
                        $whereSql .=
                            ' AND (' . $platnostDoFieldName . ' >= ' . $whereParams->addDateParameter($platnostTimestamp) . ' OR ' . $platnostDoFieldName . ' IS NULL)';
                    }

                    $rec->setWhere($whereSql, $whereParams);
                }
                $result[$tableName] = $rec;
            }
        }


        $ucetRecords = $this ->createUcetVyplatyRecords($pp);
        if ($ucetRecords) {
            $result = array_merge($result, $ucetRecords);
        }

//        foreach ($this ->prukazyFields as $prukazField) {
//            MessageManager::addDebugLog("Vytvarim DB record pro prukaz {$prukazField ->typPrukazu}");
//            $rec = $prukazField ->getPrukazRecord($this ->database, $this ->pp ? $this ->pp ->rc : $this ->osobaRC, $this, $platnostTimestamp);
//            if ($this ->pp || $rec['CisloPrukazu']) {
//                //var_dump($rec);
//                throw new Exception("Vytvarim DB record pro prukaz {$prukazField ->typPrukazu}");
//                // pri vkladani zaznam prida, jen kdyz je vyplneno cislo
//                // pri editaci (tj. uz je prirazen PP) jej prida vzdy, aby bylo mozne mazani
//                $records['prukaz_' . $prukazField->typPrukazu] = $rec;
//            }
//        }

        return $result;
    }


    private function nactiUdajeOsoby($rc)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Jmeno, Prijmeni, Titul FROM Osoba' .
            ' WHERE RC = ' . $params ->addParameter($rc),
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            // nemelo by nastat
            throw new Exception('Chyba ve funkci nactiUdajeOsoby - osoba nenalezena!');
        }

        return [
            'Jmeno' => $row[0],
            'Prijmeni' => $row[1],
            'Titul' => $row[2]
        ];
    }


    /**
     * @param DBRecord $recordBankovniUcet
     * @param $rc
     * @param array|DBRecord $recOsoba
     * @param DBRecord $recPP
     * @throws Exception
     */
    private function doplnBankovniUcet($recordBankovniUcet, $rc, $recOsoba, $recPP)
    {
        if ($f = $recordBankovniUcet ->findFieldByName('JmenoUctu')) {
            if (!$recOsoba) {
                $recOsoba = $this ->nactiUdajeOsoby($rc);
            }

            // pouzije puvodni masku jmena uctu i pri update (updatuje se jen ucet, ktery neni pouzity nikde jinde)
            $formFieldIndex = $this ->findFieldIndexByCode('field-bankovniucet-jmenouctu');

            // nastavi jmeno uctu podle jmena osoby
            $value = str_replace(
                ['%JMENO%', '%PRIJMENI%', '%TITUL%', '%OSOBNICISLO%', '%CISLOUCTU%'],
                [$recOsoba['Jmeno'], $recOsoba['Prijmeni'], $recOsoba['Titul'], $recPP['OsobniCislo'], $recordBankovniUcet['CisloUctu']],
                isset($formFieldIndex) ? $this ->fields[$formFieldIndex] ->defaultValue : $f ->value
            );

            $f ->setNewValue($value);
        }
    }


    /**
     * Ukonci platnost nacteneho zaznamu - pouzije ale novy objekt DBRecord, protoze v $rec jsou nacteny zmeny z POST
     * Zaznam ukonci ke dni predchazejicimu zadanou platnost
     * @param $rec
     * @param $platnostOd
     * @throws Exception
     */
    private function ukonciPlatnostPredchozihoZaznamu($rec, $platnostOd)
    {
        $platnostOdFieldName = $rec ->tableName === 'Duchod'
            ? 'DatumPriznani'
            : 'PlatnostOd';

        $platnostDoFieldName = $rec ->tableName === 'Duchod'
            ? 'DatumOdebrani'
            : 'PlatnostDo';

        // vytvori zaznam pro update PlatnostDo ...
        $updateRec = new RonPortalDBRecord($this ->database, $rec ->tableName, $this);
        $updateRec ->addStringField('RC', true);
        $updateRec ->addDateField($platnostOdFieldName, true);
        $updateRec ->addDateField($platnostDoFieldName, false, DateUtils::incDayStart($platnostOd, -1)); // 1 den pred
        $updateParams = $this ->database ->createParameters();
        $updateRec ->setWhere(
            'RC = ' . $updateParams ->addParameter($rec['RC']) .
            " AND $platnostOdFieldName = " . $updateParams ->addDateParameter($rec[$platnostOdFieldName]),
            $updateParams);
        // ... a aktualizuje jej
        $updateRec ->loadAndUpdateRecord();
    }


    /**
     * Provede vlozeni nebo update zadaneho zaznamu a pripadne ukonci platnost zaznamu, ktery predchazi tento zaznam
     * @param DBRecord $rec
     * @param int $datumNastupu     Datum nastupu se pouzije pro nastaveni platnostOd
     * @param bool $deleteRecord    Priznak, jestli se ma zaznam smazat, pokud existuje
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    private function updateOrInsertRecordOsobni($rec, $datumNastupu, $deleteRecord)
    {
        if ($this ->existujeAktivniPP) {
            // pokud existuje aktivni PP, neni povoleno aktualizovat osobni udaje
            return;
        }

        if ($rec ->tableName === 'Osoba' || $rec ->tableName === 'OsobniUdaj' ) {
            // tabulka Osoba - pouze update
            $rec ->updateOrInsert();
            return;
        }

        if (!$this ->posledniPlatnyPPDo) {
            // uplne novy zamestnanec, ktery nemel drivejsi pracovni pomer - provede update
            $rec ->updateOrInsert();
            return;
        }

        if ($rec ->loadRecord()) {
            // zaznam uz existuje
            $platnostOdFieldName = $rec ->tableName === 'Duchod' ? 'DatumPriznani' : 'PlatnostOd';
            $fieldPlatnostOd = $rec ->findFieldByName($platnostOdFieldName);
            // pokud je datum nastupu v minulosti, tak se platnost nastavi na datum nastupu, jinak se pouzije aktualni den
            $platnostOd = $datumNastupu < time() ? $datumNastupu : DateUtils::today();

            if ($deleteRecord) {
                if ($this ->posledniPlatnyPPDo  &&  $fieldPlatnostOd ->originalValue <= $this ->posledniPlatnyPPDo) {
                    // zaznam zasahuje do predchoziho PP - musi jej nechat v databazi - pouze nastavi platnost do
                    $this->ukonciPlatnostPredchozihoZaznamu($rec, $platnostOd);
                } else {
                    $rec ->deleteRecord();
                }
            } else if ($rec ->isChanged()) {

                if ($this ->posledniPlatnyPPDo  &&  $fieldPlatnostOd ->originalValue <= $this ->posledniPlatnyPPDo) {
                    // zaznam zasahuje do predchoziho PP - musi jej ukoncit a pak se musi vlozit novy

                    $this ->ukonciPlatnostPredchozihoZaznamu($rec, $platnostOd);

//                    // vytvori zaznam pro update PlatnostDo ...
//                    $updateRec = new RonPortalDBRecord($this ->database, $rec ->tableName, $this);
//                    $updateRec ->addStringField('RC', true);
//                    $updateRec ->addDateField($platnostOdFieldName, true);
//                    $updateRec ->addDateField($platnostDoFieldName, false, DateUtils::incDayStart($platnostOd, -1)); // 1 den pred
//                    $updateParams = $this ->database ->createParameters();
//                    $updateRec ->setWhere(
//                        'RC = ' . $updateParams ->addParameter($rec['RC']) .
//                        " AND $platnostOdFieldName = " . $updateParams ->addDateParameter($rec[$platnostOdFieldName]),
//                        $updateParams);
//                    // ... a aktualizuje jej
//                    $updateRec ->loadAndUpdateRecord();

                    // A nakonec vlozi novy zaznam
                    $fieldPlatnostOd ->setNewValue($platnostOd);
                    $rec ->insertRecord();
                    //throw new Exception("AKTUALIZACE UDAJU TABULKY {$rec ->tableName} JESTE NENI IMPLEMENTOVANA!");
                } else {
                    // zaznam nezasahuje do starsiho PP - muzu provest update
                    $rec ->updateRecord();
                }
            }
        } else if (!$deleteRecord) {
            $rec ->insertRecord();
        }
    }


    /**
     * @param PrukazOsobyField $prukazField
     * @throws Exception
     */
    private function ukonciPlatnostStarychPrukazu($prukazField)
    {
        MessageManager::addDebugLog('Ukoncim platnost starych prukazu ...');
        // neni zadan zadny prukaz => projde vsechny aktualne platne prukazy a ukonci jejich platnost
        $platnostTimestamp = $this ->pp ? $this ->pp ->datumNastupu : time();

        $rec = $prukazField ->getPrukazRecord($this ->database, $this ->pp ? $this ->pp ->rc : $this ->osobaRC, $this, $platnostTimestamp);
        $rec ->checkMultiple = false;
        $rec ->loadRecords();

        while ($rec ->next()) {
            if (!$this ->posledniPlatnyPPDo  ||  $rec['PlatnostOd'] > $this ->posledniPlatnyPPDo) {
                // nema predchozi pracovni pomer, nebo prukaz nezasahuje do aktualniho PP => muzu jej smazat
                MessageManager::addDebugLog("Smazu prukaz cislo {$rec['#CisloPrukazu']}");
                $deleteParams = $this ->database ->createParameters();
                $rec ->deleteRecord('CisloPrukazu = ' . $deleteParams ->addParameter($rec['#CisloPrukazu']), $deleteParams);
            } else {
                // prukaz zasahuje do starsiho PP => ukoncim jej
                $updateRec = new PrukazRecord($this ->database, $this, $rec['RC'], (int)$rec['TypPrukazu'], $rec['CisloPrukazu']);
                $yesterday = DateUtils::incDayStart(DateUtils::today(), -1);
                $updateRec ->fPlatnostDo ->setNewValue($platnostTimestamp < $yesterday ? $platnostTimestamp : $yesterday);
                $updateRec ->loadAndUpdateRecord();
            }
        }
    }


    private function zapisPrukazy($rc)
    {
        foreach ($this ->prukazyFields as $prukazField) {
            if ($prukazField ->visible  &&  !$prukazField ->readOnly) {
                $cislo = trim($_POST[$prukazField->fieldName]);
                if ($cislo) {
                    // je zadano cislo => insert nebo update
                    $rec = $prukazField->getPrukazRecord($this->database, $rc, $this);
                    $params = $this->database->createParameters();

                    if ($rec->loadRecord('CisloPrukazu = ' . $params->addParameter($cislo), $params)) {
                        // prukaz se zadanym cislem existuje => provede update
                        $rec->updateRecord('CisloPrukazu = ' . $params->addParameter($cislo), $params);
                    } else {
                        // ukonci platnost starych prukazu ...
                        $this->ukonciPlatnostStarychPrukazu($prukazField);
                        // ... a vlozi novy prukaz
                        $rec->insertRecord();
                    }
                } else {
                    $this->ukonciPlatnostStarychPrukazu($prukazField);
                }
            }
        }
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function insertRecord()
    {
        if ($_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // bylo odeslano jen RC => zobrazi znovu formular
            // zpracovani odeslanych udaju bylo provedeno v handleActionRC
            return false;
        }

        if (!$this->checkRequiredFields(false)) {
            return false;
        }

        $this ->loadFieldValues(true, $this ->data, true, true); // aby pri pripadne dalsi chybe ve formulari byly nacteny hodnoty

        $records = $this ->createZamestnanecRecords();

//        foreach ($this ->prukazyFields as $prukazField) {
//            $rec = $prukazField ->getPrukazRecord($this ->database, $this ->pp ->rc, $this);
//            if ($rec['CisloPrukazu']) {
//                // zaznam prida, jen kdyz je vyplneno cislo
//                $records['prukaz_' . $prukazField->typPrukazu] = $rec;
//            }
//        }

        // TODO: Pridat zaznamy pro rozdeleni pro moznost zpracovani v XML! ! !

        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_BEFORE_POST', $records, 'insert', false, true);

        $this ->database ->beginTransaction();
        try {
            // Pokud se jedna o novou osobu, tak vygeneruje nove ID osoby
            $rc = $this ->osobaRC ?: ZamestnanciUtils::getIdOsoby($this ->database);
            $recPP = $records['PracovniPomer'];
            $recOsoba = $records['Osoba'] ?? null;

            $serioveCisloUctu = null;

            foreach ($records as $tableName => $record) {
                $record['RC'] = $rc;

                if ($record != $recPP  &&  $f = $record ->findFieldByName('OsobniCislo')) {
                    // u tabulek napojenych na pracovni pomer doplni OsobniCislo a DatumNastupu
                    $f ->setNewValue($recPP['OsobniCislo']);
                    $record['DatumNastupu'] = $recPP['DatumNastupu'];
                }

                if (strtolower($tableName) === 'vyplacenimzdy' && isset($serioveCisloUctu)) {
                    $record['SerioveCisloUctuVyplaty'] = $serioveCisloUctu;
                }

                if (strtolower($tableName) === 'bankovniucet') {
                    // doplni nazev bankovniho uctu podle zadane masky
                    $this ->doplnBankovniUcet($record, $rc, $recOsoba, $recPP);
                }

                if ($this ->osobaRC  &&  ($record === $recOsoba  ||  $record === ($records['OsobniUdaj'] ?? null))) { // tabulky Osoba a OsobniUdaj
                    // novy pracovni pomer k existujici osobe - zaznam osoby jen updatuje
                    $record->loadAndUpdateRecord();
                } else if ($this ->osobaRC  &&  !$record ->findFieldByName('OsobniCislo')) {
                    // existujici osoba - je potreba zkontrolovat, jestli existuje podrizeny zaznam platny k aktualnimu dni
                    // a pripadne jej ukoncit
                    $this ->updateOrInsertRecordOsobni($record, $recPP['DatumNastupu'], false);
                } else {
                    // jedna se o uplne novou osobu, nebo je to zaznam podrizeny pracovnimu pomeru - staci provest insert
                    $record->insertRecord();
                }

                if (strtolower($tableName) === 'bankovniucet') {
                    // zaznamena si nove seriove cislo bankovniho uctu
                    $serioveCisloUctu = $record['SerioveCislo'];
                }
            }

            foreach ($this ->rozdeleniFields as $rozdeleniField) {
                $rozdeleniField ->ulozRozdeleni($this ->database, $rc, $recPP['OsobniCislo'], $recPP['DatumNastupu'], $this);
            }

            $this ->zapisPrukazy($rc);

            $pp = $this ->pp;
            foreach ($this ->dokumentyFields as $dokumentField) {
                if (!$pp) {
                    $pp = PracovniPomer::nactiPP($this ->database, $recPP['Id']);
                }
                $idDokumentu = $dokumentField ->ulozDokument($this ->database, $pp, $this);
                if ($idDokumentu) {
                    $this ->noveDokumenty[] = $idDokumentu;
                }
            }

            $this ->database ->commitTransaction();

            $this ->idPP = $records['PracovniPomer']['Id'];
        } catch (Exception $e) {
            $this ->database ->rollbackTransaction();
            throw $e;
        }

        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_AFTER_POST', $records, 'insert', false, true);

        return true;
    }


    /**
     * Editace existujicich udaju, pripadne doplneni
     * @return bool
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    public function editRecord()
    {
        $this ->noveDokumenty = [];

        if (!$this ->pp) {
            throw new Exception('Chyba v metode NovyZamestnanecForm::editRecord - neni zadan idPP!');
        }

        if ($_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // nemelo by nastat (bylo odeslano jen RC)
            return false;
        }

        if (!$this->checkRequiredFields(false)) {
            return false;
        }
        //$pp = PracovniPomer::nactiPP($this ->database, $this ->idPP);

        $this ->loadFieldValues(false, $this ->data, true, true); // aby pri pripadne dalsi chybe ve formulari byly nacteny hodnoty

        $records = $this ->createZamestnanecRecords($this ->pp);
        $recPP = $records['PracovniPomer'];
        $recOsoba = $records['Osoba'] ?? null;

        $recordLoaded = [];

//        foreach ($this ->prukazyFields as $prukazField) {
//            $rec = $prukazField ->getPrukazRecord($this ->database, $this ->pp ->rc, $this);
//            // zaznam prida vzdy - i kdyz neni vyplneno cislo - pro moznost smazani
//            $records['prukaz_' . $prukazField->typPrukazu] = $rec;
//        }


        // nejdrive nacte existujici zaznamy
        foreach ($records as $tableName => $record) {
            if (strtolower($tableName) ===  'pracovnipomer') {
                // mohlo dojit k editaci osobniho cisla a datumu nastupu => zaznamena zmenu
                if ($record['OsobniCislo']) {
                    $this ->pp ->osobniCislo = $record['OsobniCislo'];
                }

                if ($record['DatumNastupu']) {
                    $this ->pp ->datumNastupu = $record['DatumNastupu'];
                }
            }

            // zkusi nacist existujici zaznam a zapamatuje si, jestli byl nalezen
            $recordLoaded[$tableName] = $record->loadRecord();

            if (!$recordLoaded[$tableName]) {
                // novy podrizeny zaznam - doplni RC
                $record['RC'] = $this ->pp ->rc;

                // ... a pripadne osobni cislo
                if ($f = $record ->findFieldByName('OsobniCislo')) {
                    // u tabulek napojenych na pracovni pomer doplni OsobniCislo a DatumNastupu
                    $f ->setNewValue($this ->pp ->osobniCislo);
                    $record['DatumNastupu'] = $this ->pp ->datumNastupu;
                }

                // ... a u bankovniho uctu doplni nazev podle preddefinovane masky
                if (strtolower($tableName) === 'bankovniucet') {
                    $this ->doplnBankovniUcet($record, $this ->pp ->rc, $recOsoba, $recPP);
                }
            }
        }

        // zavola udalost before post
        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_BEFORE_POST', $records, 'update', false, true);

        // pouzije se po vlozeni noveho bankovniho uctu (doplni se do zaznamu VyplaceniMzdy)
        $serioveCisloUctu = null;

        // a provede update/vlozeni vsech zaznamu
        foreach ($records as $tableName => $record) {
            if (strtolower($tableName) === 'vyplacenimzdy' && isset($serioveCisloUctu)) {
                $record['SerioveCisloUctuVyplaty'] = $serioveCisloUctu;
            }

            $deleteRecord = false;
            if (strtolower($tableName) === 'prechodnebydliste') {
                $deleteRecord = !(isset($_POST['prechodnebydliste_existuje']) && $_POST['prechodnebydliste_existuje']);
            }

//            if ($this ->osobaRC  &&  $record === $recOsoba  ||  $record === $records['OsobniUdaj']) { // tabulky Osoba a OsobniUdaj
//                // novy pracovni pomer k existujici osobe - zaznam osoby jen updatuje
//                $record->loadAndUpdateRecord();
//            } else
            if (strtolower($tableName) === 'bankovniucet') {
                $this->doplnBankovniUcet($record, $this ->pp ->rc, $recOsoba, $recPP);
                $record ->updateOrInsert();
            } else if (!$record ->findFieldByName('OsobniCislo')) {
                // existujici osoba - je potreba zkontrolovat, jestli existuje podrizeny zaznam platny k aktualnimu dni
                $this ->updateOrInsertRecordOsobni($record, $recPP['DatumNastupu'], $deleteRecord);
            } else {
                if ($recordLoaded[$tableName]) {
                    if ($deleteRecord) {
                        $record->doDelete('');
                    } else {
                        $record->updateRecord();
                    }
                } else if (!$deleteRecord) {
                    $record->insertRecord();
                }
            }

            if (strtolower($tableName) === 'bankovniucet') {
                // zaznamena si nove seriove cislo bankovniho uctu (pro pripadne vlozeni do zaznamu VyplaceniMzdy)
                $serioveCisloUctu = $record['SerioveCislo'];
            }
        }

        foreach ($this ->rozdeleniFields as $rozdeleniField) {
            $rozdeleniField ->ulozRozdeleni($this ->database, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu, $this);
        }

        $this ->zapisPrukazy($this ->pp ->rc);

        foreach ($this ->dokumentyFields as $dokumentField) {
            $idDokumentu = $dokumentField ->ulozDokument($this ->database, $this ->pp, $this);
            if ($idDokumentu) {
                $this ->noveDokumenty[] = $idDokumentu;
            }
        }

        return true;
    }


}
