<?php
/**
 * stanislav.elbl, 14.02.2022 16:25
 */

namespace RonPortal\Zamestnanci;

use Ron\DB\Database;
use Ron\DB\DBUtils;

class InfoPodleRC
{
    /** @var string     RC osoby (Osoba.RC) */
    public $rc;
    /** @var string     Rodne cislo (z tabulky OsobniUdaj.RodneCislo) */
    public $rodneCislo;
    /** @var int        Id jednoho aktivniho PP */
    public $idAktivnihoPP;
    /** @var int[]      Pole Id aktivnich PP */
    public $dalsiPP;
    /** @var int        Platnost do posledniho ukonceneho pracovniho pomeru */
    public $posledniPlatnyPPDo;
}


class NovyZamestnanecUtils
{
    /**
     * Nacte informace o osobe se zadanym rodnym cislem (podle tabulky OsobniUdaj.RodneCislo). Pokud jeste neexistuje, vraci null.
     * @param Database $database
     * @param string $rodneCislo
     * @param int $preferovanyIdPP
     * @return InfoPodleRC|null     Vraci objekt s informacemi o pracovnich pomerech
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiInfoPodleRC($database, $rodneCislo, $preferovanyIdPP)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT Osoba.RC, OsobniUdaj.RodneCislo, PracovniPomer.Id, PracovniPomer.DatumUkonceni' . // , PracovniPomer.DatumNastupu' .
            ' FROM Osoba INNER JOIN OsobniUdaj' .
            '  ON OsobniUdaj.RC = Osoba.RC' .
            ' LEFT OUTER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = Osoba.RC' .
            '  AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
            //'  AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni > ' . $params->addDateParameter(time()) . ')' .
            ' WHERE OsobniUdaj.RodneCislo = ' . $params->addParameter($rodneCislo);

        $cursor = $database->getSqlResult($sql, $params);
        $row = null;
        $dalsiPP = [];
        $idPP = null;
        $posledniPPPlatnyDo = null; // maximalni platnost do (tj. posledni den, kdy mel platny pracovni pomer)

        while ($nextRow = $cursor->fetch()) {
            if (!$row) {
                // prvni radek si zapamatuje
                $row = $nextRow;
            }

            $platnostDo = $cursor ->stringToTimestamp($nextRow[3]);

            if (!$platnostDo  ||  $platnostDo > time()) {
                // aktivni pracovni pomer

                if (!$idPP) {
                    if (!$preferovanyIdPP || (int)$nextRow[2] == $preferovanyIdPP) {
                        $idPP = (int)$nextRow[2];
                    }
                }

                if ((int)$nextRow[2] != $idPP) {
                    $dalsiPP[] = (int)$nextRow[2];
                }
            } else {
                // jiz ukonceny pracovni pomer
                if (!$posledniPPPlatnyDo  ||  $platnostDo > $posledniPPPlatnyDo) {
                    $posledniPPPlatnyDo = $platnostDo;
                }
            }
        }

        $cursor ->close();

        if (!$row) {
            return null;
        }

        $result = new InfoPodleRC();
        $result ->rc = $row[0];
        $result ->rodneCislo = $row[1];
        $result ->idAktivnihoPP = $idPP;
        $result ->dalsiPP = $dalsiPP;
        $result ->posledniPlatnyPPDo = $posledniPPPlatnyDo;

        return $result;

//        return $row
//            ? [
//                'rc' => $row[0],            // Osoba.RC
//                'rodneCislo' => $row[1],    // Rodne cislo
//                'idPP' => $idPP,            // Id pracovniho pomeru osoby. V pripade, ze uz je u formulare nastaven (tj. jedna se o editaci), tak vraci ten nastaveny, jinak prvni nalezeny
//                'dalsiPP' => $dalsiPP,      // Seznam id dalsich aktivnich pracovnich pomeru
//                'posledniPPDo' => $idPP ? null : $posledniPPPlatnyDo    // Datum platnosti do posledniho platneho PP
//            ]
//            : null;
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @return string
     */
    public static function vratRodneCisloPracovnihoPomeru($database, $idPP)
    {
        $params = $database ->createParameters();
        $result = DBUtils::getSqlValue($database,
            'SELECT OsobniUdaj.RodneCislo' .
            ' FROM PracovniPomer, OsobniUdaj' .
            ' WHERE OsobniUdaj.RC = PracovniPomer.RC' .
            ' AND PracovniPomer.Id = ' . $params ->addParameter($idPP), $params);

        return $result === false ? null : $result;
    }

}
