<?php
/**
 * stanislav.elbl, 29.07.2021 18:11
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;

class PrukazOsobyField extends FormField
{

    /** @var int    Kod typu prukazu */
    public $typPrukazu;

    /** @var int    0=ne, 1=ano, 2=povinne */
    public $zadavatStat;

    //Primarni: RC 15, Typ int, CisloPrukazu 20
    //Nepovinne: VydalStat 10, PlatnostOd, PlatnostDo, Poznamka


    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_EDIT);

        $this ->zadavatStat = 0;

        $this
            ->setSelectSql('SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%','1 = 1', 'Popis')
            ->setPreferredValues(['CZ', 'SK']);
    }


    /**
     * Nacteni konfigurace
     * @param $key
     * @param $value
     */
    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'typPrukazu' :
                $this ->typPrukazu = (int)$value;
                if (!$this ->fieldName) {
                    $this ->fieldName = 'Prukaz' . $this ->typPrukazu;
                }
                break;

            case 'zadavatStat' :
                $this ->zadavatStat = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * Nacte prukaz prirazena zadane osobe k zadanemu dni
     * @param Database $database
     * @param string $rc
     * @param int $platnostKDatu
     */
    public function nactiPrukazOsoby($database, $rc, $platnostKDatu)
    {
        $prukazy = Prukaz::nactiPrukazyOsoby($database, $rc, $this ->typPrukazu, $platnostKDatu ?: time());

        $this ->value = [
            'cislo' => $prukazy ? $prukazy[0] ->cisloPrukazu : '',
            'platnostOd' => $prukazy ? StringUtils::format('date', $prukazy[0] ->platnostOd) : '',
            'platnostDo' => $prukazy ? StringUtils::format('date', $prukazy[0] ->platnostDo) : '',
            'vydalStat' => $prukazy ? $prukazy[0] ->vydalStat : ''
        ];
    }


    protected function loadPostPrukazy()
    {
        $this ->value = [
            'cislo' => $_POST[$this->fieldName],
            'platnostOd' => $_POST[$this->fieldName . '_PlatnostOd'],
            'platnostDo' => $_POST[$this->fieldName . '_PlatnostDo'],
        ];

        if ($this ->zadavatStat) {
            $this ->value['vydalStat'] = $_POST[$this->fieldName . '_VydalStat'];
        }
    }


    public function loadInsertValue($database, $usePostValues, $rowData = null)
    {
        if ($usePostValues) {
            $this ->loadPostPrukazy();
        } else {
            $this ->value = [
                'cislo' => '',
                'platnostOd' => '',
                'platnostDo' => '',
                'vydalStat' => ''
            ];
        }
    }


    public function loadEditValue($database, $usePostValues, $rowData, $canEditRecord)
    {
        if ($usePostValues) {
            $this ->loadPostPrukazy();
        } else {
            $this ->value = [
                'cislo' => '',
                'platnostOd' => '',
                'platnostDo' => '',
                'vydalStat' => ''
            ];
        }
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo
            '<div class="row">' .

            // cislo
            '<div class="col-12 col-sm-4">' .
            ($canEdit
                ? '<INPUT type="text" name="' . $this->fieldName . '" ' .
                    $this ->getHtmlParams() .
                    ' value="' . StringUtils::htmlTextConvert($value['cislo'] ?? '') . '">'
                : StringUtils::htmlTextConvert($value['cislo'] ?? '')) .
            '</div>' .

            // platnost od
            '<div class="col-6 col-sm-2 caption">' . TextManager::_('Platnost_od') . '</div>' .
            '<div class="col-6 col-sm-2">' .
            ($canEdit
                ? '<INPUT type="text" name="' . $this->fieldName . '_PlatnostOd" maxlength="10"' .
                    ' class="inputDate form-control ' . MessageManager::getClass($this->fieldName . '_PlatnostOd') . '" ' .
                    MessageManager::getTitleHtml($this->fieldName . '_PlatnostOd') .
                    ' value="' . StringUtils::htmlTextConvert($value['platnostOd'] ?? '') . '">'
                : StringUtils::htmlTextConvert($value['platnostOd'] ?? '')) .
            '</div>' .

            // platnost do
            '<div class="col-6 col-sm-2 caption">' . TextManager::_('do') . '</div>' .
            '<div class="col-6 col-sm-2">' .
            ($canEdit
                ? '<INPUT type="text" name="' . $this->fieldName . '_PlatnostDo" maxlength="10"' .
                    ' class="inputDate form-control ' . MessageManager::getClass($this->fieldName . '_PlatnostDo') . '" ' .
                    MessageManager::getTitleHtml($this->fieldName . '_PlatnostDo') .
                    ' value="' . StringUtils::htmlTextConvert($value['platnostDo'] ?? '') . '">'
                : StringUtils::htmlTextConvert($value['platnostDo'] ?? '')) .
            '</div>';

        // vydal stat
        if ($this ->zadavatStat) {
            echo '<div class="col-12 col-sm-6 padding-top-2 caption">' . TextManager::_('Vydal_stat') . '</div>';
            echo '<div class="col-12 col-sm-6 padding-top-2">';
            $this ->renderSelect($database, $this->fieldName . '_VydalStat', $value['vydalStat'] ?? '', null, $insert, $canEdit, $this ->zadavatStat == 2);
            echo '</div>';
        }


        echo
            '</div>';
    }


    public function getPrukazRecord($database, $rc, $logDescription, $platnostKDatu = null)
    {
        $rec = new PrukazRecord($database, $logDescription, $rc, $this ->typPrukazu, null, $platnostKDatu);

        $cislo = trim($_POST[$this ->fieldName]);

        if ($cislo != '') {
            $rec ->fCisloPrukazu ->setNewValue($cislo);

            $platnostStr = $_POST[$this->fieldName . '_PlatnostOd'];
            try {
                $rec->fPlatnostOd->setNewValue($platnostStr != ''
                    ? DateUtils::dateStringToTimestamp($platnostStr)
                    : null
                );
            } catch (\Exception $e) {
                throw new RonException($this ->caption . ': ' . $e ->getMessage(), $this->fieldName . '_PlatnostOd');
            }

            $platnostStr = $_POST[$this->fieldName . '_PlatnostDo'];
            try {
                $rec->fPlatnostDo->setNewValue($platnostStr != ''
                    ? DateUtils::dateStringToTimestamp($platnostStr)
                    : null
                );
            } catch (\Exception $e) {
                throw new RonException($this ->caption . ': ' . $e ->getMessage(), $this->fieldName . '_PlatnostDo');
            }

            if ($this ->zadavatStat) {
                $stat = $_POST[$this->fieldName . '_VydalStat'];
                $rec ->fVydalStat ->setNewValue($stat ?: null);
            }
        }

        return $rec;
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param object $logDescription
     */
    public function ulozPrukaz($database, $rc, $logDescription)
    {
        $rec = new PrukazRecord($database, $logDescription, $rc, $this ->typPrukazu);
        // TODO: co kdyz uz ma i starsi prukazy? (novy nastup existujiciho zamestnance)

        $cislo = trim($_POST[$this ->fieldName]);

        if ($cislo != '') {
            $rec ->fCisloPrukazu ->setNewValue($cislo);

            $platnostStr = $_POST[$this->fieldName . '_PlatnostOd'];
            $rec ->fPlatnostOd ->setNewValue( $platnostStr != ''
                ? DateUtils::dateStringToTimestamp($platnostStr)
                : null
            );

            $platnostStr = $_POST[$this->fieldName . '_PlatnostDo'];
            $rec ->fPlatnostDo ->setNewValue( $platnostStr != ''
                ? DateUtils::dateStringToTimestamp($platnostStr)
                : null
            );

            if ($this ->zadavatStat) {
                $stat = $_POST[$this->fieldName . '_VydalStat'];
                $rec ->fVydalStat ->setNewValue($stat ?: null);
            }

            $rec ->updateOrInsert();
        } else {
            $rec ->deleteRecord();
        }
    }
}
