<?php

namespace RonPortal\Zamestnanci;

use Ron\DB\Database;
use Ron\DB\DBException;


/**
 * Class RodinnyPrislusnik
 * @package Ron\RonPortal\Zamestnanci
 */
class RodinnyPrislusnik
{
    /** @var string     Id. Nacteno z pole RC_RodinnehoPrislusnika (primarni klic) */
    public $rc;

    public $jmeno;
    public $prijmeni;
    public $titul;

    public $datumNarozeni;

    public $rodinnyVztah;
    public $rodinnyVztahStr;

    public $zdravotniStav;
    public $zdravotniStavStr;
    public $drzitelZTPOd;

    /** @var int    Priznak, jestli se jedna o vyzivovanou osobu */
    public $vyzivovanaOsoba;
    /** @var int    Datum, od ktereho se jedna o vyzivovanou osobu */
    public $vyzivovanaOsobaOd;
    public $vyzivovanaOsobaDo;

    public $stejnaAdresa;
    /** @var int    Priznak, ze se jedna o kontaktni osobu */
    public $kontaktniOsoba;
    public $telefon;
    public $email;


    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->rc = $row[0];

        $this->prijmeni = $row[1];
        $this->jmeno = $row[2];
        $this->titul = $row[3];

        $this->datumNarozeni = $database->stringToTimestamp($row[7]);

        $this->rodinnyVztah = $row[8];
        $this->rodinnyVztahStr = $row[11];

        $this->zdravotniStav = $row[4];
        $this->zdravotniStavStr = $row[12];
        $this->drzitelZTPOd = $database->stringToTimestamp($row[6]);

        $this->vyzivovanaOsoba = $row[10];
        $this->vyzivovanaOsobaOd = $database->stringToTimestamp($row[16]);
        $this->vyzivovanaOsobaDo = $database->stringToTimestamp($row[17]);

        $this->stejnaAdresa = $row[9];
        $this->kontaktniOsoba = $row[13];
        $this->telefon = $row[14];
        $this->email = $row[15];
    }


    /**
     * @param Database $database
     * @param string $rc
     * @return RodinnyPrislusnik[]
     * @throws DBException
     */
    public static function nactiRodinuZamestnance($database, $rc)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT RodinnyPrislusnik.RC_RodinnehoPrislusnika,' . // 0
            ' RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.Titul,'.  // 1..3
            ' RodinnyPrislusnik.ZdravotniStav, RodinnyPrislusnik.DrzitelPrukazuZTTP, RodinnyPrislusnik.DatumPriznaniZTTP, RodinnyPrislusnik.DatumNarozeni,' . //3..7
            ' ClenoveRodiny.RodinnyVztah, ClenoveRodiny.StejnaAdresa, ClenoveRodiny.VyzivovanaOsoba, S_RodinnyVztah.Popis, S_ZdravotniStav.Popis,' . // 8..12
            ' RodinnyPrislusnik.KontaktniOsoba, Telefon, Email,' . //13..15
            ' ClenoveRodiny.VyzivovanaOsobaOd, ClenoveRodiny.VyzivovanaOsobaDo' . // 16,17

            "\nFROM RodinnyPrislusnik JOIN ClenoveRodiny ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) " .
            "  JOIN S_RodinnyVztah ON (S_RodinnyVztah.Kod = ClenoveRodiny.RodinnyVztah) " .
            "  JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = RodinnyPrislusnik.ZdravotniStav) " .
            "  LEFT JOIN S_AnoNe Kontakt ON (Kontakt.Kod = RodinnyPrislusnik.KontaktniOsoba) " .

            "\nWHERE ClenoveRodiny.RC  = " . $params->addParameter($rc) .
            ' AND (ClenoveRodiny.PartnerDatumOd IS NULL OR ClenoveRodiny.PartnerDatumOd <= ' . $params->addDateParameter(time()) . ')' .
            ' AND (ClenoveRodiny.PartnerDatumDo IS NULL OR ClenoveRodiny.PartnerDatumDo >= ' . $params->addDateParameter(time()) . ')' .

            "\nORDER BY RodinnyPrislusnik.DatumNarozeni";

        $cursor = $database->getSqlResult($sql,$params);

        $prislusnici = [];

        while ($row = $cursor->fetch()) {
            $prislusnik = new RodinnyPrislusnik();
            $prislusnik->nactiDataZRadku($database,$row);

            $prislusnici[] = $prislusnik;
        }

        return $prislusnici;
    }


    /**
     * Pokusi se najit ID rodinneho prislusnika dle RC
     * @param Database $database
     * @param string $rc
     * @return mixed
     * @throws DBException
     */
    public static function najdiIDRodinnehoPrislusnikaDleRC(Database $database, string $rc)
    {
        $params = $database->createParameters();
        $sql =
            'SELECT RC_RodinnehoPrislusnika FROM RodinnyPrislusnik' .
            ' WHERE RodneCisloRP = ' . $params->addParameter($rc);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }

}
