<?php

namespace RonPortal\Zamestnanci;

use Exception;
use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Common\RozdeleniOsobRecord;
use RonPortal\Controls\ComboBoxesRozdeleni;
use RonPortal\PracovniPomer;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;
use RonPortal\User;

class RozdeleniZamestnanceField extends FormField
{
    /** @var string     Uzivatel vybere jedno rozdeleni */
    const MODE_SELECT = 'select';
    /** @var string     Checkboxy - uzivatel muze vybrat vice rozdeleni */
    const MODE_CHECK = 'checkboxes';
    /** @var string     Pills - uzivatel muze vybrat vice rozdeleni. Zobrazuji se jako butonky. */
    const MODE_PILLS = 'pills';

    /** @var int    Kod typu rozdeleni */
    public $kodTypuRozdeleni;

    /** @var int    Urcujte, jestli uzivatel bude vybirat jen z vlastnich rozdeleni */
    public $pouzeVlastniRozdeleni = 0;

    public $mode = self::MODE_SELECT;

    /** @var int[]  Seznam Id PP uzivatele */
    private static $ppUzivatele = null;

    /** @var Rozdeleni[]    Seznam existujicich rozdeleni - napr. pro zobrazeni checkboxu */
    private $seznamRozdeleni;

    /** @var RozdeleniOsob  Seznam rozdeleni prirazenych osobe - pouzije se pri editaci existujiciho PP */
    protected $seznamRozdeleniOsoby;


    /**
     * Nacteni konfigurace
     * @param $key
     * @param $value
     */
    protected function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'kodTypuRozdeleni' :
                $this ->kodTypuRozdeleni = (int)$value;
                if (!$this ->fieldName) {
                    $this ->fieldName = 'RozdeleniOsoby' . $this ->kodTypuRozdeleni;
                }
                break;

            case 'mode' :
                $this ->mode = $value;
                break;

            case 'pouzeVlastniRozdeleni' :
                $this ->pouzeVlastniRozdeleni = (int)$value;
                break;

            default :
                parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * @param RozdeleniOsob[] $seznamRozdeleniOsoby
     */
    public function setSeznamRozdeleniOsoby($seznamRozdeleniOsoby)
    {
        $this->seznamRozdeleniOsoby = $seznamRozdeleniOsoby;

        if ($this ->mode == self::MODE_SELECT) {
            $this ->value = $seznamRozdeleniOsoby ? $seznamRozdeleniOsoby[0] ->kod : null;
        } else {
            $this ->value = [];

            foreach ($seznamRozdeleniOsoby as $rozdeleniOsoby) {
                $this ->value[$rozdeleniOsoby ->kod] = 1;
            }
        }
    }


    /**
     * Nacte rozdeleni prirazena zadane osobe
     * @param Database $database
     * @param int $idPP
     */
    public function nactiRozdeleniOsoby($database, $idPP)
    {
        $seznamRozdeleni = RozdeleniOsob::nactiRozdeleniOsoby($database, $idPP, $this ->kodTypuRozdeleni, null);

        if ($this->mode == self::MODE_SELECT) {
            // najde jen prvni prirazene rozdeleni
            /** @var RozdeleniOsob $prvniRozdeleni */
            $prvniRozdeleni = null;
            foreach ($seznamRozdeleni as $rozdeleni) {
                if (!$prvniRozdeleni  ||  $rozdeleni ->platnostOd < $prvniRozdeleni ->platnostOd) {
                    $prvniRozdeleni = $rozdeleni;
                }
            }

            $this->setSeznamRozdeleniOsoby($prvniRozdeleni ? [$prvniRozdeleni] : []);
        } else {
            $this->setSeznamRozdeleniOsoby($seznamRozdeleni);
        }
    }


    private static function getPPUzivatele($database)
    {
        if (!self::$ppUzivatele) {
            self::$ppUzivatele = [];

            $seznamPP = PracovniPomer::nactiPPOsoby($database, User::getRC());
            foreach ($seznamPP as $pp) {
                self::$ppUzivatele[] = $pp ->idPP;
            }
        }

        return self::$ppUzivatele;
    }


    /**
     * Zobrazi vyber rozdeleni (select)
     * @param Database $database
     * @param bool $canEdit
     */
    private function showSelect($database, $canEdit)
    {
        $comboBoxes = new ComboBoxesRozdeleni($database, $this ->fieldName, $this ->fieldName);

        PracovniPomer::nactiPPOsoby($database, User::getRC(), time());

        $comboBoxes ->loadRozdeleni($this ->pouzeVlastniRozdeleni,
            $this ->pouzeVlastniRozdeleni ? User::getRC() : null, // pouze vlastni - podle PP aktualniho uzivatele
            $this ->kodTypuRozdeleni, $this ->value,
            $this ->required);

        if (!$canEdit) {
            $comboBoxes ->addHtmlParams(['disabled' => 'disabled']);
        }

        $comboBoxes ->combo1 ->render();
    }

    /**
     * @param Database $database
     * @throws DBException
     */
    private function nactiSeznamRozdeleni($database)
    {
        if (!$this ->seznamRozdeleni) {
            $this ->seznamRozdeleni = $this ->pouzeVlastniRozdeleni
                ? Rozdeleni::loadRozdeleniOsoby($database, self::getPPUzivatele($database), $this ->kodTypuRozdeleni, time())
                : Rozdeleni::loadSeznamRozdeleni($database, $this ->kodTypuRozdeleni);
        }
    }


    /**
     * Zobrazi hromadny vyber rozdeleni - checkbox pro kazde rozdeleni zadaneho typu
     * @param Database $database
     */
    private function showCheckBoxes($database, $canEdit)
    {
        $this->nactiSeznamRozdeleni($database);

        foreach ($this ->seznamRozdeleni as $rozdeleni) {
            echo
                '<div>' .
                '<input type="checkbox" value="1" ' .
                    ($canEdit ? '' : ' disabled="disabled"') .
                    ((isset($this ->value[$rozdeleni ->kod]) &&  $this ->value[$rozdeleni ->kod]) ? ' checked="checked"' : '') .
                    ' name="' . $this ->fieldName . '_' . StringUtils::htmlTextConvert($rozdeleni ->kod) . '" /> ' .
                StringUtils::htmlTextConvert($rozdeleni ->popis) .
                "</div>\n";
        }
    }


    /**
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit
     * @param bool $insert
     * @param array $rowData
     */
    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        switch ($this ->mode) {
            case self::MODE_CHECK :
                $this ->showCheckBoxes($database, $canEdit);
                break;

            case self::MODE_PILLS :
                echo 'MODE pills: neni implementovano!';
                break;

            case self::MODE_SELECT :
            default :
                $this ->showSelect($database, $canEdit);
        }
    }


    /**
     * Nacte hodnoty checkboxu
     * @param $database
     * @param $usePostValue
     * @param $rowData
     */
    protected function loadCheckBoxes($database, $usePostValue, $rowData)
    {
        if ($usePostValue) {
            $this ->nactiSeznamRozdeleni($database);
            $this ->value = [];
            foreach ($this ->seznamRozdeleni as $rozdeleni) {
                $checkName = $this ->fieldName . '_' . StringUtils::htmlTextConvert($rozdeleni ->kod);
                if (isset($_POST[$checkName])  &&  $_POST[$checkName]) {
                    $this ->value[$rozdeleni ->kod] = 1;
                }
            }
        }
    }


    public function loadInsertValue($database, $usePostValues, $rowData = null)
    {
        if ($this ->mode === self::MODE_CHECK) {
            // nacist seznam zaskrtnutych checkboxu
            $this ->loadCheckBoxes($database, $usePostValues, $rowData);
        } else {
            parent::loadInsertValue($database, $usePostValues, $rowData);
        }
    }


    public function loadEditValue($database, $usePostValues, $rowData, $canEditRecord)
    {
        if ($this ->mode === self::MODE_CHECK) {
            // nacist seznam zaskrtnutych checkboxu
            $this->loadCheckBoxes($database, $usePostValues, $rowData);
        } else {
            parent::loadEditValue($database, $usePostValues, $rowData, $canEditRecord);
        }
    }


    const ACTION_UPDATE = 0;    // pokud zaznam existuje, tak provede update - umoznuje pouze jeden prirazeny zaznam rozdeleni daneho typu
    const ACTION_ADD = 1;       // pokud zaznam neexistuje, tak jej prida - umoznuje vice prirazenych rozdeleni stejneho typu
    const ACTION_DELETE = 2;    // pokud zaznam existuje, tak jej smaze

    /**
     * Vlozi nebo prepise jeden zaznam rozdeleni osoby.
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param string $kodRozdeleni
     * @param object $logDescription
     * @param int $action
     * @throws DBException
     * @throws Exception
     */
    protected function ulozZaznamRozdeleni($database, $rc, $osobniCislo, $datumNastupu, $kodRozdeleni, $logDescription, $action)
    {
        $rec = new RozdeleniOsobRecord($database, $logDescription);
        // pp
        $rec->fRC->setNewValue($rc);
        $rec->fOsobniCislo->setNewValue($osobniCislo);
        $rec->fDatumNastupu->setNewValue($datumNastupu);
        // rozdeleni
        $rec->fKodTypuRozdeleni->setNewValue($this->kodTypuRozdeleni);
        $rec->fRozdeleni->setNewValue($kodRozdeleni);
        // platnost od
        $rec->fPlatnostOd->setNewValue($datumNastupu);

        $params = $database ->createParameters();
        $rec ->setWhere(
            'KodTypuRozdeleni = ' . $params ->addParameter($this ->kodTypuRozdeleni) .
            ( ($action && isset($kodRozdeleni) && (trim($kodRozdeleni) !== ''))
                ? ' AND Rozdeleni = ' . $params ->addParameter($kodRozdeleni)
                : ''
            ) .
            ' AND RC = ' . $params ->addParameter($rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($datumNastupu),
            $params
        );

        if ($rec ->loadRecord()) {
            // zaznam uz existuje
            if ($action == self::ACTION_DELETE) {
                $rec ->doDelete(null);
            } else {
                $rec ->updateRecord();
            }
        } else if ($action != self::ACTION_DELETE  &&  isset($kodRozdeleni) && trim($kodRozdeleni) !== '') {
            $rec ->insertRecord();
        }
    }


    /**
     * Vlozi zaznamy RozdeleniOsob pro zaskrtnute chekboxy
     * @param $database
     * @param $rc
     * @param $osobniCislo
     * @param $datumNastupu
     * @param $logDescription
     */
    public function ulozCheckboxy($database, $rc, $osobniCislo, $datumNastupu, $logDescription)
    {
        $this ->nactiSeznamRozdeleni($database);

        foreach ($this ->seznamRozdeleni as $rozdeleni) {
            $fieldName = $this->fieldName . '_' . $rozdeleni->kod;
            $this->ulozZaznamRozdeleni($database,
                $rc, $osobniCislo, $datumNastupu,
                $rozdeleni->kod, $logDescription,
                (isset($_POST[$fieldName])  &&  $_POST[$fieldName]) ? self::ACTION_ADD : self::ACTION_DELETE);
        }
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param object $logDescription
     */
    public function ulozRozdeleni($database, $rc, $osobniCislo, $datumNastupu, $logDescription)
    {
        // ulozi zadane hodnoty do databaze
        switch ($this ->mode) {
            case self::MODE_CHECK :
                $this ->ulozCheckboxy($database, $rc, $osobniCislo, $datumNastupu, $logDescription);
                break;

            case self::MODE_PILLS :
                MessageManager::addWarning('Ulozeni vice rozdeleni (mode pills) neni jeste implementovano!');
                break;

            case self::MODE_SELECT :
            default :
                if (isset($_POST[$this ->fieldName])) {
                    $this ->ulozZaznamRozdeleni($database,
                        $rc, $osobniCislo, $datumNastupu,
                        $_POST[$this ->fieldName], $logDescription,
                        (trim($_POST[$this ->fieldName]) !== '') ? self::ACTION_UPDATE : self::ACTION_DELETE
                    );
                }
                break;
        }
    }
}
