<?php

/**
 * Pavel Rudol, RON Software
 *  4.9.2015
 */

namespace RonPortal\Zamestnanci;

use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use RonPortal\Prava;
use RonPortal\RozdeleniUtils;
use RonPortal\User;


class SeznamOsobBrowser extends SeznamZamestnancuBrowser
{
    public function __construct($database, $action)
    {
        parent::__construct($database, $action, false); // pracovni pomery = false

        $this->code = 'browserosoby';
        $this->tableId = 'browserosoby';
        
        $this->browserUrl = '?menuid=zamestnanec&action=seznamosob&';
    }


    public function loadColumnConfiguration($colSettings)
    {
        // kontrola, jestli neni zobrazeni sloupce v seznamu osob vypnuto
        if (isset($colSettings['osoby'])  &&  !$colSettings['osoby']) {
            return;
        }

        parent::loadColumnConfiguration($colSettings);
    }


    function getSQLFilter()
    {
        if (!$this->pravaZobrazit->jsouNejakePrava()) {
            return '1 = 0';
        }

        $params = $this ->database ->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        if ($this ->filterForm instanceof SeznamZamestnancuFilterForm) {
            switch ($this ->filterForm ->getFilterValue('stavpp')) {
                case 'aktivni' :
                    $result .=
                        ($result ? ' AND ' : '') .
                        $this->getSQLFilterPracovniPomer($params);
                    break;

                case 'neaktivni' :
                    $result .=
                        ($result ? ' AND ' : '') .
                        ' NOT ' . $this->getSQLFilterPracovniPomer($params) .
                        ' AND ' . $this->getSQLFilterPracovniPomer($params, false); // PP bez omezeni platnosti
                    break;

                default :
                    $result .=
                        ($result ? ' AND ' : '') .
                        '(' .
                        ' (NOT EXISTS(SELECT P.RC FROM PracovniPomer P WHERE (P.RC = Osoba.RC))' .
                        '  AND ' . $this->getSQLFilterPravaNaOsobu($params) . ')' .
                        ' OR ' . $this->getSQLFilterPracovniPomer($params, false) . //PP bez omezeni platnosti
                        ')';
                    break;
            }
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }

    /**
     *
     * @param integer $typRozdeleni Typ rozdeleni pro filtrovani
     * @param string $rozdeleni kod rozdeleni pro filtrovani
     * @param int $vcetnePodrizenych 0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @param Parameters $params
     * @return string SQL podminka existence prirazeneho rozdeleni
     */
    function getSQLFilterRozdeleni($typRozdeleni, $rozdeleni, $vcetnePodrizenych, $params)
    {
        return RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
            'P', $typRozdeleni, $rozdeleni,
            $this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(),
            $vcetnePodrizenych,
            $params);
    }
    
    /**
     * @param Parameters $params
     * @param boolean $omezitPlatnost pokud je TRUE omezi platnost filtrovaneho pracovniho pomeru
     * @return string SQL podminka na existenci pracovniho pomeru
     */
    function getSQLFilterPracovniPomer($params, $omezitPlatnost = TRUE)
    {
        $typZamestnance = $this ->filterForm ->getFilterValue('typzamestnance');

        $datumOd = $this ->filterForm ->getFilterValue('datumod') ?: time();
        $datumDo = $this ->filterForm ->getFilterValue('datumod', 'datumDo') ?: time();

        $result =
            'EXISTS(SELECT P.RC FROM PracovniPomer P' .
            ' WHERE (P.RC = Osoba.RC)' .
            '   AND (' . $this->pravaZobrazit->sqlFiltr('P', time(), time(), $params) . ')' .
            ( $omezitPlatnost ?
                    " AND (P.DatumNastupu <= " . $params ->addDateParameter($datumDo) . ")"
                    . " AND ((P.DatumUkonceni >= " . $params->addDateParameter($datumOd) . ") OR (P.DatumUkonceni IS NULL))"
                    : ""
            ) .
            (isset($typZamestnance) ? ' AND P.Zamestnanec = ' . $params ->addParameter((int)$typZamestnance) : '') ;

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $this->getSQLFilterRozdeleni(
                        $rozdeleniFilter ->getTypRozdeleni(),
                        $rozdeleniFilter ->getValue(),
                        $rozdeleniFilter ->getValue("vcetnePodrizenych"),
                        $params
                    );
            }
        }
//            (($this ->filterForm ->kodTypuRozdeleni && isset($this ->filterForm ->kodRozdeleni))
//                ? ' AND ' . $this->getSQLFilterRozdeleni($this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni, $this ->filterForm ->vcetnePodrizenychRozdeleni, $params)
//                : '') .
//            (($this ->filterForm ->kodTypuRozdeleni2 && isset($this ->filterForm->kodRozdeleni2))
//                ? ' AND ' . $this->getSQLFilterRozdeleni($this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2, $this ->filterForm ->vcetnePodrizenychRozdeleni2, $params)
//                : '') .

        $result .=
            ")\n";

        return $result;
    }


    /**
     * @param Parameters $params
     * @param string $jmenoTabulky
     * @return string
     */
    function getSQLFilterPravaNaOsobu($params)
    {
        return $this ->pravaZobrazit ->sqlFiltr('Osoba',
            $this ->filterForm ->getFilterValue('datumod') ?? time(),
            $this ->filterForm ->getFilterValue('datumod', 'datumDo') ?? time(),
            $params,
            false,
            true
        );

//        //$result = '(' . $this->pravaZobrazit->sqlFiltr('PracovniPomer', time(), time(), $params) . ')';
//        switch ($this ->pravaZobrazit->prava) {
//            case 3 : // vse
//                return ' (0 = 0) ';
//            case 0 : // nic
//            case 100 :
//                return ' (0 = 1) ';
//
//            case 1 :
//                return "($jmenoTabulky.RC = " . $params->addParameter(User::getRC()) . ")";
//        }
//
//        if ($this ->pravaZobrazit->prava == 2 || $this ->pravaZobrazit->prava == 4) {
//            // vybrane osoby a skupiny ( 4 = krome sebe )
//
//            $filtrosoby = '';
//
//            $pomarray = explode(';', $this ->pravaZobrazit->osoby);
//            foreach ($pomarray as $osoba) {
//                if ($osoba != '') {
//                    $filtrosoby .=
//                        ($filtrosoby ? ' OR ' : '') .
//                        "$jmenoTabulky.RC = " . $params ->addParameter($osoba);
//                }
//            }
//
//            $filtrskupiny = '';
//            $pomarray = explode(';', $this ->pravaZobrazit->skupiny);
//            foreach ($pomarray as $skupina) {
//                if ($skupina != '') {
//                    $filtrskupiny .=
//                        ($filtrskupiny ? ' OR ' : '') .
//                        "PrirazeniOsobDoPristSkup.Kod = " . $params ->addParameter($skupina);
//                }
//            }
//
//            if ($filtrskupiny) {
//                $filtrskupiny =
//                    ' EXISTS (' .
//                    'SELECT PrirazeniOsobDoPristSkup.Kod FROM PrirazeniOsobDoPristSkup' .
//                    ' WHERE (' . $filtrskupiny . ') ' .
//                    "   AND PrirazeniOsobDoPristSkup.RC = $jmenoTabulky.RC " .
//                    /*"   AND PrirazeniOsobDoPristSkup.OsobniCislo = $jmenoTabulky.OsobniCislo " .
//                    "   AND PrirazeniOsobDoPristSkup.DatumNastupu = $jmenoTabulky.DatumNastupu " .*/
//                    '   AND PrirazeniOsobDoPristSkup.PlatnostOd <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo') ?? time()) .
//                    '   AND (PrirazeniOsobDoPristSkup.PlatnostDo IS NULL  OR  PrirazeniOsobDoPristSkup.PlatnostDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod') ?? time()) . ')' .
//                    ')';
//            }
//
//            if (!$filtrosoby && !$filtrskupiny) {
//                return '(0 = 1)';
//            }
//
//            if ($filtrosoby != '') {
//                if ($filtrskupiny) {
//                    $filtr = " (($filtrosoby)  OR  ($filtrskupiny)) ";
//                } else {
//                    $filtr = $filtrosoby;
//                }
//
//            } else {
//                $filtr = $filtrskupiny;
//            }
//
//            if ($this ->pravaZobrazit->prava == 4) {
//                $filtr = " (($filtr) AND $jmenoTabulky.RC <> " . $params ->addParameter(User::getRC()) . ")";
//            }
//
//            return $filtr;
//        }
//
//        return ' (0 = 1) ';
    }
}
