<?php
/**
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 16.8.2016
 * Time: 12:28
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FotoOsobyBrowserField;
use RonPortal\Controls\PritomnostBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;


class PrijmeniDetailBrowserField extends BrowserField
{
    private $rcIndex;
    private $idPPIndex;

    /**
     * @param null|Database $database
     * @param int $rcIndex
     * @param int $idPPIndex
     */
    public function __construct($database, $rcIndex, $idPPIndex) {
        parent::__construct($database);

        $this->rcIndex = $rcIndex;
        $this->idPPIndex = $idPPIndex;

        //TODO: odstranit podminku po dokonceni formulare
        global $enable_seznamPP;
        if ($enable_seznamPP)
            $this->link = '?menuid=zamestnanec';
    }

    function getLink($text, $recordId, $rowData) {
        $result = '';

        if ($this ->link) {
            $result .= '<a href="' . $this ->link;
            if ($rowData[$this->idPPIndex] != '') {
                $result .= '&idpp=' . $rowData[$this->idPPIndex];
            }
            elseif ($rowData[$this->rcIndex] != '') {
                $result .= '&rc=' . $rowData[$this->rcIndex];
            }
            $result .= '">';
            //$result .= '" onclick="StopEventPropagation(event)">';
        }

        $result .= $text;

        if ($this ->link) $result .= '</a>';

        return $result;
    }

//    function getViewText($value, $rowNumber, $recordId, $rowData) {
//        switch ($this ->type) {
//            case BROWSER_FIELD_TYPE_LINE :
//                if ($rowNumber > 0) return $rowNumber;
//                else return '';
//
//            case BROWSER_FIELD_TYPE_DEFAULT :
//                return $this ->getLink(DBUtils::getFormatedValue($this ->database, $this ->format, $value), $rowData);
//
//            case BROWSER_FIELD_TYPE_TEXT :
//                return $this ->getLink($this ->text, $recordId);
//
//            case BROWSER_FIELD_TYPE_SELECT :
//                return $this ->getLink($this ->options[$value], $recordId);
//        }
//
//        return '';
//    }
}




/**
 * Class SeznamZamestnancuBrowser obecny predek pro prehled pracovnich pomeru/prehled osob, sloupce se nacitaji z nastaveni.
 * @package RonPortal\Zamestnanci
 */
class SeznamZamestnancuBrowser extends RonPortalBrowser
{
    /** Index pole s RC v selectu */
    const INDEX_RC = 'col-rc';
    /** Index pole s Id PP v selectu */
    const INDEX_ID_PP = 'col-idpp';

    /** @var PravaNaOsoby */
    protected $pravaZobrazit;

    /** @var boolean urcuje, zda bude dotaz i z tabulky PracovniPomer, nebo jen Osoba */
    protected $pouzitPracovniPomer = true;

    /** @var string nazev pole z nastaveni portalu s definici sloupcu (je mozne mit ruzne nastaveni pro pracovni pomery a osoby) */
    protected $nazevNastaveniSloupcu = 'pracovnipomery_sloupce';


    /**
     * SeznamZamestnancuBrowser constructor.
     * @param Database $database
     * @param string $action
     * @param bool $pouzitPracovniPomer
     * @throws \Exception
     */
    public function __construct($database, $action, $pouzitPracovniPomer)
    {
        parent::__construct($database, $action);

        $this ->pouzitPracovniPomer = $pouzitPracovniPomer;

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravaZobrazit = Prava::vratPrava('zamest_prohl');

        $this->sqlSelectPart =
            'Osoba.RC, ' . // 0
            ($this ->pouzitPracovniPomer    // ID PP - pokud se pouziva tabulka PracovniPomer, bere se primo zde, jinak posledni ID Pracovniho pomeru osoby, pokud existuje
                ? ' PracovniPomer.ID'
                : ' (SELECT MAX(PracovniPomer.ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC)');  // 1

        $this->sqlFromPart =
            'Osoba LEFT OUTER JOIN OsobniUdaj ON (Osoba.RC = OsobniUdaj.RC)' .
            ($this->pouzitPracovniPomer
                ? ' JOIN PracovniPomer ON (Osoba.RC = PracovniPomer.RC)'
                : '');

        $this->filterFormAppCode = 'ADS';
        $this->filterFormRights = 'zamest_prohl';

        //TODO: odstranit podminku po dokonceni formulare
        global $enable_seznamPP;
        if ($enable_seznamPP)
            $this->rowClickLink = '?menuid=zamestnanec';

        $this->cssClass .= ' pritomnost-container';

//        $this->tableHtmlParams = [
//            'data-ron-pritomnost-url' => '?menuid=zamestnanec&action=pritomnost',
//            'data-ron-hledatpp' => $this->pouzitPracovniPomer ? 'true' : 'false'
//        ];

        //$this->sestavHlavicku();

        $str = RonPortalSettings::getBvarSettings($this ->database,
            'RonPortal_' . $this->nazevNastaveniSloupcu,
            RonPortalSettings::getValue($this->nazevNastaveniSloupcu)); // pro zpetnou kompatibilitu pouziva i getValue()
        if (!$str) {
            throw new \Exception(TextManager::_('Zamestnanci_neni_nastaveni_sloupcu_tabulky'));
        }

        $this->filterForm = new SeznamZamestnancuFilterForm($this->database, $this->action);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldIndex(0)
            ->setCode('col-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldIndex(1)
            ->setCode('col-idpp');

        $this ->loadConfigurationXml($str);
    }


    /**
     * Nastavi filtry.
     */
    function setFilterForm()
    {
        //$this->filterForm = new SeznamZamestnancuFilterForm($this->database, $this->action);
    }


    /**
     * @param $colSettings
     * @return int|null
     */
    function createNewField($colSettings)
    {
        $newField = null;

        switch ($colSettings['typsloupce']) {
            case 'prijmeni' :
                $newField = $this ->addField(new PrijmeniDetailBrowserField($this ->database, self::INDEX_RC, self::INDEX_ID_PP))
                    ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni')
                    ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno')
                    ->setCode('col-prijmeni');
                $this->defaultSort = 'col-prijmeni';
                break;

            case 'jmeno' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno')
                    ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni')
                    ->setCode('col-jmeno');
                break;

            case 'titul' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Titul') ->setHeaderText('Titul')
                    ->setSortSQL('Osoba.Titul, Osoba.Prijmeni, Osoba.Jmeno')
                    ->setCode('col-titul');
                break;

            case 'titulza' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.TitulZa') ->setHeaderText('Titul_za')
                    ->setSortSQL('Osoba.TitulZa, Osoba.Prijmeni, Osoba.Jmeno')
                    ->setCode('col-titulza');
                break;

            case 'rodneprijmeni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.RodneJmeno') ->setHeaderText('Rodne_jmeno') ->setOrdering(true)
                    ->setCode('col-ou-rodneprijmeni');
                break;

            case 'datumnarozeni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.DatumNarozeni') ->setHeaderText('Datum_narozeni')
                    ->setOrdering(true) ->setFormat('date')
                    ->setCode('col-ou-datumnarozeni');
                break;

            case 'email' :
                $newField = $this->addField(BrowserField::FIELD_TYPE_EMAIL)
                    ->setSqlFieldName('Osoba.eMail') ->setHeaderText('e_mail') ->setOrdering(true)
                    ->setCode('col-email');
                break;

            case 'telefon' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Telefon') ->setHeaderText('Telefon') ->setOrdering(true)
                    ->setCode('col-telefon');
                break;

            case 'mobil' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.Mobil') ->setHeaderText('Mobil') ->setOrdering(true)
                    ->setCode('col-mobil');
                break;

            case 'rczahranicni' :
                $newField = $this->addField()
                    ->setSqlFieldName('OsobniUdaj.RC_Zahranicni') ->setHeaderText('RC_Zahranicni') ->setOrdering(true)
                    ->setCode('col-ou-rc_zahranicni');
                break;

            case 'osobauzivatelska' :
                $index = (int)$colSettings['index'];
                $newField = $this->addField()
                    ->setSqlFieldName("Osoba.uzivatelska$index") ->setHeaderText("osoba_uzivatelska_{$index}") ->setOrdering(true)
                    ->setCode("col-uzivatelska$index");
                break;

            case 'ppomeruzivatelska' :
                if ($this ->pouzitPracovniPomer) {
                    $index = (int)$colSettings['index'];
                    $newField = $this->addField()
                        ->setSqlFieldName("PracovniPomer.uzivatelska$index")->setHeaderText("ppomer_uzivatelska_{$index}")
                        ->setOrdering(true)
                        ->setCode("col-pp-uzivatelska$index");
                }
                break;

            case 'rc' :
            case 'idosoby' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.RC', 0) ->setHeaderText('Id_osoby') ->setOrdering(true)
                    ->setCode('col-rc');
                break;

            case 'foto' :
                $newField = $this->addField(new FotoOsobyBrowserField(self::INDEX_ID_PP, '?menuid=zamestnanec&action=foto&osoba='));
                $newField
                    ->setHeaderText('Zamestnanci_foto')
                    ->setSqlFieldName('Osoba.idFotografie');
                break;

            case 'osobnicislo' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo')
                        ->setOrdering(true)
                        ->setCode('col-pp-osobnicislo');
                }
                break;

            case 'datumnastupu' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.DatumNastupu') ->setHeaderText('Datum_nastupu')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center')
                        ->setCode('col-pp-datumnastupu');
                }
                break;

            case 'datumukonceni' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.DatumUkonceni') ->setHeaderText('Datum_ukonceni')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center')
                        ->setCode('col-pp-datumukonceni');
                }
                break;

            case 'zkusebnidobadovypoctena' :
                if ($this->pouzitPracovniPomer) {
                    $newField = $this->addField()
                        ->setSqlFieldName('PracovniPomer.ZkusebniDobaDoVypoctena') ->setHeaderText('Zkusebni_doba_do_vypoctena')
                        ->setOrdering(true) ->setFormat('date') ->setAlign('center')
                        ->setCode('col-pp-zkusebnidobadovypoctena');
                }
                break;

            case 'pritomnost' :
                $newField = $this->addField(new PritomnostBrowserField(self::INDEX_ID_PP));
                $newField ->setCode('col-pritomnost');
                break;

            case 'kodrozdeleni' :
                if ($this->pouzitPracovniPomer) {
                    $typRozdeleni = (int)$colSettings['index'];

                    $colSettings['sqlFieldName'] =
                        '(SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob' .
                        ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $typRozdeleni .
                        ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
                        ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                        ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                        ' AND RozdeleniOsob.Hlavni = 1' .
                        ' AND RozdeleniOsob.PlatnostOd <= {fn_GETDATE()}' . //$this->database->timestampToString(RonPortalSession::getDatumDo() ?: time()) .
                        ' AND (RozdeleniOsob.PlatnostDo >= {fn_GETDATE()} OR RozdeleniOsob.PlatnostDo IS NULL)' .  // $this->database->timestampToString(RonPortalSession::getDatumOd() ?: time()) .
                        ') AS KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM');   //20.12.2016 RON-PR workaround pro typ rozdeleni -1 (PM)

                    $colSettings['sortSQL'] = 'KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ', Osoba.Prijmeni, Osoba.Jmeno';
                    //$colSettings['sortSQLDesc'] = 'KodRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ' DESC, Osoba.Prijmeni DESC, Osoba.Jmeno DESC';

                    if (!isset($colSettings['header'])) {
                        $colSettings['header'] = StringUtils::htmlTextConvert(DBUtils::getSqlValue($this->database, 'SELECT Popis FROM TypRozdeleni WHERE Kod = ' . (int)$typRozdeleni));
                    }

                    if (!isset($colSettings['ordering'])) {
                        $colSettings['ordering'] = 1;
                    }

                    if (!isset($colSettings['code'])) {
                        $colSettings['code'] = "col-rozdeleni{$typRozdeleni}-kod";
                    }

                    return parent::createNewField($colSettings);
                }
                break;

            case 'popisrozdeleni' :
                if ($this->pouzitPracovniPomer) {
                    $typRozdeleni = (int)$colSettings['index'];

                    $colSettings['sqlFieldName'] =
                        '(SELECT Popis FROM Rozdeleni WHERE KodTypuRozdeleni = ' . $typRozdeleni .
                        ' AND Kod = ' .
                        ' (SELECT MIN(RozdeleniOsob.Rozdeleni) FROM RozdeleniOsob ' .
                        ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $typRozdeleni .
                        '  AND RozdeleniOsob.RC = PracovniPomer.RC' .
                        '  AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
                        '  AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
                        '  AND RozdeleniOsob.Hlavni = 1' .
                        '  AND RozdeleniOsob.PlatnostOd <= {fn_GETDATE()}' . // $this->database->timestampToString(RonPortalSession::getDatumDo() ?: time()) .
                        '  AND (RozdeleniOsob.PlatnostDo >= {fn_GETDATE()} OR RozdeleniOsob.PlatnostDo IS NULL)' . // ' . $this->database->timestampToString(RonPortalSession::getDatumOd() ?: time()) . '
                        ')) AS PopisRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM'); //20.12.2016 RON-PR workaround pro typ rozdeleni -1 (PM)

                    $colSettings['sortSQL'] = 'PopisRozdeleni' . ($typRozdeleni > 0 ? $typRozdeleni : '_PM') . ', Osoba.Prijmeni, Osoba.Jmeno';

                    if (!isset($colSettings['header'])) {
                        $colSettings['header'] = StringUtils::htmlTextConvert(DBUtils::getSqlValue($this->database, 'SELECT Popis FROM TypRozdeleni WHERE Kod = ' . (int)$typRozdeleni));
                    }

                    if (!isset($colSettings['ordering'])) {
                        $colSettings['ordering'] = 1;
                    }

                    if (!isset($colSettings['code'])) {
                        $colSettings['code'] = "col-rozdeleni{$typRozdeleni}-popis";
                    }

                    return parent::createNewField($colSettings);
                }
                break;

            default :
                return parent::createNewField($colSettings);
        }

        if ($newField) {
            $newField ->loadConfigurationArray($colSettings);
//            if ($newField->ordering  &&  $newField->visible && !$this->defaultSort) {
//                $this->defaultSort = count($this->fields);
//            }
        }

        return $this->fields ? count($this->fields) - 1 : -1;
    }


    function loadConfigurationXml($xmlText)
    {
        if ($xmlText  &&  $xmlText[0] == '<') {
            parent::loadConfigurationXml($xmlText);
        } else {
            $this ->sestavHlavicku($xmlText);
        }
    }


    function loadConfiguration()
    {
        // Nenacita konfiguraci - je to zbytecne, protoze se to konfiguruje primo v nastaveni
        // return parent::loadConfiguration();
    }


    /**
     * Prida k zobrazeni sloupce dle nastaveni. Pouze pro kompatibilitu se starym nastavenim ve formatu Nadpis:TypPolozky;...
     * @param string $str
     */
    public function sestavHlavicku($str)
    {
        $header = [];
        $sloupce = [];
        StringUtils::parseStringToArrays($str, $header, $sloupce, true);

        $this ->addField(BrowserField::FIELD_TYPE_LINE);

        foreach ($sloupce as $i => $sloupec) {
            $sloupecKod = preg_replace('/[0-9\-]/', '', $sloupec);
            $sloupecCislo = preg_replace('/[a-zA-Z]/', '', $sloupec);

            $colSettings = ['typsloupce' => $sloupecKod];
            if ($sloupecCislo) {
                $colSettings['index'] = $sloupecCislo;
            }
            if ($header[$i]) {
                $colSettings['header'] = TextManager::_($header[$i]);
            }

            $this ->createNewField($colSettings);
        }
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        if (!$this->tableHtmlParams) {
            $this->tableHtmlParams = [];
        }

        $this->tableHtmlParams['data-ron-pritomnost-url'] =
            '?menuid=zamestnanec&action=pritomnost' .
            '&rozdeleni1=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni') ?? '') .
            '&vcetnepodrizenych1=' . (int)$this ->filterForm ->getFilterValue('rozdeleni', 'vcetnePodrizenych') .
            '&rozdeleni2=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni2') ?? '') .
            '&vcetnepodrizenych2=' . (int)$this ->filterForm ->getFilterValue('rozdeleni2', 'vcetnePodrizenych') .
            '&jenrc=' . ($this->pouzitPracovniPomer ? 0 : 1);

        //'data-ron-hledatpp' => $this->pouzitPracovniPomer ? 'true' : 'false'
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @return int|string
     */
    function getClickLink($values)
    {
        if ($this ->rowClickLink) {
            if ($this->pouzitPracovniPomer || ($values[self::INDEX_ID_PP] != '')) {
                return $this ->rowClickLink . '&idpp=' . $values[self::INDEX_ID_PP];
            }

            return $this ->rowClickLink .  '&rc=' . $values[self::INDEX_RC];
        }

        return 0;
    }

    /**
     * Doplni k radku id pracovniho pomeru pro javascript.
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @return string
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        return 'data-ron-idpp="' . $values[self::INDEX_ID_PP] . '" ' . parent::getRowAttributes($values, $className, $rowNumber);
    }

}
