<?php
/**
 * stanislav.elbl, 20.05.2022 16:11
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\ComboBox;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;

/**
 * Filtr pro browsery PP, osob
 * Filtruje rozdeleni, platnost PP, typ Zamestnance, aktivni/neaktivni/vsechny osoby/PP
 */
class SeznamZamestnancuFilterForm extends RonPortalFilterForm
{
    public $actionZamestnanci;


    public function __construct($database, $action)
    {
        parent::__construct($database, 'zamestnanec', true);

        $this->actionZamestnanci = $action;

        TextManager::getInstance()->loadStrings('zamestnanci.ini');

        $prava = Prava::vratPrava('zamest_prohl');
        $this->addFilterRozdeleni1(null, $prava);
        $this->addFilterRozdeleni2(null, $prava);

        $this->addFilter(
            (new FilterFormFilter('action', FilterFormFilter::TYPE_HIDDEN))
                ->setDefaultValue($this->actionZamestnanci)
        );


        // vsichni/aktivni/neaktivni
        $this->addFilter(
            (new FilterFormFilter('stavpp'))
                ->addItem('vsichni', TextManager::_('Vsichni'))
                ->addItem('aktivni', TextManager::_('Aktivni'))
                ->addItem('neaktivni', TextManager::_('Neaktivni'))
                ->setSessionName('zamestnanci_stav_pp')
                ->setUserSettingsName('zamestnanci_stav_pp')
                ->setDefaultValue('aktivni'),
            true
        );

        $this->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setRequired(true),
            true
        );

        // typ osoby (zamestnanec/externi/...)
        $this->addFilter(
            (new FilterFormFilter('typzamestnance', FilterFormFilter::TYPE_OPTIONS))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql('SELECT Kod, Popis FROM S_TypOsoby WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
                ->setCaption(TextManager::_('Jen_typ_zamestnance'))
                ->setSessionName('zamestnanci_typ_zamestnance')
        );
    }

    protected function afterLoadFilter($filter)
    {
        parent::afterLoadFilter($filter);

        // pokud je zapnut filtr pouze na aktivni nebo neaktivni, tak zapne filtr datum od - do
        if ($filter->name == 'stavpp') {
            $filterDatum = $this->getFilter('datumod');
            if ($filterDatum) {
                $filterDatum->setEnabled($filter->getValue() != 'vsichni');
            }
        }
    }


//    protected function loadTopFilters()
//    {
//        parent::loadTopFilters();
//        $this->nastavDatumOdDo($this ->getFilterValue('stavpp') !== 'vsichni', true); // jen pro aktivni/neaktivni, povinne=true
//    }


//    function loadData()
//    {
//        $this->loadFiltrStav();
//        $this->loadTypZamestnance();
//
//        parent::loadData();
//    }
}
