<?php
/**
 * UTF8
 * stanislav.elbl, 10.07.2015
 */

namespace RonPortal;

use Ron\DB\DBException;
use Ron\Http\App;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpResponse;
use Ron\MVC\Model;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;

global $appDate;
global $appVersion;
global $copyrightYear;

$menu = RonPortalMenu::getInstance();
$menu ->load(App::getInstance() ->getDatabase());

?>
<!doctype html>
<html>
<head>
    <?php
    require('ron/apphead.php');
    require('view/apphead.php');

    if ( App::getDebugMode() ) {
        // generuje do console seznam http hlavicek (pro ladici ucely)
        HttpResponse::getInstance() ->getHeaders() ->renderDebugScript();
    }
    ?>
</head>
<body <?= HtmlHeaders::getInstance() ->getBodyAttributesHtml() ?>>

<!-- hlavicka, aplikace, jazyk -->
<?php
require('view/apptitle.php');
require('view/topmenu.php');
require('view/userpanel.php');
?>

<div class="pagecontent">
    <div class="appcontainer">
        <div class="menupanel" id="menupanel">
            <?php RonPortalMenu::getInstance() ->render(); ?>
        </div>

        <div class="appcontent">
            <!-- obsah -->
            <?php
            if (Model::getInstance() ->getViewObjects()) {
                foreach (Model::getInstance() ->getViewObjects() as $viewObject) {
                    try {
                        $viewObject->render();
                    } catch (RonException $e) {
                        global $development;

                        echo "<div class='errortext'>" . StringUtils::htmlTextConvert($e ->getMessage()) . "</div>\n";
                        if ($development || App::getDebugMode() && $e instanceof DBException) {
                            echo '<div>' . $e ->dbMessage . '</div>';
                        }
                        echo '<div>' . $e ->getErrorDescription() . '</div>';
                    } catch (\Exception $e) {
                        echo "<div class='errortext'>" . StringUtils::htmlTextConvert($e ->getMessage()) . "</div>\n";
                    }
                }
            }
            ?>
        </div>
    </div>
</div>

<!-- paticka -->
<div class="footer">
    <div class="copyright">
        &copy; <span class="desktop-only">2003-<?= $copyrightYear ?>, <?= StringUtils::fromUTF8('Všechna práva vyhrazena') ?>, </span><a href="http://www.ron.cz">RON Software</a>, s.r.o.
        <?php if (App::getDebugMode()) { ?>
            DEBUG> mem: <?= (int)(memory_get_usage() / 1024) ?> kB
        <?php } ?>

    </div>

    <div id="nojs"><a href="script/nojs.html"><?= TextManager::_('Adblock_nebo_vypnuty_javascript') ?></a></div>

    <div class='version' title='<?= $appDate ?>'><a href='verze.php' target='_blank'> <?= $appVersion ?></a></div>
</div>

<?php App::getInstance() ->renderHtmlFooter(); ?>

<?php if (Model::getInstance() ->getData('zobrazitNovinky')) { ?>
    <!-- info bublina s informaci o nove verzi -->
    <div class="app_info_bubble clickHref" data-ron-href="?menuid=novinky">
        <?= TextManager::_('Bublina_nova_verze') ?>
    </div>
<?php }
    if (PHP_VERSION_ID < $GLOBALS['phpMinVersion']) { ?>
    <!-- info bublina s informaci o stare verzi PHP -->
    <div class="app_error_bubble clickHref" data-ron-href="https://www.php.net/supported-versions.php">
        <?= TextManager::_('Bublina_stara_verze_PHP') ?>
    </div>
<?php } ?>

<?php require 'view/tasks.php'; ?>

<?php Model::getInstance() ->renderResources(); ?>

</body>
</html>
