<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:php="http://php.net/xsl"
		version="1.0">

  <xsl:output method="xml" version="1.0" indent="yes" encoding="windows-1250"/>

  <!-- korenovy element ronform nebo ronxml - nastavi priznak "neukladat" -->
  <xsl:template match="ronform">
    <xsl:if test="php:function('questform_mode', string(./@mode))">
        <xsl:value-of select="php:function('questform_initformular', string(./@neukladat), string(./@verze) )" />
        <xsl:copy>
          <xsl:apply-templates select="node()|@*" />
        </xsl:copy>
    </xsl:if>
  </xsl:template>
  
  
  <xsl:template match="ronxml">
    <xsl:if test="php:function('questform_mode', string(./@mode))">
        <xsl:value-of select="php:function('questform_initformular', string(./@neukladat), string(./@verze) )" />
        <xsl:copy>
          <xsl:apply-templates select="node()|@*" />
        </xsl:copy>
    </xsl:if>
  </xsl:template>

  <!-- datove xml - nepouziva se xsl -->
  <xsl:template match="ronxmldata">
    <xsl:value-of select="php:function('questform_mode', 'data')" />
  </xsl:template>


  <!-- Promenna - zkopiruje se do ciloveho souboru -->
  <xsl:template match="variable">
    <xsl:value-of select="php:function('questform_setvariable', string(./@name), string(./@value), '', string(./@index), string(./@preserveparams) )" />
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- Promenna s hodnotou v textu tagu - zkopiruje se do ciloveho souboru -->
  <xsl:template match="longvariable">
    <xsl:variable name="value" select="." />
    <xsl:value-of select="php:function('questform_setvariable', string(./@name), string($value), '', string(./@index), string(./@preserveparams) )" />
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- sekce - vzdy se zobrazi jen prvni sekce
             - po vyplneni a odeslani se prvni sekce odstrani, vse co v ni bylo se z ni "vyjme"
             - dokud je v dokumentu nejaka sekce, tak se dokument nepredava dale
             - sekce muzou slouzit napr. pro vyber osoby pred zacatkem vyplnovani dotazniku
  -->
  <xsl:template match="section"> <!-- OK -->
    <xsl:choose>
      <xsl:when test="php:function('questform_sekce')">
        <!-- v dokumentu jeste nebyla zadna sekce, tato je prvni (dalsi bude ignorovat) -->
        <xsl:apply-templates/>
      </xsl:when>

      <xsl:otherwise>
        <!-- jedna se o druhou nebo dalsi sekci - jen se zkopiruje -->
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template match="block"> <!-- OK -->
      <xsl:if test="@ignore != '1' or not(@ignore)">
        <xsl:variable name="ignore">
          <xsl:choose>
            <xsl:when test="@copy != '1'">
              <!-- pokud se nema prenaset do dalsich urovni, tak vse uvnitr bloku provede,
                   ale nastavi ignore=1
                   (nelze rovnou "smazat", protoze se vse uvnitr bloku musi provest)
               -->
              <xsl:value-of select="1" />
            </xsl:when>

            <xsl:otherwise>
              <xsl:value-of select="0" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <block copy="{./@copy}" ignore="{$ignore}" name="{./@name}">
          <xsl:apply-templates/>
        </block>
      </xsl:if>
  </xsl:template>


  <xsl:template match="show">
      <xsl:copy-of select="." />
  </xsl:template>
  
  <xsl:template match="fill">
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" />
    </xsl:copy>
  </xsl:template>



  <xsl:template match="infomsg">
    <xsl:variable name="text" select="." />
    <xsl:copy-of select="php:function('questform_infomsg', string($text) )" />
  </xsl:template>



  <!-- postvalue - nastavi ve formulari hodnotu zadanou uzivatelem - umoznuje napriklad zjistit hodnotu promenne jeste pred prvkem "edit" a podobne -->
  <xsl:template match="postvariable">
    <xsl:copy-of select="php:function('questform_postvariable', string(./@name), string(./@format), string(./@htmlname) )" />
    <xsl:copy-of select="." />
  </xsl:template>


  <xsl:template match="usepostvalues">
    <xsl:copy-of select="php:function('questform_setusepostvalues', string(./@value) )" />
  </xsl:template>


  <!-- Vyhodnoceni vyrazu do promenne se zadanym jmenem -->
  <xsl:template match="expression">
    <xsl:variable name="expr" select="." />
    <xsl:value-of select="php:function('questform_expression', string(./@name), string($expr) )" />
    <xsl:choose>
      <xsl:when test="@copy = '0'  or  @copy = 'no'">
        <!-- kopiruje se jen hodnota promenne -->
        <xsl:variable name="value" select="php:function('questform_variablevalue', string(./@name) )" />
        <variable name="{./@name}" value="{$value}"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- zkopiruje cely tag -->
        <xsl:copy-of select="." />
      </xsl:otherwise>  
    </xsl:choose>
  </xsl:template>


  <xsl:template match="edit">
    <xsl:variable name="defaultvalue" select="." />
    <xsl:copy-of select="php:function('questform_postedit', string(./@name), string(./@format), string(./@required), string(./@editing), string(./@size), string($defaultvalue), string(./@htmlparams) )" />
  </xsl:template>



  <xsl:template match="memo">
    <xsl:variable name="defaultvalue" select="." />
    <xsl:copy-of select="php:function('questform_postmemo', string(./@name), string(./@editing), string(./@rows), string(./@cols), string($defaultvalue) )" />
  </xsl:template>



  <!-- odpoved, ktera byla posledni v predchozim kroku, uz ted neni posledni -->
  <xsl:template match="posledniodpoved">
    <div class="dotaznik_starsiodpoved">
      <xsl:apply-templates select="node()|@*" />
    </div>
  </xsl:template>



  <xsl:template match="checkbox">
    <xsl:variable name="checked" select="php:function('questform_checkpostvalue', string(./@name))" />

    <checkbox name="{./@name}" checked="{$checked}" htmlparams="{./@htmlparams}">
       <xsl:value-of select="." />
    </checkbox>
  </xsl:template>



  <xsl:template match="radio">
    <xsl:choose>
      <xsl:when test="php:function('questform_radiopostvalue', string(./@name)) = ./@value">
        <radio name="{./@name}" value="{./@value}" checked="checked" htmlparams="{./@htmlparams}" />
      </xsl:when>
      
      <xsl:otherwise>
        <radio name="{./@name}" value="{./@value}" htmlparams="{./@htmlparams}" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template match="select">
    <xsl:choose>

      <!-- editing = yes - hodnotu lze dodatecne zmenit -->
      <xsl:when test="@editing = 'yes'  or  @editing = '1'">
        <select name="{./@name}">

        <xsl:for-each select="option">
          <xsl:choose>
            <xsl:when test="php:function('questform_postvalue', string(../@name)) = ./@value">
              <!-- Vybrana volba -->
              <!-- nastavi hodnotu promennych -->
              <xsl:value-of select="php:function('questform_setvariable', string(./@name), string(./@value) )" />
              <xsl:value-of select="php:function('questform_setvariable', string(./@name), string(./@value), '_text' )" />

              <option value="{./@value}" default="1">
                <xsl:apply-templates/>
              </option>
            </xsl:when>

            <xsl:otherwise>
              <!-- neybrana volba -->
              <option value="{./@value}">
                <xsl:apply-templates/>
              </option>
            </xsl:otherwise>
          </xsl:choose>

        </xsl:for-each>

        </select>
      </xsl:when>


      <xsl:otherwise>
        <!-- needitovatelny - po prvnim zadani hodnoty se zmeni na text -->
        <xsl:for-each select="option">
          <xsl:if test="php:function('questform_postvalue', string(../@name)) = ./@value">
            <xsl:variable name="newname" select="concat(../@name, '_text')" />

            <variable name="{../@name}" value="{./@value}" />
            <variable name="{$newname}" value="{./text()}" />

            <div class="dotaznik_odpoved">
              <xsl:value-of select="."/>
            </div>

          </xsl:if>
        </xsl:for-each>

      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- vyber pomoci comboboxu z SQL dotazu -->
  <xsl:template match="sqlselect">
    <xsl:variable name="sql" select="." />
    <xsl:copy-of select="php:function('questform_sqlselectpost', string(./@name), string($sql), string(./@sqltextvariable), string(./@format), string(./@editing), string(./@required), string(./@showcode), string(./@htmlparams) )" />
  </xsl:template>



  <!-- combobox pro vyber osoby -->
  <xsl:template match="vyberosoby">
    <!-- nacte promenne pro vybranou osobu -->
    <xsl:variable name="postvalue" select="php:function('questform_vybranaosoba', string(./@name))" />

    <!-- vlozi combo box nebo vybranou osobu -->
    <xsl:choose>

      <!-- editing = yes - vybranou osobu lze zmenit -->
      <xsl:when test="@editing = 'yes'  or  @editing = '1'">
        <vyberosoby name="{./@name}" editing="yes" value="{$postvalue}" />
      </xsl:when>

      <!-- editing = no nebo neni zadano - nastavi promenne a zobrazi jmeno osoby -->
      <xsl:otherwise>
        <xsl:copy-of select="php:function('questform_osoba_promenne', string(./@name) )" />
        <div class="dotaznik_odpoved">
          <xsl:value-of select="php:function('questform_variablevalue', string(./@name), '_jmenoprijmeni' )" />
        </div>
      </xsl:otherwise>

    </xsl:choose>
  </xsl:template>



  <!-- combobox pro vyber pracovniho pomeru -->
  <xsl:template match="vyberpracovnihopomeru">
    <!-- nacte promenne pro vybrany pracovni pomer -->
    <xsl:variable name="postvalue" select="php:function('questform_vybranypracovnipomer', string(./@name))" />

    <!-- vlozi combo box nebo vybranou osobu (jmeno, prijmeni, Os.C., Datum nastupu) -->
    <xsl:choose>

      <!-- editing = yes - vybrany PP lze zmenit -->
      <xsl:when test="@editing = 'yes'  or  @editing = '1'">
        <vyberpracovnihopomeru name="{./@name}" editing="yes" value="{$postvalue}" />
      </xsl:when>

      <!-- editing = no nebo neni zadano - nastavi promenne a zobrazi jmeno atd. -->
      <xsl:otherwise>
        <xsl:copy-of select="php:function('questform_pracovnipomer_promenne', string(./@name) )" />
        <div class="dotaznik_odpoved">
          <xsl:value-of select="php:function('questform_variablevalue', string(./@name), '_popis' )" />
        </div>
      </xsl:otherwise>

    </xsl:choose>
  </xsl:template>



  <!-- do promenne se zadanym jmenem (name) vlozi SQL filtr pro odfiltrovani zaznamu podle zadaneho typu prava -->
  <xsl:template match="filtrpodleprav">
    <xsl:copy-of select="php:function('questform_filtrpodleprav', string(./@name), string(./@typprava), string(./@tabulka), string(./@obdobiod), string(./@obdobido) )"/>
  </xsl:template>


    <!-- combobox pro vyber obdobi -->
  <xsl:template match="vyberobdobi">
    <!-- nacte promenne pro vybrane obdobi -->
    <xsl:variable name="postvalue" select="php:function('questform_vybraneobdobi', string(./@name), string(./@aplikace), string(./@variabilni) )" />

    <!-- vlozi combo box nebo vybrane obdobi -->
    <xsl:choose>

      <!-- editing = yes - vybrane obdobi lze zmenit -->
      <xsl:when test="@editing = 'yes'  or  @editing = '1'">
        <vyberobdobi name="{./@name}" editing="yes" value="{$postvalue}" aplikace="{./@aplikace}" jenneuzavrene="{./@jenneuzavrene}" jenukoncene="{./@jenukoncene}" variabilni="{./@variabilni}"/>
      </xsl:when>

      <!-- editing = no nebo neni zadano - nastavi promenne a zobrazi popis obdobi -->
      <xsl:otherwise>
        <xsl:copy-of select="php:function('questform_obdobi_promenne', string(./@name), string(./@aplikace) )" />
        <div class="dotaznik_odpoved">
          <xsl:value-of select="php:function('questform_variablevalue', string(./@name), '_popis' )" />
        </div>
      </xsl:otherwise>

    </xsl:choose>
  </xsl:template>



  <!-- combobox pro vyber rozdeleni -->
  <xsl:template match="vyberrozdeleni">
    <!-- nacte promenne pro vybrane rozdeleni -->
    <xsl:variable name="postvalue" select="php:function('questform_vybranerozdeleni', string(./@name), string(./@typ), string(./@required) )" />

    <!-- vlozi combo box nebo vybrane rozdeleni -->
    <xsl:choose>

      <!-- editing = yes - vybrane rozdeleni lze zmenit -->
      <xsl:when test="@editing = 'yes'  or  @editing = '1'">
        <vyberrozdeleni name="{./@name}" editing="yes" value="{$postvalue}" typ="{./@typ}" obdobi="{./@obdobi}" required="{./@required}" >
        <xsl:value-of select="." />
        </vyberrozdeleni>
      </xsl:when>
                                  
      <!-- editing = no nebo neni zadano - nastavi promenne a zobrazi popis rozdeleni -->
      <xsl:otherwise>
        <xsl:copy-of select="php:function('questform_rozdeleni_promenne', string(./@name), string(./@typ) )" />
        <div class="dotaznik_odpoved">
          <xsl:value-of select="php:function('questform_variablevalue', string(./@name), '_popis' )" />
        </div>
      </xsl:otherwise>

    </xsl:choose>
  </xsl:template>



  <!-- nastaveni osoby, pracovniho pomeru, rozdeleni -->
  <xsl:template match="nastavosobuformulare">
    <xsl:copy-of select="php:function('questform_nastavosobuformulare', string(./@name), string(./@rc) )"/>
    <xsl:copy-of select="." />
  </xsl:template>

  
  <xsl:template match="nastavpracovnipomer">
    <xsl:copy-of select="php:function('questform_nastavpracovnipomer', string(./@name), string(./@idpp) )" />
    <xsl:copy-of select="." />
  </xsl:template>

  
  <xsl:template match="nastavrozdeleniformulare">
    <xsl:copy-of select="php:function('questform_nastavrozdeleniformulare',
          string(./@name), string(./@typ), string(./@kod) )"/>
    <xsl:copy-of select="." />
  </xsl:template>
  

  <xsl:template match="nastavobdobiformulare">
    <xsl:copy-of select="php:function('questform_nastavobdobiformulare',
          string(./@name), string(./@aplikace), string(./@obdobi), string(./@variabilni) )"/>
    <xsl:copy-of select="." />
  </xsl:template>

  
  <!-- nastaveni podrobnosti (rozliseni, poznamka) -->
  <xsl:template match="nastavpodrobnostiformulare">
    <xsl:variable name="value" select="." />
    <xsl:copy-of select="php:function('questform_nastavpodrobnostiformulare',
          string(./@fieldname), string($value) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <xsl:template match="prejitnaxml">
    <xsl:copy-of select="php:function('questform_nastavXMLafter', string(./@idsablony) )"/>
    <xsl:copy-of select="." />
  </xsl:template>
  
  
  <xsl:template match="mapovanimenu">
    <xsl:copy-of select="php:function('questform_mapovanimenu', string(./@pokracovat), string(./@menuid) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- provede sql vyraz a zobrazi jeho hodnotu v zadanem formatu -->
  <xsl:template match="sql"> <!-- OK -->
    <xsl:variable name="sql" select="." />
    <xsl:value-of select="php:function('questform_executesql', string($sql), string(./@format) )"/>
  </xsl:template>


  <!-- ulozeni hodnoty sql dotazu do promenne -->
  <xsl:template match="sqlvariable"> <!-- OK -->
    <!-- nastavi hodnotu promenne -->
    <xsl:variable name="sql" select="." />
    <xsl:value-of select="php:function('questform_sqlvariable', string(./@name), string($sql), string(./@sqltextvariable), string(./@format), string(./@log) )"/>

    <xsl:choose>
      <xsl:when test="@copy = '0'  or  @copy = 'no'">
        <!-- kopiruje se jen hodnota promenne -->
        <xsl:variable name="value" select="php:function('questform_variablevalue', string(./@name) )" />
        <variable name="{./@name}" value="{$value}"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- zkopiruje cely tag -->
        <xsl:copy-of select="." />
      </xsl:otherwise>  
    </xsl:choose>
  </xsl:template>



  <!-- editace mzdovych polozek a mesicnich vysledku -->
  <xsl:template match="editacemesicnich">
      <xsl:copy>
        <!--<xsl:copy-of select="php:function('questform_createeditacemesicnich', string(./@datumpp), string(./@sloupce), string(./@sirkapole) )" /> -->
        <xsl:copy-of select="php:function('questform_createeditacemesicnich', ., string(./@content) )" />
        <!--<xsl:apply-templates/> -->
        <xsl:apply-templates select="node()|@*" />
        <!-- ulozi nactene hodnoty formulare -->
        <xsl:copy-of select="php:function('questform_zapismesicni')" />
        <xsl:copy-of select="php:function('questform_editacemesicnich_vratlimit')" />
      </xsl:copy>
  </xsl:template>

  <xsl:template match="sloupeceditacemesicnich">
      <xsl:copy-of select="." />
  </xsl:template>

  <xsl:template match="filtreditacemesicnich">
      <xsl:copy-of select="." />
  </xsl:template>

  <xsl:template match="editacemesicnichlimit">
      <!-- <xsl:copy-of select="php:function('questform_editacemesicnich_povolitlimit', string(./@povolit) )" /> -->
  </xsl:template>



  <!-- editace zaznamu v databazi -->
  <xsl:template match="record">
      <!-- vytvori zaznam, vnorene tagy ho nastavi -->
      <xsl:variable name="pom_record" select="php:function('questform_createdbrecord', string(./@tablename) )"/>

      <xsl:copy>
        <xsl:apply-templates select="node()|@*" />
      </xsl:copy>
  </xsl:template>


  <!-- nacteni zaznamu z databaze do promennych -->
  <xsl:template match="loadrecord">
      <xsl:copy-of select="php:function('questform_loadrecord', string(./@name) )" />
      <xsl:copy-of select="." />
  </xsl:template>


  <xsl:template match="editrecord">
      <xsl:variable name="pom_editrecord" select="php:function('questform_editrecord' )" />
      <xsl:copy-of select="." />
  </xsl:template>
  

  <!-- vlozeni noveho zaznamu -->
  <xsl:template match="insertrecord">
      <xsl:variable name="pom_insertrecord" select="php:function('questform_insertrecord' )" />
      <xsl:copy-of select="." />
  </xsl:template>

  
  <xsl:template match="updateorinsertrecord">
      <xsl:variable name="pom_updateinsertrecord" select="php:function('questform_updateorinsertrecord' )" />
      <xsl:copy-of select="." />
  </xsl:template>

  
  <!-- smazani zaznamu -->
  <xsl:template match="deleterecord">
      <xsl:variable name="pom_deleterecord" select="php:function('questform_deleterecord' )" />
      <xsl:copy-of select="." />
  </xsl:template>


  <xsl:template match="recordfield">
      <xsl:variable name="pom_recordfield" select="php:function('questform_createrecordfield', string(./@name), string(./@format), string(./@readonly), string(./@size), string(./@updatelog), string(./text()) )" />
      <xsl:copy-of select="." />
  </xsl:template>
  

  <xsl:template match="recordwhere">
      <xsl:variable name="sql" select="." />
      <xsl:variable name="pom_recordwhere" select="php:function('questform_recordwhere', string($sql) )" />
      
      <xsl:copy-of select="." />
  </xsl:template>



  <!-- pole pro vlozeni dokumentu -->
  <!--
  <xsl:template match="dokument">
      <xsl:copy-of select="php:function('questform_vlozdokument', string(./@name), string(./@popis), string(./@typ), string(./@editing), string(./@style) )" />
  </xsl:template>
  -->

  <!-- jiz pripojeny dokument -->
  <xsl:template match="pripojenydokument">
      <xsl:variable name="pom_dokument" select="php:function('questform_pripojenydokument', string(./@name), string(./@id)  )" />
      <xsl:copy-of select="." />
  </xsl:template>



  <!-- odeslani emailu -->
  <xsl:template match="email">
    <xsl:variable name="obsah" select="." />
    <xsl:value-of select="php:function('questform_email', string(./@to), string(./@subject), string($obsah), string(./@from), string(./@copyto), string(./@log), string(./@variablename) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- zapis zaznamu do LOGu -->
  <xsl:template match="log">
    <xsl:variable name="comment" select="." />
    <xsl:value-of select="php:function('questform_log', string(./@description), string(./@category), string($comment) )"/>
    <xsl:copy-of select="." />
  </xsl:template>
  
  <xsl:template match="dblog">
    <xsl:variable name="comment" select="." />
    <xsl:value-of select="php:function('questform_dblog', string(./@tablename), string(./@eventtype), string(./@srcdescription), string($comment) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- provede sql vyraz - nezobrazi nic, vyraz vraci jen true/false -->
  <xsl:template match="sqlaction">
    <xsl:variable name="sqlactionresult" select="php:function('questform_executesqlaction', string(./text()), string(./@sqltextvariable) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- spusteni skriptu -->
  <xsl:template match="script">
    <xsl:value-of select="php:function('questform_script', string(./@scriptcategory), string(./@scriptname), string(./@scriptparameter), string(./@variablename) )"/>
    <xsl:copy-of select="." />
  </xsl:template>



  <!-- ne/pokracovani vyplnovani formulare, tj. formular se po ulozeni nezavre -->
  <xsl:template match="pokracovat">
    <xsl:value-of select="php:function('questform_pokracovat', string(./@value) )"/>
    <xsl:copy-of select="." />
  </xsl:template> 


  <!-- zmena stavu formulare   <stav novystav="0" /> -->
  <xsl:template match="stav"> <!-- OK -->
    <xsl:value-of select="php:function('questform_nastavstav', string(./@novystav) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- uzavreni formulare  <uzavrit />  - je to totez jako  <stav novystav="128" /> -->
  <xsl:template match="uzavrit"> <!-- OK -->
    <xsl:value-of select="php:function('questform_nastavstav', '128')"/>
    <xsl:copy-of select="." />
  </xsl:template>

  <!-- smazani formulare z databaze -->
  <xsl:template match="delete"> <!-- OK -->
    <xsl:value-of select="php:function('questform_delete', string(./@okinfomsg) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- zobrazeni chybove hlasky -->
  <xsl:template match="error">
    <xsl:value-of select="php:function('questform_error', string(./@fieldname), string(./@text) )"/>
    <xsl:copy-of select="." />
  </xsl:template>


  <!-- podminene provedeni -->
  <xsl:template match="test"> <!-- OK -->
    <xsl:variable name="sql" select="./text()" />
    <xsl:choose>
        <xsl:when test="php:function('questform_test', string(./@testtype), string($sql), string(./@param1), string(./@param2) )">
          <xsl:choose>
            <xsl:when test="@copy = '1'">
              <xsl:copy>
                <xsl:apply-templates select="@*|text()"/>
                
                <testok>
                  <xsl:apply-templates select="./testok"/>
                </testok>
                
                <xsl:copy-of select="./else"/>
              </xsl:copy>
            </xsl:when>
            
            <xsl:otherwise>  
                <xsl:apply-templates select="./testok"/>
            </xsl:otherwise>  
          </xsl:choose>
        </xsl:when>
      
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="@copy = '1'">
              <xsl:copy>
                <xsl:apply-templates select="@*|text()"/>
                <xsl:copy-of select="./testok"/>
                
                <else>  
                  <xsl:apply-templates select="./else"/>
                </else>  
              </xsl:copy>
            </xsl:when>
            
            <xsl:otherwise>  
                <xsl:apply-templates select="./else"/>
            </xsl:otherwise>  
          </xsl:choose>
          <!--<xsl:apply-templates select="./else"/> -->
        </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template match="testok"> <!-- OK -->
      <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="else"> <!-- OK -->
      <xsl:apply-templates/>
  </xsl:template>



  <!-- podminene provedeni -->
  <xsl:template match="if">
    <xsl:choose>
        <xsl:when test="php:function('questform_if', string(./@condition) )">
          <xsl:choose>
            <xsl:when test="@copy = '1'">
              <xsl:copy>
                <xsl:apply-templates select="node()|@*"/>
              </xsl:copy>
            </xsl:when>
            
            <xsl:otherwise>  
              <xsl:apply-templates />
            </xsl:otherwise>  
          </xsl:choose>
        </xsl:when>
      
        <xsl:otherwise>
          <xsl:if test="@copy = '1'">
            <xsl:copy-of select="." />
          </xsl:if>
        </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  


  <xsl:template match="forsql"> <!-- OK -->
    <xsl:choose>
      <xsl:when test="@expand = 'yes'  or  @expand = '1'">
        <xsl:variable name="for_data" select="php:function('questform_forsql', string(./@name), string(./@sqltext), string(./@sqltextvariable) )" />
      
        <xsl:variable name="for_content" select="./*" />
      
        <xsl:for-each select="$for_data//XMLRESULT/ROW">
          <xsl:apply-templates/> <!-- nastavi hodnoty promennych podle radku databaze -->
          <xsl:apply-templates select="$for_content"/>
        </xsl:for-each> 
      </xsl:when>
      
      <xsl:otherwise>
        <!-- forsql se nema "rozgenerovat" - jen ho zkopiruje vcetne obsahu -->
        <xsl:if test="@processonfill = 'yes'  or  @processonfill = '1'">
          <!-- obsah forsql se ma provest i pri fill, ale vygenerovany obsah se ignoruje -->
          <block ignore="1">
            <xsl:variable name="for_data" select="php:function('questform_forsql', string(./@name), string(./@sqltext), string(./@sqltextvariable) )" />

            <xsl:variable name="for_content" select="./*" />

            <xsl:for-each select="$for_data//XMLRESULT/ROW">
              <xsl:apply-templates/> <!-- nastavi hodnoty promennych podle radku databaze -->
              <xsl:apply-templates select="$for_content"/>
            </xsl:for-each>
          </block>
        </xsl:if>

        <xsl:copy-of select="." />
      </xsl:otherwise>
      
    </xsl:choose>
  </xsl:template>




  <!-- hodnoceni a testy ================================================== -->
  <!--
  <xsl:template match="otazkyhodnoceni">
      <xsl:variable name="hodnoceni_data" select="php:function('questform_otazkyhodnoceni', string(./@id), string(./@druhhodnoceni), string(./@temacislo) )" />
      
      <xsl:variable name="hodnoceni_content" select="./*" />
      
      <xsl:for-each select="$hodnoceni_data//XMLRESULT/ROW">
          <xsl:apply-templates/>
          <xsl:apply-templates select="$hodnoceni_content"/>
      </xsl:for-each> 
  </xsl:template>
  
  
  <xsl:template match="ulozodpovedihodnoceni">
      <xsl:copy-of select="php:function('questform_ulozodpovedihodnoceni')" />
  </xsl:template>

  <xsl:template match="prepocitejhodnoceni">
      <xsl:copy-of select="php:function('questform_prepocitejhodnoceni', string(./@neulozene) )" />
  </xsl:template>
  -->
  

  <!-- zjisti, jestli se ma zadana akce provest a pripadne ji provede -->
  <xsl:template match="action">
    <xsl:choose>
      <xsl:when test="php:function('questform_action', string(./@name))">
        <!-- provede jednotlive akce -->
        <xsl:copy>
          <xsl:apply-templates  select="node()|@*" />
        </xsl:copy>
      </xsl:when>

      <xsl:otherwise>
        <!-- tato akce se nema provest - jen ji zkopiruje vcetne obsahu -->
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template match="button">
    <xsl:choose>
      <xsl:when test="php:function('questform_action', string(./@name))">
        <!-- provede jednotlive akce -->
        <xsl:copy>
          <xsl:apply-templates  select="node()|@*" />
        </xsl:copy>
      </xsl:when>

      <xsl:otherwise>
        <!-- tato akce se nema provest - jen ji zkopiruje vcetne obsahu -->
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- pokud bylo stisknuto "ulozit", tak se provedou pripadne vnorene akce -->
  <xsl:template match="ulozit">
    <xsl:choose>
      <xsl:when test="php:function('questform_action', 'submit')">
        <xsl:copy>
          <xsl:apply-templates  select="node()|@*" />
        </xsl:copy>
      </xsl:when>

      <xsl:otherwise>
        <!-- tato akce se nema provest - jen ji zkopiruje -->
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- pokud bylo stisknuto "odeslat", tak se provedou pripadne vnorene akce -->
  <xsl:template match="predatdale">
    <xsl:choose>
      <xsl:when test="php:function('questform_predatdale')">
        <xsl:copy>
          <xsl:apply-templates  select="node()|@*" />
        </xsl:copy>
      </xsl:when>

      <xsl:otherwise>
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- pokud bylo stisknuto "vratit", tak se provedou pripadne vnorene akce -->
  <xsl:template match="vratitzpet">
    <xsl:choose>
      <xsl:when test="php:function('questform_vratitzpet', string(./@zacatek))">
        <xsl:copy>
          <xsl:apply-templates  select="node()|@*" />
        </xsl:copy>
      </xsl:when>

      <xsl:otherwise>
        <xsl:copy-of select="." />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- tag pro programove predani -->
  <xsl:template match="predat">
    <xsl:value-of select="php:function('questform_predat', string(./@smer) )" />
    <xsl:copy-of select="." />
  </xsl:template>

  <!-- tag XMLRESULT vynecha - zpracuje jeho vnorene elementy -->
  <xsl:template match="XMLRESULT">
      <xsl:apply-templates/>
  </xsl:template>

  <!-- tag xmlresult vynecha - zpracuje jeho vnorene elementy -->
  <xsl:template match="xmlresult">
      <xsl:apply-templates/>
  </xsl:template>



  <!-- tagy, ktere nenajde, jen zkopiruje -->
  <xsl:template match="node()|@*">
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" />
    </xsl:copy>
  </xsl:template>


</xsl:stylesheet>
