<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:php="http://php.net/xsl"
		version="1.0">

  <xsl:output method="html" omit-xml-declaration="yes" indent="yes" encoding="windows-1250"/>

  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>



  <xsl:template match="ronform">
      <xsl:if test="php:function('questform_mode', string(./@mode))">
        <xsl:value-of select="php:function('questform_initformular', string(./@neukladat), string(./@verze) )" />
        <xsl:variable name="action" select="php:function('questform_xmlformaction')" />

        <div class="errormsg">XML formular je zastaraly a brzy jiz nebude podporovan - upravte formular a nastavte u nej atribut mode!</div>

        <form method="post" enctype="multipart/form-data" action="{$action}">

          <!-- Vlozeni skrytych poli (id, typ formulare) -->
          <!-- Korenovy element ignoruje -->
          <xsl:variable name="fields" select="php:function('questform_formfields')" />
          <xsl:for-each select="$fields/*/*">
            <xsl:copy-of select="." />
          </xsl:for-each>

          <xsl:apply-templates/>

        </form>
      </xsl:if>
  </xsl:template>


  <xsl:template match="ronxml">
    <div class="errormsg">XML formular je zastaraly a brzy jiz nebude podporovan - upravte formular a nastavte u nej atribut mode!</div>

    <xsl:if test="php:function('questform_mode', string(./@mode))">
      <xsl:value-of select="php:function('questform_initformular', string(./@neukladat), string(./@verze) )" />
      <xsl:apply-templates/>
    </xsl:if>
  </xsl:template>


  <xsl:template match="ronxmldata">
      <xsl:value-of select="php:function('questform_mode', 'data')" />
  </xsl:template>


  <!-- sekce - vzdy se zobrazi jen prvni sekce
             - po vyplneni a odeslani se prvni sekce odstrani, vse co v ni bylo se z ni "vyjme"
             - dokud je v dokumentu nejaka sekce, tak se dokument nepredava dale
             - sekce muzou slouzit napr. pro vyber osoby pred zacatkem vyplnovani dotazniku
  -->
  <xsl:template match="section">
      <xsl:if test="php:function('questform_sekce')">
        <!-- v dokumentu jeste nebyla zadna sekce, tato je prvni (dalsi bude ignorovat) -->
        <xsl:apply-templates/>
      </xsl:if>
  </xsl:template>



  <!-- block - nastaveni provadeni vsech tagu uvnitr bloku - OK -->
  <xsl:template match="block"> <!-- OK -->
      <xsl:if test="@ignore != '1' or not(@ignore) ">
        <xsl:apply-templates/>
      </xsl:if>
  </xsl:template>


  <!--  ADS Filter Form  =================================================  -->
  <!-- formular pro filtrovani dat - pro readonly (nahledove) formulare -->
  <xsl:template match="adsfilterform"> <!-- OK -->
      <!-- vytvori formular - zatim ho nezobrazuje, vnorene tagy ho nastavi -->
      <xsl:value-of select="php:function('questform_createadsfilterform', string(./@autosubmit))" />
      <xsl:apply-templates/>
      <!-- zobrazi formular -->
      <xsl:value-of select="php:function('questform_adsfilterform')" />
  </xsl:template>

  <xsl:template match="adsfilterform_obdobi"> <!-- OK -->
      <xsl:value-of select="php:function('questform_adsfilterform_obdobi', string(./@aplikace), string(./@jenotevrene))" />
  </xsl:template>

  <xsl:template match="adsfilterform_osoby"> <!-- OK -->
      <xsl:value-of select="php:function('questform_adsfilterform_osoby', string(./@prava), string(./@predavaciproces))" />
  </xsl:template>

  <xsl:template match="adsfilterform_rozdeleni"> <!-- OK -->
      <xsl:variable name="sqlfilter" select="." />
      <xsl:value-of select="php:function('questform_adsfilterform_rozdeleni', string(./@typrozdeleni), string(./@pouzevlastni), string(./@prava), string($sqlfilter) )" />
  </xsl:template>

  <xsl:template match="adsfilterform_rozdeleni2"> <!-- OK -->
      <xsl:variable name="sqlfilter" select="." />
      <xsl:value-of select="php:function('questform_adsfilterform_rozdeleni2', string(./@typrozdeleni), string(./@pouzevlastni), string(./@prava), string($sqlfilter) )" />
  </xsl:template>


  <!--  Browser    =======================================================  -->
  <!-- seznam z sql dotazu -->
  <xsl:template match="sqlbrowser">
    <xsl:variable name="sql" select="." />
    <xsl:copy-of select="php:function('questform_sqlbrowser', string($sql), string(./@captions) )"/>
  </xsl:template>

  <xsl:template match="browser">  <!-- OK -->
      <!-- vytvori browser - zatim ho nezobrazuje, vnorene tagy ho nastavi -->
      <xsl:copy-of select="php:function('questform_createbrowser', string(./@name))" />
      <xsl:apply-templates/>
      <!-- zobrazi browser -->
      <xsl:copy-of select="php:function('questform_browser' )" />
  </xsl:template>

  <xsl:template match="browsersql"> <!-- OK -->
      <xsl:variable name="sql" select="." />
      <xsl:copy-of select="php:function('questform_browsersql', string($sql) )" />
  </xsl:template>

  <xsl:template match="browserfield"> <!-- OK -->
      <xsl:copy-of select="php:function('questform_browserfield', string(./@sqlindex), string(./@format), string(./@caption), string(./@style), string(./@link), string(./@text), string(./@htmlparams) )" />
  </xsl:template>

  <xsl:template match="browserdetail"> <!-- V XSL odstraneno -->
  </xsl:template>

  <xsl:template match="browserlink"> <!-- V XSL odstraneno -->
  </xsl:template>


  <xsl:template match="dochazkabrowser"> <!-- V XSL odstraneno -->
  </xsl:template>



  <!--  OLAP Cube  - odstraneno 23.12.2021 - dale nepodporovano =============================================  -->

  <!-- ORAMOVANA TABULKA =================================================  -->
  <xsl:template match="inputtable"> <!-- OK -->
     <table class="XSL-inputtable">
        <xsl:apply-templates/>
     </table>
  </xsl:template>

  <xsl:template match="inputtablerow"> <!-- OK -->
     <tr>
        <td class="XSL-formtablecaption">
           <xsl:value-of select="./@caption"/>
        </td>
        <td>
           <xsl:apply-templates/>
        </td>
     </tr>
  </xsl:template>



  <!--  PROMENNE  ========================================================  -->
  <!-- Definice promenne - definuje se xml promenna -->
  <!-- <variable name="jmeno" value="hodnota" /> -->
  <xsl:template match="variable"> <!-- OK -->
    <xsl:value-of select="php:function('questform_setvariable', string(./@name), string(./@value), '', string(./@index), string(./@preserveparams) )" />
  </xsl:template>


  <!-- Definice promenne s hodnotou v textu tagu - definuje se xml promenna - OK -->
  <!-- <longvariable name="jmeno">hodnota</longvariable> -->
  <xsl:template match="longvariable">
    <xsl:variable name="value" select="." />
    <xsl:value-of select="php:function('questform_setvariable', string(./@name), string($value), '', string(./@index), string(./@preserveparams) )" />
  </xsl:template>


  <!-- Zobrazeni hodnoty promenne -->
  <!-- <variablevalue name="jmeno" format="format"/> - OK -->
  <xsl:template match="variablevalue">
    <xsl:value-of select="php:function('questform_variablevalue', string(./@name), string(./@suffix), string(./@index), string(./@format) )" />
  </xsl:template>



  <!-- Primy zapis do HTML s nahrazenim parametru - OK -->
  <xsl:template match="text">
    <xsl:variable name="value" select="." />
    <!-- disable-output-escaping, protoze se to pouziva pro skripty! -->
    <xsl:value-of disable-output-escaping="yes" select="php:function('questform_replacevariables', string($value), '' )" />
  </xsl:template>


  <!-- Vyhodnoceni vyrazu do promenne se zadanym jmenem -->
  <xsl:template match="expression">
    <xsl:variable name="expr" select="." />
    <xsl:value-of select="php:function('questform_expression', string(./@name), string($expr) )" />
  </xsl:template>


  <!--  EDITOVATELNE PRVKY  ==============================================  -->
  <!-- edit - jednoradkove textove pole -->
  <xsl:template match="edit"> <!-- OK -->
    <!-- <xsl:variable name="defaultvalue" select="." /> -->
    <xsl:variable name="defaultvalue" select="./text()" />
    <xsl:copy-of select="php:function('questform_edit', string(./@name), string(./@format), string(./@required), string($defaultvalue), string(./@size), string(./@htmlparams) )" />
  </xsl:template>


  <!-- postvalue - nastavi ve formulari hodnotu zadanou uzivatelem - umoznuje napriklad zjistit hodnotu promenne jeste pred prvkem "edit" -->
  <xsl:template match="postvariable"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_postvariable', string(./@name), string(./@format), string(./@htmlname) )" />
  </xsl:template>

  <xsl:template match="usepostvalues"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_setusepostvalues', string(./@value) )" />
  </xsl:template>



  <xsl:template match="hidden"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_hidden', string(./@name), string(./@value), string(./@format) )" />
  </xsl:template>


  <xsl:template match="calendarbutton">
    <!-- odstraneno 23.12.2021 - v XSL dale nepodporovano -->
    <!-- <xsl:copy-of select="php:function('questform_calendarbutton', string(./@name) )" /> -->
  </xsl:template>


  <!-- memo - viceradkove textove pole - OK -->
  <xsl:template match="memo">
    <xsl:variable name="value" select="." />
    <xsl:copy-of select="php:function('questform_memo', string(./@name), string(./@rows), string(./@cols), string($value) )" />
  </xsl:template>


  <!-- checkbox - OK -->
  <xsl:template match="checkbox">
    <xsl:copy-of select="php:function('questform_checkbox', string(./@name), string(./@checked), string(./@htmlparams) )" />
  </xsl:template>


  <!-- radiobutoky - skupinu tvori radiobutonky se stejnym jmenem - OK -->
  <xsl:template match="radio">
    <xsl:copy-of select="php:function('questform_radio', string(./@name), string(./@value), string(./@checked), string(./@htmlparams) )" />
  </xsl:template>


  <!-- vyber pomoci comboboxu ze zadanych hodnot - OK -->
  <xsl:template match="select">
    <SELECT name="{./@name}">
      <xsl:for-each select="option">
        <xsl:choose>
          <xsl:when test="@default = '1'">
            <OPTION value="{./@value}" selected="selected">

            <!-- nastavi hodnotu promennych napr. pro pouziti v sql vyrazech -->
            <xsl:value-of select="php:function('questform_setvariable', string(../@name), string(./@value) )" />
            <xsl:value-of select="php:function('questform_setvariable', string(../@name), string(./text()), '_text' )" />

            <xsl:apply-templates/>
            </OPTION>
          </xsl:when>

          <xsl:otherwise>
            <OPTION value="{./@value}">
            <xsl:apply-templates/>
            </OPTION>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </SELECT>
  </xsl:template>


  <!-- posledni zadana odpoved u typu append - OK -->
  <xsl:template match="posledniodpoved">
    <div class="dotaznik_odpoved">
      <xsl:apply-templates/>
    </div>
  </xsl:template>



  <!-- combobox pro vyber osoby -->
  <xsl:template match="vyberosoby">
    <xsl:copy-of select="php:function('questform_vyberosoby', string(./@name), string(./@value) )"/>
  </xsl:template>


  <!-- combobox pro vyber pracovnihopomeru -->
  <xsl:template match="vyberpracovnihopomeru"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_vyberpracovnihopomeru', string(./@name), string(./@value), string(./@pouzevlastni) )"/>
  </xsl:template>


  <!-- do promenne se zadanym jmenem (name) vlozi SQL filtr pro odfiltrovani zaznamu podle zadaneho typu prava -->
  <xsl:template match="filtrpodleprav">
      <xsl:copy-of select="php:function('questform_filtrpodleprav', string(./@name), string(./@typprava), string(./@tabulka), string(./@obdobiod), string(./@obdobido) )"/>
  </xsl:template>
  
  
  <!-- combobox pro vyber obdobi -->
  <xsl:template match="vyberobdobi"> <!-- OK -->
    <xsl:variable name="sqlfilter" select="." />
    <xsl:copy-of select="php:function('questform_vyberobdobi',
          string(./@name), string(./@value), string(./@aplikace),
          string(./@jenneuzavrene), string(./@jenukoncene), string($sqlfilter), string(./@variabilni) )"/>
  </xsl:template>


  <!-- combobox pro vyber rozdeleni -->
  <xsl:template match="vyberrozdeleni"> <!-- OK -->
    <xsl:variable name="sqlfilter" select="." />
    <xsl:copy-of select="php:function('questform_vyberrozdeleni',
          string(./@name), string(./@value), string(./@typ), string($sqlfilter), string(./@obdobi), string(./@pouzevlastni) )"/>
  </xsl:template>


  <xsl:template match="nastavrozdeleniformulare"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_nastavrozdeleniformulare',
          string(./@name), string(./@typ), string(./@kod) )"/>
  </xsl:template>


  <xsl:template match="nastavobdobiformulare"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_nastavobdobiformulare',
          string(./@name), string(./@aplikace), string(./@obdobi), string(./@variabilni) )"/>
  </xsl:template>


  <!-- nastaveni osoby, pracovniho pomeru, rozdeleni -->
  <xsl:template match="nastavosobuformulare"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_nastavosobuformulare', string(./@name), string(./@rc) )"/>
  </xsl:template>

  
  <xsl:template match="nastavpracovnipomer"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_nastavpracovnipomer', string(./@name), string(./@idpp) )" />
  </xsl:template>
  

  <!-- nastaveni podrobnosti (rozliseni, poznamka) -->
  <xsl:template match="nastavpodrobnostiformulare"> <!-- OK -->
    <xsl:variable name="value" select="." />
    <xsl:copy-of select="php:function('questform_nastavpodrobnostiformulare',
          string(./@fieldname), string($value) )"/>
  </xsl:template>


  <xsl:template match="prejitnaxml">
    <xsl:copy-of select="php:function('questform_nastavXMLafter', string(./@idsablony) )"/>
  </xsl:template>
  

  <xsl:template match="mapovanimenu">
    <xsl:copy-of select="php:function('questform_mapovanimenu', string(./@pokracovat), string(./@menuid) )"/>
  </xsl:template>


  <!-- vyber combobox z SQL dotazu - OK -->
  <xsl:template match="sqlselect">
    <xsl:variable name="sql" select="." />
    <xsl:copy-of select="php:function('questform_sqlselect', string(./@name), string($sql), string(./@sqltextvariable), string(./@format), string(./@value), string(./@required), string(./@showcode), string(./@htmlparams) )"/>
  </xsl:template>


  <!-- provede sql vyraz a zobrazi jeho hodnotu v zadanem formatu - OK -->
  <xsl:template match="sql">
    <xsl:variable name="sql" select="." />
    <xsl:value-of select="php:function('questform_executesql', string($sql), string(./@format) )"/>
  </xsl:template>


  <!-- ulozeni hodnoty sql dotazu do promenne - OK -->
  <xsl:template match="sqlvariable">
    <xsl:variable name="sql" select="." />
    <xsl:value-of select="php:function('questform_sqlvariable', string(./@name), string($sql), string(./@sqltextvariable), string(./@format), string(./@log) )"/>
  </xsl:template>



  <!-- editace mzdovych polozek a mesicnich vysledku -->
  <xsl:template match="editacemesicnich">
      <xsl:copy-of select="php:function('questform_createeditacemesicnich', ., string(./@content) )" />

      <!-- zobrazi formular -->
      <xsl:copy-of select="php:function('questform_editacemesicnich')" />
  </xsl:template>


  <!-- editace zaznamu v databazi - OK -->
  <xsl:template match="record">
      <!-- vytvori zaznam, vnorene tagy ho nastavi a provedou pripadnou akci -->
      <xsl:copy-of select="php:function('questform_createdbrecord', string(./@tablename) )" />
      <xsl:apply-templates/>
  </xsl:template>


  <!-- nacteni zaznamu z databaze do promennych - OK -->
  <xsl:template match="loadrecord">
      <xsl:copy-of select="php:function('questform_loadrecord', string(./@name) )" />
  </xsl:template>

  
  <!-- editace zaznamu v databazi - OK -->
  <xsl:template match="editrecord">
      <xsl:copy-of select="php:function('questform_editrecord' )" />
  </xsl:template>
  

  <!-- vlozeni noveho zaznamu - OK -->
  <xsl:template match="insertrecord">
      <xsl:copy-of select="php:function('questform_insertrecord' )" />
  </xsl:template>
  

  <xsl:template match="updateorinsertrecord">  <!-- OK -->
      <xsl:copy-of select="php:function('questform_updateorinsertrecord' )" />
  </xsl:template>


  <!-- smazani zaznamu - OK -->
  <xsl:template match="deleterecord">
      <xsl:copy-of select="php:function('questform_deleterecord' )" />
  </xsl:template>


  <xsl:template match="recordfield">  <!-- OK -->
      <xsl:copy-of select="php:function('questform_createrecordfield', string(./@name), string(./@format), string(./@readonly), string(./@size), string(./@updatelog), string(./text()) )" />
  </xsl:template>
  

  <xsl:template match="recordwhere">  <!-- OK -->
      <xsl:variable name="sql" select="." />
      <xsl:copy-of select="php:function('questform_recordwhere', string($sql) )" />
  </xsl:template>



  <!-- pole pro vlozeni dokumentu -->
  <xsl:template match="dokument"> <!-- OK -->
      <div class="errormsg">XML formular je zastaraly a brzy jiz nebude podporovan - upravte formular a nastavte u nej atribut mode!</div>
   <!--   <xsl:copy-of select="php:function('questform_dokument', string(./@name), string(./@popis), string(./@typ), string(./@style) )" /> -->
  </xsl:template>

  <!-- jiz pripojeny dokument -->
  <xsl:template match="pripojenydokument"> <!-- OK -->
      <xsl:copy-of select="php:function('questform_pripojenydokument', string(./@name), string(./@id)  )" />
  </xsl:template>
  
  

  <!-- ZAKLADNI TLACITKA =================================================  -->
  <xsl:template match="ulozit"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_submitbutton', string(./@text), string(./@question) )"/>
  </xsl:template>

  <xsl:template match="predatdale"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_submitnextbutton', string(./@text), string(./@question) )"/>
  </xsl:template>

  <xsl:template match="vratitzpet"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_submitprevbutton', string(./@text), string(./@question) )"/>
  </xsl:template>



  <!-- zobrazi tlacitko pro provedeni akce -->
  <xsl:template match="action"> <!-- OK -->
      <xsl:if test="php:function('questform_action_show', string(./@name))">
          <!-- provede jednotlive akce -->
          <xsl:apply-templates/>
          <!--<xsl:apply-templates  select="node()|@*" /> -->
      </xsl:if>

      <xsl:copy-of select="php:function('questform_actionbutton', string(./@name), string(./@text), string(./@question), string(./@urovne) )"/>
  </xsl:template>

  <!-- button - stejne jako action, ale zobrazi se vsem -->
  <xsl:template match="button"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_actionbutton', string(./@name), string(./@text), string(./@question), '#all' )"/>
  </xsl:template>
  
  
  <xsl:template match="link">
    <xsl:copy-of select="php:function('questform_link', string(./@action), string(./@params), string(./@text), string(./@question) )" />
  </xsl:template>



  <xsl:template match="email"> <!-- OK -->
    <xsl:variable name="obsah" select="." />
    <xsl:value-of select="php:function('questform_email', string(./@to), string(./@subject), string($obsah), string(./@from), string(./@copyto), string(./@log), string(./@variablename) )"/>
  </xsl:template>

  <xsl:template match="script"> <!-- OK -->
    <xsl:value-of select="php:function('questform_script', string(./@scriptcategory), string(./@scriptname), string(./@scriptparameter), string(./@variablename) )"/>
    <xsl:value-of select="." />
  </xsl:template>

  <xsl:template match="sqlaction"> <!-- OK -->
    <xsl:variable name="sqlactionresult" select="php:function('questform_executesqlaction', string(./text()), string(./@sqltextvariable) )"/>
  </xsl:template>

  <xsl:template match="log"> <!-- OK -->
    <xsl:variable name="comment" select="." />
    <xsl:value-of select="php:function('questform_log', string(./@description), string(./@category), string($comment) )"/>
  </xsl:template>

  <xsl:template match="dblog"> <!-- OK -->
    <xsl:variable name="comment" select="." />
    <xsl:value-of select="php:function('questform_dblog', string(./@tablename), string(./@eventtype), string(./@srcdescription), string($comment) )"/>
  </xsl:template>



  <xsl:template match="stav">
  </xsl:template>

  <xsl:template match="uzavrit">
  </xsl:template>

  <xsl:template match="delete"> <!-- OK -->
  </xsl:template>

  <xsl:template match="error"> <!-- OK -->
    <xsl:copy-of select="php:function('questform_showerror', string(./@fieldname), string(./@text) )"/>
  </xsl:template>
  

  <xsl:template match="show"> <!-- OK -->
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" />
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="fill"> <!-- OK -->
  </xsl:template>

  
  
  <xsl:template match="debugmsg"> <!-- OK -->
  </xsl:template>

  <xsl:template match="infomsg"> <!-- OK -->
    <xsl:variable name="text" select="." />
    <xsl:copy-of select="php:function('questform_infomsg', string($text) )" />
  </xsl:template>



  <xsl:template match="test"> <!-- OK -->
      <xsl:choose>
        <xsl:when test="php:function('questform_test', string(./@testtype), string(./text()), string(./@param1), string(./@param2) )">
          <xsl:apply-templates select="./testok"/>
        </xsl:when>
      
        <xsl:otherwise>
          <xsl:apply-templates select="./else"/>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>
  
  
  <xsl:template match="if">   <!-- OK -->
      <xsl:if test="php:function('questform_if', string(./@condition) )">
        <xsl:apply-templates/>
      </xsl:if>
  </xsl:template>


  <xsl:template match="forsql">      <!-- OK -->
      <xsl:variable name="for_data" select="php:function('questform_forsql', string(./@name), string(./@sqltext), string(./@sqltextvariable) )" />
      
      <xsl:variable name="for_content" select="./*" />
      
      <xsl:for-each select="$for_data//XMLRESULT/ROW">
          <xsl:apply-templates/> <!-- nastavi hodnoty promennych podle radku databaze -->
          <xsl:apply-templates select="$for_content"/>
      </xsl:for-each> 
      <!--    <xsl:for-each select="$postvalue//XMLRESULT/node()"> -->
  </xsl:template>


    <xsl:template match="foreach">
        <xsl:variable name="foreach_data" select="php:function('questform_foreach', string(./@name), string(./@arrayvariable) )" />

        <xsl:variable name="foreach_content" select="./*" />

        <xsl:for-each select="$foreach_data//XMLRESULT/ROW">
            <xsl:apply-templates/> <!-- nastavi hodnoty promennych podle radku databaze -->
            <xsl:apply-templates select="$foreach_content"/>
        </xsl:for-each>
    </xsl:template>


  <!-- uzivatelske nastaveni -->
  <xsl:template match="nactinastaveniuzivatele"> <!-- OK -->
  </xsl:template>

  <xsl:template match="uloznastaveniuzivatele"> <!-- OK -->
  </xsl:template>



  <!-- reporty ============================================================ -->
  <xsl:template match="report"> <!-- v XSL odstraneno -->
      <div class="errormsg">report <xsl:value-of select="./@reportname"/>: XML formular je zastaraly a v teto verzi Portalu jiz neni podporovan - upravte formular a nastavte u nej atribut mode!</div>
  </xsl:template>


  <xsl:template match="reportvariable"> <!-- v XSL odstraneno -->
  </xsl:template>



  <!-- hodnoceni a testy ================================================== -->
    <xsl:template match="hlavickahodnoceni">
        <div class="errormsg">XML sablona pro zobrazeni hodnoceni je zastarala a v teto verzi Portalu jiz neni podporovana! Aktualizujte XML sablonu nebo ji v sablone hodnoceni odstrante!</div>
    </xsl:template>

    <xsl:template match="otazkyhodnoceni">
    </xsl:template>
    <!--
  <xsl:template match="otazkyhodnoceni">
      <xsl:variable name="hodnoceni_data" select="php:function('questform_otazkyhodnoceni', string(./@id), string(./@druhhodnoceni), string(./@temacislo) )" />
      
      <xsl:variable name="hodnoceni_content" select="./*" />
      
      <xsl:for-each select="$hodnoceni_data//XMLRESULT/ROW">
          <xsl:apply-templates/>
          <xsl:apply-templates select="$hodnoceni_content"/>
      </xsl:for-each> 
  </xsl:template>

  <xsl:template match="hlavickahodnoceni">
      <xsl:copy-of select="php:function('questform_hlavickahodnoceni')"/>
  </xsl:template>

  <xsl:template match="otazkahodnoceni">
      <xsl:copy-of select="php:function('questform_otazkahodnoceni', string(./@id), string(./@druhhodnoceni), string(./@temacislo), string(./@kodkriteria), string(./@zpusobzobrazeni) )"/>
  </xsl:template>
  
  <xsl:template match="poznamkahodnoceni">
      <xsl:copy-of select="php:function('questform_poznamkahodnoceni')"/>
  </xsl:template>

  <xsl:template match="tlacitkahodnoceni">
      <xsl:copy-of select="php:function('questform_tlacitkahodnoceni', string(./@ulozit), string(./@odeslat), string(./@vratit), string(./@uzavrit)  )"/>
  </xsl:template>
  -->



  <!-- tag XMLRESULT ignoruje -->
  <xsl:template match="XMLRESULT">
      <xsl:apply-templates/>
  </xsl:template>

  <!-- tag XMLRESULT ignoruje -->
  <xsl:template match="xmlresult">
      <xsl:apply-templates/>
  </xsl:template>



  <!-- tagy, ktere nenajde, jen zkopiruje -->
  <xsl:template match="node()|@*">
    <xsl:copy>
      <xsl:apply-templates select="node()|@*" />
    </xsl:copy>
  </xsl:template>


</xsl:stylesheet>
